@@ -11,12 +11,13 @@ import (
 	"strings"
 	"time"
 
-	"github.com/go-macaroon-bakery/macaroon-bakery/v3/httpbakery"
 	"github.com/gorilla/websocket"
+	"github.com/zitadel/oidc/v2/pkg/oidc"
 
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/simplestreams"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/simplestreams"
 )
 
 // ConnectionArgs represents a set of common connection properties.
@@ -39,21 +40,24 @@ type ConnectionArgs struct {
 	// Authentication type
 	AuthType string
 
-	// Authentication interactor
-	AuthInteractor []httpbakery.Interactor
-
 	// Custom proxy
 	Proxy func(*http.Request) (*url.URL, error)
 
 	// Custom HTTP Client (used as base for the connection)
 	HTTPClient *http.Client
 
+	// TransportWrapper wraps the *http.Transport set by lxd
+	TransportWrapper func(*http.Transport) HTTPTransporter
+
 	// Controls whether a client verifies the server's certificate chain and host name.
 	InsecureSkipVerify bool
 
 	// Cookie jar
 	CookieJar http.CookieJar
 
+	// OpenID Connect tokens
+	OIDCTokens *oidc.Tokens[*oidc.IDTokenClaims]
+
 	// Skip automatic GetServer request upon connection
 	SkipGetServer bool
 
@@ -81,11 +85,11 @@ func ConnectLXD(url string, args *ConnectionArgs) (InstanceServer, error) {
 //
 // Unless the remote server is trusted by the system CA, the remote certificate must be provided (TLSServerCert).
 func ConnectLXDWithContext(ctx context.Context, url string, args *ConnectionArgs) (InstanceServer, error) {
-	logger.Debug("Connecting to a remote LXD over HTTPS")
-
 	// Cleanup URL
 	url = strings.TrimSuffix(url, "/")
 
+	logger.Debug("Connecting to a remote LXD over HTTPS", logger.Ctx{"url": url})
+
 	return httpsLXD(ctx, url, args)
 }
 
@@ -259,7 +263,7 @@ func ConnectSimpleStreams(url string, args *ConnectionArgs) (ImageServer, error)
 	}
 
 	// Setup the HTTP client
-	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy)
+	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy, args.TransportWrapper)
 	if err != nil {
 		return nil, err
 	}
@@ -318,19 +322,18 @@ func httpsLXD(ctx context.Context, requestURL string, args *ConnectionArgs) (Ins
 		httpBaseURL:        *httpBaseURL,
 		httpProtocol:       "https",
 		httpUserAgent:      args.UserAgent,
-		bakeryInteractor:   args.AuthInteractor,
 		ctxConnected:       ctxConnected,
 		ctxConnectedCancel: ctxConnectedCancel,
 		eventConns:         make(map[string]*websocket.Conn),
 		eventListeners:     make(map[string][]*EventListener),
 	}
 
-	if args.AuthType == "candid" {
+	if shared.ValueInSlice(args.AuthType, []string{api.AuthenticationMethodOIDC}) {
 		server.RequireAuthenticated(true)
 	}
 
 	// Setup the HTTP client
-	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy)
+	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy, args.TransportWrapper)
 	if err != nil {
 		return nil, err
 	}
@@ -340,8 +343,8 @@ func httpsLXD(ctx context.Context, requestURL string, args *ConnectionArgs) (Ins
 	}
 
 	server.http = httpClient
-	if args.AuthType == "candid" {
-		server.setupBakeryClient()
+	if args.AuthType == api.AuthenticationMethodOIDC {
+		server.setupOIDCClient(args.OIDCTokens)
 	}
 
 	// Test the connection and seed the server information
