@@ -9,9 +9,9 @@ import (
 	"github.com/gorilla/websocket"
 	"github.com/pkg/sftp"
 
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/cancel"
-	"github.com/lxc/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/cancel"
+	"github.com/canonical/lxd/shared/ioprogress"
 )
 
 // The Operation type represents a currently running operation.
@@ -173,6 +173,10 @@ type InstanceServer interface {
 	MigrateInstance(name string, instance api.InstancePost) (op Operation, err error)
 	DeleteInstance(name string) (op Operation, err error)
 	UpdateInstances(state api.InstancesPut, ETag string) (op Operation, err error)
+	RebuildInstance(instanceName string, req api.InstanceRebuildPost) (op Operation, err error)
+	RebuildInstanceFromImage(source ImageServer, image api.Image, instanceName string, req api.InstanceRebuildPost) (op RemoteOperation, err error)
+	GetInstanceUEFIVars(name string) (instanceUEFI *api.InstanceUEFIVars, ETag string, err error)
+	UpdateInstanceUEFIVars(name string, instanceUEFI api.InstanceUEFIVars, ETag string) (err error)
 
 	ExecInstance(instanceName string, exec api.InstanceExecPost, args *InstanceExecArgs) (op Operation, err error)
 	ConsoleInstance(instanceName string, console api.InstanceConsolePost, args *InstanceConsoleArgs) (op Operation, err error)
@@ -284,6 +288,9 @@ type InstanceServer interface {
 	RenameNetworkACL(name string, acl api.NetworkACLPost) (err error)
 	DeleteNetworkACL(name string) (err error)
 
+	// Network allocations functions ("network_allocations" API extension)
+	GetNetworkAllocations(allProjects bool) (allocations []api.NetworkAllocations, err error)
+
 	// Network zone functions ("network_dns" API extension)
 	GetNetworkZoneNames() (names []string, err error)
 	GetNetworkZones() (zones []api.NetworkZone, err error)
@@ -302,6 +309,7 @@ type InstanceServer interface {
 	// Operation functions
 	GetOperationUUIDs() (uuids []string, err error)
 	GetOperations() (operations []api.Operation, err error)
+	GetOperationsAllProjects() (operations []api.Operation, err error)
 	GetOperation(uuid string) (op *api.Operation, ETag string, err error)
 	GetOperationWait(uuid string, timeout int) (op *api.Operation, ETag string, err error)
 	GetOperationWaitSecret(uuid string, secret string, timeout int) (op *api.Operation, ETag string, err error)
@@ -350,9 +358,13 @@ type InstanceServer interface {
 	UpdateStoragePoolBucketKey(poolName string, bucketName string, keyName string, key api.StorageBucketKeyPut, ETag string) (err error)
 	DeleteStoragePoolBucketKey(poolName string, bucketName string, keyName string) (err error)
 
+	// List all volumes functions ("storage_volumes_all" API extension)
+	GetVolumesWithFilter(filters []string) (volumes []api.StorageVolume, err error)
+	GetVolumesWithFilterAllProjects(filters []string) (volumes []api.StorageVolume, err error)
+
 	// Storage volume functions ("storage" API extension)
 	GetStoragePoolVolumeNames(pool string) (names []string, err error)
-	GetStoragePoolVolumeNamesAllProjects(pool string) (names []string, err error)
+	GetStoragePoolVolumeNamesAllProjects(pool string) (names map[string][]string, err error)
 	GetStoragePoolVolumes(pool string) (volumes []api.StorageVolume, err error)
 	GetStoragePoolVolumesAllProjects(pool string) (volumes []api.StorageVolume, err error)
 	GetStoragePoolVolumesWithFilter(pool string, filters []string) (volumes []api.StorageVolume, err error)
@@ -386,6 +398,9 @@ type InstanceServer interface {
 	GetStoragePoolVolumeBackupFile(pool string, volName string, name string, req *BackupFileRequest) (resp *BackupFileResponse, err error)
 	CreateStoragePoolVolumeFromBackup(pool string, args StoragePoolVolumeBackupArgs) (op Operation, err error)
 
+	// Storage volume ISO import function ("custom_volume_iso" API extension)
+	CreateStoragePoolVolumeFromISO(pool string, args StoragePoolVolumeBackupArgs) (op Operation, err error)
+
 	// Cluster functions ("cluster" API extensions)
 	GetCluster() (cluster *api.Cluster, ETag string, err error)
 	UpdateCluster(cluster api.ClusterPut, ETag string) (op Operation, err error)
@@ -397,6 +412,7 @@ type InstanceServer interface {
 	RenameClusterMember(name string, member api.ClusterMemberPost) (err error)
 	CreateClusterMember(member api.ClusterMembersPost) (op Operation, err error)
 	UpdateClusterCertificate(certs api.ClusterCertificatePut, ETag string) (err error)
+	GetClusterMemberState(name string) (*api.ClusterMemberState, string, error)
 	UpdateClusterMemberState(name string, state api.ClusterMemberStatePost) (op Operation, err error)
 	GetClusterGroups() ([]api.ClusterGroup, error)
 	GetClusterGroupNames() ([]string, error)
@@ -413,6 +429,31 @@ type InstanceServer interface {
 	UpdateWarning(UUID string, warning api.WarningPut, ETag string) (err error)
 	DeleteWarning(UUID string) (err error)
 
+	// Authorization functions
+	GetAuthGroupNames() (groupNames []string, err error)
+	GetAuthGroups() (groups []api.AuthGroup, err error)
+	GetAuthGroup(groupName string) (group *api.AuthGroup, ETag string, err error)
+	CreateAuthGroup(groupsPost api.AuthGroupsPost) error
+	UpdateAuthGroup(groupName string, groupPut api.AuthGroupPut, ETag string) error
+	RenameAuthGroup(groupName string, groupPost api.AuthGroupPost) error
+	DeleteAuthGroup(groupName string) error
+	GetIdentityAuthenticationMethodsIdentifiers() (authMethodsIdentifiers map[string][]string, err error)
+	GetIdentityIdentifiersByAuthenticationMethod(authenticationMethod string) (identifiers []string, err error)
+	GetIdentities() (identities []api.Identity, err error)
+	GetIdentitiesByAuthenticationMethod(authenticationMethod string) (identities []api.Identity, err error)
+	GetIdentity(authenticationMethod string, nameOrIdentifier string) (identity *api.Identity, ETag string, err error)
+	GetCurrentIdentityInfo() (identityInfo *api.IdentityInfo, ETag string, err error)
+	UpdateIdentity(authenticationMethod string, nameOrIdentifier string, identityPut api.IdentityPut, ETag string) error
+	GetIdentityProviderGroupNames() (identityProviderGroupNames []string, err error)
+	GetIdentityProviderGroups() (identityProviderGroups []api.IdentityProviderGroup, err error)
+	GetIdentityProviderGroup(identityProviderGroupName string) (identityProviderGroup *api.IdentityProviderGroup, ETag string, err error)
+	CreateIdentityProviderGroup(identityProviderGroup api.IdentityProviderGroup) error
+	UpdateIdentityProviderGroup(identityProviderGroupName string, identityProviderGroupPut api.IdentityProviderGroupPut, ETag string) error
+	RenameIdentityProviderGroup(identityProviderGroupName string, identityProviderGroupPost api.IdentityProviderGroupPost) error
+	DeleteIdentityProviderGroup(identityProviderGroupName string) error
+	GetPermissions(args GetPermissionsArgs) (permissions []api.Permission, err error)
+	GetPermissionsInfo(args GetPermissionsArgs) (permissions []api.PermissionInfo, err error)
+
 	// Internal functions (for internal use)
 	RawQuery(method string, path string, data any, queryETag string) (resp *api.Response, ETag string, err error)
 	RawWebsocket(path string) (conn *websocket.Conn, err error)
@@ -572,6 +613,9 @@ type InstanceBackupArgs struct {
 
 	// Name to import backup as
 	Name string
+
+	// If set, it would override devices
+	Devices map[string]map[string]string
 }
 
 // The InstanceCopyArgs struct is used to pass additional options during instance copy.
@@ -630,13 +674,13 @@ type InstanceConsoleLogArgs struct {
 // The InstanceExecArgs struct is used to pass additional options during instance exec.
 type InstanceExecArgs struct {
 	// Standard input
-	Stdin io.ReadCloser
+	Stdin io.Reader
 
 	// Standard output
-	Stdout io.WriteCloser
+	Stdout io.Writer
 
 	// Standard error
-	Stderr io.WriteCloser
+	Stderr io.Writer
 
 	// Control message handler (window resize, signals, ...)
 	Control func(conn *websocket.Conn)
@@ -659,6 +703,12 @@ type InstanceFileArgs struct {
 	// File permissions
 	Mode int
 
+	// Whether to modify the permissions of existing files (see the
+	// instances_files_modify_permissions api extension)
+	GIDModifyExisting  bool
+	UIDModifyExisting  bool
+	ModeModifyExisting bool
+
 	// File type (file or directory)
 	Type string
 
@@ -683,3 +733,15 @@ type InstanceFileResponse struct {
 	// If a directory, the list of files inside it
 	Entries []string
 }
+
+// GetPermissionsArgs is used in the call to GetPermissions to specify filtering behaviour.
+type GetPermissionsArgs struct {
+	// EntityType is the type of entity to filter against.
+	// If left unspecified, permissions will be returned for all entity types.
+	EntityType string
+
+	// ProjectName is the project to filter against.
+	// If the project name is specified, only permissions for resources in the given project will be returned and server
+	// level permissions will not be returned.
+	ProjectName string
+}
