@@ -10,11 +10,12 @@ import (
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/cancel"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/cancel"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/ws"
 )
 
 // Container handling functions
@@ -53,12 +54,13 @@ func (r *ProtocolLXD) GetContainers() ([]api.Container, error) {
 func (r *ProtocolLXD) GetContainersFull() ([]api.ContainerFull, error) {
 	containers := []api.ContainerFull{}
 
-	if !r.HasExtension("container_full") {
-		return nil, fmt.Errorf("The server is missing the required \"container_full\" API extension")
+	err := r.CheckExtension("container_full")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", "/containers?recursion=2", nil, "", &containers)
+	_, err = r.queryStruct("GET", "/containers?recursion=2", nil, "", &containers)
 	if err != nil {
 		return nil, err
 	}
@@ -82,13 +84,14 @@ func (r *ProtocolLXD) GetContainer(name string) (*api.Container, string, error)
 // CreateContainerFromBackup is a convenience function to make it easier to
 // create a container from a backup.
 func (r *ProtocolLXD) CreateContainerFromBackup(args ContainerBackupArgs) (Operation, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	if args.PoolName == "" {
 		// Send the request
-		op, _, err := r.queryOperation("POST", "/containers", args.BackupFile, "")
+		op, _, err := r.queryOperation("POST", "/containers", args.BackupFile, "", true)
 		if err != nil {
 			return nil, err
 		}
@@ -96,8 +99,9 @@ func (r *ProtocolLXD) CreateContainerFromBackup(args ContainerBackupArgs) (Opera
 		return op, nil
 	}
 
-	if !r.HasExtension("container_backup_override_pool") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup_override_pool\" API extension")
+	err = r.CheckExtension("container_backup_override_pool")
+	if err != nil {
+		return nil, err
 	}
 
 	// Prepare the HTTP request
@@ -147,13 +151,14 @@ func (r *ProtocolLXD) CreateContainerFromBackup(args ContainerBackupArgs) (Opera
 // CreateContainer requests that LXD creates a new container.
 func (r *ProtocolLXD) CreateContainer(container api.ContainersPost) (Operation, error) {
 	if container.Source.ContainerOnly {
-		if !r.HasExtension("container_only_migration") {
-			return nil, fmt.Errorf("The server is missing the required \"container_only_migration\" API extension")
+		err := r.CheckExtension("container_only_migration")
+		if err != nil {
+			return nil, err
 		}
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", "/containers", container, "")
+	op, _, err := r.queryOperation("POST", "/containers", container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -306,7 +311,7 @@ func (r *ProtocolLXD) CopyContainer(source InstanceServer, container api.Contain
 			}
 		}
 
-		if shared.StringInSlice(args.Mode, []string{"push", "relay"}) {
+		if shared.ValueInSlice(args.Mode, []string{"push", "relay"}) {
 			if !r.HasExtension("container_push") {
 				return nil, fmt.Errorf("The target server is missing the required \"container_push\" API extension")
 			}
@@ -355,11 +360,12 @@ func (r *ProtocolLXD) CopyContainer(source InstanceServer, container api.Contain
 	}
 
 	// Optimization for the local copy case
-	if destInfo.URL == sourceInfo.URL && destInfo.SocketPath == sourceInfo.SocketPath && (!r.IsClustered() || container.Location == r.clusterTarget || r.HasExtension("cluster_internal_copy")) {
+	if destInfo.URL == sourceInfo.URL && destInfo.SocketPath == sourceInfo.SocketPath && (!r.IsClustered() || container.Location == r.clusterTarget || r.CheckExtension("cluster_internal_copy") == nil) {
 		// Project handling
 		if destInfo.Project != sourceInfo.Project {
-			if !r.HasExtension("container_copy_project") {
-				return nil, fmt.Errorf("The server is missing the required \"container_copy_project\" API extension")
+			err := r.CheckExtension("container_copy_project")
+			if err != nil {
+				return nil, err
 			}
 
 			req.Source.Project = sourceInfo.Project
@@ -503,7 +509,7 @@ func (r *ProtocolLXD) CopyContainer(source InstanceServer, container api.Contain
 // UpdateContainer updates the container definition.
 func (r *ProtocolLXD) UpdateContainer(name string, container api.ContainerPut, ETag string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation("PUT", fmt.Sprintf("/containers/%s", url.PathEscape(name)), container, ETag)
+	op, _, err := r.queryOperation("PUT", fmt.Sprintf("/containers/%s", url.PathEscape(name)), container, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -519,7 +525,7 @@ func (r *ProtocolLXD) RenameContainer(name string, container api.ContainerPost)
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s", url.PathEscape(name)), container, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s", url.PathEscape(name)), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -585,8 +591,9 @@ func (r *ProtocolLXD) tryMigrateContainer(source InstanceServer, name string, re
 // MigrateContainer requests that LXD prepares for a container migration.
 func (r *ProtocolLXD) MigrateContainer(name string, container api.ContainerPost) (Operation, error) {
 	if container.ContainerOnly {
-		if !r.HasExtension("container_only_migration") {
-			return nil, fmt.Errorf("The server is missing the required \"container_only_migration\" API extension")
+		err := r.CheckExtension("container_only_migration")
+		if err != nil {
+			return nil, err
 		}
 	}
 
@@ -596,7 +603,7 @@ func (r *ProtocolLXD) MigrateContainer(name string, container api.ContainerPost)
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s", url.PathEscape(name)), container, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s", url.PathEscape(name)), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -607,7 +614,7 @@ func (r *ProtocolLXD) MigrateContainer(name string, container api.ContainerPost)
 // DeleteContainer requests that LXD deletes the container.
 func (r *ProtocolLXD) DeleteContainer(name string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/containers/%s", url.PathEscape(name)), nil, "")
+	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/containers/%s", url.PathEscape(name)), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -618,19 +625,21 @@ func (r *ProtocolLXD) DeleteContainer(name string) (Operation, error) {
 // ExecContainer requests that LXD spawns a command inside the container.
 func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExecPost, args *ContainerExecArgs) (Operation, error) {
 	if exec.RecordOutput {
-		if !r.HasExtension("container_exec_recording") {
-			return nil, fmt.Errorf("The server is missing the required \"container_exec_recording\" API extension")
+		err := r.CheckExtension("container_exec_recording")
+		if err != nil {
+			return nil, err
 		}
 	}
 
 	if exec.User > 0 || exec.Group > 0 || exec.Cwd != "" {
-		if !r.HasExtension("container_exec_user_group_cwd") {
-			return nil, fmt.Errorf("The server is missing the required \"container_exec_user_group_cwd\" API extension")
+		err := r.CheckExtension("container_exec_user_group_cwd")
+		if err != nil {
+			return nil, err
 		}
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/exec", url.PathEscape(containerName)), exec, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/exec", url.PathEscape(containerName)), exec, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -651,8 +660,8 @@ func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExec
 		}
 
 		// Call the control handler with a connection to the control socket
-		if args.Control != nil && fds["control"] != "" {
-			conn, err := r.GetOperationWebsocket(opAPI.ID, fds["control"])
+		if args.Control != nil && fds[api.SecretNameControl] != "" {
+			conn, err := r.GetOperationWebsocket(opAPI.ID, fds[api.SecretNameControl])
 			if err != nil {
 				return nil, err
 			}
@@ -671,8 +680,8 @@ func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExec
 
 				// And attach stdin and stdout to it
 				go func() {
-					shared.WebsocketSendStream(conn, args.Stdin, -1)
-					<-shared.WebsocketRecvStream(args.Stdout, conn)
+					ws.MirrorRead(conn, args.Stdin)
+					<-ws.MirrorWrite(conn, args.Stdout)
 					_ = conn.Close()
 
 					if args.DataDone != nil {
@@ -686,7 +695,7 @@ func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExec
 			}
 		} else {
 			// Handle non-interactive sessions
-			dones := map[int]chan bool{}
+			dones := make(map[int]chan error)
 			conns := []*websocket.Conn{}
 
 			// Handle stdin
@@ -697,9 +706,11 @@ func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExec
 				}
 
 				conns = append(conns, conn)
-				dones[0] = shared.WebsocketSendStream(conn, args.Stdin, -1)
+				dones[0] = ws.MirrorRead(conn, args.Stdin)
 			}
 
+			waitConns := 0 // Used for keeping track of when stdout and stderr have finished.
+
 			// Handle stdout
 			if fds["1"] != "" {
 				conn, err := r.GetOperationWebsocket(opAPI.ID, fds["1"])
@@ -708,7 +719,8 @@ func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExec
 				}
 
 				conns = append(conns, conn)
-				dones[1] = shared.WebsocketRecvStream(args.Stdout, conn)
+				dones[1] = ws.MirrorWrite(conn, args.Stdout)
+				waitConns++
 			}
 
 			// Handle stderr
@@ -719,34 +731,37 @@ func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExec
 				}
 
 				conns = append(conns, conn)
-				dones[2] = shared.WebsocketRecvStream(args.Stderr, conn)
+				dones[2] = ws.MirrorWrite(conn, args.Stderr)
+				waitConns++
 			}
 
 			// Wait for everything to be done
 			go func() {
-				for i, chDone := range dones {
-					// Skip stdin, dealing with it separately below
-					if i == 0 {
-						continue
+				for {
+					select {
+					case <-dones[0]:
+						// Handle stdin finish, but don't wait for it if output channels
+						// have all finished.
+						dones[0] = nil
+						_ = conns[0].Close()
+					case <-dones[1]:
+						dones[1] = nil
+						_ = conns[1].Close()
+						waitConns--
+					case <-dones[2]:
+						dones[2] = nil
+						_ = conns[2].Close()
+						waitConns--
 					}
 
-					<-chDone
-				}
+					if waitConns <= 0 {
+						// Close stdin websocket if defined and not already closed.
+						if dones[0] != nil {
+							conns[0].Close()
+						}
 
-				if fds["0"] != "" {
-					if args.Stdin != nil {
-						_ = args.Stdin.Close()
+						break
 					}
-
-					// Empty the stdin channel but don't block on it as
-					// stdin may be stuck in Read()
-					go func() {
-						<-dones[0]
-					}()
-				}
-
-				for _, conn := range conns {
-					_ = conn.Close()
 				}
 
 				if args.DataDone != nil {
@@ -794,12 +809,16 @@ func (r *ProtocolLXD) GetContainerFile(containerName string, path string) (io.Re
 	}
 
 	// Parse the headers
-	uid, gid, mode, fileType, _ := shared.ParseLXDFileHeaders(resp.Header)
+	headers, err := shared.ParseLXDFileHeaders(resp.Header)
+	if err != nil {
+		return nil, nil, fmt.Errorf("Failed to parse response headers: %w", err)
+	}
+
 	fileResp := ContainerFileResponse{
-		UID:  uid,
-		GID:  gid,
-		Mode: mode,
-		Type: fileType,
+		UID:  headers.UID,
+		GID:  headers.GID,
+		Mode: headers.Mode,
+		Type: headers.Type,
 	}
 
 	if fileResp.Type == "directory" {
@@ -830,20 +849,23 @@ func (r *ProtocolLXD) GetContainerFile(containerName string, path string) (io.Re
 // CreateContainerFile tells LXD to create a file in the container.
 func (r *ProtocolLXD) CreateContainerFile(containerName string, path string, args ContainerFileArgs) error {
 	if args.Type == "directory" {
-		if !r.HasExtension("directory_manipulation") {
-			return fmt.Errorf("The server is missing the required \"directory_manipulation\" API extension")
+		err := r.CheckExtension("directory_manipulation")
+		if err != nil {
+			return err
 		}
 	}
 
 	if args.Type == "symlink" {
-		if !r.HasExtension("file_symlinks") {
-			return fmt.Errorf("The server is missing the required \"file_symlinks\" API extension")
+		err := r.CheckExtension("file_symlinks")
+		if err != nil {
+			return err
 		}
 	}
 
 	if args.WriteMode == "append" {
-		if !r.HasExtension("file_append") {
-			return fmt.Errorf("The server is missing the required \"file_append\" API extension")
+		err := r.CheckExtension("file_append")
+		if err != nil {
+			return err
 		}
 	}
 
@@ -898,12 +920,13 @@ func (r *ProtocolLXD) CreateContainerFile(containerName string, path string, arg
 
 // DeleteContainerFile deletes a file in the container.
 func (r *ProtocolLXD) DeleteContainerFile(containerName string, path string) error {
-	if !r.HasExtension("file_delete") {
-		return fmt.Errorf("The server is missing the required \"file_delete\" API extension")
+	err := r.CheckExtension("file_delete")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
-	_, _, err := r.query("DELETE", fmt.Sprintf("/containers/%s/files?path=%s", url.PathEscape(containerName), url.QueryEscape(path)), nil, "")
+	_, _, err = r.query("DELETE", fmt.Sprintf("/containers/%s/files?path=%s", url.PathEscape(containerName), url.QueryEscape(path)), nil, "")
 	if err != nil {
 		return err
 	}
@@ -954,12 +977,15 @@ func (r *ProtocolLXD) GetContainerSnapshot(containerName string, name string) (*
 // CreateContainerSnapshot requests that LXD creates a new snapshot for the container.
 func (r *ProtocolLXD) CreateContainerSnapshot(containerName string, snapshot api.ContainerSnapshotsPost) (Operation, error) {
 	// Validate the request
-	if snapshot.ExpiresAt != nil && !r.HasExtension("snapshot_expiry_creation") {
-		return nil, fmt.Errorf("The server is missing the required \"snapshot_expiry_creation\" API extension")
+	if snapshot.ExpiresAt != nil {
+		err := r.CheckExtension("snapshot_expiry_creation")
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/snapshots", url.PathEscape(containerName)), snapshot, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/snapshots", url.PathEscape(containerName)), snapshot, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -987,12 +1013,13 @@ func (r *ProtocolLXD) CopyContainerSnapshot(source InstanceServer, containerName
 	}
 
 	if snapshot.Stateful && args.Live {
-		if !r.HasExtension("container_snapshot_stateful_migration") {
-			return nil, fmt.Errorf("The server is missing the required \"container_snapshot_stateful_migration\" API extension")
+		err := r.CheckExtension("container_snapshot_stateful_migration")
+		if err != nil {
+			return nil, err
 		}
 
 		req.ContainerPut.Stateful = snapshot.Stateful
-		req.Source.Live = args.Live
+		req.Source.Live = false // Snapshots are never running and so we don't need live migration.
 	}
 
 	req.Source.BaseImage = snapshot.Config["volatile.base_image"]
@@ -1000,7 +1027,7 @@ func (r *ProtocolLXD) CopyContainerSnapshot(source InstanceServer, containerName
 	// Process the copy arguments
 	if args != nil {
 		// Quick checks.
-		if shared.StringInSlice(args.Mode, []string{"push", "relay"}) {
+		if shared.ValueInSlice(args.Mode, []string{"push", "relay"}) {
 			if !r.HasExtension("container_push") {
 				return nil, fmt.Errorf("The target server is missing the required \"container_push\" API extension")
 			}
@@ -1036,11 +1063,12 @@ func (r *ProtocolLXD) CopyContainerSnapshot(source InstanceServer, containerName
 	}
 
 	// Optimization for the local copy case
-	if destInfo.URL == sourceInfo.URL && destInfo.SocketPath == sourceInfo.SocketPath && (!r.IsClustered() || container.Location == r.clusterTarget || r.HasExtension("cluster_internal_copy")) {
+	if destInfo.URL == sourceInfo.URL && destInfo.SocketPath == sourceInfo.SocketPath && (!r.IsClustered() || container.Location == r.clusterTarget || r.CheckExtension("cluster_internal_copy") == nil) {
 		// Project handling
 		if destInfo.Project != sourceInfo.Project {
-			if !r.HasExtension("container_copy_project") {
-				return nil, fmt.Errorf("The server is missing the required \"container_copy_project\" API extension")
+			err := r.CheckExtension("container_copy_project")
+			if err != nil {
+				return nil, err
 			}
 
 			req.Source.Project = sourceInfo.Project
@@ -1191,7 +1219,7 @@ func (r *ProtocolLXD) RenameContainerSnapshot(containerName string, name string,
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/snapshots/%s", url.PathEscape(containerName), url.PathEscape(name)), container, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/snapshots/%s", url.PathEscape(containerName), url.PathEscape(name)), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1199,6 +1227,8 @@ func (r *ProtocolLXD) RenameContainerSnapshot(containerName string, name string,
 	return op, nil
 }
 
+// tryMigrateContainerSnapshot attempts to migrate a container snapshot from the source instance server to one of the target URLs.
+// It runs the migration asynchronously and returns a RemoteOperation to track the migration status and any errors.
 func (r *ProtocolLXD) tryMigrateContainerSnapshot(source InstanceServer, containerName string, name string, req api.ContainerSnapshotPost, urls []string) (RemoteOperation, error) {
 	if len(urls) == 0 {
 		return nil, fmt.Errorf("The target server isn't listening on the network")
@@ -1262,7 +1292,7 @@ func (r *ProtocolLXD) MigrateContainerSnapshot(containerName string, name string
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/snapshots/%s", url.PathEscape(containerName), url.PathEscape(name)), container, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/snapshots/%s", url.PathEscape(containerName), url.PathEscape(name)), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1273,7 +1303,7 @@ func (r *ProtocolLXD) MigrateContainerSnapshot(containerName string, name string
 // DeleteContainerSnapshot requests that LXD deletes the container snapshot.
 func (r *ProtocolLXD) DeleteContainerSnapshot(containerName string, name string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/containers/%s/snapshots/%s", url.PathEscape(containerName), url.PathEscape(name)), nil, "")
+	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/containers/%s/snapshots/%s", url.PathEscape(containerName), url.PathEscape(name)), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1283,13 +1313,14 @@ func (r *ProtocolLXD) DeleteContainerSnapshot(containerName string, name string)
 
 // UpdateContainerSnapshot requests that LXD updates the container snapshot.
 func (r *ProtocolLXD) UpdateContainerSnapshot(containerName string, name string, container api.ContainerSnapshotPut, ETag string) (Operation, error) {
-	if !r.HasExtension("snapshot_expiry") {
-		return nil, fmt.Errorf("The server is missing the required \"snapshot_expiry\" API extension")
+	err := r.CheckExtension("snapshot_expiry")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
 	op, _, err := r.queryOperation("PUT", fmt.Sprintf("/containers/%s/snapshots/%s",
-		url.PathEscape(containerName), url.PathEscape(name)), container, ETag)
+		url.PathEscape(containerName), url.PathEscape(name)), container, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -1313,7 +1344,7 @@ func (r *ProtocolLXD) GetContainerState(name string) (*api.ContainerState, strin
 // UpdateContainerState updates the container to match the requested state.
 func (r *ProtocolLXD) UpdateContainerState(name string, state api.ContainerStatePut, ETag string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation("PUT", fmt.Sprintf("/containers/%s/state", url.PathEscape(name)), state, ETag)
+	op, _, err := r.queryOperation("PUT", fmt.Sprintf("/containers/%s/state", url.PathEscape(name)), state, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -1382,8 +1413,9 @@ func (r *ProtocolLXD) DeleteContainerLogfile(name string, filename string) error
 
 // GetContainerMetadata returns container metadata.
 func (r *ProtocolLXD) GetContainerMetadata(name string) (*api.ImageMetadata, string, error) {
-	if !r.HasExtension("container_edit_metadata") {
-		return nil, "", fmt.Errorf("The server is missing the required \"container_edit_metadata\" API extension")
+	err := r.CheckExtension("container_edit_metadata")
+	if err != nil {
+		return nil, "", err
 	}
 
 	metadata := api.ImageMetadata{}
@@ -1399,12 +1431,13 @@ func (r *ProtocolLXD) GetContainerMetadata(name string) (*api.ImageMetadata, str
 
 // SetContainerMetadata sets the content of the container metadata file.
 func (r *ProtocolLXD) SetContainerMetadata(name string, metadata api.ImageMetadata, ETag string) error {
-	if !r.HasExtension("container_edit_metadata") {
-		return fmt.Errorf("The server is missing the required \"container_edit_metadata\" API extension")
+	err := r.CheckExtension("container_edit_metadata")
+	if err != nil {
+		return err
 	}
 
 	url := fmt.Sprintf("/containers/%s/metadata", url.PathEscape(name))
-	_, _, err := r.query("PUT", url, metadata, ETag)
+	_, _, err = r.query("PUT", url, metadata, ETag)
 	if err != nil {
 		return err
 	}
@@ -1414,14 +1447,15 @@ func (r *ProtocolLXD) SetContainerMetadata(name string, metadata api.ImageMetada
 
 // GetContainerTemplateFiles returns the list of names of template files for a container.
 func (r *ProtocolLXD) GetContainerTemplateFiles(containerName string) ([]string, error) {
-	if !r.HasExtension("container_edit_metadata") {
-		return nil, fmt.Errorf("The server is missing the required \"container_edit_metadata\" API extension")
+	err := r.CheckExtension("container_edit_metadata")
+	if err != nil {
+		return nil, err
 	}
 
 	templates := []string{}
 
 	url := fmt.Sprintf("/containers/%s/metadata/templates", url.PathEscape(containerName))
-	_, err := r.queryStruct("GET", url, nil, "", &templates)
+	_, err = r.queryStruct("GET", url, nil, "", &templates)
 	if err != nil {
 		return nil, err
 	}
@@ -1431,13 +1465,14 @@ func (r *ProtocolLXD) GetContainerTemplateFiles(containerName string) ([]string,
 
 // GetContainerTemplateFile returns the content of a template file for a container.
 func (r *ProtocolLXD) GetContainerTemplateFile(containerName string, templateName string) (io.ReadCloser, error) {
-	if !r.HasExtension("container_edit_metadata") {
-		return nil, fmt.Errorf("The server is missing the required \"container_edit_metadata\" API extension")
+	err := r.CheckExtension("container_edit_metadata")
+	if err != nil {
+		return nil, err
 	}
 
 	url := fmt.Sprintf("%s/1.0/containers/%s/metadata/templates?path=%s", r.httpBaseURL.String(), url.PathEscape(containerName), url.QueryEscape(templateName))
 
-	url, err := r.setQueryAttributes(url)
+	url, err = r.setQueryAttributes(url)
 	if err != nil {
 		return nil, err
 	}
@@ -1466,13 +1501,14 @@ func (r *ProtocolLXD) GetContainerTemplateFile(containerName string, templateNam
 
 // CreateContainerTemplateFile creates an a template for a container.
 func (r *ProtocolLXD) CreateContainerTemplateFile(containerName string, templateName string, content io.ReadSeeker) error {
-	if !r.HasExtension("container_edit_metadata") {
-		return fmt.Errorf("The server is missing the required \"container_edit_metadata\" API extension")
+	err := r.CheckExtension("container_edit_metadata")
+	if err != nil {
+		return err
 	}
 
 	url := fmt.Sprintf("%s/1.0/containers/%s/metadata/templates?path=%s", r.httpBaseURL.String(), url.PathEscape(containerName), url.QueryEscape(templateName))
 
-	url, err := r.setQueryAttributes(url)
+	url, err = r.setQueryAttributes(url)
 	if err != nil {
 		return err
 	}
@@ -1503,22 +1539,24 @@ func (r *ProtocolLXD) UpdateContainerTemplateFile(containerName string, template
 
 // DeleteContainerTemplateFile deletes a template file for a container.
 func (r *ProtocolLXD) DeleteContainerTemplateFile(name string, templateName string) error {
-	if !r.HasExtension("container_edit_metadata") {
-		return fmt.Errorf("The server is missing the required \"container_edit_metadata\" API extension")
+	err := r.CheckExtension("container_edit_metadata")
+	if err != nil {
+		return err
 	}
 
-	_, _, err := r.query("DELETE", fmt.Sprintf("/containers/%s/metadata/templates?path=%s", url.PathEscape(name), url.QueryEscape(templateName)), nil, "")
+	_, _, err = r.query("DELETE", fmt.Sprintf("/containers/%s/metadata/templates?path=%s", url.PathEscape(name), url.QueryEscape(templateName)), nil, "")
 	return err
 }
 
 // ConsoleContainer requests that LXD attaches to the console device of a container.
 func (r *ProtocolLXD) ConsoleContainer(containerName string, console api.ContainerConsolePost, args *ContainerConsoleArgs) (Operation, error) {
-	if !r.HasExtension("console") {
-		return nil, fmt.Errorf("The server is missing the required \"console\" API extension")
+	err := r.CheckExtension("console")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/console", url.PathEscape(containerName)), console, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/console", url.PathEscape(containerName)), console, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1546,11 +1584,11 @@ func (r *ProtocolLXD) ConsoleContainer(containerName string, console api.Contain
 
 	var controlConn *websocket.Conn
 	// Call the control handler with a connection to the control socket
-	if fds["control"] == "" {
+	if fds[api.SecretNameControl] == "" {
 		return nil, fmt.Errorf("Did not receive a file descriptor for the control channel")
 	}
 
-	controlConn, err = r.GetOperationWebsocket(opAPI.ID, fds["control"])
+	controlConn, err = r.GetOperationWebsocket(opAPI.ID, fds[api.SecretNameControl])
 	if err != nil {
 		return nil, err
 	}
@@ -1574,8 +1612,8 @@ func (r *ProtocolLXD) ConsoleContainer(containerName string, console api.Contain
 
 	// And attach stdin and stdout to it
 	go func() {
-		shared.WebsocketSendStream(conn, args.Terminal, -1)
-		<-shared.WebsocketRecvStream(args.Terminal, conn)
+		_, writeDone := ws.Mirror(conn, args.Terminal)
+		<-writeDone
 		_ = conn.Close()
 	}()
 
@@ -1586,14 +1624,15 @@ func (r *ProtocolLXD) ConsoleContainer(containerName string, console api.Contain
 //
 // Note that it's the caller's responsibility to close the returned ReadCloser.
 func (r *ProtocolLXD) GetContainerConsoleLog(containerName string, args *ContainerConsoleLogArgs) (io.ReadCloser, error) {
-	if !r.HasExtension("console") {
-		return nil, fmt.Errorf("The server is missing the required \"console\" API extension")
+	err := r.CheckExtension("console")
+	if err != nil {
+		return nil, err
 	}
 
 	// Prepare the HTTP request
 	url := fmt.Sprintf("%s/1.0/containers/%s/console", r.httpBaseURL.String(), url.PathEscape(containerName))
 
-	url, err := r.setQueryAttributes(url)
+	url, err = r.setQueryAttributes(url)
 	if err != nil {
 		return nil, err
 	}
@@ -1622,12 +1661,13 @@ func (r *ProtocolLXD) GetContainerConsoleLog(containerName string, args *Contain
 
 // DeleteContainerConsoleLog deletes the requested container's console log.
 func (r *ProtocolLXD) DeleteContainerConsoleLog(containerName string, args *ContainerConsoleLogArgs) error {
-	if !r.HasExtension("console") {
-		return fmt.Errorf("The server is missing the required \"console\" API extension")
+	err := r.CheckExtension("console")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
-	_, _, err := r.query("DELETE", fmt.Sprintf("/containers/%s/console", url.PathEscape(containerName)), nil, "")
+	_, _, err = r.query("DELETE", fmt.Sprintf("/containers/%s/console", url.PathEscape(containerName)), nil, "")
 	if err != nil {
 		return err
 	}
@@ -1637,14 +1677,15 @@ func (r *ProtocolLXD) DeleteContainerConsoleLog(containerName string, args *Cont
 
 // GetContainerBackupNames returns a list of backup names for the container.
 func (r *ProtocolLXD) GetContainerBackupNames(containerName string) ([]string, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := fmt.Sprintf("/containers/%s/backups", url.PathEscape(containerName))
-	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -1655,14 +1696,15 @@ func (r *ProtocolLXD) GetContainerBackupNames(containerName string) ([]string, e
 
 // GetContainerBackups returns a list of backups for the container.
 func (r *ProtocolLXD) GetContainerBackups(containerName string) ([]api.ContainerBackup, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw value
 	backups := []api.ContainerBackup{}
 
-	_, err := r.queryStruct("GET", fmt.Sprintf("/containers/%s/backups?recursion=1", url.PathEscape(containerName)), nil, "", &backups)
+	_, err = r.queryStruct("GET", fmt.Sprintf("/containers/%s/backups?recursion=1", url.PathEscape(containerName)), nil, "", &backups)
 	if err != nil {
 		return nil, err
 	}
@@ -1672,8 +1714,9 @@ func (r *ProtocolLXD) GetContainerBackups(containerName string) ([]api.Container
 
 // GetContainerBackup returns a Backup struct for the provided container and backup names.
 func (r *ProtocolLXD) GetContainerBackup(containerName string, name string) (*api.ContainerBackup, string, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, "", fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, "", err
 	}
 
 	// Fetch the raw value
@@ -1688,13 +1731,14 @@ func (r *ProtocolLXD) GetContainerBackup(containerName string, name string) (*ap
 
 // CreateContainerBackup requests that LXD creates a new backup for the container.
 func (r *ProtocolLXD) CreateContainerBackup(containerName string, backup api.ContainerBackupsPost) (Operation, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
 	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/backups",
-		url.PathEscape(containerName)), backup, "")
+		url.PathEscape(containerName)), backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1704,13 +1748,14 @@ func (r *ProtocolLXD) CreateContainerBackup(containerName string, backup api.Con
 
 // RenameContainerBackup requests that LXD renames the backup.
 func (r *ProtocolLXD) RenameContainerBackup(containerName string, name string, backup api.ContainerBackupPost) (Operation, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
 	op, _, err := r.queryOperation("POST", fmt.Sprintf("/containers/%s/backups/%s",
-		url.PathEscape(containerName), url.PathEscape(name)), backup, "")
+		url.PathEscape(containerName), url.PathEscape(name)), backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1720,13 +1765,14 @@ func (r *ProtocolLXD) RenameContainerBackup(containerName string, name string, b
 
 // DeleteContainerBackup requests that LXD deletes the container backup.
 func (r *ProtocolLXD) DeleteContainerBackup(containerName string, name string) (Operation, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
 	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/containers/%s/backups/%s",
-		url.PathEscape(containerName), url.PathEscape(name)), nil, "")
+		url.PathEscape(containerName), url.PathEscape(name)), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1736,8 +1782,9 @@ func (r *ProtocolLXD) DeleteContainerBackup(containerName string, name string) (
 
 // GetContainerBackupFile requests the container backup content.
 func (r *ProtocolLXD) GetContainerBackupFile(containerName string, name string, req *BackupFileRequest) (*BackupFileResponse, error) {
-	if !r.HasExtension("container_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"container_backup\" API extension")
+	err := r.CheckExtension("container_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Build the URL
@@ -1758,7 +1805,7 @@ func (r *ProtocolLXD) GetContainerBackupFile(containerName string, name string,
 	}
 
 	// Start the request
-	response, doneCh, err := cancel.CancelableDownload(req.Canceler, r.http, request)
+	response, doneCh, err := cancel.CancelableDownload(req.Canceler, r.DoHTTP, request)
 	if err != nil {
 		return nil, err
 	}
