@@ -4,21 +4,22 @@ import (
 	"fmt"
 	"net/url"
 
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // Project handling functions
 
 // GetProjectNames returns a list of available project names.
 func (r *ProtocolLXD) GetProjectNames() ([]string, error) {
-	if !r.HasExtension("projects") {
-		return nil, fmt.Errorf("The server is missing the required \"projects\" API extension")
+	err := r.CheckExtension("projects")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := "/projects"
-	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -29,14 +30,15 @@ func (r *ProtocolLXD) GetProjectNames() ([]string, error) {
 
 // GetProjects returns a list of available Project structs.
 func (r *ProtocolLXD) GetProjects() ([]api.Project, error) {
-	if !r.HasExtension("projects") {
-		return nil, fmt.Errorf("The server is missing the required \"projects\" API extension")
+	err := r.CheckExtension("projects")
+	if err != nil {
+		return nil, err
 	}
 
 	projects := []api.Project{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", "/projects?recursion=1", nil, "", &projects)
+	_, err = r.queryStruct("GET", "/projects?recursion=1", nil, "", &projects)
 	if err != nil {
 		return nil, err
 	}
@@ -46,8 +48,9 @@ func (r *ProtocolLXD) GetProjects() ([]api.Project, error) {
 
 // GetProject returns a Project entry for the provided name.
 func (r *ProtocolLXD) GetProject(name string) (*api.Project, string, error) {
-	if !r.HasExtension("projects") {
-		return nil, "", fmt.Errorf("The server is missing the required \"projects\" API extension")
+	err := r.CheckExtension("projects")
+	if err != nil {
+		return nil, "", err
 	}
 
 	project := api.Project{}
@@ -63,14 +66,15 @@ func (r *ProtocolLXD) GetProject(name string) (*api.Project, string, error) {
 
 // GetProjectState returns a Project state for the provided name.
 func (r *ProtocolLXD) GetProjectState(name string) (*api.ProjectState, error) {
-	if !r.HasExtension("project_usage") {
-		return nil, fmt.Errorf("The server is missing the required \"project_usage\" API extension")
+	err := r.CheckExtension("project_usage")
+	if err != nil {
+		return nil, err
 	}
 
 	projectState := api.ProjectState{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", fmt.Sprintf("/projects/%s/state", url.PathEscape(name)), nil, "", &projectState)
+	_, err = r.queryStruct("GET", fmt.Sprintf("/projects/%s/state", url.PathEscape(name)), nil, "", &projectState)
 	if err != nil {
 		return nil, err
 	}
@@ -80,12 +84,13 @@ func (r *ProtocolLXD) GetProjectState(name string) (*api.ProjectState, error) {
 
 // CreateProject defines a new container project.
 func (r *ProtocolLXD) CreateProject(project api.ProjectsPost) error {
-	if !r.HasExtension("projects") {
-		return fmt.Errorf("The server is missing the required \"projects\" API extension")
+	err := r.CheckExtension("projects")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
-	_, _, err := r.query("POST", "/projects", project, "")
+	_, _, err = r.query("POST", "/projects", project, "")
 	if err != nil {
 		return err
 	}
@@ -95,12 +100,13 @@ func (r *ProtocolLXD) CreateProject(project api.ProjectsPost) error {
 
 // UpdateProject updates the project to match the provided Project struct.
 func (r *ProtocolLXD) UpdateProject(name string, project api.ProjectPut, ETag string) error {
-	if !r.HasExtension("projects") {
-		return fmt.Errorf("The server is missing the required \"projects\" API extension")
+	err := r.CheckExtension("projects")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
-	_, _, err := r.query("PUT", fmt.Sprintf("/projects/%s", url.PathEscape(name)), project, ETag)
+	_, _, err = r.query("PUT", fmt.Sprintf("/projects/%s", url.PathEscape(name)), project, ETag)
 	if err != nil {
 		return err
 	}
@@ -110,12 +116,13 @@ func (r *ProtocolLXD) UpdateProject(name string, project api.ProjectPut, ETag st
 
 // RenameProject renames an existing project entry.
 func (r *ProtocolLXD) RenameProject(name string, project api.ProjectPost) (Operation, error) {
-	if !r.HasExtension("projects") {
-		return nil, fmt.Errorf("The server is missing the required \"projects\" API extension")
+	err := r.CheckExtension("projects")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/projects/%s", url.PathEscape(name)), project, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/projects/%s", url.PathEscape(name)), project, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -125,12 +132,13 @@ func (r *ProtocolLXD) RenameProject(name string, project api.ProjectPost) (Opera
 
 // DeleteProject deletes a project.
 func (r *ProtocolLXD) DeleteProject(name string) error {
-	if !r.HasExtension("projects") {
-		return fmt.Errorf("The server is missing the required \"projects\" API extension")
+	err := r.CheckExtension("projects")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
-	_, _, err := r.query("DELETE", fmt.Sprintf("/projects/%s", url.PathEscape(name)), nil, "")
+	_, _, err = r.query("DELETE", fmt.Sprintf("/projects/%s", url.PathEscape(name)), nil, "")
 	if err != nil {
 		return err
 	}
