@@ -7,8 +7,8 @@ import (
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // Server handling functions
@@ -34,7 +34,7 @@ func (r *ProtocolLXD) GetServer() (*api.Server, string, error) {
 
 	if !server.Public && len(server.AuthMethods) == 0 {
 		// TLS is always available for LXD servers
-		server.AuthMethods = []string{"tls"}
+		server.AuthMethods = []string{api.AuthenticationMethodTLS}
 	}
 
 	// Add the value to the cache
@@ -55,6 +55,7 @@ func (r *ProtocolLXD) UpdateServer(server api.ServerPut, ETag string) error {
 }
 
 // HasExtension returns true if the server supports a given API extension.
+// Deprecated: Use CheckExtension instead.
 func (r *ProtocolLXD) HasExtension(extension string) bool {
 	// If no cached API information, just assume we're good
 	// This is needed for those rare cases where we must avoid a GetServer call
@@ -62,13 +63,7 @@ func (r *ProtocolLXD) HasExtension(extension string) bool {
 		return true
 	}
 
-	for _, entry := range r.server.APIExtensions {
-		if entry == extension {
-			return true
-		}
-	}
-
-	return false
+	return shared.ValueInSlice(extension, r.server.APIExtensions)
 }
 
 // CheckExtension checks if the server has the specified extension.
@@ -87,14 +82,15 @@ func (r *ProtocolLXD) IsClustered() bool {
 
 // GetServerResources returns the resources available to a given LXD server.
 func (r *ProtocolLXD) GetServerResources() (*api.Resources, error) {
-	if !r.HasExtension("resources") {
-		return nil, fmt.Errorf("The server is missing the required \"resources\" API extension")
+	err := r.CheckExtension("resources")
+	if err != nil {
+		return nil, err
 	}
 
 	resources := api.Resources{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", "/resources", nil, "", &resources)
+	_, err = r.queryStruct("GET", "/resources", nil, "", &resources)
 	if err != nil {
 		return nil, err
 	}
@@ -114,13 +110,12 @@ func (r *ProtocolLXD) UseProject(name string) InstanceServer {
 		httpBaseURL:          r.httpBaseURL,
 		httpProtocol:         r.httpProtocol,
 		httpUserAgent:        r.httpUserAgent,
-		bakeryClient:         r.bakeryClient,
-		bakeryInteractor:     r.bakeryInteractor,
 		requireAuthenticated: r.requireAuthenticated,
 		clusterTarget:        r.clusterTarget,
 		project:              name,
 		eventConns:           make(map[string]*websocket.Conn),  // New project specific listener conns.
 		eventListeners:       make(map[string][]*EventListener), // New project specific listeners.
+		oidcClient:           r.oidcClient,
 	}
 }
 
@@ -138,12 +133,11 @@ func (r *ProtocolLXD) UseTarget(name string) InstanceServer {
 		httpBaseURL:          r.httpBaseURL,
 		httpProtocol:         r.httpProtocol,
 		httpUserAgent:        r.httpUserAgent,
-		bakeryClient:         r.bakeryClient,
-		bakeryInteractor:     r.bakeryInteractor,
 		requireAuthenticated: r.requireAuthenticated,
 		project:              r.project,
 		eventConns:           make(map[string]*websocket.Conn),  // New target specific listener conns.
 		eventListeners:       make(map[string][]*EventListener), // New target specific listeners.
+		oidcClient:           r.oidcClient,
 		clusterTarget:        name,
 	}
 }
@@ -156,8 +150,9 @@ func (r *ProtocolLXD) IsAgent() bool {
 // GetMetrics returns the text OpenMetrics data.
 func (r *ProtocolLXD) GetMetrics() (string, error) {
 	// Check that the server supports it.
-	if !r.HasExtension("metrics") {
-		return "", fmt.Errorf("The server is missing the required \"metrics\" API extension")
+	err := r.CheckExtension("metrics")
+	if err != nil {
+		return "", err
 	}
 
 	// Prepare the request.
