@@ -5,26 +5,28 @@ import (
 	"io"
 	"net/http"
 	"net/url"
+	"strings"
 
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/cancel"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/cancel"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/units"
 )
 
 // Storage volumes handling function
 
 // GetStoragePoolVolumeNames returns the names of all volumes in a pool.
 func (r *ProtocolLXD) GetStoragePoolVolumeNames(pool string) ([]string, error) {
-	if !r.HasExtension("storage") {
-		return nil, fmt.Errorf("The server is missing the required \"storage\" API extension")
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := fmt.Sprintf("/storage-pools/%s/volumes", url.PathEscape(pool))
-	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -34,7 +36,7 @@ func (r *ProtocolLXD) GetStoragePoolVolumeNames(pool string) ([]string, error) {
 }
 
 // GetStoragePoolVolumeNamesAllProjects returns the names of all volumes in a pool for all projects.
-func (r *ProtocolLXD) GetStoragePoolVolumeNamesAllProjects(pool string) ([]string, error) {
+func (r *ProtocolLXD) GetStoragePoolVolumeNamesAllProjects(pool string) (map[string][]string, error) {
 	err := r.CheckExtension("storage")
 	if err != nil {
 		return nil, err
@@ -53,20 +55,40 @@ func (r *ProtocolLXD) GetStoragePoolVolumeNamesAllProjects(pool string) ([]strin
 		return nil, err
 	}
 
-	// Parse it.
-	return urlsToResourceNames(u.String(), urls...)
+	names := make(map[string][]string)
+	for _, urlString := range urls {
+		resourceURL, err := url.Parse(urlString)
+		if err != nil {
+			return nil, fmt.Errorf("Could not parse unexpected URL %q: %w", urlString, err)
+		}
+
+		project := resourceURL.Query().Get("project")
+		if project == "" {
+			project = "default"
+		}
+
+		_, after, found := strings.Cut(resourceURL.Path, fmt.Sprintf("%s/", u.URL.Path))
+		if !found {
+			return nil, fmt.Errorf("Unexpected URL path %q", resourceURL)
+		}
+
+		names[project] = append(names[project], after)
+	}
+
+	return names, nil
 }
 
 // GetStoragePoolVolumes returns a list of StorageVolume entries for the provided pool.
 func (r *ProtocolLXD) GetStoragePoolVolumes(pool string) ([]api.StorageVolume, error) {
-	if !r.HasExtension("storage") {
-		return nil, fmt.Errorf("The server is missing the required \"storage\" API extension")
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return nil, err
 	}
 
 	volumes := []api.StorageVolume{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/volumes?recursion=1", url.PathEscape(pool)), nil, "", &volumes)
+	_, err = r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/volumes?recursion=1", url.PathEscape(pool)), nil, "", &volumes)
 	if err != nil {
 		return nil, err
 	}
@@ -101,10 +123,71 @@ func (r *ProtocolLXD) GetStoragePoolVolumesAllProjects(pool string) ([]api.Stora
 	return volumes, nil
 }
 
+// GetVolumesWithFilter returns a filtered list of StorageVolume entries for all storage pools.
+func (r *ProtocolLXD) GetVolumesWithFilter(filters []string) ([]api.StorageVolume, error) {
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return nil, err
+	}
+
+	err = r.CheckExtension("storage_volumes_all")
+	if err != nil {
+		return nil, err
+	}
+
+	volumes := []api.StorageVolume{}
+
+	url := api.NewURL().Path("storage-volumes").
+		WithQuery("recursion", "1").
+		WithQuery("filter", parseFilters(filters))
+
+	// Fetch the raw value
+	_, err = r.queryStruct(http.MethodGet, url.String(), nil, "", &volumes)
+	if err != nil {
+		return nil, err
+	}
+
+	return volumes, nil
+}
+
+// GetVolumesWithFilterAllProjects returns a filtered list of StorageVolume entries for all storage pools and for all projects.
+func (r *ProtocolLXD) GetVolumesWithFilterAllProjects(filters []string) ([]api.StorageVolume, error) {
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return nil, err
+	}
+
+	err = r.CheckExtension("storage_volumes_all_projects")
+	if err != nil {
+		return nil, err
+	}
+
+	err = r.CheckExtension("storage_volumes_all")
+	if err != nil {
+		return nil, err
+	}
+
+	volumes := []api.StorageVolume{}
+
+	url := api.NewURL().Path("storage-volumes").
+		WithQuery("recursion", "1").
+		WithQuery("filter", parseFilters(filters)).
+		WithQuery("all-projects", "true")
+
+	// Fetch the raw value.
+	_, err = r.queryStruct(http.MethodGet, url.String(), nil, "", &volumes)
+	if err != nil {
+		return nil, err
+	}
+
+	return volumes, nil
+}
+
 // GetStoragePoolVolumesWithFilter returns a filtered list of StorageVolume entries for the provided pool.
 func (r *ProtocolLXD) GetStoragePoolVolumesWithFilter(pool string, filters []string) ([]api.StorageVolume, error) {
-	if !r.HasExtension("storage") {
-		return nil, fmt.Errorf("The server is missing the required \"storage\" API extension")
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return nil, err
 	}
 
 	volumes := []api.StorageVolume{}
@@ -113,7 +196,7 @@ func (r *ProtocolLXD) GetStoragePoolVolumesWithFilter(pool string, filters []str
 	v.Set("recursion", "1")
 	v.Set("filter", parseFilters(filters))
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/volumes?%s", url.PathEscape(pool), v.Encode()), nil, "", &volumes)
+	_, err = r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/volumes?%s", url.PathEscape(pool), v.Encode()), nil, "", &volumes)
 	if err != nil {
 		return nil, err
 	}
@@ -151,8 +234,9 @@ func (r *ProtocolLXD) GetStoragePoolVolumesWithFilterAllProjects(pool string, fi
 
 // GetStoragePoolVolume returns a StorageVolume entry for the provided pool and volume name.
 func (r *ProtocolLXD) GetStoragePoolVolume(pool string, volType string, name string) (*api.StorageVolume, string, error) {
-	if !r.HasExtension("storage") {
-		return nil, "", fmt.Errorf("The server is missing the required \"storage\" API extension")
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return nil, "", err
 	}
 
 	volume := api.StorageVolume{}
@@ -169,14 +253,15 @@ func (r *ProtocolLXD) GetStoragePoolVolume(pool string, volType string, name str
 
 // GetStoragePoolVolumeState returns a StorageVolumeState entry for the provided pool and volume name.
 func (r *ProtocolLXD) GetStoragePoolVolumeState(pool string, volType string, name string) (*api.StorageVolumeState, error) {
-	if !r.HasExtension("storage_volume_state") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_volume_state\" API extension")
+	err := r.CheckExtension("storage_volume_state")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw value
 	state := api.StorageVolumeState{}
 	path := fmt.Sprintf("/storage-pools/%s/volumes/%s/%s/state", url.PathEscape(pool), url.PathEscape(volType), url.PathEscape(name))
-	_, err := r.queryStruct("GET", path, nil, "", &state)
+	_, err = r.queryStruct("GET", path, nil, "", &state)
 	if err != nil {
 		return nil, err
 	}
@@ -186,13 +271,14 @@ func (r *ProtocolLXD) GetStoragePoolVolumeState(pool string, volType string, nam
 
 // CreateStoragePoolVolume defines a new storage volume.
 func (r *ProtocolLXD) CreateStoragePoolVolume(pool string, volume api.StorageVolumesPost) error {
-	if !r.HasExtension("storage") {
-		return fmt.Errorf("The server is missing the required \"storage\" API extension")
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
 	path := fmt.Sprintf("/storage-pools/%s/volumes/%s", url.PathEscape(pool), url.PathEscape(volume.Type))
-	_, _, err := r.query("POST", path, volume, "")
+	_, _, err = r.query("POST", path, volume, "")
 	if err != nil {
 		return err
 	}
@@ -202,8 +288,9 @@ func (r *ProtocolLXD) CreateStoragePoolVolume(pool string, volume api.StorageVol
 
 // CreateStoragePoolVolumeSnapshot defines a new storage volume.
 func (r *ProtocolLXD) CreateStoragePoolVolumeSnapshot(pool string, volumeType string, volumeName string, snapshot api.StorageVolumeSnapshotsPost) (Operation, error) {
-	if !r.HasExtension("storage_api_volume_snapshots") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	err := r.CheckExtension("storage_api_volume_snapshots")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
@@ -211,7 +298,7 @@ func (r *ProtocolLXD) CreateStoragePoolVolumeSnapshot(pool string, volumeType st
 		url.PathEscape(pool),
 		url.PathEscape(volumeType),
 		url.PathEscape(volumeName))
-	op, _, err := r.queryOperation("POST", path, snapshot, "")
+	op, _, err := r.queryOperation("POST", path, snapshot, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -222,14 +309,15 @@ func (r *ProtocolLXD) CreateStoragePoolVolumeSnapshot(pool string, volumeType st
 // GetStoragePoolVolumeSnapshotNames returns a list of snapshot names for the
 // storage volume.
 func (r *ProtocolLXD) GetStoragePoolVolumeSnapshotNames(pool string, volumeType string, volumeName string) ([]string, error) {
-	if !r.HasExtension("storage_api_volume_snapshots") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	err := r.CheckExtension("storage_api_volume_snapshots")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := fmt.Sprintf("/storage-pools/%s/volumes/%s/%s/snapshots", url.PathEscape(pool), url.PathEscape(volumeType), url.PathEscape(volumeName))
-	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -241,8 +329,9 @@ func (r *ProtocolLXD) GetStoragePoolVolumeSnapshotNames(pool string, volumeType
 // GetStoragePoolVolumeSnapshots returns a list of snapshots for the storage
 // volume.
 func (r *ProtocolLXD) GetStoragePoolVolumeSnapshots(pool string, volumeType string, volumeName string) ([]api.StorageVolumeSnapshot, error) {
-	if !r.HasExtension("storage_api_volume_snapshots") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	err := r.CheckExtension("storage_api_volume_snapshots")
+	if err != nil {
+		return nil, err
 	}
 
 	snapshots := []api.StorageVolumeSnapshot{}
@@ -251,7 +340,7 @@ func (r *ProtocolLXD) GetStoragePoolVolumeSnapshots(pool string, volumeType stri
 		url.PathEscape(pool),
 		url.PathEscape(volumeType),
 		url.PathEscape(volumeName))
-	_, err := r.queryStruct("GET", path, nil, "", &snapshots)
+	_, err = r.queryStruct("GET", path, nil, "", &snapshots)
 	if err != nil {
 		return nil, err
 	}
@@ -261,8 +350,9 @@ func (r *ProtocolLXD) GetStoragePoolVolumeSnapshots(pool string, volumeType stri
 
 // GetStoragePoolVolumeSnapshot returns a snapshots for the storage volume.
 func (r *ProtocolLXD) GetStoragePoolVolumeSnapshot(pool string, volumeType string, volumeName string, snapshotName string) (*api.StorageVolumeSnapshot, string, error) {
-	if !r.HasExtension("storage_api_volume_snapshots") {
-		return nil, "", fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	err := r.CheckExtension("storage_api_volume_snapshots")
+	if err != nil {
+		return nil, "", err
 	}
 
 	snapshot := api.StorageVolumeSnapshot{}
@@ -282,13 +372,14 @@ func (r *ProtocolLXD) GetStoragePoolVolumeSnapshot(pool string, volumeType strin
 
 // RenameStoragePoolVolumeSnapshot renames a storage volume snapshot.
 func (r *ProtocolLXD) RenameStoragePoolVolumeSnapshot(pool string, volumeType string, volumeName string, snapshotName string, snapshot api.StorageVolumeSnapshotPost) (Operation, error) {
-	if !r.HasExtension("storage_api_volume_snapshots") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	err := r.CheckExtension("storage_api_volume_snapshots")
+	if err != nil {
+		return nil, err
 	}
 
 	path := fmt.Sprintf("/storage-pools/%s/volumes/%s/%s/snapshots/%s", url.PathEscape(pool), url.PathEscape(volumeType), url.PathEscape(volumeName), url.PathEscape(snapshotName))
 	// Send the request
-	op, _, err := r.queryOperation("POST", path, snapshot, "")
+	op, _, err := r.queryOperation("POST", path, snapshot, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -298,8 +389,9 @@ func (r *ProtocolLXD) RenameStoragePoolVolumeSnapshot(pool string, volumeType st
 
 // DeleteStoragePoolVolumeSnapshot deletes a storage volume snapshot.
 func (r *ProtocolLXD) DeleteStoragePoolVolumeSnapshot(pool string, volumeType string, volumeName string, snapshotName string) (Operation, error) {
-	if !r.HasExtension("storage_api_volume_snapshots") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	err := r.CheckExtension("storage_api_volume_snapshots")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
@@ -307,7 +399,7 @@ func (r *ProtocolLXD) DeleteStoragePoolVolumeSnapshot(pool string, volumeType st
 		"/storage-pools/%s/volumes/%s/%s/snapshots/%s",
 		url.PathEscape(pool), url.PathEscape(volumeType), url.PathEscape(volumeName), url.PathEscape(snapshotName))
 
-	op, _, err := r.queryOperation("DELETE", path, nil, "")
+	op, _, err := r.queryOperation("DELETE", path, nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -317,13 +409,14 @@ func (r *ProtocolLXD) DeleteStoragePoolVolumeSnapshot(pool string, volumeType st
 
 // UpdateStoragePoolVolumeSnapshot updates the volume to match the provided StoragePoolVolume struct.
 func (r *ProtocolLXD) UpdateStoragePoolVolumeSnapshot(pool string, volumeType string, volumeName string, snapshotName string, volume api.StorageVolumeSnapshotPut, ETag string) error {
-	if !r.HasExtension("storage_api_volume_snapshots") {
-		return fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	err := r.CheckExtension("storage_api_volume_snapshots")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
 	path := fmt.Sprintf("/storage-pools/%s/volumes/%s/%s/snapshots/%s", url.PathEscape(pool), url.PathEscape(volumeType), url.PathEscape(volumeName), url.PathEscape(snapshotName))
-	_, _, err := r.queryOperation("PUT", path, volume, ETag)
+	_, _, err = r.queryOperation("PUT", path, volume, ETag, true)
 	if err != nil {
 		return err
 	}
@@ -333,8 +426,9 @@ func (r *ProtocolLXD) UpdateStoragePoolVolumeSnapshot(pool string, volumeType st
 
 // MigrateStoragePoolVolume requests that LXD prepares for a storage volume migration.
 func (r *ProtocolLXD) MigrateStoragePoolVolume(pool string, volume api.StorageVolumePost) (Operation, error) {
-	if !r.HasExtension("storage_api_remote_volume_handling") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_remote_volume_handling\" API extension")
+	err := r.CheckExtension("storage_api_remote_volume_handling")
+	if err != nil {
+		return nil, err
 	}
 
 	// Quick check.
@@ -342,9 +436,31 @@ func (r *ProtocolLXD) MigrateStoragePoolVolume(pool string, volume api.StorageVo
 		return nil, fmt.Errorf("Can't ask for a rename through MigrateStoragePoolVolume")
 	}
 
+	var req any
+	var path string
+
+	srcVolParentName, srcVolSnapName, srcIsSnapshot := api.GetParentAndSnapshotName(volume.Name)
+	if srcIsSnapshot {
+		err := r.CheckExtension("storage_api_remote_volume_snapshot_copy")
+		if err != nil {
+			return nil, err
+		}
+
+		// Set the actual name of the snapshot without delimiter.
+		req = api.StorageVolumeSnapshotPost{
+			Name:      srcVolSnapName,
+			Migration: volume.Migration,
+			Target:    volume.Target,
+		}
+
+		path = api.NewURL().Path("storage-pools", pool, "volumes", "custom", srcVolParentName, "snapshots", srcVolSnapName).String()
+	} else {
+		req = volume
+		path = api.NewURL().Path("storage-pools", pool, "volumes", "custom", volume.Name).String()
+	}
+
 	// Send the request
-	path := fmt.Sprintf("/storage-pools/%s/volumes/custom/%s", url.PathEscape(pool), volume.Name)
-	op, _, err := r.queryOperation("POST", path, volume, "")
+	op, _, err := r.queryOperation("POST", path, req, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -411,6 +527,8 @@ func (r *ProtocolLXD) tryMigrateStoragePoolVolume(source InstanceServer, pool st
 	return &rop, nil
 }
 
+// tryCreateStoragePoolVolume attempts to create a storage volume in the specified storage pool.
+// It will try to do this on every server in the provided list of urls, and waits for the creation to be complete.
 func (r *ProtocolLXD) tryCreateStoragePoolVolume(pool string, req api.StorageVolumesPost, urls []string) (RemoteOperation, error) {
 	if len(urls) == 0 {
 		return nil, fmt.Errorf("The source server isn't listening on the network")
@@ -431,7 +549,7 @@ func (r *ProtocolLXD) tryCreateStoragePoolVolume(pool string, req api.StorageVol
 
 			// Send the request
 			path := fmt.Sprintf("/storage-pools/%s/volumes/%s", url.PathEscape(pool), url.PathEscape(req.Type))
-			top, _, err := r.queryOperation("POST", path, req, "")
+			top, _, err := r.queryOperation("POST", path, req, "", true)
 			if err != nil {
 				errors = append(errors, remoteOperationResult{URL: serverURL, Error: err})
 				continue
@@ -473,15 +591,16 @@ func (r *ProtocolLXD) tryCreateStoragePoolVolume(pool string, req api.StorageVol
 
 // CopyStoragePoolVolume copies an existing storage volume.
 func (r *ProtocolLXD) CopyStoragePoolVolume(pool string, source InstanceServer, sourcePool string, volume api.StorageVolume, args *StoragePoolVolumeCopyArgs) (RemoteOperation, error) {
-	if !r.HasExtension("storage_api_local_volume_handling") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_local_volume_handling\" API extension")
+	err := r.CheckExtension("storage_api_local_volume_handling")
+	if err != nil {
+		return nil, err
 	}
 
-	if args != nil && args.VolumeOnly && !r.HasExtension("storage_api_volume_snapshots") {
+	if args != nil && args.VolumeOnly && r.CheckExtension("storage_api_volume_snapshots") != nil {
 		return nil, fmt.Errorf("The target server is missing the required \"storage_api_volume_snapshots\" API extension")
 	}
 
-	if args != nil && args.Refresh && !r.HasExtension("custom_volume_refresh") {
+	if args != nil && args.Refresh && r.CheckExtension("custom_volume_refresh") != nil {
 		return nil, fmt.Errorf("The target server is missing the required \"custom_volume_refresh\" API extension")
 	}
 
@@ -511,18 +630,26 @@ func (r *ProtocolLXD) CopyStoragePoolVolume(pool string, source InstanceServer,
 		return nil, fmt.Errorf("Failed to get destination connection info: %w", err)
 	}
 
-	if destInfo.URL == sourceInfo.URL && destInfo.SocketPath == sourceInfo.SocketPath && volume.Location == r.clusterTarget {
+	clusterInternalVolumeCopy := r.CheckExtension("cluster_internal_custom_volume_copy") == nil
+
+	// Copy the storage pool volume locally.
+	if destInfo.URL == sourceInfo.URL && destInfo.SocketPath == sourceInfo.SocketPath && (volume.Location == r.clusterTarget || (volume.Location == "none" && r.clusterTarget == "") || clusterInternalVolumeCopy) {
 		// Project handling
 		if destInfo.Project != sourceInfo.Project {
-			if !r.HasExtension("storage_api_project") {
-				return nil, fmt.Errorf("The server is missing the required \"storage_api_project\" API extension")
+			err := r.CheckExtension("storage_api_project")
+			if err != nil {
+				return nil, err
 			}
 
 			req.Source.Project = sourceInfo.Project
 		}
 
+		if clusterInternalVolumeCopy {
+			req.Source.Location = sourceInfo.Target
+		}
+
 		// Send the request
-		op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/%s", url.PathEscape(pool), url.PathEscape(volume.Type)), req, "")
+		op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/%s", url.PathEscape(pool), url.PathEscape(volume.Type)), req, "", true)
 		if err != nil {
 			return nil, err
 		}
@@ -541,8 +668,9 @@ func (r *ProtocolLXD) CopyStoragePoolVolume(pool string, source InstanceServer,
 		return &rop, nil
 	}
 
-	if !r.HasExtension("storage_api_remote_volume_handling") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_remote_volume_handling\" API extension")
+	err = r.CheckExtension("storage_api_remote_volume_handling")
+	if err != nil {
+		return nil, err
 	}
 
 	sourceReq := api.StorageVolumePost{
@@ -571,7 +699,7 @@ func (r *ProtocolLXD) CopyStoragePoolVolume(pool string, source InstanceServer,
 		path := fmt.Sprintf("/storage-pools/%s/volumes/%s", url.PathEscape(pool), url.PathEscape(volume.Type))
 
 		// Send the request
-		op, _, err := r.queryOperation("POST", path, req, "")
+		op, _, err := r.queryOperation("POST", path, req, "", true)
 		if err != nil {
 			return nil, err
 		}
@@ -623,7 +751,7 @@ func (r *ProtocolLXD) CopyStoragePoolVolume(pool string, source InstanceServer,
 		path := fmt.Sprintf("/storage-pools/%s/volumes/%s", url.PathEscape(pool), url.PathEscape(volume.Type))
 
 		// Send the request
-		targetOp, _, err := r.queryOperation("POST", path, req, "")
+		targetOp, _, err := r.queryOperation("POST", path, req, "", true)
 		if err != nil {
 			return nil, err
 		}
@@ -669,8 +797,9 @@ func (r *ProtocolLXD) CopyStoragePoolVolume(pool string, source InstanceServer,
 
 // MoveStoragePoolVolume renames or moves an existing storage volume.
 func (r *ProtocolLXD) MoveStoragePoolVolume(pool string, source InstanceServer, sourcePool string, volume api.StorageVolume, args *StoragePoolVolumeMoveArgs) (RemoteOperation, error) {
-	if !r.HasExtension("storage_api_local_volume_handling") {
-		return nil, fmt.Errorf("The server is missing the required \"storage_api_local_volume_handling\" API extension")
+	err := r.CheckExtension("storage_api_local_volume_handling")
+	if err != nil {
+		return nil, err
 	}
 
 	if r != source {
@@ -683,15 +812,16 @@ func (r *ProtocolLXD) MoveStoragePoolVolume(pool string, source InstanceServer,
 	}
 
 	if args.Project != "" {
-		if !r.HasExtension("storage_volume_project_move") {
-			return nil, fmt.Errorf("The server is missing the required \"storage_volume_project_move\" API extension")
+		err := r.CheckExtension("storage_volume_project_move")
+		if err != nil {
+			return nil, err
 		}
 
 		req.Project = args.Project
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/%s/%s", url.PathEscape(sourcePool), url.PathEscape(volume.Type), volume.Name), req, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/%s/%s", url.PathEscape(sourcePool), url.PathEscape(volume.Type), volume.Name), req, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -712,17 +842,21 @@ func (r *ProtocolLXD) MoveStoragePoolVolume(pool string, source InstanceServer,
 
 // UpdateStoragePoolVolume updates the volume to match the provided StoragePoolVolume struct.
 func (r *ProtocolLXD) UpdateStoragePoolVolume(pool string, volType string, name string, volume api.StorageVolumePut, ETag string) error {
-	if !r.HasExtension("storage") {
-		return fmt.Errorf("The server is missing the required \"storage\" API extension")
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return err
 	}
 
-	if volume.Restore != "" && !r.HasExtension("storage_api_volume_snapshots") {
-		return fmt.Errorf("The server is missing the required \"storage_api_volume_snapshots\" API extension")
+	if volume.Restore != "" {
+		err := r.CheckExtension("storage_api_volume_snapshots")
+		if err != nil {
+			return err
+		}
 	}
 
 	// Send the request
 	path := fmt.Sprintf("/storage-pools/%s/volumes/%s/%s", url.PathEscape(pool), url.PathEscape(volType), url.PathEscape(name))
-	_, _, err := r.query("PUT", path, volume, ETag)
+	_, _, err = r.query("PUT", path, volume, ETag)
 	if err != nil {
 		return err
 	}
@@ -732,13 +866,14 @@ func (r *ProtocolLXD) UpdateStoragePoolVolume(pool string, volType string, name
 
 // DeleteStoragePoolVolume deletes a storage pool.
 func (r *ProtocolLXD) DeleteStoragePoolVolume(pool string, volType string, name string) error {
-	if !r.HasExtension("storage") {
-		return fmt.Errorf("The server is missing the required \"storage\" API extension")
+	err := r.CheckExtension("storage")
+	if err != nil {
+		return err
 	}
 
 	// Send the request
 	path := fmt.Sprintf("/storage-pools/%s/volumes/%s/%s", url.PathEscape(pool), url.PathEscape(volType), url.PathEscape(name))
-	_, _, err := r.query("DELETE", path, nil, "")
+	_, _, err = r.query("DELETE", path, nil, "")
 	if err != nil {
 		return err
 	}
@@ -748,14 +883,15 @@ func (r *ProtocolLXD) DeleteStoragePoolVolume(pool string, volType string, name
 
 // RenameStoragePoolVolume renames a storage volume.
 func (r *ProtocolLXD) RenameStoragePoolVolume(pool string, volType string, name string, volume api.StorageVolumePost) error {
-	if !r.HasExtension("storage_api_volume_rename") {
-		return fmt.Errorf("The server is missing the required \"storage_api_volume_rename\" API extension")
+	err := r.CheckExtension("storage_api_volume_rename")
+	if err != nil {
+		return err
 	}
 
 	path := fmt.Sprintf("/storage-pools/%s/volumes/%s/%s", url.PathEscape(pool), url.PathEscape(volType), url.PathEscape(name))
 
 	// Send the request
-	_, _, err := r.query("POST", path, volume, "")
+	_, _, err = r.query("POST", path, volume, "")
 	if err != nil {
 		return err
 	}
@@ -765,14 +901,15 @@ func (r *ProtocolLXD) RenameStoragePoolVolume(pool string, volType string, name
 
 // GetStoragePoolVolumeBackupNames returns a list of volume backup names.
 func (r *ProtocolLXD) GetStoragePoolVolumeBackupNames(pool string, volName string) ([]string, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"custom_volume_backup\" API extension")
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups", url.PathEscape(pool), url.PathEscape(volName))
-	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -783,14 +920,15 @@ func (r *ProtocolLXD) GetStoragePoolVolumeBackupNames(pool string, volName strin
 
 // GetStoragePoolVolumeBackups returns a list of custom volume backups.
 func (r *ProtocolLXD) GetStoragePoolVolumeBackups(pool string, volName string) ([]api.StoragePoolVolumeBackup, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"custom_volume_backup\" API extension")
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Fetch the raw value
 	backups := []api.StoragePoolVolumeBackup{}
 
-	_, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups?recursion=1", url.PathEscape(pool), url.PathEscape(volName)), nil, "", &backups)
+	_, err = r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups?recursion=1", url.PathEscape(pool), url.PathEscape(volName)), nil, "", &backups)
 	if err != nil {
 		return nil, err
 	}
@@ -800,8 +938,9 @@ func (r *ProtocolLXD) GetStoragePoolVolumeBackups(pool string, volName string) (
 
 // GetStoragePoolVolumeBackup returns a custom volume backup.
 func (r *ProtocolLXD) GetStoragePoolVolumeBackup(pool string, volName string, name string) (*api.StoragePoolVolumeBackup, string, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, "", fmt.Errorf("The server is missing the required \"custom_volume_backup\" API extension")
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, "", err
 	}
 
 	// Fetch the raw value
@@ -816,12 +955,13 @@ func (r *ProtocolLXD) GetStoragePoolVolumeBackup(pool string, volName string, na
 
 // CreateStoragePoolVolumeBackup creates new custom volume backup.
 func (r *ProtocolLXD) CreateStoragePoolVolumeBackup(pool string, volName string, backup api.StoragePoolVolumeBackupsPost) (Operation, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"custom_volume_backup\" API extension")
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups", url.PathEscape(pool), url.PathEscape(volName)), backup, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups", url.PathEscape(pool), url.PathEscape(volName)), backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -831,12 +971,13 @@ func (r *ProtocolLXD) CreateStoragePoolVolumeBackup(pool string, volName string,
 
 // RenameStoragePoolVolumeBackup renames a custom volume backup.
 func (r *ProtocolLXD) RenameStoragePoolVolumeBackup(pool string, volName string, name string, backup api.StoragePoolVolumeBackupPost) (Operation, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"custom_volume_backup\" API extension")
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups/%s", url.PathEscape(pool), url.PathEscape(volName), url.PathEscape(name)), backup, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups/%s", url.PathEscape(pool), url.PathEscape(volName), url.PathEscape(name)), backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -846,12 +987,13 @@ func (r *ProtocolLXD) RenameStoragePoolVolumeBackup(pool string, volName string,
 
 // DeleteStoragePoolVolumeBackup deletes a custom volume backup.
 func (r *ProtocolLXD) DeleteStoragePoolVolumeBackup(pool string, volName string, name string) (Operation, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"custom_volume_backup\" API extension")
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups/%s", url.PathEscape(pool), url.PathEscape(volName), url.PathEscape(name)), nil, "")
+	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/storage-pools/%s/volumes/custom/%s/backups/%s", url.PathEscape(pool), url.PathEscape(volName), url.PathEscape(name)), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -861,8 +1003,9 @@ func (r *ProtocolLXD) DeleteStoragePoolVolumeBackup(pool string, volName string,
 
 // GetStoragePoolVolumeBackupFile requests the custom volume backup content.
 func (r *ProtocolLXD) GetStoragePoolVolumeBackupFile(pool string, volName string, name string, req *BackupFileRequest) (*BackupFileResponse, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, fmt.Errorf("The server is missing the required \"custom_volume_backup\" API extension")
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, err
 	}
 
 	// Build the URL
@@ -883,7 +1026,7 @@ func (r *ProtocolLXD) GetStoragePoolVolumeBackupFile(pool string, volName string
 	}
 
 	// Start the request
-	response, doneCh, err := cancel.CancelableDownload(req.Canceler, r.http, request)
+	response, doneCh, err := cancel.CancelableDownload(req.Canceler, r.DoHTTP, request)
 	if err != nil {
 		return nil, err
 	}
@@ -923,14 +1066,76 @@ func (r *ProtocolLXD) GetStoragePoolVolumeBackupFile(pool string, volName string
 	return &resp, nil
 }
 
-// CreateStoragePoolVolumeFromBackup creates a custom volume from a backup file.
-func (r *ProtocolLXD) CreateStoragePoolVolumeFromBackup(pool string, args StoragePoolVolumeBackupArgs) (Operation, error) {
-	if !r.HasExtension("custom_volume_backup") {
-		return nil, fmt.Errorf(`The server is missing the required "custom_volume_backup" API extension`)
+// CreateStoragePoolVolumeFromISO creates a custom volume from an ISO file.
+func (r *ProtocolLXD) CreateStoragePoolVolumeFromISO(pool string, args StoragePoolVolumeBackupArgs) (Operation, error) {
+	err := r.CheckExtension("custom_volume_iso")
+	if err != nil {
+		return nil, err
 	}
 
-	if args.Name != "" && !r.HasExtension("backup_override_name") {
-		return nil, fmt.Errorf(`The server is missing the required "backup_override_name" API extension`)
+	path := fmt.Sprintf("/storage-pools/%s/volumes/custom", url.PathEscape(pool))
+
+	// Prepare the HTTP request.
+	reqURL, err := r.setQueryAttributes(fmt.Sprintf("%s/1.0%s", r.httpBaseURL.String(), path))
+	if err != nil {
+		return nil, err
+	}
+
+	req, err := http.NewRequest("POST", reqURL, args.BackupFile)
+	if err != nil {
+		return nil, err
+	}
+
+	if args.Name == "" {
+		return nil, fmt.Errorf("Missing volume name")
+	}
+
+	req.Header.Set("Content-Type", "application/octet-stream")
+	req.Header.Set("X-LXD-name", args.Name)
+	req.Header.Set("X-LXD-type", "iso")
+
+	// Send the request.
+	resp, err := r.DoHTTP(req)
+	if err != nil {
+		return nil, err
+	}
+
+	defer func() { _ = resp.Body.Close() }()
+
+	// Handle errors.
+	response, _, err := lxdParseResponse(resp)
+	if err != nil {
+		return nil, err
+	}
+
+	// Get to the operation.
+	respOperation, err := response.MetadataAsOperation()
+	if err != nil {
+		return nil, err
+	}
+
+	// Setup an Operation wrapper.
+	op := operation{
+		Operation: *respOperation,
+		r:         r,
+		chActive:  make(chan bool),
+	}
+
+	return &op, nil
+}
+
+// CreateStoragePoolVolumeFromBackup creates a custom volume from a backup file.
+func (r *ProtocolLXD) CreateStoragePoolVolumeFromBackup(pool string, args StoragePoolVolumeBackupArgs) (Operation, error) {
+	err := r.CheckExtension("custom_volume_backup")
+	if err != nil {
+		return nil, err
+	}
+
+	if args.Name != "" {
+		err := r.CheckExtension("backup_override_name")
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	path := fmt.Sprintf("/storage-pools/%s/volumes/custom", url.PathEscape(pool))
