@@ -4,7 +4,7 @@ import (
 	"fmt"
 	"net/http"
 
-	"github.com/lxc/lxd/shared/simplestreams"
+	"github.com/canonical/lxd/shared/simplestreams"
 )
 
 // ProtocolSimpleStreams implements a SimpleStreams API client.
@@ -41,7 +41,7 @@ func (r *ProtocolSimpleStreams) GetHTTPClient() (*http.Client, error) {
 	return r.http, nil
 }
 
-// DoHTTP performs a Request, using macaroon authentication if set.
+// DoHTTP performs a Request.
 func (r *ProtocolSimpleStreams) DoHTTP(req *http.Request) (*http.Response, error) {
 	// Set the user agent
 	if r.httpUserAgent != "" {
