@@ -7,13 +7,20 @@ import (
 	"net"
 	"net/http"
 	"net/url"
+	"os"
+	"os/exec"
+	"runtime"
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/shared"
+	"github.com/canonical/lxd/shared"
 )
 
-func tlsHTTPClient(client *http.Client, tlsClientCert string, tlsClientKey string, tlsCA string, tlsServerCert string, insecureSkipVerify bool, proxy func(req *http.Request) (*url.URL, error)) (*http.Client, error) {
+// tlsHTTPClient creates an HTTP client with a specified Transport Layer Security (TLS) configuration.
+// It takes in parameters for client certificates, keys, Certificate Authority, server certificates,
+// a boolean for skipping verification, a proxy function, and a transport wrapper function.
+// It returns the HTTP client with the provided configurations and handles any errors that might occur during the setup process.
+func tlsHTTPClient(client *http.Client, tlsClientCert string, tlsClientKey string, tlsCA string, tlsServerCert string, insecureSkipVerify bool, proxy func(req *http.Request) (*url.URL, error), transportWrapper func(t *http.Transport) HTTPTransporter) (*http.Client, error) {
 	// Get the TLS configuration
 	tlsConfig, err := shared.GetTLSConfigMem(tlsClientCert, tlsClientKey, tlsCA, tlsServerCert, insecureSkipVerify)
 	if err != nil {
@@ -88,7 +95,11 @@ func tlsHTTPClient(client *http.Client, tlsClientCert string, tlsClientKey strin
 		client = &http.Client{}
 	}
 
-	client.Transport = transport
+	if transportWrapper != nil {
+		client.Transport = transportWrapper(transport)
+	} else {
+		client.Transport = transport
+	}
 
 	// Setup redirect policy
 	client.CheckRedirect = func(req *http.Request, via []*http.Request) error {
@@ -101,6 +112,10 @@ func tlsHTTPClient(client *http.Client, tlsClientCert string, tlsClientKey strin
 	return client, nil
 }
 
+// unixHTTPClient creates an HTTP client that communicates over a Unix socket.
+// It takes in the connection arguments and the Unix socket path as parameters.
+// The function sets up a Unix socket dialer, configures the HTTP transport, and returns the HTTP client with the specified configurations.
+// Any errors encountered during the setup process are also handled by the function.
 func unixHTTPClient(args *ConnectionArgs, path string) (*http.Client, error) {
 	// Setup a Unix socket dialer
 	unixDial := func(_ context.Context, network, addr string) (net.Conn, error) {
@@ -206,12 +221,12 @@ func urlsToResourceNames(matchPathPrefix string, urls ...string) ([]string, erro
 			return nil, fmt.Errorf("Failed parsing URL %q: %w", urlRaw, err)
 		}
 
-		fields := strings.Split(u.Path, fmt.Sprintf("%s/", matchPathPrefix))
-		if len(fields) != 2 {
+		_, after, found := strings.Cut(u.Path, fmt.Sprintf("%s/", matchPathPrefix))
+		if !found {
 			return nil, fmt.Errorf("Unexpected URL path %q", u)
 		}
 
-		resourceNames = append(resourceNames, fields[len(fields)-1])
+		resourceNames = append(resourceNames, after)
 	}
 
 	return resourceNames, nil
@@ -228,3 +243,43 @@ func parseFilters(filters []string) string {
 	}
 	return strings.Join(result, " and ")
 }
+
+// HTTPTransporter represents a wrapper around *http.Transport.
+// It is used to add some pre and postprocessing logic to http requests / responses.
+type HTTPTransporter interface {
+	http.RoundTripper
+
+	// Transport what this struct wraps
+	Transport() *http.Transport
+}
+
+func openBrowser(url string) error {
+	var err error
+
+	browser := os.Getenv("BROWSER")
+	if browser != "" {
+		if browser == "none" {
+			return nil
+		}
+
+		err = exec.Command(browser, url).Start()
+		return err
+	}
+
+	switch runtime.GOOS {
+	case "linux":
+		err = exec.Command("xdg-open", url).Start()
+	case "windows":
+		err = exec.Command("rundll32", "url.dll,FileProtocolHandler", url).Start()
+	case "darwin":
+		err = exec.Command("open", url).Start()
+	default:
+		err = fmt.Errorf("unsupported platform")
+	}
+
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
