@@ -11,12 +11,12 @@ import (
 	liblxc "github.com/lxc/go-lxc"
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/osarch"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/osarch"
 )
 
 type cmdMigrate struct {
@@ -72,7 +72,7 @@ func (c *cmdMigrate) RunE(cmd *cobra.Command, args []string) error {
 
 	// Retrieve LXC containers
 	for _, container := range liblxc.Containers(c.flagLXCPath) {
-		if !c.flagAll && !shared.StringInSlice(container.Name(), c.flagContainers) {
+		if !c.flagAll && !shared.ValueInSlice(container.Name(), c.flagContainers) {
 			continue
 		}
 
@@ -335,7 +335,7 @@ func convertContainer(d lxd.ContainerServer, container *liblxc.Container, storag
 	if value != nil {
 		for _, cap := range strings.Split(value[0], " ") {
 			// Ignore capabilities that are dropped in LXD containers by default.
-			if shared.StringInSlice(cap, []string{"mac_admin", "mac_override", "sys_module",
+			if shared.ValueInSlice(cap, []string{"mac_admin", "mac_override", "sys_module",
 				"sys_time"}) {
 				continue
 			}
@@ -462,7 +462,7 @@ func convertContainer(d lxd.ContainerServer, container *liblxc.Container, storag
 			return err
 		}
 
-		progress := utils.ProgressRenderer{Format: "Transferring container: %s"}
+		progress := cli.ProgressRenderer{Format: "Transferring container: %s"}
 		_, err = op.AddHandler(progress.UpdateOp)
 		if err != nil {
 			progress.Done("")
@@ -570,7 +570,7 @@ func convertStorageConfig(conf []string, devices map[string]map[string]string) e
 		}
 
 		// Ignore mounts that are present in LXD containers by default.
-		if shared.StringInSlice(parts[0], []string{"proc", "sysfs"}) {
+		if shared.ValueInSlice(parts[0], []string{"proc", "sysfs"}) {
 			continue
 		}
 
@@ -578,12 +578,12 @@ func convertStorageConfig(conf []string, devices map[string]map[string]string) e
 		device["type"] = "disk"
 
 		// Deal with read-only mounts
-		if shared.StringInSlice("ro", strings.Split(parts[3], ",")) {
+		if shared.ValueInSlice("ro", strings.Split(parts[3], ",")) {
 			device["readonly"] = "true"
 		}
 
 		// Deal with optional mounts
-		if shared.StringInSlice("optional", strings.Split(parts[3], ",")) {
+		if shared.ValueInSlice("optional", strings.Split(parts[3], ",")) {
 			device["optional"] = "true"
 		} else {
 			if strings.HasPrefix(parts[0], "/") {
