@@ -7,12 +7,11 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
+	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
 )
 
 // Start.
@@ -21,6 +20,8 @@ type cmdStart struct {
 	action *cmdAction
 }
 
+// The function Command() returns a cobra.Command object representing the "start" command.
+// It is used to start one or more instances specified by the user.
 func (c *cmdStart) Command() *cobra.Command {
 	cmdAction := cmdAction{global: c.global}
 	c.action = &cmdAction
@@ -40,6 +41,8 @@ type cmdPause struct {
 	action *cmdAction
 }
 
+// The function Command() returns a cobra.Command object representing the "pause" command.
+// It is used to pause (or freeze) one or more instances specified by the user. This command is hidden and has an alias "freeze".
 func (c *cmdPause) Command() *cobra.Command {
 	cmdAction := cmdAction{global: c.global}
 	c.action = &cmdAction
@@ -49,7 +52,7 @@ func (c *cmdPause) Command() *cobra.Command {
 	cmd.Short = i18n.G("Pause instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Pause instances`))
-	cmd.Hidden = true
+	cmd.Aliases = []string{"freeze"}
 
 	return cmd
 }
@@ -60,6 +63,8 @@ type cmdRestart struct {
 	action *cmdAction
 }
 
+// The function Command() returns a cobra.Command object representing the "restart" command.
+// It is used to restart one or more instances specified by the user. This command restarts the instances, which is the opposite of the "pause" command.
 func (c *cmdRestart) Command() *cobra.Command {
 	cmdAction := cmdAction{global: c.global}
 	c.action = &cmdAction
@@ -81,6 +86,8 @@ type cmdStop struct {
 	action *cmdAction
 }
 
+// The function Command() returns a cobra.Command object representing the "stop" command.
+// It is used to stop one or more instances specified by the user. This command stops the instances, effectively shutting them down.
 func (c *cmdStop) Command() *cobra.Command {
 	cmdAction := cmdAction{global: c.global}
 	c.action = &cmdAction
@@ -105,6 +112,8 @@ type cmdAction struct {
 	flagTimeout   int
 }
 
+// Command is a method of the cmdAction structure which constructs and configures a cobra Command object.
+// It creates a command with a specific action, defines flags based on that action, and assigns appropriate help text.
 func (c *cmdAction) Command(action string) *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.RunE = c.Run
@@ -117,19 +126,21 @@ func (c *cmdAction) Command(action string) *cobra.Command {
 		cmd.Flags().BoolVar(&c.flagStateless, "stateless", false, i18n.G("Ignore the instance state"))
 	}
 
-	if shared.StringInSlice(action, []string{"start", "restart", "stop"}) {
+	if shared.ValueInSlice(action, []string{"start", "restart", "stop"}) {
 		cmd.Flags().StringVar(&c.flagConsole, "console", "", i18n.G("Immediately attach to the console")+"``")
 		cmd.Flags().Lookup("console").NoOptDefVal = "console"
 	}
 
-	if shared.StringInSlice(action, []string{"restart", "stop"}) {
-		cmd.Flags().BoolVarP(&c.flagForce, "force", "f", false, i18n.G("Force the instance to shutdown"))
-		cmd.Flags().IntVar(&c.flagTimeout, "timeout", -1, i18n.G("Time to wait for the instance before killing it")+"``")
+	if shared.ValueInSlice(action, []string{"restart", "stop"}) {
+		cmd.Flags().BoolVarP(&c.flagForce, "force", "f", false, i18n.G("Force the instance to stop"))
+		cmd.Flags().IntVar(&c.flagTimeout, "timeout", -1, i18n.G("Time to wait for the instance to shutdown cleanly")+"``")
 	}
 
 	return cmd
 }
 
+// doActionAll is a method of the cmdAction structure. It performs a specified action on all instances of a remote resource.
+// It ensures that flags and parameters are appropriately set, and handles any errors that may occur during the process.
 func (c *cmdAction) doActionAll(action string, resource remoteResource) error {
 	if resource.name != "" {
 		// both --all and instance name given.
@@ -168,7 +179,7 @@ func (c *cmdAction) doActionAll(action string, resource remoteResource) error {
 		return err
 	}
 
-	progress := utils.ProgressRenderer{
+	progress := cli.ProgressRenderer{
 		Quiet: c.global.flagQuiet,
 	}
 
@@ -178,7 +189,7 @@ func (c *cmdAction) doActionAll(action string, resource remoteResource) error {
 		return err
 	}
 
-	err = utils.CancelableWait(op, &progress)
+	err = cli.CancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return err
@@ -189,6 +200,8 @@ func (c *cmdAction) doActionAll(action string, resource remoteResource) error {
 	return nil
 }
 
+// doAction is a method of the cmdAction structure. It carries out a specified action on an instance,
+// using a given config and instance name. It manages state changes, flag checks, error handling and console attachment.
 func (c *cmdAction) doAction(action string, conf *config.Config, nameArg string) error {
 	state := false
 
@@ -257,7 +270,7 @@ func (c *cmdAction) doAction(action string, conf *config.Config, nameArg string)
 		return console.Console(d, name)
 	}
 
-	progress := utils.ProgressRenderer{
+	progress := cli.ProgressRenderer{
 		Quiet: c.global.flagQuiet,
 	}
 
@@ -268,7 +281,7 @@ func (c *cmdAction) doAction(action string, conf *config.Config, nameArg string)
 	}
 
 	// Wait for operation to finish
-	err = utils.CancelableWait(op, &progress)
+	err = cli.CancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return fmt.Errorf("%s\n"+i18n.G("Try `lxc info --show-log %s` for more info"), err, nameArg)
@@ -287,6 +300,8 @@ func (c *cmdAction) doAction(action string, conf *config.Config, nameArg string)
 	return nil
 }
 
+// Run is a method of the cmdAction structure that implements the execution logic for the given Cobra command.
+// It handles actions on instances (single or all) and manages error handling, console flag restrictions, and batch operations.
 func (c *cmdAction) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
