@@ -11,12 +11,11 @@ import (
 	"github.com/spf13/cobra"
 	yaml "gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
 )
 
 type cmdCluster struct {
@@ -46,6 +45,10 @@ func (c *cmdCluster) Command() *cobra.Command {
 	clusterShowCmd := cmdClusterShow{global: c.global, cluster: c}
 	cmd.AddCommand(clusterShowCmd.Command())
 
+	// Info
+	clusterInfoCmd := cmdClusterInfo{global: c.global, cluster: c}
+	cmd.AddCommand(clusterInfoCmd.Command())
+
 	// Get
 	clusterGetCmd := cmdClusterGet{global: c.global, cluster: c}
 	cmd.AddCommand(clusterGetCmd.Command())
@@ -164,7 +167,7 @@ func (c *cmdClusterList) Run(cmd *cobra.Command, args []string) error {
 	// Render the table
 	data := [][]string{}
 	for _, member := range members {
-		roles := member.ClusterMemberPut.Roles
+		roles := member.Roles
 		rolesDelimiter := "\n"
 		if c.flagFormat == "csv" {
 			rolesDelimiter = ","
@@ -174,7 +177,7 @@ func (c *cmdClusterList) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, line)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
@@ -187,7 +190,7 @@ func (c *cmdClusterList) Run(cmd *cobra.Command, args []string) error {
 		i18n.G("MESSAGE"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, members)
+	return cli.RenderTable(c.flagFormat, header, data, members)
 }
 
 // Show.
@@ -239,10 +242,61 @@ func (c *cmdClusterShow) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
+// Info.
+type cmdClusterInfo struct {
+	global  *cmdGlobal
+	cluster *cmdCluster
+}
+
+func (c *cmdClusterInfo) Command() *cobra.Command {
+	cmd := &cobra.Command{}
+	cmd.Use = usage("info", i18n.G("[<remote>:]<member>"))
+	cmd.Short = i18n.G("Show useful information about a cluster member")
+	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
+		`Show useful information about a cluster member`))
+
+	cmd.RunE = c.Run
+
+	return cmd
+}
+
+func (c *cmdClusterInfo) Run(cmd *cobra.Command, args []string) error {
+	// Quick checks.
+	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
+	if exit {
+		return err
+	}
+
+	// Parse remote.
+	resources, err := c.global.ParseServers(args[0])
+	if err != nil {
+		return err
+	}
+
+	resource := resources[0]
+
+	// Get the member state information.
+	member, _, err := resource.server.GetClusterMemberState(resource.name)
+	if err != nil {
+		return err
+	}
+
+	// Render as YAML.
+	data, err := yaml.Marshal(&member)
+	if err != nil {
+		return err
+	}
+
+	fmt.Printf("%s", data)
+	return nil
+}
+
 // Get.
 type cmdClusterGet struct {
 	global  *cmdGlobal
 	cluster *cmdCluster
+
+	flagIsProperty bool
 }
 
 func (c *cmdClusterGet) Command() *cobra.Command {
@@ -251,6 +305,7 @@ func (c *cmdClusterGet) Command() *cobra.Command {
 	cmd.Short = i18n.G("Get values for cluster member configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), cmd.Short)
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a cluster property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -277,6 +332,17 @@ func (c *cmdClusterGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	if c.flagIsProperty {
+		w := member.Writable()
+		res, err := getFieldByJsonTag(&w, args[1])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the cluster member %q: %v"), args[1], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+		return nil
+	}
+
 	value, ok := member.Config[args[1]]
 	if !ok {
 		return fmt.Errorf(i18n.G("The key %q does not exist on cluster member %q"), args[1], resource.name)
@@ -290,6 +356,8 @@ func (c *cmdClusterGet) Run(cmd *cobra.Command, args []string) error {
 type cmdClusterSet struct {
 	global  *cmdGlobal
 	cluster *cmdCluster
+
+	flagIsProperty bool
 }
 
 func (c *cmdClusterSet) Command() *cobra.Command {
@@ -298,6 +366,7 @@ func (c *cmdClusterSet) Command() *cobra.Command {
 	cmd.Short = i18n.G("Set a cluster member's configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), cmd.Short)
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a cluster property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -324,17 +393,34 @@ func (c *cmdClusterSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	// Get the new config entries
-	entries, err := getConfig(args[1:]...)
+	// Get the new config keys
+	keys, err := getConfig(args[1:]...)
 	if err != nil {
 		return err
 	}
 
-	for k, v := range entries {
-		member.Config[k] = v
+	writable := member.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	return resource.server.UpdateClusterMember(resource.name, member.Writable(), "")
+	return resource.server.UpdateClusterMember(resource.name, writable, "")
 }
 
 // Unset.
@@ -342,6 +428,8 @@ type cmdClusterUnset struct {
 	global     *cmdGlobal
 	cluster    *cmdCluster
 	clusterSet *cmdClusterSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdClusterUnset) Command() *cobra.Command {
@@ -350,6 +438,7 @@ func (c *cmdClusterUnset) Command() *cobra.Command {
 	cmd.Short = i18n.G("Unset a cluster member's configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), cmd.Short)
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a cluster property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -362,6 +451,8 @@ func (c *cmdClusterUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.clusterSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.clusterSet.Run(cmd, args)
 }
@@ -457,7 +548,7 @@ Are you really sure you want to force removing %s? (yes/no): `), name)
 	input, _ := reader.ReadString('\n')
 	input = strings.TrimSuffix(input, "\n")
 
-	if !shared.StringInSlice(strings.ToLower(input), []string{i18n.G("yes")}) {
+	if !shared.ValueInSlice(strings.ToLower(input), []string{i18n.G("yes")}) {
 		return fmt.Errorf(i18n.G("User aborted delete operation"))
 	}
 
@@ -708,10 +799,10 @@ type cmdClusterAdd struct {
 
 func (c *cmdClusterAdd) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("add", i18n.G("[[<remote>:]<member>]"))
+	cmd.Use = usage("add", i18n.G("[[<remote>:]<name>]"))
 	cmd.Short = i18n.G("Request a join token for adding a cluster member")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Request a join token for adding a cluster member`))
-	cmd.Flags().StringVar(&c.flagName, "name", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().StringVar(&c.flagName, "name", "", i18n.G("Cluster member name (alternative to passing it as an argument)")+"``")
 
 	cmd.RunE = c.Run
 
@@ -733,9 +824,14 @@ func (c *cmdClusterAdd) Run(cmd *cobra.Command, args []string) error {
 
 	resource := resources[0]
 
+	// Determine the machine name.
+	if resource.name != "" && c.flagName != "" && resource.name != c.flagName {
+		return fmt.Errorf(i18n.G("Cluster member name was provided as both a flag and as an argument"))
+	}
+
 	if resource.name == "" {
 		if c.flagName == "" {
-			resource.name, err = cli.AskString(i18n.G("Please provide cluster member name: "), "", nil)
+			resource.name, err = c.global.asker.AskString(i18n.G("Please provide cluster member name: "), "", nil)
 			if err != nil {
 				return err
 			}
@@ -862,7 +958,7 @@ func (c *cmdClusterListTokens) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, line)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
@@ -870,7 +966,7 @@ func (c *cmdClusterListTokens) Run(cmd *cobra.Command, args []string) error {
 		i18n.G("EXPIRES AT"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, displayTokens)
+	return cli.RenderTable(c.flagFormat, header, data, displayTokens)
 }
 
 // Revoke Tokens.
@@ -1050,7 +1146,8 @@ func (c *cmdClusterUpdateCertificate) Run(cmd *cobra.Command, args []string) err
 type cmdClusterEvacuateAction struct {
 	global *cmdGlobal
 
-	flagForce bool
+	flagAction string
+	flagForce  bool
 }
 
 // Cluster member evacuation.
@@ -1070,6 +1167,9 @@ func (c *cmdClusterEvacuate) Command() *cobra.Command {
 	cmd.Short = i18n.G("Evacuate cluster member")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Evacuate cluster member`))
 
+	cmd.Flags().BoolVar(&c.action.flagForce, "force", false, i18n.G(`Force evacuation without user confirmation`)+"``")
+	cmd.Flags().StringVar(&c.action.flagAction, "action", "", i18n.G(`Force a particular evacuation action`)+"``")
+
 	return cmd
 }
 
@@ -1089,13 +1189,14 @@ func (c *cmdClusterRestore) Command() *cobra.Command {
 	cmd.Short = i18n.G("Restore cluster member")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Restore cluster member`))
 
+	cmd.Flags().BoolVar(&c.action.flagForce, "force", false, i18n.G(`Force restoration without user confirmation`)+"``")
+
 	return cmd
 }
 
 func (c *cmdClusterEvacuateAction) Command(action string) *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.RunE = c.Run
-	cmd.Flags().BoolVar(&c.flagForce, "force", false, i18n.G(`Force evacuation without user confirmation`)+"``")
 
 	return cmd
 }
@@ -1120,7 +1221,7 @@ func (c *cmdClusterEvacuateAction) Run(cmd *cobra.Command, args []string) error
 	}
 
 	if !c.flagForce {
-		evacuate, err := cli.AskBool(fmt.Sprintf(i18n.G("Are you sure you want to %s cluster member %q? (yes/no) [default=no]: "), cmd.Name(), resource.name), "no")
+		evacuate, err := c.global.asker.AskBool(fmt.Sprintf(i18n.G("Are you sure you want to %s cluster member %q? (yes/no) [default=no]: "), cmd.Name(), resource.name), "no")
 		if err != nil {
 			return err
 		}
@@ -1132,6 +1233,7 @@ func (c *cmdClusterEvacuateAction) Run(cmd *cobra.Command, args []string) error
 
 	state := api.ClusterMemberStatePost{
 		Action: cmd.Name(),
+		Mode:   c.flagAction,
 	}
 
 	op, err := resource.server.UpdateClusterMemberState(resource.name, state)
@@ -1147,7 +1249,7 @@ func (c *cmdClusterEvacuateAction) Run(cmd *cobra.Command, args []string) error
 		format = i18n.G("Evacuating cluster member: %s")
 	}
 
-	progress := utils.ProgressRenderer{
+	progress := cli.ProgressRenderer{
 		Format: format,
 		Quiet:  c.global.flagQuiet,
 	}
