@@ -6,13 +6,13 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
 )
 
 type cmdCopy struct {
@@ -85,19 +85,21 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		return fmt.Errorf(i18n.G("You must specify a source instance name"))
 	}
 
-	// Check that a destination instance was specified, if --target is passed.
-	if destName == "" && c.flagTarget != "" {
-		return fmt.Errorf(i18n.G("You must specify a destination instance name when using --target"))
-	}
-
 	// Don't allow refreshing without profiles.
 	if c.flagRefresh && c.flagNoProfiles {
 		return fmt.Errorf(i18n.G("--no-profiles cannot be used with --refresh"))
 	}
 
-	// If no destination name was provided, use the same as the source
-	if destName == "" && destResource != "" {
-		destName = sourceName
+	// If the instance is being copied to a different remote and no destination name is
+	// specified, use the source name with snapshot suffix trimmed (in case a new instance
+	// is being created from a snapshot).
+	if destName == "" && destResource != "" && c.flagTarget == "" {
+		destName = strings.SplitN(sourceName, shared.SnapshotDelimiter, 2)[0]
+	}
+
+	// Ensure that a destination name is provided.
+	if destName == "" {
+		return fmt.Errorf(i18n.G("You must specify a destination instance name"))
 	}
 
 	// Connect to the source host
@@ -132,15 +134,15 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 	// Parse the config overrides
 	configMap := map[string]string{}
 	for _, entry := range c.flagConfig {
-		if !strings.Contains(entry, "=") {
-			return fmt.Errorf(i18n.G("Bad key=value pair: %s"), entry)
+		key, value, found := strings.Cut(entry, "=")
+		if !found {
+			return fmt.Errorf(i18n.G("Bad key=value pair: %q"), entry)
 		}
 
-		fields := strings.SplitN(entry, "=", 2)
-		configMap[fields[0]] = fields[1]
+		configMap[key] = value
 	}
 
-	deviceMap, err := parseDeviceOverrides(c.flagDevice)
+	deviceOverrides, err := parseDeviceOverrides(c.flagDevice)
 	if err != nil {
 		return err
 	}
@@ -179,23 +181,40 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 			entry.Profiles = []string{}
 		}
 
-		// Allow setting additional config keys
-		for key, value := range configMap {
-			entry.Config[key] = value
+		// Check to see if any of the overridden devices are for devices that are not yet defined in the
+		// local devices (and thus maybe expected to be coming from profiles).
+		needProfileExpansion := false
+		for deviceName := range deviceOverrides {
+			_, isLocalDevice := entry.Devices[deviceName]
+			if !isLocalDevice {
+				needProfileExpansion = true
+				break
+			}
 		}
 
-		// Allow setting device overrides
-		for k, m := range deviceMap {
-			if entry.Devices[k] == nil {
-				entry.Devices[k] = m
-				continue
-			}
+		profileDevices := make(map[string]map[string]string)
 
-			for key, value := range m {
-				entry.Devices[k][key] = value
+		// If there are device overrides that are expected to be applied to profile devices then perform
+		// profile expansion.
+		if needProfileExpansion {
+			// If the list of profiles is empty then LXD would apply the default profile on the server side.
+			profileDevices, err = getProfileDevices(dest, entry.Profiles)
+			if err != nil {
+				return err
 			}
 		}
 
+		// Apply device overrides.
+		entry.Devices, err = shared.ApplyDeviceOverrides(profileDevices, entry.Devices, deviceOverrides)
+		if err != nil {
+			return err
+		}
+
+		// Allow setting additional config keys.
+		for key, value := range configMap {
+			entry.Config[key] = value
+		}
+
 		// Allow overriding the ephemeral status
 		if ephemeral == 1 {
 			entry.Ephemeral = true
@@ -203,10 +222,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 			entry.Ephemeral = false
 		}
 
-		rootDiskDeviceKey, _, _ := shared.GetRootDiskDevice(entry.Devices)
-		if err != nil {
-			return err
-		}
+		rootDiskDeviceKey, _, _ := instancetype.GetRootDiskDevice(entry.Devices)
 
 		if rootDiskDeviceKey != "" && pool != "" {
 			entry.Devices[rootDiskDeviceKey]["pool"] = pool
@@ -224,7 +240,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 
 			if !keepVolatile {
 				for k := range entry.Config {
-					if !shared.InstanceIncludeWhenCopying(k, true) {
+					if !instancetype.InstanceIncludeWhenCopying(k, true) {
 						delete(entry.Config, k)
 					}
 				}
@@ -270,23 +286,39 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 			entry.Profiles = []string{}
 		}
 
-		// Allow setting additional config keys
-		for key, value := range configMap {
-			entry.Config[key] = value
+		// Check to see if any of the devices overrides are for devices that are not yet defined in the
+		// local devices and thus are expected to be coming from profiles.
+		needProfileExpansion := false
+		for deviceName := range deviceOverrides {
+			_, isLocalDevice := entry.Devices[deviceName]
+			if !isLocalDevice {
+				needProfileExpansion = true
+				break
+			}
 		}
 
-		// Allow setting device overrides
-		for k, m := range deviceMap {
-			if entry.Devices[k] == nil {
-				entry.Devices[k] = m
-				continue
-			}
+		profileDevices := make(map[string]map[string]string)
 
-			for key, value := range m {
-				entry.Devices[k][key] = value
+		// If there are device overrides that are expected to be applied to profile devices then perform
+		// profile expansion.
+		if needProfileExpansion {
+			profileDevices, err = getProfileDevices(dest, entry.Profiles)
+			if err != nil {
+				return err
 			}
 		}
 
+		// Apply device overrides.
+		entry.Devices, err = shared.ApplyDeviceOverrides(entry.Devices, profileDevices, deviceOverrides)
+		if err != nil {
+			return err
+		}
+
+		// Allow setting additional config keys.
+		for key, value := range configMap {
+			entry.Config[key] = value
+		}
+
 		// Allow overriding the ephemeral status
 		if ephemeral == 1 {
 			entry.Ephemeral = true
@@ -294,11 +326,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 			entry.Ephemeral = false
 		}
 
-		rootDiskDeviceKey, _, _ := shared.GetRootDiskDevice(entry.Devices)
-		if err != nil {
-			return err
-		}
-
+		rootDiskDeviceKey, _, _ := instancetype.GetRootDiskDevice(entry.Devices)
 		if rootDiskDeviceKey != "" && pool != "" {
 			entry.Devices[rootDiskDeviceKey]["pool"] = pool
 		} else if pool != "" {
@@ -312,7 +340,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		// Strip the volatile keys if requested
 		if !keepVolatile {
 			for k := range entry.Config {
-				if !shared.InstanceIncludeWhenCopying(k, true) {
+				if !instancetype.InstanceIncludeWhenCopying(k, true) {
 					delete(entry.Config, k)
 				}
 			}
@@ -337,7 +365,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 	}
 
 	// Watch the background operation
-	progress := utils.ProgressRenderer{
+	progress := cli.ProgressRenderer{
 		Format: i18n.G("Transferring instance: %s"),
 		Quiet:  c.global.flagQuiet,
 	}
@@ -349,7 +377,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 	}
 
 	// Wait for the copy to complete
-	err = utils.CancelableWait(op, &progress)
+	err = cli.CancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return err
@@ -364,7 +392,16 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		}
 
 		// Ensure we don't change the target's volatile.idmap.next value.
-		writable.Config["volatile.idmap.next"] = inst.Config["volatile.idmap.next"]
+		if inst.Config["volatile.idmap.next"] != writable.Config["volatile.idmap.next"] {
+			writable.Config["volatile.idmap.next"] = inst.Config["volatile.idmap.next"]
+		}
+
+		// Ensure we don't change the target's root disk pool.
+		srcRootDiskDeviceKey, _, _ := instancetype.GetRootDiskDevice(writable.Devices)
+		destRootDiskDeviceKey, destRootDiskDevice, _ := instancetype.GetRootDiskDevice(inst.Devices)
+		if srcRootDiskDeviceKey != "" && srcRootDiskDeviceKey == destRootDiskDeviceKey {
+			writable.Devices[destRootDiskDeviceKey]["pool"] = destRootDiskDevice["pool"]
+		}
 
 		op, err := dest.UpdateInstance(destName, writable, etag)
 		if err != nil {
@@ -372,7 +409,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		}
 
 		// Watch the background operation
-		progress := utils.ProgressRenderer{
+		progress := cli.ProgressRenderer{
 			Format: i18n.G("Refreshing instance: %s"),
 			Quiet:  c.global.flagQuiet,
 		}
@@ -384,7 +421,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		}
 
 		// Wait for the copy to complete
-		err = utils.CancelableWait(op, &progress)
+		err = cli.CancelableWait(op, &progress)
 		if err != nil {
 			progress.Done("")
 			return err
@@ -393,33 +430,10 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		progress.Done("")
 	}
 
-	// If choosing a random name, show it to the user
-	if destResource == "" && c.flagTargetProject == "" {
-		// Get the successful operation data
-		opInfo, err := op.GetTarget()
-		if err != nil {
-			return err
-		}
-
-		// Extract the list of affected instances
-		instances, ok := opInfo.Resources["instances"]
-		if !ok || len(instances) != 1 {
-			// Extract the list of affected instances using old "containers" field
-			instances, ok = opInfo.Resources["containers"]
-			if !ok || len(instances) != 1 {
-				return fmt.Errorf(i18n.G("Failed to get the new instance name"))
-			}
-		}
-
-		// Extract the name of the instance
-		fields := strings.Split(instances[0], "/")
-		fmt.Printf(i18n.G("Instance name is: %s")+"\n", fields[len(fields)-1])
-	}
-
 	// Start the instance if needed
 	if start {
 		req := api.InstanceStatePut{
-			Action: string(shared.Start),
+			Action: string(instancetype.Start),
 		}
 
 		op, err := dest.UpdateInstanceState(destName, req, "")
