@@ -18,15 +18,14 @@ import (
 	"github.com/spf13/cobra"
 	"golang.org/x/crypto/ssh"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/termios"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/shared"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/termios"
+	"github.com/canonical/lxd/shared/units"
 )
 
 // DirMode represents the file mode for creating dirs on `lxc file pull/push`.
@@ -405,7 +404,7 @@ func (c *cmdFilePull) Run(cmd *cobra.Command, args []string) error {
 			}
 		}
 
-		progress := utils.ProgressRenderer{
+		progress := cli.ProgressRenderer{
 			Format: fmt.Sprintf(i18n.G("Pulling %s from %s: %%s"), targetPath, pathSpec[1]),
 			Quiet:  c.global.flagQuiet,
 		}
@@ -571,12 +570,16 @@ func (c *cmdFilePush) Run(cmd *cobra.Command, args []string) error {
 		return nil
 	}
 
+	modifyExistingUID := c.file.flagUID != -1
+
 	// Determine the target uid
 	uid := 0
 	if c.file.flagUID >= 0 {
 		uid = c.file.flagUID
 	}
 
+	modifyExistingGID := c.file.flagGID != -1
+
 	// Determine the target gid
 	gid := 0
 	if c.file.flagGID >= 0 {
@@ -618,8 +621,9 @@ func (c *cmdFilePush) Run(cmd *cobra.Command, args []string) error {
 				return err
 			}
 
-			_, dUID, dGID := shared.GetOwnerMode(finfo)
 			if c.file.flagUID == -1 || c.file.flagGID == -1 {
+				_, dUID, dGID := shared.GetOwnerMode(finfo)
+
 				if c.file.flagUID == -1 {
 					uid = dUID
 				}
@@ -637,9 +641,12 @@ func (c *cmdFilePush) Run(cmd *cobra.Command, args []string) error {
 
 		// Transfer the files
 		args := lxd.InstanceFileArgs{
-			UID:  -1,
-			GID:  -1,
-			Mode: -1,
+			UID:                -1,
+			UIDModifyExisting:  modifyExistingUID,
+			GID:                -1,
+			GIDModifyExisting:  modifyExistingGID,
+			Mode:               -1,
+			ModeModifyExisting: c.file.flagMode != "",
 		}
 
 		if !c.noModeChange {
@@ -679,7 +686,7 @@ func (c *cmdFilePush) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
-		progress := utils.ProgressRenderer{
+		progress := cli.ProgressRenderer{
 			Format: fmt.Sprintf(i18n.G("Pushing %s to %s: %%s"), f.Name(), fpath),
 			Quiet:  c.global.flagQuiet,
 		}
@@ -745,7 +752,7 @@ func (c *cmdFile) recursivePullFile(d lxd.InstanceServer, inst string, p string,
 			return err
 		}
 
-		progress := utils.ProgressRenderer{
+		progress := cli.ProgressRenderer{
 			Format: fmt.Sprintf(i18n.G("Pulling %s from %s: %%s"), p, target),
 			Quiet:  c.global.flagQuiet,
 		}
@@ -845,7 +852,7 @@ func (c *cmdFile) recursivePushFile(d lxd.InstanceServer, inst string, source st
 			readCloser = f
 		}
 
-		progress := utils.ProgressRenderer{
+		progress := cli.ProgressRenderer{
 			Format: fmt.Sprintf(i18n.G("Pushing %s to %s: %%s"), p, targetPath),
 			Quiet:  c.global.flagQuiet,
 		}
@@ -1197,7 +1204,7 @@ func (c *cmdFileMount) sshSFTPServer(ctx context.Context, instName string, resou
 	if config.PasswordCallback != nil {
 		fmt.Printf("Login with username %q and password %q\n", authUser, authPass)
 	} else {
-		fmt.Printf("Login without username and password\n")
+		fmt.Println("Login without username and password")
 	}
 
 	for {
