@@ -9,14 +9,13 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/units"
 )
 
 type cmdInfo struct {
@@ -140,7 +139,7 @@ func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial boo
 		if len(gpu.SRIOV.VFs) > 0 {
 			fmt.Printf(prefix+"  "+i18n.G("VFs: %d")+"\n", gpu.SRIOV.MaximumVFs)
 			for _, vf := range gpu.SRIOV.VFs {
-				fmt.Printf(prefix + "  - ")
+				fmt.Print(prefix + "  - ")
 				c.renderGPU(vf, prefix+"    ", false)
 			}
 		}
@@ -159,7 +158,7 @@ func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial boo
 		for _, k := range keys {
 			v := gpu.Mdev[k]
 
-			fmt.Println(prefix + "  - " + fmt.Sprintf(i18n.G("%s (%d available)"), k, v.Available))
+			fmt.Println(prefix + "  - " + fmt.Sprintf(i18n.G("%s (%s) (%d available)"), k, v.Name, v.Available))
 			if v.Description != "" {
 				for _, line := range strings.Split(v.Description, "\n") {
 					fmt.Printf(prefix+"      %s\n", line)
@@ -249,7 +248,7 @@ func (c *cmdInfo) renderNIC(nic api.ResourcesNetworkCard, prefix string, initial
 		if len(nic.SRIOV.VFs) > 0 {
 			fmt.Printf(prefix+"  "+i18n.G("VFs: %d")+"\n", nic.SRIOV.MaximumVFs)
 			for _, vf := range nic.SRIOV.VFs {
-				fmt.Printf(prefix + "  - ")
+				fmt.Print(prefix + "  - ")
 				c.renderNIC(vf, prefix+"    ", false)
 			}
 		}
@@ -281,6 +280,7 @@ func (c *cmdInfo) renderDisk(disk api.ResourcesStorageDisk, prefix string, initi
 	}
 
 	fmt.Printf(prefix+i18n.G("Read-Only: %v")+"\n", disk.ReadOnly)
+	fmt.Printf(prefix+i18n.G("Mounted: %v")+"\n", disk.Mounted)
 	fmt.Printf(prefix+i18n.G("Removable: %v")+"\n", disk.Removable)
 
 	if len(disk.Partitions) != 0 {
@@ -290,6 +290,7 @@ func (c *cmdInfo) renderDisk(disk api.ResourcesStorageDisk, prefix string, initi
 			fmt.Printf(prefix+"    "+i18n.G("ID: %s")+"\n", partition.ID)
 			fmt.Printf(prefix+"    "+i18n.G("Device: %s")+"\n", partition.Device)
 			fmt.Printf(prefix+"    "+i18n.G("Read-Only: %v")+"\n", partition.ReadOnly)
+			fmt.Printf(prefix+"    "+i18n.G("Mounted: %v")+"\n", partition.Mounted)
 			fmt.Printf(prefix+"    "+i18n.G("Size: %s")+"\n", units.GetByteSizeStringIEC(int64(partition.Size), 2))
 		}
 	}
@@ -632,7 +633,7 @@ func (c *cmdInfo) instanceInfo(d lxd.InstanceServer, remote config.Remote, name
 			i18n.G("Stateful"),
 		}
 
-		_ = utils.RenderTable(utils.TableFormatTable, snapHeader, snapData, inst.Snapshots)
+		_ = cli.RenderTable(cli.TableFormatTable, snapHeader, snapData, inst.Snapshots)
 	}
 
 	// List backups
@@ -684,7 +685,7 @@ func (c *cmdInfo) instanceInfo(d lxd.InstanceServer, remote config.Remote, name
 			i18n.G("Optimized Storage"),
 		}
 
-		_ = utils.RenderTable(utils.TableFormatTable, backupHeader, backupData, inst.Backups)
+		_ = cli.RenderTable(cli.TableFormatTable, backupHeader, backupData, inst.Backups)
 	}
 
 	if showLog {
