@@ -3,19 +3,21 @@ package main
 import (
 	"fmt"
 	"io"
+	"net/url"
 	"os"
+	"path"
 	"strings"
 
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
 )
 
 type cmdInit struct {
@@ -43,7 +45,6 @@ func (c *cmdInit) Command() *cobra.Command {
 
 lxc init ubuntu:22.04 u1 < config.yaml
     Create the instance with configuration from config.yaml`))
-	cmd.Hidden = true
 
 	cmd.RunE = c.Run
 	cmd.Flags().StringArrayVarP(&c.flagConfig, "config", "c", nil, i18n.G("Config key/value to apply to the new instance")+"``")
@@ -211,12 +212,12 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	}
 
 	for _, entry := range c.flagConfig {
-		if !strings.Contains(entry, "=") {
-			return nil, "", fmt.Errorf(i18n.G("Bad key=value pair: %s"), entry)
+		key, value, found := strings.Cut(entry, "=")
+		if !found {
+			return nil, "", fmt.Errorf(i18n.G("Bad key=value pair: %q"), entry)
 		}
 
-		fields := strings.SplitN(entry, "=", 2)
-		configMap[fields[0]] = fields[1]
+		configMap[key] = value
 	}
 
 	// Check if the specified storage pool exists.
@@ -248,6 +249,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 	req.Config = configMap
 	req.Ephemeral = c.flagEphemeral
+	req.Description = stdinData.Description
 
 	if !c.flagNoProfiles && len(profiles) == 0 {
 		if len(stdinData.Profiles) > 0 {
@@ -281,46 +283,16 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	// that would be applied server-side.
 	if needProfileExpansion {
 		// If the list of profiles is empty then LXD would apply the default profile on the server side.
-		serverSideProfiles := req.Profiles
-		if len(serverSideProfiles) == 0 {
-			serverSideProfiles = []string{"default"}
-		}
-
-		// Get the effective expanded devices by overlaying each profile's devices in order.
-		for _, profileName := range serverSideProfiles {
-			profile, _, err := d.GetProfile(profileName)
-			if err != nil {
-				return nil, "", fmt.Errorf(i18n.G("Failed loading profile %q for device override: %w"), profileName, err)
-			}
-
-			for k, v := range profile.Devices {
-				profileDevices[k] = v
-			}
+		profileDevices, err = getProfileDevices(d, req.Profiles)
+		if err != nil {
+			return nil, "", err
 		}
 	}
 
 	// Apply device overrides.
-	for deviceName := range deviceOverrides {
-		_, isLocalDevice := devicesMap[deviceName]
-		if isLocalDevice {
-			// Apply overrides to local device.
-			for k, v := range deviceOverrides[deviceName] {
-				devicesMap[deviceName][k] = v
-			}
-		} else {
-			// Check device exists in expanded profile devices.
-			profileDeviceConfig, found := profileDevices[deviceName]
-			if !found {
-				return nil, "", fmt.Errorf(i18n.G("Cannot override config for device %q: Device not found in profile devices"), deviceName)
-			}
-
-			for k, v := range deviceOverrides[deviceName] {
-				profileDeviceConfig[k] = v
-			}
-
-			// Add device to local devices.
-			devicesMap[deviceName] = profileDeviceConfig
-		}
+	devicesMap, err = shared.ApplyDeviceOverrides(devicesMap, profileDevices, deviceOverrides)
+	if err != nil {
+		return nil, "", err
 	}
 
 	req.Devices = devicesMap
@@ -328,45 +300,19 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	var opInfo api.Operation
 	if !c.flagEmpty {
 		// Get the image server and image info
-		iremote, image = c.guessImage(conf, d, remote, iremote, image)
-		var imgRemote lxd.ImageServer
-		var imgInfo *api.Image
-
-		// Connect to the image server
-		if iremote == remote {
-			imgRemote = d
-		} else {
-			imgRemote, err = conf.GetImageServer(iremote)
-			if err != nil {
-				return nil, "", err
-			}
-		}
+		iremote, image = guessImage(conf, d, remote, iremote, image)
 
 		// Deal with the default image
 		if image == "" {
 			image = "default"
 		}
 
-		// Optimisation for simplestreams
-		if conf.Remotes[iremote].Protocol == "simplestreams" {
-			imgInfo = &api.Image{}
-			imgInfo.Fingerprint = image
-			imgInfo.Public = true
-			req.Source.Alias = image
-		} else {
-			// Attempt to resolve an image alias
-			alias, _, err := imgRemote.GetImageAlias(image)
-			if err == nil {
-				req.Source.Alias = image
-				image = alias.Target
-			}
-
-			// Get the image info
-			imgInfo, _, err = imgRemote.GetImage(image)
-			if err != nil {
-				return nil, "", err
-			}
+		imgRemote, imgInfo, err := getImgInfo(d, conf, iremote, remote, image, &req.Source)
+		if err != nil {
+			return nil, "", err
+		}
 
+		if conf.Remotes[iremote].Protocol != "simplestreams" {
 			if imgInfo.Type != "virtual-machine" && c.flagVM {
 				return nil, "", fmt.Errorf(i18n.G("Asked for a VM but image is of type container"))
 			}
@@ -381,7 +327,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 		}
 
 		// Watch the background operation
-		progress := utils.ProgressRenderer{
+		progress := cli.ProgressRenderer{
 			Format: i18n.G("Retrieving image: %s"),
 			Quiet:  c.global.flagQuiet,
 		}
@@ -392,7 +338,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 			return nil, "", err
 		}
 
-		err = utils.CancelableWait(op, &progress)
+		err = cli.CancelableWait(op, &progress)
 		if err != nil {
 			progress.Done("")
 			return nil, "", err
@@ -433,8 +379,12 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	}
 
 	if len(instances) == 1 && name == "" {
-		fields := strings.Split(instances[0], "/")
-		name = fields[len(fields)-1]
+		url, err := url.Parse(instances[0])
+		if err != nil {
+			return nil, "", err
+		}
+
+		name = path.Base(url.Path)
 		fmt.Printf(i18n.G("Instance name is: %s")+"\n", name)
 	}
 
@@ -444,36 +394,6 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	return d, name, nil
 }
 
-func (c *cmdInit) guessImage(conf *config.Config, d lxd.InstanceServer, remote string, iremote string, image string) (string, string) {
-	if remote != iremote {
-		return iremote, image
-	}
-
-	fields := strings.SplitN(image, "/", 2)
-	_, ok := conf.Remotes[fields[0]]
-	if !ok {
-		return iremote, image
-	}
-
-	_, _, err := d.GetImageAlias(image)
-	if err == nil {
-		return iremote, image
-	}
-
-	_, _, err = d.GetImage(image)
-	if err == nil {
-		return iremote, image
-	}
-
-	if len(fields) == 1 {
-		fmt.Fprintf(os.Stderr, i18n.G("The local image '%s' couldn't be found, trying '%s:' instead.")+"\n", image, fields[0])
-		return fields[0], "default"
-	}
-
-	fmt.Fprintf(os.Stderr, i18n.G("The local image '%s' couldn't be found, trying '%s:%s' instead.")+"\n", image, fields[0], fields[1])
-	return fields[0], fields[1]
-}
-
 func (c *cmdInit) checkNetwork(d lxd.InstanceServer, name string) {
 	ct, _, err := d.GetInstance(name)
 	if err != nil {
