@@ -11,15 +11,14 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/units"
 )
 
 type column struct {
@@ -437,14 +436,14 @@ func (c *cmdList) showInstances(instances []api.InstanceFull, filters []string,
 		data = append(data, col)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	headers := []string{}
 	for _, column := range columns {
 		headers = append(headers, column.Name)
 	}
 
-	return utils.RenderTable(c.flagFormat, headers, data, instancesFiltered)
+	return cli.RenderTable(c.flagFormat, headers, data, instancesFiltered)
 }
 
 func (c *cmdList) Run(cmd *cobra.Command, args []string) error {
@@ -644,7 +643,7 @@ func (c *cmdList) parseColumns(clustered bool) ([]column, bool, error) {
 
 			k := cc[0]
 			if colType == configColumnType {
-				_, err := shared.ConfigKeyChecker(k, instancetype.Any)
+				_, err := instancetype.ConfigKeyChecker(k, instancetype.Any)
 				if err != nil {
 					return nil, false, fmt.Errorf(i18n.G("Invalid config key '%s' in '%s'"), k, columnEntry)
 				}
@@ -766,7 +765,7 @@ func (c *cmdList) IP4ColumnData(cInfo api.InstanceFull) string {
 			}
 
 			for _, addr := range net.Addresses {
-				if shared.StringInSlice(addr.Scope, []string{"link", "local"}) {
+				if shared.ValueInSlice(addr.Scope, []string{"link", "local"}) {
 					continue
 				}
 
@@ -792,7 +791,7 @@ func (c *cmdList) IP6ColumnData(cInfo api.InstanceFull) string {
 			}
 
 			for _, addr := range net.Addresses {
-				if shared.StringInSlice(addr.Scope, []string{"link", "local"}) {
+				if shared.ValueInSlice(addr.Scope, []string{"link", "local"}) {
 					continue
 				}
 
@@ -849,7 +848,7 @@ func (c *cmdList) cpuUsageSecondsColumnData(cInfo api.InstanceFull) string {
 }
 
 func (c *cmdList) diskUsageColumnData(cInfo api.InstanceFull) string {
-	rootDisk, _, _ := shared.GetRootDiskDevice(cInfo.ExpandedDevices)
+	rootDisk, _, _ := instancetype.GetRootDiskDevice(cInfo.ExpandedDevices)
 
 	if cInfo.State != nil && cInfo.State.Disk != nil && cInfo.State.Disk[rootDisk].Usage > 0 {
 		return units.GetByteSizeStringIEC(cInfo.State.Disk[rootDisk].Usage, 2)
@@ -945,7 +944,7 @@ func (c *cmdList) matchByStatus(cInfo *api.Instance, cState *api.InstanceState,
 }
 
 func (c *cmdList) matchByArchitecture(cInfo *api.Instance, cState *api.InstanceState, query string) bool {
-	return strings.EqualFold(cInfo.InstancePut.Architecture, query)
+	return strings.EqualFold(cInfo.Architecture, query)
 }
 
 func (c *cmdList) matchByLocation(cInfo *api.Instance, cState *api.InstanceState, query string) bool {
