@@ -1,6 +1,7 @@
 package main
 
 import (
+	"bufio"
 	"fmt"
 	"os"
 	"os/user"
@@ -9,16 +10,18 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/shared"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/shared"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/version"
 )
 
 type cmdGlobal struct {
+	asker cli.Asker
+
 	conf     *config.Config
 	confPath string
 	cmd      *cobra.Command
@@ -86,7 +89,7 @@ For help with any of those, simply call them with --help.`))
 	app.CompletionOptions = cobra.CompletionOptions{DisableDefaultCmd: true}
 
 	// Global flags
-	globalCmd := cmdGlobal{cmd: app}
+	globalCmd := cmdGlobal{cmd: app, asker: cli.NewAsker(bufio.NewReader(os.Stdin))}
 	app.PersistentFlags().BoolVar(&globalCmd.flagVersion, "version", false, i18n.G("Print version number"))
 	app.PersistentFlags().BoolVarP(&globalCmd.flagHelp, "help", "h", false, i18n.G("Print help"))
 	app.PersistentFlags().BoolVar(&globalCmd.flagForceLocal, "force-local", false, i18n.G("Force using the local unix socket"))
@@ -103,6 +106,9 @@ For help with any of those, simply call them with --help.`))
 	// Version handling
 	app.SetVersionTemplate("{{.Version}}\n")
 	app.Version = version.Version
+	if version.IsLTSVersion {
+		app.Version = fmt.Sprintf("%s LTS", version.Version)
+	}
 
 	// alias sub-command
 	aliasCmd := cmdAlias{global: &globalCmd}
@@ -196,7 +202,7 @@ For help with any of those, simply call them with --help.`))
 	profileCmd := cmdProfile{global: &globalCmd}
 	app.AddCommand(profileCmd.Command())
 
-	// profile sub-command
+	// project sub-command
 	projectCmd := cmdProject{global: &globalCmd}
 	app.AddCommand(projectCmd.Command())
 
@@ -204,6 +210,10 @@ For help with any of those, simply call them with --help.`))
 	queryCmd := cmdQuery{global: &globalCmd}
 	app.AddCommand(queryCmd.Command())
 
+	// rebuild sub-command
+	rebuildCmd := cmdRebuild{global: &globalCmd}
+	app.AddCommand(rebuildCmd.Command())
+
 	// rename sub-command
 	renameCmd := cmdRename{global: &globalCmd}
 	app.AddCommand(renameCmd.Command())
@@ -244,6 +254,9 @@ For help with any of those, simply call them with --help.`))
 	warningCmd := cmdWarning{global: &globalCmd}
 	app.AddCommand(warningCmd.Command())
 
+	authCmd := cmdAuth{global: &globalCmd}
+	app.AddCommand(authCmd.command())
+
 	// Get help command
 	app.InitDefaultHelpCmd()
 	var help *cobra.Command
@@ -299,6 +312,8 @@ To easily setup a local LXD server in a virtual machine, consider using: https:/
 	}
 }
 
+// PreRun is set as the (*cobra.Command).PersistentPreRunE for the top level lxc command. It loads configuration and
+// performs additional checks if it detects that LXD has not been configured yet.
 func (c *cmdGlobal) PreRun(cmd *cobra.Command, args []string) error {
 	var err error
 
@@ -384,7 +399,7 @@ func (c *cmdGlobal) PreRun(cmd *cobra.Command, args []string) error {
 			flush = true
 		}
 
-		if !shared.StringInSlice(cmd.Name(), []string{"init", "launch"}) {
+		if !shared.ValueInSlice(cmd.Name(), []string{"init", "launch"}) {
 			fmt.Fprintf(os.Stderr, i18n.G(`To start your first container, try: lxc launch ubuntu:22.04
 Or for a virtual machine: lxc launch ubuntu:22.04 --vm`)+"\n")
 			flush = true
@@ -413,11 +428,12 @@ Or for a virtual machine: lxc launch ubuntu:22.04 --vm`)+"\n")
 	return nil
 }
 
+// PostRun is set as the (*cobra.Command).PersistentPostRunE hook on the top level lxc command.
+// It saves any configuration that must persist between runs.
 func (c *cmdGlobal) PostRun(cmd *cobra.Command, args []string) error {
-	// Macaroon teardown
 	if c.conf != nil && shared.PathExists(c.confPath) {
-		// Save cookies on exit
-		c.conf.SaveCookies()
+		// Save OIDC tokens on exit
+		c.conf.SaveOIDCTokens()
 	}
 
 	return nil
@@ -429,6 +445,8 @@ type remoteResource struct {
 	name   string
 }
 
+// ParseServers parses a list of remotes (`<remote>:<resource>...`) and calls (*config.Config).GetInstanceServer
+// for each remote to configure a new connection.
 func (c *cmdGlobal) ParseServers(remotes ...string) ([]remoteResource, error) {
 	servers := map[string]lxd.InstanceServer{}
 	resources := []remoteResource{}
@@ -468,6 +486,7 @@ func (c *cmdGlobal) ParseServers(remotes ...string) ([]remoteResource, error) {
 	return resources, nil
 }
 
+// CheckArgs checks that the given list of arguments has length between minArgs and maxArgs.
 func (c *cmdGlobal) CheckArgs(cmd *cobra.Command, args []string, minArgs int, maxArgs int) (bool, error) {
 	if len(args) < minArgs || (maxArgs != -1 && len(args) > maxArgs) {
 		_ = cmd.Help()
