@@ -2,15 +2,15 @@ package main
 
 import (
 	"fmt"
+	"strings"
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
+	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
 )
 
 type cmdMove struct {
@@ -193,35 +193,41 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 		}
 	}
 
-	// Support for server-side pool move.
-	if c.flagStorage != "" && sourceRemote == destRemote {
+	// Support for server-side move. Currently, such migration can only move an instance to different project
+	// or storage pool. If specific profile, device or config is provided, the instance should be copied (move using copy).
+	if sourceRemote == destRemote && (c.flagStorage != "" || c.flagTargetProject != "") {
 		source, err := conf.GetInstanceServer(sourceRemote)
 		if err != nil {
 			return err
 		}
 
-		if source.HasExtension("instance_pool_move") {
+		if source.HasExtension("instance_move_config") {
 			if c.flagMode != moveDefaultMode {
-				return fmt.Errorf(i18n.G("The --mode flag can't be used with --storage"))
+				return fmt.Errorf(i18n.G("The --mode flag can't be used with --storage or --target-project"))
 			}
 
-			return moveInstancePool(conf, sourceResource, destResource, c.flagInstanceOnly, c.flagStorage, stateful)
-		}
-	}
-
-	// Support for server-side project move.
-	if c.flagTargetProject != "" && sourceRemote == destRemote {
-		source, err := conf.GetInstanceServer(sourceRemote)
-		if err != nil {
-			return err
-		}
-
-		if source.HasExtension("instance_project_move") {
-			if c.flagMode != moveDefaultMode {
-				return fmt.Errorf(i18n.G("The --mode flag can't be used with --target-project"))
+			// Evaluate provided profiles. If no profiles were provided and flag noProfiles is false,
+			// existing instance profiles will be retained. Otherwise, flags are respected.
+			var profiles *[]string
+			if len(c.flagProfile) > 0 {
+				profiles = &c.flagProfile
+			} else if c.flagNoProfiles {
+				profiles = &[]string{}
 			}
 
-			return moveInstanceProject(conf, sourceResource, destResource, c.flagTargetProject, c.flagInstanceOnly, stateful)
+			return moveInstance(conf, sourceResource, destResource, c.flagStorage, c.flagTargetProject, c.flagConfig, c.flagDevice, profiles, c.flagInstanceOnly, stateful)
+		}
+
+		if source.HasExtension("instance_pool_move") && source.HasExtension("instance_project_move") {
+			if len(c.flagConfig) != 0 || len(c.flagDevice) != 0 || len(c.flagProfile) != 0 || c.flagNoProfiles {
+				return fmt.Errorf("The move command does not support flags --config, --device, --profile, and --no-profiles. Please use copy instead")
+			}
+
+			if c.flagMode != moveDefaultMode {
+				return fmt.Errorf(i18n.G("The --mode flag can't be used with --storage or --target-project"))
+			}
+
+			return moveInstance(conf, sourceResource, destResource, c.flagStorage, c.flagTargetProject, []string{}, []string{}, nil, c.flagInstanceOnly, stateful)
 		}
 	}
 
@@ -304,7 +310,7 @@ func moveClusterInstance(conf *config.Config, sourceResource string, destResourc
 	}
 
 	// Watch the background operation
-	progress := utils.ProgressRenderer{
+	progress := cli.ProgressRenderer{
 		Format: i18n.G("Transferring instance: %s"),
 		Quiet:  quiet,
 	}
@@ -315,7 +321,7 @@ func moveClusterInstance(conf *config.Config, sourceResource string, destResourc
 		return err
 	}
 	// Wait for the move to complete
-	err = utils.CancelableWait(op, &progress)
+	err = cli.CancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return err
@@ -331,8 +337,8 @@ func moveClusterInstance(conf *config.Config, sourceResource string, destResourc
 	return nil
 }
 
-// Move an instance between pools using special POST /instances/<name> API.
-func moveInstancePool(conf *config.Config, sourceResource string, destResource string, instanceOnly bool, storage string, stateful bool) error {
+// Move an instance between pools and projects using special POST /instances/<name> API.
+func moveInstance(conf *config.Config, sourceResource string, destResource string, storage string, targetProject string, config []string, devices []string, profiles *[]string, instanceOnly bool, stateful bool) error {
 	// Parse the source.
 	sourceRemote, sourceName, err := conf.ParseRemote(sourceResource)
 	if err != nil {
@@ -361,65 +367,54 @@ func moveInstancePool(conf *config.Config, sourceResource string, destResource s
 		return fmt.Errorf(i18n.G("Failed to connect to cluster member: %w"), err)
 	}
 
+	// Copy of an instance into a new instance.
+	inst, _, err := source.GetInstance(sourceName)
+	if err != nil {
+		return err
+	}
+
+	// Overwrite the config values.
+	for _, entry := range config {
+		key, value, found := strings.Cut(entry, "=")
+		if !found {
+			return fmt.Errorf(i18n.G("Bad key=value pair: %q"), entry)
+		}
+
+		inst.Config[key] = value
+	}
+
+	// Parse device map and overwrite device settings.
+	deviceMap, err := parseDeviceOverrides(devices)
+	if err != nil {
+		return err
+	}
+
+	for k, m := range deviceMap {
+		if inst.Devices[k] == nil {
+			inst.Devices[k] = m
+			continue
+		}
+
+		for key, value := range m {
+			inst.Devices[k][key] = value
+		}
+	}
+
 	// Pass the new pool to the migration API.
 	req := api.InstancePost{
 		Name:         destName,
 		Migration:    true,
+		InstanceOnly: instanceOnly,
 		Pool:         storage,
-		InstanceOnly: instanceOnly,
-		Live:         stateful,
-	}
-
-	op, err := source.MigrateInstance(sourceName, req)
-	if err != nil {
-		return fmt.Errorf(i18n.G("Migration API failure: %w"), err)
-	}
-
-	err = op.Wait()
-	if err != nil {
-		return fmt.Errorf(i18n.G("Migration operation failure: %w"), err)
-	}
-
-	return nil
-}
-
-// Move an instance between projects using special POST /instances/<name> API.
-func moveInstanceProject(conf *config.Config, sourceResource string, destResource string, targetProject string, instanceOnly bool, stateful bool) error {
-	// Parse the source.
-	sourceRemote, sourceName, err := conf.ParseRemote(sourceResource)
-	if err != nil {
-		return err
-	}
-
-	// Parse the destination.
-	_, destName, err := conf.ParseRemote(destResource)
-	if err != nil {
-		return err
-	}
-
-	// Make sure we have an instance or snapshot name.
-	if sourceName == "" {
-		return fmt.Errorf(i18n.G("You must specify a source instance name"))
-	}
-
-	// The destination name is optional.
-	if destName == "" {
-		destName = sourceName
-	}
-
-	// Connect to the source host.
-	source, err := conf.GetInstanceServer(sourceRemote)
-	if err != nil {
-		return fmt.Errorf(i18n.G("Failed to connect to cluster member: %w"), err)
-	}
-
-	// Pass the new project to the migration API.
-	req := api.InstancePost{
-		Name:         destName,
-		Migration:    true,
 		Project:      targetProject,
-		InstanceOnly: instanceOnly,
 		Live:         stateful,
+		Config:       inst.Config,
+		Devices:      inst.Devices,
+	}
+
+	// Overwrite profiles.
+	if profiles != nil {
+		req.Profiles = *profiles
 	}
 
 	op, err := source.MigrateInstance(sourceName, req)
