@@ -10,13 +10,12 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
+	"github.com/canonical/lxd/shared/units"
 )
 
 type cmdNetwork struct {
@@ -73,6 +72,10 @@ func (c *cmdNetwork) Command() *cobra.Command {
 	networkListCmd := cmdNetworkList{global: c.global, network: c}
 	cmd.AddCommand(networkListCmd.Command())
 
+	// List allocations
+	networkListAllocationsCmd := cmdNetworkListAllocations{global: c.global, network: c}
+	cmd.AddCommand(networkListAllocationsCmd.Command())
+
 	// List leases
 	networkListLeasesCmd := cmdNetworkListLeases{global: c.global, network: c}
 	cmd.AddCommand(networkListLeasesCmd.Command())
@@ -101,6 +104,10 @@ func (c *cmdNetwork) Command() *cobra.Command {
 	networkForwardCmd := cmdNetworkForward{global: c.global}
 	cmd.AddCommand(networkForwardCmd.Command())
 
+	// Load Balancer
+	networkLoadBalancerCmd := cmdNetworkLoadBalancer{global: c.global}
+	cmd.AddCommand(networkLoadBalancerCmd.Command())
+
 	// Peer
 	networkPeerCmd := cmdNetworkPeer{global: c.global}
 	cmd.AddCommand(networkPeerCmd.Command())
@@ -284,6 +291,11 @@ func (c *cmdNetworkCreate) Command() *cobra.Command {
 	cmd.Use = usage("create", i18n.G("[<remote>:]<network> [key=value...]"))
 	cmd.Short = i18n.G("Create new networks")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Create new networks`))
+	cmd.Example = cli.FormatSection("", i18n.G(`lxc network create foo
+    Create a new network called foo
+
+lxc network create bar network=baz --type ovn
+    Create a new OVN network called bar using baz as its uplink network`))
 
 	cmd.Flags().StringVar(&c.network.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.Flags().StringVarP(&c.network.flagType, "type", "t", "", i18n.G("Network type")+"``")
@@ -697,6 +709,8 @@ func (c *cmdNetworkEdit) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkGet struct {
 	global  *cmdGlobal
 	network *cmdNetwork
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkGet) Command() *cobra.Command {
@@ -707,6 +721,7 @@ func (c *cmdNetworkGet) Command() *cobra.Command {
 		`Get values for network configuration keys`))
 
 	cmd.Flags().StringVar(&c.network.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a network property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -742,9 +757,19 @@ func (c *cmdNetworkGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range resp.Config {
-		if k == args[1] {
-			fmt.Printf("%s\n", v)
+	if c.flagIsProperty {
+		w := resp.Writable()
+		res, err := getFieldByJsonTag(&w, args[1])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the network %q: %v"), args[1], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		for k, v := range resp.Config {
+			if k == args[1] {
+				fmt.Printf("%s\n", v)
+			}
 		}
 	}
 
@@ -925,7 +950,7 @@ func (c *cmdNetworkList) Run(cmd *cobra.Command, args []string) error {
 
 	data := [][]string{}
 	for _, network := range networks {
-		if shared.StringInSlice(network.Type, []string{"loopback", "unknown"}) {
+		if shared.ValueInSlice(network.Type, []string{"loopback", "unknown"}) {
 			continue
 		}
 
@@ -949,7 +974,7 @@ func (c *cmdNetworkList) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, details)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
@@ -962,7 +987,7 @@ func (c *cmdNetworkList) Run(cmd *cobra.Command, args []string) error {
 		i18n.G("STATE"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, networks)
+	return cli.RenderTable(c.flagFormat, header, data, networks)
 }
 
 // List leases.
@@ -1021,7 +1046,7 @@ func (c *cmdNetworkListLeases) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, entry)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("HOSTNAME"),
@@ -1034,7 +1059,7 @@ func (c *cmdNetworkListLeases) Run(cmd *cobra.Command, args []string) error {
 		header = append(header, i18n.G("LOCATION"))
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, leases)
+	return cli.RenderTable(c.flagFormat, header, data, leases)
 }
 
 // Rename.
@@ -1092,6 +1117,8 @@ func (c *cmdNetworkRename) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkSet struct {
 	global  *cmdGlobal
 	network *cmdNetwork
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkSet) Command() *cobra.Command {
@@ -1105,6 +1132,7 @@ For backward compatibility, a single configuration key may still be set with:
     lxc network set [<remote>:]<network> <key> <value>`))
 
 	cmd.Flags().StringVar(&c.network.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a network property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -1151,11 +1179,28 @@ func (c *cmdNetworkSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range keys {
-		network.Config[k] = v
+	writable := network.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	return client.UpdateNetwork(resource.name, network.Writable(), etag)
+	return client.UpdateNetwork(resource.name, writable, etag)
 }
 
 // Show.
@@ -1224,6 +1269,8 @@ type cmdNetworkUnset struct {
 	global     *cmdGlobal
 	network    *cmdNetwork
 	networkSet *cmdNetworkSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkUnset) Command() *cobra.Command {
@@ -1234,6 +1281,7 @@ func (c *cmdNetworkUnset) Command() *cobra.Command {
 		`Unset network configuration keys`))
 
 	cmd.Flags().StringVar(&c.network.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a network property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -1246,6 +1294,8 @@ func (c *cmdNetworkUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.networkSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.networkSet.Run(cmd, args)
 }
