@@ -11,12 +11,11 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
 )
 
 type cmdNetworkACL struct {
@@ -142,7 +141,7 @@ func (c *cmdNetworkACLList) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, details)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
@@ -150,7 +149,7 @@ func (c *cmdNetworkACLList) Run(cmd *cobra.Command, args []string) error {
 		i18n.G("USED BY"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, acls)
+	return cli.RenderTable(c.flagFormat, header, data, acls)
 }
 
 // Show.
@@ -256,6 +255,8 @@ func (c *cmdNetworkACLShowLog) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkACLGet struct {
 	global     *cmdGlobal
 	networkACL *cmdNetworkACL
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkACLGet) Command() *cobra.Command {
@@ -263,6 +264,8 @@ func (c *cmdNetworkACLGet) Command() *cobra.Command {
 	cmd.Use = usage("get", i18n.G("[<remote>:]<ACL> <key>"))
 	cmd.Short = i18n.G("Get values for network ACL configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Get values for network ACL configuration keys"))
+
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a network ACL property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -292,9 +295,19 @@ func (c *cmdNetworkACLGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range resp.Config {
-		if k == args[1] {
-			fmt.Printf("%s\n", v)
+	if c.flagIsProperty {
+		w := resp.Writable()
+		res, err := getFieldByJsonTag(&w, args[1])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the network ACL %q: %v"), args[1], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		for k, v := range resp.Config {
+			if k == args[1] {
+				fmt.Printf("%s\n", v)
+			}
 		}
 	}
 
@@ -388,6 +401,8 @@ func (c *cmdNetworkACLCreate) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkACLSet struct {
 	global     *cmdGlobal
 	networkACL *cmdNetworkACL
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkACLSet) Command() *cobra.Command {
@@ -400,6 +415,7 @@ func (c *cmdNetworkACLSet) Command() *cobra.Command {
 For backward compatibility, a single configuration key may still be set with:
     lxc network set [<remote>:]<ACL> <key> <value>`))
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a network ACL property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -436,11 +452,28 @@ func (c *cmdNetworkACLSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range keys {
-		netACL.Config[k] = v
+	writable := netACL.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	return resource.server.UpdateNetworkACL(resource.name, netACL.Writable(), etag)
+	return resource.server.UpdateNetworkACL(resource.name, writable, etag)
 }
 
 // Unset.
@@ -448,6 +481,8 @@ type cmdNetworkACLUnset struct {
 	global        *cmdGlobal
 	networkACL    *cmdNetworkACL
 	networkACLSet *cmdNetworkACLSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkACLUnset) Command() *cobra.Command {
@@ -457,6 +492,7 @@ func (c *cmdNetworkACLUnset) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Unset network ACL configuration keys"))
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a network ACL property"))
 	return cmd
 }
 
@@ -467,6 +503,8 @@ func (c *cmdNetworkACLUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.networkACLSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.networkACLSet.Run(cmd, args)
 }
@@ -541,7 +579,7 @@ func (c *cmdNetworkACLEdit) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
-		// Allow output of `lxc network acl show` command to passed in here, but only take the contents
+		// Allow output of `lxc network acl show` command to be passed in here, but only take the contents
 		// of the NetworkACLPut fields when updating the ACL. The other fields are silently discarded.
 		newdata := api.NetworkACL{}
 		err = yaml.UnmarshalStrict(contents, &newdata)
@@ -549,7 +587,7 @@ func (c *cmdNetworkACLEdit) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
-		return resource.server.UpdateNetworkACL(resource.name, newdata.NetworkACLPut, "")
+		return resource.server.UpdateNetworkACL(resource.name, newdata.Writable(), "")
 	}
 
 	// Get the current config.
