@@ -10,12 +10,11 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
 )
 
 type cmdNetworkForward struct {
@@ -139,7 +138,7 @@ func (c *cmdNetworkForwardList) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, details)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("LISTEN ADDRESS"),
@@ -152,7 +151,7 @@ func (c *cmdNetworkForwardList) Run(cmd *cobra.Command, args []string) error {
 		header = append(header, i18n.G("LOCATION"))
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, forwards)
+	return cli.RenderTable(c.flagFormat, header, data, forwards)
 }
 
 // Show.
@@ -319,6 +318,8 @@ func (c *cmdNetworkForwardCreate) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkForwardGet struct {
 	global         *cmdGlobal
 	networkForward *cmdNetworkForward
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkForwardGet) Command() *cobra.Command {
@@ -326,6 +327,8 @@ func (c *cmdNetworkForwardGet) Command() *cobra.Command {
 	cmd.Use = usage("get", i18n.G("[<remote>:]<network> <listen_address> <key>"))
 	cmd.Short = i18n.G("Get values for network forward configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Get values for network forward configuration keys"))
+
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a network forward property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -361,9 +364,19 @@ func (c *cmdNetworkForwardGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range forward.Config {
-		if k == args[2] {
-			fmt.Printf("%s\n", v)
+	if c.flagIsProperty {
+		w := forward.Writable()
+		res, err := getFieldByJsonTag(&w, args[2])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the network forward %q: %v"), args[1], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		for k, v := range forward.Config {
+			if k == args[2] {
+				fmt.Printf("%s\n", v)
+			}
 		}
 	}
 
@@ -374,6 +387,8 @@ func (c *cmdNetworkForwardGet) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkForwardSet struct {
 	global         *cmdGlobal
 	networkForward *cmdNetworkForward
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkForwardSet) Command() *cobra.Command {
@@ -387,6 +402,7 @@ For backward compatibility, a single configuration key may still be set with:
     lxc network set [<remote>:]<network> <listen_address> <key> <value>`))
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a network forward property"))
 	cmd.Flags().StringVar(&c.networkForward.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 
 	return cmd
@@ -438,13 +454,30 @@ func (c *cmdNetworkForwardSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range keys {
-		forward.Config[k] = v
+	writable := forward.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	forward.Normalise()
+	writable.Normalise()
 
-	return client.UpdateNetworkForward(resource.name, forward.ListenAddress, forward.Writable(), etag)
+	return client.UpdateNetworkForward(resource.name, forward.ListenAddress, writable, etag)
 }
 
 // Unset.
@@ -452,6 +485,8 @@ type cmdNetworkForwardUnset struct {
 	global            *cmdGlobal
 	networkForward    *cmdNetworkForward
 	networkForwardSet *cmdNetworkForwardSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkForwardUnset) Command() *cobra.Command {
@@ -461,6 +496,7 @@ func (c *cmdNetworkForwardUnset) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Unset network forward keys"))
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a network forward property"))
 	return cmd
 }
 
@@ -471,6 +507,8 @@ func (c *cmdNetworkForwardUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.networkForwardSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.networkForwardSet.Run(cmd, args)
 }
@@ -505,7 +543,7 @@ func (c *cmdNetworkForwardEdit) helpTemplate() string {
 ### config:
 ###   target_address: 198.51.100.2
 ### description: test desc
-### port:
+### ports:
 ### - description: port forward
 ###   protocol: tcp
 ###   listen_port: 80,81,8080-8090
@@ -553,8 +591,8 @@ func (c *cmdNetworkForwardEdit) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
-		// Allow output of `lxc network forward show` command to passed in here, but only take the contents
-		// of the NetworkForwardPut fields when updating. The other fields are silently discarded.
+		// Allow output of `lxc network forward show` command to be passed in here, but only take the
+		// contents of the NetworkForwardPut fields when updating. The other fields are silently discarded.
 		newData := api.NetworkForward{}
 		err = yaml.UnmarshalStrict(contents, &newData)
 		if err != nil {
@@ -563,7 +601,7 @@ func (c *cmdNetworkForwardEdit) Run(cmd *cobra.Command, args []string) error {
 
 		newData.Normalise()
 
-		return client.UpdateNetworkForward(resource.name, args[1], newData.NetworkForwardPut, "")
+		return client.UpdateNetworkForward(resource.name, args[1], newData.Writable(), "")
 	}
 
 	// Get the current config.
