@@ -10,12 +10,11 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
 )
 
 type cmdNetworkZone struct {
@@ -133,7 +132,7 @@ func (c *cmdNetworkZoneList) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, details)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
@@ -141,7 +140,7 @@ func (c *cmdNetworkZoneList) Run(cmd *cobra.Command, args []string) error {
 		i18n.G("USED BY"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, zones)
+	return cli.RenderTable(c.flagFormat, header, data, zones)
 }
 
 // Show.
@@ -201,6 +200,8 @@ func (c *cmdNetworkZoneShow) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkZoneGet struct {
 	global      *cmdGlobal
 	networkZone *cmdNetworkZone
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkZoneGet) Command() *cobra.Command {
@@ -210,6 +211,7 @@ func (c *cmdNetworkZoneGet) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Get values for network zone configuration keys"))
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a network zone property"))
 	return cmd
 }
 
@@ -237,9 +239,19 @@ func (c *cmdNetworkZoneGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range resp.Config {
-		if k == args[1] {
-			fmt.Printf("%s\n", v)
+	if c.flagIsProperty {
+		w := resp.Writable()
+		res, err := getFieldByJsonTag(&w, args[1])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the network zone %q: %v"), args[1], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		for k, v := range resp.Config {
+			if k == args[1] {
+				fmt.Printf("%s\n", v)
+			}
 		}
 	}
 
@@ -331,6 +343,8 @@ func (c *cmdNetworkZoneCreate) Run(cmd *cobra.Command, args []string) error {
 type cmdNetworkZoneSet struct {
 	global      *cmdGlobal
 	networkZone *cmdNetworkZone
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkZoneSet) Command() *cobra.Command {
@@ -344,6 +358,7 @@ For backward compatibility, a single configuration key may still be set with:
     lxc network set [<remote>:]<Zone> <key> <value>`))
 
 	cmd.RunE = c.Run
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a network zone property"))
 
 	return cmd
 }
@@ -379,11 +394,28 @@ func (c *cmdNetworkZoneSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range keys {
-		netZone.Config[k] = v
+	writable := netZone.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	return resource.server.UpdateNetworkZone(resource.name, netZone.Writable(), etag)
+	return resource.server.UpdateNetworkZone(resource.name, writable, etag)
 }
 
 // Unset.
@@ -391,6 +423,8 @@ type cmdNetworkZoneUnset struct {
 	global         *cmdGlobal
 	networkZone    *cmdNetworkZone
 	networkZoneSet *cmdNetworkZoneSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkZoneUnset) Command() *cobra.Command {
@@ -400,6 +434,8 @@ func (c *cmdNetworkZoneUnset) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Unset network zone configuration keys"))
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a network zone property"))
+
 	return cmd
 }
 
@@ -410,6 +446,8 @@ func (c *cmdNetworkZoneUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.networkZoneSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.networkZoneSet.Run(cmd, args)
 }
@@ -472,7 +510,7 @@ func (c *cmdNetworkZoneEdit) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
-		// Allow output of `lxc network zone show` command to passed in here, but only take the contents
+		// Allow output of `lxc network zone show` command to be passed in here, but only take the contents
 		// of the NetworkZonePut fields when updating the Zone. The other fields are silently discarded.
 		newdata := api.NetworkZone{}
 		err = yaml.UnmarshalStrict(contents, &newdata)
@@ -480,7 +518,7 @@ func (c *cmdNetworkZoneEdit) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
-		return resource.server.UpdateNetworkZone(resource.name, newdata.NetworkZonePut, "")
+		return resource.server.UpdateNetworkZone(resource.name, newdata.Writable(), "")
 	}
 
 	// Get the current config.
@@ -697,7 +735,7 @@ func (c *cmdNetworkZoneRecordList) Run(cmd *cobra.Command, args []string) error
 		data = append(data, details)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
@@ -705,7 +743,7 @@ func (c *cmdNetworkZoneRecordList) Run(cmd *cobra.Command, args []string) error
 		i18n.G("ENTRIES"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, records)
+	return cli.RenderTable(c.flagFormat, header, data, records)
 }
 
 // Show.
@@ -762,6 +800,8 @@ func (c *cmdNetworkZoneRecordShow) Run(cmd *cobra.Command, args []string) error
 type cmdNetworkZoneRecordGet struct {
 	global            *cmdGlobal
 	networkZoneRecord *cmdNetworkZoneRecord
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkZoneRecordGet) Command() *cobra.Command {
@@ -771,6 +811,7 @@ func (c *cmdNetworkZoneRecordGet) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Get values for network zone record configuration keys"))
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a network zone record property"))
 	return cmd
 }
 
@@ -797,9 +838,19 @@ func (c *cmdNetworkZoneRecordGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range resp.Config {
-		if k == args[2] {
-			fmt.Printf("%s\n", v)
+	if c.flagIsProperty {
+		w := resp.Writable()
+		res, err := getFieldByJsonTag(&w, args[2])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the network zone record %q: %v"), args[2], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		for k, v := range resp.Config {
+			if k == args[2] {
+				fmt.Printf("%s\n", v)
+			}
 		}
 	}
 
@@ -890,6 +941,8 @@ func (c *cmdNetworkZoneRecordCreate) Run(cmd *cobra.Command, args []string) erro
 type cmdNetworkZoneRecordSet struct {
 	global            *cmdGlobal
 	networkZoneRecord *cmdNetworkZoneRecord
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkZoneRecordSet) Command() *cobra.Command {
@@ -901,6 +954,7 @@ func (c *cmdNetworkZoneRecordSet) Command() *cobra.Command {
 
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a network zone record property"))
 	return cmd
 }
 
@@ -934,11 +988,28 @@ func (c *cmdNetworkZoneRecordSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range keys {
-		netRecord.Config[k] = v
+	writable := netRecord.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	return resource.server.UpdateNetworkZoneRecord(resource.name, args[1], netRecord.Writable(), etag)
+	return resource.server.UpdateNetworkZoneRecord(resource.name, args[1], writable, etag)
 }
 
 // Unset.
@@ -946,6 +1017,8 @@ type cmdNetworkZoneRecordUnset struct {
 	global               *cmdGlobal
 	networkZoneRecord    *cmdNetworkZoneRecord
 	networkZoneRecordSet *cmdNetworkZoneRecordSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdNetworkZoneRecordUnset) Command() *cobra.Command {
@@ -955,6 +1028,7 @@ func (c *cmdNetworkZoneRecordUnset) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("Unset network zone record configuration keys"))
 	cmd.RunE = c.Run
 
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a network zone record property"))
 	return cmd
 }
 
@@ -965,6 +1039,8 @@ func (c *cmdNetworkZoneRecordUnset) Run(cmd *cobra.Command, args []string) error
 		return err
 	}
 
+	c.networkZoneRecordSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.networkZoneRecordSet.Run(cmd, args)
 }
@@ -1026,7 +1102,7 @@ func (c *cmdNetworkZoneRecordEdit) Run(cmd *cobra.Command, args []string) error
 			return err
 		}
 
-		// Allow output of `lxc network zone show` command to passed in here, but only take the contents
+		// Allow output of `lxc network zone show` command to be passed in here, but only take the contents
 		// of the NetworkZonePut fields when updating the Zone. The other fields are silently discarded.
 		newdata := api.NetworkZoneRecord{}
 		err = yaml.UnmarshalStrict(contents, &newdata)
@@ -1034,7 +1110,7 @@ func (c *cmdNetworkZoneRecordEdit) Run(cmd *cobra.Command, args []string) error
 			return err
 		}
 
-		return resource.server.UpdateNetworkZoneRecord(resource.name, args[1], newdata.NetworkZoneRecordPut, "")
+		return resource.server.UpdateNetworkZoneRecord(resource.name, args[1], newdata.Writable(), "")
 	}
 
 	// Get the current config.
