@@ -10,13 +10,12 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
+	"github.com/canonical/lxd/shared/units"
 )
 
 type cmdProject struct {
@@ -125,12 +124,12 @@ func (c *cmdProjectCreate) Run(cmd *cobra.Command, args []string) error {
 
 	project.Config = map[string]string{}
 	for _, entry := range c.flagConfig {
-		if !strings.Contains(entry, "=") {
-			return fmt.Errorf(i18n.G("Bad key=value pair: %s"), entry)
+		key, value, found := strings.Cut(entry, "=")
+		if !found {
+			return fmt.Errorf(i18n.G("Bad key=value pair: %q"), entry)
 		}
 
-		fields := strings.SplitN(entry, "=", 2)
-		project.Config[fields[0]] = fields[1]
+		project.Config[key] = value
 	}
 
 	err = resource.server.CreateProject(project)
@@ -238,11 +237,15 @@ func (c *cmdProjectEdit) helpTemplate() string {
 ### A project consists of a set of features and a description.
 ###
 ### An example would look like:
-### name: my-project
-### features:
-###   images: True
-###   profiles: True
+### config:
+###   features.images: "true"
+###   features.networks: "true"
+###   features.networks.zones: "true"
+###   features.profiles: "true"
+###   features.storage.buckets: "true"
+###   features.storage.volumes: "true"
 ### description: My own project
+### name: my-project
 ###
 ### Note that the name is shown but cannot be changed`)
 }
@@ -335,6 +338,8 @@ func (c *cmdProjectEdit) Run(cmd *cobra.Command, args []string) error {
 type cmdProjectGet struct {
 	global  *cmdGlobal
 	project *cmdProject
+
+	flagIsProperty bool
 }
 
 func (c *cmdProjectGet) Command() *cobra.Command {
@@ -345,7 +350,7 @@ func (c *cmdProjectGet) Command() *cobra.Command {
 		`Get values for project configuration keys`))
 
 	cmd.RunE = c.Run
-
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a project property"))
 	return cmd
 }
 
@@ -374,7 +379,18 @@ func (c *cmdProjectGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf("%s\n", project.Config[args[1]])
+	if c.flagIsProperty {
+		w := project.Writable()
+		res, err := getFieldByJsonTag(&w, args[1])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the project %q: %v"), args[1], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		fmt.Printf("%s\n", project.Config[args[1]])
+	}
+
 	return nil
 }
 
@@ -452,33 +468,45 @@ func (c *cmdProjectList) Run(cmd *cobra.Command, args []string) error {
 			storageVolumes = i18n.G("YES")
 		}
 
+		storageBuckets := i18n.G("NO")
+		if shared.IsTrue(project.Config["features.storage.buckets"]) {
+			storageBuckets = i18n.G("YES")
+		}
+
 		networks := i18n.G("NO")
 		if shared.IsTrue(project.Config["features.networks"]) {
 			networks = i18n.G("YES")
 		}
 
+		networkZones := i18n.G("NO")
+		if shared.IsTrue(project.Config["features.networks.zones"]) {
+			networkZones = i18n.G("YES")
+		}
+
 		name := project.Name
 		if name == currentProject {
 			name = fmt.Sprintf("%s (%s)", name, i18n.G("current"))
 		}
 
 		strUsedBy := fmt.Sprintf("%d", len(project.UsedBy))
-		data = append(data, []string{name, images, profiles, storageVolumes, networks, project.Description, strUsedBy})
+		data = append(data, []string{name, images, profiles, storageVolumes, storageBuckets, networks, networkZones, project.Description, strUsedBy})
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
 		i18n.G("IMAGES"),
 		i18n.G("PROFILES"),
 		i18n.G("STORAGE VOLUMES"),
+		i18n.G("STORAGE BUCKETS"),
 		i18n.G("NETWORKS"),
+		i18n.G("NETWORK ZONES"),
 		i18n.G("DESCRIPTION"),
 		i18n.G("USED BY"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, projects)
+	return cli.RenderTable(c.flagFormat, header, data, projects)
 }
 
 // Rename.
@@ -541,6 +569,8 @@ func (c *cmdProjectRename) Run(cmd *cobra.Command, args []string) error {
 type cmdProjectSet struct {
 	global  *cmdGlobal
 	project *cmdProject
+
+	flagIsProperty bool
 }
 
 func (c *cmdProjectSet) Command() *cobra.Command {
@@ -554,7 +584,7 @@ For backward compatibility, a single configuration key may still be set with:
     lxc project set [<remote>:]<project> <key> <value>`))
 
 	cmd.RunE = c.Run
-
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a project property"))
 	return cmd
 }
 
@@ -589,11 +619,28 @@ func (c *cmdProjectSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range keys {
-		project.Config[k] = v
+	writable := project.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	return resource.server.UpdateProject(resource.name, project.Writable(), etag)
+	return resource.server.UpdateProject(resource.name, writable, etag)
 }
 
 // Unset.
@@ -601,6 +648,8 @@ type cmdProjectUnset struct {
 	global     *cmdGlobal
 	project    *cmdProject
 	projectSet *cmdProjectSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdProjectUnset) Command() *cobra.Command {
@@ -611,7 +660,7 @@ func (c *cmdProjectUnset) Command() *cobra.Command {
 		`Unset project configuration keys`))
 
 	cmd.RunE = c.Run
-
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a project property"))
 	return cmd
 }
 
@@ -622,6 +671,8 @@ func (c *cmdProjectUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.projectSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.projectSet.Run(cmd, args)
 }
@@ -788,7 +839,7 @@ func (c *cmdProjectInfo) Run(cmd *cobra.Command, args []string) error {
 	for k, v := range projectState.Resources {
 		limit := i18n.G("UNLIMITED")
 		if v.Limit >= 0 {
-			if shared.StringInSlice(k, byteLimits) {
+			if shared.ValueInSlice(k, byteLimits) {
 				limit = units.GetByteSizeStringIEC(v.Limit, 2)
 			} else {
 				limit = fmt.Sprintf("%d", v.Limit)
@@ -796,7 +847,7 @@ func (c *cmdProjectInfo) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		usage := ""
-		if shared.StringInSlice(k, byteLimits) {
+		if shared.ValueInSlice(k, byteLimits) {
 			usage = units.GetByteSizeStringIEC(v.Usage, 2)
 		} else {
 			usage = fmt.Sprintf("%d", v.Usage)
@@ -805,7 +856,7 @@ func (c *cmdProjectInfo) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, []string{strings.ToUpper(k), limit, usage})
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("RESOURCE"),
@@ -813,5 +864,5 @@ func (c *cmdProjectInfo) Run(cmd *cobra.Command, args []string) error {
 		i18n.G("USAGE"),
 	}
 
-	return utils.RenderTable(c.flagFormat, header, data, projectState)
+	return cli.RenderTable(c.flagFormat, header, data, projectState)
 }
