@@ -12,9 +12,10 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
 )
 
 type cmdQuery struct {
@@ -35,7 +36,6 @@ func (c *cmdQuery) Command() *cobra.Command {
 	cmd.Example = cli.FormatSection("", i18n.G(
 		`lxc query -X DELETE --wait /1.0/instances/c1
     Delete local instance "c1".`))
-	cmd.Hidden = true
 
 	cmd.RunE = c.Run
 	cmd.Flags().BoolVar(&c.flagRespWait, "wait", false, i18n.G("Wait for the operation to complete"))
@@ -72,6 +72,10 @@ func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("--project cannot be used with the query command"))
 	}
 
+	if !shared.ValueInSlice(c.flagAction, []string{"GET", "PUT", "POST", "PATCH", "DELETE"}) {
+		return fmt.Errorf(i18n.G("Action %q isn't supported by this tool"), c.flagAction)
+	}
+
 	// Parse the remote
 	remote, path, err := conf.ParseRemote(args[0])
 	if err != nil {
@@ -100,9 +104,10 @@ func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 	resp, _, err := d.RawQuery(c.flagAction, path, data, "")
 	if err != nil {
 		var jsonSyntaxError *json.SyntaxError
+		var jsonUnmarshalTypeError *json.UnmarshalTypeError
 
 		// If not JSON decoding error then fail immediately.
-		if !errors.As(err, &jsonSyntaxError) {
+		if !errors.As(err, &jsonSyntaxError) && !errors.As(err, &jsonUnmarshalTypeError) && err.Error() != "EOF" {
 			return err
 		}
 
