@@ -12,13 +12,12 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/termios"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/termios"
+	"github.com/canonical/lxd/shared/units"
 )
 
 type cmdStorage struct {
@@ -70,6 +69,10 @@ func (c *cmdStorage) Command() *cobra.Command {
 	storageUnsetCmd := cmdStorageUnset{global: c.global, storage: c, storageSet: &storageSetCmd}
 	cmd.AddCommand(storageUnsetCmd.Command())
 
+	// Bucket
+	storageBucketCmd := cmdStorageBucket{global: c.global}
+	cmd.AddCommand(storageBucketCmd.Command())
+
 	// Volume
 	storageVolumeCmd := cmdStorageVolume{global: c.global, storage: c}
 	cmd.AddCommand(storageVolumeCmd.Command())
@@ -330,6 +333,8 @@ func (c *cmdStorageEdit) Run(cmd *cobra.Command, args []string) error {
 type cmdStorageGet struct {
 	global  *cmdGlobal
 	storage *cmdStorage
+
+	flagIsProperty bool
 }
 
 func (c *cmdStorageGet) Command() *cobra.Command {
@@ -340,6 +345,7 @@ func (c *cmdStorageGet) Command() *cobra.Command {
 		`Get values for storage pool configuration keys`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a storage property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -375,9 +381,18 @@ func (c *cmdStorageGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range resp.Config {
-		if k == args[1] {
-			fmt.Printf("%s\n", v)
+	if c.flagIsProperty {
+		w := resp.Writable()
+		res, err := getFieldByJsonTag(&w, args[1])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the storage pool %q: %v"), args[1], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		v, ok := resp.Config[args[1]]
+		if ok {
+			fmt.Println(v)
 		}
 	}
 
@@ -459,53 +474,69 @@ func (c *cmdStorageInfo) Run(cmd *cobra.Command, args []string) error {
 	spaceusedstring := i18n.G("space used")
 
 	// Initialize the usedby map
-	poolusedby[usedbystring] = map[string][]string{}
+	poolusedby[usedbystring] = make(map[string][]string)
 
-	/* Build up the usedby map
-	/1.0/{instances,images,profiles}/storagepoolname and
-	/1.0/storage-pools/<poolname>/<type>/<volname>
-	remove the /1.0/ and build the map based on the resources name as key
-	and resources details as value */
+	// Build up the usedby map
 	for _, v := range pool.UsedBy {
 		u, err := url.Parse(v)
 		if err != nil {
 			continue
 		}
 
-		fields := strings.Split(u.Path[5:], "/")
-		bytype := fields[0]
-		bywhat := fields[1]
+		fields := strings.Split(strings.TrimPrefix(u.Path, "/1.0/"), "/")
+		fieldsLen := len(fields)
 
-		if bytype == "storage-pools" {
-			bytype = "volumes"
-			bywhat = fields[4]
-		}
+		entityType := "unrecognized"
+		entityName := u.Path
 
-		var info string
+		if fieldsLen > 1 {
+			entityType = fields[0]
+			entityName = fields[1]
 
-		// Show info regarding the project and target if present.
-		values := u.Query()
+			if fields[fieldsLen-2] == "snapshots" {
+				continue // Skip snapshots as the parent entity will be included once in the list.
+			}
 
-		proj := values.Get("project")
-		target := values.Get("target")
+			if fields[0] == "storage-pools" && fieldsLen > 3 {
+				entityType = fields[2]
+				entityName = fields[3]
 
-		if proj != "" {
-			info = fmt.Sprintf("project %q", proj)
-		}
-
-		if target != "" {
-			if info == "" {
-				info = fmt.Sprintf("target %q", target)
-			} else {
-				info = fmt.Sprintf("%s, target %q", info, target)
+				if entityType == "volumes" && fieldsLen > 4 {
+					entityName = fields[4]
+				}
 			}
 		}
 
-		if info != "" {
-			bywhat = fmt.Sprintf("%s (%s)", bywhat, info)
+		var sb strings.Builder
+		var attribs []string
+		sb.WriteString(entityName)
+
+		// Show info regarding the project and location if present.
+		values := u.Query()
+		projectName := values.Get("project")
+		if projectName != "" {
+			attribs = append(attribs, fmt.Sprintf("project %q", projectName))
 		}
 
-		poolusedby[usedbystring][bytype] = append(poolusedby[usedbystring][bytype], bywhat)
+		locationName := values.Get("target")
+		if locationName != "" {
+			attribs = append(attribs, fmt.Sprintf("location %q", locationName))
+		}
+
+		if len(attribs) > 0 {
+			sb.WriteString(" (")
+			for i, attrib := range attribs {
+				if i > 0 {
+					sb.WriteString(", ")
+				}
+
+				sb.WriteString(attrib)
+			}
+
+			sb.WriteString(")")
+		}
+
+		poolusedby[usedbystring][entityType] = append(poolusedby[usedbystring][entityType], sb.String())
 	}
 
 	// Initialize the info map
@@ -601,7 +632,7 @@ func (c *cmdStorageList) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, details)
 	}
 
-	sort.Sort(utils.ByName(data))
+	sort.Sort(cli.SortColumnsNaturally(data))
 
 	header := []string{
 		i18n.G("NAME"),
@@ -616,13 +647,15 @@ func (c *cmdStorageList) Run(cmd *cobra.Command, args []string) error {
 	header = append(header, i18n.G("USED BY"))
 	header = append(header, i18n.G("STATE"))
 
-	return utils.RenderTable(c.flagFormat, header, data, pools)
+	return cli.RenderTable(c.flagFormat, header, data, pools)
 }
 
 // Set.
 type cmdStorageSet struct {
 	global  *cmdGlobal
 	storage *cmdStorage
+
+	flagIsProperty bool
 }
 
 func (c *cmdStorageSet) Command() *cobra.Command {
@@ -636,6 +669,7 @@ For backward compatibility, a single configuration key may still be set with:
     lxc storage set [<remote>:]<pool> <key> <value>`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a storage property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -678,12 +712,33 @@ func (c *cmdStorageSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	// Update the pool
-	for k, v := range keys {
-		pool.Config[k] = v
+	writable := pool.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		if writable.Config == nil {
+			writable.Config = make(map[string]string)
+		}
+
+		// Update the volume config keys.
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	err = client.UpdateStoragePool(resource.name, pool.Writable(), etag)
+	err = client.UpdateStoragePool(resource.name, writable, etag)
 	if err != nil {
 		return err
 	}
@@ -781,6 +836,8 @@ type cmdStorageUnset struct {
 	global     *cmdGlobal
 	storage    *cmdStorage
 	storageSet *cmdStorageSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdStorageUnset) Command() *cobra.Command {
@@ -791,6 +848,7 @@ func (c *cmdStorageUnset) Command() *cobra.Command {
 		`Unset storage pool configuration keys`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a storage property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -803,6 +861,8 @@ func (c *cmdStorageUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.storageSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.storageSet.Run(cmd, args)
 }
