@@ -3,7 +3,9 @@ package main
 import (
 	"fmt"
 	"io"
+	"net/url"
 	"os"
+	"path"
 	"sort"
 	"strconv"
 	"strings"
@@ -12,15 +14,14 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	lxd "github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/termios"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/termios"
+	"github.com/canonical/lxd/shared/units"
 )
 
 type volumeColumn struct {
@@ -96,14 +97,14 @@ Unless specified through a prefix, all volume operations affect "custom" (user c
 	storageVolumeListCmd := cmdStorageVolumeList{global: c.global, storage: c.storage, storageVolume: c}
 	cmd.AddCommand(storageVolumeListCmd.Command())
 
-	// Move
-	storageVolumeMoveCmd := cmdStorageVolumeMove{global: c.global, storage: c.storage, storageVolume: c, storageVolumeCopy: &storageVolumeCopyCmd}
-	cmd.AddCommand(storageVolumeMoveCmd.Command())
-
 	// Rename
 	storageVolumeRenameCmd := cmdStorageVolumeRename{global: c.global, storage: c.storage, storageVolume: c}
 	cmd.AddCommand(storageVolumeRenameCmd.Command())
 
+	// Move
+	storageVolumeMoveCmd := cmdStorageVolumeMove{global: c.global, storage: c.storage, storageVolume: c, storageVolumeCopy: &storageVolumeCopyCmd, storageVolumeRename: &storageVolumeRenameCmd}
+	cmd.AddCommand(storageVolumeMoveCmd.Command())
+
 	// Set
 	storageVolumeSetCmd := cmdStorageVolumeSet{global: c.global, storage: c.storage, storageVolume: c}
 	cmd.AddCommand(storageVolumeSetCmd.Command())
@@ -134,7 +135,7 @@ func (c *cmdStorageVolume) parseVolume(defaultType string, name string) (string,
 	fields := strings.SplitN(name, "/", 2)
 	if len(fields) == 1 {
 		return fields[0], defaultType
-	} else if len(fields) == 2 && !shared.StringInSlice(fields[0], []string{"custom", "image", "container", "virtual-machine"}) {
+	} else if len(fields) == 2 && !shared.ValueInSlice(fields[0], []string{"custom", "image", "container", "virtual-machine"}) {
 		return name, defaultType
 	}
 
@@ -363,20 +364,58 @@ func (c *cmdStorageVolumeCopy) Run(cmd *cobra.Command, args []string) error {
 	srcServer := srcResource.server
 	srcPath := srcResource.name
 
-	// If a target was specified, specify the volume on the given member.
+	// If the source server is standalone then --target cannot be provided.
+	if c.storage.flagTarget != "" && !srcServer.IsClustered() {
+		return fmt.Errorf(i18n.G("Cannot set --target when source server is not clustered"))
+	}
+
+	// Get source pool and volume name
+	srcVolName, srcVolPool := c.storageVolume.parseVolumeWithPool(srcPath)
+	if srcVolPool == "" {
+		return fmt.Errorf(i18n.G("No storage pool for source volume specified"))
+	}
+
 	if c.storage.flagTarget != "" {
 		srcServer = srcServer.UseTarget(c.storage.flagTarget)
 	}
 
+	// Check if requested storage volume exists.
+	srcVolParentName, srcVolSnapName, srcIsSnapshot := api.GetParentAndSnapshotName(srcVolName)
+	srcVol, _, err := srcServer.GetStoragePoolVolume(srcVolPool, "custom", srcVolParentName)
+	if err != nil {
+		return err
+	}
+
+	if srcIsSnapshot && c.flagVolumeOnly {
+		return fmt.Errorf("Cannot set --volume-only when copying a snapshot")
+	}
+
+	// If the volume is in local storage, set the target to its location (or provide a helpful error
+	// message if the target is incorrect). If the volume is in remote storage (and the source server is clustered) we
+	// can use any provided target. Note that for standalone servers, this will set the target to "none".
+	if srcVol.Location != "" && srcVol.Location != "none" {
+		if c.storage.flagTarget != "" && c.storage.flagTarget != srcVol.Location {
+			return fmt.Errorf(i18n.G("Given target %q does not match source volume location %q"), c.storage.flagTarget, srcVol.Location)
+		}
+
+		srcServer = srcServer.UseTarget(srcVol.Location)
+	} else if c.storage.flagTarget != "" && srcServer.IsClustered() {
+		srcServer = srcServer.UseTarget(c.storage.flagTarget)
+	}
+
 	// Destination
 	dstResource := resources[1]
 	dstServer := dstResource.server
 	dstPath := dstResource.name
 
-	// Get source pool and volume name
-	srcVolName, srcVolPool := c.storageVolume.parseVolumeWithPool(srcPath)
-	if srcVolPool == "" {
-		return fmt.Errorf(i18n.G("No storage pool for source volume specified"))
+	// We can always set the destination target if the destination server is clustered (for local storage volumes this
+	// places the volume on the target member, for remote volumes this does nothing).
+	if c.storageVolume.flagDestinationTarget != "" {
+		if !dstServer.IsClustered() {
+			return fmt.Errorf(i18n.G("Cannot set --destination-target when destination server is not clustered"))
+		}
+
+		dstServer = dstServer.UseTarget(c.storageVolume.flagDestinationTarget)
 	}
 
 	// Get destination pool and volume name
@@ -403,15 +442,6 @@ func (c *cmdStorageVolumeCopy) Run(cmd *cobra.Command, args []string) error {
 		finalMsg = i18n.G("Storage volume moved successfully!")
 	}
 
-	var srcVol *api.StorageVolume
-
-	// Check if requested storage volume exists.
-	srcVolParentName, srcVolSnapName, srcIsSnapshot := api.GetParentAndSnapshotName(srcVolName)
-	srcVol, _, err = srcServer.GetStoragePoolVolume(srcVolPool, "custom", srcVolParentName)
-	if err != nil {
-		return err
-	}
-
 	// If source is a snapshot get source snapshot volume info and apply to the srcVol.
 	if srcIsSnapshot {
 		srcVolSnapshot, _, err := srcServer.GetStoragePoolVolumeSnapshot(srcVolPool, "custom", srcVolParentName, srcVolSnapName)
@@ -425,23 +455,6 @@ func (c *cmdStorageVolumeCopy) Run(cmd *cobra.Command, args []string) error {
 		srcVol.Description = srcVolSnapshot.Description
 	}
 
-	if srcResource.remote == dstResource.remote {
-		// If destination target was specified, copy the volume onto the given member.
-		// If no destination target is specified, this will be the same as the source.
-		if c.storageVolume.flagDestinationTarget != "" {
-			dstServer = dstServer.UseTarget(c.storageVolume.flagDestinationTarget)
-		} else {
-			dstServer = dstServer.UseTarget(srcVol.Location)
-		}
-	} else if c.storageVolume.flagDestinationTarget != "" {
-		return fmt.Errorf("Cannot use ---destination-target when copying to another remote")
-	}
-
-	// If no target is specified, use the member that contains the source volume.
-	if c.storage.flagTarget == "" {
-		srcServer = srcServer.UseTarget(srcVol.Location)
-	}
-
 	if cmd.Name() == "move" && srcServer == dstServer {
 		args := &lxd.StoragePoolVolumeMoveArgs{}
 		args.Name = dstVolName
@@ -471,7 +484,7 @@ func (c *cmdStorageVolumeCopy) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Register progress handler
-	progress := utils.ProgressRenderer{
+	progress := cli.ProgressRenderer{
 		Format: opMsg,
 		Quiet:  c.global.flagQuiet,
 	}
@@ -483,7 +496,7 @@ func (c *cmdStorageVolumeCopy) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Wait for operation to finish
-	err = utils.CancelableWait(op, &progress)
+	err = cli.CancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return err
@@ -629,7 +642,7 @@ func (c *cmdStorageVolumeDelete) Run(cmd *cobra.Command, args []string) error {
 	// Parse the input
 	volName, volType := c.storageVolume.parseVolume("custom", args[1])
 
-	// If a target was specified, create the volume on the given member.
+	// If a target was specified, delete the volume on the given member.
 	if c.storage.flagTarget != "" {
 		client = client.UseTarget(c.storage.flagTarget)
 	}
@@ -833,12 +846,15 @@ type cmdStorageVolumeEdit struct {
 
 func (c *cmdStorageVolumeEdit) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("edit", i18n.G("[<remote>:]<pool> <volume>[/<snapshot>]"))
+	cmd.Use = usage("edit", i18n.G("[<remote>:]<pool> [<type>/]<volume>"))
 	cmd.Short = i18n.G("Edit storage volume configurations as YAML")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Edit storage volume configurations as YAML`))
 	cmd.Example = cli.FormatSection("", i18n.G(
-		`lxc storage volume edit [<remote>:]<pool> <volume> < volume.yaml
+		`Provide the type of the storage volume if it is not custom.
+Supported types are custom, image, container and virtual-machine.
+
+lxc storage volume edit [<remote>:]<pool> [<type>/]<volume> < volume.yaml
     Update a storage volume using the content of pool.yaml.`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
@@ -1033,16 +1049,30 @@ type cmdStorageVolumeGet struct {
 	global        *cmdGlobal
 	storage       *cmdStorage
 	storageVolume *cmdStorageVolume
+
+	flagIsProperty bool
 }
 
 func (c *cmdStorageVolumeGet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("get", i18n.G("[<remote>:]<pool> <volume>[/<snapshot>] <key>"))
+	cmd.Use = usage("get", i18n.G("[<remote>:]<pool> [<type>/]<volume>[/<snapshot>] <key>"))
 	cmd.Short = i18n.G("Get values for storage volume configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Get values for storage volume configuration keys`))
+	cmd.Example = cli.FormatSection("", i18n.G(
+		`Provide the type of the storage volume if it is not custom.
+Supported types are custom, image, container and virtual-machine.
+
+Add the name of the snapshot if type is one of custom, container or virtual-machine.
+
+lxc storage volume get default data size
+    Returns the size of a custom volume "data" in pool "default".
+
+lxc storage volume get default virtual-machine/data snapshots.expiry
+    Returns the snapshot expiration period for a virtual machine "data" in pool "default".`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a storage volume property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -1086,15 +1116,22 @@ func (c *cmdStorageVolumeGet) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if isSnapshot {
-		// Get the storage volume snapshot entry
 		resp, _, err := client.GetStoragePoolVolumeSnapshot(resource.name, volType, fields[0], fields[1])
 		if err != nil {
 			return err
 		}
 
-		for k, v := range resp.Config {
-			if k == args[2] {
-				fmt.Printf("%s\n", v)
+		if c.flagIsProperty {
+			res, err := getFieldByJsonTag(resp, args[2])
+			if err != nil {
+				return fmt.Errorf(i18n.G("The property %q does not exist on the storage pool volume snapshot %s/%s: %v"), args[2], fields[0], fields[1], err)
+			}
+
+			fmt.Printf("%v\n", res)
+		} else {
+			v, ok := resp.Config[args[2]]
+			if ok {
+				fmt.Println(v)
 			}
 		}
 
@@ -1107,9 +1144,17 @@ func (c *cmdStorageVolumeGet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	for k, v := range resp.Config {
-		if k == args[2] {
-			fmt.Printf("%s\n", v)
+	if c.flagIsProperty {
+		res, err := getFieldByJsonTag(resp, args[2])
+		if err != nil {
+			return fmt.Errorf(i18n.G("The property %q does not exist on the storage pool volume %q: %v"), args[2], resource.name, err)
+		}
+
+		fmt.Printf("%v\n", res)
+	} else {
+		v, ok := resp.Config[args[2]]
+		if ok {
+			fmt.Println(v)
 		}
 	}
 
@@ -1125,10 +1170,19 @@ type cmdStorageVolumeInfo struct {
 
 func (c *cmdStorageVolumeInfo) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("info", i18n.G("[<remote>:]<pool> <volume>"))
+	cmd.Use = usage("info", i18n.G("[<remote>:]<pool> [<type>/]<volume>"))
 	cmd.Short = i18n.G("Show storage volume state information")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show storage volume state information`))
+	cmd.Example = cli.FormatSection("", i18n.G(
+		`Provide the type of the storage volume if it is not custom.
+Supported types are custom, container and virtual-machine.
+
+lxc storage volume info default data
+    Returns state information for a custom volume "data" in pool "default".
+
+lxc storage volume info default virtual-machine/data
+    Returns state information for a virtual machine "data" in pool "default".`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
@@ -1185,10 +1239,8 @@ func (c *cmdStorageVolumeInfo) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	volState, err := client.GetStoragePoolVolumeState(resource.name, volType, volName)
-	if err != nil {
-		return err
-	}
+	// Instead of failing here if the usage cannot be determined, it is just omitted.
+	volState, _ := client.GetStoragePoolVolumeState(resource.name, volType, volName)
 
 	volSnapshots, err := client.GetStoragePoolVolumeSnapshots(resource.name, volType, volName)
 	if err != nil {
@@ -1227,13 +1279,17 @@ func (c *cmdStorageVolumeInfo) Run(cmd *cobra.Command, args []string) error {
 		fmt.Printf(i18n.G("Location: %s")+"\n", vol.Location)
 	}
 
-	if volState.Usage != nil {
+	if volState != nil && volState.Usage != nil {
 		fmt.Printf(i18n.G("Usage: %s")+"\n", units.GetByteSizeStringIEC(int64(volState.Usage.Used), 2))
 		if volState.Usage.Total > 0 {
 			fmt.Printf(i18n.G("Total: %s")+"\n", units.GetByteSizeStringIEC(int64(volState.Usage.Total), 2))
 		}
 	}
 
+	if shared.TimeIsSet(vol.CreatedAt) {
+		fmt.Printf(i18n.G("Created: %s")+"\n", vol.CreatedAt.Local().Format(layout))
+	}
+
 	// List snapshots
 	firstSnapshot := true
 	if len(volSnapshots) > 0 {
@@ -1260,14 +1316,14 @@ func (c *cmdStorageVolumeInfo) Run(cmd *cobra.Command, args []string) error {
 			snapData = append(snapData, row)
 		}
 
-		sort.Sort(utils.ByName(snapData))
+		sort.Sort(cli.SortColumnsNaturally(snapData))
 		snapHeader := []string{
 			i18n.G("Name"),
 			i18n.G("Description"),
 			i18n.G("Expires at"),
 		}
 
-		_ = utils.RenderTable(utils.TableFormatTable, snapHeader, snapData, volSnapshots)
+		_ = cli.RenderTable(cli.TableFormatTable, snapHeader, snapData, volSnapshots)
 	}
 
 	// List backups
@@ -1319,7 +1375,7 @@ func (c *cmdStorageVolumeInfo) Run(cmd *cobra.Command, args []string) error {
 			i18n.G("Optimized Storage"),
 		}
 
-		_ = utils.RenderTable(utils.TableFormatTable, backupHeader, backupData, volBackups)
+		_ = cli.RenderTable(cli.TableFormatTable, backupHeader, backupData, volBackups)
 	}
 
 	return nil
@@ -1340,11 +1396,11 @@ type cmdStorageVolumeList struct {
 
 func (c *cmdStorageVolumeList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("list", i18n.G("[<remote>:]<pool> [<filter>...]"))
+	cmd.Use = usage("list", i18n.G("[<remote>:][<pool>] [<filter>...]"))
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List storage volumes")
 
-	c.defaultColumns = "etndcuL"
+	c.defaultColumns = "petndcuL"
 	cmd.Flags().StringVarP(&c.flagColumns, "columns", "c", c.defaultColumns, i18n.G("Columns")+"``")
 	cmd.Flags().BoolVar(&c.flagAllProjects, "all-projects", false, i18n.G("All projects")+"``")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -1355,6 +1411,7 @@ that control which image attributes to output when displaying in table
 or csv format.
 
 Column shorthand chars:
+    p - Storage pool name
     c - Content type (filesystem or block)
     d - Description
     e - Project name
@@ -1371,35 +1428,38 @@ Column shorthand chars:
 }
 
 func (c *cmdStorageVolumeList) Run(cmd *cobra.Command, args []string) error {
-	// Quick checks.
-	exit, err := c.global.CheckArgs(cmd, args, 1, -1)
+	// Quick checks
+	exit, err := c.global.CheckArgs(cmd, args, 0, -1)
 	if exit {
 		return err
 	}
 
-	// Parse remote
-	resources, err := c.global.ParseServers(args[0])
-	if err != nil {
-		return err
-	}
-
-	resource := resources[0]
-
-	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing pool name"))
-	}
-
 	// Process the filters
 	filters := []string{}
 	if len(args) > 1 {
 		filters = append(filters, args[1:]...)
 	}
 
+	// Parse resource
+	resource, allVolumes, err := c.getResource(args)
+	if err != nil {
+		return err
+	}
+
 	var volumes []api.StorageVolume
-	if c.flagAllProjects {
-		volumes, err = resource.server.GetStoragePoolVolumesWithFilterAllProjects(resource.name, filters)
+
+	if allVolumes {
+		if c.flagAllProjects {
+			volumes, err = resource.server.GetVolumesWithFilterAllProjects(filters)
+		} else {
+			volumes, err = resource.server.GetVolumesWithFilter(filters)
+		}
 	} else {
-		volumes, err = resource.server.GetStoragePoolVolumesWithFilter(resource.name, filters)
+		if c.flagAllProjects {
+			volumes, err = resource.server.GetStoragePoolVolumesWithFilterAllProjects(resource.name, filters)
+		} else {
+			volumes, err = resource.server.GetStoragePoolVolumesWithFilter(resource.name, filters)
+		}
 	}
 
 	if err != nil {
@@ -1407,7 +1467,7 @@ func (c *cmdStorageVolumeList) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Process the columns
-	columns, err := c.parseColumns(resource.server.IsClustered())
+	columns, err := c.parseColumns(resource.server.IsClustered(), allVolumes)
 	if err != nil {
 		return err
 	}
@@ -1418,7 +1478,7 @@ func (c *cmdStorageVolumeList) Run(cmd *cobra.Command, args []string) error {
 		row := []string{}
 		for _, column := range columns {
 			if column.NeedsState && !shared.IsSnapshot(vol.Name) && vol.Type != "image" {
-				state, err := resource.server.GetStoragePoolVolumeState(resource.name, vol.Type, vol.Name)
+				state, err := resource.server.GetStoragePoolVolumeState(vol.Pool, vol.Type, vol.Name)
 				if err != nil {
 					return err
 				}
@@ -1432,7 +1492,7 @@ func (c *cmdStorageVolumeList) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if len(columns) >= 2 {
-		sort.Sort(utils.ByNameAndType(data))
+		sort.Sort(cli.ByNameAndType(data))
 	}
 
 	rawData := make([]*api.StorageVolume, len(volumes))
@@ -1445,10 +1505,34 @@ func (c *cmdStorageVolumeList) Run(cmd *cobra.Command, args []string) error {
 		headers = append(headers, column.Name)
 	}
 
-	return utils.RenderTable(c.flagFormat, headers, data, rawData)
+	return cli.RenderTable(c.flagFormat, headers, data, rawData)
 }
 
-func (c *cmdStorageVolumeList) parseColumns(clustered bool) ([]volumeColumn, error) {
+func (c *cmdStorageVolumeList) getResource(args []string) (*remoteResource, bool, error) {
+	var resource remoteResource
+
+	if len(args) == 0 {
+		resources, err := c.global.ParseServers("")
+		if err != nil {
+			return nil, false, err
+		}
+
+		resource = resources[0]
+
+		return &resource, true, nil
+	}
+
+	resources, err := c.global.ParseServers(args[0])
+	if err != nil {
+		return nil, false, err
+	}
+
+	resource = resources[0]
+
+	return &resource, resource.name == "", nil
+}
+
+func (c *cmdStorageVolumeList) parseColumns(clustered bool, allVolumes bool) ([]volumeColumn, error) {
 	columnsShorthandMap := map[rune]volumeColumn{
 		't': {Name: i18n.G("TYPE"), Data: c.typeColumnData},
 		'n': {Name: i18n.G("NAME"), Data: c.nameColumnData},
@@ -1469,6 +1553,12 @@ func (c *cmdStorageVolumeList) parseColumns(clustered bool) ([]volumeColumn, err
 		c.flagColumns = strings.Replace(c.flagColumns, "L", "", -1)
 	}
 
+	if allVolumes {
+		columnsShorthandMap['p'] = volumeColumn{Name: i18n.G("POOL"), Data: c.poolColumnData}
+	} else {
+		c.flagColumns = strings.Replace(c.flagColumns, "p", "", -1)
+	}
+
 	if c.flagAllProjects {
 		columnsShorthandMap['e'] = volumeColumn{Name: i18n.G("PROJECT"), Data: c.projectColumnData}
 	} else {
@@ -1496,6 +1586,10 @@ func (c *cmdStorageVolumeList) parseColumns(clustered bool) ([]volumeColumn, err
 	return columns, nil
 }
 
+func (c *cmdStorageVolumeList) poolColumnData(vol api.StorageVolume, state api.StorageVolumeState) string {
+	return vol.Pool
+}
+
 func (c *cmdStorageVolumeList) typeColumnData(vol api.StorageVolume, state api.StorageVolumeState) string {
 	if shared.IsSnapshot(vol.Name) {
 		return fmt.Sprintf("%s (snapshot)", vol.Type)
@@ -1542,10 +1636,11 @@ func (c *cmdStorageVolumeList) projectColumnData(vol api.StorageVolume, state ap
 
 // Move.
 type cmdStorageVolumeMove struct {
-	global            *cmdGlobal
-	storage           *cmdStorage
-	storageVolume     *cmdStorageVolume
-	storageVolumeCopy *cmdStorageVolumeCopy
+	global              *cmdGlobal
+	storage             *cmdStorage
+	storageVolume       *cmdStorageVolume
+	storageVolumeCopy   *cmdStorageVolumeCopy
+	storageVolumeRename *cmdStorageVolumeRename
 }
 
 func (c *cmdStorageVolumeMove) Command() *cobra.Command {
@@ -1572,6 +1667,54 @@ func (c *cmdStorageVolumeMove) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	// Parse remote
+	resources, err := c.global.ParseServers(args[0], args[1])
+	if err != nil {
+		return err
+	}
+
+	// Source
+	srcResource := resources[0]
+	if srcResource.name == "" {
+		return fmt.Errorf(i18n.G("Missing source volume name"))
+	}
+
+	srcRemote := srcResource.remote
+	srcPath := srcResource.name
+
+	// Get source pool and volume name
+	srcVolName, srcVolPool := c.storageVolume.parseVolumeWithPool(srcPath)
+	if srcVolPool == "" {
+		return fmt.Errorf(i18n.G("No storage pool for source volume specified"))
+	}
+
+	// Destination
+	dstResource := resources[1]
+	dstRemote := dstResource.remote
+	dstPath := dstResource.name
+
+	// Get target pool and volume name
+	dstVolName, dstVolPool := c.storageVolume.parseVolumeWithPool(dstPath)
+	if dstVolPool == "" {
+		return fmt.Errorf(i18n.G("No storage pool for target volume specified"))
+	}
+
+	// Rename volume if both remotes and pools of source and target are equal
+	// and neither destination cluster member name nor target project are set.
+	if srcRemote == dstRemote && srcVolPool == dstVolPool && c.storageVolume.flagDestinationTarget == "" && c.storageVolumeCopy.flagTargetProject == "" {
+		var args []string
+
+		if srcRemote != "" {
+			args = append(args, fmt.Sprintf("%s:%s", srcRemote, srcVolPool))
+		} else {
+			args = append(args, srcVolPool)
+		}
+
+		args = append(args, srcVolName, dstVolName)
+
+		return c.storageVolumeRename.Run(cmd, args)
+	}
+
 	return c.storageVolumeCopy.Run(cmd, args)
 }
 
@@ -1693,19 +1836,31 @@ type cmdStorageVolumeSet struct {
 	global        *cmdGlobal
 	storage       *cmdStorage
 	storageVolume *cmdStorageVolume
+
+	flagIsProperty bool
 }
 
 func (c *cmdStorageVolumeSet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("set", i18n.G("[<remote>:]<pool> <volume> <key>=<value>..."))
+	cmd.Use = usage("set", i18n.G("[<remote>:]<pool> [<type>/]<volume> <key>=<value>..."))
 	cmd.Short = i18n.G("Set storage volume configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Set storage volume configuration keys
 
 For backward compatibility, a single configuration key may still be set with:
-    lxc storage volume set [<remote>:]<pool> <volume> <key> <value>`))
+    lxc storage volume set [<remote>:]<pool> [<type>/]<volume> <key> <value>`))
+	cmd.Example = cli.FormatSection("", i18n.G(
+		`Provide the type of the storage volume if it is not custom.
+Supported types are custom, image, container and virtual-machine.
+
+lxc storage volume set default data size=1GiB
+    Sets the size of a custom volume "data" in pool "default" to 1 GiB.
+
+lxc storage volume set default virtual-machine/data snapshots.expiry=7d
+    Sets the snapshot expiration period for a virtual machine "data" in pool "default" to seven days.`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a storage volume property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -1749,6 +1904,35 @@ func (c *cmdStorageVolumeSet) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if isSnapshot {
+		if c.flagIsProperty {
+			snapVol, etag, err := client.GetStoragePoolVolumeSnapshot(resource.name, volType, fields[0], fields[1])
+			if err != nil {
+				return err
+			}
+
+			writable := snapVol.Writable()
+			if cmd.Name() == "unset" {
+				for k := range keys {
+					err := unsetFieldByJsonTag(&writable, k)
+					if err != nil {
+						return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+					}
+				}
+			} else {
+				err := unpackKVToWritable(&writable, keys)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+				}
+			}
+
+			err = client.UpdateStoragePoolVolumeSnapshot(resource.name, volType, fields[0], fields[1], writable, etag)
+			if err != nil {
+				return err
+			}
+
+			return nil
+		}
+
 		return fmt.Errorf(i18n.G("Snapshots are read-only and can't have their configuration changed"))
 	}
 
@@ -1763,12 +1947,29 @@ func (c *cmdStorageVolumeSet) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	// Update the volume.
-	for k, v := range keys {
-		vol.Config[k] = v
+	writable := vol.Writable()
+	if c.flagIsProperty {
+		if cmd.Name() == "unset" {
+			for k := range keys {
+				err := unsetFieldByJsonTag(&writable, k)
+				if err != nil {
+					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
+				}
+			}
+		} else {
+			err := unpackKVToWritable(&writable, keys)
+			if err != nil {
+				return fmt.Errorf(i18n.G("Error setting properties: %v"), err)
+			}
+		}
+	} else {
+		// Update the volume config keys.
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
-	err = client.UpdateStoragePoolVolume(resource.name, vol.Type, vol.Name, vol.Writable(), etag)
+	err = client.UpdateStoragePoolVolume(resource.name, vol.Type, vol.Name, writable, etag)
 	if err != nil {
 		return err
 	}
@@ -1785,16 +1986,24 @@ type cmdStorageVolumeShow struct {
 
 func (c *cmdStorageVolumeShow) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("show", i18n.G("[<remote>:]<pool> <volume>[/<snapshot>]"))
+	cmd.Use = usage("show", i18n.G("[<remote>:]<pool> [<type>/]<volume>[/<snapshot>]"))
 	cmd.Short = i18n.G("Show storage volume configurations")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show storage volume configurations`))
 	cmd.Example = cli.FormatSection("", i18n.G(
-		`lxc storage volume show default data
+		`Provide the type of the storage volume if it is not custom.
+Supported types are custom, image, container and virtual-machine.
+
+Add the name of the snapshot if type is one of custom, container or virtual-machine.
+
+lxc storage volume show default data
     Will show the properties of a custom volume called "data" in the "default" pool.
 
 lxc storage volume show default container/data
-    Will show the properties of the filesystem for a container called "data" in the "default" pool.`))
+    Will show the properties of the filesystem for a container called "data" in the "default" pool.
+
+lxc storage volume show default virtual-machine/data/snap0
+    Will show the properties of snapshot "snap0" for a virtual machine called "data" in the "default" pool.`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
@@ -1880,16 +2089,28 @@ type cmdStorageVolumeUnset struct {
 	storage          *cmdStorage
 	storageVolume    *cmdStorageVolume
 	storageVolumeSet *cmdStorageVolumeSet
+
+	flagIsProperty bool
 }
 
 func (c *cmdStorageVolumeUnset) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("unset", i18n.G("[<remote>:]<pool> <volume> <key>"))
+	cmd.Use = usage("unset", i18n.G("[<remote>:]<pool> [<type>/]<volume> <key>"))
 	cmd.Short = i18n.G("Unset storage volume configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Unset storage volume configuration keys`))
+	cmd.Example = cli.FormatSection("", i18n.G(
+		`Provide the type of the storage volume if it is not custom.
+Supported types are custom, image, container and virtual-machine.
+
+lxc storage volume unset default data size
+    Remotes the size/quota of a custom volume "data" in pool "default".
+
+lxc storage volume unset default virtual-machine/data snapshots.expiry
+    Removes the snapshot expiration period for a virtual machine "data" in pool "default".`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a storage volume property"))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -1902,6 +2123,8 @@ func (c *cmdStorageVolumeUnset) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
+	c.storageVolumeSet.flagIsProperty = c.flagIsProperty
+
 	args = append(args, "")
 	return c.storageVolumeSet.Run(cmd, args)
 }
@@ -2143,7 +2366,7 @@ func (c *cmdStorageVolumeExport) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Watch the background operation
-	progress := utils.ProgressRenderer{
+	progress := cli.ProgressRenderer{
 		Format: i18n.G("Backing up storage volume: %s"),
 		Quiet:  c.global.flagQuiet,
 	}
@@ -2155,7 +2378,7 @@ func (c *cmdStorageVolumeExport) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Wait until backup is done
-	err = utils.CancelableWait(op, &progress)
+	err = cli.CancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return err
@@ -2169,8 +2392,16 @@ func (c *cmdStorageVolumeExport) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Get name of backup
-	backupName := strings.TrimPrefix(op.Get().Resources["backups"][0],
-		"/1.0/backups/")
+	uStr := op.Get().Resources["backups"][0]
+	u, err := url.Parse(uStr)
+	if err != nil {
+		return fmt.Errorf("Invalid URL %q: %w", uStr, err)
+	}
+
+	backupName, err := url.PathUnescape(path.Base(u.EscapedPath()))
+	if err != nil {
+		return fmt.Errorf("Invalid backup name segment in path %q: %w", u.EscapedPath(), err)
+	}
 
 	defer func() {
 		// Delete backup after we're done
@@ -2195,7 +2426,7 @@ func (c *cmdStorageVolumeExport) Run(cmd *cobra.Command, args []string) error {
 	defer func() { _ = target.Close() }()
 
 	// Prepare the download request
-	progress = utils.ProgressRenderer{
+	progress = cli.ProgressRenderer{
 		Format: i18n.G("Exporting the backup: %s"),
 		Quiet:  c.global.flagQuiet,
 	}
@@ -2222,6 +2453,8 @@ type cmdStorageVolumeImport struct {
 	global        *cmdGlobal
 	storage       *cmdStorage
 	storageVolume *cmdStorageVolume
+
+	flagType string
 }
 
 func (c *cmdStorageVolumeImport) Command() *cobra.Command {
@@ -2235,6 +2468,7 @@ func (c *cmdStorageVolumeImport) Command() *cobra.Command {
 		Create a new custom volume using backup0.tar.gz as the source.`))
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
+	cmd.Flags().StringVar(&c.flagType, "type", "", i18n.G("Import type, backup or iso (default \"backup\")")+"``")
 
 	return cmd
 }
@@ -2281,7 +2515,25 @@ func (c *cmdStorageVolumeImport) Run(cmd *cobra.Command, args []string) error {
 		volName = args[2]
 	}
 
-	progress := utils.ProgressRenderer{
+	if c.flagType == "" {
+		// Set type to iso if filename suffix is .iso
+		if strings.HasSuffix(file.Name(), ".iso") {
+			c.flagType = "iso"
+		} else {
+			c.flagType = "backup"
+		}
+	} else {
+		// Validate type flag
+		if !shared.ValueInSlice(c.flagType, []string{"backup", "iso"}) {
+			return fmt.Errorf("Import type needs to be \"backup\" or \"iso\"")
+		}
+	}
+
+	if c.flagType == "iso" && volName == "" {
+		return fmt.Errorf("Importing ISO images requires a volume name to be set")
+	}
+
+	progress := cli.ProgressRenderer{
 		Format: i18n.G("Importing custom volume: %s"),
 		Quiet:  c.global.flagQuiet,
 	}
@@ -2299,13 +2551,20 @@ func (c *cmdStorageVolumeImport) Run(cmd *cobra.Command, args []string) error {
 		Name: volName,
 	}
 
-	op, err := d.CreateStoragePoolVolumeFromBackup(pool, createArgs)
+	var op lxd.Operation
+
+	if c.flagType == "iso" {
+		op, err = d.CreateStoragePoolVolumeFromISO(pool, createArgs)
+	} else {
+		op, err = d.CreateStoragePoolVolumeFromBackup(pool, createArgs)
+	}
+
 	if err != nil {
 		return err
 	}
 
 	// Wait for operation to finish.
-	err = utils.CancelableWait(op, &progress)
+	err = cli.CancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return err
