@@ -8,16 +8,17 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"time"
 
 	"github.com/gorilla/mux"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/daemon"
-	"github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/daemon"
+	"github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // DevLxdServer creates an http.Server capable of handling requests against the
@@ -42,7 +43,7 @@ type devLxdHandler struct {
 
 func getVsockClient(d *Daemon) (lxd.InstanceServer, error) {
 	// Try connecting to LXD server.
-	client, err := getClient(int(d.serverCID), int(d.serverPort), d.serverCertificate)
+	client, err := getClient(d.serverCID, int(d.serverPort), d.serverCertificate)
 	if err != nil {
 		return nil, err
 	}
@@ -58,7 +59,7 @@ func getVsockClient(d *Daemon) (lxd.InstanceServer, error) {
 var devlxdConfigGet = devLxdHandler{"/1.0/config", func(d *Daemon, w http.ResponseWriter, r *http.Request) *devLxdResponse {
 	client, err := getVsockClient(d)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed connecting to LXD over vsock: %w", err))
 	}
 
 	defer client.Disconnect()
@@ -72,7 +73,7 @@ var devlxdConfigGet = devLxdHandler{"/1.0/config", func(d *Daemon, w http.Respon
 
 	err = resp.MetadataAsStruct(&config)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed parsing response from LXD: %w", err))
 	}
 
 	filtered := []string{}
@@ -96,7 +97,7 @@ var devlxdConfigKeyGet = devLxdHandler{"/1.0/config/{key}", func(d *Daemon, w ht
 
 	client, err := getVsockClient(d)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed connecting to LXD over vsock: %w", err))
 	}
 
 	defer client.Disconnect()
@@ -110,16 +111,27 @@ var devlxdConfigKeyGet = devLxdHandler{"/1.0/config/{key}", func(d *Daemon, w ht
 
 	err = resp.MetadataAsStruct(&value)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed parsing response from LXD: %w", err))
 	}
 
 	return okResponse(value, "raw")
 }}
 
 var devlxdMetadataGet = devLxdHandler{"/1.0/meta-data", func(d *Daemon, w http.ResponseWriter, r *http.Request) *devLxdResponse {
-	client, err := getVsockClient(d)
+	var client lxd.InstanceServer
+	var err error
+
+	for i := 0; i < 10; i++ {
+		client, err = getVsockClient(d)
+		if err == nil {
+			break
+		}
+
+		time.Sleep(500 * time.Millisecond)
+	}
+
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed connecting to LXD over vsock: %w", err))
 	}
 
 	defer client.Disconnect()
@@ -133,7 +145,7 @@ var devlxdMetadataGet = devLxdHandler{"/1.0/meta-data", func(d *Daemon, w http.R
 
 	err = resp.MetadataAsStruct(&metaData)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed parsing response from LXD: %w", err))
 	}
 
 	return okResponse(metaData, "raw")
@@ -151,30 +163,41 @@ var devLxdEventsGet = devLxdHandler{"/1.0/events", func(d *Daemon, w http.Respon
 var devlxdAPIGet = devLxdHandler{"/1.0", func(d *Daemon, w http.ResponseWriter, r *http.Request) *devLxdResponse {
 	client, err := getVsockClient(d)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed connecting to LXD over vsock: %w", err))
 	}
 
 	defer client.Disconnect()
 
-	resp, _, err := client.RawQuery("GET", "/1.0", nil, "")
-	if err != nil {
-		return smartResponse(err)
+	if r.Method == "GET" {
+		resp, _, err := client.RawQuery(r.Method, "/1.0", nil, "")
+		if err != nil {
+			return smartResponse(err)
+		}
+
+		var instanceData api.DevLXDGet
+
+		err = resp.MetadataAsStruct(&instanceData)
+		if err != nil {
+			return smartResponse(fmt.Errorf("Failed parsing response from LXD: %w", err))
+		}
+
+		return okResponse(instanceData, "json")
+	} else if r.Method == "PATCH" {
+		_, _, err := client.RawQuery(r.Method, "/1.0", r.Body, "")
+		if err != nil {
+			return smartResponse(err)
+		}
+
+		return okResponse("", "raw")
 	}
 
-	var instanceData api.DevLXDGet
-
-	err = resp.MetadataAsStruct(&instanceData)
-	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
-	}
-
-	return okResponse(instanceData, "json")
+	return &devLxdResponse{fmt.Sprintf("method %q not allowed", r.Method), http.StatusBadRequest, "raw"}
 }}
 
 var devlxdDevicesGet = devLxdHandler{"/1.0/devices", func(d *Daemon, w http.ResponseWriter, r *http.Request) *devLxdResponse {
 	client, err := getVsockClient(d)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed connecting to LXD over vsock: %w", err))
 	}
 
 	defer client.Disconnect()
@@ -188,7 +211,7 @@ var devlxdDevicesGet = devLxdHandler{"/1.0/devices", func(d *Daemon, w http.Resp
 
 	err = resp.MetadataAsStruct(&devices)
 	if err != nil {
-		return &devLxdResponse{"internal server error", http.StatusInternalServerError, "raw"}
+		return smartResponse(fmt.Errorf("Failed parsing response from LXD: %w", err))
 	}
 
 	return okResponse(devices, "json")
