@@ -1,14 +1,18 @@
 package main
 
 import (
+	"bufio"
 	"os"
 
 	"github.com/spf13/cobra"
 
-	"github.com/lxc/lxd/shared/version"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/version"
 )
 
 type cmdGlobal struct {
+	asker cli.Asker
+
 	flagVersion bool
 	flagHelp    bool
 }
@@ -16,7 +20,7 @@ type cmdGlobal struct {
 func main() {
 	// migrate command (main)
 	migrateCmd := cmdMigrate{}
-	app := migrateCmd.Command()
+	app := migrateCmd.command()
 	app.SilenceUsage = true
 	app.CompletionOptions = cobra.CompletionOptions{DisableDefaultCmd: true}
 
@@ -24,7 +28,7 @@ func main() {
 	app.Args = cobra.ArbitraryArgs
 
 	// Global flags
-	globalCmd := cmdGlobal{}
+	globalCmd := cmdGlobal{asker: cli.NewAsker(bufio.NewReader(os.Stdin))}
 	migrateCmd.global = &globalCmd
 	app.PersistentFlags().BoolVar(&globalCmd.flagVersion, "version", false, "Print version number")
 	app.PersistentFlags().BoolVarP(&globalCmd.flagHelp, "help", "h", false, "Print help")
