@@ -17,15 +17,14 @@ import (
 	"golang.org/x/sys/unix"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/version"
 )
 
 type cmdMigrate struct {
@@ -34,7 +33,7 @@ type cmdMigrate struct {
 	flagRsyncArgs string
 }
 
-func (c *cmdMigrate) Command() *cobra.Command {
+func (c *cmdMigrate) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = "lxd-migrate"
 	cmd.Short = "Physical to instance migration tool"
@@ -50,7 +49,7 @@ func (c *cmdMigrate) Command() *cobra.Command {
 
   The same set of options as ` + "`lxc launch`" + ` are also supported.
 `
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 	cmd.Flags().StringVar(&c.flagRsyncArgs, "rsync-args", "", "Extra arguments to pass to rsync"+"``")
 
 	return cmd
@@ -63,7 +62,7 @@ type cmdMigrateData struct {
 	Project      string
 }
 
-func (c *cmdMigrateData) Render() string {
+func (c *cmdMigrateData) render() string {
 	data := struct {
 		Name        string            `yaml:"Name"`
 		Project     string            `yaml:"Project"`
@@ -113,7 +112,7 @@ func (c *cmdMigrateData) Render() string {
 
 func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 	// Server address
-	serverURL, err := cli.AskString("Please provide LXD server URL: ", "", nil)
+	serverURL, err := c.global.asker.AskString("Please provide LXD server URL: ", "", nil)
 	if err != nil {
 		return nil, "", err
 	}
@@ -135,8 +134,8 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 
 	digest := shared.CertFingerprint(certificate)
 
-	fmt.Printf("Certificate fingerprint: %s\n", digest)
-	fmt.Printf("ok (y/n)? ")
+	fmt.Println("Certificate fingerprint:", digest)
+	fmt.Print("ok (y/n)? ")
 	line, err := shared.ReadStdin()
 	if err != nil {
 		return nil, "", err
@@ -161,8 +160,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 	type AuthMethod int
 
 	const (
-		authMethodCandid AuthMethod = iota
-		authMethodTLSCertificate
+		authMethodTLSCertificate AuthMethod = iota
 		authMethodTLSTemporaryCertificate
 		authMethodTLSCertificateToken
 	)
@@ -173,13 +171,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 
 	i := 1
 
-	if shared.StringInSlice("candid", apiServer.AuthMethods) {
-		fmt.Printf("%d) Candid/RBAC based authentication\n", i)
-		availableAuthMethods = append(availableAuthMethods, authMethodCandid)
-		i++
-	}
-
-	if shared.StringInSlice("tls", apiServer.AuthMethods) {
+	if shared.ValueInSlice(api.AuthenticationMethodTLS, apiServer.AuthMethods) {
 		fmt.Printf("%d) Use a certificate token\n", i)
 		availableAuthMethods = append(availableAuthMethods, authMethodTLSCertificateToken)
 		i++
@@ -190,8 +182,8 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 		availableAuthMethods = append(availableAuthMethods, authMethodTLSTemporaryCertificate)
 	}
 
-	if len(apiServer.AuthMethods) > 1 || shared.StringInSlice("tls", apiServer.AuthMethods) {
-		authMethodInt, err := cli.AskInt("Please pick an authentication mechanism above: ", 1, int64(i), "", nil)
+	if len(apiServer.AuthMethods) > 1 || shared.ValueInSlice(api.AuthenticationMethodTLS, apiServer.AuthMethods) {
+		authMethodInt, err := c.global.asker.AskInt("Please pick an authentication mechanism above: ", 1, int64(i), "", nil)
 		if err != nil {
 			return nil, "", err
 		}
@@ -204,7 +196,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 	var token string
 
 	if authMethod == authMethodTLSCertificate {
-		certPath, err = cli.AskString("Please provide the certificate path: ", "", func(path string) error {
+		certPath, err = c.global.asker.AskString("Please provide the certificate path: ", "", func(path string) error {
 			if !shared.PathExists(path) {
 				return errors.New("File does not exist")
 			}
@@ -215,7 +207,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 			return nil, "", err
 		}
 
-		keyPath, err = cli.AskString("Please provide the keyfile path: ", "", func(path string) error {
+		keyPath, err = c.global.asker.AskString("Please provide the keyfile path: ", "", func(path string) error {
 			if !shared.PathExists(path) {
 				return errors.New("File does not exist")
 			}
@@ -226,7 +218,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 			return nil, "", err
 		}
 	} else if authMethod == authMethodTLSCertificateToken {
-		token, err = cli.AskString("Please provide the certificate token: ", "", func(token string) error {
+		token, err = c.global.asker.AskString("Please provide the certificate token: ", "", func(token string) error {
 			_, err := shared.CertificateTokenDecode(token)
 			if err != nil {
 				return err
@@ -242,16 +234,14 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 	var authType string
 
 	switch authMethod {
-	case authMethodCandid:
-		authType = "candid"
 	case authMethodTLSCertificate, authMethodTLSTemporaryCertificate, authMethodTLSCertificateToken:
-		authType = "tls"
+		authType = api.AuthenticationMethodTLS
 	}
 
-	return connectTarget(serverURL, certPath, keyPath, authType, token)
+	return c.connectTarget(serverURL, certPath, keyPath, authType, token)
 }
 
-func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData, error) {
+func (c *cmdMigrate) runInteractive(server lxd.InstanceServer) (cmdMigrateData, error) {
 	var err error
 
 	config := cmdMigrateData{}
@@ -267,7 +257,7 @@ func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData,
 	config.InstanceArgs.Devices = map[string]map[string]string{}
 
 	// Provide instance type
-	instanceType, err := cli.AskInt("Would you like to create a container (1) or virtual-machine (2)?: ", 1, 2, "1", nil)
+	instanceType, err := c.global.asker.AskInt("Would you like to create a container (1) or virtual-machine (2)?: ", 1, 2, "1", nil)
 	if err != nil {
 		return cmdMigrateData{}, err
 	}
@@ -285,7 +275,7 @@ func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData,
 	}
 
 	if len(projectNames) > 1 {
-		project, err := cli.AskChoice("Project to create the instance in [default=default]: ", projectNames, "default")
+		project, err := c.global.asker.AskChoice("Project to create the instance in [default=default]: ", projectNames, "default")
 		if err != nil {
 			return cmdMigrateData{}, err
 		}
@@ -302,12 +292,12 @@ func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData,
 	}
 
 	for {
-		instanceName, err := cli.AskString("Name of the new instance: ", "", nil)
+		instanceName, err := c.global.asker.AskString("Name of the new instance: ", "", nil)
 		if err != nil {
 			return cmdMigrateData{}, err
 		}
 
-		if shared.StringInSlice(instanceName, instanceNames) {
+		if shared.ValueInSlice(instanceName, instanceNames) {
 			fmt.Printf("Instance %q already exists\n", instanceName)
 			continue
 		}
@@ -325,7 +315,7 @@ func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData,
 		question = "Please provide the path to a root filesystem: "
 	}
 
-	config.SourcePath, err = cli.AskString(question, "", func(s string) error {
+	config.SourcePath, err = c.global.asker.AskString(question, "", func(s string) error {
 		if !shared.PathExists(s) {
 			return errors.New("Path does not exist")
 		}
@@ -344,14 +334,14 @@ func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData,
 	if config.InstanceArgs.Type == api.InstanceTypeVM {
 		architectureName, _ := osarch.ArchitectureGetLocal()
 
-		if shared.StringInSlice(architectureName, []string{"x86_64", "aarch64"}) {
-			hasSecureBoot, err := cli.AskBool("Does the VM support UEFI Secure Boot? [default=no]: ", "no")
+		if shared.ValueInSlice(architectureName, []string{"x86_64", "aarch64"}) {
+			hasSecureBoot, err := c.global.asker.AskBool("Does the VM support UEFI Secure Boot? [default=no]: ", "no")
 			if err != nil {
 				return cmdMigrateData{}, err
 			}
 
-			if hasSecureBoot {
-				config.InstanceArgs.Config["security.secureboot"] = "true"
+			if !hasSecureBoot {
+				config.InstanceArgs.Config["security.secureboot"] = "false"
 			}
 		}
 	}
@@ -360,14 +350,14 @@ func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData,
 
 	// Additional mounts for containers
 	if config.InstanceArgs.Type == api.InstanceTypeContainer {
-		addMounts, err := cli.AskBool("Do you want to add additional filesystem mounts? [default=no]: ", "no")
+		addMounts, err := c.global.asker.AskBool("Do you want to add additional filesystem mounts? [default=no]: ", "no")
 		if err != nil {
 			return cmdMigrateData{}, err
 		}
 
 		if addMounts {
 			for {
-				path, err := cli.AskString("Please provide a path the filesystem mount path [empty value to continue]: ", "", func(s string) error {
+				path, err := c.global.asker.AskString("Please provide a path the filesystem mount path [empty value to continue]: ", "", func(s string) error {
 					if s != "" {
 						if shared.PathExists(s) {
 							return nil
@@ -394,14 +384,14 @@ func (c *cmdMigrate) RunInteractive(server lxd.InstanceServer) (cmdMigrateData,
 	}
 
 	for {
-		fmt.Printf("\nInstance to be created:\n")
+		fmt.Println("\nInstance to be created:")
 
-		scanner := bufio.NewScanner(strings.NewReader(config.Render()))
+		scanner := bufio.NewScanner(strings.NewReader(config.render()))
 		for scanner.Scan() {
 			fmt.Printf("  %s\n", scanner.Text())
 		}
 
-		fmt.Printf(`
+		fmt.Print(`
 Additional overrides can be applied at this stage:
 1) Begin the migration with the above configuration
 2) Override profile list
@@ -411,7 +401,7 @@ Additional overrides can be applied at this stage:
 
 `)
 
-		choice, err := cli.AskInt("Please pick one of the options above [default=1]: ", 1, 5, "1", nil)
+		choice, err := c.global.asker.AskInt("Please pick one of the options above [default=1]: ", 1, 5, "1", nil)
 		if err != nil {
 			return cmdMigrateData{}, err
 		}
@@ -435,7 +425,7 @@ Additional overrides can be applied at this stage:
 	}
 }
 
-func (c *cmdMigrate) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdMigrate) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	if os.Geteuid() != 0 {
 		return fmt.Errorf("This tool must be run as root")
@@ -464,14 +454,18 @@ func (c *cmdMigrate) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		cancel()
-		os.Exit(1)
+
+		// The following nolint directive ignores the "deep-exit" rule of the revive linter.
+		// We should be exiting cleanly by passing the above context into each invoked method and checking for
+		// cancellation. Unfortunately our client methods do not accept a context argument.
+		os.Exit(1) //nolint:revive
 	}()
 
 	if clientFingerprint != "" {
 		defer func() { _ = server.DeleteCertificate(clientFingerprint) }()
 	}
 
-	config, err := c.RunInteractive(server)
+	config, err := c.runInteractive(server)
 	if err != nil {
 		return err
 	}
@@ -572,7 +566,7 @@ func (c *cmdMigrate) Run(cmd *cobra.Command, args []string) error {
 		_, _ = server.DeleteInstance(config.InstanceArgs.Name)
 	})
 
-	progress := utils.ProgressRenderer{Format: "Transferring instance: %s"}
+	progress := cli.ProgressRenderer{Format: "Transferring instance: %s"}
 	_, err = op.AddHandler(progress.UpdateOp)
 	if err != nil {
 		progress.Done("")
@@ -596,7 +590,7 @@ func (c *cmdMigrate) askProfiles(server lxd.InstanceServer, config *cmdMigrateDa
 		return err
 	}
 
-	profiles, err := cli.AskString("Which profiles do you want to apply to the instance? (space separated) [default=default, \"-\" for none]: ", "default", func(s string) error {
+	profiles, err := c.global.asker.AskString("Which profiles do you want to apply to the instance? (space separated) [default=default, \"-\" for none]: ", "default", func(s string) error {
 		// This indicates that no profiles should be applied.
 		if s == "-" {
 			return nil
@@ -605,7 +599,7 @@ func (c *cmdMigrate) askProfiles(server lxd.InstanceServer, config *cmdMigrateDa
 		profiles := strings.Split(s, " ")
 
 		for _, profile := range profiles {
-			if !shared.StringInSlice(profile, profileNames) {
+			if !shared.ValueInSlice(profile, profileNames) {
 				return fmt.Errorf("Unknown profile %q", profile)
 			}
 		}
@@ -624,7 +618,7 @@ func (c *cmdMigrate) askProfiles(server lxd.InstanceServer, config *cmdMigrateDa
 }
 
 func (c *cmdMigrate) askConfig(config *cmdMigrateData) error {
-	configs, err := cli.AskString("Please specify config keys and values (key=value ...): ", "", func(s string) error {
+	configs, err := c.global.asker.AskString("Please specify config keys and values (key=value ...): ", "", func(s string) error {
 		if s == "" {
 			return nil
 		}
@@ -642,8 +636,8 @@ func (c *cmdMigrate) askConfig(config *cmdMigrateData) error {
 	}
 
 	for _, entry := range strings.Split(configs, " ") {
-		fields := strings.SplitN(entry, "=", 2)
-		config.InstanceArgs.Config[fields[0]] = fields[1]
+		key, value, _ := strings.Cut(entry, "=")
+		config.InstanceArgs.Config[key] = value
 	}
 
 	return nil
@@ -659,7 +653,7 @@ func (c *cmdMigrate) askStorage(server lxd.InstanceServer, config *cmdMigrateDat
 		return fmt.Errorf("No storage pools available")
 	}
 
-	storagePool, err := cli.AskChoice("Please provide the storage pool to use: ", storagePools, "")
+	storagePool, err := c.global.asker.AskChoice("Please provide the storage pool to use: ", storagePools, "")
 	if err != nil {
 		return err
 	}
@@ -670,13 +664,13 @@ func (c *cmdMigrate) askStorage(server lxd.InstanceServer, config *cmdMigrateDat
 		"path": "/",
 	}
 
-	changeStorageSize, err := cli.AskBool("Do you want to change the storage size? [default=no]: ", "no")
+	changeStorageSize, err := c.global.asker.AskBool("Do you want to change the storage size? [default=no]: ", "no")
 	if err != nil {
 		return err
 	}
 
 	if changeStorageSize {
-		size, err := cli.AskString("Please specify the storage size: ", "", func(s string) error {
+		size, err := c.global.asker.AskString("Please specify the storage size: ", "", func(s string) error {
 			_, err := units.ParseByteSizeString(s)
 			return err
 		})
@@ -696,7 +690,7 @@ func (c *cmdMigrate) askNetwork(server lxd.InstanceServer, config *cmdMigrateDat
 		return err
 	}
 
-	network, err := cli.AskChoice("Please specify the network to use for the instance: ", networks, "")
+	network, err := c.global.asker.AskChoice("Please specify the network to use for the instance: ", networks, "")
 	if err != nil {
 		return err
 	}
