@@ -10,37 +10,39 @@ import (
 	"net/url"
 	"os"
 	"path/filepath"
+	"reflect"
 	"strings"
 
-	"github.com/go-macaroon-bakery/macaroon-bakery/v3/httpbakery"
-	"github.com/go-macaroon-bakery/macaroon-bakery/v3/httpbakery/form"
-	cookiejar "github.com/juju/persistent-cookiejar"
 	"golang.org/x/sys/unix"
 	"golang.org/x/term"
-	schemaform "gopkg.in/juju/environschema.v1/form"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/migration"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/migration"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/version"
+	"github.com/canonical/lxd/shared/ws"
 )
 
 func transferRootfs(ctx context.Context, dst lxd.InstanceServer, op lxd.Operation, rootfs string, rsyncArgs string, instanceType api.InstanceType) error {
 	opAPI := op.Get()
 
 	// Connect to the websockets
-	wsControl, err := op.GetWebsocket(opAPI.Metadata["control"].(string))
+	wsControl, err := op.GetWebsocket(opAPI.Metadata[api.SecretNameControl].(string))
 	if err != nil {
 		return err
 	}
 
-	wsFs, err := op.GetWebsocket(opAPI.Metadata["fs"].(string))
-	if err != nil {
+	abort := func(err error) error {
+		protoSendError(wsControl, err)
 		return err
 	}
 
+	wsFs, err := op.GetWebsocket(opAPI.Metadata[api.SecretNameFilesystem].(string))
+	if err != nil {
+		return abort(err)
+	}
+
 	// Setup control struct
 	var fs migration.MigrationFSType
 	var rsyncHasFeature bool
@@ -53,7 +55,7 @@ func transferRootfs(ctx context.Context, dst lxd.InstanceServer, op lxd.Operatio
 		rsyncHasFeature = true
 	}
 
-	header := migration.MigrationHeader{
+	offerHeader := migration.MigrationHeader{
 		RsyncFeatures: &migration.RsyncFeatures{
 			Xattrs:   &rsyncHasFeature,
 			Delete:   &rsyncHasFeature,
@@ -65,47 +67,48 @@ func transferRootfs(ctx context.Context, dst lxd.InstanceServer, op lxd.Operatio
 	if instanceType == api.InstanceTypeVM {
 		stat, err := os.Stat(filepath.Join(rootfs, "root.img"))
 		if err != nil {
-			return err
+			return abort(err)
 		}
 
 		size := stat.Size()
-		header.VolumeSize = &size
+		offerHeader.VolumeSize = &size
 		rootfs = shared.AddSlash(rootfs)
 	}
 
-	err = migration.ProtoSend(wsControl, &header)
+	err = migration.ProtoSend(wsControl, &offerHeader)
 	if err != nil {
-		protoSendError(wsControl, err)
-		return err
+		return abort(err)
 	}
 
-	err = migration.ProtoRecv(wsControl, &header)
+	var respHeader migration.MigrationHeader
+	err = migration.ProtoRecv(wsControl, &respHeader)
 	if err != nil {
-		protoSendError(wsControl, err)
-		return err
+		return abort(err)
+	}
+
+	rsyncFeaturesOffered := offerHeader.GetRsyncFeaturesSlice()
+	rsyncFeaturesResponse := respHeader.GetRsyncFeaturesSlice()
+
+	if !reflect.DeepEqual(rsyncFeaturesOffered, rsyncFeaturesResponse) {
+		return abort(fmt.Errorf("Offered rsync features (%v) differ from those in the migration response (%v)", rsyncFeaturesOffered, rsyncFeaturesResponse))
 	}
 
 	// Send the filesystem
-	abort := func(err error) error {
-		protoSendError(wsControl, err)
-		return err
-	}
-
 	err = rsyncSend(ctx, wsFs, rootfs, rsyncArgs, instanceType)
 	if err != nil {
-		return abort(err)
+		return abort(fmt.Errorf("Failed sending filesystem volume: %w", err))
 	}
 
 	// Send block volume
 	if instanceType == api.InstanceTypeVM {
 		f, err := os.Open(filepath.Join(rootfs, "root.img"))
 		if err != nil {
-			return err
+			return abort(err)
 		}
 
 		defer func() { _ = f.Close() }()
 
-		conn := &shared.WebsocketIO{Conn: wsFs}
+		conn := ws.NewWrapper(wsFs)
 
 		go func() {
 			<-ctx.Done()
@@ -115,12 +118,12 @@ func transferRootfs(ctx context.Context, dst lxd.InstanceServer, op lxd.Operatio
 
 		_, err = io.Copy(conn, f)
 		if err != nil {
-			return err
+			return abort(fmt.Errorf("Failed sending block volume: %w", err))
 		}
 
 		err = conn.Close()
 		if err != nil {
-			return err
+			return abort(err)
 		}
 	}
 
@@ -132,21 +135,21 @@ func transferRootfs(ctx context.Context, dst lxd.InstanceServer, op lxd.Operatio
 		return err
 	}
 
-	if !*msg.Success {
-		return fmt.Errorf(*msg.Message)
+	if !msg.GetSuccess() {
+		return fmt.Errorf(msg.GetMessage())
 	}
 
 	return nil
 }
 
-func connectTarget(url string, certPath string, keyPath string, authType string, token string) (lxd.InstanceServer, string, error) {
+func (c *cmdMigrate) connectTarget(url string, certPath string, keyPath string, authType string, token string) (lxd.InstanceServer, string, error) {
 	args := lxd.ConnectionArgs{
 		AuthType: authType,
 	}
 
 	clientFingerprint := ""
 
-	if authType == "tls" {
+	if authType == api.AuthenticationMethodTLS {
 		var clientCrt []byte
 		var clientKey []byte
 
@@ -184,35 +187,11 @@ func connectTarget(url string, certPath string, keyPath string, authType string,
 
 		args.TLSClientCert = string(clientCrt)
 		args.TLSClientKey = string(clientKey)
-	} else if authType == "candid" {
-		args.AuthInteractor = []httpbakery.Interactor{
-			form.Interactor{Filler: schemaform.IOFiller{}},
-			httpbakery.WebBrowserInteractor{
-				OpenWebBrowser: httpbakery.OpenWebBrowser,
-			},
-		}
-
-		f, err := os.CreateTemp("", "lxd-migrate_")
-		if err != nil {
-			return nil, "", err
-		}
-
-		_ = f.Close()
-
-		jar, err := cookiejar.New(
-			&cookiejar.Options{
-				Filename: f.Name(),
-			})
-		if err != nil {
-			return nil, "", err
-		}
-
-		args.CookieJar = jar
 	}
 
 	// Attempt to connect using the system CA
 	args.UserAgent = fmt.Sprintf("LXC-MIGRATE %s", version.Version)
-	c, err := lxd.ConnectLXD(url, &args)
+	instanceServer, err := lxd.ConnectLXD(url, &args)
 
 	var certificate *x509.Certificate
 	if err != nil {
@@ -229,18 +208,14 @@ func connectTarget(url string, certPath string, keyPath string, authType string,
 		args.TLSServerCert = string(serverCrt)
 
 		// Setup a new connection, this time with the remote certificate
-		c, err = lxd.ConnectLXD(url, &args)
+		instanceServer, err = lxd.ConnectLXD(url, &args)
 		if err != nil {
 			return nil, "", err
 		}
 	}
 
-	if authType == "candid" {
-		c.RequireAuthenticated(false)
-	}
-
 	// Get server information
-	srv, _, err := c.GetServer()
+	srv, _, err := instanceServer.GetServer()
 	if err != nil {
 		return nil, "", err
 	}
@@ -248,16 +223,16 @@ func connectTarget(url string, certPath string, keyPath string, authType string,
 	// Check if our cert is already trusted
 	if srv.Auth == "trusted" {
 		fmt.Printf("\nRemote LXD server:\n  Hostname: %s\n  Version: %s\n\n", srv.Environment.ServerName, srv.Environment.ServerVersion)
-		return c, "", nil
+		return instanceServer, "", nil
 	}
 
-	if authType == "tls" {
+	if authType == api.AuthenticationMethodTLS {
 		if token != "" {
 			req := api.CertificatesPost{
 				Password: token,
 			}
 
-			err = c.CreateCertificate(req)
+			err = instanceServer.CreateCertificate(req)
 			if err != nil {
 				return nil, "", fmt.Errorf("Failed to create certificate: %w", err)
 			}
@@ -266,7 +241,7 @@ func connectTarget(url string, certPath string, keyPath string, authType string,
 
 			fmt.Println("")
 
-			useTrustPassword, err := cli.AskBool("Would you like to use a trust password? [default=no]: ", "no")
+			useTrustPassword, err := c.global.asker.AskBool("Would you like to use a trust password? [default=no]: ", "no")
 			if err != nil {
 				return nil, "", err
 			}
@@ -288,7 +263,7 @@ func connectTarget(url string, certPath string, keyPath string, authType string,
 
 				req.Type = api.CertificateTypeClient
 
-				err = c.CreateCertificate(req)
+				err = instanceServer.CreateCertificate(req)
 				if err != nil {
 					return nil, "", err
 				}
@@ -301,14 +276,14 @@ func connectTarget(url string, certPath string, keyPath string, authType string,
 			}
 		}
 	} else {
-		c.RequireAuthenticated(true)
+		instanceServer.RequireAuthenticated(true)
 	}
 
 	// Get full server information
-	srv, _, err = c.GetServer()
+	srv, _, err = instanceServer.GetServer()
 	if err != nil {
 		if clientFingerprint != "" {
-			_ = c.DeleteCertificate(clientFingerprint)
+			_ = instanceServer.DeleteCertificate(clientFingerprint)
 		}
 
 		return nil, "", err
@@ -320,7 +295,7 @@ func connectTarget(url string, certPath string, keyPath string, authType string,
 
 	fmt.Printf("\nRemote LXD server:\n  Hostname: %s\n  Version: %s\n\n", srv.Environment.ServerName, srv.Environment.ServerVersion)
 
-	return c, clientFingerprint, nil
+	return instanceServer, clientFingerprint, nil
 }
 
 func setupSource(path string, mounts []string) error {
