@@ -7,32 +7,34 @@ import (
 	"net"
 	"net/http"
 	"os"
-	"strings"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/cluster"
-	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
-	"github.com/lxc/lxd/lxd/config"
-	"github.com/lxc/lxd/lxd/db"
-	instanceDrivers "github.com/lxc/lxd/lxd/instance/drivers"
-	"github.com/lxc/lxd/lxd/lifecycle"
-	"github.com/lxc/lxd/lxd/node"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/rbac"
-	"github.com/lxc/lxd/lxd/request"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/auth"
+	"github.com/canonical/lxd/lxd/auth/oidc"
+	"github.com/canonical/lxd/lxd/cluster"
+	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
+	"github.com/canonical/lxd/lxd/config"
+	"github.com/canonical/lxd/lxd/db"
+	instanceDrivers "github.com/canonical/lxd/lxd/instance/drivers"
+	"github.com/canonical/lxd/lxd/lifecycle"
+	"github.com/canonical/lxd/lxd/node"
+	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/lxd/response"
+	scriptletLoad "github.com/canonical/lxd/lxd/scriptlet/load"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/version"
 )
 
 var api10Cmd = APIEndpoint{
 	Get:   APIEndpointAction{Handler: api10Get, AllowUntrusted: true},
-	Patch: APIEndpointAction{Handler: api10Patch},
-	Put:   APIEndpointAction{Handler: api10Put},
+	Patch: APIEndpointAction{Handler: api10Patch, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
+	Put:   APIEndpointAction{Handler: api10Put, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var api10 = []APIEndpoint{
@@ -54,15 +56,19 @@ var api10 = []APIEndpoint{
 	instanceConsoleCmd,
 	instanceExecCmd,
 	instanceFileCmd,
+	instanceExecOutputCmd,
+	instanceExecOutputsCmd,
 	instanceLogCmd,
 	instanceLogsCmd,
 	instanceMetadataCmd,
 	instanceMetadataTemplatesCmd,
 	instancesCmd,
+	instanceRebuildCmd,
 	instanceSFTPCmd,
 	instanceSnapshotCmd,
 	instanceSnapshotsCmd,
 	instanceStateCmd,
+	instanceUEFIVarsCmd,
 	eventsCmd,
 	imageAliasCmd,
 	imageAliasesCmd,
@@ -71,6 +77,7 @@ var api10 = []APIEndpoint{
 	imageRefreshCmd,
 	imagesCmd,
 	imageSecretCmd,
+	metadataConfigurationCmd,
 	networkCmd,
 	networkLeasesCmd,
 	networksCmd,
@@ -78,8 +85,11 @@ var api10 = []APIEndpoint{
 	networkACLCmd,
 	networkACLsCmd,
 	networkACLLogCmd,
+	networkAllocationsCmd,
 	networkForwardCmd,
 	networkForwardsCmd,
+	networkLoadBalancerCmd,
+	networkLoadBalancersCmd,
 	networkPeerCmd,
 	networkPeersCmd,
 	networkZoneCmd,
@@ -98,6 +108,10 @@ var api10 = []APIEndpoint{
 	storagePoolCmd,
 	storagePoolResourcesCmd,
 	storagePoolsCmd,
+	storagePoolBucketsCmd,
+	storagePoolBucketCmd,
+	storagePoolBucketKeysCmd,
+	storagePoolBucketKeyCmd,
 	storagePoolVolumesCmd,
 	storagePoolVolumeSnapshotsTypeCmd,
 	storagePoolVolumeSnapshotTypeCmd,
@@ -110,97 +124,107 @@ var api10 = []APIEndpoint{
 	warningsCmd,
 	warningCmd,
 	metricsCmd,
+	identitiesCmd,
+	identitiesByAuthenticationMethodCmd,
+	identityCmd,
+	authGroupsCmd,
+	authGroupCmd,
+	identityProviderGroupsCmd,
+	identityProviderGroupCmd,
+	permissionsCmd,
+	storageVolumesCmd,
+	storageVolumesTypeCmd,
 }
 
 // swagger:operation GET /1.0?public server server_get_untrusted
 //
-// Get the server environment
+//  Get the server environment
 //
-// Shows a small subset of the server environment and configuration
-// which is required by untrusted clients to reach a server.
+//  Shows a small subset of the server environment and configuration
+//  which is required by untrusted clients to reach a server.
 //
-// The `?public` part of the URL isn't required, it's simply used to
-// separate the two behaviors of this endpoint.
+//  The `?public` part of the URL isn't required, it's simply used to
+//  separate the two behaviors of this endpoint.
 //
-// ---
-// produces:
-//   - application/json
-// responses:
-//   "200":
-//     description: Server environment and configuration
-//     schema:
-//       type: object
-//       description: Sync response
-//       properties:
-//         type:
-//           type: string
-//           description: Response type
-//           example: sync
-//         status:
-//           type: string
-//           description: Status description
-//           example: Success
-//         status_code:
-//           type: integer
-//           description: Status code
-//           example: 200
-//         metadata:
-//           $ref: "#/definitions/ServerUntrusted"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//  ---
+//  produces:
+//    - application/json
+//  responses:
+//    "200":
+//      description: Server environment and configuration
+//      schema:
+//        type: object
+//        description: Sync response
+//        properties:
+//          type:
+//            type: string
+//            description: Response type
+//            example: sync
+//          status:
+//            type: string
+//            description: Status description
+//            example: Success
+//          status_code:
+//            type: integer
+//            description: Status code
+//            example: 200
+//          metadata:
+//            $ref: "#/definitions/ServerUntrusted"
+//    "500":
+//      $ref: "#/responses/InternalServerError"
 
 // swagger:operation GET /1.0 server server_get
 //
-// Get the server environment and configuration
+//	Get the server environment and configuration
 //
-// Shows the full server environment and configuration.
+//	Shows the full server environment and configuration.
 //
-// ---
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: target
-//     description: Cluster member name
-//     type: string
-//     example: lxd01
-//   - in: query
-//     name: project
-//     description: Project name
-//     type: string
-//     example: default
-// responses:
-//   "200":
-//     description: Server environment and configuration
-//     schema:
-//       type: object
-//       description: Sync response
-//       properties:
-//         type:
-//           type: string
-//           description: Response type
-//           example: sync
-//         status:
-//           type: string
-//           description: Status description
-//           example: Success
-//         status_code:
-//           type: integer
-//           description: Status code
-//           example: 200
-//         metadata:
-//           $ref: "#/definitions/Server"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: target
+//	    description: Cluster member name
+//	    type: string
+//	    example: lxd01
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	responses:
+//	  "200":
+//	    description: Server environment and configuration
+//	    schema:
+//	      type: object
+//	      description: Sync response
+//	      properties:
+//	        type:
+//	          type: string
+//	          description: Response type
+//	          example: sync
+//	        status:
+//	          type: string
+//	          description: Status description
+//	          example: Success
+//	        status_code:
+//	          type: integer
+//	          description: Status code
+//	          example: 200
+//	        metadata:
+//	          $ref: "#/definitions/Server"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func api10Get(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// Get the authentication methods.
-	authMethods := []string{"tls"}
-	candidURL, _, _, _ := s.GlobalConfig.CandidServer()
-	rbacURL, _, _, _, _, _, _ := s.GlobalConfig.RBACServer()
-	if candidURL != "" || rbacURL != "" {
-		authMethods = append(authMethods, "candid")
+	authMethods := []string{api.AuthenticationMethodTLS}
+
+	oidcIssuer, oidcClientID, _, _ := s.GlobalConfig.OIDCServer()
+	if oidcIssuer != "" && oidcClientID != "" {
+		authMethods = append(authMethods, api.AuthenticationMethodOIDC)
 	}
 
 	srv := api.ServerUntrusted{
@@ -212,13 +236,16 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 		AuthMethods:   authMethods,
 	}
 
-	// If untrusted, return now
-	if d.checkTrustedClient(r) != nil {
+	// If not authorized, return now. Untrusted users are not authorized.
+	err := s.Authorizer.CheckPermission(r.Context(), r, entity.ServerURL(), auth.EntitlementCanView)
+	if err != nil && auth.IsDeniedError(err) {
 		return response.SyncResponseETag(true, srv, nil)
+	} else if err != nil {
+		return response.SmartError(err)
 	}
 
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(d, r)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -232,14 +259,9 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 		return response.InternalError(err)
 	}
 
-	clustered, err := cluster.Enabled(d.db.Node)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	// When clustered, use the node name, otherwise use the hostname.
 	var serverName string
-	if clustered {
+	if s.ServerClustered {
 		serverName = s.ServerName
 	} else {
 		hostname, err := os.Hostname()
@@ -250,7 +272,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 		serverName = hostname
 	}
 
-	certificate := string(d.endpoints.NetworkPublicKey())
+	certificate := string(s.Endpoints.NetworkPublicKey())
 	var certificateFingerprint string
 	if certificate != "" {
 		certificateFingerprint, err = shared.CertFingerprintStr(certificate)
@@ -261,7 +283,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 	architectures := []string{}
 
-	for _, architecture := range d.os.Architectures {
+	for _, architecture := range s.OS.Architectures {
 		architectureName, err := osarch.ArchitectureName(architecture)
 		if err != nil {
 			return response.InternalError(err)
@@ -272,7 +294,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 	projectName := r.FormValue("project")
 	if projectName == "" {
-		projectName = project.Default
+		projectName = api.ProjectDefaultName
 	}
 
 	env := api.ServerEnvironment{
@@ -280,29 +302,29 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 		Architectures:          architectures,
 		Certificate:            certificate,
 		CertificateFingerprint: certificateFingerprint,
-		Kernel:                 d.os.Uname.Sysname,
-		KernelArchitecture:     d.os.Uname.Machine,
-		KernelVersion:          d.os.Uname.Release,
-		OSName:                 d.os.ReleaseInfo["NAME"],
-		OSVersion:              d.os.ReleaseInfo["VERSION_ID"],
+		Kernel:                 s.OS.Uname.Sysname,
+		KernelArchitecture:     s.OS.Uname.Machine,
+		KernelVersion:          s.OS.Uname.Release,
+		OSName:                 s.OS.ReleaseInfo["NAME"],
+		OSVersion:              s.OS.ReleaseInfo["VERSION_ID"],
 		Project:                projectName,
 		Server:                 "lxd",
 		ServerPid:              os.Getpid(),
 		ServerVersion:          version.Version,
-		ServerClustered:        clustered,
+		ServerLTS:              version.IsLTSVersion,
+		ServerClustered:        s.ServerClustered,
 		ServerEventMode:        string(cluster.ServerEventMode()),
 		ServerName:             serverName,
-		Firewall:               d.firewall.String(),
+		Firewall:               s.Firewall.String(),
 	}
 
 	env.KernelFeatures = map[string]string{
-		"netnsid_getifaddrs":        fmt.Sprintf("%v", d.os.NetnsGetifaddrs),
-		"uevent_injection":          fmt.Sprintf("%v", d.os.UeventInjection),
-		"unpriv_fscaps":             fmt.Sprintf("%v", d.os.VFS3Fscaps),
-		"seccomp_listener":          fmt.Sprintf("%v", d.os.SeccompListener),
-		"seccomp_listener_continue": fmt.Sprintf("%v", d.os.SeccompListenerContinue),
-		"shiftfs":                   fmt.Sprintf("%v", d.os.Shiftfs),
-		"idmapped_mounts":           fmt.Sprintf("%v", d.os.IdmappedMounts),
+		"netnsid_getifaddrs":        fmt.Sprintf("%v", s.OS.NetnsGetifaddrs),
+		"uevent_injection":          fmt.Sprintf("%v", s.OS.UeventInjection),
+		"unpriv_fscaps":             fmt.Sprintf("%v", s.OS.VFS3Fscaps),
+		"seccomp_listener":          fmt.Sprintf("%v", s.OS.SeccompListener),
+		"seccomp_listener_continue": fmt.Sprintf("%v", s.OS.SeccompListenerContinue),
+		"idmapped_mounts":           fmt.Sprintf("%v", s.OS.IdmappedMounts),
 	}
 
 	drivers := instanceDrivers.DriverStatuses()
@@ -324,11 +346,17 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 		} else {
 			env.DriverVersion = driver.Info.Version
 		}
+
+		// Add supported instance types.
+		instType := driver.Info.Type.String()
+		if !shared.ValueInSlice(instType, env.InstanceTypes) {
+			env.InstanceTypes = append(env.InstanceTypes, instType)
+		}
 	}
 
-	if d.os.LXCFeatures != nil {
+	if s.OS.LXCFeatures != nil {
 		env.LXCFeatures = map[string]string{}
-		for k, v := range d.os.LXCFeatures {
+		for k, v := range s.OS.LXCFeatures {
 			env.LXCFeatures[k] = fmt.Sprintf("%v", v)
 		}
 	}
@@ -353,9 +381,16 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 	fullSrv := api.Server{ServerUntrusted: srv}
 	fullSrv.Environment = env
+	requestor := request.CreateRequestor(r)
+	fullSrv.AuthUserName = requestor.Username
+	fullSrv.AuthUserMethod = requestor.Protocol
 
-	if rbac.UserIsAdmin(r) {
-		fullSrv.Config, err = daemonConfigRender(d.State())
+	// Only allow identities that can edit configuration to view it as sensitive information may be stored there.
+	err = s.Authorizer.CheckPermission(r.Context(), r, entity.ServerURL(), auth.EntitlementCanEdit)
+	if err != nil && !auth.IsDeniedError(err) {
+		return response.SmartError(err)
+	} else if err == nil {
+		fullSrv.Config, err = daemonConfigRender(s)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -366,41 +401,43 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 // swagger:operation PUT /1.0 server server_put
 //
-// Update the server configuration
+//	Update the server configuration
 //
-// Updates the entire server configuration.
+//	Updates the entire server configuration.
 //
-// ---
-// consumes:
-//   - application/json
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: target
-//     description: Cluster member name
-//     type: string
-//     example: lxd01
-//   - in: body
-//     name: server
-//     description: Server configuration
-//     required: true
-//     schema:
-//       $ref: "#/definitions/ServerPut"
-// responses:
-//   "200":
-//     $ref: "#/responses/EmptySyncResponse"
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "412":
-//     $ref: "#/responses/PreconditionFailed"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	consumes:
+//	  - application/json
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: target
+//	    description: Cluster member name
+//	    type: string
+//	    example: lxd01
+//	  - in: body
+//	    name: server
+//	    description: Server configuration
+//	    required: true
+//	    schema:
+//	      $ref: "#/definitions/ServerPut"
+//	responses:
+//	  "200":
+//	    $ref: "#/responses/EmptySyncResponse"
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "412":
+//	    $ref: "#/responses/PreconditionFailed"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func api10Put(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(d, r)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -420,12 +457,12 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 		logger.Debug("Handling config changed notification")
 		changed := make(map[string]string)
 		for key, value := range req.Config {
-			changed[key] = value.(string)
+			changed[key], _ = value.(string)
 		}
 
 		// Get the current (updated) config.
 		var config *clusterConfig.Config
-		err := d.db.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err := s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
 			var err error
 			config, err = clusterConfig.Load(ctx, tx)
 			return err
@@ -440,7 +477,7 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 		d.globalConfigMu.Unlock()
 
 		// Run any update triggers.
-		err = doApi10UpdateTriggers(d, nil, changed, nil, config)
+		err = doAPI10UpdateTriggers(d, nil, changed, s.LocalConfig, config)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -448,7 +485,7 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 		return response.EmptySyncResponse
 	}
 
-	render, err := daemonConfigRender(d.State())
+	render, err := daemonConfigRender(s)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -458,46 +495,48 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 		return response.PreconditionFailed(err)
 	}
 
-	return doApi10Update(d, r, req, false)
+	return doAPI10Update(d, r, req, false)
 }
 
 // swagger:operation PATCH /1.0 server server_patch
 //
-// Partially update the server configuration
+//	Partially update the server configuration
 //
-// Updates a subset of the server configuration.
+//	Updates a subset of the server configuration.
 //
-// ---
-// consumes:
-//   - application/json
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: target
-//     description: Cluster member name
-//     type: string
-//     example: lxd01
-//   - in: body
-//     name: server
-//     description: Server configuration
-//     required: true
-//     schema:
-//       $ref: "#/definitions/ServerPut"
-// responses:
-//   "200":
-//     $ref: "#/responses/EmptySyncResponse"
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "412":
-//     $ref: "#/responses/PreconditionFailed"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	consumes:
+//	  - application/json
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: target
+//	    description: Cluster member name
+//	    type: string
+//	    example: lxd01
+//	  - in: body
+//	    name: server
+//	    description: Server configuration
+//	    required: true
+//	    schema:
+//	      $ref: "#/definitions/ServerPut"
+//	responses:
+//	  "200":
+//	    $ref: "#/responses/EmptySyncResponse"
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "412":
+//	    $ref: "#/responses/PreconditionFailed"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func api10Patch(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	// If a target was specified, forward the request to the relevant node.
-	resp := forwardedResponseIfTargetIsRemote(d, r)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -505,7 +544,7 @@ func api10Patch(d *Daemon, r *http.Request) response.Response {
 	// Don't apply changes to settings until daemon is fully started.
 	<-d.waitReady.Done()
 
-	render, err := daemonConfigRender(d.State())
+	render, err := daemonConfigRender(s)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -525,10 +564,10 @@ func api10Patch(d *Daemon, r *http.Request) response.Response {
 		return response.EmptySyncResponse
 	}
 
-	return doApi10Update(d, r, req, true)
+	return doAPI10Update(d, r, req, true)
 }
 
-func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) response.Response {
+func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) response.Response {
 	s := d.State()
 
 	// First deal with config specific to the local daemon
@@ -542,35 +581,38 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		}
 	}
 
-	clustered, err := cluster.Enabled(d.db.Node)
-	if err != nil {
-		return response.InternalError(fmt.Errorf("Failed to check for cluster state: %w", err))
-	}
-
 	nodeChanged := map[string]string{}
 	var newNodeConfig *node.Config
-	err = d.db.Node.Transaction(r.Context(), func(ctx context.Context, tx *db.NodeTx) error {
+	oldNodeConfig := make(map[string]any)
+
+	err := s.DB.Node.Transaction(r.Context(), func(ctx context.Context, tx *db.NodeTx) error {
 		var err error
 		newNodeConfig, err = node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return fmt.Errorf("Failed to load node config: %w", err)
 		}
 
+		// Keep old config around in case something goes wrong. In that case the config will be reverted.
+		for k, v := range newNodeConfig.Dump() {
+			oldNodeConfig[k] = v
+		}
+
 		// We currently don't allow changing the cluster.https_address once it's set.
-		if clustered {
+		if s.ServerClustered {
 			curConfig, err := tx.Config(ctx)
 			if err != nil {
 				return fmt.Errorf("Cannot fetch node config from database: %w", err)
 			}
 
-			newClusterHTTPSAddress, found := nodeValues["cluster.https_address"]
-			if !found && patch {
+			newClusterHTTPSAddress := ""
+			newClusterHTTPSAddressAny, found := nodeValues["cluster.https_address"]
+			if found {
+				newClusterHTTPSAddress, _ = newClusterHTTPSAddressAny.(string)
+			} else if patch {
 				newClusterHTTPSAddress = curConfig["cluster.https_address"]
-			} else if !found {
-				newClusterHTTPSAddress = ""
 			}
 
-			if curConfig["cluster.https_address"] != newClusterHTTPSAddress.(string) {
+			if curConfig["cluster.https_address"] != newClusterHTTPSAddress {
 				return fmt.Errorf("Changing cluster.https_address is currently not supported")
 			}
 		}
@@ -607,35 +649,55 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		}
 	}
 
-	// Validate global configuration
-	hasRBAC := false
-	hasCandid := false
-	for k, v := range req.Config {
-		if v == "" {
-			continue
+	revert := revert.New()
+	defer revert.Fail()
+
+	revert.Add(func() {
+		for key := range nodeValues {
+			val, ok := oldNodeConfig[key]
+			if !ok {
+				nodeValues[key] = nil
+			} else {
+				nodeValues[key] = val
+			}
 		}
 
-		if strings.HasPrefix(k, "candid.") {
-			hasCandid = true
-		} else if strings.HasPrefix(k, "rbac.") {
-			hasRBAC = true
-		}
+		err = s.DB.Node.Transaction(r.Context(), func(ctx context.Context, tx *db.NodeTx) error {
+			newNodeConfig, err := node.ConfigLoad(ctx, tx)
+			if err != nil {
+				return fmt.Errorf("Failed to load node config: %w", err)
+			}
 
-		if hasCandid && hasRBAC {
-			return response.BadRequest(fmt.Errorf("RBAC and Candid are mutually exclusive"))
+			_, err = newNodeConfig.Replace(nodeValues)
+			if err != nil {
+				return fmt.Errorf("Failed updating node config: %w", err)
+			}
+
+			return nil
+		})
+
+		if err != nil {
+			logger.Warn("Failed reverting node config", logger.Ctx{"err": err})
 		}
-	}
+	})
 
 	// Then deal with cluster wide configuration
 	var clusterChanged map[string]string
 	var newClusterConfig *clusterConfig.Config
-	err = d.db.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
+	oldClusterConfig := make(map[string]any)
+
+	err = s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 		newClusterConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return fmt.Errorf("Failed to load cluster config: %w", err)
 		}
 
+		// Keep old config around in case something goes wrong. In that case the config will be reverted.
+		for k, v := range newClusterConfig.Dump() {
+			oldClusterConfig[k] = v
+		}
+
 		if patch {
 			clusterChanged, err = newClusterConfig.Patch(req.Config)
 		} else {
@@ -653,8 +715,37 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		}
 	}
 
+	revert.Add(func() {
+		for key := range req.Config {
+			val, ok := oldClusterConfig[key]
+			if !ok {
+				req.Config[key] = nil
+			} else {
+				req.Config[key] = val
+			}
+		}
+
+		err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+			newClusterConfig, err = clusterConfig.Load(ctx, tx)
+			if err != nil {
+				return fmt.Errorf("Failed to load cluster config: %w", err)
+			}
+
+			_, err = newClusterConfig.Replace(req.Config)
+			if err != nil {
+				return fmt.Errorf("Failed updating cluster config: %w", err)
+			}
+
+			return nil
+		})
+
+		if err != nil {
+			logger.Warn("Failed reverting cluster config", logger.Ctx{"err": err})
+		}
+	})
+
 	// Notify the other nodes about changes
-	notifier, err := cluster.NewNotifier(s, d.endpoints.NetworkCert(), d.serverCert(), cluster.NotifyAlive)
+	notifier, err := cluster.NewNotifier(s, s.Endpoints.NetworkCert(), s.ServerCert(), cluster.NotifyAlive)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -686,29 +777,34 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 	d.globalConfigMu.Unlock()
 
 	// Run any update triggers.
-	err = doApi10UpdateTriggers(d, nodeChanged, clusterChanged, newNodeConfig, newClusterConfig)
+	err = doAPI10UpdateTriggers(d, nodeChanged, clusterChanged, newNodeConfig, newClusterConfig)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	d.State().Events.SendLifecycle(project.Default, lifecycle.ConfigUpdated.Event(request.CreateRequestor(r), nil))
+	revert.Success()
+
+	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.ConfigUpdated.Event(request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
 
-func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]string, nodeConfig *node.Config, clusterConfig *clusterConfig.Config) error {
+func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]string, nodeConfig *node.Config, clusterConfig *clusterConfig.Config) error {
 	s := d.State()
 
 	maasChanged := false
-	candidChanged := false
-	rbacChanged := false
 	bgpChanged := false
 	dnsChanged := false
+	lokiChanged := false
+	acmeDomainChanged := false
+	acmeCAURLChanged := false
+	oidcChanged := false
+	syslogSocketChanged := false
 
 	for key := range clusterChanged {
 		switch key {
 		case "core.https_trusted_proxy":
-			d.endpoints.NetworkUpdateTrustedProxy(clusterChanged[key])
+			s.Endpoints.NetworkUpdateTrustedProxy(clusterChanged[key])
 		case "core.proxy_http":
 			fallthrough
 		case "core.proxy_https":
@@ -719,16 +815,8 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			fallthrough
 		case "maas.api.key":
 			maasChanged = true
-		case "candid.domains":
-			fallthrough
-		case "candid.expiry":
-			fallthrough
-		case "candid.api.key":
-			fallthrough
-		case "candid.api.url":
-			candidChanged = true
 		case "cluster.images_minimal_replica":
-			err := autoSyncImages(d.shutdownCtx, d)
+			err := autoSyncImages(s.ShutdownCtx, s)
 			if err != nil {
 				logger.Warn("Could not auto-sync images", logger.Ctx{"err": err})
 			}
@@ -739,26 +827,34 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 		case "images.auto_update_interval":
 			fallthrough
 		case "images.remote_cache_expiry":
-			if !d.os.MockMode {
+			if !s.OS.MockMode {
 				d.taskPruneImages.Reset()
 			}
 
-		case "rbac.agent.url":
-			fallthrough
-		case "rbac.agent.username":
-			fallthrough
-		case "rbac.agent.private_key":
-			fallthrough
-		case "rbac.agent.public_key":
-			fallthrough
-		case "rbac.api.url":
-			fallthrough
-		case "rbac.api.key":
-			fallthrough
-		case "rbac.expiry":
-			rbacChanged = true
 		case "core.bgp_asn":
 			bgpChanged = true
+		case "loki.api.url":
+			fallthrough
+		case "loki.auth.username":
+			fallthrough
+		case "loki.auth.password":
+			fallthrough
+		case "loki.api.ca_cert":
+			fallthrough
+		case "loki.instance":
+			fallthrough
+		case "loki.labels":
+			fallthrough
+		case "loki.loglevel":
+			fallthrough
+		case "loki.types":
+			lokiChanged = true
+		case "acme.ca_url":
+			acmeCAURLChanged = true
+		case "acme.domain":
+			acmeDomainChanged = true
+		case "oidc.issuer", "oidc.client.id", "oidc.audience", "oidc.groups.claim":
+			oidcChanged = true
 		}
 	}
 
@@ -772,6 +868,8 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			bgpChanged = true
 		case "core.dns_address":
 			dnsChanged = true
+		case "core.syslog_socket":
+			syslogSocketChanged = true
 		}
 	}
 
@@ -782,27 +880,27 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 
 	value, ok := nodeChanged["core.https_address"]
 	if ok {
-		err := d.endpoints.NetworkUpdateAddress(value)
+		err := s.Endpoints.NetworkUpdateAddress(value)
 		if err != nil {
 			return err
 		}
 
-		d.endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
+		s.Endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
 	}
 
 	value, ok = nodeChanged["cluster.https_address"]
 	if ok {
-		err := d.endpoints.ClusterUpdateAddress(value)
+		err := s.Endpoints.ClusterUpdateAddress(value)
 		if err != nil {
 			return err
 		}
 
-		d.endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
+		s.Endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
 	}
 
 	value, ok = nodeChanged["core.debug_address"]
 	if ok {
-		err := d.endpoints.PprofUpdateAddress(value)
+		err := s.Endpoints.PprofUpdateAddress(value)
 		if err != nil {
 			return err
 		}
@@ -810,7 +908,15 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 
 	value, ok = nodeChanged["core.metrics_address"]
 	if ok {
-		err := d.endpoints.MetricsUpdateAddress(value, d.endpoints.NetworkCert())
+		err := s.Endpoints.MetricsUpdateAddress(value, s.Endpoints.NetworkCert())
+		if err != nil {
+			return err
+		}
+	}
+
+	value, ok = nodeChanged["core.storage_buckets_address"]
+	if ok {
+		err := s.Endpoints.StorageBucketsUpdateAddress(value, s.Endpoints.NetworkCert())
 		if err != nil {
 			return err
 		}
@@ -841,34 +947,6 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 		}
 	}
 
-	if candidChanged {
-		apiURL, apiKey, expiry, domains := clusterConfig.CandidServer()
-		err := d.setupExternalAuthentication(apiURL, apiKey, expiry, domains)
-		if err != nil {
-			return err
-		}
-	}
-
-	if rbacChanged {
-		apiURL, apiKey, apiExpiry, agentURL, agentUsername, agentPrivateKey, agentPublicKey := clusterConfig.RBACServer()
-
-		// Since RBAC seems to have been set up already, we need to disable it temporarily
-		if d.rbac != nil {
-			err := d.setupExternalAuthentication("", "", 0, "")
-			if err != nil {
-				return err
-			}
-
-			d.rbac.StopStatusCheck()
-			d.rbac = nil
-		}
-
-		err := d.setupRBACServer(apiURL, apiKey, apiExpiry, agentURL, agentUsername, agentPrivateKey, agentPublicKey)
-		if err != nil {
-			return err
-		}
-	}
-
 	if bgpChanged {
 		address := nodeConfig.BGPAddress()
 		asn := clusterConfig.BGPASN()
@@ -876,7 +954,7 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 
 		err := s.BGP.Reconfigure(address, uint32(asn), net.ParseIP(routerid))
 		if err != nil {
-			return err
+			return fmt.Errorf("Failed reconfiguring BGP: %w", err)
 		}
 	}
 
@@ -884,6 +962,61 @@ func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 		address := nodeConfig.DNSAddress()
 
 		err := s.DNS.Reconfigure(address)
+		if err != nil {
+			return fmt.Errorf("Failed reconfiguring DNS: %w", err)
+		}
+	}
+
+	if lokiChanged {
+		lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes := clusterConfig.LokiServer()
+
+		if lokiURL == "" || lokiLoglevel == "" || len(lokiTypes) == 0 {
+			d.internalListener.RemoveHandler("loki")
+		} else {
+			err := d.setupLoki(lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes)
+			if err != nil {
+				return err
+			}
+		}
+	}
+
+	if acmeCAURLChanged || acmeDomainChanged {
+		err := autoRenewCertificate(s.ShutdownCtx, d, acmeCAURLChanged)
+		if err != nil {
+			return err
+		}
+	}
+
+	// Compile and load the instance placement scriptlet.
+	value, ok = clusterChanged["instances.placement.scriptlet"]
+	if ok {
+		err := scriptletLoad.InstancePlacementSet(value)
+		if err != nil {
+			return fmt.Errorf("Failed saving instance placement scriptlet: %w", err)
+		}
+	}
+
+	if oidcChanged {
+		oidcIssuer, oidcClientID, oidcAudience, oidcGroupsClaim := clusterConfig.OIDCServer()
+
+		if oidcIssuer == "" || oidcClientID == "" {
+			d.oidcVerifier = nil
+		} else {
+			var err error
+
+			httpClientFunc := func() (*http.Client, error) {
+				return util.HTTPClient("", d.proxy)
+			}
+
+			d.oidcVerifier, err = oidc.NewVerifier(oidcIssuer, oidcClientID, oidcAudience, s.ServerCert, d.identityCache, httpClientFunc, &oidc.Opts{GroupsClaim: oidcGroupsClaim})
+			if err != nil {
+				return fmt.Errorf("Failed creating verifier: %w", err)
+			}
+		}
+	}
+
+	if syslogSocketChanged {
+		err := d.setupSyslogSocket(nodeConfig.SyslogSocket())
 		if err != nil {
 			return err
 		}
