@@ -5,7 +5,6 @@ import (
 	"context"
 	"database/sql"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -20,25 +19,28 @@ import (
 	"github.com/gorilla/mux"
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/lxd/backup"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/lxd/db/warningtype"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	storagePools "github.com/lxc/lxd/lxd/storage"
-	storageDrivers "github.com/lxc/lxd/lxd/storage/drivers"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/lxd/auth"
+	"github.com/canonical/lxd/lxd/backup"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/lxd/db/warningtype"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	storagePools "github.com/canonical/lxd/lxd/storage"
+	storageDrivers "github.com/canonical/lxd/lxd/storage/drivers"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
 )
 
 var apiInternal = []APIEndpoint{
@@ -46,9 +48,9 @@ var apiInternal = []APIEndpoint{
 	internalClusterAcceptCmd,
 	internalClusterAssignCmd,
 	internalClusterHandoverCmd,
-	internalClusterInstanceMovedCmd,
 	internalClusterRaftNodeCmd,
 	internalClusterRebalanceCmd,
+	internalClusterHealCmd,
 	internalContainerOnStartCmd,
 	internalContainerOnStopCmd,
 	internalContainerOnStopNSCmd,
@@ -60,93 +62,100 @@ var apiInternal = []APIEndpoint{
 	internalShutdownCmd,
 	internalSQLCmd,
 	internalWarningCreateCmd,
+	internalIdentityCacheRefreshCmd,
 }
 
 var internalShutdownCmd = APIEndpoint{
 	Path: "shutdown",
 
-	Put: APIEndpointAction{Handler: internalShutdown},
+	Put: APIEndpointAction{Handler: internalShutdown, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalReadyCmd = APIEndpoint{
 	Path: "ready",
 
-	Get: APIEndpointAction{Handler: internalWaitReady},
+	Get: APIEndpointAction{Handler: internalWaitReady, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalContainerOnStartCmd = APIEndpoint{
 	Path: "containers/{instanceRef}/onstart",
 
-	Get: APIEndpointAction{Handler: internalContainerOnStart},
+	Get: APIEndpointAction{Handler: internalContainerOnStart, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalContainerOnStopNSCmd = APIEndpoint{
 	Path: "containers/{instanceRef}/onstopns",
 
-	Get: APIEndpointAction{Handler: internalContainerOnStopNS},
+	Get: APIEndpointAction{Handler: internalContainerOnStopNS, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalContainerOnStopCmd = APIEndpoint{
 	Path: "containers/{instanceRef}/onstop",
 
-	Get: APIEndpointAction{Handler: internalContainerOnStop},
+	Get: APIEndpointAction{Handler: internalContainerOnStop, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalSQLCmd = APIEndpoint{
 	Path: "sql",
 
-	Get:  APIEndpointAction{Handler: internalSQLGet},
-	Post: APIEndpointAction{Handler: internalSQLPost},
+	Get:  APIEndpointAction{Handler: internalSQLGet, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
+	Post: APIEndpointAction{Handler: internalSQLPost, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalGarbageCollectorCmd = APIEndpoint{
 	Path: "gc",
 
-	Get: APIEndpointAction{Handler: internalGC},
+	Get: APIEndpointAction{Handler: internalGC, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalRAFTSnapshotCmd = APIEndpoint{
 	Path: "raft-snapshot",
 
-	Get: APIEndpointAction{Handler: internalRAFTSnapshot},
+	Get: APIEndpointAction{Handler: internalRAFTSnapshot, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalImageRefreshCmd = APIEndpoint{
 	Path: "testing/image-refresh",
 
-	Get: APIEndpointAction{Handler: internalRefreshImage},
+	Get: APIEndpointAction{Handler: internalRefreshImage, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalImageOptimizeCmd = APIEndpoint{
 	Path: "image-optimize",
 
-	Post: APIEndpointAction{Handler: internalOptimizeImage},
+	Post: APIEndpointAction{Handler: internalOptimizeImage, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalWarningCreateCmd = APIEndpoint{
 	Path: "testing/warnings",
 
-	Post: APIEndpointAction{Handler: internalCreateWarning},
+	Post: APIEndpointAction{Handler: internalCreateWarning, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalBGPStateCmd = APIEndpoint{
 	Path: "testing/bgp",
 
-	Get: APIEndpointAction{Handler: internalBGPState},
+	Get: APIEndpointAction{Handler: internalBGPState, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalIdentityCacheRefreshCmd = APIEndpoint{
+	Path: "identity-cache-refresh",
+
+	Post: APIEndpointAction{Handler: internalIdentityCacheRefresh, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 type internalImageOptimizePost struct {
 	Image api.Image `json:"image" yaml:"image"`
-	Pool  string    `json:"pool" yaml:"pool"`
+	Pool  string    `json:"pool"  yaml:"pool"`
 }
 
 type internalWarningCreatePost struct {
-	Location       string `json:"location" yaml:"location"`
-	Project        string `json:"project" yaml:"project"`
-	EntityTypeCode int    `json:"entity_type_code" yaml:"entity_type_code"`
-	EntityID       int    `json:"entity_id" yaml:"entity_id"`
-	TypeCode       int    `json:"type_code" yaml:"type_code"`
-	Message        string `json:"message" yaml:"message"`
+	Location   string      `json:"location"    yaml:"location"`
+	Project    string      `json:"project"     yaml:"project"`
+	EntityType entity.Type `json:"entity_type" yaml:"entity_type"`
+	EntityID   int         `json:"entity_id"   yaml:"entity_id"`
+	TypeCode   int         `json:"type_code"   yaml:"type_code"`
+	Message    string      `json:"message"     yaml:"message"`
 }
 
 // internalCreateWarning creates a warning, and is used for testing only.
@@ -171,16 +180,21 @@ func internalCreateWarning(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	req.EntityTypeCode, _ = reqRaw.GetInt("entity_type_code")
+	entityTypeStr, _ := reqRaw.GetString("entity_type")
 	req.EntityID, _ = reqRaw.GetInt("entity_id")
 
-	// Check if the entity exists, and fail if it doesn't.
-	_, ok := cluster.EntityNames[req.EntityTypeCode]
-	if req.EntityTypeCode != -1 && !ok {
-		return response.SmartError(fmt.Errorf("Invalid entity type"))
+	// If entity type is set, check it is valid and fail if it isn't.
+	if entityTypeStr != "" {
+		req.EntityType = entity.Type(entityTypeStr)
+		err = req.EntityType.Validate()
+		if err != nil {
+			return response.BadRequest(fmt.Errorf("Invalid entity type: %w", err))
+		}
 	}
 
-	err = d.db.Cluster.UpsertWarning(req.Location, req.Project, req.EntityTypeCode, req.EntityID, warningtype.Type(req.TypeCode), req.Message)
+	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.UpsertWarning(ctx, req.Location, req.Project, req.EntityType, req.EntityID, warningtype.Type(req.TypeCode), req.Message)
+	})
 	if err != nil {
 		return response.SmartError(fmt.Errorf("Failed to create warning: %w", err))
 	}
@@ -189,6 +203,8 @@ func internalCreateWarning(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalOptimizeImage(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	req := &internalImageOptimizePost{}
 
 	// Parse the request.
@@ -197,7 +213,7 @@ func internalOptimizeImage(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	err = imageCreateInPool(d, &req.Image, req.Pool)
+	err = imageCreateInPool(s, &req.Image, req.Pool)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -206,7 +222,9 @@ func internalOptimizeImage(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalRefreshImage(d *Daemon, r *http.Request) response.Response {
-	err := autoUpdateImages(d.shutdownCtx, d)
+	s := d.State()
+
+	err := autoUpdateImages(s.ShutdownCtx, s)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -216,7 +234,7 @@ func internalRefreshImage(d *Daemon, r *http.Request) response.Response {
 
 func internalWaitReady(d *Daemon, r *http.Request) response.Response {
 	// Check that we're not shutting down.
-	isClosing := d.shutdownCtx.Err() != nil
+	isClosing := d.State().ShutdownCtx.Err() != nil
 	if isClosing {
 		return response.Unavailable(fmt.Errorf("LXD daemon is shutting down"))
 	}
@@ -229,11 +247,11 @@ func internalWaitReady(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalShutdown(d *Daemon, r *http.Request) response.Response {
-	force := queryParam(r, "force")
+	force := request.QueryParam(r, "force")
 	logger.Info("Asked to shutdown by API", logger.Ctx{"force": force})
 
-	if d.shutdownCtx.Err() != nil {
-		return response.SmartError(fmt.Errorf("Shutdown already in progress"))
+	if d.State().ShutdownCtx.Err() != nil {
+		return response.SmartError(api.StatusErrorf(http.StatusTooManyRequests, "Shutdown already in progress"))
 	}
 
 	forceCtx, forceCtxCancel := context.WithCancel(context.Background())
@@ -256,12 +274,12 @@ func internalShutdown(d *Daemon, r *http.Request) response.Response {
 
 		// Send the response before the LXD daemon process ends.
 		f, ok := w.(http.Flusher)
-		if ok {
-			f.Flush()
-		} else {
+		if !ok {
 			return fmt.Errorf("http.ResponseWriter is not type http.Flusher")
 		}
 
+		f.Flush()
+
 		// Send result of d.Stop() to cmdDaemon so that process stops with correct exit code from Stop().
 		go func() {
 			<-r.Context().Done() // Wait until request is finished.
@@ -281,7 +299,7 @@ func internalContainerHookLoadFromReference(s *state.State, r *http.Request) (in
 		return nil, err
 	}
 
-	projectName := projectParam(r)
+	projectName := request.ProjectParam(r)
 
 	instanceID, err := strconv.Atoi(instanceRef)
 	if err == nil {
@@ -315,7 +333,9 @@ func internalContainerHookLoadFromReference(s *state.State, r *http.Request) (in
 }
 
 func internalContainerOnStart(d *Daemon, r *http.Request) response.Response {
-	inst, err := internalContainerHookLoadFromReference(d.State(), r)
+	s := d.State()
+
+	inst, err := internalContainerHookLoadFromReference(s, r)
 	if err != nil {
 		logger.Error("The start hook failed to load", logger.Ctx{"err": err})
 		return response.SmartError(err)
@@ -331,18 +351,20 @@ func internalContainerOnStart(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalContainerOnStopNS(d *Daemon, r *http.Request) response.Response {
-	inst, err := internalContainerHookLoadFromReference(d.State(), r)
+	s := d.State()
+
+	inst, err := internalContainerHookLoadFromReference(s, r)
 	if err != nil {
 		logger.Error("The stopns hook failed to load", logger.Ctx{"err": err})
 		return response.SmartError(err)
 	}
 
-	target := queryParam(r, "target")
+	target := request.QueryParam(r, "target")
 	if target == "" {
 		target = "unknown"
 	}
 
-	netns := queryParam(r, "netns")
+	netns := request.QueryParam(r, "netns")
 
 	args := map[string]string{
 		"target": target,
@@ -359,13 +381,15 @@ func internalContainerOnStopNS(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalContainerOnStop(d *Daemon, r *http.Request) response.Response {
-	inst, err := internalContainerHookLoadFromReference(d.State(), r)
+	s := d.State()
+
+	inst, err := internalContainerHookLoadFromReference(s, r)
 	if err != nil {
 		logger.Error("The stop hook failed to load", logger.Ctx{"err": err})
 		return response.SmartError(err)
 	}
 
-	target := queryParam(r, "target")
+	target := request.QueryParam(r, "target")
 	if target == "" {
 		target = "unknown"
 	}
@@ -389,7 +413,7 @@ type internalSQLDump struct {
 
 type internalSQLQuery struct {
 	Database string `json:"database" yaml:"database"`
-	Query    string `json:"query" yaml:"query"`
+	Query    string `json:"query"    yaml:"query"`
 }
 
 type internalSQLBatch struct {
@@ -397,17 +421,19 @@ type internalSQLBatch struct {
 }
 
 type internalSQLResult struct {
-	Type         string   `json:"type" yaml:"type"`
-	Columns      []string `json:"columns" yaml:"columns"`
-	Rows         [][]any  `json:"rows" yaml:"rows"`
+	Type         string   `json:"type"          yaml:"type"`
+	Columns      []string `json:"columns"       yaml:"columns"`
+	Rows         [][]any  `json:"rows"          yaml:"rows"`
 	RowsAffected int64    `json:"rows_affected" yaml:"rows_affected"`
 }
 
 // Perform a database dump.
 func internalSQLGet(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	database := r.FormValue("database")
 
-	if !shared.StringInSlice(database, []string{"local", "global"}) {
+	if !shared.ValueInSlice(database, []string{"local", "global"}) {
 		return response.BadRequest(fmt.Errorf("Invalid database"))
 	}
 
@@ -419,9 +445,9 @@ func internalSQLGet(d *Daemon, r *http.Request) response.Response {
 
 	var db *sql.DB
 	if database == "global" {
-		db = d.db.Cluster.DB()
+		db = s.DB.Cluster.DB()
 	} else {
-		db = d.db.Node.DB()
+		db = s.DB.Node.DB()
 	}
 
 	tx, err := db.BeginTx(r.Context(), nil)
@@ -441,6 +467,8 @@ func internalSQLGet(d *Daemon, r *http.Request) response.Response {
 
 // Execute queries.
 func internalSQLPost(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	req := &internalSQLQuery{}
 	// Parse the request.
 	err := json.NewDecoder(r.Body).Decode(&req)
@@ -448,7 +476,7 @@ func internalSQLPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	if !shared.StringInSlice(req.Database, []string{"local", "global"}) {
+	if !shared.ValueInSlice(req.Database, []string{"local", "global"}) {
 		return response.BadRequest(fmt.Errorf("Invalid database"))
 	}
 
@@ -458,9 +486,9 @@ func internalSQLPost(d *Daemon, r *http.Request) response.Response {
 
 	var db *sql.DB
 	if req.Database == "global" {
-		db = d.db.Cluster.DB()
+		db = s.DB.Cluster.DB()
 	} else {
-		db = d.db.Node.DB()
+		db = s.DB.Node.DB()
 	}
 
 	batch := internalSQLBatch{}
@@ -569,7 +597,8 @@ func internalSQLExec(tx *sql.Tx, query string, result *internalSQLResult) error
 
 // internalImportFromBackup creates instance, storage pool and volume DB records from an instance's backup file.
 // It expects the instance volume to be mounted so that the backup.yaml file is readable.
-func internalImportFromBackup(d *Daemon, projectName string, instName string, force bool, allowNameOverride bool) error {
+// Also accepts an optional map of device overrides.
+func internalImportFromBackup(s *state.State, projectName string, instName string, allowNameOverride bool, deviceOverrides map[string]map[string]string) error {
 	if instName == "" {
 		return fmt.Errorf("The name of the instance is required")
 	}
@@ -594,7 +623,7 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 	instancePoolName := ""
 	instanceType := instancetype.Container
 	instanceVolType := storageDrivers.VolumeTypeContainer
-	instanceDBVolType := db.StoragePoolVolumeTypeContainer
+	instanceDBVolType := cluster.StoragePoolVolumeTypeContainer
 
 	for _, volType := range []storageDrivers.VolumeType{storageDrivers.VolumeTypeVM, storageDrivers.VolumeTypeContainer} {
 		for _, poolName := range storagePoolNames {
@@ -608,10 +637,10 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 
 				if volType == storageDrivers.VolumeTypeVM {
 					instanceType = instancetype.VM
-					instanceDBVolType = db.StoragePoolVolumeTypeVM
+					instanceDBVolType = cluster.StoragePoolVolumeTypeVM
 				} else {
 					instanceType = instancetype.Container
-					instanceDBVolType = db.StoragePoolVolumeTypeContainer
+					instanceDBVolType = cluster.StoragePoolVolumeTypeContainer
 				}
 			}
 		}
@@ -652,19 +681,19 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 
 	if backupConf.Pool == nil {
 		// We don't know what kind of storage type the pool is.
-		return fmt.Errorf(`No storage pool struct in the backup file found. The storage pool needs to be recovered manually`)
+		return fmt.Errorf("No storage pool struct in the backup file found. The storage pool needs to be recovered manually")
 	}
 
 	// Try to retrieve the storage pool the instance supposedly lives on.
-	pool, err := storagePools.LoadByName(d.State(), instancePoolName)
+	pool, err := storagePools.LoadByName(s, instancePoolName)
 	if response.IsNotFoundError(err) {
 		// Create the storage pool db entry if it doesn't exist.
-		_, err = storagePoolDBCreate(d.State(), instancePoolName, "", backupConf.Pool.Driver, backupConf.Pool.Config)
+		_, err = storagePoolDBCreate(s, instancePoolName, "", backupConf.Pool.Driver, backupConf.Pool.Config)
 		if err != nil {
 			return fmt.Errorf("Create storage pool database entry: %w", err)
 		}
 
-		pool, err = storagePools.LoadByName(d.State(), instancePoolName)
+		pool, err = storagePools.LoadByName(s, instancePoolName)
 		if err != nil {
 			return fmt.Errorf("Load storage pool database entry: %w", err)
 		}
@@ -680,19 +709,15 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		return fmt.Errorf(`The storage pool's %q driver %q conflicts with the driver %q recorded in the instance's backup file`, instancePoolName, pool.Driver().Info().Name, backupConf.Pool.Driver)
 	}
 
-	// Check snapshots are consistent, and if not, if req.Force is true, then delete snapshots that do not exist in backup.yaml.
-	existingSnapshots, err := pool.CheckInstanceBackupFileSnapshots(backupConf, projectName, force, nil)
+	// Check snapshots are consistent.
+	existingSnapshots, err := pool.CheckInstanceBackupFileSnapshots(backupConf, projectName, nil)
 	if err != nil {
-		if errors.Is(err, storagePools.ErrBackupSnapshotsMismatch) {
-			return fmt.Errorf(`%s. Set "force" to discard non-existing snapshots`, err)
-		}
-
-		return fmt.Errorf("Checking snapshots: %w", err)
+		return fmt.Errorf("Failed checking snapshots: %w", err)
 	}
 
 	// Check if a storage volume entry for the instance already exists.
 	var dbVolume *db.StorageVolume
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, instanceDBVolType, backupConf.Container.Name, true)
 		if err != nil && !response.IsNotFoundError(err) {
 			return err
@@ -704,20 +729,22 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		return err
 	}
 
-	// If a storage volume entry exists only proceed if force was specified.
-	if dbVolume != nil && !force {
-		return fmt.Errorf(`Storage volume for instance %q already exists in the database. Set "force" to overwrite`, backupConf.Container.Name)
+	if dbVolume != nil {
+		return fmt.Errorf(`Storage volume for instance %q already exists in the database`, backupConf.Container.Name)
 	}
 
-	// Check if an entry for the instance already exists in the db.
-	_, instanceErr := d.db.Cluster.GetInstanceID(projectName, backupConf.Container.Name)
-	if instanceErr != nil && !response.IsNotFoundError(instanceErr) {
-		return instanceErr
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Check if an entry for the instance already exists in the db.
+		_, err := tx.GetInstanceID(ctx, projectName, backupConf.Container.Name)
+
+		return err
+	})
+	if err != nil && !response.IsNotFoundError(err) {
+		return err
 	}
 
-	// If a db entry exists only proceed if force was specified.
-	if instanceErr == nil && !force {
-		return fmt.Errorf(`Entry for instance %q already exists in the database. Set "force" to overwrite`, backupConf.Container.Name)
+	if err == nil {
+		return fmt.Errorf(`Entry for instance %q already exists in the database`, backupConf.Container.Name)
 	}
 
 	if backupConf.Volume == nil {
@@ -733,27 +760,27 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 			return fmt.Errorf(`The type %q of the storage volume is not identical to the instance's type %q`, dbVolume.Type, backupConf.Volume.Type)
 		}
 
-		// Remove the storage volume db entry for the instance since force was specified.
-		err := d.db.Cluster.RemoveStoragePoolVolume(projectName, backupConf.Container.Name, instanceDBVolType, pool.ID())
+		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Remove the storage volume db entry for the instance since force was specified.
+			return tx.RemoveStoragePoolVolume(ctx, projectName, backupConf.Container.Name, instanceDBVolType, pool.ID())
+		})
 		if err != nil {
 			return err
 		}
 	}
 
-	if instanceErr == nil {
-		// Remove the storage volume db entry for the instance since force was specified.
-		err := d.db.Cluster.DeleteInstance(projectName, backupConf.Container.Name)
-		if err != nil {
-			return err
-		}
-	}
+	var profiles []api.Profile
 
-	profiles, err := d.State().DB.Cluster.GetProfiles(projectName, backupConf.Container.Profiles)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		profiles, err = tx.GetProfiles(ctx, projectName, backupConf.Container.Profiles)
+
+		return err
+	})
 	if err != nil {
 		return fmt.Errorf("Failed loading profiles for instance: %w", err)
 	}
 
-	// Add root device if needed.
+	// Initialise the devices maps.
 	if backupConf.Container.Devices == nil {
 		backupConf.Container.Devices = make(map[string]map[string]string, 0)
 	}
@@ -762,6 +789,17 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		backupConf.Container.ExpandedDevices = make(map[string]map[string]string, 0)
 	}
 
+	// Apply device overrides.
+	// Do this before calling internalImportRootDevicePopulate so that device overrides are taken into account.
+	resultingDevices, err := shared.ApplyDeviceOverrides(backupConf.Container.Devices, backupConf.Container.ExpandedDevices, deviceOverrides)
+	if err != nil {
+		return err
+	}
+
+	backupConf.Container.Devices = resultingDevices
+
+	// Add root device if needed.
+	// And ensure root device is associated with same pool as instance has been imported to.
 	internalImportRootDevicePopulate(instancePoolName, backupConf.Container.Devices, backupConf.Container.ExpandedDevices, profiles)
 
 	revert := revert.New()
@@ -771,12 +809,12 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		return fmt.Errorf("No instance config in backup config")
 	}
 
-	instDBArgs, err := backup.ConfigToInstanceDBArgs(d.State(), backupConf, projectName, true)
+	instDBArgs, err := backup.ConfigToInstanceDBArgs(s, backupConf, projectName, true)
 	if err != nil {
 		return err
 	}
 
-	_, instOp, cleanup, err := instance.CreateInternal(d.State(), *instDBArgs, true)
+	_, instOp, cleanup, err := instance.CreateInternal(s, *instDBArgs, true)
 	if err != nil {
 		return fmt.Errorf("Failed creating instance record: %w", err)
 	}
@@ -798,20 +836,23 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 	for _, snap := range existingSnapshots {
 		snapInstName := fmt.Sprintf("%s%s%s", backupConf.Container.Name, shared.SnapshotDelimiter, snap.Name)
 
-		// Check if an entry for the snapshot already exists in the db.
-		_, snapErr := d.db.Cluster.GetInstanceSnapshotID(projectName, backupConf.Container.Name, snap.Name)
+		snapErr := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Check if an entry for the snapshot already exists in the db.
+			_, err := tx.GetInstanceSnapshotID(ctx, projectName, backupConf.Container.Name, snap.Name)
+
+			return err
+		})
 		if snapErr != nil && !response.IsNotFoundError(snapErr) {
 			return snapErr
 		}
 
-		// If a db entry exists only proceed if force was specified.
-		if snapErr == nil && !force {
-			return fmt.Errorf(`Entry for snapshot %q already exists in the database. Set "force" to overwrite`, snapInstName)
+		if snapErr == nil {
+			return fmt.Errorf(`Entry for snapshot %q already exists in the database`, snapInstName)
 		}
 
 		// Check if a storage volume entry for the snapshot already exists.
 		var dbVolume *db.StorageVolume
-		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, instanceDBVolType, snapInstName, true)
 			if err != nil && !response.IsNotFoundError(err) {
 				return err
@@ -824,22 +865,8 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		}
 
 		// If a storage volume entry exists only proceed if force was specified.
-		if dbVolume != nil && !force {
-			return fmt.Errorf(`Storage volume for snapshot %q already exists in the database. Set "force" to overwrite`, snapInstName)
-		}
-
-		if snapErr == nil {
-			err := d.db.Cluster.DeleteInstance(projectName, snapInstName)
-			if err != nil {
-				return err
-			}
-		}
-
 		if dbVolume != nil {
-			err := d.db.Cluster.RemoveStoragePoolVolume(projectName, snapInstName, instanceDBVolType, pool.ID())
-			if err != nil {
-				return err
-			}
+			return fmt.Errorf(`Storage volume for snapshot %q already exists in the database`, snapInstName)
 		}
 
 		baseImage := snap.Config["volatile.base_image"]
@@ -849,7 +876,11 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 			return err
 		}
 
-		profiles, err := d.State().DB.Cluster.GetProfiles(projectName, snap.Profiles)
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			profiles, err = tx.GetProfiles(ctx, projectName, snap.Profiles)
+
+			return err
+		})
 		if err != nil {
 			return fmt.Errorf("Failed loading profiles for instance snapshot %q: %w", snapInstName, err)
 		}
@@ -865,7 +896,7 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 
 		internalImportRootDevicePopulate(instancePoolName, snap.Devices, snap.ExpandedDevices, profiles)
 
-		_, snapInstOp, cleanup, err := instance.CreateInternal(d.State(), db.InstanceArgs{
+		_, snapInstOp, cleanup, err := instance.CreateInternal(s, db.InstanceArgs{
 			Project:      projectName,
 			Architecture: arch,
 			BaseImage:    baseImage,
@@ -885,7 +916,7 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		}
 
 		revert.Add(cleanup)
-		defer snapInstOp.Done(err)
+		defer snapInstOp.Done(err) //nolint:revive
 
 		// Recreate missing mountpoints and symlinks.
 		volStorageName := project.Instance(projectName, snapInstName)
@@ -909,7 +940,7 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 // device will be added, if the root disk config in the current profiles matches the effective backup.yaml config.
 func internalImportRootDevicePopulate(instancePoolName string, localDevices map[string]map[string]string, expandedDevices map[string]map[string]string, profiles []api.Profile) {
 	// First, check if localDevices from backup.yaml has a root disk.
-	rootName, _, _ := shared.GetRootDiskDevice(localDevices)
+	rootName, _, _ := instancetype.GetRootDiskDevice(localDevices)
 	if rootName != "" {
 		localDevices[rootName]["pool"] = instancePoolName
 
@@ -917,11 +948,11 @@ func internalImportRootDevicePopulate(instancePoolName string, localDevices map[
 	}
 
 	// Next check if expandedDevices from backup.yaml has a root disk.
-	expandedRootName, expandedRootConfig, _ := shared.GetRootDiskDevice(expandedDevices)
+	expandedRootName, expandedRootConfig, _ := instancetype.GetRootDiskDevice(expandedDevices)
 
 	// Extract root disk from expanded profile devices.
-	profileExpandedDevices := db.ExpandInstanceDevices(deviceConfig.NewDevices(localDevices), profiles)
-	profileExpandedRootName, profileExpandedRootConfig, _ := shared.GetRootDiskDevice(profileExpandedDevices.CloneNative())
+	profileExpandedDevices := instancetype.ExpandInstanceDevices(deviceConfig.NewDevices(localDevices), profiles)
+	profileExpandedRootName, profileExpandedRootConfig, _ := instancetype.GetRootDiskDevice(profileExpandedDevices.CloneNative())
 
 	// Record whether we need to add a new local disk device.
 	addLocalDisk := false
@@ -1015,18 +1046,19 @@ func internalGC(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalRAFTSnapshot(d *Daemon, r *http.Request) response.Response {
-	logger.Infof("Started forced RAFT snapshot")
-	err := d.gateway.Snapshot()
-	if err != nil {
-		logger.Errorf("Failed forced RAFT snapshot: %v", err)
-		return response.InternalError(err)
-	}
+	logger.Warn("Forced RAFT snapshot not supported")
 
-	logger.Infof("Completed forced RAFT snapshot")
-
-	return response.EmptySyncResponse
+	return response.InternalError(fmt.Errorf("Not supported"))
 }
 
 func internalBGPState(d *Daemon, r *http.Request) response.Response {
-	return response.SyncResponse(true, d.State().BGP.Debug())
+	s := d.State()
+
+	return response.SyncResponse(true, s.BGP.Debug())
+}
+
+func internalIdentityCacheRefresh(d *Daemon, r *http.Request) response.Response {
+	logger.Debug("Received identity cache update notification - refreshing cache")
+	d.State().UpdateIdentityCache()
+	return response.EmptySyncResponse
 }
