@@ -3,39 +3,42 @@ package main
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"net/http"
 
-	"github.com/lxc/lxd/lxd/backup"
-	backupConfig "github.com/lxc/lxd/lxd/backup/config"
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/db"
-	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	storagePools "github.com/lxc/lxd/lxd/storage"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/osarch"
+	"github.com/canonical/lxd/lxd/auth"
+	"github.com/canonical/lxd/lxd/backup"
+	backupConfig "github.com/canonical/lxd/lxd/backup/config"
+	"github.com/canonical/lxd/lxd/db"
+	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	storagePools "github.com/canonical/lxd/lxd/storage"
+	storageDrivers "github.com/canonical/lxd/lxd/storage/drivers"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // Define API endpoints for recover actions.
 var internalRecoverValidateCmd = APIEndpoint{
 	Path: "recover/validate",
 
-	Post: APIEndpointAction{Handler: internalRecoverValidate},
+	Post: APIEndpointAction{Handler: internalRecoverValidate, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 var internalRecoverImportCmd = APIEndpoint{
 	Path: "recover/import",
 
-	Post: APIEndpointAction{Handler: internalRecoverImport},
+	Post: APIEndpointAction{Handler: internalRecoverImport, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
 // init recover adds API endpoints to handler slice.
@@ -69,7 +72,7 @@ type internalRecoverImportPost struct {
 }
 
 // internalRecoverScan provides the discovery and import functionality for both recovery validate and import steps.
-func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOnly bool) response.Response {
+func internalRecoverScan(s *state.State, userPools []api.StoragePoolsPost, validateOnly bool) response.Response {
 	var err error
 	var projects map[string]*api.Project
 	var projectProfiles map[string][]*api.Profile
@@ -77,7 +80,7 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 
 	// Retrieve all project, profile and network info in a single transaction so we can use it for all
 	// imported instances and volumes, and avoid repeatedly querying the same information.
-	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Load list of projects for validation.
 		ps, err := dbCluster.GetProjects(ctx, tx.Tx())
 		if err != nil {
@@ -128,11 +131,6 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 		return response.SmartError(fmt.Errorf("Failed getting validate dependency check info: %w", err))
 	}
 
-	isClustered, err := cluster.Enabled(d.db.Node)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	res := internalRecoverValidateResult{}
 
 	revert := revert.New()
@@ -142,7 +140,7 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 	addDependencyError := func(err error) {
 		errStr := err.Error()
 
-		if !shared.StringInSlice(errStr, res.DependencyErrors) {
+		if !shared.ValueInSlice(errStr, res.DependencyErrors) {
 			res.DependencyErrors = append(res.DependencyErrors, errStr)
 		}
 	}
@@ -155,28 +153,35 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 
 	// Iterate the pools finding unknown volumes and perform validation.
 	for _, p := range userPools {
-		pool, err := storagePools.LoadByName(d.State(), p.Name)
+		pool, err := storagePools.LoadByName(s, p.Name)
 		if err != nil {
 			if response.IsNotFoundError(err) {
 				// If the pool DB record doesn't exist, and we are clustered, then don't proceed
 				// any further as we do not support pool DB record recovery when clustered.
-				if isClustered {
+				if s.ServerClustered {
 					return response.BadRequest(fmt.Errorf("Storage pool recovery not supported when clustered"))
 				}
 
 				// If pool doesn't exist in DB, initialise a temporary pool with the supplied info.
 				poolInfo := api.StoragePool{
-					Name:           p.Name,
-					Driver:         p.Driver,
-					StoragePoolPut: p.StoragePoolPut,
-					Status:         api.StoragePoolStatusCreated,
+					Name:   p.Name,
+					Driver: p.Driver,
+					Status: api.StoragePoolStatusCreated,
 				}
 
-				pool, err = storagePools.NewTemporary(d.State(), &poolInfo)
+				poolInfo.SetWritable(p.StoragePoolPut)
+
+				pool, err = storagePools.NewTemporary(s, &poolInfo)
 				if err != nil {
 					return response.SmartError(fmt.Errorf("Failed to initialise unknown pool %q: %w", p.Name, err))
 				}
 
+				// Populate configuration with default values.
+				err := pool.Driver().FillConfig()
+				if err != nil {
+					return response.SmartError(fmt.Errorf("Failed to evaluate the default configuration values for unknown pool %q: %w", p.Name, err))
+				}
+
 				err = pool.Driver().Validate(poolInfo.Config)
 				if err != nil {
 					return response.SmartError(fmt.Errorf("Failed config validation for unknown pool %q: %w", p.Name, err))
@@ -215,6 +220,10 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 		// Get list of unknown volumes on pool.
 		poolProjectVols, err := pool.ListUnknownVolumes(nil)
 		if err != nil {
+			if errors.Is(err, storageDrivers.ErrNotSupported) {
+				continue // Ignore unsupported storage drivers.
+			}
+
 			return response.SmartError(fmt.Errorf("Failed checking volumes on pool %q: %w", pool.Name(), err))
 		}
 
@@ -297,6 +306,10 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 						displayType = poolVol.Container.Type
 						displayName = poolVol.Container.Name
 						displaySnapshotCount = len(poolVol.Snapshots)
+					} else if poolVol.Bucket != nil {
+						displayType = "bucket"
+						displayName = poolVol.Bucket.Name
+						displaySnapshotCount = 0
 					} else {
 						displayType = "volume"
 						displayName = poolVol.Volume.Name
@@ -319,8 +332,72 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 
 	// If in import mode and no dependency errors, then re-create missing DB records.
 
-	// Create any missing instance and storage volume records.
 	for _, pool := range pools {
+		// Create missing storage pool DB record if neeed.
+		if pool.ID() == storagePools.PoolIDTemporary {
+			var instPoolVol *backupConfig.Config // Instance volume used for new pool record.
+			var poolID int64                     // Pool ID of created pool record.
+
+			var poolVols []*backupConfig.Config
+			for _, value := range poolsProjectVols[pool.Name()] {
+				poolVols = append(poolVols, value...)
+			}
+
+			// Search unknown volumes looking for an instance volume that can be used to
+			// restore the pool DB config from. This is preferable over using the user
+			// supplied config as it will include any additional settings not supplied.
+			for _, poolVol := range poolVols {
+				if poolVol.Pool != nil && poolVol.Pool.Config != nil {
+					instPoolVol = poolVol
+					break // Stop search once we've found an instance with pool config.
+				}
+			}
+
+			if instPoolVol != nil {
+				// Create storage pool DB record from config in the instance.
+				logger.Info("Creating storage pool DB record from instance config", logger.Ctx{"name": instPoolVol.Pool.Name, "description": instPoolVol.Pool.Description, "driver": instPoolVol.Pool.Driver, "config": instPoolVol.Pool.Config})
+				poolID, err = dbStoragePoolCreateAndUpdateCache(s, instPoolVol.Pool.Name, instPoolVol.Pool.Description, instPoolVol.Pool.Driver, instPoolVol.Pool.Config)
+				if err != nil {
+					return response.SmartError(fmt.Errorf("Failed creating storage pool %q database entry: %w", pool.Name(), err))
+				}
+			} else {
+				// Create storage pool DB record from config supplied by user if not
+				// instance volume pool config found.
+				poolDriverName := pool.Driver().Info().Name
+				poolDriverConfig := pool.Driver().Config()
+				logger.Info("Creating storage pool DB record from user config", logger.Ctx{"name": pool.Name(), "driver": poolDriverName, "config": poolDriverConfig})
+				poolID, err = dbStoragePoolCreateAndUpdateCache(s, pool.Name(), "", poolDriverName, poolDriverConfig)
+				if err != nil {
+					return response.SmartError(fmt.Errorf("Failed creating storage pool %q database entry: %w", pool.Name(), err))
+				}
+			}
+
+			revert.Add(func() {
+				_ = dbStoragePoolDeleteAndUpdateCache(s, pool.Name())
+			})
+
+			// Set storage pool node to storagePoolCreated.
+			// Must come before storage pool is loaded from the database.
+			err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				return tx.StoragePoolNodeCreated(poolID)
+			})
+			if err != nil {
+				return response.SmartError(fmt.Errorf("Failed marking storage pool %q local status as created: %w", pool.Name(), err))
+			}
+
+			logger.Debug("Marked storage pool local status as created", logger.Ctx{"pool": pool.Name()})
+
+			newPool, err := storagePools.LoadByName(s, pool.Name())
+			if err != nil {
+				return response.SmartError(fmt.Errorf("Failed loading created storage pool %q: %w", pool.Name(), err))
+			}
+
+			// Record this newly created pool so that defer doesn't unmount on return.
+			pools[pool.Name()] = newPool
+			pool = newPool // Replace temporary pool handle with proper one from DB.
+		}
+
+		// Create any missing instance, storage volume, and storage bucket records.
 		for projectName, poolVols := range poolsProjectVols[pool.Name()] {
 			projectInfo := projects[projectName]
 
@@ -330,87 +407,30 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 			}
 
 			profileProjectName := project.ProfileProjectFromRecord(projectInfo)
-			customStorageProjectName := project.StorageVolumeProjectFromRecord(projectInfo, db.StoragePoolVolumeTypeCustom)
-
-			// Create missing storage pool DB record if neeed.
-			if pool.ID() == storagePools.PoolIDTemporary {
-				var instPoolVol *backupConfig.Config // Instance volume used for new pool record.
-				var poolID int64                     // Pool ID of created pool record.
-
-				// Search unknown volumes looking for an instance volume that can be used to
-				// restore the pool DB config from. This is preferable over using the user
-				// supplied config as it will include any additional settings not supplied.
-				for _, poolVol := range poolVols {
-					if poolVol.Pool != nil && poolVol.Pool.Config != nil {
-						instPoolVol = poolVol
-						break // Stop search once we've found an instance with pool config.
-					}
-				}
-
-				if instPoolVol != nil {
-					// Create storage pool DB record from config in the instance.
-					logger.Info("Creating storage pool DB record from instance config", logger.Ctx{"name": instPoolVol.Pool.Name, "description": instPoolVol.Pool.Description, "driver": instPoolVol.Pool.Driver, "config": instPoolVol.Pool.Config})
-					poolID, err = dbStoragePoolCreateAndUpdateCache(d.State(), instPoolVol.Pool.Name, instPoolVol.Pool.Description, instPoolVol.Pool.Driver, instPoolVol.Pool.Config)
-					if err != nil {
-						return response.SmartError(fmt.Errorf("Failed creating storage pool %q database entry: %w", pool.Name(), err))
-					}
-				} else {
-					// Create storage pool DB record from config supplied by user if not
-					// instance volume pool config found.
-					poolDriverName := pool.Driver().Info().Name
-					poolDriverConfig := pool.Driver().Config()
-					logger.Info("Creating storage pool DB record from user config", logger.Ctx{"name": pool.Name(), "driver": poolDriverName, "config": poolDriverConfig})
-					poolID, err = dbStoragePoolCreateAndUpdateCache(d.State(), pool.Name(), "", poolDriverName, poolDriverConfig)
-					if err != nil {
-						return response.SmartError(fmt.Errorf("Failed creating storage pool %q database entry: %w", pool.Name(), err))
-					}
-				}
-
-				revert.Add(func() {
-					_ = dbStoragePoolDeleteAndUpdateCache(d.State(), pool.Name())
-				})
-
-				// Set storage pool node to storagePoolCreated.
-				// Must come before storage pool is loaded from the database.
-				err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-					return tx.StoragePoolNodeCreated(poolID)
-				})
-				if err != nil {
-					return response.SmartError(fmt.Errorf("Failed marking storage pool %q local status as created: %w", pool.Name(), err))
-				}
-
-				logger.Debug("Marked storage pool local status as created", logger.Ctx{"pool": pool.Name()})
-
-				newPool, err := storagePools.LoadByName(d.State(), pool.Name())
-				if err != nil {
-					return response.SmartError(fmt.Errorf("Failed loading created storage pool %q: %w", pool.Name(), err))
-				}
-
-				// Record this newly created pool so that defer doesn't unmount on return.
-				pools[pool.Name()] = newPool
-				pool = newPool // Replace temporary pool handle with proper one from DB.
-			}
+			customStorageProjectName := project.StorageVolumeProjectFromRecord(projectInfo, dbCluster.StoragePoolVolumeTypeCustom)
 
 			// Recover unknown custom volumes (do this first before recovering instances so that any
 			// instances that reference unknown custom volume disk devices can be created).
 			for _, poolVol := range poolVols {
-				if poolVol.Container != nil {
-					continue // Skip instance volumes.
+				if poolVol.Container != nil || poolVol.Bucket != nil {
+					continue // Skip instance volumes and buckets.
 				} else if poolVol.Container == nil && poolVol.Volume == nil {
 					return response.SmartError(fmt.Errorf("Volume is neither instance nor custom volume"))
 				}
 
 				// Import custom volume and any snapshots.
-				err = pool.ImportCustomVolume(customStorageProjectName, poolVol, nil)
+				cleanup, err := pool.ImportCustomVolume(customStorageProjectName, poolVol, nil)
 				if err != nil {
 					return response.SmartError(fmt.Errorf("Failed importing custom volume %q in project %q: %w", poolVol.Volume.Name, projectName, err))
 				}
+
+				revert.Add(cleanup)
 			}
 
 			// Recover unknown instance volumes.
 			for _, poolVol := range poolVols {
-				if poolVol.Container == nil && poolVol.Volume != nil {
-					continue // Skip custom volumes and invalid volumes.
+				if poolVol.Container == nil && (poolVol.Volume != nil || poolVol.Bucket != nil) {
+					continue // Skip custom volumes, invalid volumes and buckets.
 				}
 
 				// Recover instance volumes and any snapshots.
@@ -423,7 +443,7 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 					}
 				}
 
-				inst, cleanup, err := internalRecoverImportInstance(d.State(), pool, projectName, poolVol, profiles)
+				inst, cleanup, err := internalRecoverImportInstance(s, pool, projectName, poolVol, profiles)
 				if err != nil {
 					return response.SmartError(fmt.Errorf("Failed creating instance %q record in project %q: %w", poolVol.Container.Name, projectName, err))
 				}
@@ -441,7 +461,7 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 						}
 					}
 
-					cleanup, err := internalRecoverImportInstanceSnapshot(d.State(), pool, projectName, poolVol, poolInstSnap, profiles)
+					cleanup, err := internalRecoverImportInstanceSnapshot(s, pool, projectName, poolVol, poolInstSnap, profiles)
 					if err != nil {
 						return response.SmartError(fmt.Errorf("Failed creating instance %q snapshot %q record in project %q: %w", poolVol.Container.Name, poolInstSnap.Name, projectName, err))
 					}
@@ -450,14 +470,16 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 				}
 
 				// Recreate instance mount path and symlinks (must come after snapshot recovery).
-				err = pool.ImportInstance(inst, poolVol, nil)
+				cleanup, err = pool.ImportInstance(inst, poolVol, nil)
 				if err != nil {
 					return response.SmartError(fmt.Errorf("Failed importing instance %q in project %q: %w", poolVol.Container.Name, projectName, err))
 				}
 
+				revert.Add(cleanup)
+
 				// Reinitialise the instance's root disk quota even if no size specified (allows the storage driver the
 				// opportunity to reinitialise the quota based on the new storage volume's DB ID).
-				_, rootConfig, err := shared.GetRootDiskDevice(inst.ExpandedDevices().CloneNative())
+				_, rootConfig, err := instancetype.GetRootDiskDevice(inst.ExpandedDevices().CloneNative())
 				if err == nil {
 					err = pool.SetInstanceQuota(inst, rootConfig["size"], rootConfig["size.state"], nil)
 					if err != nil {
@@ -465,6 +487,22 @@ func internalRecoverScan(d *Daemon, userPools []api.StoragePoolsPost, validateOn
 					}
 				}
 			}
+
+			// Recover unknown buckets.
+			for _, poolVol := range poolVols {
+				// Skip non bucket volumes.
+				if poolVol.Bucket == nil {
+					continue
+				}
+
+				// Import bucket.
+				cleanup, err := pool.ImportBucket(projectName, poolVol, nil)
+				if err != nil {
+					return response.SmartError(fmt.Errorf("Failed importing bucket %q in project %q: %w", poolVol.Bucket.Name, projectName, err))
+				}
+
+				revert.Add(cleanup)
+			}
 		}
 	}
 
@@ -569,7 +607,7 @@ func internalRecoverValidate(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	return internalRecoverScan(d, req.Pools, true)
+	return internalRecoverScan(d.State(), req.Pools, true)
 }
 
 // internalRecoverImport performs the pool volume recovery.
@@ -581,5 +619,5 @@ func internalRecoverImport(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	return internalRecoverScan(d, req.Pools, false)
+	return internalRecoverScan(d.State(), req.Pools, false)
 }
