@@ -6,8 +6,8 @@ import (
 	"strings"
 	"text/template"
 
-	"github.com/lxc/lxd/lxd/sys"
-	"github.com/lxc/lxd/shared"
+	"github.com/canonical/lxd/lxd/sys"
+	"github.com/canonical/lxd/shared"
 )
 
 var archiveProfileTpl = template.Must(template.New("archiveProfile").Parse(`#include <tunables/global>
@@ -24,7 +24,7 @@ profile "{{.name}}" {
   {{ .backupsPath }}/** rw,
   {{ .imagesPath }}/** r,
 
-  signal (receive) set=("term") peer=unconfined,
+  signal (receive) set=("term"),
 
   # Capabilities
   capability chown,
@@ -130,6 +130,7 @@ func archiveProfile(outputPath string, allowedCommandPaths []string) (string, er
 		"backupsPath":         backupsPath,
 		"imagesPath":          imagesPath,
 		"allowedCommandPaths": derefCommandPaths,
+		"snap":                shared.InSnap(),
 	})
 	if err != nil {
 		return "", err
