@@ -7,11 +7,11 @@ import (
 	"strings"
 	"text/template"
 
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/sys"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/sys"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
 )
 
 // Internal copy of the device interface.
@@ -52,6 +52,11 @@ profile "{{ .name }}" flags=(attach_disconnected,mediate_deleted) {
   ptrace (read),
   ptrace (trace),
 
+  /etc/machine-id r,
+  {{ .rootPath }}/run/systemd/resolve/stub-resolv.conf r,
+  {{ .rootPath }}/run/{resolvconf,NetworkManager,systemd/resolve,connman,netconfig}/resolv.conf r,
+  {{ .rootPath }}/usr/lib/systemd/resolv.conf r,
+
   # Needed for lxd fork commands
   {{ .exePath }} mr,
   @{PROC}/@{pid}/cmdline r,
@@ -66,6 +71,7 @@ profile "{{ .name }}" flags=(attach_disconnected,mediate_deleted) {
   deny @{PROC}/@{pid}/cgroup r,
   deny /sys/module/apparmor/parameters/enabled r,
   deny /sys/kernel/mm/transparent_hugepage/hpage_pmd_size r,
+  deny /sys/devices/virtual/dmi/id/product_uuid r,
 
 {{- if .snap }}
   # The binary itself (for nesting)
@@ -137,7 +143,7 @@ func forkproxyProfile(sysOS *sys.OS, inst instance, dev device) (string, error)
 			return "", err
 		}
 
-		if !shared.StringInSlice(v, sockets) {
+		if !shared.ValueInSlice(v, sockets) {
 			sockets = append(sockets, v)
 		}
 	}
