@@ -10,31 +10,31 @@ import (
 
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxd/backup"
-	"github.com/lxc/lxd/lxd/db"
-	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/operationtype"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/lifecycle"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	storagePools "github.com/lxc/lxd/lxd/storage"
-	"github.com/lxc/lxd/lxd/task"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/instancewriter"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/lxd/backup"
+	"github.com/canonical/lxd/lxd/db"
+	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/operationtype"
+	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/instancewriter"
+	"github.com/canonical/lxd/lxd/lifecycle"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	storagePools "github.com/canonical/lxd/lxd/storage"
+	"github.com/canonical/lxd/lxd/task"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
 )
 
 // Create a new backup.
 func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.Instance, op *operations.Operation) error {
-	l := logger.AddContext(logger.Log, logger.Ctx{"project": sourceInst.Project().Name, "instance": sourceInst.Name(), "name": args.Name})
+	l := logger.AddContext(logger.Ctx{"project": sourceInst.Project().Name, "instance": sourceInst.Name(), "name": args.Name})
 	l.Debug("Instance backup started")
 	defer l.Debug("Instance backup finished")
 
@@ -53,7 +53,9 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 	}
 
 	// Create the database entry.
-	err = s.DB.Cluster.CreateInstanceBackup(args)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.CreateInstanceBackup(ctx, args)
+	})
 	if err != nil {
 		if err == db.ErrAlreadyDefined {
 			return fmt.Errorf("Backup %q already exists", args.Name)
@@ -62,7 +64,11 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 		return fmt.Errorf("Insert backup info into database: %w", err)
 	}
 
-	revert.Add(func() { _ = s.DB.Cluster.DeleteInstanceBackup(args.Name) })
+	revert.Add(func() {
+		_ = s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
+			return tx.DeleteInstanceBackup(ctx, args.Name)
+		})
+	})
 
 	// Get the backup struct.
 	b, err := instance.BackupLoadByName(s, sourceInst.Project().Name, args.Name)
@@ -124,7 +130,11 @@ func backupCreate(s *state.State, args db.InstanceBackup, sourceInst instance.In
 	// Get IDMap to unshift container as the tarball is created.
 	var idmap *idmap.IdmapSet
 	if sourceInst.Type() == instancetype.Container {
-		c := sourceInst.(instance.Container)
+		c, ok := sourceInst.(instance.Container)
+		if !ok {
+			return fmt.Errorf("Invalid instance type")
+		}
+
 		idmap, err = c.DiskIdmap()
 		if err != nil {
 			return fmt.Errorf("Error getting container IDMAP: %w", err)
@@ -290,26 +300,44 @@ func backupWriteIndex(sourceInst instance.Instance, pool storagePools.Pool, opti
 	return nil
 }
 
-func pruneExpiredContainerBackupsTask(d *Daemon) (task.Func, task.Schedule) {
+func pruneExpiredBackupsTask(d *Daemon) (task.Func, task.Schedule) {
 	f := func(ctx context.Context) {
+		s := d.State()
+
 		opRun := func(op *operations.Operation) error {
-			return pruneExpiredContainerBackups(ctx, d)
+			err := pruneExpiredInstanceBackups(ctx, s)
+			if err != nil {
+				return fmt.Errorf("Failed pruning expired instance backups: %w", err)
+			}
+
+			err = pruneExpiredStorageVolumeBackups(ctx, s)
+			if err != nil {
+				return fmt.Errorf("Failed pruning expired storage volume backups: %w", err)
+			}
+
+			return nil
 		}
 
-		op, err := operations.OperationCreate(d.State(), "", operations.OperationClassTask, operationtype.BackupsExpire, nil, nil, opRun, nil, nil, nil)
+		op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.BackupsExpire, nil, nil, opRun, nil, nil, nil)
 		if err != nil {
-			logger.Error("Failed to start expired instance backups operation", logger.Ctx{"err": err})
+			logger.Error("Failed creating expired backups operation", logger.Ctx{"err": err})
 			return
 		}
 
-		logger.Info("Pruning expired instance backups")
+		logger.Info("Pruning expired backups")
 		err = op.Start()
 		if err != nil {
-			logger.Error("Failed to expire instance backups", logger.Ctx{"err": err})
+			logger.Error("Failed starting expired backups operation", logger.Ctx{"err": err})
+			return
 		}
 
-		_, _ = op.Wait(ctx)
-		logger.Info("Done pruning expired instance backups")
+		err = op.Wait(ctx)
+		if err != nil {
+			logger.Error("Failed pruning expired backups", logger.Ctx{"err": err})
+			return
+		}
+
+		logger.Info("Done pruning expired backups")
 	}
 
 	f(context.Background())
@@ -329,20 +357,26 @@ func pruneExpiredContainerBackupsTask(d *Daemon) (task.Func, task.Schedule) {
 	return f, schedule
 }
 
-func pruneExpiredContainerBackups(ctx context.Context, d *Daemon) error {
+func pruneExpiredInstanceBackups(ctx context.Context, s *state.State) error {
+	var backups []db.InstanceBackup
+
 	// Get the list of expired backups.
-	backups, err := d.db.Cluster.GetExpiredInstanceBackups()
+	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+		backups, err = tx.GetExpiredInstanceBackups(ctx)
+		return err
+	})
 	if err != nil {
 		return fmt.Errorf("Unable to retrieve the list of expired instance backups: %w", err)
 	}
 
 	for _, b := range backups {
-		inst, err := instance.LoadByID(d.State(), b.InstanceID)
+		inst, err := instance.LoadByID(s, b.InstanceID)
 		if err != nil {
 			return fmt.Errorf("Error loading instance for deleting backup %q: %w", b.Name, err)
 		}
 
-		instBackup := backup.NewInstanceBackup(d.State(), inst, b.ID, b.Name, b.CreationDate, b.ExpiryDate, b.InstanceOnly, b.OptimizedStorage)
+		instBackup := backup.NewInstanceBackup(s, inst, b.ID, b.Name, b.CreationDate, b.ExpiryDate, b.InstanceOnly, b.OptimizedStorage)
 		err = instBackup.Delete()
 		if err != nil {
 			return fmt.Errorf("Error deleting instance backup %q: %w", b.Name, err)
@@ -353,7 +387,7 @@ func pruneExpiredContainerBackups(ctx context.Context, d *Daemon) error {
 }
 
 func volumeBackupCreate(s *state.State, args db.StoragePoolVolumeBackup, projectName string, poolName string, volumeName string) error {
-	l := logger.AddContext(logger.Log, logger.Ctx{"project": projectName, "storage_volume": volumeName, "name": args.Name})
+	l := logger.AddContext(logger.Ctx{"project": projectName, "storage_volume": volumeName, "name": args.Name})
 	l.Debug("Volume backup started")
 	defer l.Debug("Volume backup finished")
 
@@ -372,7 +406,9 @@ func volumeBackupCreate(s *state.State, args db.StoragePoolVolumeBackup, project
 	}
 
 	// Create the database entry.
-	err = s.DB.Cluster.CreateStoragePoolVolumeBackup(args)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.CreateStoragePoolVolumeBackup(ctx, args)
+	})
 	if err != nil {
 		if err == db.ErrAlreadyDefined {
 			return fmt.Errorf("Backup %q already exists", args.Name)
@@ -381,9 +417,18 @@ func volumeBackupCreate(s *state.State, args db.StoragePoolVolumeBackup, project
 		return fmt.Errorf("Failed creating backup record: %w", err)
 	}
 
-	revert.Add(func() { _ = s.DB.Cluster.DeleteStoragePoolVolumeBackup(args.Name) })
+	revert.Add(func() {
+		_ = s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
+			return tx.DeleteStoragePoolVolumeBackup(ctx, args.Name)
+		})
+	})
 
-	backupRow, err := s.DB.Cluster.GetStoragePoolVolumeBackup(projectName, poolName, args.Name)
+	var backupRow db.StoragePoolVolumeBackup
+
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		backupRow, err = tx.GetStoragePoolVolumeBackup(ctx, projectName, poolName, args.Name)
+		return err
+	})
 	if err != nil {
 		return fmt.Errorf("Failed getting backup record: %w", err)
 	}
@@ -546,3 +591,50 @@ func volumeBackupWriteIndex(s *state.State, projectName string, volumeName strin
 
 	return nil
 }
+
+func pruneExpiredStorageVolumeBackups(ctx context.Context, s *state.State) error {
+	var volumeBackups []*backup.VolumeBackup
+
+	// Get the list of expired backups.
+	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		nodeID := tx.GetNodeID()
+
+		backups, err := tx.GetExpiredStorageVolumeBackups(ctx)
+		if err != nil {
+			return fmt.Errorf("Unable to retrieve the list of expired storage volume backups: %w", err)
+		}
+
+		for _, b := range backups {
+			vol, err := tx.GetStoragePoolVolumeWithID(ctx, int(b.VolumeID))
+			if err != nil {
+				logger.Warn("Failed getting storage pool of backup", logger.Ctx{"backup": b.Name, "err": err})
+				continue
+			}
+
+			// Ignore volumes on other nodes, but include remote pools (NodeID == -1).
+			if vol.NodeID != -1 && vol.NodeID != nodeID {
+				continue
+			}
+
+			volBackup := backup.NewVolumeBackup(s, vol.ProjectName, vol.PoolName, vol.Name, b.ID, b.Name, b.CreationDate, b.ExpiryDate, b.VolumeOnly, b.OptimizedStorage)
+
+			volumeBackups = append(volumeBackups, volBackup)
+		}
+
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+
+	// The deletion is done outside of the transaction to avoid any unnecessary IO while inside of
+	// the transaction.
+	for _, b := range volumeBackups {
+		err := b.Delete()
+		if err != nil {
+			return fmt.Errorf("Error deleting storage volume backup %q: %w", b.Name(), err)
+		}
+	}
+
+	return nil
+}
