@@ -4,7 +4,6 @@ import (
 	"bufio"
 	"context"
 	"crypto/tls"
-	"crypto/x509"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -17,18 +16,18 @@ import (
 	"sync"
 	"time"
 
-	dqlite "github.com/canonical/go-dqlite"
-	client "github.com/canonical/go-dqlite/client"
+	"github.com/canonical/go-dqlite"
+	"github.com/canonical/go-dqlite/client"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/tcp"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/identity"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/tcp"
 )
 
 // NewGateway creates a new Gateway for managing access to the dqlite cluster.
@@ -154,10 +153,10 @@ func setDqliteVersionHeader(request *http.Request) {
 // These handlers might return 404, either because this LXD node is a
 // non-clustered node not available over the network or because it is not a
 // database node part of the dqlite cluster.
-func (g *Gateway) HandlerFuncs(heartbeatHandler HeartbeatHandler, trustedCerts func() map[cluster.CertificateType]map[string]x509.Certificate) map[string]http.HandlerFunc {
+func (g *Gateway) HandlerFuncs(heartbeatHandler HeartbeatHandler, identityCache *identity.Cache) map[string]http.HandlerFunc {
 	database := func(w http.ResponseWriter, r *http.Request) {
 		g.lock.RLock()
-		if !tlsCheckCert(r, g.networkCert, g.state().ServerCert(), trustedCerts()) {
+		if !tlsCheckCert(r, g.networkCert, g.state().ServerCert(), identityCache) {
 			g.lock.RUnlock()
 			http.Error(w, "403 invalid client certificate", http.StatusForbidden)
 			return
@@ -337,15 +336,6 @@ func (g *Gateway) HandlerFuncs(heartbeatHandler HeartbeatHandler, trustedCerts f
 	}
 }
 
-// Snapshot can be used to manually trigger a RAFT snapshot.
-func (g *Gateway) Snapshot() error {
-	g.lock.RLock()
-	defer g.lock.RUnlock()
-
-	// TODO: implement support for forcing a snapshot in dqlite v1
-	return fmt.Errorf("Not supported")
-}
-
 // WaitUpgradeNotification waits for a notification from another node that all
 // nodes in the cluster should now have been upgraded and have matching schema
 // and API versions.
@@ -616,7 +606,7 @@ func (g *Gateway) Reset(networkCert *shared.CertInfo) error {
 }
 
 // ErrNodeIsNotClustered indicates the node is not clustered.
-var ErrNodeIsNotClustered error = fmt.Errorf("Server is not clustered")
+var ErrNodeIsNotClustered = fmt.Errorf("Server is not clustered")
 
 // LeaderAddress returns the address of the current raft leader.
 func (g *Gateway) LeaderAddress() (string, error) {
@@ -700,46 +690,10 @@ func (g *Gateway) LeaderAddress() (string, error) {
 	defer cleanup()
 
 	for _, address := range addresses {
-		timeout := 2 * time.Second
-		client := &http.Client{
-			Transport: transport,
-			Timeout:   timeout,
-		}
-
-		url := fmt.Sprintf("https://%s%s", address, databaseEndpoint)
-		request, err := http.NewRequest("GET", url, nil)
+		leader, err := attemptGetLeaderAddressFromNodeAddress(g.ctx, transport, address)
 		if err != nil {
-			return "", err
-		}
-
-		setDqliteVersionHeader(request)
-
-		// Use 1s later timeout to give HTTP client chance timeout with
-		// more useful info.
-		ctx, cancel := context.WithTimeout(g.ctx, timeout+time.Second)
-		defer cancel()
-		request = request.WithContext(ctx)
-		response, err := client.Do(request)
-		if err != nil {
-			logger.Debugf("Failed to fetch leader address from %s", address)
-			continue
-		}
-
-		if response.StatusCode != http.StatusOK {
-			logger.Debugf("Request for leader address from %s failed", address)
-			continue
-		}
-
-		info := map[string]string{}
-		err = json.NewDecoder(response.Body).Decode(&info)
-		if err != nil {
-			logger.Debugf("Failed to parse leader address from %s", address)
-			continue
-		}
-
-		leader := info["leader"]
-		if leader == "" {
-			logger.Debugf("Raft node %s returned no leader address", address)
+			return "", fmt.Errorf("Failed to find leader address: %w", err)
+		} else if leader == "" {
 			continue
 		}
 
@@ -1018,6 +972,56 @@ func (g *Gateway) nodeAddress(raftAddress string) (string, error) {
 	return address, nil
 }
 
+// attemptGetLeaderAddressFromNodeAddress calls the /internal/database endpoint of the given node address to ascertain
+// the leader address. An error is returned if the given address is invalid. Other errors are ignored. The returned
+// leader address may be empty.
+func attemptGetLeaderAddressFromNodeAddress(ctx context.Context, transport http.RoundTripper, nodeAddress string) (string, error) {
+	timeout := 2 * time.Second
+	httpClient := &http.Client{
+		Transport: transport,
+		Timeout:   timeout,
+	}
+
+	url := fmt.Sprintf("https://%s%s", nodeAddress, databaseEndpoint)
+	request, err := http.NewRequest("GET", url, nil)
+	if err != nil {
+		return "", err
+	}
+
+	setDqliteVersionHeader(request)
+
+	// Use 1s later timeout to give HTTP client chance timeout with
+	// more useful info.
+	ctx, cancel := context.WithTimeout(ctx, timeout+time.Second)
+	defer cancel()
+	request = request.WithContext(ctx)
+	response, err := httpClient.Do(request)
+	if err != nil {
+		logger.Debugf("Failed to fetch leader address from %s", request.URL.Host)
+		return "", nil
+	}
+
+	if response.StatusCode != http.StatusOK {
+		logger.Debugf("Request for leader address from %s failed", request.URL.Host)
+		return "", nil
+	}
+
+	info := map[string]string{}
+	err = json.NewDecoder(response.Body).Decode(&info)
+	if err != nil {
+		logger.Debugf("Failed to parse leader address from %s", request.URL.Host)
+		return "", nil
+	}
+
+	leader := info["leader"]
+	if leader == "" {
+		logger.Debugf("Raft node %s returned no leader address", request.URL.Host)
+		return "", nil
+	}
+
+	return leader, nil
+}
+
 func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway) (net.Conn, error) {
 	config, err := tlsClientConfig(g.networkCert, g.state().ServerCert())
 	if err != nil {
@@ -1058,16 +1062,16 @@ func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway
 
 	revert.Add(func() { _ = conn.Close() })
 
-	l := logger.AddContext(logger.Log, logger.Ctx{"name": name, "local": conn.LocalAddr(), "remote": conn.RemoteAddr()})
+	l := logger.AddContext(logger.Ctx{"name": name, "local": conn.LocalAddr(), "remote": conn.RemoteAddr()})
 	l.Info("Dqlite connected outbound")
 
 	remoteTCP, err := tcp.ExtractConn(conn)
 	if err != nil {
-		l.Error("Failed extracting TCP connection from remote connection", logger.Ctx{"err": err})
+		l.Warn("Failed extracting TCP connection from remote connection", logger.Ctx{"err": err})
 	} else {
 		err := tcp.SetTimeouts(remoteTCP, time.Second*30)
 		if err != nil {
-			l.Error("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
+			l.Warn("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
 		}
 	}
 
@@ -1150,17 +1154,17 @@ func runDqliteProxy(stopCh chan struct{}, bindAddress string, acceptCh chan net.
 // Copies data between a remote TLS network connection and a local unix socket.
 // Accepts name argument that can be used to identify the connection in the logs.
 func dqliteProxy(name string, stopCh chan struct{}, remote net.Conn, local net.Conn) {
-	l := logger.AddContext(logger.Log, logger.Ctx{"name": name, "local": remote.LocalAddr(), "remote": remote.RemoteAddr()})
+	l := logger.AddContext(logger.Ctx{"name": name, "local": remote.LocalAddr(), "remote": remote.RemoteAddr()})
 	l.Info("Dqlite proxy started")
 	defer l.Info("Dqlite proxy stopped")
 
 	remoteTCP, err := tcp.ExtractConn(remote)
 	if err != nil {
-		l.Error("Failed extracting TCP connection from remote connection", logger.Ctx{"err": err})
+		l.Warn("Failed extracting TCP connection from remote connection", logger.Ctx{"err": err})
 	} else {
 		err := tcp.SetTimeouts(remoteTCP, time.Second*30)
 		if err != nil {
-			l.Error("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
+			l.Warn("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
 		}
 	}
 
@@ -1250,6 +1254,7 @@ type dqliteNodeStore struct {
 	onDisk   client.NodeStore
 }
 
+// Get gets node information from the dqlite node store.
 func (s *dqliteNodeStore) Get(ctx context.Context) ([]client.NodeInfo, error) {
 	if s.inMemory != nil {
 		return s.inMemory.Get(ctx)
@@ -1258,6 +1263,7 @@ func (s *dqliteNodeStore) Get(ctx context.Context) ([]client.NodeInfo, error) {
 	return s.onDisk.Get(ctx)
 }
 
+// Set sets node information in the dqlite node store.
 func (s *dqliteNodeStore) Set(ctx context.Context, servers []client.NodeInfo) error {
 	if s.inMemory != nil {
 		return s.inMemory.Set(ctx, servers)
