@@ -14,15 +14,16 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/cluster"
-	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
-	"github.com/lxc/lxd/lxd/db"
-	clusterDB "github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/cluster"
+	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
+	"github.com/canonical/lxd/lxd/db"
+	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/identity"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/version"
 )
 
 func TestBootstrap_UnmetPreconditions(t *testing.T) {
@@ -130,12 +131,8 @@ func TestBootstrap(t *testing.T) {
 	// The cluster certificate is in place.
 	assert.True(t, shared.PathExists(filepath.Join(state.OS.VarDir, "cluster.crt")))
 
-	trustedCerts := func() map[clusterDB.CertificateType]map[string]x509.Certificate {
-		return nil
-	}
-
 	// The dqlite driver is now exposed over the network.
-	for path, handler := range gateway.HandlerFuncs(nil, trustedCerts) {
+	for path, handler := range gateway.HandlerFuncs(nil, &identity.Cache{}) {
 		mux.HandleFunc(path, handler)
 	}
 
@@ -288,15 +285,18 @@ func TestJoin(t *testing.T) {
 	altServerCert := shared.TestingAltKeyPair()
 	trustedAltServerCert, _ := x509.ParseCertificate(altServerCert.KeyPair().Certificate[0])
 
-	trustedCerts := func() map[clusterDB.CertificateType]map[string]x509.Certificate {
-		return map[clusterDB.CertificateType]map[string]x509.Certificate{
-			clusterDB.CertificateTypeServer: {
-				altServerCert.Fingerprint(): *trustedAltServerCert,
-			},
-		}
-	}
+	identityCache := &identity.Cache{}
+	err := identityCache.ReplaceAll([]identity.CacheEntry{
+		{
+			AuthenticationMethod: api.AuthenticationMethodTLS,
+			IdentityType:         api.IdentityTypeCertificateServer,
+			Identifier:           altServerCert.Fingerprint(),
+			Certificate:          trustedAltServerCert,
+		},
+	}, nil)
+	require.NoError(t, err)
 
-	for path, handler := range targetGateway.HandlerFuncs(nil, trustedCerts) {
+	for path, handler := range targetGateway.HandlerFuncs(nil, identityCache) {
 		targetMux.HandleFunc(path, handler)
 	}
 
@@ -307,7 +307,6 @@ func TestJoin(t *testing.T) {
 	targetStore := targetGateway.NodeStore()
 	targetDialFunc := targetGateway.DialFunc()
 
-	var err error
 	targetState.DB.Cluster, err = db.OpenCluster(context.Background(), "db.bin", targetStore, targetAddress, "/unused/db/dir", 10*time.Second, nil, driver.WithDialFunc(targetDialFunc))
 	targetState.ServerCert = func() *shared.CertInfo { return targetCert }
 	require.NoError(t, err)
@@ -329,14 +328,19 @@ func TestJoin(t *testing.T) {
 	require.NoError(t, err)
 
 	// PreparedStmts is a global variable and will be overwritten by the OpenCluster call below, so save it here.
-	targetStmts := clusterDB.PreparedStmts
+	targetStmts := dbCluster.PreparedStmts
 
 	targetF := &membershipFixtures{t: t, state: targetState}
 	targetF.ClusterAddress(targetAddress)
 
 	err = cluster.Bootstrap(targetState, targetGateway, "buzz")
 	require.NoError(t, err)
-	_, err = targetState.DB.Cluster.GetNetworks(project.Default)
+
+	err = targetState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		_, err = tx.GetNetworks(ctx, api.ProjectDefaultName)
+
+		return err
+	})
 	require.NoError(t, err)
 
 	// Setup a joining node
@@ -353,7 +357,7 @@ func TestJoin(t *testing.T) {
 
 	defer func() { _ = gateway.Shutdown() }()
 
-	for path, handler := range gateway.HandlerFuncs(nil, trustedCerts) {
+	for path, handler := range gateway.HandlerFuncs(nil, identityCache) {
 		mux.HandleFunc(path, handler)
 	}
 
@@ -384,19 +388,19 @@ func TestJoin(t *testing.T) {
 	require.NoError(t, err)
 
 	// Save the other instance of PreparedStmts here.
-	sourceStmts := clusterDB.PreparedStmts
+	sourceStmts := dbCluster.PreparedStmts
 
 	f := &membershipFixtures{t: t, state: state}
 	f.ClusterAddress(address)
 
 	// Accept the joining node.
-	clusterDB.PreparedStmts = targetStmts
+	dbCluster.PreparedStmts = targetStmts
 	raftNodes, err := cluster.Accept(
 		targetState, targetGateway, "rusp", address, cluster.SchemaVersion, len(version.APIExtensions), osarch.ARCH_64BIT_INTEL_X86)
 	require.NoError(t, err)
 
 	// Actually join the cluster.
-	clusterDB.PreparedStmts = sourceStmts
+	dbCluster.PreparedStmts = sourceStmts
 	err = cluster.Join(state, gateway, targetCert, altServerCert, "rusp", raftNodes)
 	require.NoError(t, err)
 
@@ -421,7 +425,7 @@ func TestJoin(t *testing.T) {
 	leaving, err := cluster.Leave(state, targetGateway, "rusp", false /* force */)
 	require.NoError(t, err)
 	assert.Equal(t, address, leaving)
-	clusterDB.PreparedStmts = targetStmts
+	dbCluster.PreparedStmts = targetStmts
 	err = cluster.Purge(targetState.DB.Cluster, "rusp")
 	require.NoError(t, err)
 
