@@ -5,8 +5,10 @@ import (
 	"reflect"
 	"sort"
 	"strconv"
+	"strings"
+	"unicode"
 
-	"github.com/lxc/lxd/shared"
+	"github.com/canonical/lxd/shared"
 )
 
 // Map is a structured map of config keys to config values.
@@ -203,6 +205,13 @@ func (m *Map) update(values map[string]string) ([]string, error) {
 func (m *Map) set(name string, value string, initial bool) (bool, error) {
 	// Bypass schema for user.* keys
 	if shared.IsUserConfig(name) {
+		for _, r := range strings.TrimPrefix(name, "user.") {
+			// Only allow letters, digits, and punctuation characters.
+			if !unicode.In(r, unicode.Letter, unicode.Digit, unicode.Punct) {
+				return false, fmt.Errorf("Invalid key name")
+			}
+		}
+
 		current, ok := m.values[name]
 		if ok && value == current {
 			// Value is unchanged
@@ -223,6 +232,12 @@ func (m *Map) set(name string, value string, initial bool) (bool, error) {
 		return false, fmt.Errorf("unknown key")
 	}
 
+	// When unsetting a config key, the value argument will be empty.
+	// This ensures that the default value is set if the provided value is empty.
+	if value == "" {
+		value = key.Default
+	}
+
 	err := key.validate(value)
 	if err != nil {
 		return false, err
