@@ -10,22 +10,23 @@ import (
 	"path/filepath"
 	"time"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/lifecycle"
-	"github.com/lxc/lxd/lxd/locking"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/request"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/cancel"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/lifecycle"
+	"github.com/canonical/lxd/lxd/locking"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/cancel"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // ImageDownloadArgs used with ImageDownload.
@@ -47,8 +48,8 @@ type ImageDownloadArgs struct {
 }
 
 // imageOperationLock acquires a lock for operating on an image and returns the unlock function.
-func (d *Daemon) imageOperationLock(fingerprint string) locking.UnlockFunc {
-	l := logger.AddContext(logger.Log, logger.Ctx{"fingerprint": fingerprint})
+func imageOperationLock(fingerprint string) (locking.UnlockFunc, error) {
+	l := logger.AddContext(logger.Ctx{"fingerprint": fingerprint})
 	l.Debug("Acquiring lock for image")
 	defer l.Debug("Lock acquired for image")
 
@@ -56,32 +57,32 @@ func (d *Daemon) imageOperationLock(fingerprint string) locking.UnlockFunc {
 }
 
 // ImageDownload resolves the image fingerprint and if not in the database, downloads it.
-func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *ImageDownloadArgs) (*api.Image, error) {
+func ImageDownload(r *http.Request, s *state.State, op *operations.Operation, args *ImageDownloadArgs) (*api.Image, error) {
 	var err error
 	var ctxMap logger.Ctx
 
 	var remote lxd.ImageServer
 	var info *api.Image
 
-	// Default protocol is LXD. Copy so that local modifications aren't propgated to args.
+	// Default protocol is LXD. Copy so that local modifications aren't propagated to args.
 	protocol := args.Protocol
 	if protocol == "" {
 		protocol = "lxd"
 	}
 
-	// Copy so that local modifications aren't propgated to args.
+	// Copy so that local modifications aren't propagated to args.
 	alias := args.Alias
 
 	// Default the fingerprint to the alias string we received
 	fp := alias
 
 	// Attempt to resolve the alias
-	if shared.StringInSlice(protocol, []string{"lxd", "simplestreams"}) {
+	if shared.ValueInSlice(protocol, []string{"lxd", "simplestreams"}) {
 		clientArgs := &lxd.ConnectionArgs{
 			TLSServerCert: args.Certificate,
 			UserAgent:     version.UserAgent,
-			Proxy:         d.proxy,
-			CachePath:     d.os.CacheDir,
+			Proxy:         s.Proxy,
+			CachePath:     s.OS.CacheDir,
 			CacheExpiry:   time.Hour,
 		}
 
@@ -123,7 +124,11 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 	}
 
 	// Ensure we are the only ones operating on this image.
-	unlock := d.imageOperationLock(fp)
+	unlock, err := imageOperationLock(fp)
+	if err != nil {
+		return nil, err
+	}
+
 	defer unlock()
 
 	// If auto-update is on and we're being given the image by
@@ -131,11 +136,11 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 	// server/protocol/alias, regardless of whether it's stale or
 	// not (we can assume that it will be not *too* stale since
 	// auto-update is on).
-	interval := d.State().GlobalConfig.ImagesAutoUpdateIntervalHours()
+	interval := s.GlobalConfig.ImagesAutoUpdateIntervalHours()
 
 	if args.PreferCached && interval > 0 && alias != fp {
-		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			for _, architecture := range d.os.Architectures {
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			for _, architecture := range s.OS.Architectures {
 				cachedFingerprint, err := tx.GetCachedImageSourceFingerprint(ctx, args.Server, args.Protocol, alias, args.Type, architecture)
 				if err == nil && cachedFingerprint != fp {
 					fp = cachedFingerprint
@@ -150,60 +155,83 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 		}
 	}
 
-	// Check if the image already exists in this project (partial hash match).
-	_, imgInfo, err := d.db.Cluster.GetImage(fp, cluster.ImageFilter{Project: &args.ProjectName})
+	var imgInfo *api.Image
+
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Check if the image already exists in this project (partial hash match).
+		_, imgInfo, err = tx.GetImage(ctx, fp, cluster.ImageFilter{Project: &args.ProjectName})
+
+		return err
+	})
 	if err == nil {
-		// Check if the image is available locally or it's on another node.
-		nodeAddress, err := d.State().DB.Cluster.LocateImage(imgInfo.Fingerprint)
+		var nodeAddress string
+
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Check if the image is available locally or it's on another node.
+			nodeAddress, err = tx.LocateImage(ctx, imgInfo.Fingerprint)
+
+			return err
+		})
 		if err != nil {
 			return nil, fmt.Errorf("Failed locating image %q in the cluster: %w", imgInfo.Fingerprint, err)
 		}
 
 		if nodeAddress != "" {
 			// The image is available from another node, let's try to import it.
-			err = instanceImageTransfer(d, r, args.ProjectName, imgInfo.Fingerprint, nodeAddress)
+			err = instanceImageTransfer(s, r, args.ProjectName, imgInfo.Fingerprint, nodeAddress)
 			if err != nil {
 				return nil, fmt.Errorf("Failed transferring image %q from %q: %w", imgInfo.Fingerprint, nodeAddress, err)
 			}
 
-			// As the image record already exists in the project, just add the node ID to the image.
-			err = d.db.Cluster.AddImageToLocalNode(args.ProjectName, imgInfo.Fingerprint)
+			err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				// As the image record already exists in the project, just add the node ID to the image.
+				return tx.AddImageToLocalNode(ctx, args.ProjectName, imgInfo.Fingerprint)
+			})
 			if err != nil {
 				return nil, fmt.Errorf("Failed adding transferred image %q to local cluster member: %w", imgInfo.Fingerprint, err)
 			}
 		}
 	} else if response.IsNotFoundError(err) {
-		// Check if the image already exists in some other project.
-		_, imgInfo, err = d.db.Cluster.GetImageFromAnyProject(fp)
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Check if the image already exists in some other project.
+			_, imgInfo, err = tx.GetImageFromAnyProject(ctx, fp)
+
+			return err
+		})
 		if err == nil {
-			// Check if the image is available locally or it's on another node. Do this before creating
-			// the missing DB record so we don't include ourself in the search results.
-			nodeAddress, err := d.State().DB.Cluster.LocateImage(imgInfo.Fingerprint)
-			if err != nil {
-				return nil, fmt.Errorf("Locate image %q in the cluster: %w", imgInfo.Fingerprint, err)
-			}
+			var nodeAddress string
 
-			// We need to insert the database entry for this project, including the node ID entry.
-			err = d.db.Cluster.CreateImage(args.ProjectName, imgInfo.Fingerprint, imgInfo.Filename, imgInfo.Size, args.Public, imgInfo.AutoUpdate, imgInfo.Architecture, imgInfo.CreatedAt, imgInfo.ExpiresAt, imgInfo.Properties, imgInfo.Type)
-			if err != nil {
-				return nil, err
-			}
-
-			// Mark the image as "cached" if downloading for an instance.
-			if args.SetCached {
-				err := d.db.Cluster.SetImageCachedAndLastUseDate(args.ProjectName, imgInfo.Fingerprint, time.Now().UTC())
+			err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				// Check if the image is available locally or it's on another node. Do this before creating
+				// the missing DB record so we don't include ourself in the search results.
+				nodeAddress, err = tx.LocateImage(ctx, imgInfo.Fingerprint)
 				if err != nil {
-					return nil, fmt.Errorf("Failed setting cached flag and last use date: %w", err)
+					return fmt.Errorf("Locate image %q in the cluster: %w", imgInfo.Fingerprint, err)
 				}
-			}
 
-			var id int
-			id, imgInfo, err = d.db.Cluster.GetImage(fp, cluster.ImageFilter{Project: &args.ProjectName})
-			if err != nil {
-				return nil, err
-			}
+				// We need to insert the database entry for this project, including the node ID entry.
+				err = tx.CreateImage(ctx, args.ProjectName, imgInfo.Fingerprint, imgInfo.Filename, imgInfo.Size, args.Public, imgInfo.AutoUpdate, imgInfo.Architecture, imgInfo.CreatedAt, imgInfo.ExpiresAt, imgInfo.Properties, imgInfo.Type, nil)
+				if err != nil {
+					return fmt.Errorf("Failed creating image record for project: %w", err)
+				}
 
-			err = d.db.Cluster.CreateImageSource(id, args.Server, args.Protocol, args.Certificate, alias)
+				// Mark the image as "cached" if downloading for an instance.
+				if args.SetCached {
+					err = tx.SetImageCachedAndLastUseDate(ctx, args.ProjectName, imgInfo.Fingerprint, time.Now().UTC())
+					if err != nil {
+						return fmt.Errorf("Failed setting cached flag and last use date: %w", err)
+					}
+				}
+
+				var id int
+
+				id, imgInfo, err = tx.GetImage(ctx, fp, cluster.ImageFilter{Project: &args.ProjectName})
+				if err != nil {
+					return err
+				}
+
+				return tx.CreateImageSource(ctx, id, args.Server, args.Protocol, args.Certificate, alias)
+			})
 			if err != nil {
 				return nil, err
 			}
@@ -211,7 +239,7 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 			// Transfer image if needed (after database record has been created above).
 			if nodeAddress != "" {
 				// The image is available from another node, let's try to import it.
-				err = instanceImageTransfer(d, r, args.ProjectName, info.Fingerprint, nodeAddress)
+				err = instanceImageTransfer(s, r, args.ProjectName, info.Fingerprint, nodeAddress)
 				if err != nil {
 					return nil, fmt.Errorf("Failed transferring image: %w", err)
 				}
@@ -231,19 +259,26 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 
 		ctxMap["pool"] = args.StoragePool
 
-		// Get the ID of the target storage pool.
-		poolID, err := d.db.Cluster.GetStoragePoolID(args.StoragePool)
+		var poolID int64
+		var poolIDs []int64
+
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Get the ID of the target storage pool.
+			poolID, err = tx.GetStoragePoolID(ctx, args.StoragePool)
+			if err != nil {
+				return err
+			}
+
+			// Check if the image is already in the pool.
+			poolIDs, err = tx.GetPoolsWithImage(ctx, info.Fingerprint)
+
+			return err
+		})
 		if err != nil {
 			return nil, err
 		}
 
-		// Check if the image is already in the pool.
-		poolIDs, err := d.db.Cluster.GetPoolsWithImage(info.Fingerprint)
-		if err != nil {
-			return nil, err
-		}
-
-		if shared.Int64InSlice(poolID, poolIDs) {
+		if shared.ValueInSlice(poolID, poolIDs) {
 			logger.Debug("Image already exists on storage pool", ctxMap)
 			return info, nil
 		}
@@ -251,7 +286,7 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 		// Import the image in the pool.
 		logger.Debug("Image does not exist on storage pool", ctxMap)
 
-		err = imageCreateInPool(d, info, args.StoragePool)
+		err = imageCreateInPool(s, info, args.StoragePool)
 		if err != nil {
 			ctxMap["err"] = err
 			logger.Debug("Failed to create image on storage pool", ctxMap)
@@ -410,13 +445,17 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 		}
 	} else if protocol == "direct" {
 		// Setup HTTP client
-		httpClient, err := util.HTTPClient(args.Certificate, d.proxy)
+		httpClient, err := util.HTTPClient(args.Certificate, s.Proxy)
 		if err != nil {
 			return nil, err
 		}
 
 		// Use relatively short response header timeout so as not to hold the image lock open too long.
-		httpTransport := httpClient.Transport.(*http.Transport)
+		httpTransport, ok := httpClient.Transport.(*http.Transport)
+		if !ok {
+			return nil, fmt.Errorf("Invalid http client type")
+		}
+
 		httpTransport.ResponseHeaderTimeout = 30 * time.Second
 
 		req, err := http.NewRequest("GET", args.Server, nil)
@@ -427,7 +466,7 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 		req.Header.Set("User-Agent", version.UserAgent)
 
 		// Make the request
-		raw, doneCh, err := cancel.CancelableDownload(canceler, httpClient, req)
+		raw, doneCh, err := cancel.CancelableDownload(canceler, httpClient.Do, req)
 		if err != nil {
 			return nil, err
 		}
@@ -506,10 +545,12 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 		info.AutoUpdate = args.AutoUpdate
 	}
 
-	// Create the database entry
-	err = d.db.Cluster.CreateImage(args.ProjectName, info.Fingerprint, info.Filename, info.Size, info.Public, info.AutoUpdate, info.Architecture, info.CreatedAt, info.ExpiresAt, info.Properties, info.Type)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Create the database entry
+		return tx.CreateImage(ctx, args.ProjectName, info.Fingerprint, info.Filename, info.Size, info.Public, info.AutoUpdate, info.Architecture, info.CreatedAt, info.ExpiresAt, info.Properties, info.Type, nil)
+	})
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("Failed creating image record: %w", err)
 	}
 
 	// Image is in the DB now, don't wipe on-disk files on failure
@@ -533,12 +574,14 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 
 	// Record the image source
 	if alias != fp {
-		id, _, err := d.db.Cluster.GetImage(fp, cluster.ImageFilter{Project: &args.ProjectName})
-		if err != nil {
-			return nil, err
-		}
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			id, _, err := tx.GetImage(ctx, fp, cluster.ImageFilter{Project: &args.ProjectName})
+			if err != nil {
+				return err
+			}
 
-		err = d.db.Cluster.CreateImageSource(id, args.Server, protocol, args.Certificate, alias)
+			return tx.CreateImageSource(ctx, id, args.Server, protocol, args.Certificate, alias)
+		})
 		if err != nil {
 			return nil, err
 		}
@@ -546,7 +589,7 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 
 	// Import into the requested storage pool
 	if args.StoragePool != "" {
-		err = imageCreateInPool(d, info, args.StoragePool)
+		err = imageCreateInPool(s, info, args.StoragePool)
 		if err != nil {
 			return nil, err
 		}
@@ -554,7 +597,9 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 
 	// Mark the image as "cached" if downloading for an instance
 	if args.SetCached {
-		err := d.db.Cluster.SetImageCachedAndLastUseDate(args.ProjectName, fp, time.Now().UTC())
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			return tx.SetImageCachedAndLastUseDate(ctx, args.ProjectName, fp, time.Now().UTC())
+		})
 		if err != nil {
 			return nil, fmt.Errorf("Failed setting cached flag and last use date: %w", err)
 		}
@@ -569,7 +614,7 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 		requestor = request.CreateRequestor(r)
 	}
 
-	d.State().Events.SendLifecycle(args.ProjectName, lifecycle.ImageCreated.Event(info.Fingerprint, args.ProjectName, requestor, logger.Ctx{"type": info.Type}))
+	s.Events.SendLifecycle(args.ProjectName, lifecycle.ImageCreated.Event(info.Fingerprint, args.ProjectName, requestor, logger.Ctx{"type": info.Type}))
 
 	return info, nil
 }
