@@ -11,10 +11,10 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // If the node is not clustered, the schema updates works normally.
@@ -65,7 +65,7 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "2", schema+1, apiExtensions)
 			},
 			false,
-			"this node's version is behind, please upgrade",
+			"Failed to ensure schema: This cluster member's version is behind, please upgrade",
 		},
 		{
 			`this node's number of API extensions is behind`,
@@ -74,7 +74,7 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "2", schema, apiExtensions+1)
 			},
 			false,
-			"this node's version is behind, please upgrade",
+			"Failed to ensure schema: This cluster member's version is behind, please upgrade",
 		},
 		{
 			`inconsistent schema version and API extensions number`,
@@ -83,7 +83,7 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "2", schema+1, apiExtensions-1)
 			},
 			false,
-			"nodes have inconsistent versions",
+			"Failed to ensure schema: Cluster members have inconsistent versions",
 		},
 	}
 
