@@ -17,11 +17,11 @@ import (
 
 	"github.com/canonical/go-dqlite/driver"
 
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/node"
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/node"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // DB represents access to LXD's global and local databases.
@@ -82,8 +82,8 @@ func DirectAccess(db *sql.DB) *Node {
 // DB returns the low level database handle to the node-local SQLite
 // database.
 //
-// FIXME: this is used for compatibility with some legacy code, and should be
-//        dropped once there are no call sites left.
+//	FIXME: this is used for compatibility with some legacy code, and should be
+//		dropped once there are no call sites left.
 func (n *Node) DB() *sql.DB {
 	return n.db
 }
@@ -365,7 +365,7 @@ func (c *Cluster) transaction(ctx context.Context, f func(context.Context, *Clus
 		nodeID: c.nodeID,
 	}
 
-	return c.retry(func() error {
+	return query.Retry(ctx, func(ctx context.Context) error {
 		txFunc := func(ctx context.Context, tx *sql.Tx) error {
 			clusterTx.tx = tx
 			return f(ctx, clusterTx)
@@ -384,14 +384,6 @@ func (c *Cluster) transaction(ctx context.Context, f func(context.Context, *Clus
 	})
 }
 
-func (c *Cluster) retry(f func() error) error {
-	if c.closingCtx.Err() != nil {
-		return f()
-	}
-
-	return query.Retry(f)
-}
-
 // NodeID sets the node NodeID associated with this cluster instance. It's used for
 // backward-compatibility of all db-related APIs that were written before
 // clustering and don't accept a node NodeID, so in those cases we automatically
@@ -411,8 +403,8 @@ func (c *Cluster) Close() error {
 
 // DB returns the low level database handle to the cluster database.
 //
-// FIXME: this is used for compatibility with some legacy code, and should be
-//        dropped once there are no call sites left.
+//	FIXME: this is used for compatibility with some legacy code, and should be
+//		dropped once there are no call sites left.
 func (c *Cluster) DB() *sql.DB {
 	return c.db
 }
@@ -475,7 +467,10 @@ func DqliteLatestSegment() (string, error) {
 
 	sort.Strings(fileNames)
 
-	r, _ := regexp.Compile("^[0-9]*-[0-9]*$")
+	r, err := regexp.Compile(`^[0-9]+-[0-9]+$`)
+	if err != nil {
+		return "none", err
+	}
 
 	for i := range fileNames {
 		fileName := fileNames[len(fileNames)-1-i]
@@ -497,82 +492,8 @@ func DqliteLatestSegment() (string, error) {
 	return "none", nil
 }
 
-func dbQueryRowScan(c *Cluster, q string, args []any, outargs []any) error {
-	return c.retry(func() error {
-		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
-			return tx.QueryRowContext(ctx, q, args...).Scan(outargs...)
-		})
-	})
-}
-
-func doDbScan(c *Cluster, q string, args []any, outargs []any) ([][]any, error) {
-	result := [][]any{}
-
-	err := c.retry(func() error {
-		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
-			rows, err := tx.QueryContext(ctx, q, args...)
-			if err != nil {
-				return err
-			}
-
-			defer func() { _ = rows.Close() }()
-
-			for rows.Next() {
-				ptrargs := make([]any, len(outargs))
-				for i := range outargs {
-					switch t := outargs[i].(type) {
-					case string:
-						str := ""
-						ptrargs[i] = &str
-					case int:
-						integer := 0
-						ptrargs[i] = &integer
-					case int64:
-						integer := int64(0)
-						ptrargs[i] = &integer
-					case bool:
-						boolean := bool(false)
-						ptrargs[i] = &boolean
-					default:
-						return fmt.Errorf("Bad interface type: %s", t)
-					}
-				}
-				err = rows.Scan(ptrargs...)
-				if err != nil {
-					return err
-				}
-
-				newargs := make([]any, len(outargs))
-				for i := range ptrargs {
-					switch t := outargs[i].(type) {
-					case string:
-						newargs[i] = *ptrargs[i].(*string)
-					case int:
-						newargs[i] = *ptrargs[i].(*int)
-					case int64:
-						newargs[i] = *ptrargs[i].(*int64)
-					case bool:
-						newargs[i] = *ptrargs[i].(*bool)
-					default:
-						return fmt.Errorf("Bad interface type: %s", t)
-					}
-				}
-				result = append(result, newargs)
-			}
-
-			err = rows.Err()
-			if err != nil {
-				return err
-			}
-
-			return nil
-		})
-	})
-	if err != nil {
-		return [][]any{}, err
-	}
-
-	return result, nil
+func dbQueryRowScan(ctx context.Context, c *ClusterTx, q string, args []any, outargs []any) error {
+	return c.tx.QueryRowContext(ctx, q, args...).Scan(outargs...)
 }
 
 /*
@@ -588,16 +509,63 @@ func doDbScan(c *Cluster, q string, args []any, outargs []any) ([][]any, error)
  * The result will be an array (one per output row) of arrays (one per output argument)
  * of interfaces, containing pointers to the actual output arguments.
  */
-func queryScan(c *Cluster, q string, inargs []any, outfmt []any) ([][]any, error) {
-	return doDbScan(c, q, inargs, outfmt)
-}
+func queryScan(ctx context.Context, c *ClusterTx, q string, inargs []any, outfmt []any) ([][]any, error) {
+	result := [][]any{}
 
-func exec(c *Cluster, q string, args ...any) error {
-	err := c.retry(func() error {
-		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
-			_, err := tx.Exec(q, args...)
-			return err
-		})
-	})
-	return err
+	rows, err := c.tx.QueryContext(ctx, q, inargs...)
+	if err != nil {
+		return [][]any{}, err
+	}
+
+	defer func() { _ = rows.Close() }()
+
+	for rows.Next() {
+		ptrargs := make([]any, len(outfmt))
+		for i := range outfmt {
+			switch t := outfmt[i].(type) {
+			case string:
+				str := ""
+				ptrargs[i] = &str
+			case int:
+				integer := 0
+				ptrargs[i] = &integer
+			case int64:
+				integer := int64(0)
+				ptrargs[i] = &integer
+			case bool:
+				boolean := bool(false)
+				ptrargs[i] = &boolean
+			default:
+				return [][]any{}, fmt.Errorf("Bad interface type: %s", t)
+			}
+		}
+		err = rows.Scan(ptrargs...)
+		if err != nil {
+			return [][]any{}, err
+		}
+
+		newargs := make([]any, len(outfmt))
+		for i := range ptrargs {
+			switch t := outfmt[i].(type) {
+			case string:
+				newargs[i] = *ptrargs[i].(*string)
+			case int:
+				newargs[i] = *ptrargs[i].(*int)
+			case int64:
+				newargs[i] = *ptrargs[i].(*int64)
+			case bool:
+				newargs[i] = *ptrargs[i].(*bool)
+			default:
+				return [][]any{}, fmt.Errorf("Bad interface type: %s", t)
+			}
+		}
+		result = append(result, newargs)
+	}
+
+	err = rows.Err()
+	if err != nil {
+		return [][]any{}, err
+	}
+
+	return result, nil
 }
