@@ -11,12 +11,12 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/query"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/query"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // InstanceArgs is a value object holding all db-related details about an instance.
@@ -49,9 +49,9 @@ func (c *ClusterTx) GetInstanceNames(ctx context.Context, project string) ([]str
 	stmt := `
 SELECT instances.name FROM instances
   JOIN projects ON projects.id = instances.project_id
-  WHERE projects.name = ? AND instances.type = ?
+  WHERE projects.name = ?
 `
-	return query.SelectStrings(ctx, c.tx, stmt, project, instancetype.Any)
+	return query.SelectStrings(ctx, c.tx, stmt, project)
 }
 
 // GetNodeAddressOfInstance returns the address of the node hosting the
@@ -141,71 +141,71 @@ SELECT nodes.id, nodes.address
 	return address, nil
 }
 
-// GetProjectAndInstanceNamesByNodeAddress returns the project and name of all instances grouped by
-// cluster node address. Each node address has a slice of instances, where each instance is represented
-// as an array of length 2 in which element 0 is the project and element 1 is the instance name.
-//
-// The node address of instances running on the local member is set to the empty
-// string, to distinguish it from remote nodes.
-//
-// Instances whose node is down are added to the special address "0.0.0.0".
-func (c *ClusterTx) GetProjectAndInstanceNamesByNodeAddress(ctx context.Context, projects []string, instType instancetype.Type) (map[string][][2]string, error) {
-	offlineThreshold, err := c.GetNodeOfflineThreshold(ctx)
-	if err != nil {
-		return nil, err
-	}
+// Instance represents basic instance info.
+type Instance struct {
+	ID       int64
+	Name     string
+	Project  string
+	Location string
+	Type     instancetype.Type
+}
 
+// GetInstancesByMemberAddress returns the instances associated to each cluster member address.
+// The member address of instances running on the local member is set to the empty string, to distinguish it from
+// remote nodes. Instances whose member is down are added to the special address "0.0.0.0".
+func (c *ClusterTx) GetInstancesByMemberAddress(ctx context.Context, offlineThreshold time.Duration, projects []string, instType instancetype.Type) (map[string][]Instance, error) {
 	args := make([]any, 0, 2) // Expect up to 2 filters.
-	var filters strings.Builder
+	var q strings.Builder
+
+	q.WriteString(`SELECT
+		instances.id, instances.name, instances.type,
+		nodes.id, nodes.name, nodes.address, nodes.heartbeat,
+		projects.name
+	FROM instances
+	JOIN nodes ON nodes.id = instances.node_id
+	JOIN projects ON projects.id = instances.project_id
+	`)
 
 	// Project filter.
-	filters.WriteString(fmt.Sprintf("projects.name IN %s", query.Params(len(projects))))
+	q.WriteString(fmt.Sprintf("WHERE projects.name IN %s", query.Params(len(projects))))
 	for _, project := range projects {
 		args = append(args, project)
 	}
 
 	// Instance type filter.
 	if instType != instancetype.Any {
-		filters.WriteString(" AND instances.type = ?")
+		q.WriteString(" AND instances.type = ?")
 		args = append(args, instType)
 	}
 
-	stmt := fmt.Sprintf(`
-SELECT instances.name, nodes.id, nodes.address, nodes.heartbeat, projects.name
-  FROM instances
-  JOIN nodes ON nodes.id = instances.node_id
-  JOIN projects ON projects.id = instances.project_id
-  WHERE %s
-  ORDER BY instances.id
-`, filters.String())
+	q.WriteString(" ORDER BY instances.id")
 
-	rows, err := c.tx.QueryContext(ctx, stmt, args...)
+	rows, err := c.tx.QueryContext(ctx, q.String(), args...)
 	if err != nil {
 		return nil, err
 	}
 
 	defer func() { _ = rows.Close() }()
 
-	result := map[string][][2]string{}
+	memberAddressInstances := make(map[string][]Instance)
 
-	for i := 0; rows.Next(); i++ {
-		var instanceName string
-		var nodeAddress string
-		var nodeID int64
-		var nodeHeartbeat time.Time
-		var projectName string
-		err := rows.Scan(&instanceName, &nodeID, &nodeAddress, &nodeHeartbeat, &projectName)
+	for rows.Next() {
+		var inst Instance
+		var memberAddress string
+		var memberID int64
+		var memberHeartbeat time.Time
+		err := rows.Scan(&inst.ID, &inst.Name, &inst.Type, &memberID, &inst.Location, &memberAddress, &memberHeartbeat, &inst.Project)
 		if err != nil {
 			return nil, err
 		}
 
-		if nodeID == c.nodeID {
-			nodeAddress = ""
-		} else if nodeIsOffline(offlineThreshold, nodeHeartbeat) {
-			nodeAddress = "0.0.0.0"
+		if memberID == c.nodeID {
+			memberAddress = ""
+		} else if nodeIsOffline(offlineThreshold, memberHeartbeat) {
+			memberAddress = "0.0.0.0"
 		}
 
-		result[nodeAddress] = append(result[nodeAddress], [2]string{projectName, instanceName})
+		memberAddressInstances[memberAddress] = append(memberAddressInstances[memberAddress], inst)
 	}
 
 	err = rows.Err()
@@ -213,7 +213,7 @@ SELECT instances.name, nodes.id, nodes.address, nodes.heartbeat, projects.name
 		return nil, err
 	}
 
-	return result, nil
+	return memberAddressInstances, nil
 }
 
 // ErrInstanceListStop used as return value from InstanceList's instanceFunc when prematurely stopping the search.
@@ -221,7 +221,7 @@ var ErrInstanceListStop = fmt.Errorf("search stopped")
 
 // InstanceList loads all instances across all projects and for each instance runs the instanceFunc passing in the
 // instance and it's project and profiles. Accepts optional filter arguments to specify a subset of instances.
-func (c *Cluster) InstanceList(ctx context.Context, instanceFunc func(inst InstanceArgs, project api.Project) error, filters ...cluster.InstanceFilter) error {
+func (c *ClusterTx) InstanceList(ctx context.Context, instanceFunc func(inst InstanceArgs, project api.Project) error, filters ...cluster.InstanceFilter) error {
 	projectsByName := make(map[string]*api.Project)
 	var instances map[int]InstanceArgs
 
@@ -238,52 +238,45 @@ func (c *Cluster) InstanceList(ctx context.Context, instanceFunc func(inst Insta
 	}
 
 	// Retrieve required info from the database in single transaction for performance.
-	err := c.Transaction(ctx, func(ctx context.Context, tx *ClusterTx) error {
-		// Get all projects.
-		projects, err := cluster.GetProjects(ctx, tx.tx)
-		if err != nil {
-			return fmt.Errorf("Failed loading projects: %w", err)
+	// Get all projects.
+	projects, err := cluster.GetProjects(ctx, c.tx)
+	if err != nil {
+		return fmt.Errorf("Failed loading projects: %w", err)
+	}
+
+	// Get all instances using supplied filter.
+	dbInstances, err := cluster.GetInstances(ctx, c.tx, validFilters...)
+	if err != nil {
+		return fmt.Errorf("Failed loading instances: %w", err)
+	}
+
+	// Fill instances with config, devices and profiles.
+	instances, err = c.InstancesToInstanceArgs(ctx, true, dbInstances...)
+	if err != nil {
+		return err
+	}
+
+	// Record which projects are referenced by at least one instance in the list.
+	for _, instance := range instances {
+		_, ok := projectsByName[instance.Project]
+		if !ok {
+			projectsByName[instance.Project] = nil
+		}
+	}
+
+	// Populate projectsByName map entry for referenced projects.
+	// This way we only call ToAPI() on the projects actually referenced by the instances in
+	// the list, which can reduce the number of queries run.
+	for _, project := range projects {
+		_, ok := projectsByName[project.Name]
+		if !ok {
+			continue
 		}
 
-		// Get all instances using supplied filter.
-		dbInstances, err := cluster.GetInstances(ctx, tx.tx, validFilters...)
-		if err != nil {
-			return fmt.Errorf("Failed loading instances: %w", err)
-		}
-
-		// Fill instances with config, devices and profiles.
-		instances, err = tx.InstancesToInstanceArgs(ctx, true, dbInstances...)
+		projectsByName[project.Name], err = project.ToAPI(ctx, c.tx)
 		if err != nil {
 			return err
 		}
-
-		// Record which projects are referenced by at least one instance in the list.
-		for _, instance := range instances {
-			_, ok := projectsByName[instance.Project]
-			if !ok {
-				projectsByName[instance.Project] = nil
-			}
-		}
-
-		// Populate projectsByName map entry for referenced projects.
-		// This way we only call ToAPI() on the projects actually referenced by the instances in
-		// the list, which can reduce the number of queries run.
-		for _, project := range projects {
-			_, ok := projectsByName[project.Name]
-			if !ok {
-				continue
-			}
-
-			projectsByName[project.Name], err = project.ToAPI(ctx, tx.tx)
-			if err != nil {
-				return err
-			}
-		}
-
-		return nil
-	})
-	if err != nil {
-		return err
 	}
 
 	// Call the instanceFunc provided for each instance after the transaction has ended, as we don't know if
@@ -638,101 +631,24 @@ func (c *ClusterTx) InstancesToInstanceArgs(ctx context.Context, fillProfiles bo
 	return instanceArgs, nil
 }
 
-// GetProjectInstanceToNodeMap returns a map associating the project (key element 0) and name (key element 1) of each
-// instance in the given projects to the name of the node hosting the instance.
-func (c *ClusterTx) GetProjectInstanceToNodeMap(ctx context.Context, projects []string, instType instancetype.Type) (map[[2]string]string, error) {
-	args := make([]any, 0, 2) // Expect up to 2 filters.
-	var filters strings.Builder
-
-	// Project filter.
-	filters.WriteString(fmt.Sprintf("projects.name IN %s", query.Params(len(projects))))
-	for _, project := range projects {
-		args = append(args, project)
-	}
-
-	// Instance type filter.
-	if instType != instancetype.Any {
-		filters.WriteString(" AND instances.type = ?")
-		args = append(args, instType)
-	}
-
-	stmt := fmt.Sprintf(`
-SELECT instances.name, nodes.name, projects.name
-  FROM instances
-  JOIN nodes ON nodes.id = instances.node_id
-  JOIN projects ON projects.id = instances.project_id
-  WHERE %s
-`, filters.String())
-
-	rows, err := c.tx.QueryContext(ctx, stmt, args...)
-	if err != nil {
-		return nil, err
-	}
-
-	defer func() { _ = rows.Close() }()
-
-	result := map[[2]string]string{}
-
-	for i := 0; rows.Next(); i++ {
-		var instanceName string
-		var nodeName string
-		var projectName string
-		err := rows.Scan(&instanceName, &nodeName, &projectName)
-		if err != nil {
-			return nil, err
-		}
-
-		result[[2]string{projectName, instanceName}] = nodeName
-	}
-
-	err = rows.Err()
-	if err != nil {
-		return nil, err
-	}
-
-	return result, nil
-}
-
 // UpdateInstanceNode changes the name of an instance and the cluster member hosting it.
 // It's meant to be used when moving a non-running instance backed by ceph from one cluster node to another.
-func (c *ClusterTx) UpdateInstanceNode(ctx context.Context, project, oldName string, newName string, newNode string, volumeType int) error {
-	// First check that the container to be moved is backed by a ceph
-	// volume.
-	poolName, err := c.GetInstancePool(ctx, project, oldName)
-	if err != nil {
-		return fmt.Errorf("Failed to get instance's storage pool name: %w", err)
-	}
-
-	poolID, err := c.GetStoragePoolID(ctx, poolName)
-	if err != nil {
-		return fmt.Errorf("Failed to get instance's storage pool ID: %w", err)
-	}
-
-	poolDriver, err := c.GetStoragePoolDriver(ctx, poolID)
-	if err != nil {
-		return fmt.Errorf("Failed to get instance's storage pool driver: %w", err)
-	}
-
-	if poolDriver != "ceph" {
-		return fmt.Errorf("Instance's storage pool is not of type ceph")
-	}
-
-	// Update the name of the container and of its snapshots, and the node
-	// ID they are associated with.
-	containerID, err := cluster.GetInstanceID(ctx, c.tx, project, oldName)
+func (c *ClusterTx) UpdateInstanceNode(ctx context.Context, project string, oldName string, newName string, newMemberName string, poolID int64, volumeType int) error {
+	// Update the name of the instance and its snapshots, and the member ID they are associated with.
+	instanceID, err := cluster.GetInstanceID(ctx, c.tx, project, oldName)
 	if err != nil {
 		return fmt.Errorf("Failed to get instance's ID: %w", err)
 	}
 
-	node, err := c.GetNodeByName(ctx, newNode)
+	member, err := c.GetNodeByName(ctx, newMemberName)
 	if err != nil {
-		return fmt.Errorf("Failed to get new node's info: %w", err)
+		return fmt.Errorf("Failed to get new member %q info: %w", newMemberName, err)
 	}
 
 	stmt := "UPDATE instances SET node_id=?, name=? WHERE id=?"
-	result, err := c.tx.Exec(stmt, node.ID, newName, containerID)
+	result, err := c.tx.Exec(stmt, member.ID, newName, instanceID)
 	if err != nil {
-		return fmt.Errorf("Failed to update instance's name and node ID: %w", err)
+		return fmt.Errorf("Failed to update instance's name and member ID: %w", err)
 	}
 
 	n, err := result.RowsAffected()
@@ -783,8 +699,8 @@ func (c *ClusterTx) GetLocalInstancesInProject(ctx context.Context, filter clust
 }
 
 // CreateInstanceConfig inserts a new config for the container with the given ID.
-func (c *ClusterTx) CreateInstanceConfig(id int, config map[string]string) error {
-	return CreateInstanceConfig(c.tx, id, config)
+func (c *ClusterTx) CreateInstanceConfig(ctx context.Context, id int, config map[string]string) error {
+	return CreateInstanceConfig(ctx, c.tx, id, config)
 }
 
 // UpdateInstanceConfig inserts/updates/deletes the provided keys.
@@ -845,9 +761,9 @@ func (c *ClusterTx) configUpdate(id int, values map[string]string, insertSQL, de
 
 // DeleteInstanceConfigKey removes the given key from the config of the instance
 // with the given ID.
-func (c *ClusterTx) DeleteInstanceConfigKey(id int64, key string) error {
+func (c *ClusterTx) DeleteInstanceConfigKey(ctx context.Context, id int64, key string) error {
 	q := "DELETE FROM instances_config WHERE key=? AND instance_id=?"
-	_, err := c.tx.Exec(q, key, id)
+	_, err := c.tx.ExecContext(ctx, q, key, id)
 	return err
 }
 
@@ -949,7 +865,7 @@ SELECT storage_pools.name FROM storage_pools
    AND storage_volumes_all.type IN (?,?)
    AND storage_volumes_all.project_id = instances.project_id
    AND (storage_volumes_all.node_id=? OR storage_volumes_all.node_id IS NULL AND storage_pools.driver IN %s)`, query.Params(len(remoteDrivers)))
-	inargs := []any{projectName, instanceName, StoragePoolVolumeTypeContainer, StoragePoolVolumeTypeVM, c.nodeID}
+	inargs := []any{projectName, instanceName, cluster.StoragePoolVolumeTypeContainer, cluster.StoragePoolVolumeTypeVM, c.nodeID}
 	outargs := []any{&poolName}
 
 	for _, driver := range remoteDrivers {
@@ -969,34 +885,25 @@ SELECT storage_pools.name FROM storage_pools
 }
 
 // DeleteInstance removes the instance with the given name from the database.
-func (c *Cluster) DeleteInstance(project, name string) error {
+func (c *ClusterTx) DeleteInstance(ctx context.Context, project, name string) error {
 	if strings.Contains(name, shared.SnapshotDelimiter) {
 		parts := strings.SplitN(name, shared.SnapshotDelimiter, 2)
-		return c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-			return cluster.DeleteInstanceSnapshot(ctx, tx.tx, project, parts[0], parts[1])
-		})
+		return cluster.DeleteInstanceSnapshot(ctx, c.tx, project, parts[0], parts[1])
 	}
 
-	return c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return cluster.DeleteInstance(ctx, tx.tx, project, name)
-	})
+	return cluster.DeleteInstance(ctx, c.tx, project, name)
 }
 
 // GetInstanceProjectAndName returns the project and the name of the instance
 // with the given ID.
-func (c *Cluster) GetInstanceProjectAndName(id int) (string, string, error) {
-	var project string
-	var name string
+func (c *ClusterTx) GetInstanceProjectAndName(ctx context.Context, id int) (project string, name string, err error) {
 	q := `
 SELECT projects.name, instances.name
   FROM instances
   JOIN projects ON projects.id = instances.project_id
 WHERE instances.id=?
 `
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return tx.tx.QueryRowContext(ctx, q, id).Scan(&project, &name)
-	})
-
+	err = c.tx.QueryRowContext(ctx, q, id).Scan(&project, &name)
 	if err == sql.ErrNoRows {
 		return "", "", api.StatusErrorf(http.StatusNotFound, "Instance not found")
 	}
@@ -1005,24 +912,19 @@ WHERE instances.id=?
 }
 
 // GetInstanceID returns the ID of the instance with the given name.
-func (c *Cluster) GetInstanceID(project, name string) (int, error) {
-	var id int64
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		var err error
-		id, err = cluster.GetInstanceID(ctx, tx.tx, project, name)
-		return err
-	})
+func (c *ClusterTx) GetInstanceID(ctx context.Context, project, name string) (int, error) {
+	id, err := cluster.GetInstanceID(ctx, c.tx, project, name)
+
 	return int(id), err
 }
 
 // GetInstanceConfig returns the value of the given key in the configuration
 // of the instance with the given ID.
-func (c *Cluster) GetInstanceConfig(id int, key string) (string, error) {
+func (c *ClusterTx) GetInstanceConfig(ctx context.Context, id int, key string) (string, error) {
 	q := "SELECT value FROM instances_config WHERE instance_id=? AND key=?"
 	value := ""
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return tx.tx.QueryRowContext(ctx, q, id, key).Scan(&value)
-	})
+
+	err := c.tx.QueryRowContext(ctx, q, id, key).Scan(&value)
 	if err == sql.ErrNoRows {
 		return "", api.StatusErrorf(http.StatusNotFound, "Instance config not found")
 	}
@@ -1030,39 +932,38 @@ func (c *Cluster) GetInstanceConfig(id int, key string) (string, error) {
 	return value, err
 }
 
-// DeleteInstanceConfigKey removes the given key from the config of the instance
-// with the given ID.
-func (c *Cluster) DeleteInstanceConfigKey(id int, key string) error {
-	return c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return tx.DeleteInstanceConfigKey(int64(id), key)
-	})
-}
-
 // UpdateInstanceStatefulFlag toggles the stateful flag of the instance with
 // the given ID.
-func (c *Cluster) UpdateInstanceStatefulFlag(id int, stateful bool) error {
+func (c *ClusterTx) UpdateInstanceStatefulFlag(ctx context.Context, id int, stateful bool) error {
 	statefulInt := 0
 	if stateful {
 		statefulInt = 1
 	}
 
-	return c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		_, err := tx.tx.Exec("UPDATE instances SET stateful=? WHERE id=?", statefulInt, id)
-		return err
-	})
+	_, err := c.tx.ExecContext(ctx, "UPDATE instances SET stateful=? WHERE id=?", statefulInt, id)
+	if err != nil {
+		return fmt.Errorf("Failed updating instance stateful flag: %w", err)
+	}
+
+	return nil
 }
 
 // UpdateInstanceSnapshotCreationDate updates the creation_date field of the instance snapshot with ID.
-func (c *Cluster) UpdateInstanceSnapshotCreationDate(instanceID int, date time.Time) error {
+func (c *ClusterTx) UpdateInstanceSnapshotCreationDate(ctx context.Context, instanceID int, date time.Time) error {
 	stmt := `UPDATE instances_snapshots SET creation_date=? WHERE id=?`
-	err := exec(c, stmt, date, instanceID)
-	return err
+
+	_, err := c.tx.ExecContext(ctx, stmt, date, instanceID)
+	if err != nil {
+		return fmt.Errorf("Failed updating instance snapshot creation date: %w", err)
+	}
+
+	return nil
 }
 
 // GetInstanceSnapshotsNames returns the names of all snapshots of the instance
 // in the given project with the given name.
 // Returns snapshots slice ordered by when they were created, oldest first.
-func (c *Cluster) GetInstanceSnapshotsNames(project, name string) ([]string, error) {
+func (c *ClusterTx) GetInstanceSnapshotsNames(ctx context.Context, project, name string) ([]string, error) {
 	result := []string{}
 
 	q := `
@@ -1075,7 +976,8 @@ ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 `
 	inargs := []any{project, name}
 	outfmt := []any{name}
-	dbResults, err := queryScan(c, q, inargs, outfmt)
+
+	dbResults, err := queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return result, err
 	}
@@ -1089,7 +991,7 @@ ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 
 // GetNextInstanceSnapshotIndex returns the index that the next snapshot of the
 // instance with the given name and pattern should have.
-func (c *Cluster) GetNextInstanceSnapshotIndex(project string, name string, pattern string) int {
+func (c *ClusterTx) GetNextInstanceSnapshotIndex(ctx context.Context, project string, name string, pattern string) int {
 	q := `
 SELECT instances_snapshots.name
   FROM instances_snapshots
@@ -1101,7 +1003,8 @@ ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 	var numstr string
 	inargs := []any{project, name}
 	outfmt := []any{numstr}
-	results, err := queryScan(c, q, inargs, outfmt)
+
+	results, err := queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return 0
 	}
@@ -1109,7 +1012,11 @@ ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 	max := 0
 
 	for _, r := range results {
-		snapOnlyName := r[0].(string)
+		snapOnlyName, ok := r[0].(string)
+		if !ok {
+			continue
+		}
+
 		fields := strings.SplitN(pattern, "%d", 2)
 
 		var num int
@@ -1126,28 +1033,35 @@ ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 	return max
 }
 
-// GetInstancePool returns the storage pool of a given instance.
-//
-// This is a non-transactional variant of ClusterTx.GetInstancePool().
-func (c *Cluster) GetInstancePool(project, instanceName string) (string, error) {
-	var poolName string
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		var err error
-		poolName, err = tx.GetInstancePool(ctx, project, instanceName)
-		return err
-	})
-	return poolName, err
+// DeleteReadyStateFromLocalInstances deletes the volatile.last_state.ready config key
+// from all local instances.
+func (c *ClusterTx) DeleteReadyStateFromLocalInstances(ctx context.Context) error {
+	nodeID := c.GetNodeID()
+
+	_, err := c.tx.ExecContext(ctx, `
+DELETE FROM instances_config
+WHERE instances_config.id IN (
+	SELECT instances_config.id FROM instances_config
+	JOIN instances ON instances_config.instance_id=instances.id
+	JOIN nodes ON instances.node_id=nodes.id
+	WHERE key="volatile.last_state.ready" AND nodes.id=?
+)`, nodeID)
+	if err != nil {
+		return fmt.Errorf("Failed deleting ready state from local instances: %w", err)
+	}
+
+	return nil
 }
 
 // CreateInstanceConfig inserts a new config for the instance with the given ID.
-func CreateInstanceConfig(tx *sql.Tx, id int, config map[string]string) error {
+func CreateInstanceConfig(ctx context.Context, tx *sql.Tx, id int, config map[string]string) error {
 	sql := "INSERT INTO instances_config (instance_id, key, value) values (?, ?, ?)"
 	for k, v := range config {
 		if v == "" {
 			continue
 		}
 
-		_, err := tx.Exec(sql, id, k, v)
+		_, err := tx.ExecContext(ctx, sql, id, k, v)
 		if err != nil {
 			return fmt.Errorf("Error adding configuration item %q = %q to instance %d: %w", k, v, id, err)
 		}
