@@ -9,13 +9,13 @@ import (
 	"fmt"
 	"net/http"
 
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // GetNetworkACLs returns the names of existing Network ACLs.
-func (c *Cluster) GetNetworkACLs(project string) ([]string, error) {
+func (c *ClusterTx) GetNetworkACLs(ctx context.Context, project string) ([]string, error) {
 	q := `SELECT name FROM networks_acls
 		WHERE project_id = (SELECT id FROM projects WHERE name = ? LIMIT 1)
 		ORDER BY id
@@ -23,20 +23,18 @@ func (c *Cluster) GetNetworkACLs(project string) ([]string, error) {
 
 	var aclNames []string
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return query.Scan(ctx, tx.Tx(), q, func(scan func(dest ...any) error) error {
-			var aclName string
+	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
+		var aclName string
 
-			err := scan(&aclName)
-			if err != nil {
-				return err
-			}
+		err := scan(&aclName)
+		if err != nil {
+			return err
+		}
 
-			aclNames = append(aclNames, aclName)
+		aclNames = append(aclNames, aclName)
 
-			return nil
-		}, project)
-	})
+		return nil
+	}, project)
 	if err != nil {
 		return nil, err
 	}
@@ -45,7 +43,7 @@ func (c *Cluster) GetNetworkACLs(project string) ([]string, error) {
 }
 
 // GetNetworkACLIDsByNames returns a map of names to IDs of existing Network ACLs.
-func (c *Cluster) GetNetworkACLIDsByNames(project string) (map[string]int64, error) {
+func (c *ClusterTx) GetNetworkACLIDsByNames(ctx context.Context, project string) (map[string]int64, error) {
 	q := `SELECT id, name FROM networks_acls
 		WHERE project_id = (SELECT id FROM projects WHERE name = ? LIMIT 1)
 		ORDER BY id
@@ -53,21 +51,19 @@ func (c *Cluster) GetNetworkACLIDsByNames(project string) (map[string]int64, err
 
 	acls := make(map[string]int64)
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return query.Scan(ctx, tx.Tx(), q, func(scan func(dest ...any) error) error {
-			var aclID int64
-			var aclName string
+	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
+		var aclID int64
+		var aclName string
 
-			err := scan(&aclID, &aclName)
-			if err != nil {
-				return err
-			}
+		err := scan(&aclID, &aclName)
+		if err != nil {
+			return err
+		}
 
-			acls[aclName] = aclID
+		acls[aclName] = aclID
 
-			return nil
-		}, project)
-	})
+		return nil
+	}, project)
 	if err != nil {
 		return nil, err
 	}
@@ -76,15 +72,13 @@ func (c *Cluster) GetNetworkACLIDsByNames(project string) (map[string]int64, err
 }
 
 // GetNetworkACL returns the Network ACL with the given name in the given project.
-func (c *Cluster) GetNetworkACL(projectName string, name string) (int64, *api.NetworkACL, error) {
-	var id int64 = int64(-1)
+func (c *ClusterTx) GetNetworkACL(ctx context.Context, projectName string, name string) (int64, *api.NetworkACL, error) {
+	var id = int64(-1)
 	var ingressJSON string
 	var egressJSON string
 
 	acl := api.NetworkACL{
-		NetworkACLPost: api.NetworkACLPost{
-			Name: name,
-		},
+		Name: name,
 	}
 
 	q := `
@@ -94,19 +88,7 @@ func (c *Cluster) GetNetworkACL(projectName string, name string) (int64, *api.Ne
 		LIMIT 1
 	`
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		err := tx.tx.QueryRowContext(ctx, q, projectName, name).Scan(&id, &acl.Description, &ingressJSON, &egressJSON)
-		if err != nil {
-			return err
-		}
-
-		err = networkACLConfig(ctx, tx, id, &acl)
-		if err != nil {
-			return fmt.Errorf("Failed loading config: %w", err)
-		}
-
-		return nil
-	})
+	err := c.tx.QueryRowContext(ctx, q, projectName, name).Scan(&id, &acl.Description, &ingressJSON, &egressJSON)
 	if err != nil {
 		if err == sql.ErrNoRows {
 			return -1, nil, api.StatusErrorf(http.StatusNotFound, "Network ACL not found")
@@ -115,6 +97,15 @@ func (c *Cluster) GetNetworkACL(projectName string, name string) (int64, *api.Ne
 		return -1, nil, err
 	}
 
+	err = networkACLConfig(ctx, c, id, &acl)
+	if err != nil {
+		if err == sql.ErrNoRows {
+			return -1, nil, api.StatusErrorf(http.StatusNotFound, "Network ACL not found")
+		}
+
+		return -1, nil, fmt.Errorf("Failed loading config: %w", err)
+	}
+
 	acl.Ingress = []api.NetworkACLRule{}
 	if ingressJSON != "" {
 		err = json.Unmarshal([]byte(ingressJSON), &acl.Ingress)
@@ -135,15 +126,10 @@ func (c *Cluster) GetNetworkACL(projectName string, name string) (int64, *api.Ne
 }
 
 // GetNetworkACLNameAndProjectWithID returns the network ACL name and project name for the given ID.
-func (c *Cluster) GetNetworkACLNameAndProjectWithID(networkACLID int) (string, string, error) {
-	var networkACLName string
-	var projectName string
-
+func (c *ClusterTx) GetNetworkACLNameAndProjectWithID(ctx context.Context, networkACLID int) (networkACLName string, projectName string, err error) {
 	q := `SELECT networks_acls.name, projects.name FROM networks_acls JOIN projects ON projects.id=networks.project_id WHERE networks_acls.id=?`
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return tx.tx.QueryRowContext(ctx, q, networkACLID).Scan(&networkACLName, &projectName)
-	})
+	err = c.tx.QueryRowContext(ctx, q, networkACLID).Scan(&networkACLName, &projectName)
 	if err != nil {
 		if err == sql.ErrNoRows {
 			return "", "", api.StatusErrorf(http.StatusNotFound, "Network ACL not found")
@@ -184,8 +170,7 @@ func networkACLConfig(ctx context.Context, tx *ClusterTx, id int64, acl *api.Net
 }
 
 // CreateNetworkACL creates a new Network ACL.
-func (c *Cluster) CreateNetworkACL(projectName string, info *api.NetworkACLsPost) (int64, error) {
-	var id int64
+func (c *ClusterTx) CreateNetworkACL(ctx context.Context, projectName string, info *api.NetworkACLsPost) (int64, error) {
 	var err error
 	var ingressJSON, egressJSON []byte
 
@@ -203,30 +188,23 @@ func (c *Cluster) CreateNetworkACL(projectName string, info *api.NetworkACLsPost
 		}
 	}
 
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		// Insert a new Network ACL record.
-		result, err := tx.tx.Exec(`
+	// Insert a new Network ACL record.
+	result, err := c.tx.ExecContext(ctx, `
 			INSERT INTO networks_acls (project_id, name, description, ingress, egress)
 			VALUES ((SELECT id FROM projects WHERE name = ? LIMIT 1), ?, ?, ?, ?)
 		`, projectName, info.Name, info.Description, string(ingressJSON), string(egressJSON))
-		if err != nil {
-			return err
-		}
-
-		id, err := result.LastInsertId()
-		if err != nil {
-			return err
-		}
-
-		err = networkACLConfigAdd(tx.tx, id, info.Config)
-		if err != nil {
-			return err
-		}
-
-		return nil
-	})
 	if err != nil {
-		id = -1
+		return -1, err
+	}
+
+	id, err := result.LastInsertId()
+	if err != nil {
+		return -1, err
+	}
+
+	err = networkACLConfigAdd(c.tx, id, info.Config)
+	if err != nil {
+		return -1, err
 	}
 
 	return id, err
@@ -257,7 +235,7 @@ func networkACLConfigAdd(tx *sql.Tx, id int64, config map[string]string) error {
 }
 
 // UpdateNetworkACL updates the Network ACL with the given ID.
-func (c *Cluster) UpdateNetworkACL(id int64, config *api.NetworkACLPut) error {
+func (c *ClusterTx) UpdateNetworkACL(ctx context.Context, id int64, config api.NetworkACLPut) error {
 	var err error
 	var ingressJSON, egressJSON []byte
 
@@ -275,44 +253,40 @@ func (c *Cluster) UpdateNetworkACL(id int64, config *api.NetworkACLPut) error {
 		}
 	}
 
-	return c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		_, err := tx.tx.Exec(`
+	_, err = c.tx.ExecContext(ctx, `
 			UPDATE networks_acls
 			SET description=?, ingress = ?, egress = ?
 			WHERE id=?
 		`, config.Description, ingressJSON, egressJSON, id)
-		if err != nil {
-			return err
-		}
+	if err != nil {
+		return err
+	}
 
-		_, err = tx.tx.Exec("DELETE FROM networks_acls_config WHERE network_acl_id=?", id)
-		if err != nil {
-			return err
-		}
+	_, err = c.tx.ExecContext(ctx, "DELETE FROM networks_acls_config WHERE network_acl_id=?", id)
+	if err != nil {
+		return err
+	}
 
-		err = networkACLConfigAdd(tx.tx, id, config.Config)
-		if err != nil {
-			return err
-		}
+	err = networkACLConfigAdd(c.tx, id, config.Config)
+	if err != nil {
+		return err
+	}
 
-		return nil
-	})
+	return nil
 }
 
 // RenameNetworkACL renames a Network ACL.
-func (c *Cluster) RenameNetworkACL(id int64, newName string) error {
-	return c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		_, err := tx.tx.Exec("UPDATE networks_acls SET name=? WHERE id=?", newName, id)
-		return err
-	})
+func (c *ClusterTx) RenameNetworkACL(ctx context.Context, id int64, newName string) error {
+	_, err := c.tx.ExecContext(ctx, "UPDATE networks_acls SET name=? WHERE id=?", newName, id)
+
+	return err
 }
 
 // DeleteNetworkACL deletes the Network ACL.
-func (c *Cluster) DeleteNetworkACL(id int64) error {
-	return c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		_, err := tx.tx.Exec("DELETE FROM networks_acls WHERE id=?", id)
-		return err
-	})
+func (c *ClusterTx) DeleteNetworkACL(ctx context.Context, id int64) error {
+	_, err := c.tx.ExecContext(ctx, "DELETE FROM networks_acls WHERE id=?", id)
+
+	return err
 }
 
 // GetNetworkACLURIs returns the URIs for the network ACLs with the given project.
