@@ -9,9 +9,9 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // The GetNetworksLocalConfigs method returns only node-specific config values.
@@ -19,9 +19,13 @@ func TestGetNetworksLocalConfigs(t *testing.T) {
 	cluster, cleanup := db.NewTestCluster(t)
 	defer cleanup()
 
-	_, err := cluster.CreateNetwork(project.Default, "lxdbr0", "", db.NetworkTypeBridge, map[string]string{
-		"dns.mode":                   "none",
-		"bridge.external_interfaces": "vlan0",
+	err := cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		_, err := tx.CreateNetwork(ctx, api.ProjectDefaultName, "lxdbr0", "", db.NetworkTypeBridge, map[string]string{
+			"dns.mode":                   "none",
+			"bridge.external_interfaces": "vlan0",
+		})
+
+		return err
 	})
 	require.NoError(t, err)
 
@@ -49,15 +53,15 @@ func TestCreatePendingNetwork(t *testing.T) {
 	require.NoError(t, err)
 
 	config := map[string]string{"bridge.external_interfaces": "foo"}
-	err = tx.CreatePendingNetwork(context.Background(), "buzz", project.Default, "network1", db.NetworkTypeBridge, config)
+	err = tx.CreatePendingNetwork(context.Background(), "buzz", api.ProjectDefaultName, "network1", db.NetworkTypeBridge, config)
 	require.NoError(t, err)
 
-	networkID, err := tx.GetNetworkID(context.Background(), project.Default, "network1")
+	networkID, err := tx.GetNetworkID(context.Background(), api.ProjectDefaultName, "network1")
 	require.NoError(t, err)
 	assert.True(t, networkID > 0)
 
 	config = map[string]string{"bridge.external_interfaces": "bar"}
-	err = tx.CreatePendingNetwork(context.Background(), "rusp", project.Default, "network1", db.NetworkTypeBridge, config)
+	err = tx.CreatePendingNetwork(context.Background(), "rusp", api.ProjectDefaultName, "network1", db.NetworkTypeBridge, config)
 	require.NoError(t, err)
 
 	// The initial node (whose name is 'none' by default) is missing.
@@ -65,7 +69,7 @@ func TestCreatePendingNetwork(t *testing.T) {
 	require.EqualError(t, err, "Network not defined on nodes: none")
 
 	config = map[string]string{"bridge.external_interfaces": "egg"}
-	err = tx.CreatePendingNetwork(context.Background(), "none", project.Default, "network1", db.NetworkTypeBridge, config)
+	err = tx.CreatePendingNetwork(context.Background(), "none", api.ProjectDefaultName, "network1", db.NetworkTypeBridge, config)
 	require.NoError(t, err)
 
 	// Now the storage is defined on all nodes.
@@ -86,10 +90,10 @@ func TestNetworksCreatePending_AlreadyDefined(t *testing.T) {
 	_, err := tx.CreateNode("buzz", "1.2.3.4:666")
 	require.NoError(t, err)
 
-	err = tx.CreatePendingNetwork(context.Background(), "buzz", project.Default, "network1", db.NetworkTypeBridge, map[string]string{})
+	err = tx.CreatePendingNetwork(context.Background(), "buzz", api.ProjectDefaultName, "network1", db.NetworkTypeBridge, map[string]string{})
 	require.NoError(t, err)
 
-	err = tx.CreatePendingNetwork(context.Background(), "buzz", project.Default, "network1", db.NetworkTypeBridge, map[string]string{})
+	err = tx.CreatePendingNetwork(context.Background(), "buzz", api.ProjectDefaultName, "network1", db.NetworkTypeBridge, map[string]string{})
 	require.Equal(t, db.ErrAlreadyDefined, err)
 }
 
@@ -98,6 +102,6 @@ func TestNetworksCreatePending_NonExistingNode(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
 
-	err := tx.CreatePendingNetwork(context.Background(), "buzz", project.Default, "network1", db.NetworkTypeBridge, map[string]string{})
+	err := tx.CreatePendingNetwork(context.Background(), "buzz", api.ProjectDefaultName, "network1", db.NetworkTypeBridge, map[string]string{})
 	require.True(t, response.IsNotFoundError(err))
 }
