@@ -11,14 +11,14 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/operationtype"
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/operationtype"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // ClusterRole represents the role of a member in a cluster.
@@ -681,7 +681,7 @@ func (c *ClusterTx) UpdateNodeClusterGroups(ctx context.Context, id int64, group
 
 	// Check if node already belongs to the given groups.
 	for _, newGroup := range groups {
-		if shared.StringInSlice(newGroup, oldGroups) {
+		if shared.ValueInSlice(newGroup, oldGroups) {
 			// Node already belongs to this group.
 			skipGroups = append(skipGroups, newGroup)
 			continue
@@ -695,7 +695,7 @@ func (c *ClusterTx) UpdateNodeClusterGroups(ctx context.Context, id int64, group
 	}
 
 	for _, oldGroup := range oldGroups {
-		if shared.StringInSlice(oldGroup, skipGroups) {
+		if shared.ValueInSlice(oldGroup, skipGroups) {
 			continue
 		}
 
@@ -905,18 +905,18 @@ func (c *ClusterTx) SetNodeHeartbeat(address string, heartbeat time.Time) error
 }
 
 // NodeIsEmpty returns an empty string if the node with the given ID has no
-// containers or images associated with it. Otherwise, it returns a message
+// instances or images associated with it. Otherwise, it returns a message
 // say what's left.
 func (c *ClusterTx) NodeIsEmpty(ctx context.Context, id int64) (string, error) {
 	// Check if the node has any instances.
-	containers, err := query.SelectStrings(ctx, c.tx, "SELECT name FROM instances WHERE node_id=?", id)
+	instances, err := query.SelectStrings(ctx, c.tx, "SELECT name FROM instances WHERE node_id=?", id)
 	if err != nil {
 		return "", fmt.Errorf("Failed to get instances for node %d: %w", id, err)
 	}
 
-	if len(containers) > 0 {
+	if len(instances) > 0 {
 		message := fmt.Sprintf(
-			"Node still has the following containers: %s", strings.Join(containers, ", "))
+			"Node still has the following instances: %s", strings.Join(instances, ", "))
 		return message, nil
 	}
 
@@ -972,7 +972,7 @@ SELECT storage_volumes.name
   JOIN storage_pools ON storage_volumes.storage_pool_id=storage_pools.id
   WHERE storage_volumes.node_id=? AND storage_volumes.type=? AND storage_pools.driver NOT IN ('ceph', 'cephfs')
 `
-	volumes, err := query.SelectStrings(ctx, c.tx, sql, id, StoragePoolVolumeTypeCustom)
+	volumes, err := query.SelectStrings(ctx, c.tx, sql, id, cluster.StoragePoolVolumeTypeCustom)
 	if err != nil {
 		return "", fmt.Errorf("Failed to get custom volumes for node %d: %w", id, err)
 	}
@@ -1065,12 +1065,12 @@ func (c *ClusterTx) GetCandidateMembers(ctx context.Context, allMembers []NodeIn
 		}
 
 		// Skip group-only members if targeted cluster group doesn't match.
-		if member.Config["scheduler.instance"] == "group" && !shared.StringInSlice(targetClusterGroup, member.Groups) {
+		if member.Config["scheduler.instance"] == "group" && !shared.ValueInSlice(targetClusterGroup, member.Groups) {
 			continue
 		}
 
 		// Skip if a group is requested and member isn't part of it.
-		if targetClusterGroup != "" && !shared.StringInSlice(targetClusterGroup, member.Groups) {
+		if targetClusterGroup != "" && !shared.ValueInSlice(targetClusterGroup, member.Groups) {
 			continue
 		}
 
@@ -1078,7 +1078,7 @@ func (c *ClusterTx) GetCandidateMembers(ctx context.Context, allMembers []NodeIn
 		if allowedClusterGroups != nil {
 			found := false
 			for _, allowedClusterGroup := range allowedClusterGroups {
-				if shared.StringInSlice(allowedClusterGroup, member.Groups) {
+				if shared.ValueInSlice(allowedClusterGroup, member.Groups) {
 					found = true
 					break
 				}
@@ -1099,7 +1099,7 @@ func (c *ClusterTx) GetCandidateMembers(ctx context.Context, allMembers []NodeIn
 
 			supportedArchitectures := append([]int{member.Architecture}, personalities...)
 			for _, supportedArchitecture := range supportedArchitectures {
-				if shared.IntInSlice(supportedArchitecture, targetArchitectures) {
+				if shared.ValueInSlice(supportedArchitecture, targetArchitectures) {
 					candidateMembers = append(candidateMembers, member)
 					break
 				}
@@ -1139,6 +1139,10 @@ func (c *ClusterTx) GetNodeWithLeastInstances(ctx context.Context, members []Nod
 		}
 	}
 
+	if member == nil {
+		return nil, api.StatusErrorf(http.StatusNotFound, "No suitable cluster member could be found")
+	}
+
 	return member, nil
 }
 
