@@ -9,11 +9,11 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/lxd/db/schema"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/lxd/db/schema"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/units"
 )
 
 // Schema for the local database.
@@ -98,6 +98,7 @@ var updates = map[int]schema.Update{
 	40: updateFromV39,
 	41: updateFromV40,
 	42: updateFromV41,
+	43: updateFromV42,
 }
 
 // UpdateFromPreClustering is the last schema version where clustering support
@@ -106,6 +107,23 @@ const UpdateFromPreClustering = 36
 
 // Schema updates begin here
 
+// updateFromV42 ensures key and value fields in config table are TEXT NOT NULL.
+func updateFromV42(ctx context.Context, tx *sql.Tx) error {
+	stmt := `
+CREATE TABLE "config_new" (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    key TEXT NOT NULL,
+    value TEXT NOT NULL,
+    UNIQUE (key)
+);
+INSERT INTO "config_new" SELECT * FROM "config";
+DROP TABLE "config";
+ALTER TABLE "config_new" RENAME TO "config";
+`
+	_, err := tx.Exec(stmt)
+	return err
+}
+
 func updateFromV41(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 	ALTER TABLE raft_nodes ADD COLUMN name TEXT NOT NULL default "";
