@@ -10,12 +10,12 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/operationtype"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/operationtype"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // Add a new raft node.
@@ -220,8 +220,8 @@ func TestSetNodeHeartbeat(t *testing.T) {
 	assert.True(t, node.IsOffline(20*time.Second))
 }
 
-// A node is considered empty only if it has no containers.
-func TestNodeIsEmpty_Containers(t *testing.T) {
+// A node is considered empty only if it has no instances.
+func TestNodeIsEmpty_Instances(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
 
@@ -239,7 +239,7 @@ INSERT INTO instances (id, node_id, name, architecture, type, project_id, descri
 
 	message, err = tx.NodeIsEmpty(context.Background(), id)
 	require.NoError(t, err)
-	assert.Equal(t, "Node still has the following containers: foo", message)
+	assert.Equal(t, "Node still has the following instances: foo", message)
 
 	err = tx.ClearNode(context.Background(), id)
 	require.NoError(t, err)
@@ -295,7 +295,7 @@ INSERT INTO storage_pools (id, name, driver, description) VALUES (1, 'local', 'z
 
 	_, err = tx.Tx().Exec(`
 INSERT INTO storage_volumes(name, storage_pool_id, node_id, type, project_id, description)
-  VALUES ('data', 1, ?, ?, 1, '')`, id, db.StoragePoolVolumeTypeCustom)
+  VALUES ('data', 1, ?, ?, 1, '')`, id, cluster.StoragePoolVolumeTypeCustom)
 	require.NoError(t, err)
 
 	message, err := tx.NodeIsEmpty(context.Background(), id)
@@ -304,7 +304,7 @@ INSERT INTO storage_volumes(name, storage_pool_id, node_id, type, project_id, de
 }
 
 // If there are 2 online nodes, return the address of the one with the least
-// number of containers.
+// number of instances.
 func TestGetNodeWithLeastInstances(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
@@ -312,7 +312,7 @@ func TestGetNodeWithLeastInstances(t *testing.T) {
 	_, err := tx.CreateNode("buzz", "1.2.3.4:666")
 	require.NoError(t, err)
 
-	// Add a container to the default node (ID 1)
+	// Add an instance to the default node (ID 1)
 	_, err = tx.Tx().Exec(`
 INSERT INTO instances (id, node_id, name, architecture, type, project_id, description) VALUES (1, 1, 'foo', 1, 1, 1, '')
 `)
@@ -331,7 +331,7 @@ INSERT INTO instances (id, node_id, name, architecture, type, project_id, descri
 }
 
 // If there are nodes, and one of them is offline, return the name of the
-// online node, even if the offline one has more containers.
+// online node, even if the offline one has more instances.
 func TestGetNodeWithLeastInstances_OfflineNode(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
@@ -339,7 +339,7 @@ func TestGetNodeWithLeastInstances_OfflineNode(t *testing.T) {
 	id, err := tx.CreateNode("buzz", "1.2.3.4:666")
 	require.NoError(t, err)
 
-	// Add a container to the newly created node.
+	// Add an instance to the newly created node.
 	_, err = tx.Tx().Exec(`
 INSERT INTO instances (id, node_id, name, architecture, type, project_id, description) VALUES (1, ?, 'foo', 1, 1, 1, '')
 `, id)
@@ -361,8 +361,8 @@ INSERT INTO instances (id, node_id, name, architecture, type, project_id, descri
 	assert.Equal(t, "buzz", member.Name)
 }
 
-// If there are 2 online nodes, and a container is pending on one of them,
-// return the address of the other one number of containers.
+// If there are 2 online nodes, and an instance is pending on one of them,
+// return the address of the other one number of instances.
 func TestGetNodeWithLeastInstances_Pending(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
@@ -370,7 +370,7 @@ func TestGetNodeWithLeastInstances_Pending(t *testing.T) {
 	_, err := tx.CreateNode("buzz", "1.2.3.4:666")
 	require.NoError(t, err)
 
-	// Add a pending container to the default node (ID 1)
+	// Add a pending instance to the default node (ID 1)
 	_, err = tx.Tx().Exec(`
 INSERT INTO operations (id, uuid, node_id, type, project_id) VALUES (1, 'abc', 1, ?, 1)
 `, operationtype.InstanceCreate)
@@ -405,7 +405,7 @@ func TestGetNodeWithLeastInstances_Architecture(t *testing.T) {
 	_, err = tx.CreateNodeWithArch("buzz", "1.2.3.4:666", testArch)
 	require.NoError(t, err)
 
-	// Add a container to the default node (ID 1)
+	// Add an instance to the default node (ID 1)
 	_, err = tx.Tx().Exec(`
 INSERT INTO instances (id, node_id, name, architecture, type, project_id, description) VALUES (1, 1, 'foo', 1, 1, 1, '')
 `)
@@ -418,7 +418,7 @@ INSERT INTO instances (id, node_id, name, architecture, type, project_id, descri
 	require.NoError(t, err)
 	require.Len(t, members, 1)
 
-	// The local member is returned despite it has more containers.
+	// The local member is returned despite it has more instances.
 	member, err := tx.GetNodeWithLeastInstances(context.Background(), members)
 	require.NoError(t, err)
 	assert.Equal(t, "none", member.Name)
@@ -467,7 +467,7 @@ func TestGetNodeWithLeastInstances_DefaultArch(t *testing.T) {
 	id, err := tx.CreateNodeWithArch("buzz", "1.2.3.4:666", testArch)
 	require.NoError(t, err)
 
-	// Add a container to the newly created node.
+	// Add an instance to the newly created node.
 	_, err = tx.Tx().Exec(`
 INSERT INTO instances (id, node_id, name, architecture, type, project_id, description) VALUES (1, ?, 'foo', 1, 1, 1, '')
 `, id)
