@@ -11,34 +11,52 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // GetStoragePoolVolumesWithType return a list of all volumes of the given type.
-func (c *ClusterTx) GetStoragePoolVolumesWithType(ctx context.Context, volumeType int) ([]StorageVolumeArgs, error) {
-	stmt := `
-SELECT storage_volumes.id, storage_volumes.name, storage_volumes.description, storage_pools.name, projects.name, IFNULL(storage_volumes.node_id, -1)
+// If memberSpecific is true, then the search is restricted to volumes that belong to this member or belong to
+// all members.
+func (c *ClusterTx) GetStoragePoolVolumesWithType(ctx context.Context, volumeType int, memberSpecific bool) ([]StorageVolumeArgs, error) {
+	var q strings.Builder
+	q.WriteString(`
+SELECT
+	storage_volumes.id,
+	storage_volumes.name,
+	storage_volumes.description,
+	storage_volumes.creation_date,
+	storage_pools.name,
+	projects.name,
+	IFNULL(storage_volumes.node_id, -1)
 FROM storage_volumes
 JOIN storage_pools ON storage_pools.id = storage_volumes.storage_pool_id
 JOIN projects ON projects.id = storage_volumes.project_id
 WHERE storage_volumes.type = ?
-`
+`)
+
+	args := []any{volumeType}
+
+	if memberSpecific {
+		q.WriteString("AND (storage_volumes.node_id = ? OR storage_volumes.node_id IS NULL) ")
+		args = append(args, c.nodeID)
+	}
 
 	result := []StorageVolumeArgs{}
-	err := query.Scan(ctx, c.Tx(), stmt, func(scan func(dest ...any) error) error {
+	err := query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
 		entry := StorageVolumeArgs{}
 
-		err := scan(&entry.ID, &entry.Name, &entry.Description, &entry.PoolName, &entry.ProjectName, &entry.NodeID)
+		err := scan(&entry.ID, &entry.Name, &entry.Description, &entry.CreationDate, &entry.PoolName, &entry.ProjectName, &entry.NodeID)
 		if err != nil {
 			return err
 		}
 
 		result = append(result, entry)
 		return nil
-	}, volumeType)
+	}, args...)
 	if err != nil {
 		return nil, err
 	}
@@ -54,21 +72,27 @@ WHERE storage_volumes.type = ?
 }
 
 // GetStoragePoolVolumeWithID returns the volume with the given ID.
-func (c *Cluster) GetStoragePoolVolumeWithID(volumeID int) (StorageVolumeArgs, error) {
+func (c *ClusterTx) GetStoragePoolVolumeWithID(ctx context.Context, volumeID int) (StorageVolumeArgs, error) {
 	var response StorageVolumeArgs
 
 	stmt := `
-SELECT storage_volumes.id, storage_volumes.name, storage_volumes.description, storage_pools.name, storage_pools.type, projects.name
+SELECT
+	storage_volumes.id,
+	storage_volumes.name,
+	storage_volumes.description,
+	storage_volumes.creation_date,
+	storage_volumes.type,
+	IFNULL(storage_volumes.node_id, -1),
+	storage_pools.name,
+	projects.name
 FROM storage_volumes
 JOIN storage_pools ON storage_pools.id = storage_volumes.storage_pool_id
 JOIN projects ON projects.id = storage_volumes.project_id
+LEFT JOIN nodes ON nodes.id = storage_volumes.node_id
 WHERE storage_volumes.id = ?
 `
 
-	inargs := []any{volumeID}
-	outargs := []any{&response.ID, &response.Name, &response.Description, &response.PoolName, &response.Type, &response.ProjectName}
-
-	err := dbQueryRowScan(c, stmt, inargs, outargs)
+	err := c.tx.QueryRowContext(ctx, stmt, volumeID).Scan(&response.ID, &response.Name, &response.Description, &response.CreationDate, &response.Type, &response.NodeID, &response.PoolName, &response.ProjectName)
 	if err != nil {
 		if err == sql.ErrNoRows {
 			return StorageVolumeArgs{}, api.StatusErrorf(http.StatusNotFound, "Storage pool volume not found")
@@ -77,12 +101,12 @@ WHERE storage_volumes.id = ?
 		return StorageVolumeArgs{}, err
 	}
 
-	response.Config, err = c.storageVolumeConfigGet(response.ID, false)
+	response.Config, err = c.storageVolumeConfigGet(ctx, response.ID, false)
 	if err != nil {
 		return StorageVolumeArgs{}, err
 	}
 
-	response.TypeName = StoragePoolVolumeTypeNames[response.Type]
+	response.TypeName = cluster.StoragePoolVolumeTypeNames[response.Type]
 
 	return response, nil
 }
@@ -106,7 +130,7 @@ type StorageVolume struct {
 // Accepts filters for narrowing down the results returned. If memberSpecific is true, then the search is
 // restricted to volumes that belong to this member or belong to all members.
 func (c *ClusterTx) GetStoragePoolVolumes(ctx context.Context, poolID int64, memberSpecific bool, filters ...StorageVolumeFilter) ([]*StorageVolume, error) {
-	var q *strings.Builder = &strings.Builder{}
+	var q = &strings.Builder{}
 	args := []any{poolID}
 
 	q.WriteString(`
@@ -117,10 +141,13 @@ func (c *ClusterTx) GetStoragePoolVolumes(ctx context.Context, poolID int64, mem
 			IFNULL(nodes.name, "") as location,
 			storage_volumes_all.type,
 			storage_volumes_all.content_type,
-			storage_volumes_all.description
+			storage_volumes_all.description,
+			storage_volumes_all.creation_date,
+			storage_pools.name as pool
 		FROM storage_volumes_all
 		JOIN projects ON projects.id = storage_volumes_all.project_id
 		LEFT JOIN nodes ON nodes.id = storage_volumes_all.node_id
+		JOIN storage_pools ON storage_pools.id = storage_volumes_all.storage_pool_id
 		WHERE storage_volumes_all.storage_pool_id = ?
 	`)
 
@@ -177,11 +204,11 @@ func (c *ClusterTx) GetStoragePoolVolumes(ctx context.Context, poolID int64, mem
 	var volumes []*StorageVolume
 
 	err = query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
-		var volumeType int = int(-1)
-		var contentType int = int(-1)
+		var volumeType = int(-1)
+		var contentType = int(-1)
 		var vol StorageVolume
 
-		err := scan(&vol.Project, &vol.ID, &vol.Name, &vol.Location, &volumeType, &contentType, &vol.Description)
+		err := scan(&vol.Project, &vol.ID, &vol.Name, &vol.Location, &volumeType, &contentType, &vol.Description, &vol.CreatedAt, &vol.Pool)
 		if err != nil {
 			return err
 		}
@@ -239,16 +266,17 @@ func (c *ClusterTx) GetStoragePoolVolume(ctx context.Context, poolID int64, proj
 // GetLocalStoragePoolVolumeSnapshotsWithType get all snapshots of a storage volume
 // attached to a given storage pool of a given volume type, on the local member.
 // Returns snapshots slice ordered by when they were created, oldest first.
-func (c *Cluster) GetLocalStoragePoolVolumeSnapshotsWithType(projectName string, volumeName string, volumeType int, poolID int64) ([]StorageVolumeArgs, error) {
+func (c *ClusterTx) GetLocalStoragePoolVolumeSnapshotsWithType(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64) ([]StorageVolumeArgs, error) {
 	remoteDrivers := StorageRemoteDriverNames()
 
-	// ORDER BY id is important here as the users of this function can expect that the results
-	// will be returned in the order that the snapshots were created. This is specifically used
+	// ORDER BY creation_date and then id is important here as the users of this function can expect that the
+	// results will be returned in the order that the snapshots were created. This is specifically used
 	// during migration to ensure that the storage engines can re-create snapshots using the
 	// correct deltas.
 	queryStr := fmt.Sprintf(`
   SELECT
-    storage_volumes_snapshots.id, storage_volumes_snapshots.name, storage_volumes_snapshots.description, storage_volumes_snapshots.expiry_date,
+    storage_volumes_snapshots.id, storage_volumes_snapshots.name, storage_volumes_snapshots.description,
+    storage_volumes_snapshots.creation_date, storage_volumes_snapshots.expiry_date,
     storage_volumes.content_type
   FROM storage_volumes_snapshots
   JOIN storage_volumes ON storage_volumes_snapshots.storage_volume_id = storage_volumes.id
@@ -259,61 +287,53 @@ func (c *Cluster) GetLocalStoragePoolVolumeSnapshotsWithType(projectName string,
     AND storage_volumes.name=?
     AND projects.name=?
     AND (storage_volumes.node_id=? OR storage_volumes.node_id IS NULL AND storage_pools.driver IN %s)
-  ORDER BY storage_volumes_snapshots.id`, query.Params(len(remoteDrivers)))
+  ORDER BY storage_volumes_snapshots.creation_date, storage_volumes_snapshots.id`, query.Params(len(remoteDrivers)))
 
 	args := []any{poolID, volumeType, volumeName, projectName, c.nodeID}
 	for _, driver := range remoteDrivers {
 		args = append(args, driver)
 	}
 
-	var err error
 	var snapshots []StorageVolumeArgs
 
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		err = query.Scan(ctx, tx.Tx(), queryStr, func(scan func(dest ...any) error) error {
-			var s StorageVolumeArgs
-			var snapName string
-			var expiryDate sql.NullTime
-			var contentType int
+	err := query.Scan(ctx, c.Tx(), queryStr, func(scan func(dest ...any) error) error {
+		var s StorageVolumeArgs
+		var snapName string
+		var expiryDate sql.NullTime
+		var contentType int
 
-			err := scan(&s.ID, &snapName, &s.Description, &expiryDate, &contentType)
-			if err != nil {
-				return err
-			}
-
-			s.Name = volumeName + shared.SnapshotDelimiter + snapName
-			s.PoolID = poolID
-			s.ProjectName = projectName
-			s.Snapshot = true
-			s.ExpiryDate = expiryDate.Time // Convert null to zero.
-
-			s.ContentType, err = storagePoolVolumeContentTypeToName(contentType)
-			if err != nil {
-				return err
-			}
-
-			snapshots = append(snapshots, s)
-
-			return nil
-		}, args...)
+		err := scan(&s.ID, &snapName, &s.Description, &s.CreationDate, &expiryDate, &contentType)
 		if err != nil {
 			return err
 		}
 
-		// Populate config.
-		for i := range snapshots {
-			err := storageVolumeSnapshotConfig(ctx, tx, snapshots[i].ID, &snapshots[i])
-			if err != nil {
-				return err
-			}
+		s.Name = volumeName + shared.SnapshotDelimiter + snapName
+		s.PoolID = poolID
+		s.ProjectName = projectName
+		s.Snapshot = true
+		s.ExpiryDate = expiryDate.Time // Convert null to zero.
+
+		s.ContentType, err = storagePoolVolumeContentTypeToName(contentType)
+		if err != nil {
+			return err
 		}
 
+		snapshots = append(snapshots, s)
+
 		return nil
-	})
+	}, args...)
 	if err != nil {
 		return nil, err
 	}
 
+	// Populate config.
+	for i := range snapshots {
+		err := storageVolumeSnapshotConfig(ctx, c, snapshots[i].ID, &snapshots[i])
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	return snapshots, nil
 }
 
@@ -342,43 +362,35 @@ func storageVolumeSnapshotConfig(ctx context.Context, tx *ClusterTx, volumeSnaps
 }
 
 // UpdateStoragePoolVolume updates the storage volume attached to a given storage pool.
-func (c *Cluster) UpdateStoragePoolVolume(projectName string, volumeName string, volumeType int, poolID int64, volumeDescription string, volumeConfig map[string]string) error {
-	var err error
-
+func (c *ClusterTx) UpdateStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64, volumeDescription string, volumeConfig map[string]string) error {
 	isSnapshot := strings.Contains(volumeName, shared.SnapshotDelimiter)
 
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		volume, err := tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
-		if err != nil {
-			return err
-		}
+	volume, err := c.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
+	if err != nil {
+		return err
+	}
 
-		err = storageVolumeConfigClear(tx.tx, volume.ID, isSnapshot)
-		if err != nil {
-			return err
-		}
+	err = storageVolumeConfigClear(c.tx, volume.ID, isSnapshot)
+	if err != nil {
+		return err
+	}
 
-		err = storageVolumeConfigAdd(tx.tx, volume.ID, volumeConfig, isSnapshot)
-		if err != nil {
-			return err
-		}
+	err = storageVolumeConfigAdd(c.tx, volume.ID, volumeConfig, isSnapshot)
+	if err != nil {
+		return err
+	}
 
-		err = storageVolumeDescriptionUpdate(tx.tx, volume.ID, volumeDescription, isSnapshot)
-		if err != nil {
-			return err
-		}
+	err = storageVolumeDescriptionUpdate(c.tx, volume.ID, volumeDescription, isSnapshot)
+	if err != nil {
+		return err
+	}
 
-		return nil
-	})
-
-	return err
+	return nil
 }
 
 // RemoveStoragePoolVolume deletes the storage volume attached to a given storage
 // pool.
-func (c *Cluster) RemoveStoragePoolVolume(projectName string, volumeName string, volumeType int, poolID int64) error {
-	var err error
-
+func (c *ClusterTx) RemoveStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64) error {
 	isSnapshot := strings.Contains(volumeName, shared.SnapshotDelimiter)
 	var stmt string
 	if isSnapshot {
@@ -387,27 +399,21 @@ func (c *Cluster) RemoveStoragePoolVolume(projectName string, volumeName string,
 		stmt = "DELETE FROM storage_volumes WHERE id=?"
 	}
 
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		volume, err := tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
-		if err != nil {
-			return err
-		}
+	volume, err := c.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
+	if err != nil {
+		return err
+	}
 
-		_, err = tx.tx.Exec(stmt, volume.ID)
-		if err != nil {
-			return err
-		}
+	_, err = c.tx.ExecContext(ctx, stmt, volume.ID)
+	if err != nil {
+		return err
+	}
 
-		return nil
-	})
-
-	return err
+	return nil
 }
 
 // RenameStoragePoolVolume renames the storage volume attached to a given storage pool.
-func (c *Cluster) RenameStoragePoolVolume(projectName string, oldVolumeName string, newVolumeName string, volumeType int, poolID int64) error {
-	var err error
-
+func (c *ClusterTx) RenameStoragePoolVolume(ctx context.Context, projectName string, oldVolumeName string, newVolumeName string, volumeType int, poolID int64) error {
 	isSnapshot := strings.Contains(oldVolumeName, shared.SnapshotDelimiter)
 	var stmt string
 	if isSnapshot {
@@ -418,25 +424,21 @@ func (c *Cluster) RenameStoragePoolVolume(projectName string, oldVolumeName stri
 		stmt = "UPDATE storage_volumes SET name=? WHERE id=?"
 	}
 
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		volume, err := tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, oldVolumeName, true)
-		if err != nil {
-			return err
-		}
+	volume, err := c.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, oldVolumeName, true)
+	if err != nil {
+		return err
+	}
 
-		_, err = tx.tx.Exec(stmt, newVolumeName, volume.ID)
-		if err != nil {
-			return err
-		}
+	_, err = c.tx.ExecContext(ctx, stmt, newVolumeName, volume.ID)
+	if err != nil {
+		return err
+	}
 
-		return nil
-	})
-
-	return err
+	return nil
 }
 
 // CreateStoragePoolVolume creates a new storage volume attached to a given storage pool.
-func (c *Cluster) CreateStoragePoolVolume(projectName string, volumeName string, volumeDescription string, volumeType int, poolID int64, volumeConfig map[string]string, contentType int) (int64, error) {
+func (c *ClusterTx) CreateStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeDescription string, volumeType int, poolID int64, volumeConfig map[string]string, contentType int, creationDate time.Time) (int64, error) {
 	var volumeID int64
 
 	if shared.IsSnapshot(volumeName) {
@@ -445,46 +447,39 @@ func (c *Cluster) CreateStoragePoolVolume(projectName string, volumeName string,
 
 	remoteDrivers := StorageRemoteDriverNames()
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		driver, err := tx.GetStoragePoolDriver(ctx, poolID)
-		if err != nil {
-			return err
-		}
-
-		var result sql.Result
-
-		if shared.StringInSlice(driver, remoteDrivers) {
-			result, err = tx.tx.Exec(`
-INSERT INTO storage_volumes (storage_pool_id, type, name, description, project_id, content_type)
- VALUES (?, ?, ?, ?, (SELECT id FROM projects WHERE name = ?), ?)
-`,
-				poolID, volumeType, volumeName, volumeDescription, projectName, contentType)
-		} else {
-			result, err = tx.tx.Exec(`
-INSERT INTO storage_volumes (storage_pool_id, node_id, type, name, description, project_id, content_type)
- VALUES (?, ?, ?, ?, ?, (SELECT id FROM projects WHERE name = ?), ?)
-`,
-				poolID, c.nodeID, volumeType, volumeName, volumeDescription, projectName, contentType)
-		}
-
-		if err != nil {
-			return err
-		}
-
-		volumeID, err = result.LastInsertId()
-		if err != nil {
-			return err
-		}
-
-		err = storageVolumeConfigAdd(tx.tx, volumeID, volumeConfig, false)
-		if err != nil {
-			return fmt.Errorf("Failed inserting storage volume record configuration: %w", err)
-		}
-
-		return nil
-	})
+	driver, err := c.GetStoragePoolDriver(ctx, poolID)
 	if err != nil {
-		volumeID = -1
+		return -1, err
+	}
+
+	var result sql.Result
+
+	if shared.ValueInSlice(driver, remoteDrivers) {
+		result, err = c.tx.ExecContext(ctx, `
+INSERT INTO storage_volumes (storage_pool_id, type, name, description, project_id, content_type, creation_date)
+ VALUES (?, ?, ?, ?, (SELECT id FROM projects WHERE name = ?), ?, ?)
+`,
+			poolID, volumeType, volumeName, volumeDescription, projectName, contentType, creationDate)
+	} else {
+		result, err = c.tx.ExecContext(ctx, `
+INSERT INTO storage_volumes (storage_pool_id, node_id, type, name, description, project_id, content_type, creation_date)
+ VALUES (?, ?, ?, ?, ?, (SELECT id FROM projects WHERE name = ?), ?, ?)
+`,
+			poolID, c.nodeID, volumeType, volumeName, volumeDescription, projectName, contentType, creationDate)
+	}
+
+	if err != nil {
+		return -1, err
+	}
+
+	volumeID, err = result.LastInsertId()
+	if err != nil {
+		return -1, err
+	}
+
+	err = storageVolumeConfigAdd(c.tx, volumeID, volumeConfig, false)
+	if err != nil {
+		return -1, fmt.Errorf("Failed inserting storage volume record configuration: %w", err)
 	}
 
 	return volumeID, err
@@ -492,20 +487,6 @@ INSERT INTO storage_volumes (storage_pool_id, node_id, type, name, description,
 
 // Return the ID of a storage volume on a given storage pool of a given storage
 // volume type, on the given node.
-func (c *Cluster) storagePoolVolumeGetTypeID(project string, volumeName string, volumeType int, poolID, nodeID int64) (int64, error) {
-	var id int64
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		var err error
-		id, err = tx.storagePoolVolumeGetTypeID(ctx, project, volumeName, volumeType, poolID, nodeID)
-		return err
-	})
-	if err != nil {
-		return -1, err
-	}
-
-	return id, nil
-}
-
 func (c *ClusterTx) storagePoolVolumeGetTypeID(ctx context.Context, project string, volumeName string, volumeType int, poolID, nodeID int64) (int64, error) {
 	remoteDrivers := StorageRemoteDriverNames()
 
@@ -540,49 +521,10 @@ SELECT storage_volumes_all.id
 
 // GetStoragePoolNodeVolumeID gets the ID of a storage volume on a given storage pool
 // of a given storage volume type and project, on the current node.
-func (c *Cluster) GetStoragePoolNodeVolumeID(projectName string, volumeName string, volumeType int, poolID int64) (int64, error) {
-	return c.storagePoolVolumeGetTypeID(projectName, volumeName, volumeType, poolID, c.nodeID)
+func (c *ClusterTx) GetStoragePoolNodeVolumeID(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64) (int64, error) {
+	return c.storagePoolVolumeGetTypeID(ctx, projectName, volumeName, volumeType, poolID, c.nodeID)
 }
 
-// XXX: this was extracted from lxd/storage_volume_utils.go, we find a way to
-//      factor it independently from both the db and main packages.
-const (
-	StoragePoolVolumeTypeContainer = iota
-	StoragePoolVolumeTypeImage
-	StoragePoolVolumeTypeCustom
-	StoragePoolVolumeTypeVM
-)
-
-// Leave the string type in here! This guarantees that go treats this is as a
-// typed string constant. Removing it causes go to treat these as untyped string
-// constants which is not what we want.
-const (
-	StoragePoolVolumeTypeNameContainer string = "container"
-	StoragePoolVolumeTypeNameVM        string = "virtual-machine"
-	StoragePoolVolumeTypeNameImage     string = "image"
-	StoragePoolVolumeTypeNameCustom    string = "custom"
-)
-
-// StoragePoolVolumeTypeNames represents a map of storage volume types and their names.
-var StoragePoolVolumeTypeNames = map[int]string{
-	StoragePoolVolumeTypeContainer: "container",
-	StoragePoolVolumeTypeImage:     "image",
-	StoragePoolVolumeTypeCustom:    "custom",
-	StoragePoolVolumeTypeVM:        "virtual-machine",
-}
-
-// Content types.
-const (
-	StoragePoolVolumeContentTypeFS = iota
-	StoragePoolVolumeContentTypeBlock
-)
-
-// Content type names.
-const (
-	StoragePoolVolumeContentTypeNameFS    string = "filesystem"
-	StoragePoolVolumeContentTypeNameBlock string = "block"
-)
-
 // StorageVolumeArgs is a value object holding all db-related details about a
 // storage volume.
 type StorageVolumeArgs struct {
@@ -672,7 +614,7 @@ func (c *ClusterTx) GetStorageVolumeNodes(ctx context.Context, poolID int64, pro
 		}
 
 		remoteDrivers := StorageRemoteDriverNames()
-		if shared.StringInSlice(driver, remoteDrivers) {
+		if shared.ValueInSlice(driver, remoteDrivers) {
 			return nil, ErrNoClusterMember
 		}
 	}
@@ -680,22 +622,6 @@ func (c *ClusterTx) GetStorageVolumeNodes(ctx context.Context, poolID int64, pro
 	return nodes, nil
 }
 
-// Get the config of a storage volume.
-func (c *Cluster) storageVolumeConfigGet(volumeID int64, isSnapshot bool) (map[string]string, error) {
-	var err error
-	var result map[string]string
-
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		result, err = tx.storageVolumeConfigGet(ctx, volumeID, isSnapshot)
-		return err
-	})
-	if err != nil {
-		return nil, err
-	}
-
-	return result, nil
-}
-
 // Get the config of a storage volume.
 func (c *ClusterTx) storageVolumeConfigGet(ctx context.Context, volumeID int64, isSnapshot bool) (map[string]string, error) {
 	var queryStr string
@@ -731,7 +657,7 @@ func (c *ClusterTx) storageVolumeConfigGet(ctx context.Context, volumeID int64,
 //
 // Note, the code below doesn't deal with snapshots of snapshots.
 // To do that, we'll need to weed out based on # slashes in names.
-func (c *Cluster) GetNextStorageVolumeSnapshotIndex(pool, name string, typ int, pattern string) int {
+func (c *ClusterTx) GetNextStorageVolumeSnapshotIndex(ctx context.Context, pool, name string, typ int, pattern string) int {
 	remoteDrivers := StorageRemoteDriverNames()
 
 	q := fmt.Sprintf(`
@@ -751,7 +677,7 @@ SELECT storage_volumes_snapshots.name FROM storage_volumes_snapshots
 		inargs = append(inargs, driver)
 	}
 
-	results, err := queryScan(c, q, inargs, outfmt)
+	results, err := queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return 0
 	}
@@ -759,7 +685,11 @@ SELECT storage_volumes_snapshots.name FROM storage_volumes_snapshots
 	max := 0
 
 	for _, r := range results {
-		substr := r[0].(string)
+		substr, ok := r[0].(string)
+		if !ok {
+			continue
+		}
+
 		fields := strings.SplitN(pattern, "%d", 2)
 
 		var num int
@@ -837,32 +767,17 @@ func storageVolumeConfigClear(tx *sql.Tx, volumeID int64, isSnapshot bool) error
 	return nil
 }
 
-// RemoveStorageVolumeImages removes the volumes associated with the images
-// with the given fingerprints.
-func (c *Cluster) RemoveStorageVolumeImages(fingerprints []string) error {
-	stmt := fmt.Sprintf(
-		"DELETE FROM storage_volumes WHERE type=? AND name NOT IN %s",
-		query.Params(len(fingerprints)))
-	args := []any{StoragePoolVolumeTypeImage}
-	for _, fingerprint := range fingerprints {
-		args = append(args, fingerprint)
-	}
-
-	err := exec(c, stmt, args...)
-	return err
-}
-
 // Convert a volume integer type code to its human-readable name.
 func storagePoolVolumeTypeToName(volumeType int) (string, error) {
 	switch volumeType {
-	case StoragePoolVolumeTypeContainer:
-		return StoragePoolVolumeTypeNameContainer, nil
-	case StoragePoolVolumeTypeVM:
-		return StoragePoolVolumeTypeNameVM, nil
-	case StoragePoolVolumeTypeImage:
-		return StoragePoolVolumeTypeNameImage, nil
-	case StoragePoolVolumeTypeCustom:
-		return StoragePoolVolumeTypeNameCustom, nil
+	case cluster.StoragePoolVolumeTypeContainer:
+		return cluster.StoragePoolVolumeTypeNameContainer, nil
+	case cluster.StoragePoolVolumeTypeVM:
+		return cluster.StoragePoolVolumeTypeNameVM, nil
+	case cluster.StoragePoolVolumeTypeImage:
+		return cluster.StoragePoolVolumeTypeNameImage, nil
+	case cluster.StoragePoolVolumeTypeCustom:
+		return cluster.StoragePoolVolumeTypeNameCustom, nil
 	}
 
 	return "", fmt.Errorf("Invalid storage volume type")
@@ -871,10 +786,12 @@ func storagePoolVolumeTypeToName(volumeType int) (string, error) {
 // Convert a volume integer content type code to its human-readable name.
 func storagePoolVolumeContentTypeToName(contentType int) (string, error) {
 	switch contentType {
-	case StoragePoolVolumeContentTypeFS:
-		return StoragePoolVolumeContentTypeNameFS, nil
-	case StoragePoolVolumeContentTypeBlock:
-		return StoragePoolVolumeContentTypeNameBlock, nil
+	case cluster.StoragePoolVolumeContentTypeFS:
+		return cluster.StoragePoolVolumeContentTypeNameFS, nil
+	case cluster.StoragePoolVolumeContentTypeBlock:
+		return cluster.StoragePoolVolumeContentTypeNameBlock, nil
+	case cluster.StoragePoolVolumeContentTypeISO:
+		return cluster.StoragePoolVolumeContentTypeNameISO, nil
 	}
 
 	return "", fmt.Errorf("Invalid storage volume content type")
@@ -883,7 +800,12 @@ func storagePoolVolumeContentTypeToName(contentType int) (string, error) {
 // GetCustomVolumesInProject returns all custom volumes in the given project.
 func (c *ClusterTx) GetCustomVolumesInProject(ctx context.Context, project string) ([]StorageVolumeArgs, error) {
 	sql := `
-SELECT storage_volumes.id, storage_volumes.name, storage_pools.name, IFNULL(storage_volumes.node_id, -1)
+SELECT
+	storage_volumes.id,
+	storage_volumes.name,
+	storage_volumes.creation_date,
+	storage_pools.name,
+	IFNULL(storage_volumes.node_id, -1)
 FROM storage_volumes
 JOIN storage_pools ON storage_pools.id = storage_volumes.storage_pool_id
 JOIN projects ON projects.id = storage_volumes.project_id
@@ -893,7 +815,7 @@ WHERE storage_volumes.type = ? AND projects.name = ?
 	volumes := []StorageVolumeArgs{}
 	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
 		volume := StorageVolumeArgs{}
-		err := scan(&volume.ID, &volume.Name, &volume.PoolName, &volume.NodeID)
+		err := scan(&volume.ID, &volume.Name, &volume.CreationDate, &volume.PoolName, &volume.NodeID)
 		if err != nil {
 			return err
 		}
@@ -901,7 +823,7 @@ WHERE storage_volumes.type = ? AND projects.name = ?
 		volumes = append(volumes, volume)
 
 		return nil
-	}, StoragePoolVolumeTypeCustom, project)
+	}, cluster.StoragePoolVolumeTypeCustom, project)
 	if err != nil {
 		return nil, fmt.Errorf("Fetch custom volumes: %w", err)
 	}
@@ -950,3 +872,34 @@ func (c *ClusterTx) GetStorageVolumeURIs(ctx context.Context, project string) ([
 
 	return uris, nil
 }
+
+// UpdateStorageVolumeNode changes the name of a storage volume and the cluster member hosting it.
+// It's meant to be used when moving a storage volume backed by ceph from one cluster node to another.
+func (c *ClusterTx) UpdateStorageVolumeNode(ctx context.Context, projectName string, oldName string, newName string, newMemberName string, poolID int64, volumeType int) error {
+	volume, err := c.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, oldName, false)
+	if err != nil {
+		return err
+	}
+
+	member, err := c.GetNodeByName(ctx, newMemberName)
+	if err != nil {
+		return fmt.Errorf("Failed to get new member %q info: %w", newMemberName, err)
+	}
+
+	stmt := "UPDATE storage_volumes SET node_id=?, name=? WHERE id=?"
+	result, err := c.tx.Exec(stmt, member.ID, newName, volume.ID)
+	if err != nil {
+		return fmt.Errorf("Failed to update volumes's name and member ID: %w", err)
+	}
+
+	n, err := result.RowsAffected()
+	if err != nil {
+		return fmt.Errorf("Failed to get rows affected by volume update: %w", err)
+	}
+
+	if n != 1 {
+		return fmt.Errorf("Unexpected number of updated rows in storage_volumes table: %d", n)
+	}
+
+	return nil
+}
