@@ -13,14 +13,14 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/revert"
-	storageDrivers "github.com/lxc/lxd/lxd/storage/drivers"
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/subprocess"
+	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/instance"
+	storageDrivers "github.com/canonical/lxd/lxd/storage/drivers"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/lxd/subprocess"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // RBDFormatPrefix is the prefix used in disk paths to identify RBD.
@@ -93,13 +93,14 @@ func IsBlockdev(path string) bool {
 }
 
 // DiskMount mounts a disk device.
-func DiskMount(srcPath string, dstPath string, readonly bool, recursive bool, propagation string, mountOptions []string, fsName string) error {
+func DiskMount(srcPath string, dstPath string, recursive bool, propagation string, mountOptions []string, fsName string) error {
 	var err error
 
-	// Prepare the mount flags
-	flags := 0
-	if readonly {
-		flags |= unix.MS_RDONLY
+	flags, mountOptionsStr := filesystem.ResolveMountOptions(mountOptions)
+
+	var readonly bool
+	if shared.ValueInSlice("ro", mountOptions) {
+		readonly = true
 	}
 
 	// Detect the filesystem
@@ -135,7 +136,7 @@ func DiskMount(srcPath string, dstPath string, readonly bool, recursive bool, pr
 	}
 
 	// Mount the filesystem
-	err = unix.Mount(srcPath, dstPath, fsName, uintptr(flags), strings.Join(mountOptions, ","))
+	err = unix.Mount(srcPath, dstPath, fsName, uintptr(flags), mountOptionsStr)
 	if err != nil {
 		return fmt.Errorf("Unable to mount %q at %q with filesystem %q: %w", srcPath, dstPath, fsName, err)
 	}
@@ -441,6 +442,8 @@ func DiskVMVirtiofsdStart(execPath string, inst instance.Instance, socketPath st
 			cmd = "/usr/lib/qemu/virtiofsd"
 		} else if shared.PathExists("/usr/libexec/virtiofsd") {
 			cmd = "/usr/libexec/virtiofsd"
+		} else if shared.PathExists("/usr/lib/virtiofsd") {
+			cmd = "/usr/lib/virtiofsd"
 		}
 	}
 
@@ -458,6 +461,10 @@ func DiskVMVirtiofsdStart(execPath string, inst instance.Instance, socketPath st
 		return nil, nil, UnsupportedError{"Stateful migration unsupported"}
 	}
 
+	if shared.IsTrue(inst.ExpandedConfig()["security.sev"]) || shared.IsTrue(inst.ExpandedConfig()["security.sev.policy.es"]) {
+		return nil, nil, UnsupportedError{"SEV unsupported"}
+	}
+
 	// Trickery to handle paths > 107 chars.
 	socketFileDir, err := os.Open(filepath.Dir(socketPath))
 	if err != nil {
