@@ -15,18 +15,18 @@ import (
 	"github.com/j-keck/arping"
 	"github.com/mdlayher/ndp"
 
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	pcidev "github.com/lxc/lxd/lxd/device/pci"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/ip"
-	"github.com/lxc/lxd/lxd/network"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/validate"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	pcidev "github.com/canonical/lxd/lxd/device/pci"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/ip"
+	"github.com/canonical/lxd/lxd/network"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 // Instances can be started in parallel, so lock the creation of VLANs.
@@ -42,7 +42,7 @@ func NetworkSetDevMTU(devName string, mtu uint32) error {
 	// Only try and change the MTU if the requested mac is different to current one.
 	if curMTU != mtu {
 		link := &ip.Link{Name: devName}
-		err := link.SetMTU(fmt.Sprintf("%d", mtu))
+		err := link.SetMTU(mtu)
 		if err != nil {
 			return err
 		}
@@ -70,8 +70,13 @@ func NetworkSetDevMAC(devName string, mac string) error {
 
 	// Only try and change the MAC if the requested mac is different to current one.
 	if curMac != mac {
+		hwaddr, err := net.ParseMAC(mac)
+		if err != nil {
+			return fmt.Errorf("Failed parsing MAC address %q: %w", mac, err)
+		}
+
 		link := &ip.Link{Name: devName}
-		err := link.SetAddress(mac)
+		err = link.SetAddress(hwaddr)
 		if err != nil {
 			return err
 		}
@@ -206,67 +211,93 @@ func networkRestorePhysicalNIC(hostName string, volatile map[string]string) erro
 // is supplied in config, then the MTU of the new peer interface will inherit the parent MTU.
 // Accepts the name of the host side interface as a parameter and returns the peer interface name and MTU used.
 func networkCreateVethPair(hostName string, m deviceConfig.Device) (string, uint32, error) {
-	peerName := network.RandomDevName("veth")
+	var err error
 
 	veth := &ip.Veth{
 		Link: ip.Link{
 			Name: hostName,
+			Up:   true,
+		},
+		Peer: ip.Link{
+			Name: network.RandomDevName("veth"),
 		},
-		PeerName: peerName,
 	}
 
-	err := veth.Add()
-	if err != nil {
-		return "", 0, fmt.Errorf("Failed to create the veth interfaces %q and %q: %w", hostName, peerName, err)
-	}
+	// Set the MTU on both ends.
+	// The host side should always line up with the bridge to avoid accidentally lowering the bridge MTU.
+	// The instance side should use the configured MTU (if any), if not, it should match the host side.
+	var instanceMTU uint32
+	var parentMTU uint32
 
-	err = veth.SetUp()
-	if err != nil {
-		_ = network.InterfaceRemove(hostName)
-		return "", 0, fmt.Errorf("Failed to bring up the veth interface %q: %w", hostName, err)
-	}
-
-	// Set the MAC address on peer.
-	if m["hwaddr"] != "" {
-		link := &ip.Link{Name: peerName}
-		err := link.SetAddress(m["hwaddr"])
+	if m["parent"] != "" {
+		mtu, err := network.GetDevMTU(m["parent"])
 		if err != nil {
-			_ = network.InterfaceRemove(peerName)
-			return "", 0, fmt.Errorf("Failed to set the MAC address: %w", err)
+			return "", 0, fmt.Errorf("Failed to get the parent MTU: %w", err)
 		}
+
+		parentMTU = uint32(mtu)
 	}
 
-	// Set the MTU on peer. If not specified and has parent, will inherit MTU from parent.
-	var mtu uint32
 	if m["mtu"] != "" {
-		nicMTU, err := strconv.ParseUint(m["mtu"], 10, 32)
+		mtu, err := strconv.ParseUint(m["mtu"], 10, 32)
 		if err != nil {
 			return "", 0, fmt.Errorf("Invalid MTU specified: %w", err)
 		}
 
-		mtu = uint32(nicMTU)
+		instanceMTU = uint32(mtu)
+	}
+
+	if instanceMTU == 0 && parentMTU > 0 {
+		instanceMTU = parentMTU
+	}
+
+	if parentMTU == 0 && instanceMTU > 0 {
+		parentMTU = instanceMTU
+	}
+
+	if instanceMTU > 0 {
+		veth.Peer.MTU = instanceMTU
+	}
+
+	if parentMTU > 0 {
+		veth.MTU = parentMTU
+	}
+
+	// Set the MAC address on peer.
+	if m["hwaddr"] != "" {
+		hwaddr, err := net.ParseMAC(m["hwaddr"])
+		if err != nil {
+			return "", 0, fmt.Errorf("Failed parsing MAC address %q: %w", m["hwaddr"], err)
+		}
+
+		veth.Peer.Address = hwaddr
+	}
+
+	// Set TX queue length on both ends.
+	if m["queue.tx.length"] != "" {
+		nicTXqlen, err := strconv.ParseUint(m["queue.tx.length"], 10, 32)
+		if err != nil {
+			return "", 0, fmt.Errorf("Invalid txqueuelen specified: %w", err)
+		}
+
+		veth.TXQueueLength = uint32(nicTXqlen)
 	} else if m["parent"] != "" {
-		mtu, err = network.GetDevMTU(m["parent"])
+		veth.TXQueueLength, err = network.GetTXQueueLength(m["parent"])
 		if err != nil {
-			return "", 0, fmt.Errorf("Failed to get the parent MTU: %w", err)
+			return "", 0, fmt.Errorf("Failed to get the parent txqueuelen: %w", err)
 		}
 	}
 
-	if mtu > 0 {
-		err = NetworkSetDevMTU(peerName, mtu)
-		if err != nil {
-			_ = network.InterfaceRemove(peerName)
-			return "", 0, fmt.Errorf("Failed to set the MTU %d: %w", mtu, err)
-		}
+	veth.Peer.TXQueueLength = veth.TXQueueLength
 
-		err = NetworkSetDevMTU(hostName, mtu)
-		if err != nil {
-			_ = network.InterfaceRemove(peerName)
-			return "", 0, fmt.Errorf("Failed to set the MTU %d: %w", mtu, err)
-		}
+	// Add and configure the interface in one operation to reduce the number of executions and to avoid
+	// systemd-udevd from applying the default MACAddressPolicy=persistent policy.
+	err = veth.Add()
+	if err != nil {
+		return "", 0, fmt.Errorf("Failed to create the veth interfaces %q and %q: %w", hostName, veth.Peer.Name, err)
 	}
 
-	return peerName, mtu, nil
+	return veth.Peer.Name, veth.Peer.MTU, nil
 }
 
 // networkCreateTap creates and configures a TAP device.
@@ -294,7 +325,9 @@ func networkCreateTap(hostName string, m deviceConfig.Device) (uint32, error) {
 
 	revert.Add(func() { _ = network.InterfaceRemove(hostName) })
 
-	// Set the MTU on peer. If not specified and has parent, will inherit MTU from parent.
+	// Set the MTU on both ends.
+	// The host side should always line up with the bridge to avoid accidentally lowering the bridge MTU.
+	// The instance side should use the configured MTU (if any), if not, it should match the host side.
 	var mtu uint32
 	if m["mtu"] != "" {
 		nicMTU, err := strconv.ParseUint(m["mtu"], 10, 32)
@@ -303,20 +336,45 @@ func networkCreateTap(hostName string, m deviceConfig.Device) (uint32, error) {
 		}
 
 		mtu = uint32(nicMTU)
-	} else if m["parent"] != "" {
+	}
+
+	if m["parent"] != "" {
 		parentMTU, err := network.GetDevMTU(m["parent"])
 		if err != nil {
 			return 0, fmt.Errorf("Failed to get the parent MTU: %w", err)
 		}
 
-		mtu = parentMTU
-	}
-
-	if mtu > 0 {
-		err = NetworkSetDevMTU(hostName, mtu)
+		err = NetworkSetDevMTU(hostName, parentMTU)
 		if err != nil {
 			return 0, fmt.Errorf("Failed to set the MTU %d: %w", mtu, err)
 		}
+
+		if mtu == 0 {
+			mtu = parentMTU
+		}
+	}
+
+	// Set TX queue length on both ends.
+	var txqueuelen uint32
+	if m["queue.tx.length"] != "" {
+		nicTXqlen, err := strconv.ParseUint(m["queue.tx.length"], 10, 32)
+		if err != nil {
+			return 0, fmt.Errorf("Invalid txqueuelen specified: %w", err)
+		}
+
+		txqueuelen = uint32(nicTXqlen)
+	} else if m["parent"] != "" {
+		txqueuelen, err = network.GetTXQueueLength(m["parent"])
+		if err != nil {
+			return 0, fmt.Errorf("Failed to get the parent txqueuelen: %w", err)
+		}
+	}
+
+	if txqueuelen > 0 {
+		err = link.SetTXQueueLength(txqueuelen)
+		if err != nil {
+			return 0, fmt.Errorf("Failed to set the TX queue length %d: %w", txqueuelen, err)
+		}
 	}
 
 	revert.Success()
@@ -428,33 +486,33 @@ func networkNICRouteDelete(routeDev string, routes ...string) {
 }
 
 // networkSetupHostVethLimits applies any network rate limits to the veth device specified in the config.
-func networkSetupHostVethLimits(m deviceConfig.Device) error {
+func networkSetupHostVethLimits(d *deviceCommon, oldConfig deviceConfig.Device, bridged bool) error {
 	var err error
 
-	veth := m["host_name"]
+	veth := d.config["host_name"]
 
 	if veth == "" || !network.InterfaceExists(veth) {
 		return fmt.Errorf("Unknown or missing host side veth device %q", veth)
 	}
 
 	// Apply max limit
-	if m["limits.max"] != "" {
-		m["limits.ingress"] = m["limits.max"]
-		m["limits.egress"] = m["limits.max"]
+	if d.config["limits.max"] != "" {
+		d.config["limits.ingress"] = d.config["limits.max"]
+		d.config["limits.egress"] = d.config["limits.max"]
 	}
 
 	// Parse the values
 	var ingressInt int64
-	if m["limits.ingress"] != "" {
-		ingressInt, err = units.ParseBitSizeString(m["limits.ingress"])
+	if d.config["limits.ingress"] != "" {
+		ingressInt, err = units.ParseBitSizeString(d.config["limits.ingress"])
 		if err != nil {
 			return err
 		}
 	}
 
 	var egressInt int64
-	if m["limits.egress"] != "" {
-		egressInt, err = units.ParseBitSizeString(m["limits.egress"])
+	if d.config["limits.egress"] != "" {
+		egressInt, err = units.ParseBitSizeString(d.config["limits.egress"])
 		if err != nil {
 			return err
 		}
@@ -467,7 +525,7 @@ func networkSetupHostVethLimits(m deviceConfig.Device) error {
 	_ = qdisc.Delete()
 
 	// Apply new limits
-	if m["limits.ingress"] != "" {
+	if d.config["limits.ingress"] != "" {
 		qdiscHTB := &ip.QdiscHTB{Qdisc: ip.Qdisc{Dev: veth, Handle: "1:0", Root: true}, Default: "10"}
 		err := qdiscHTB.Add()
 		if err != nil {
@@ -487,7 +545,7 @@ func networkSetupHostVethLimits(m deviceConfig.Device) error {
 		}
 	}
 
-	if m["limits.egress"] != "" {
+	if d.config["limits.egress"] != "" {
 		qdisc = &ip.Qdisc{Dev: veth, Handle: "ffff:0", Ingress: true}
 		err := qdisc.Add()
 		if err != nil {
@@ -502,12 +560,50 @@ func networkSetupHostVethLimits(m deviceConfig.Device) error {
 		}
 	}
 
+	var networkPriority uint64
+	if d.config["limits.priority"] != "" {
+		networkPriority, err = strconv.ParseUint(d.config["limits.priority"], 10, 32)
+		if err != nil {
+			return fmt.Errorf("Failed to parse limits.priority %q: %w", d.config["limits.priority"], err)
+		}
+	}
+
+	if oldConfig != nil && oldConfig["limits.priority"] != d.config["limits.priority"] {
+		err = d.state.Firewall.InstanceClearNetPrio(d.inst.Project().Name, d.inst.Name(), veth)
+		if err != nil {
+			return err
+		}
+	}
+
+	if oldConfig == nil || (oldConfig != nil && oldConfig["limits.priority"] != d.config["limits.priority"]) {
+		if networkPriority != 0 {
+			if bridged && d.state.Firewall.String() == "xtables" {
+				return fmt.Errorf("Failed to setup instance device network priority. The xtables firewall driver does not support required functionality.")
+			}
+
+			err = d.state.Firewall.InstanceSetupNetPrio(d.inst.Project().Name, d.inst.Name(), veth, uint32(networkPriority))
+			if err != nil {
+				return fmt.Errorf("Failed to setup instance device network priority: %w", err)
+			}
+		}
+	}
+
+	return nil
+}
+
+// networkClearHostVethLimits clears any network rate limits to the veth device specified in the config.
+func networkClearHostVethLimits(d *deviceCommon) error {
+	err := d.state.Firewall.InstanceClearNetPrio(d.inst.Project().Name, d.inst.Name(), d.config["host_name"])
+	if err != nil {
+		return err
+	}
+
 	return nil
 }
 
 // networkValidGateway validates the gateway value.
 func networkValidGateway(value string) error {
-	if shared.StringInSlice(value, []string{"none", "auto"}) {
+	if shared.ValueInSlice(value, []string{"none", "auto"}) {
 		return nil
 	}
 
@@ -624,13 +720,13 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 	// Record properties of VF device.
 	err = networkSnapshotPhysicalNIC(volatile["host_name"], volatile)
 	if err != nil {
-		return vfPCIDev, 0, err
+		return vfPCIDev, 0, fmt.Errorf("Failed recording NIC %q settings: %w", volatile["host_name"], err)
 	}
 
 	// Get VF device's PCI Slot Name so we can unbind and rebind it from the host.
 	vfPCIDev, err = network.SRIOVGetVFDevicePCISlot(vfParent, volatile["last_state.vf.id"])
 	if err != nil {
-		return vfPCIDev, 0, err
+		return vfPCIDev, 0, fmt.Errorf("Failed getting PCI slot for VF %q: %w", volatile["last_state.vf.id"], err)
 	}
 
 	// Unbind VF device from the host so that the settings will take effect when we rebind it.
@@ -646,7 +742,7 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 		link := &ip.Link{Name: vfParent}
 		err := link.SetVfVlan(volatile["last_state.vf.id"], d.config["vlan"])
 		if err != nil {
-			return vfPCIDev, 0, err
+			return vfPCIDev, 0, fmt.Errorf("Failed setting VLAN for VF %q: %w", volatile["last_state.vf.id"], err)
 		}
 	}
 
@@ -668,13 +764,13 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 		link := &ip.Link{Name: vfParent}
 		err = link.SetVfAddress(volatile["last_state.vf.id"], mac)
 		if err != nil {
-			return vfPCIDev, 0, err
+			return vfPCIDev, 0, fmt.Errorf("Failed setting MAC for VF %q: %w", volatile["last_state.vf.id"], err)
 		}
 
 		// Now that MAC is set on VF, we can enable spoof checking.
 		err = link.SetVfSpoofchk(volatile["last_state.vf.id"], "on")
 		if err != nil {
-			return vfPCIDev, 0, err
+			return vfPCIDev, 0, fmt.Errorf("Failed enabling spoof check for VF %q: %w", volatile["last_state.vf.id"], err)
 		}
 	} else {
 		// Try to reset VF to ensure no previous MAC restriction exists, as some devices require this
@@ -683,14 +779,14 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 		link := &ip.Link{Name: vfParent}
 		err = link.SetVfAddress(volatile["last_state.vf.id"], "00:00:00:00:00:00")
 		if err != nil {
-			return vfPCIDev, 0, err
+			return vfPCIDev, 0, fmt.Errorf("Failed clearing MAC for VF %q: %w", volatile["last_state.vf.id"], err)
 		}
 
 		if useSpoofCheck {
 			// Ensure spoof checking is disabled if not enabled in instance (only for real VF).
 			err = link.SetVfSpoofchk(volatile["last_state.vf.id"], "off")
 			if err != nil {
-				return vfPCIDev, 0, err
+				return vfPCIDev, 0, fmt.Errorf("Failed disabling spoof check for VF %q: %w", volatile["last_state.vf.id"], err)
 			}
 		}
 
@@ -704,7 +800,7 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 
 			err = link.SetVfAddress(volatile["last_state.vf.id"], mac)
 			if err != nil {
-				return vfPCIDev, 0, err
+				return vfPCIDev, 0, fmt.Errorf("Failed setting MAC for VF %q: %w", volatile["last_state.vf.id"], err)
 			}
 		}
 	}
@@ -714,29 +810,28 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 
 	if d.inst.Type() == instancetype.Container {
 		// Bind VF device onto the host so that the settings will take effect.
-		// This will remove the VF interface temporarily, and it will re-appear shortly after.
-		err = pcidev.DeviceProbe(vfPCIDev)
-		if err != nil {
-			return vfPCIDev, 0, err
-		}
-
-		// Wait for VF driver to be reloaded. Unfortunately the time between sending the bind event
-		// to the nic and it actually appearing on the host is non-zero, so we need to watch and wait,
-		// otherwise next steps of applying settings to interface will fail.
-		err = network.InterfaceBindWait(volatile["host_name"])
+		err = networkPCIBindWaitInterface(vfPCIDev, volatile["host_name"])
 		if err != nil {
 			return vfPCIDev, 0, err
 		}
 	} else if d.inst.Type() == instancetype.VM {
 		pciIOMMUGroup, err = pcidev.DeviceIOMMUGroup(vfPCIDev.SlotName)
 		if err != nil {
-			return vfPCIDev, 0, err
+			return vfPCIDev, 0, fmt.Errorf("Failed getting IOMMU group for VF device %q: %w", vfPCIDev.SlotName, err)
 		}
 
-		// Register VF device with vfio-pci driver so it can be passed to VM.
-		err = pcidev.DeviceDriverOverride(vfPCIDev, "vfio-pci")
-		if err != nil {
-			return vfPCIDev, 0, err
+		if d.config["acceleration"] != "vdpa" {
+			// Register VF device with vfio-pci driver so it can be passed to VM.
+			err = pcidev.DeviceDriverOverride(vfPCIDev, "vfio-pci")
+			if err != nil {
+				return vfPCIDev, 0, fmt.Errorf("Failed overriding driver for VF device %q: %w", vfPCIDev.SlotName, err)
+			}
+		} else {
+			// Bind VF device onto the host so that the settings will take effect.
+			err = networkPCIBindWaitInterface(vfPCIDev, volatile["host_name"])
+			if err != nil {
+				return vfPCIDev, 0, err
+			}
 		}
 
 		// Record original driver used by VF device for restore.
@@ -824,16 +919,7 @@ func networkSRIOVRestoreVF(d deviceCommon, useSpoofCheck bool, volatile map[stri
 	}
 
 	// Bind VF device onto the host so that the settings will take effect.
-	err = pcidev.DeviceProbe(vfPCIDev)
-	if err != nil {
-		return err
-	}
-
-	// Wait for VF driver to be reloaded, this will remove the VF interface from the instance
-	// and it will re-appear on the host. Unfortunately the time between sending the bind event
-	// to the nic and it actually appearing on the host is non-zero, so we need to watch and wait,
-	// otherwise next step of restoring MAC and MTU settings in restorePhysicalNic will fail.
-	err = network.InterfaceBindWait(volatile["host_name"])
+	err = networkPCIBindWaitInterface(vfPCIDev, volatile["host_name"])
 	if err != nil {
 		return err
 	}
@@ -848,13 +934,55 @@ func networkSRIOVRestoreVF(d deviceCommon, useSpoofCheck bool, volatile map[stri
 	return nil
 }
 
+// networkPCIBindWaitInterface repeatedly requests the pciDev is probed to be bound to the override driver and
+// checks whether the expected network interface has appeared as the result of the device driver being bound.
+func networkPCIBindWaitInterface(pciDev pcidev.Device, ifName string) error {
+	var err error
+
+	waitDuration := time.Second * 10
+	waitUntil := time.Now().Add(waitDuration)
+
+	// Keep requesting the device driver be probed in case it was not ready previously or the expected
+	// interface has not appeared yet. The device can be probed multiple times safely.
+
+	i := 0
+	for {
+		err = pcidev.DeviceProbe(pciDev)
+		if err == nil && network.InterfaceExists(ifName) {
+			return nil
+		}
+
+		if time.Now().After(waitUntil) {
+			if err != nil {
+				return fmt.Errorf("Failed binding interface %q after %v: %w", ifName, waitDuration, err)
+			}
+
+			return fmt.Errorf("Failed binding interface %q after %v", ifName, waitDuration)
+		}
+
+		if i <= 5 {
+			// Retry more quickly early on.
+			time.Sleep(time.Millisecond * time.Duration(i) * 10)
+		} else {
+			time.Sleep(time.Second)
+		}
+
+		i++
+	}
+}
+
 // networkSRIOVSetupContainerVFNIC configures the VF NIC interface ready for moving into container.
 // It configures the MAC address and MTU, then brings the interface up.
 func networkSRIOVSetupContainerVFNIC(hostName string, config map[string]string) error {
 	// Set the MAC address.
 	if config["hwaddr"] != "" {
+		hwaddr, err := net.ParseMAC(config["hwaddr"])
+		if err != nil {
+			return fmt.Errorf("Failed parsing MAC address %q: %w", config["hwaddr"], err)
+		}
+
 		link := &ip.Link{Name: hostName}
-		err := link.SetAddress(config["hwaddr"])
+		err = link.SetAddress(hwaddr)
 		if err != nil {
 			return fmt.Errorf("Failed setting MAC address %q on %q: %w", config["hwaddr"], hostName, err)
 		}
@@ -862,8 +990,13 @@ func networkSRIOVSetupContainerVFNIC(hostName string, config map[string]string)
 
 	// Set the MTU.
 	if config["mtu"] != "" {
+		mtu, err := strconv.ParseUint(config["mtu"], 10, 32)
+		if err != nil {
+			return fmt.Errorf("Invalid VF MTU specified %q: %w", config["mtu"], err)
+		}
+
 		link := &ip.Link{Name: hostName}
-		err := link.SetMTU(config["mtu"])
+		err = link.SetMTU(uint32(mtu))
 		if err != nil {
 			return fmt.Errorf("Failed setting MTU %q on %q: %w", config["mtu"], hostName, err)
 		}
@@ -898,8 +1031,13 @@ func networkSRIOVSetupContainerVFNIC(hostName string, config map[string]string)
 			return fmt.Errorf("Failed generating random MAC for VF %q: %w", hostName, err)
 		}
 
+		hwaddr, err := net.ParseMAC(randMAC)
+		if err != nil {
+			return fmt.Errorf("Failed parsing MAC address %q: %w", randMAC, err)
+		}
+
 		link := &ip.Link{Name: hostName}
-		err = link.SetAddress(randMAC)
+		err = link.SetAddress(hwaddr)
 		if err != nil {
 			return fmt.Errorf("Failed to set random MAC address %q on %q: %w", randMAC, hostName, err)
 		}
