@@ -4,19 +4,22 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"sync"
 
-	"github.com/pborman/uuid"
+	"github.com/google/uuid"
 
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	pcidev "github.com/lxc/lxd/lxd/device/pci"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/resources"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	pcidev "github.com/canonical/lxd/lxd/device/pci"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/resources"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
 )
 
+var gpuMdevMu sync.Mutex
+
 type gpuMdev struct {
 	deviceCommon
 }
@@ -44,6 +47,10 @@ func (d *gpuMdev) Stop() (*deviceConfig.RunConfig, error) {
 func (d *gpuMdev) startVM() (*deviceConfig.RunConfig, error) {
 	runConf := deviceConfig.RunConfig{}
 
+	// Lock to prevent multiple concurrent mdev devices being setup.
+	gpuMdevMu.Lock()
+	defer gpuMdevMu.Unlock()
+
 	// Get any existing UUID.
 	v := d.volatileGet()
 	mdevUUID := v["vgpu.uuid"]
@@ -59,11 +66,8 @@ func (d *gpuMdev) startVM() (*deviceConfig.RunConfig, error) {
 
 	var pciAddress string
 	for _, gpu := range gpus.Cards {
-		// Skip any cards that don't match the vendorid, pci, productid or DRM ID settings (if specified).
-		if (d.config["vendorid"] != "" && gpu.VendorID != d.config["vendorid"]) ||
-			(d.config["pci"] != "" && gpu.PCIAddress != d.config["pci"]) ||
-			(d.config["productid"] != "" && gpu.ProductID != d.config["productid"]) ||
-			(d.config["id"] != "" && (gpu.DRM == nil || fmt.Sprintf("%d", gpu.DRM.ID) != d.config["id"])) {
+		// Skip any cards that are not selected.
+		if !gpuSelected(d.Config(), gpu) {
 			continue
 		}
 
@@ -120,7 +124,7 @@ func (d *gpuMdev) startVM() (*deviceConfig.RunConfig, error) {
 
 		// Create the vGPU.
 		if mdevUUID == "" || !shared.PathExists(fmt.Sprintf("/sys/bus/pci/devices/%s/%s", pciAddress, mdevUUID)) {
-			mdevUUID = uuid.New()
+			mdevUUID = uuid.New().String()
 
 			err = os.WriteFile(filepath.Join(fmt.Sprintf("/sys/bus/pci/devices/%s/mdev_supported_types/%s/create", pciAddress, d.config["mdev"])), []byte(mdevUUID), 0200)
 			if err != nil {
@@ -229,7 +233,7 @@ func (d *gpuMdev) validateConfig(instConf instance.ConfigReader) error {
 	if d.config["pci"] != "" {
 		for _, field := range []string{"id", "productid", "vendorid"} {
 			if d.config[field] != "" {
-				return fmt.Errorf(`Cannot use %q when when "pci" is set`, field)
+				return fmt.Errorf(`Cannot use %q when "pci" is set`, field)
 			}
 		}
 
@@ -239,7 +243,7 @@ func (d *gpuMdev) validateConfig(instConf instance.ConfigReader) error {
 	if d.config["id"] != "" {
 		for _, field := range []string{"pci", "productid", "vendorid"} {
 			if d.config[field] != "" {
-				return fmt.Errorf(`Cannot use %q when when "id" is set`, field)
+				return fmt.Errorf(`Cannot use %q when "id" is set`, field)
 			}
 		}
 	}
