@@ -3,6 +3,7 @@ package device
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"encoding/binary"
 	"encoding/hex"
 	"fmt"
@@ -17,24 +18,23 @@ import (
 	"github.com/google/gopacket/layers"
 	"github.com/mdlayher/netx/eui64"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/dnsmasq"
-	"github.com/lxc/lxd/lxd/dnsmasq/dhcpalloc"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/ip"
-	"github.com/lxc/lxd/lxd/network"
-	"github.com/lxc/lxd/lxd/network/openvswitch"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/resources"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/validate"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/dnsmasq"
+	"github.com/canonical/lxd/lxd/dnsmasq/dhcpalloc"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/ip"
+	"github.com/canonical/lxd/lxd/network"
+	"github.com/canonical/lxd/lxd/network/openvswitch"
+	"github.com/canonical/lxd/lxd/resources"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 type bridgeNetwork interface {
@@ -54,7 +54,7 @@ func (d *nicBridged) CanHotPlug() bool {
 
 // CanMigrate returns whether the device can be migrated to any other cluster member.
 func (d *nicBridged) CanMigrate() bool {
-	return d.config["network"] != ""
+	return d.network != nil
 }
 
 // validateConfig checks the supplied config for correctness.
@@ -69,11 +69,13 @@ func (d *nicBridged) validateConfig(instConf instance.ConfigReader) error {
 		"network",
 		"parent",
 		"mtu",
+		"queue.tx.length",
 		"hwaddr",
 		"host_name",
 		"limits.ingress",
 		"limits.egress",
 		"limits.max",
+		"limits.priority",
 		"ipv4.address",
 		"ipv6.address",
 		"ipv4.routes",
@@ -118,7 +120,7 @@ func (d *nicBridged) validateConfig(instConf instance.ConfigReader) error {
 			}
 
 			parentAddress := netConfig["ipv4.address"]
-			if shared.StringInSlice(parentAddress, []string{"", "none"}) {
+			if shared.ValueInSlice(parentAddress, []string{"", "none"}) {
 				return nil
 			}
 
@@ -153,7 +155,7 @@ func (d *nicBridged) validateConfig(instConf instance.ConfigReader) error {
 			}
 
 			parentAddress := netConfig["ipv6.address"]
-			if shared.StringInSlice(parentAddress, []string{"", "none"}) {
+			if shared.ValueInSlice(parentAddress, []string{"", "none"}) {
 				return nil
 			}
 
@@ -174,7 +176,7 @@ func (d *nicBridged) validateConfig(instConf instance.ConfigReader) error {
 
 		// When we know the parent network is managed, we can validate the NIC's VLAN settings based on
 		// on the bridge driver type.
-		if shared.StringInSlice(netConfig["bridge.driver"], []string{"", "native"}) {
+		if shared.ValueInSlice(netConfig["bridge.driver"], []string{"", "native"}) {
 			// Check VLAN 0 isn't set when using a native Linux managed bridge, as not supported.
 			if d.config["vlan"] == "0" {
 				return fmt.Errorf("VLAN ID 0 is not allowed for native Linux bridges")
@@ -208,9 +210,9 @@ func (d *nicBridged) validateConfig(instConf instance.ConfigReader) error {
 			}
 		}
 
-		// Load managed network. project.Default is used here as bridge networks don't support projects.
+		// Load managed network. api.ProjectDefaultName is used here as bridge networks don't support projects.
 		var err error
-		d.network, err = network.LoadByName(d.state, project.Default, d.config["network"])
+		d.network, err = network.LoadByName(d.state, api.ProjectDefaultName, d.config["network"])
 		if err != nil {
 			return fmt.Errorf("Error loading network config for %q: %w", d.config["network"], err)
 		}
@@ -245,8 +247,8 @@ func (d *nicBridged) validateConfig(instConf instance.ConfigReader) error {
 		requiredFields = append(requiredFields, "parent")
 
 		// Check if parent is a managed network.
-		// project.Default is used here as bridge networks don't support projects.
-		d.network, _ = network.LoadByName(d.state, project.Default, d.config["parent"])
+		// api.ProjectDefaultName is used here as bridge networks don't support projects.
+		d.network, _ = network.LoadByName(d.state, api.ProjectDefaultName, d.config["parent"])
 		if d.network != nil {
 			// Validate NIC settings with managed network.
 			err := checkWithManagedNetwork(d.network)
@@ -380,10 +382,11 @@ func (d *nicBridged) checkAddressConflict() error {
 	}
 
 	// Bridge networks are always in the default project.
-	return network.UsedByInstanceDevices(d.state, project.Default, networkName, "bridge", func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
+	return network.UsedByInstanceDevices(d.state, api.ProjectDefaultName, networkName, "bridge", func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
 		// Skip our own device. This avoids triggering duplicate device errors during
 		// updates or when making temporary copies of our instance during migrations.
-		if instance.IsSameLogicalInstance(d.inst, &inst) && d.Name() == nicName {
+		sameLogicalInstance := instance.IsSameLogicalInstance(d.inst, &inst)
+		if sameLogicalInstance && d.Name() == nicName {
 			return nil
 		}
 
@@ -398,6 +401,10 @@ func (d *nicBridged) checkAddressConflict() error {
 		// Check there isn't another instance with the same DNS name connected to a managed network
 		// that has DNS enabled and is connected to the same untagged VLAN.
 		if d.network != nil && d.network.Config()["dns.mode"] != "none" && nicCheckDNSNameConflict(d.inst.Name(), inst.Name) {
+			if sameLogicalInstance {
+				return api.StatusErrorf(http.StatusConflict, "Instance DNS name %q conflict between %q and %q because both are connected to same network", strings.ToLower(inst.Name), d.name, nicName)
+			}
+
 			return api.StatusErrorf(http.StatusConflict, "Instance DNS name %q already used on network", strings.ToLower(inst.Name))
 		}
 
@@ -450,7 +457,7 @@ func (d *nicBridged) UpdatableFields(oldDevice Type) []string {
 		return []string{}
 	}
 
-	return []string{"limits.ingress", "limits.egress", "limits.max", "ipv4.routes", "ipv6.routes", "ipv4.routes.external", "ipv6.routes.external", "ipv4.address", "ipv6.address", "security.mac_filtering", "security.ipv4_filtering", "security.ipv6_filtering"}
+	return []string{"limits.ingress", "limits.egress", "limits.max", "limits.priority", "ipv4.routes", "ipv6.routes", "ipv4.routes.external", "ipv6.routes.external", "ipv4.address", "ipv6.address", "security.mac_filtering", "security.ipv4_filtering", "security.ipv6_filtering"}
 }
 
 // Add is run when a device is added to a non-snapshot instance whether or not the instance is running.
@@ -500,15 +507,19 @@ func (d *nicBridged) Start() (*deviceConfig.RunConfig, error) {
 	// Create veth pair and configure the peer end with custom hwaddr and mtu if supplied.
 	if d.inst.Type() == instancetype.Container {
 		if saveData["host_name"] == "" {
-			saveData["host_name"] = network.RandomDevName("veth")
+			saveData["host_name"], err = d.generateHostName("veth", d.config["hwaddr"])
+			if err != nil {
+				return nil, err
+			}
 		}
-
 		peerName, mtu, err = networkCreateVethPair(saveData["host_name"], d.config)
 	} else if d.inst.Type() == instancetype.VM {
 		if saveData["host_name"] == "" {
-			saveData["host_name"] = network.RandomDevName("tap")
+			saveData["host_name"], err = d.generateHostName("tap", d.config["hwaddr"])
+			if err != nil {
+				return nil, err
+			}
 		}
-
 		peerName = saveData["host_name"] // VMs use the host_name to link to the TAP FD.
 		mtu, err = networkCreateTap(saveData["host_name"], d.config)
 	}
@@ -547,7 +558,7 @@ func (d *nicBridged) Start() (*deviceConfig.RunConfig, error) {
 	}
 
 	// Apply host-side limits.
-	err = networkSetupHostVethLimits(d.config)
+	err = networkSetupHostVethLimits(&d.deviceCommon, nil, true)
 	if err != nil {
 		return nil, err
 	}
@@ -615,7 +626,13 @@ func (d *nicBridged) Start() (*deviceConfig.RunConfig, error) {
 		}
 
 		if brNetfilterEnabled {
-			listenAddresses, err := d.state.DB.Cluster.GetNetworkForwardListenAddresses(d.network.ID(), true)
+			var listenAddresses map[int64]string
+
+			err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				listenAddresses, err = tx.GetNetworkForwardListenAddresses(ctx, d.network.ID(), true)
+
+				return err
+			})
 			if err != nil {
 				return nil, fmt.Errorf("Failed loading network forwards: %w", err)
 			}
@@ -731,7 +748,7 @@ func (d *nicBridged) Update(oldDevices deviceConfig.Devices, isRunning bool) err
 		}
 
 		// Apply host-side limits.
-		err = networkSetupHostVethLimits(d.config)
+		err = networkSetupHostVethLimits(&d.deviceCommon, oldConfig, true)
 		if err != nil {
 			return err
 		}
@@ -790,6 +807,14 @@ func (d *nicBridged) Stop() (*deviceConfig.RunConfig, error) {
 		return nil, err
 	}
 
+	// Populate device config with volatile fields (hwaddr and host_name) if needed.
+	networkVethFillFromVolatile(d.config, d.volatileGet())
+
+	err = networkClearHostVethLimits(&d.deviceCommon)
+	if err != nil {
+		return nil, err
+	}
+
 	// Setup post-stop actions.
 	runConf := deviceConfig.RunConfig{
 		PostHooks: []func() error{d.postStop},
@@ -1326,7 +1351,7 @@ func (d *nicBridged) networkDHCPv4Release(srcMAC net.HardwareAddr, srcIP net.IP,
 
 	defer func() { _ = conn.Close() }()
 
-	//Random DHCP transaction ID
+	// Random DHCP transaction ID
 	xid := rand.Uint32()
 
 	// Construct a DHCP packet pretending to be from the source IP and MAC supplied.
@@ -1650,14 +1675,14 @@ func (d *nicBridged) State() (*api.InstanceStateNetwork, error) {
 
 		// Add any valid-state neighbour IP entries first.
 		for _, neighIP := range neighIPs {
-			if shared.StringInSlice(string(neighIP.State), validStates) {
+			if shared.ValueInSlice(string(neighIP.State), validStates) {
 				ipStore(neighIP.Addr)
 			}
 		}
 
 		// Add any non-failed-state entries.
 		for _, neighIP := range neighIPs {
-			if neighIP.State != ip.NeighbourIPStateFailed && !shared.StringInSlice(string(neighIP.State), validStates) {
+			if neighIP.State != ip.NeighbourIPStateFailed && !shared.ValueInSlice(string(neighIP.State), validStates) {
 				ipStore(neighIP.Addr)
 			}
 		}
