@@ -2,31 +2,33 @@ package device
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"net"
 	"net/http"
 	"os"
+	"strconv"
 	"strings"
 
 	"github.com/mdlayher/netx/eui64"
 
-	"github.com/lxc/lxd/lxd/db"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	pcidev "github.com/lxc/lxd/lxd/device/pci"
-	"github.com/lxc/lxd/lxd/dnsmasq/dhcpalloc"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/ip"
-	"github.com/lxc/lxd/lxd/network"
-	"github.com/lxc/lxd/lxd/network/acl"
-	"github.com/lxc/lxd/lxd/network/openvswitch"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/resources"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	pcidev "github.com/canonical/lxd/lxd/device/pci"
+	"github.com/canonical/lxd/lxd/dnsmasq/dhcpalloc"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/ip"
+	"github.com/canonical/lxd/lxd/network"
+	"github.com/canonical/lxd/lxd/network/acl"
+	"github.com/canonical/lxd/lxd/network/openvswitch"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/resources"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // ovnNet defines an interface for accessing instance specific functions on OVN network.
@@ -35,10 +37,10 @@ type ovnNet interface {
 
 	InstanceDevicePortValidateExternalRoutes(deviceInstance instance.Instance, deviceName string, externalRoutes []*net.IPNet) error
 	InstanceDevicePortAdd(instanceUUID string, deviceName string, deviceConfig deviceConfig.Device) error
-	InstanceDevicePortStart(opts *network.OVNInstanceNICSetupOpts, securityACLsRemove []string) (openvswitch.OVNSwitchPort, error)
+	InstanceDevicePortStart(opts *network.OVNInstanceNICSetupOpts, securityACLsRemove []string) (openvswitch.OVNSwitchPort, []net.IP, error)
 	InstanceDevicePortStop(ovsExternalOVNPort openvswitch.OVNSwitchPort, opts *network.OVNInstanceNICStopOpts) error
 	InstanceDevicePortRemove(instanceUUID string, deviceName string, deviceConfig deviceConfig.Device) error
-	InstanceDevicePortDynamicIPs(instanceUUID string, deviceName string) ([]net.IP, error)
+	InstanceDevicePortIPs(instanceUUID string, deviceName string) ([]net.IP, error)
 }
 
 type nicOVN struct {
@@ -47,6 +49,11 @@ type nicOVN struct {
 	network ovnNet // Populated in validateConfig().
 }
 
+// CanHotPlug returns whether the device can be managed whilst the instance is running.
+func (d *nicOVN) CanHotPlug() bool {
+	return true
+}
+
 // CanMigrate returns whether the device can be migrated to any other cluster member.
 func (d *nicOVN) CanMigrate() bool {
 	return true
@@ -91,6 +98,8 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 		"security.acls.default.ingress.logged",
 		"security.acls.default.egress.logged",
 		"acceleration",
+		"nested",
+		"vlan",
 	}
 
 	// The NIC's network may be a non-default project, so lookup project and get network's project name.
@@ -157,6 +166,11 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 			return fmt.Errorf("Cannot specify %q when DHCP or %q are disabled on network %q", "ipv6.address", "ipv6.dhcp.stateful", d.config["network"])
 		}
 
+		// Static IPv6 is allowed only if static IPv4 is set as well.
+		if d.config["ipv4.address"] == "" {
+			return fmt.Errorf("Cannot specify %q when %q is not set", "ipv6.address", "ipv4.address")
+		}
+
 		ip, subnet, err := net.ParseCIDR(netConfig["ipv6.address"])
 		if err != nil {
 			return fmt.Errorf("Invalid network ipv6.address: %w", err)
@@ -177,9 +191,52 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 	// Apply network level config options to device config before validation.
 	d.config["mtu"] = netConfig["bridge.mtu"]
 
-	// Check there isn't another NIC with any of the same addresses specified on the same network.
-	// Can only validate this when the instance is supplied (and not doing profile validation).
+	// Check VLAN ID is valid.
+	if d.config["vlan"] != "" {
+		nestedVLAN, err := strconv.ParseUint(d.config["vlan"], 10, 16)
+		if err != nil {
+			return fmt.Errorf("Invalid VLAN ID %q: %w", d.config["vlan"], err)
+		}
+
+		if nestedVLAN < 1 || nestedVLAN > 4095 {
+			return fmt.Errorf("Invalid VLAN ID %q: Must be between 1 and 4095 inclusive", d.config["vlan"])
+		}
+	}
+
+	// Perform checks that require instance (those not appropriate to do during profile validation).
 	if d.inst != nil {
+		// Check nested VLAN combination settings are valid. Requires instance for validation as settings
+		// may come from a combination of profile and instance configs.
+		if d.config["nested"] != "" {
+			if d.config["vlan"] == "" {
+				return fmt.Errorf("VLAN must be specified with a nested NIC")
+			}
+
+			// Check the NIC that this NIC is neted under exists on this instance and shares same
+			// parent network.
+			var nestedParentNIC string
+			for devName, devConfig := range instConf.ExpandedDevices() {
+				if devName != d.config["nested"] || devConfig["type"] != "nic" {
+					continue
+				}
+
+				if devConfig["network"] != d.config["network"] {
+					return fmt.Errorf("The nested parent NIC must be connected to same network as this NIC")
+				}
+
+				nestedParentNIC = devName
+				break
+			}
+
+			if nestedParentNIC == "" {
+				return fmt.Errorf("Instance does not have a NIC called %q for nesting under", d.config["nested"])
+			}
+		} else if d.config["vlan"] != "" {
+			return fmt.Errorf("Specifying a VLAN requires that this NIC be nested")
+		}
+
+		// Check there isn't another NIC with any of the same addresses specified on the same network.
+		// Can only validate this when the instance is supplied (and not doing profile validation).
 		err := d.checkAddressConflict()
 		if err != nil {
 			return err
@@ -242,12 +299,18 @@ func (d *nicOVN) checkAddressConflict() error {
 	return network.UsedByInstanceDevices(d.state, d.network.Project(), d.network.Name(), d.network.Type(), func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
 		// Skip our own device. This avoids triggering duplicate device errors during
 		// updates or when making temporary copies of our instance during migrations.
-		if instance.IsSameLogicalInstance(d.inst, &inst) && d.Name() == nicName {
+		sameLogicalInstance := instance.IsSameLogicalInstance(d.inst, &inst)
+		if sameLogicalInstance && d.Name() == nicName {
 			return nil
 		}
 
 		// Check there isn't another instance with the same DNS name connected to managed network.
-		if d.network != nil && nicCheckDNSNameConflict(d.inst.Name(), inst.Name) {
+		sameLogicalInstanceNestedNIC := sameLogicalInstance && (d.config["nested"] != "" || nicConfig["nested"] != "")
+		if d.network != nil && !sameLogicalInstanceNestedNIC && nicCheckDNSNameConflict(d.inst.Name(), inst.Name) {
+			if sameLogicalInstance {
+				return api.StatusErrorf(http.StatusConflict, "Instance DNS name %q conflict between %q and %q because both are connected to same network", strings.ToLower(inst.Name), d.name, nicName)
+			}
+
 			return api.StatusErrorf(http.StatusConflict, "Instance DNS name %q already used on network", strings.ToLower(inst.Name))
 		}
 
@@ -329,132 +392,304 @@ func (d *nicOVN) Start() (*deviceConfig.RunConfig, error) {
 
 	// Load uplink network config.
 	uplinkNetworkName := d.network.Config()["network"]
-	_, uplink, _, err := d.state.DB.Cluster.GetNetworkInAnyState(project.Default, uplinkNetworkName)
+
+	var uplink *api.Network
+
+	err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		_, uplink, _, err = tx.GetNetworkInAnyState(ctx, api.ProjectDefaultName, uplinkNetworkName)
+
+		return err
+	})
 	if err != nil {
 		return nil, fmt.Errorf("Failed to load uplink network %q: %w", uplinkNetworkName, err)
 	}
 
-	// Setup the network interface pair.
-	var peerName string
+	// Setup the host network interface (if not nested).
+	var peerName, integrationBridgeNICName string
 	var mtu uint32
 	var vfPCIDev pcidev.Device
+	var vDPADevice *ip.VDPADev
 	var pciIOMMUGroup uint64
 
-	if d.config["acceleration"] == "sriov" {
-		ovs := openvswitch.NewOVS()
-		if !ovs.HardwareOffloadingEnabled() {
-			return nil, fmt.Errorf("SR-IOV acceleration requires hardware offloading be enabled in OVS")
-		}
-
-		// If VM, then try and load the vfio-pci module first.
-		if d.inst.Type() == instancetype.VM {
-			err := util.LoadModule("vfio-pci")
-			if err != nil {
-				return nil, fmt.Errorf("Error loading %q module: %w", "vfio-pci", err)
-			}
-		}
-
-		integrationBridge := d.state.GlobalConfig.NetworkOVNIntegrationBridge()
-
-		// Find free VF exclusively.
-		network.SRIOVVirtualFunctionMutex.Lock()
-		vfParent, vfRepresentor, vfDev, vfID, err := network.SRIOVFindFreeVFAndRepresentor(d.state, integrationBridge)
-		if err != nil {
-			network.SRIOVVirtualFunctionMutex.Unlock()
-			return nil, fmt.Errorf("Failed finding a suitable free virtual function on %q: %w", integrationBridge, err)
-		}
-
-		// Claim the SR-IOV virtual function (VF) on the parent (PF) and get the PCI information.
-		vfPCIDev, pciIOMMUGroup, err = networkSRIOVSetupVF(d.deviceCommon, vfParent, vfDev, vfID, false, saveData)
-		if err != nil {
-			network.SRIOVVirtualFunctionMutex.Unlock()
-			return nil, err
-		}
-
-		network.SRIOVVirtualFunctionMutex.Unlock()
-
-		// Setup the guest network interface.
-		if d.inst.Type() == instancetype.Container {
-			err := networkSRIOVSetupContainerVFNIC(saveData["host_name"], d.config)
-			if err != nil {
-				return nil, err
-			}
-		}
-
-		peerName = vfDev
-		saveData["host_name"] = vfRepresentor
+	if d.config["nested"] != "" {
+		delete(saveData, "host_name") // Nested NICs don't have a host side interface.
 	} else {
-		// Create veth pair and configure the peer end with custom hwaddr and mtu if supplied.
-		if d.inst.Type() == instancetype.Container {
-			if saveData["host_name"] == "" {
-				saveData["host_name"] = network.RandomDevName("veth")
+		if d.config["acceleration"] == "sriov" {
+			ovs := openvswitch.NewOVS()
+			if !ovs.HardwareOffloadingEnabled() {
+				return nil, fmt.Errorf("SR-IOV acceleration requires hardware offloading be enabled in OVS")
 			}
 
-			peerName, mtu, err = networkCreateVethPair(saveData["host_name"], d.config)
-			if err != nil {
-				return nil, err
-			}
-		} else if d.inst.Type() == instancetype.VM {
-			if saveData["host_name"] == "" {
-				saveData["host_name"] = network.RandomDevName("tap")
+			// If VM, then try and load the vfio-pci module first.
+			if d.inst.Type() == instancetype.VM {
+				err := util.LoadModule("vfio-pci")
+				if err != nil {
+					return nil, fmt.Errorf("Error loading %q module: %w", "vfio-pci", err)
+				}
 			}
 
-			peerName = saveData["host_name"] // VMs use the host_name to link to the TAP FD.
-			mtu, err = networkCreateTap(saveData["host_name"], d.config)
+			integrationBridge := d.state.GlobalConfig.NetworkOVNIntegrationBridge()
+
+			// Find free VF exclusively.
+			network.SRIOVVirtualFunctionMutex.Lock()
+			vfParent, vfRepresentor, vfDev, vfID, err := network.SRIOVFindFreeVFAndRepresentor(d.state, integrationBridge)
 			if err != nil {
+				network.SRIOVVirtualFunctionMutex.Unlock()
+				return nil, fmt.Errorf("Failed finding a suitable free virtual function on %q: %w", integrationBridge, err)
+			}
+
+			// Claim the SR-IOV virtual function (VF) on the parent (PF) and get the PCI information.
+			vfPCIDev, pciIOMMUGroup, err = networkSRIOVSetupVF(d.deviceCommon, vfParent, vfDev, vfID, false, saveData)
+			if err != nil {
+				network.SRIOVVirtualFunctionMutex.Unlock()
+				return nil, fmt.Errorf("Failed setting up VF: %w", err)
+			}
+
+			revert.Add(func() {
+				_ = networkSRIOVRestoreVF(d.deviceCommon, false, saveData)
+			})
+
+			network.SRIOVVirtualFunctionMutex.Unlock()
+
+			// Setup the guest network interface.
+			if d.inst.Type() == instancetype.Container {
+				err := networkSRIOVSetupContainerVFNIC(saveData["host_name"], d.config)
+				if err != nil {
+					return nil, fmt.Errorf("Failed setting up container VF NIC: %w", err)
+				}
+			}
+
+			integrationBridgeNICName = vfRepresentor
+			peerName = vfDev
+		} else if d.config["acceleration"] == "vdpa" {
+			ovs := openvswitch.NewOVS()
+			if !ovs.HardwareOffloadingEnabled() {
+				return nil, fmt.Errorf("SR-IOV acceleration requires hardware offloading be enabled in OVS")
+			}
+
+			err := util.LoadModule("vdpa")
+			if err != nil {
+				return nil, fmt.Errorf("Error loading %q module: %w", "vdpa", err)
+			}
+
+			// If VM, then try and load the vhost_vdpa module first.
+			if d.inst.Type() == instancetype.VM {
+				err = util.LoadModule("vhost_vdpa")
+				if err != nil {
+					return nil, fmt.Errorf("Error loading %q module: %w", "vhost_vdpa", err)
+				}
+			}
+
+			integrationBridge := d.state.GlobalConfig.NetworkOVNIntegrationBridge()
+
+			// Find free VF exclusively.
+			network.SRIOVVirtualFunctionMutex.Lock()
+			vfParent, vfRepresentor, vfDev, vfID, err := network.SRIOVFindFreeVFAndRepresentor(d.state, integrationBridge)
+			if err != nil {
+				network.SRIOVVirtualFunctionMutex.Unlock()
+				return nil, fmt.Errorf("Failed finding a suitable free virtual function on %q: %w", integrationBridge, err)
+			}
+
+			// Claim the SR-IOV virtual function (VF) on the parent (PF) and get the PCI information.
+			vfPCIDev, pciIOMMUGroup, err = networkSRIOVSetupVF(d.deviceCommon, vfParent, vfDev, vfID, false, saveData)
+			if err != nil {
+				network.SRIOVVirtualFunctionMutex.Unlock()
 				return nil, err
 			}
+
+			revert.Add(func() {
+				_ = networkSRIOVRestoreVF(d.deviceCommon, false, saveData)
+			})
+
+			// Create the vDPA management device
+			vDPADevice, err = ip.AddVDPADevice(vfPCIDev.SlotName, saveData)
+			if err != nil {
+				network.SRIOVVirtualFunctionMutex.Unlock()
+				return nil, err
+			}
+
+			network.SRIOVVirtualFunctionMutex.Unlock()
+
+			// Setup the guest network interface.
+			if d.inst.Type() == instancetype.Container {
+				return nil, fmt.Errorf("VDPA acceleration is not supported for containers")
+			}
+
+			integrationBridgeNICName = vfRepresentor
+			peerName = vfDev
+		} else {
+			// Create veth pair and configure the peer end with custom hwaddr and mtu if supplied.
+			if d.inst.Type() == instancetype.Container {
+				if saveData["host_name"] == "" {
+					saveData["host_name"], err = d.generateHostName("veth", d.config["hwaddr"])
+					if err != nil {
+						return nil, err
+					}
+				}
+
+				integrationBridgeNICName = saveData["host_name"]
+				peerName, mtu, err = networkCreateVethPair(saveData["host_name"], d.config)
+				if err != nil {
+					return nil, err
+				}
+			} else if d.inst.Type() == instancetype.VM {
+				if saveData["host_name"] == "" {
+					saveData["host_name"], err = d.generateHostName("tap", d.config["hwaddr"])
+					if err != nil {
+						return nil, err
+					}
+				}
+
+				integrationBridgeNICName = saveData["host_name"]
+				peerName = saveData["host_name"] // VMs use the host_name to link to the TAP FD.
+				mtu, err = networkCreateTap(saveData["host_name"], d.config)
+				if err != nil {
+					return nil, err
+				}
+			}
+
+			revert.Add(func() { _ = network.InterfaceRemove(saveData["host_name"]) })
 		}
-
-		revert.Add(func() { _ = network.InterfaceRemove(saveData["host_name"]) })
 	}
 
 	// Populate device config with volatile fields if needed.
 	networkVethFillFromVolatile(d.config, saveData)
 
-	cleanup, err := d.setupHostNIC(saveData["host_name"], uplink)
-	if err != nil {
-		return nil, err
+	v := d.volatileGet()
+
+	// Retrieve any last state IPs from volatile and pass them to OVN driver for potential use with sticky
+	// DHCPv4 allocations.
+	var lastStateIPs []net.IP
+	for _, ipStr := range shared.SplitNTrimSpace(v["last_state.ip_addresses"], ",", -1, true) {
+		lastStateIP := net.ParseIP(ipStr)
+		if lastStateIP != nil {
+			lastStateIPs = append(lastStateIPs, lastStateIP)
+		}
 	}
 
-	revert.Add(cleanup)
+	// Add new OVN logical switch port for instance.
+	logicalPortName, dnsIPs, err := d.network.InstanceDevicePortStart(&network.OVNInstanceNICSetupOpts{
+		InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
+		DNSName:      d.inst.Name(),
+		DeviceName:   d.name,
+		DeviceConfig: d.config,
+		UplinkConfig: uplink.Config,
+		LastStateIPs: lastStateIPs, // Pass in volatile last state IPs for use with sticky DHCPv4 hint.
+	}, nil)
+	if err != nil {
+		return nil, fmt.Errorf("Failed setting up OVN port: %w", err)
+	}
+
+	// Record switch port DNS IPs to volatile so they can be used as sticky DHCPv4 hint in the future in order
+	// to allocate the same IPs on next start if they are still available/appropriate.
+	// This volatile key will not be removed when instance stops.
+	var dnsIPsStr strings.Builder
+	for i, dnsIP := range dnsIPs {
+		if i > 0 {
+			dnsIPsStr.WriteString(",")
+		}
+
+		dnsIPsStr.WriteString(dnsIP.String())
+	}
+
+	saveData["last_state.ip_addresses"] = dnsIPsStr.String()
+
+	revert.Add(func() {
+		_ = d.network.InstanceDevicePortStop("", &network.OVNInstanceNICStopOpts{
+			InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
+			DeviceName:   d.name,
+			DeviceConfig: d.config,
+		})
+	})
+
+	// Associated host side interface to OVN logical switch port (if not nested).
+	if integrationBridgeNICName != "" {
+		cleanup, err := d.setupHostNIC(integrationBridgeNICName, logicalPortName, uplink)
+		if err != nil {
+			return nil, err
+		}
+
+		revert.Add(cleanup)
+	}
+
+	runConf := deviceConfig.RunConfig{}
+
+	// Get local chassis ID for chassis group.
+	ovs := openvswitch.NewOVS()
+	chassisID, err := ovs.ChassisID()
+	if err != nil {
+		return nil, fmt.Errorf("Failed getting OVS Chassis ID: %w", err)
+	}
+
+	ovnClient, err := openvswitch.NewOVN(d.state)
+	if err != nil {
+		return nil, fmt.Errorf("Failed to get OVN client: %w", err)
+	}
+
+	// Add post start hook for setting logical switch port chassis once instance has been started.
+	runConf.PostHooks = append(runConf.PostHooks, func() error {
+		err := ovnClient.LogicalSwitchPortOptionsSet(logicalPortName, map[string]string{"requested-chassis": chassisID})
+		if err != nil {
+			return fmt.Errorf("Failed setting logical switch port chassis ID: %w", err)
+		}
+
+		return nil
+	})
+
+	runConf.PostHooks = append(runConf.PostHooks, d.postStart)
 
 	err = d.volatileSet(saveData)
 	if err != nil {
 		return nil, err
 	}
 
-	runConf := deviceConfig.RunConfig{}
-	runConf.PostHooks = []func() error{d.postStart}
-	runConf.NetworkInterface = []deviceConfig.RunConfigItem{
-		{Key: "type", Value: "phys"},
-		{Key: "name", Value: d.config["name"]},
-		{Key: "flags", Value: "up"},
-		{Key: "link", Value: peerName},
-	}
+	// Return instance network interface configuration (if not nested).
+	if saveData["host_name"] != "" {
+		runConf.NetworkInterface = []deviceConfig.RunConfigItem{
+			{Key: "type", Value: "phys"},
+			{Key: "name", Value: d.config["name"]},
+			{Key: "flags", Value: "up"},
+			{Key: "link", Value: peerName},
+		}
 
-	instType := d.inst.Type()
-	if instType == instancetype.VM {
-		if d.config["acceleration"] == "sriov" {
-			runConf.NetworkInterface = append(runConf.NetworkInterface,
-				[]deviceConfig.RunConfigItem{
-					{Key: "devName", Value: d.name},
-					{Key: "pciSlotName", Value: vfPCIDev.SlotName},
-					{Key: "pciIOMMUGroup", Value: fmt.Sprintf("%d", pciIOMMUGroup)},
-					{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
-				}...)
-		} else {
+		instType := d.inst.Type()
+		if instType == instancetype.VM {
+			if d.config["acceleration"] == "sriov" {
+				runConf.NetworkInterface = append(runConf.NetworkInterface,
+					[]deviceConfig.RunConfigItem{
+						{Key: "devName", Value: d.name},
+						{Key: "pciSlotName", Value: vfPCIDev.SlotName},
+						{Key: "pciIOMMUGroup", Value: fmt.Sprintf("%d", pciIOMMUGroup)},
+						{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
+					}...)
+			} else if d.config["acceleration"] == "vdpa" {
+				if vDPADevice == nil {
+					return nil, fmt.Errorf("vDPA device is nil")
+				}
+
+				runConf.NetworkInterface = append(runConf.NetworkInterface,
+					[]deviceConfig.RunConfigItem{
+						{Key: "devName", Value: d.name},
+						{Key: "pciSlotName", Value: vfPCIDev.SlotName},
+						{Key: "pciIOMMUGroup", Value: fmt.Sprintf("%d", pciIOMMUGroup)},
+						{Key: "maxVQP", Value: fmt.Sprintf("%d", vDPADevice.MaxVQs/2)},
+						{Key: "vDPADevName", Value: vDPADevice.Name},
+						{Key: "vhostVDPAPath", Value: vDPADevice.VhostVDPA.Path},
+						{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
+					}...)
+			} else {
+				runConf.NetworkInterface = append(runConf.NetworkInterface,
+					[]deviceConfig.RunConfigItem{
+						{Key: "devName", Value: d.name},
+						{Key: "hwaddr", Value: d.config["hwaddr"]},
+						{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
+					}...)
+			}
+		} else if instType == instancetype.Container {
 			runConf.NetworkInterface = append(runConf.NetworkInterface,
-				[]deviceConfig.RunConfigItem{
-					{Key: "devName", Value: d.name},
-					{Key: "hwaddr", Value: d.config["hwaddr"]},
-					{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
-				}...)
+				deviceConfig.RunConfigItem{Key: "hwaddr", Value: d.config["hwaddr"]},
+			)
 		}
-	} else if instType == instancetype.Container {
-		runConf.NetworkInterface = append(runConf.NetworkInterface,
-			deviceConfig.RunConfigItem{Key: "hwaddr", Value: d.config["hwaddr"]},
-		)
 	}
 
 	revert.Success()
@@ -501,7 +736,7 @@ func (d *nicOVN) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
 		newACLs := shared.SplitNTrimSpace(d.config["security.acls"], ",", -1, true)
 		removedACLs := []string{}
 		for _, oldACL := range oldACLs {
-			if !shared.StringInSlice(oldACL, newACLs) {
+			if !shared.ValueInSlice(oldACL, newACLs) {
 				removedACLs = append(removedACLs, oldACL)
 			}
 		}
@@ -510,13 +745,22 @@ func (d *nicOVN) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
 		if isRunning {
 			// Load uplink network config.
 			uplinkNetworkName := d.network.Config()["network"]
-			_, uplink, _, err := d.state.DB.Cluster.GetNetworkInAnyState(project.Default, uplinkNetworkName)
+
+			var uplink *api.Network
+
+			err := d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				var err error
+
+				_, uplink, _, err = tx.GetNetworkInAnyState(ctx, api.ProjectDefaultName, uplinkNetworkName)
+
+				return err
+			})
 			if err != nil {
 				return fmt.Errorf("Failed to load uplink network %q: %w", uplinkNetworkName, err)
 			}
 
 			// Update OVN logical switch port for instance.
-			_, err = d.network.InstanceDevicePortStart(&network.OVNInstanceNICSetupOpts{
+			_, _, err = d.network.InstanceDevicePortStart(&network.OVNInstanceNICSetupOpts{
 				InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
 				DNSName:      d.inst.Name(),
 				DeviceName:   d.name,
@@ -555,47 +799,91 @@ func (d *nicOVN) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
 	return nil
 }
 
+func (d *nicOVN) findRepresentorPort(volatile map[string]string) (string, error) {
+	physSwitchID, pfID, err := network.SRIOVGetSwitchAndPFID(volatile["last_state.vf.parent"])
+	if err != nil {
+		return "", fmt.Errorf("Failed finding physical parent switch and PF ID to release representor port: %w", err)
+	}
+
+	sysClassNet := "/sys/class/net"
+	nics, err := os.ReadDir(sysClassNet)
+	if err != nil {
+		return "", fmt.Errorf("Failed reading NICs directory %q: %w", sysClassNet, err)
+	}
+
+	vfID, err := strconv.Atoi(volatile["last_state.vf.id"])
+	if err != nil {
+		return "", fmt.Errorf("Failed parsing last VF ID %q: %w", volatile["last_state.vf.id"], err)
+	}
+
+	// Track down the representor port to remove it from the integration bridge.
+	representorPort := network.SRIOVFindRepresentorPort(nics, string(physSwitchID), pfID, vfID)
+	if representorPort == "" {
+		return "", fmt.Errorf("Failed finding representor")
+	}
+
+	return representorPort, nil
+}
+
 // Stop is run when the device is removed from the instance.
 func (d *nicOVN) Stop() (*deviceConfig.RunConfig, error) {
 	runConf := deviceConfig.RunConfig{
 		PostHooks: []func() error{d.postStop},
 	}
 
+	v := d.volatileGet()
+
+	var err error
+
 	// Try and retrieve the last associated OVN switch port for the instance interface in the local OVS DB.
 	// If we cannot get this, don't fail, as InstanceDevicePortStop will then try and generate the likely
 	// port name using the same regime it does for new ports. This part is only here in order to allow
 	// instance ports generated under an older regime to be cleaned up properly.
-	networkVethFillFromVolatile(d.config, d.volatileGet())
+	networkVethFillFromVolatile(d.config, v)
 	ovs := openvswitch.NewOVS()
-	ovsExternalOVNPort, err := ovs.InterfaceAssociatedOVNSwitchPort(d.config["host_name"])
-	if err != nil {
-		d.logger.Warn("Could not find OVN Switch port associated to OVS interface", logger.Ctx{"interface": d.config["host_name"]})
-	}
-
-	// If there is a host_name specified, then try and remove it from the OVS integration bridge.
-	// Do this early on during the stop process to prevent any future error from leaving the OVS port present
-	// as if the instance is being migrated, this can cause port conflicts in OVN if the instance comes up on
-	// another LXD host later.
-	if d.config["host_name"] != "" {
-		integrationBridge := d.state.GlobalConfig.NetworkOVNIntegrationBridge()
-
-		// Detach host-side end of veth pair from OVS integration bridge.
-		err = ovs.BridgePortDelete(integrationBridge, d.config["host_name"])
+	var ovsExternalOVNPort openvswitch.OVNSwitchPort
+	if d.config["nested"] == "" {
+		ovsExternalOVNPort, err = ovs.InterfaceAssociatedOVNSwitchPort(d.config["host_name"])
 		if err != nil {
-			// Don't fail here as we want the postStop hook to run to clean up the local veth pair.
-			d.logger.Error("Failed detaching interface from OVS integration bridge", logger.Ctx{"interface": d.config["host_name"], "bridge": integrationBridge, "err": err})
+			d.logger.Warn("Could not find OVN Switch port associated to OVS interface", logger.Ctx{"interface": d.config["host_name"]})
 		}
 	}
 
-	instanceUUID := d.inst.LocalConfig()["volatile.uuid"]
-	err = d.network.InstanceDevicePortStop(ovsExternalOVNPort, &network.OVNInstanceNICStopOpts{
-		InstanceUUID: instanceUUID,
-		DeviceName:   d.name,
-		DeviceConfig: d.config,
-	})
-	if err != nil {
-		// Don't fail here as we still want the postStop hook to run to clean up the local veth pair.
-		d.logger.Error("Failed to remove OVN device port", logger.Ctx{"err": err})
+	integrationBridgeNICName := d.config["host_name"]
+	if d.config["acceleration"] == "sriov" || d.config["acceleration"] == "vdpa" {
+		integrationBridgeNICName, err = d.findRepresentorPort(v)
+		if err != nil {
+			d.logger.Error("Failed finding representor port to detach from OVS integration bridge", logger.Ctx{"err": err})
+		}
+	}
+
+	// If there is integrationBridgeNICName specified, then try and remove it from the OVS integration bridge.
+	// Do this early on during the stop process to prevent any future error from leaving the OVS port present
+	// as if the instance is being migrated, this can cause port conflicts in OVN if the instance comes up on
+	// another LXD host later.
+	if integrationBridgeNICName != "" {
+		integrationBridge := d.state.GlobalConfig.NetworkOVNIntegrationBridge()
+
+		// Detach host-side end of veth pair from OVS integration bridge.
+		err = ovs.BridgePortDelete(integrationBridge, integrationBridgeNICName)
+		if err != nil {
+			// Don't fail here as we want the postStop hook to run to clean up the local veth pair.
+			d.logger.Error("Failed detaching interface from OVS integration bridge", logger.Ctx{"interface": integrationBridgeNICName, "bridge": integrationBridge, "err": err})
+		}
+	}
+
+	// If the devices config is invalid validateConfig() won't populate this field.
+	if d.network != nil {
+		instanceUUID := d.inst.LocalConfig()["volatile.uuid"]
+		err = d.network.InstanceDevicePortStop(ovsExternalOVNPort, &network.OVNInstanceNICStopOpts{
+			InstanceUUID: instanceUUID,
+			DeviceName:   d.name,
+			DeviceConfig: d.config,
+		})
+		if err != nil {
+			// Don't fail here as we still want the postStop hook to run to clean up the local veth pair.
+			d.logger.Error("Failed to remove OVN device port", logger.Ctx{"err": err})
+		}
 	}
 
 	// Remove BGP announcements.
@@ -615,6 +903,7 @@ func (d *nicOVN) postStop() error {
 			"last_state.hwaddr":        "",
 			"last_state.mtu":           "",
 			"last_state.created":       "",
+			"last_state.vdpa.name":     "",
 			"last_state.vf.parent":     "",
 			"last_state.vf.id":         "",
 			"last_state.vf.hwaddr":     "",
@@ -628,29 +917,58 @@ func (d *nicOVN) postStop() error {
 
 	networkVethFillFromVolatile(d.config, v)
 
-	if d.config["host_name"] != "" && shared.PathExists(fmt.Sprintf("/sys/class/net/%s", d.config["host_name"])) {
-		if d.config["acceleration"] == "sriov" {
-			// Restoring host-side interface.
-			network.SRIOVVirtualFunctionMutex.Lock()
-			err := networkSRIOVRestoreVF(d.deviceCommon, false, v)
-			if err != nil {
-				network.SRIOVVirtualFunctionMutex.Unlock()
-				return err
-			}
-
+	if d.config["acceleration"] == "sriov" {
+		// Restoring host-side interface.
+		network.SRIOVVirtualFunctionMutex.Lock()
+		err := networkSRIOVRestoreVF(d.deviceCommon, false, v)
+		if err != nil {
 			network.SRIOVVirtualFunctionMutex.Unlock()
+			return err
+		}
 
-			link := &ip.Link{Name: d.config["host_name"]}
-			err = link.SetDown()
-			if err != nil {
-				return fmt.Errorf("Failed to bring down the host interface %s: %w", d.config["host_name"], err)
-			}
-		} else {
-			// Removing host-side end of veth pair will delete the peer end too.
-			err := network.InterfaceRemove(d.config["host_name"])
-			if err != nil {
-				return fmt.Errorf("Failed to remove interface %q: %w", d.config["host_name"], err)
-			}
+		network.SRIOVVirtualFunctionMutex.Unlock()
+
+		link := &ip.Link{Name: d.config["host_name"]}
+		err = link.SetDown()
+		if err != nil {
+			return fmt.Errorf("Failed to bring down the host interface %s: %w", d.config["host_name"], err)
+		}
+	} else if d.config["acceleration"] == "vdpa" {
+		// Retrieve the last state vDPA device name.
+		network.SRIOVVirtualFunctionMutex.Lock()
+		vDPADevName, ok := v["last_state.vdpa.name"]
+		if !ok {
+			network.SRIOVVirtualFunctionMutex.Unlock()
+			return fmt.Errorf("Failed to find PCI slot name for vDPA device")
+		}
+
+		// Delete the vDPA management device.
+		err := ip.DeleteVDPADevice(vDPADevName)
+		if err != nil {
+			network.SRIOVVirtualFunctionMutex.Unlock()
+			return err
+		}
+
+		// Restoring host-side interface.
+		network.SRIOVVirtualFunctionMutex.Lock()
+		err = networkSRIOVRestoreVF(d.deviceCommon, false, v)
+		if err != nil {
+			network.SRIOVVirtualFunctionMutex.Unlock()
+			return err
+		}
+
+		network.SRIOVVirtualFunctionMutex.Unlock()
+
+		link := &ip.Link{Name: d.config["host_name"]}
+		err = link.SetDown()
+		if err != nil {
+			return fmt.Errorf("Failed to bring down the host interface %q: %w", d.config["host_name"], err)
+		}
+	} else if d.config["host_name"] != "" && shared.PathExists(fmt.Sprintf("/sys/class/net/%s", d.config["host_name"])) {
+		// Removing host-side end of veth pair will delete the peer end too.
+		err := network.InterfaceRemove(d.config["host_name"])
+		if err != nil {
+			return fmt.Errorf("Failed to remove interface %q: %w", d.config["host_name"], err)
 		}
 	}
 
@@ -703,26 +1021,26 @@ func (d *nicOVN) State() (*api.InstanceStateNetwork, error) {
 	// OVN only supports dynamic IP allocation if neither IPv4 or IPv6 are statically set.
 	if d.config["ipv4.address"] == "" && d.config["ipv6.address"] == "" {
 		instanceUUID := d.inst.LocalConfig()["volatile.uuid"]
-		dynamicIPs, err := d.network.InstanceDevicePortDynamicIPs(instanceUUID, d.name)
+		devIPs, err := d.network.InstanceDevicePortIPs(instanceUUID, d.name)
 		if err == nil {
-			for _, dynamicIP := range dynamicIPs {
+			for _, devIP := range devIPs {
 				family := "inet"
 				netmask := v4mask
 
-				if dynamicIP.To4() == nil {
+				if devIP.To4() == nil {
 					family = "inet6"
 					netmask = v6mask
 				}
 
 				addresses = append(addresses, api.InstanceStateNetworkAddress{
 					Family:  family,
-					Address: dynamicIP.String(),
+					Address: devIP.String(),
 					Netmask: netmask,
 					Scope:   "global",
 				})
 			}
 		} else {
-			d.logger.Warn("Failed getting OVN port dynamic IPs", logger.Ctx{"err": err})
+			d.logger.Warn("Failed getting OVN port device IPs", logger.Ctx{"err": err})
 		}
 	} else {
 		if d.config["ipv4.address"] != "" {
@@ -807,7 +1125,7 @@ func (d *nicOVN) Register() error {
 	return nil
 }
 
-func (d *nicOVN) setupHostNIC(hostName string, uplink *api.Network) (revert.Hook, error) {
+func (d *nicOVN) setupHostNIC(hostName string, ovnPortName openvswitch.OVNSwitchPort, uplink *api.Network) (revert.Hook, error) {
 	revert := revert.New()
 	defer revert.Fail()
 
@@ -824,26 +1142,6 @@ func (d *nicOVN) setupHostNIC(hostName string, uplink *api.Network) (revert.Hook
 		return nil, err
 	}
 
-	// Add new OVN logical switch port for instance.
-	logicalPortName, err := d.network.InstanceDevicePortStart(&network.OVNInstanceNICSetupOpts{
-		InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
-		DNSName:      d.inst.Name(),
-		DeviceName:   d.name,
-		DeviceConfig: d.config,
-		UplinkConfig: uplink.Config,
-	}, nil)
-	if err != nil {
-		return nil, fmt.Errorf("Failed setting up OVN port: %w", err)
-	}
-
-	revert.Add(func() {
-		_ = d.network.InstanceDevicePortStop("", &network.OVNInstanceNICStopOpts{
-			InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
-			DeviceName:   d.name,
-			DeviceConfig: d.config,
-		})
-	})
-
 	// Attach host side veth interface to bridge.
 	integrationBridge := d.state.GlobalConfig.NetworkOVNIntegrationBridge()
 
@@ -856,7 +1154,7 @@ func (d *nicOVN) setupHostNIC(hostName string, uplink *api.Network) (revert.Hook
 	revert.Add(func() { _ = ovs.BridgePortDelete(integrationBridge, hostName) })
 
 	// Link OVS port to OVN logical port.
-	err = ovs.InterfaceAssociateOVNSwitchPort(hostName, logicalPortName)
+	err = ovs.InterfaceAssociateOVNSwitchPort(hostName, ovnPortName)
 	if err != nil {
 		return nil, err
 	}
