@@ -9,14 +9,14 @@ import (
 	"strings"
 	"time"
 
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/subprocess"
-	"github.com/lxc/lxd/shared/validate"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/subprocess"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 type tpm struct {
@@ -36,6 +36,21 @@ func (d *tpm) validateConfig(instConf instance.ConfigReader) error {
 
 	rules := map[string]func(string) error{}
 
+	// lxdmeta:generate(entities=device-tpm; group=device-conf; key=path)
+	// For example: `/dev/tpm0`
+	// ---
+	//  type: string
+	//  required: for containers
+	//  condition: containers
+	//  shortdesc: Path inside the container
+
+	// lxdmeta:generate(entities=device-tpm; group=device-conf; key=pathrm)
+	// For example: `/dev/tpmrm0`
+	// ---
+	//  type: string
+	//  required: for containers
+	//  condition: containers
+	//  shortdesc: Resource manager path inside the container
 	if instConf.Type() == instancetype.Container {
 		rules["path"] = validate.IsNotEmpty
 		rules["pathrm"] = validate.IsNotEmpty
