@@ -11,12 +11,12 @@ import (
 	"strings"
 	"text/template"
 
-	"github.com/pborman/uuid"
+	"github.com/google/uuid"
 
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/validate"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/validate"
+	"github.com/canonical/lxd/shared/version"
 )
 
 const nftablesNamespace = "lxd"
@@ -97,7 +97,7 @@ func (d Nftables) Compat() (bool, error) {
 	}
 
 	// Check that nftables works at all (some kernels let you list ruleset despite missing support).
-	testTable := fmt.Sprintf("lxd_test_%s", uuid.New())
+	testTable := fmt.Sprintf("lxd_test_%s", uuid.New().String())
 
 	_, err = shared.RunCommandCLocale("nft", "create", "table", testTable)
 	if err != nil {
@@ -184,7 +184,7 @@ func (d Nftables) nftParseRuleset() ([]nftGenericItem, error) {
 	return items, nil
 }
 
-// GetVersion returns the version of dnsmasq.
+// GetVersion returns the version of nftables.
 func (d Nftables) hostVersion() (*version.DottedVersion, error) {
 	output, err := shared.RunCommandCLocale("nft", "--version")
 	if err != nil {
@@ -372,11 +372,12 @@ func (d Nftables) NetworkClear(networkName string, _ bool, _ []uint) error {
 		"fwd", "pstrt", "in", "out", // Chains used for network operation rules.
 		"aclin", "aclout", "aclfwd", "acl", // Chains used by ACL rules.
 		"fwdprert", "fwdout", "fwdpstrt", // Chains used by Address Forward rules.
+		"egress", // Chains added for limits.priority option
 	}
 
 	// Remove chains created by network rules.
 	// Remove from ip and ip6 tables to ensure cleanup for instances started before we moved to inet table
-	err := d.removeChains([]string{"inet", "ip", "ip6"}, networkName, removeChains...)
+	err := d.removeChains([]string{"inet", "ip", "ip6", "netdev"}, networkName, removeChains...)
 	if err != nil {
 		return fmt.Errorf("Failed clearing nftables rules for network %q: %w", networkName, err)
 	}
@@ -384,7 +385,7 @@ func (d Nftables) NetworkClear(networkName string, _ bool, _ []uint) error {
 	return nil
 }
 
-//instanceDeviceLabel returns the unique label used for instance device chains.
+// instanceDeviceLabel returns the unique label used for instance device chains.
 func (d Nftables) instanceDeviceLabel(projectName, instanceName, deviceName string) string {
 	return fmt.Sprintf("%s%s%s", project.Instance(projectName, instanceName), nftablesChainSeparator, deviceName)
 }
@@ -613,7 +614,7 @@ func (d Nftables) removeChains(families []string, chainSuffix string, chains ...
 	foundChains := make(map[string]nftGenericItem)
 	for _, family := range families {
 		for _, item := range ruleset {
-			if item.ItemType == "chain" && item.Family == family && item.Table == nftablesNamespace && shared.StringInSlice(item.Name, fullChains) {
+			if item.ItemType == "chain" && item.Family == family && item.Table == nftablesNamespace && shared.ValueInSlice(item.Name, fullChains) {
 				foundChains[item.Name] = item
 			}
 		}
@@ -667,6 +668,43 @@ func (d Nftables) InstanceClearRPFilter(projectName string, instanceName string,
 	return nil
 }
 
+// InstanceSetupNetPrio activates setting of skb->priority for the specified instance device on the host interface.
+func (d Nftables) InstanceSetupNetPrio(projectName string, instanceName string, deviceName string, netPrio uint32) error {
+	deviceLabel := d.instanceDeviceLabel(projectName, instanceName, deviceName)
+	tplFields := map[string]any{
+		"namespace":      nftablesNamespace,
+		"family":         "netdev",
+		"chainSeparator": nftablesChainSeparator,
+		"deviceLabel":    deviceLabel,
+		"deviceName":     deviceName,
+		"netPrio":        netPrio,
+	}
+
+	err := d.applyNftConfig(nftablesInstanceNetPrio, tplFields)
+	if err != nil {
+		return fmt.Errorf("Failed adding netprio rules for instance device %q: %w", deviceLabel, err)
+	}
+
+	return nil
+}
+
+// InstanceClearNetPrio removes setting of skb->priority for the specified instance device on the host interface.
+func (d Nftables) InstanceClearNetPrio(projectName string, instanceName string, deviceName string) error {
+	if deviceName == "" {
+		return fmt.Errorf("Failed clearing netprio rules for instance %q in project %q: device name is empty", projectName, instanceName)
+	}
+
+	deviceLabel := d.instanceDeviceLabel(projectName, instanceName, deviceName)
+	chainLabel := fmt.Sprintf("netprio%s%s", nftablesChainSeparator, deviceLabel)
+
+	err := d.removeChains([]string{"netdev"}, chainLabel, "egress")
+	if err != nil {
+		return fmt.Errorf("Failed clearing netprio rules for instance device %q: %w", deviceLabel, err)
+	}
+
+	return nil
+}
+
 // NetworkApplyACLRules applies ACL rules to the existing firewall chains.
 func (d Nftables) NetworkApplyACLRules(networkName string, rules []ACLRule) error {
 	nftRules := make([]string, 0)
@@ -769,7 +807,7 @@ func (d Nftables) aclRuleCriteriaToRules(networkName string, ipVersion uint, rul
 	}
 
 	// Add protocol filters.
-	if shared.StringInSlice(rule.Protocol, []string{"tcp", "udp"}) {
+	if shared.ValueInSlice(rule.Protocol, []string{"tcp", "udp"}) {
 		args = append(args, "meta", "l4proto", rule.Protocol)
 
 		if rule.SourcePort != "" {
@@ -779,7 +817,7 @@ func (d Nftables) aclRuleCriteriaToRules(networkName string, ipVersion uint, rul
 		if rule.DestinationPort != "" {
 			args = append(args, d.aclRulePortToACLMatch("dport", shared.SplitNTrimSpace(rule.DestinationPort, ",", -1, false)...)...)
 		}
-	} else if shared.StringInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
+	} else if shared.ValueInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
 		var icmpIPVersion uint
 		var protoName string
 
@@ -929,6 +967,13 @@ func (d Nftables) NetworkApplyForwards(networkName string, rules []AddressForwar
 	// This is so the generated firewall rules will apply the port specific rules first.
 	for _, listenPortsOnly := range []bool{true, false} {
 		for ruleIndex, rule := range rules {
+			// Process the rules in order of outer loop.
+			listenPortsLen := len(rule.ListenPorts)
+			if (listenPortsOnly && listenPortsLen < 1) || (!listenPortsOnly && listenPortsLen > 0) {
+				continue
+			}
+
+			// Validate the rule.
 			if rule.ListenAddress == nil {
 				return fmt.Errorf("Invalid rule %d, listen address is required", ruleIndex)
 			}
@@ -937,16 +982,19 @@ func (d Nftables) NetworkApplyForwards(networkName string, rules []AddressForwar
 				return fmt.Errorf("Invalid rule %d, target address is required", ruleIndex)
 			}
 
-			listenPortsLen := len(rule.ListenPorts)
-
-			// Process the rules in order of outer loop.
-			if (listenPortsOnly && listenPortsLen < 1) || (!listenPortsOnly && listenPortsLen > 0) {
-				continue
+			if listenPortsLen == 0 && rule.Protocol != "" {
+				return fmt.Errorf("Invalid rule %d, default target rule but non-empty protocol", ruleIndex)
 			}
 
-			// If multiple target ports supplied, check they match the listen port(s) count.
-			targetPortsLen := len(rule.TargetPorts)
-			if targetPortsLen > 1 && targetPortsLen != listenPortsLen {
+			switch len(rule.TargetPorts) {
+			case 0:
+				// No target ports specified, use listen ports (only valid when protocol is specified).
+				rule.TargetPorts = rule.ListenPorts
+			case 1:
+				// Single target port specified, OK.
+			case len(rule.ListenPorts):
+				// One-to-one match with listen ports, OK.
+			default:
 				return fmt.Errorf("Invalid rule %d, mismatch between listen port(s) and target port(s) count", ruleIndex)
 			}
 
@@ -958,52 +1006,39 @@ func (d Nftables) NetworkApplyForwards(networkName string, rules []AddressForwar
 			listenAddressStr := rule.ListenAddress.String()
 			targetAddressStr := rule.TargetAddress.String()
 
-			if listenPortsLen > 0 {
-				for i := range rule.ListenPorts {
-					// Use the target port that corresponds to the listen port (unless only 1
-					// is specified, in which case use same target port for all listen ports).
-					var targetPort uint64
+			if rule.Protocol != "" {
+				targetPortRanges := portRangesFromSlice(rule.TargetPorts)
+				for _, targetPortRange := range targetPortRanges {
+					targetPortRangeStr := portRangeStr(targetPortRange, "-")
+					snatRules = append(snatRules, map[string]any{
+						"ipFamily":    ipFamily,
+						"protocol":    rule.Protocol,
+						"targetHost":  targetAddressStr,
+						"targetPorts": targetPortRangeStr,
+					})
+				}
 
-					switch {
-					case targetPortsLen <= 0:
-						// No target ports specified, use same port as listen port index.
-						targetPort = rule.ListenPorts[i]
-					case targetPortsLen == 1:
-						// Single target port specified, use that for all listen ports.
-						targetPort = rule.TargetPorts[0]
-					case targetPortsLen > 1:
-						// Multiple target ports specified, user port associated with
-						// listen port index.
-						targetPort = rule.TargetPorts[i]
-					}
-
-					// Format the destination host/port as appropriate.
-					targetDest := fmt.Sprintf("%s:%d", targetAddressStr, targetPort)
-					if ipFamily == "ip6" {
-						targetDest = fmt.Sprintf("[%s]:%d", targetAddressStr, targetPort)
+				dnatRanges := getOptimisedDNATRanges(&rule)
+				for listenPortRange, targetPortRange := range dnatRanges {
+					// Format the destination host/port as appropriate
+					targetDest := targetAddressStr
+					if targetPortRange[1] == 1 {
+						targetPortStr := portRangeStr(targetPortRange, ":")
+						targetDest = fmt.Sprintf("%s:%s", targetAddressStr, targetPortStr)
+						if ipFamily == "ip6" {
+							targetDest = fmt.Sprintf("[%s]:%s", targetAddressStr, targetPortStr)
+						}
 					}
 
 					dnatRules = append(dnatRules, map[string]any{
 						"ipFamily":      ipFamily,
 						"protocol":      rule.Protocol,
 						"listenAddress": listenAddressStr,
-						"listenPorts":   rule.ListenPorts[i],
+						"listenPorts":   portRangeStr(listenPortRange, "-"),
 						"targetDest":    targetDest,
-						"targetHost":    targetAddressStr,
-						"targetPorts":   targetPort,
 					})
-
-					// Only add >1 hairpin NAT rules if multiple target ports being used.
-					if i == 0 || targetPortsLen != 1 {
-						snatRules = append(snatRules, map[string]any{
-							"ipFamily":    ipFamily,
-							"protocol":    rule.Protocol,
-							"targetHost":  targetAddressStr,
-							"targetPorts": targetPort,
-						})
-					}
 				}
-			} else if rule.Protocol == "" {
+			} else {
 				// Format the destination host/port as appropriate.
 				targetDest := targetAddressStr
 				if ipFamily == "ip6" {
@@ -1021,8 +1056,6 @@ func (d Nftables) NetworkApplyForwards(networkName string, rules []AddressForwar
 					"ipFamily":   ipFamily,
 					"targetHost": targetAddressStr,
 				})
-			} else {
-				return fmt.Errorf("Invalid rule %d, default target rule but non-empty protocol", ruleIndex)
 			}
 		}
 	}
