@@ -2,7 +2,6 @@ package drivers
 
 import (
 	"context"
-	"database/sql"
 	"errors"
 	"fmt"
 	"net/http"
@@ -12,37 +11,39 @@ import (
 	"strings"
 	"time"
 
-	"github.com/pborman/uuid"
+	"github.com/google/uuid"
 
-	"github.com/lxc/lxd/lxd/backup"
-	"github.com/lxc/lxd/lxd/db"
-	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/lxd/device"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/device/nictype"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/instance/operationlock"
-	"github.com/lxc/lxd/lxd/maas"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	storagePools "github.com/lxc/lxd/lxd/storage"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/backup"
+	"github.com/canonical/lxd/lxd/db"
+	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/device"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/device/nictype"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/instance/operationlock"
+	"github.com/canonical/lxd/lxd/lifecycle"
+	"github.com/canonical/lxd/lxd/locking"
+	"github.com/canonical/lxd/lxd/maas"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	storagePools "github.com/canonical/lxd/lxd/storage"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // ErrExecCommandNotFound indicates the command is not found.
-var ErrExecCommandNotFound = fmt.Errorf("Command not found")
+var ErrExecCommandNotFound = api.StatusErrorf(http.StatusBadRequest, "Command not found")
 
 // ErrExecCommandNotExecutable indicates the command is not executable.
-var ErrExecCommandNotExecutable = fmt.Errorf("Command not executable")
+var ErrExecCommandNotExecutable = api.StatusErrorf(http.StatusBadRequest, "Command not executable")
 
 // ErrInstanceIsStopped indicates that the instance is stopped.
-var ErrInstanceIsStopped error = fmt.Errorf("The instance is already stopped")
+var ErrInstanceIsStopped error = api.StatusErrorf(http.StatusBadRequest, "The instance is already stopped")
 
 // deviceManager is an interface that allows managing device lifecycle.
 type deviceManager interface {
@@ -74,13 +75,16 @@ type common struct {
 	node            string
 	profiles        []api.Profile
 	project         api.Project
-	snapshot        bool
+	isSnapshot      bool
 	stateful        bool
 
 	// Cached handles.
 	// Do not use these variables directly, instead use their associated get functions so they
 	// will be initialised on demand.
 	storagePool storagePools.Pool
+
+	// volatileSetPersistDisable indicates whether the VolatileSet function should persist changes to the DB.
+	volatileSetPersistDisable bool
 }
 
 //
@@ -124,7 +128,7 @@ func (d *common) ExpandedDevices() deviceConfig.Devices {
 
 // ExpiryDate returns when this snapshot expires.
 func (d *common) ExpiryDate() time.Time {
-	if d.snapshot {
+	if d.isSnapshot {
 		return d.expiryDate
 	}
 
@@ -184,7 +188,7 @@ func (d *common) Project() api.Project {
 
 // IsSnapshot returns whether instance is snapshot or not.
 func (d *common) IsSnapshot() bool {
-	return d.snapshot
+	return d.isSnapshot
 }
 
 // IsStateful returns whether instance is stateful or not.
@@ -203,8 +207,14 @@ func (d *common) Operation() *operations.Operation {
 
 // Backups returns a list of backups.
 func (d *common) Backups() ([]backup.InstanceBackup, error) {
+	var backupNames []string
+
 	// Get all the backups
-	backupNames, err := d.state.DB.Cluster.GetInstanceBackups(d.project.Name, d.name)
+	err := d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+		backupNames, err = tx.GetInstanceBackups(ctx, d.project.Name, d.name)
+		return err
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -245,7 +255,7 @@ func (d *common) SetOperation(op *operations.Operation) {
 
 // Snapshots returns a list of snapshots.
 func (d *common) Snapshots() ([]instance.Instance, error) {
-	if d.snapshot {
+	if d.isSnapshot {
 		return []instance.Instance{}, nil
 	}
 
@@ -316,25 +326,27 @@ func (d *common) Snapshots() ([]instance.Instance, error) {
 func (d *common) VolatileSet(changes map[string]string) error {
 	// Quick check.
 	for key := range changes {
-		if !strings.HasPrefix(key, shared.ConfigVolatilePrefix) {
+		if !strings.HasPrefix(key, instancetype.ConfigVolatilePrefix) {
 			return fmt.Errorf("Only volatile keys can be modified with VolatileSet")
 		}
 	}
 
-	// Update the database.
-	var err error
-	if d.snapshot {
-		err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			return tx.UpdateInstanceSnapshotConfig(d.id, changes)
-		})
-	} else {
-		err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			return tx.UpdateInstanceConfig(d.id, changes)
-		})
-	}
+	// Update the database if required.
+	if !d.volatileSetPersistDisable {
+		var err error
+		if d.isSnapshot {
+			err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				return tx.UpdateInstanceSnapshotConfig(d.id, changes)
+			})
+		} else {
+			err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				return tx.UpdateInstanceConfig(d.id, changes)
+			})
+		}
 
-	if err != nil {
-		return fmt.Errorf("Failed to set volatile config: %w", err)
+		if err != nil {
+			return fmt.Errorf("Failed to set volatile config: %w", err)
+		}
 	}
 
 	// Apply the change locally.
@@ -375,7 +387,12 @@ func (d *common) LogPath() string {
 
 // Path returns the instance's path.
 func (d *common) Path() string {
-	return storagePools.InstancePath(d.dbType, d.project.Name, d.name, d.snapshot)
+	return storagePools.InstancePath(d.dbType, d.project.Name, d.name, d.isSnapshot)
+}
+
+// ExecOutputPath returns the instance's exec output path.
+func (d *common) ExecOutputPath() string {
+	return filepath.Join(d.Path(), "exec-output")
 }
 
 // RootfsPath returns the instance's rootfs path.
@@ -492,8 +509,13 @@ func (d *common) deviceVolatileSetFunc(devName string) func(save map[string]stri
 
 // expandConfig applies the config of each profile in order, followed by the local config.
 func (d *common) expandConfig() error {
-	d.expandedConfig = db.ExpandInstanceConfig(d.localConfig, d.profiles)
-	d.expandedDevices = db.ExpandInstanceDevices(d.localDevices, d.profiles)
+	var globalConfigDump map[string]any
+	if d.state.GlobalConfig != nil {
+		globalConfigDump = d.state.GlobalConfig.Dump()
+	}
+
+	d.expandedConfig = instancetype.ExpandInstanceConfig(globalConfigDump, d.localConfig, d.profiles)
+	d.expandedDevices = instancetype.ExpandInstanceDevices(d.localDevices, d.profiles)
 
 	return nil
 }
@@ -508,6 +530,16 @@ func (d *common) restartCommon(inst instance.Instance, timeout time.Duration) er
 
 	// Handle ephemeral instances.
 	ephemeral := inst.IsEphemeral()
+
+	ctxMap := logger.Ctx{
+		"action":    "shutdown",
+		"created":   d.creationDate,
+		"ephemeral": ephemeral,
+		"used":      d.lastUsedDate,
+		"timeout":   timeout}
+
+	d.logger.Info("Restarting instance", ctxMap)
+
 	if ephemeral {
 		// Unset ephemeral flag
 		args := db.InstanceArgs{
@@ -547,7 +579,7 @@ func (d *common) restartCommon(inst instance.Instance, timeout time.Duration) er
 			return err
 		}
 
-		err := inst.Shutdown(timeout * time.Second)
+		err := inst.Shutdown(timeout)
 		if err != nil {
 			op.Done(err)
 			return err
@@ -566,6 +598,80 @@ func (d *common) restartCommon(inst instance.Instance, timeout time.Duration) er
 		return err
 	}
 
+	d.logger.Info("Restarted instance", ctxMap)
+	d.state.Events.SendLifecycle(d.project.Name, lifecycle.InstanceRestarted.Event(d, nil))
+
+	return nil
+}
+
+// rebuildCommon handles the common part of instance rebuilds.
+func (d *common) rebuildCommon(inst instance.Instance, img *api.Image, op *operations.Operation) error {
+	instLocalConfig := d.localConfig
+
+	// Reset the "image.*" keys.
+	for k := range instLocalConfig {
+		if strings.HasPrefix(k, "image.") {
+			delete(instLocalConfig, k)
+		}
+	}
+
+	delete(instLocalConfig, "volatile.base_image")
+	if img != nil {
+		for k, v := range img.Properties {
+			instLocalConfig[fmt.Sprintf("image.%s", k)] = v
+		}
+
+		instLocalConfig["volatile.base_image"] = img.Fingerprint
+		instLocalConfig["volatile.uuid.generation"] = instLocalConfig["volatile.uuid"]
+	}
+
+	// Reset relevant volatile keys.
+	delete(instLocalConfig, "volatile.idmap.next")
+	delete(instLocalConfig, "volatile.last_state.idmap")
+
+	pool, err := d.getStoragePool()
+	if err != nil {
+		return err
+	}
+
+	err = pool.DeleteInstance(inst, op)
+	if err != nil {
+		return err
+	}
+
+	// Rebuild as empty if there is no image provided.
+	if img == nil {
+		err = pool.CreateInstance(inst, nil)
+		if err != nil {
+			return err
+		}
+	} else {
+		err = pool.CreateInstanceFromImage(inst, img.Fingerprint, op)
+		if err != nil {
+			return err
+		}
+	}
+
+	err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = dbCluster.UpdateInstanceConfig(ctx, tx.Tx(), int64(inst.ID()), instLocalConfig)
+		if err != nil {
+			return err
+		}
+
+		if img != nil {
+			err = tx.UpdateImageLastUseDate(ctx, inst.Project().Name, img.Fingerprint, time.Now().UTC())
+			if err != nil {
+				return err
+			}
+		}
+
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+
+	d.localConfig = instLocalConfig
 	return nil
 }
 
@@ -663,20 +769,20 @@ func (d *common) updateProgress(progress string) {
 // unpopulated then the insert querty is retried until it succeeds or a retry limit is reached.
 // If the insert succeeds or the key is found to have been populated then the value of the key is returned.
 func (d *common) insertConfigkey(key string, value string) (string, error) {
-	err := query.Retry(func() error {
-		err := query.Transaction(context.TODO(), d.state.DB.Cluster.DB(), func(ctx context.Context, tx *sql.Tx) error {
-			return db.CreateInstanceConfig(tx, d.id, map[string]string{key: value})
-		})
-		if err != nil {
-			// Check if something else filled it in behind our back.
-			existingValue, errCheckExists := d.state.DB.Cluster.GetInstanceConfig(d.id, key)
-			if errCheckExists != nil {
-				return err
-			}
-
-			value = existingValue
+	err := d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err := tx.CreateInstanceConfig(ctx, d.id, map[string]string{key: value})
+		if err == nil {
+			return nil
 		}
 
+		// Check if something else filled it in behind our back.
+		existingValue, errCheckExists := tx.GetInstanceConfig(ctx, d.id, key)
+		if errCheckExists != nil {
+			return err
+		}
+
+		value = existingValue
+
 		return nil
 	})
 	if err != nil {
@@ -706,29 +812,29 @@ func (d *common) isStartableStatusCode(statusCode api.StatusCode) error {
 	return nil
 }
 
-// startupSnapshot triggers a snapshot if configured.
-func (d *common) startupSnapshot(inst instance.Instance) error {
+// getStartupSnapNameAndExpiry returns the name and expiry for a snapshot to be taken at startup.
+func (d *common) getStartupSnapNameAndExpiry(inst instance.Instance) (string, *time.Time, error) {
 	schedule := strings.ToLower(d.expandedConfig["snapshots.schedule"])
 	if schedule == "" {
-		return nil
+		return "", nil, nil
 	}
 
 	triggers := strings.Split(schedule, ", ")
-	if !shared.StringInSlice("@startup", triggers) {
-		return nil
+	if !shared.ValueInSlice("@startup", triggers) {
+		return "", nil, nil
 	}
 
 	expiry, err := shared.GetExpiry(time.Now(), d.expandedConfig["snapshots.expiry"])
 	if err != nil {
-		return err
+		return "", nil, err
 	}
 
 	name, err := instance.NextSnapshotName(d.state, inst, "snap%d")
 	if err != nil {
-		return err
+		return "", nil, err
 	}
 
-	return inst.Snapshot(name, expiry, false)
+	return name, &expiry, nil
 }
 
 // Internal MAAS handling.
@@ -893,6 +999,31 @@ func (d *common) maasDelete(inst instance.Instance) error {
 	return d.state.MAAS.DeleteContainer(d)
 }
 
+// validateStartup checks any constraints that would prevent start up from succeeding under normal circumstances.
+func (d *common) validateStartup(stateful bool, statusCode api.StatusCode) error {
+	// Because the root disk is special and is mounted before the root disk device is setup we duplicate the
+	// pre-start check here before the isStartableStatusCode check below so that if there is a problem loading
+	// the instance status because the storage pool isn't available we don't mask the StatusServiceUnavailable
+	// error with an ERROR status code from the instance check instead.
+	_, rootDiskConf, err := instancetype.GetRootDiskDevice(d.expandedDevices.CloneNative())
+	if err != nil {
+		return err
+	}
+
+	if !storagePools.IsAvailable(rootDiskConf["pool"]) {
+		return api.StatusErrorf(http.StatusServiceUnavailable, "Storage pool %q unavailable on this server", rootDiskConf["pool"])
+	}
+
+	// Must happen before creating operation Start lock to avoid the status check returning Stopped due to the
+	// existence of a Start operation lock.
+	err = d.isStartableStatusCode(statusCode)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
 // onStopOperationSetup creates or picks up the relevant operation. This is used in the stopns and stop hooks to
 // ensure that a lock on their activities is held before the instance process is stopped. This prevents a start
 // request run at the same time from overlapping with the stop process.
@@ -900,13 +1031,13 @@ func (d *common) maasDelete(inst instance.Instance) error {
 func (d *common) onStopOperationSetup(target string) (*operationlock.InstanceOperation, error) {
 	var err error
 
-	// Pick up the existing stop operation lock created in Stop() function.
-	// If there is another ongoing operation (such as start), wait until that has finished before proceeding
-	// to run the hook (this should be quick as it will fail showing instance is already running).
+	// Pick up the existing stop operation lock created in Start(), Restart(), Shutdown() or Stop() functions.
+	// If there is another ongoing operation that isn't in our inheritable list, wait until that has finished
+	// before proceeding to run the hook.
 	op := operationlock.Get(d.Project().Name, d.Name())
-	if op != nil && !op.ActionMatch(operationlock.ActionStop, operationlock.ActionRestart, operationlock.ActionRestore) {
+	if op != nil && !op.ActionMatch(operationlock.ActionStart, operationlock.ActionRestart, operationlock.ActionStop, operationlock.ActionRestore) {
 		d.logger.Debug("Waiting for existing operation lock to finish before running hook", logger.Ctx{"action": op.Action()})
-		_ = op.Wait()
+		_ = op.Wait(context.Background())
 		op = nil
 	}
 
@@ -924,6 +1055,8 @@ func (d *common) onStopOperationSetup(target string) (*operationlock.InstanceOpe
 		}
 
 		op.SetInstanceInitiated(true)
+	} else {
+		d.logger.Debug("Instance operation lock inherited for stop", logger.Ctx{"action": op.Action()})
 	}
 
 	return op, nil
@@ -932,7 +1065,7 @@ func (d *common) onStopOperationSetup(target string) (*operationlock.InstanceOpe
 // warningsDelete deletes any persistent warnings for the instance.
 func (d *common) warningsDelete() error {
 	err := d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		return dbCluster.DeleteWarnings(ctx, tx.Tx(), dbCluster.TypeInstance, d.ID())
+		return dbCluster.DeleteWarnings(ctx, tx.Tx(), dbCluster.EntityType(entity.TypeInstance), d.ID())
 	})
 	if err != nil {
 		return fmt.Errorf("Failed deleting persistent warnings: %w", err)
@@ -941,8 +1074,10 @@ func (d *common) warningsDelete() error {
 	return nil
 }
 
-// canMigrate returns whether the instance can be migrated.
-func (d *common) canMigrate(inst instance.Instance) (bool, bool) {
+// canMigrate determines if the given instance can be migrated and whether the migration
+// can be live. In "auto" mode, the function checks each attached device of the instance
+// to ensure they are all migratable.
+func (d *common) canMigrate(inst instance.Instance) (migrate bool, live bool) {
 	// Check policy for the instance.
 	config := d.ExpandedConfig()
 	val, ok := config["cluster.evacuate"]
@@ -966,19 +1101,22 @@ func (d *common) canMigrate(inst instance.Instance) (bool, bool) {
 	volatileGet := func() map[string]string { return map[string]string{} }
 	volatileSet := func(_ map[string]string) error { return nil }
 	for deviceName, rawConfig := range d.ExpandedDevices() {
-		dev, err := device.New(inst, d.state, deviceName, rawConfig, volatileGet, volatileSet)
+		// Make sure to clone the devices config for new devices.
+		// Some device drivers might modify the configuration and populate additional settings.
+		dev, err := device.New(inst, d.state, deviceName, rawConfig.Clone(), volatileGet, volatileSet)
 		if err != nil {
+			logger.Warn("Instance will not be migrated due to a device error", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "device": dev.Name(), "err": err})
 			return false, false
 		}
 
 		if !dev.CanMigrate() {
+			logger.Warn("Instance will not be migrated because its device cannot be migrated", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "device": dev.Name()})
 			return false, false
 		}
 	}
 
 	// Check if set up for live migration.
 	// Limit automatic live-migration to virtual machines for now.
-	live := false
 	if inst.Type() == instancetype.VM {
 		live = shared.IsTrue(config["migration.stateful"])
 	}
@@ -991,13 +1129,13 @@ func (d *common) recordLastState() error {
 	var err error
 
 	// Record power state.
-	d.localConfig["volatile.last_state.power"] = "RUNNING"
-	d.expandedConfig["volatile.last_state.power"] = "RUNNING"
+	d.localConfig["volatile.last_state.power"] = instance.PowerStateRunning
+	d.expandedConfig["volatile.last_state.power"] = instance.PowerStateRunning
 
 	// Database updates
 	return d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Record power state.
-		err = tx.UpdateInstancePowerState(d.id, "RUNNING")
+		err = tx.UpdateInstancePowerState(d.id, instance.PowerStateRunning)
 		if err != nil {
 			err = fmt.Errorf("Error updating instance power state: %w", err)
 			return err
@@ -1048,7 +1186,7 @@ func (d *common) getRootDiskDevice() (string, map[string]string, error) {
 	}
 
 	// Retrieve the instance's storage pool.
-	name, configuration, err := shared.GetRootDiskDevice(devices.CloneNative())
+	name, configuration, err := instancetype.GetRootDiskDevice(devices.CloneNative())
 	if err != nil {
 		return "", nil, err
 	}
@@ -1058,7 +1196,7 @@ func (d *common) getRootDiskDevice() (string, map[string]string, error) {
 
 // resetInstanceID generates a new UUID and puts it in volatile.
 func (d *common) resetInstanceID() error {
-	err := d.VolatileSet(map[string]string{"volatile.cloud-init.instance-id": uuid.New()})
+	err := d.VolatileSet(map[string]string{"volatile.cloud-init.instance-id": uuid.New().String()})
 	if err != nil {
 		return fmt.Errorf("Failed to set volatile.cloud-init.instance-id: %w", err)
 	}
@@ -1077,7 +1215,7 @@ func (d *common) needsNewInstanceID(changedConfig []string, oldExpandedDevices d
 		"user.user-data",
 		"user.network-config",
 	} {
-		if shared.StringInSlice(key, changedConfig) {
+		if shared.ValueInSlice(key, changedConfig) {
 			return true
 		}
 	}
@@ -1112,13 +1250,13 @@ func (d *common) needsNewInstanceID(changedConfig []string, oldExpandedDevices d
 	newNames := getNICNames(d.expandedDevices)
 
 	for _, entry := range oldNames {
-		if !shared.StringInSlice(entry, newNames) {
+		if !shared.ValueInSlice(entry, newNames) {
 			return true
 		}
 	}
 
 	for _, entry := range newNames {
-		if !shared.StringInSlice(entry, oldNames) {
+		if !shared.ValueInSlice(entry, oldNames) {
 			return true
 		}
 	}
@@ -1133,7 +1271,18 @@ func (d *common) getStoragePool() (storagePools.Pool, error) {
 		return d.storagePool, nil
 	}
 
-	poolName, err := d.state.DB.Cluster.GetInstancePool(d.Project().Name, d.Name())
+	var poolName string
+
+	err := d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		poolName, err = tx.GetInstancePool(ctx, d.Project().Name, d.Name())
+		if err != nil {
+			return fmt.Errorf("Failed getting instance pool: %w", err)
+		}
+
+		return nil
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -1154,7 +1303,7 @@ func (d *common) deviceLoad(inst instance.Instance, deviceName string, rawConfig
 	var err error
 
 	// Create copy of config and load some fields from volatile if device is nic or infiniband.
-	if shared.StringInSlice(rawConfig["type"], []string{"nic", "infiniband"}) {
+	if shared.ValueInSlice(rawConfig["type"], []string{"nic", "infiniband"}) {
 		configCopy, err = inst.FillNetworkDevice(deviceName, rawConfig)
 		if err != nil {
 			return nil, err
@@ -1439,3 +1588,30 @@ func (d *common) devicesRemove(inst instance.Instance) {
 		}
 	}
 }
+
+// updateBackupFileLock acquires the update backup file lock that protects concurrent access to actions that will call UpdateBackupFile() as part of their operation.
+func (d *common) updateBackupFileLock(ctx context.Context) (locking.UnlockFunc, error) {
+	parentName, _, _ := api.GetParentAndSnapshotName(d.Name())
+	return locking.Lock(ctx, fmt.Sprintf("instance_updatebackupfile_%s_%s", d.Project().Name, parentName))
+}
+
+// deleteSnapshots calls the deleteFunc on each of the instance's snapshots in reverse order.
+func (d *common) deleteSnapshots(deleteFunc func(snapInst instance.Instance) error) error {
+	snapInsts, err := d.Snapshots()
+	if err != nil {
+		return err
+	}
+
+	snapInstsCount := len(snapInsts)
+
+	for k := range snapInsts {
+		// Delete the snapshots in reverse order.
+		k = snapInstsCount - 1 - k
+		err = deleteFunc(snapInsts[k])
+		if err != nil {
+			return fmt.Errorf("Failed deleting snapshot %q: %w", snapInsts[k].Name(), err)
+		}
+	}
+
+	return nil
+}
