@@ -6,7 +6,7 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/lxc/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/osarch"
 )
 
 func TestQemuConfigTemplates(t *testing.T) {
@@ -132,10 +132,15 @@ func TestQemuConfigTemplates(t *testing.T) {
 
 			[device "qemu_serial"]
 			driver = "virtserialport"
-			name = "org.linuxcontainers.lxd"
+			name = "com.canonical.lxd"
 			chardev = "qemu_serial-chardev"
 			bus = "dev-qemu_serial.0"
 
+			[device "qemu_serial_legacy"]
+			driver = "virtserialport"
+			name = "org.linuxcontainers.lxd"
+			bus = "dev-qemu_serial.0"
+
 			# Spice agent
 			[chardev "qemu_spice-chardev"]
 			backend = "spicevmc"
@@ -204,7 +209,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			addr = "00.0"
 			`,
 		}, {
-			qemuDevOpts{"ccw", "qemu_pcie2", "00.2", true},
+			qemuDevOpts{"ccw", "devBus", "busAddr", true},
 			`# SCSI controller
 			[device "qemu_scsi"]
 			driver = "virtio-scsi-ccw"
@@ -230,7 +235,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			multifunction = "on"
 			`,
 		}, {
-			qemuDevOpts{"ccw", "qemu_pcie0", "00.0", false},
+			qemuDevOpts{"ccw", "devBus", "busAddr", false},
 			`# Balloon driver
 			[device "qemu_balloon"]
 			driver = "virtio-balloon-ccw"
@@ -259,7 +264,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			rng = "qemu_rng"
 			`,
 		}, {
-			qemuDevOpts{"ccw", "qemu_pcie0", "00.1", true},
+			qemuDevOpts{"ccw", "devBus", "busAddr", true},
 			`# Random number generator
 			[object "qemu_rng"]
 			qom-type = "rng-random"
@@ -281,7 +286,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			opts     qemuVsockOpts
 			expected string
 		}{{
-			qemuVsockOpts{qemuDevOpts{"pcie", "qemu_pcie0", "00.4", true}, 14},
+			qemuVsockOpts{qemuDevOpts{"pcie", "qemu_pcie0", "00.4", true}, 4, 14},
 			`# Vsock
 			[device "qemu_vsock"]
 			driver = "vhost-vsock-pci"
@@ -289,13 +294,15 @@ func TestQemuConfigTemplates(t *testing.T) {
 			addr = "00.4"
 			multifunction = "on"
 			guest-cid = "14"
+			vhostfd = "4"
 			`,
 		}, {
-			qemuVsockOpts{qemuDevOpts{"ccw", "qemu_pcie0", "00.4", false}, 3},
+			qemuVsockOpts{qemuDevOpts{"ccw", "devBus", "busAddr", false}, 4, 3},
 			`# Vsock
 			[device "qemu_vsock"]
 			driver = "vhost-vsock-ccw"
 			guest-cid = "3"
+			vhostfd = "4"
 			`,
 		}}
 		for _, tc := range testCases {
@@ -359,12 +366,12 @@ func TestQemuConfigTemplates(t *testing.T) {
 			addr = "00.0"
 			multifunction = "on"`,
 		}, {
-			qemuDevOpts{"ccw", "qemu_pcie3", "00.0", false},
+			qemuDevOpts{"ccw", "devBus", "busAddr", false},
 			`# Input
 			[device "qemu_keyboard"]
 			driver = "virtio-keyboard-ccw"`,
 		}, {
-			qemuDevOpts{"ccw", "qemu_pcie3", "00.0", true},
+			qemuDevOpts{"ccw", "devBus", "busAddr", true},
 			`# Input
 			[device "qemu_keyboard"]
 			driver = "virtio-keyboard-ccw"
@@ -389,7 +396,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			multifunction = "on"
 			`,
 		}, {
-			qemuDevOpts{"ccw", "qemu_pcie0", "00.3", true},
+			qemuDevOpts{"ccw", "devBus", "busAddr", true},
 			`# Input
 			[device "qemu_tablet"]
 			driver = "virtio-tablet-ccw"
@@ -779,7 +786,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			chardev = "qemu_config"`,
 		}, {
 			qemuDriveConfigOpts{
-				dev:      qemuDevOpts{"ccw", "qemu_pcie0", "00.0", false},
+				dev:      qemuDevOpts{"ccw", "devBus", "busAddr", false},
 				path:     "/var/virtio-fs",
 				protocol: "virtio-fs",
 			},
@@ -794,7 +801,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			chardev = "qemu_config"`,
 		}, {
 			qemuDriveConfigOpts{
-				dev:      qemuDevOpts{"ccw", "qemu_pcie0", "00.0", true},
+				dev:      qemuDevOpts{"ccw", "devBus", "busAddr", true},
 				path:     "/dev/9p",
 				protocol: "9p",
 			},
@@ -812,7 +819,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			fsdev = "qemu_config"`,
 		}, {
 			qemuDriveConfigOpts{
-				dev:      qemuDevOpts{"ccw", "qemu_pcie0", "00.0", true},
+				dev:      qemuDevOpts{"ccw", "devBus", "busAddr", true},
 				path:     "/dev/9p",
 				protocol: "invalid",
 			},
@@ -870,7 +877,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			chardev = "lxd_vfs"`,
 		}, {
 			qemuDriveDirOpts{
-				dev:      qemuDevOpts{"ccw", "qemu_pcie0", "00.0", true},
+				dev:      qemuDevOpts{"ccw", "devBus", "busAddr", true},
 				path:     "/dev/vio",
 				devName:  "vfs",
 				mountTag: "vtag",
@@ -888,7 +895,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			chardev = "lxd_vfs"`,
 		}, {
 			qemuDriveDirOpts{
-				dev:      qemuDevOpts{"ccw", "qemu_pcie0", "00.0", false},
+				dev:      qemuDevOpts{"ccw", "devBus", "busAddr", false},
 				devName:  "stub2",
 				mountTag: "mtag2",
 				protocol: "9p",
@@ -907,7 +914,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			fsdev = "lxd_stub2"`,
 		}, {
 			qemuDriveDirOpts{
-				dev:      qemuDevOpts{"ccw", "qemu_pcie0", "00.0", true},
+				dev:      qemuDevOpts{"ccw", "devBus", "busAddr", true},
 				path:     "/dev/9p",
 				protocol: "invalid",
 			},
@@ -936,7 +943,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			host = "host-slot"`,
 		}, {
 			qemuPCIPhysicalOpts{
-				dev:         qemuDevOpts{"ccw", "qemu_pcie2", "00.2", true},
+				dev:         qemuDevOpts{"ccw", "devBus", "busAddr", true},
 				devName:     "physical-ccw-name",
 				pciSlotName: "host-slot-ccw",
 			},
@@ -969,7 +976,7 @@ func TestQemuConfigTemplates(t *testing.T) {
 			host = "gpu-slot"`,
 		}, {
 			qemuGPUDevPhysicalOpts{
-				dev:         qemuDevOpts{"ccw", "qemu_pcie1", "00.0", true},
+				dev:         qemuDevOpts{"ccw", "devBus", "busAddr", true},
 				devName:     "gpu-name",
 				pciSlotName: "gpu-slot",
 				vga:         true,
