@@ -8,12 +8,13 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/lxc/lxd/lxd/instance/drivers/qmp"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/metrics"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/drivers/qmp"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/metrics"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/units"
 )
 
 func (d *qemu) getQemuMetrics() (*metrics.MetricSet, error) {
@@ -66,7 +67,8 @@ func (d *qemu) getQemuMetrics() (*metrics.MetricSet, error) {
 		}
 	}
 
-	metricSet, err := metrics.MetricSetFromAPI(&out, map[string]string{"project": d.project.Name, "name": d.name, "type": instancetype.VM.String()})
+	// The running state is hard-coded here as if we've made it to this point, the VM is running.
+	metricSet, err := metrics.MetricSetFromAPI(&out, map[string]string{"project": d.project.Name, "name": d.name, "type": instancetype.VM.String(), "state": instance.PowerStateRunning})
 	if err != nil {
 		return nil, err
 	}
@@ -144,7 +146,7 @@ func (d *qemu) getQemuMemoryMetrics(monitor *qmp.Monitor) (metrics.MemoryMetrics
 	// Get max memory usage.
 	memTotal := d.expandedConfig["limits.memory"]
 	if memTotal == "" {
-		memTotal = qemuDefaultMemSize // Default if no memory limit specified.
+		memTotal = QEMUDefaultMemSize // Default if no memory limit specified.
 	}
 
 	memTotalBytes, err := units.ParseByteSizeString(memTotal)
