@@ -4,8 +4,8 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/lxc/lxd/lxd/resources"
-	"github.com/lxc/lxd/shared/osarch"
+	"github.com/canonical/lxd/lxd/resources"
+	"github.com/canonical/lxd/shared/osarch"
 )
 
 type cfgEntry struct {
@@ -177,19 +177,33 @@ func qemuSerial(opts *qemuSerialOpts) []cfgSection {
 		comment: "Virtual serial bus",
 		entries: qemuDeviceEntries(&entriesOpts),
 	}, {
+		// Ring buffer used by the lxd agent to report (write) its status to. LXD server will read
+		// its content via QMP using "ringbuf-read" command.
 		name:    fmt.Sprintf(`chardev "%s"`, opts.charDevName),
 		comment: "LXD serial identifier",
 		entries: []cfgEntry{
 			{key: "backend", value: "ringbuf"},
 			{key: "size", value: fmt.Sprintf("%dB", opts.ringbufSizeBytes)}},
 	}, {
+		// QEMU serial device connected to the above ring buffer.
 		name: `device "qemu_serial"`,
 		entries: []cfgEntry{
 			{key: "driver", value: "virtserialport"},
-			{key: "name", value: "org.linuxcontainers.lxd"},
+			{key: "name", value: "com.canonical.lxd"},
 			{key: "chardev", value: opts.charDevName},
 			{key: "bus", value: "dev-qemu_serial.0"},
 		},
+	}, {
+		// Legacy QEMU serial device, not connected to any ring buffer. Its purpose is to
+		// create a symlink in /dev/virtio-ports/, triggering a udev rule to start the lxd-agent.
+		// This is necessary for backward compatibility with virtual machines lacking the
+		// updated lxd-agent-loader package, which includes updated udev rules and a systemd unit.
+		name: `device "qemu_serial_legacy"`,
+		entries: []cfgEntry{
+			{key: "driver", value: "virtserialport"},
+			{key: "name", value: "org.linuxcontainers.lxd"},
+			{key: "bus", value: "dev-qemu_serial.0"},
+		},
 	}, {
 		name:    `chardev "qemu_spice-chardev"`,
 		comment: "Spice agent",
@@ -297,9 +311,37 @@ func qemuRNG(opts *qemuDevOpts) []cfgSection {
 	}}
 }
 
+type qemuSevOpts struct {
+	cbitpos         int
+	reducedPhysBits int
+	policy          string
+	dhCertFD        string
+	sessionDataFD   string
+}
+
+func qemuSEV(opts *qemuSevOpts) []cfgSection {
+	entries := []cfgEntry{
+		{key: "qom-type", value: "sev-guest"},
+		{key: "cbitpos", value: fmt.Sprintf("%d", opts.cbitpos)},
+		{key: "reduced-phys-bits", value: fmt.Sprintf("%d", opts.reducedPhysBits)},
+		{key: "policy", value: opts.policy},
+	}
+
+	if opts.dhCertFD != "" && opts.sessionDataFD != "" {
+		entries = append(entries, cfgEntry{key: "dh-cert-file", value: opts.dhCertFD}, cfgEntry{key: "session-file", value: opts.sessionDataFD})
+	}
+
+	return []cfgSection{{
+		name:    `object "sev0"`,
+		comment: "Secure Encrypted Virtualization",
+		entries: entries,
+	}}
+}
+
 type qemuVsockOpts struct {
 	dev     qemuDevOpts
-	vsockID int
+	vsockFD int
+	vsockID uint32
 }
 
 func qemuVsock(opts *qemuVsockOpts) []cfgSection {
@@ -313,7 +355,8 @@ func qemuVsock(opts *qemuVsockOpts) []cfgSection {
 		name:    `device "qemu_vsock"`,
 		comment: "Vsock",
 		entries: append(qemuDeviceEntries(&entriesOpts),
-			cfgEntry{key: "guest-cid", value: fmt.Sprintf("%d", opts.vsockID)}),
+			cfgEntry{key: "guest-cid", value: fmt.Sprintf("%d", opts.vsockID)},
+			cfgEntry{key: "vhostfd", value: fmt.Sprintf("%d", opts.vsockFD)}),
 	}}
 }
 
@@ -382,6 +425,7 @@ type qemuNumaEntry struct {
 type qemuCPUOpts struct {
 	architecture        string
 	cpuCount            int
+	cpuRequested        int
 	cpuSockets          int
 	cpuCores            int
 	cpuThreads          int
@@ -441,8 +485,18 @@ func qemuCPU(opts *qemuCPUOpts, pinning bool) []cfgSection {
 			return nil
 		}
 
+		// Cap the max number of CPUs to 64 unless directly assigned more.
+		max := 64
+		if int(cpu.Total) < max {
+			max = int(cpu.Total)
+		} else if opts.cpuRequested > max {
+			max = opts.cpuRequested
+		} else if opts.cpuCount > max {
+			max = opts.cpuCount
+		}
+
 		entries = append(entries, cfgEntry{
-			key: "maxcpus", value: fmt.Sprintf("%d", cpu.Total),
+			key: "maxcpus", value: fmt.Sprintf("%d", max),
 		})
 	}
 
@@ -834,3 +888,18 @@ func qemuTPM(opts *qemuTPMOpts) []cfgSection {
 		},
 	}}
 }
+
+type qemuVmgenIDOpts struct {
+	guid string
+}
+
+func qemuVmgen(opts *qemuVmgenIDOpts) []cfgSection {
+	return []cfgSection{{
+		name:    `device "vmgenid0"`,
+		comment: "VM Generation ID",
+		entries: []cfgEntry{
+			{key: "driver", value: "vmgenid"},
+			{key: "guid", value: opts.guid},
+		},
+	}}
+}
