@@ -5,18 +5,19 @@ import (
 	"fmt"
 	"sync"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/warningtype"
-	"github.com/lxc/lxd/lxd/device"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/warningtype"
+	"github.com/canonical/lxd/lxd/device"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // Instance driver definitions.
@@ -36,6 +37,10 @@ type DriverStatus struct {
 var driverStatusesMu sync.Mutex
 var driverStatuses map[instancetype.Type]*DriverStatus
 
+// Temporary instance reference storage (for hooks).
+var instanceRefsMu sync.Mutex
+var instanceRefs map[string]instance.Instance
+
 func init() {
 	// Expose load to the instance package, to avoid circular imports.
 	instance.Load = load
@@ -81,7 +86,7 @@ func validDevices(state *state.State, p api.Project, instanceType instancetype.T
 	checkDevices := func(devices deviceConfig.Devices, expanded bool) error {
 		// Check each device individually using the device package.
 		for deviceName, deviceConfig := range devices {
-			if expanded && shared.StringInSlice(deviceName, checkedDevices) {
+			if expanded && shared.ValueInSlice(deviceName, checkedDevices) {
 				continue // Don't check the device twice if present in both local and expanded.
 			}
 
@@ -118,7 +123,7 @@ func validDevices(state *state.State, p api.Project, instanceType instancetype.T
 
 	if len(expandedDevices) > 0 {
 		// Check we have a root disk if in expanded validation mode.
-		_, _, err := shared.GetRootDiskDevice(expandedDevices.CloneNative())
+		_, _, err := instancetype.GetRootDiskDevice(expandedDevices.CloneNative())
 		if err != nil {
 			return fmt.Errorf("Failed detecting root disk device: %w", err)
 		}
@@ -181,3 +186,31 @@ func DriverStatuses() map[instancetype.Type]*DriverStatus {
 
 	return driverStatuses
 }
+
+// instanceRefGet retrieves an instance reference.
+func instanceRefGet(projectName string, instName string) instance.Instance {
+	instanceRefsMu.Lock()
+	defer instanceRefsMu.Unlock()
+
+	return instanceRefs[project.Instance(projectName, instName)]
+}
+
+// instanceRefSet stores a reference to an instance.
+func instanceRefSet(inst instance.Instance) {
+	instanceRefsMu.Lock()
+	defer instanceRefsMu.Unlock()
+
+	if instanceRefs == nil {
+		instanceRefs = make(map[string]instance.Instance)
+	}
+
+	instanceRefs[project.Instance(inst.Project().Name, inst.Name())] = inst
+}
+
+// instanceRefClear removes an instance reference.
+func instanceRefClear(inst instance.Instance) {
+	instanceRefsMu.Lock()
+	defer instanceRefsMu.Unlock()
+
+	delete(instanceRefs, project.Instance(inst.Project().Name, inst.Name()))
+}
