@@ -1,15 +1,17 @@
 package qmp
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
+	"net"
 	"net/http"
 	"os"
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // FdsetFdInfo contains information about a file descriptor that belongs to an FD set.
@@ -61,11 +63,6 @@ type HotpluggableCPU struct {
 
 // QueryCPUs returns a list of CPUs.
 func (m *Monitor) QueryCPUs() ([]CPU, error) {
-	// Check if disconnected
-	if m.disconnected {
-		return nil, ErrMonitorDisconnect
-	}
-
 	// Prepare the response.
 	var resp struct {
 		Return []CPU `json:"return"`
@@ -81,11 +78,6 @@ func (m *Monitor) QueryCPUs() ([]CPU, error) {
 
 // QueryHotpluggableCPUs returns a list of hotpluggable CPUs.
 func (m *Monitor) QueryHotpluggableCPUs() ([]HotpluggableCPU, error) {
-	// Check if disconnected
-	if m.disconnected {
-		return nil, ErrMonitorDisconnect
-	}
-
 	// Prepare the response.
 	var resp struct {
 		Return []HotpluggableCPU `json:"return"`
@@ -119,7 +111,7 @@ func (m *Monitor) Status() (string, error) {
 
 // SendFile adds a new file descriptor to the QMP fd table associated to name.
 func (m *Monitor) SendFile(name string, file *os.File) error {
-	// Check if disconnected
+	// Check if disconnected.
 	if m.disconnected {
 		return ErrMonitorDisconnect
 	}
@@ -156,86 +148,73 @@ func (m *Monitor) SendFile(name string, file *os.File) error {
 
 // CloseFile closes an existing file descriptor in the QMP fd table associated to name.
 func (m *Monitor) CloseFile(name string) error {
-	// Check if disconnected
-	if m.disconnected {
-		return ErrMonitorDisconnect
-	}
-
 	var req struct {
-		Execute   string `json:"execute"`
-		Arguments struct {
-			FDName string `json:"fdname"`
-		} `json:"arguments"`
+		FDName string `json:"fdname"`
 	}
 
-	req.Execute = "closefd"
-	req.Arguments.FDName = name
+	req.FDName = name
 
-	reqJSON, err := json.Marshal(req)
+	err := m.run("closefd", req, nil)
 	if err != nil {
 		return err
 	}
 
-	// Query the status.
-	_, err = m.qmp.Run(reqJSON)
-	if err != nil {
-		// Confirm the daemon didn't die.
-		errPing := m.ping()
-		if errPing != nil {
-			return errPing
-		}
-
-		return err
-	}
-
 	return nil
 }
 
 // SendFileWithFDSet adds a new file descriptor to an FD set.
-func (m *Monitor) SendFileWithFDSet(name string, file *os.File, readonly bool) (AddFdInfo, error) {
+func (m *Monitor) SendFileWithFDSet(name string, file *os.File, readonly bool) (*AddFdInfo, error) {
+	// Check if disconnected.
+	if m.disconnected {
+		return nil, ErrMonitorDisconnect
+	}
+
+	var req struct {
+		Execute   string `json:"execute"`
+		Arguments struct {
+			Opaque string `json:"opaque"`
+		} `json:"arguments"`
+	}
+
+	permissions := "rdwr"
+	if readonly {
+		permissions = "rdonly"
+	}
+
+	req.Execute = "add-fd"
+	req.Arguments.Opaque = fmt.Sprintf("%s:%s", permissions, name)
+
+	reqJSON, err := json.Marshal(req)
+	if err != nil {
+		return nil, err
+	}
+
+	ret, err := m.qmp.RunWithFile(reqJSON, file)
+	if err != nil {
+		// Confirm the daemon didn't die.
+		errPing := m.ping()
+		if errPing != nil {
+			return nil, errPing
+		}
+
+		return nil, err
+	}
+
 	// Prepare the response.
 	var resp struct {
 		Return AddFdInfo `json:"return"`
 	}
 
-	// Check if disconnected
-	if m.disconnected {
-		return resp.Return, ErrMonitorDisconnect
-	}
-
-	permissions := "rdwr"
-
-	if readonly {
-		permissions = "rdonly"
-	}
-
-	// Query the status.
-	ret, err := m.qmp.RunWithFile([]byte(fmt.Sprintf("{'execute': 'add-fd', 'arguments': {'opaque': '%s:%s'}}", permissions, name)), file)
-	if err != nil {
-		// Confirm the daemon didn't die.
-		errPing := m.ping()
-		if errPing != nil {
-			return resp.Return, errPing
-		}
-
-		return resp.Return, err
-	}
-
 	err = json.Unmarshal(ret, &resp)
 	if err != nil {
-		return resp.Return, err
+		return nil, err
 	}
 
-	return resp.Return, nil
+	return &resp.Return, nil
 }
 
 // RemoveFDFromFDSet removes an FD with the given name from an FD set.
 func (m *Monitor) RemoveFDFromFDSet(name string) error {
-	// Check if disconnected
-	if m.disconnected {
-		return ErrMonitorDisconnect
-	}
-
 	// Prepare the response.
 	var resp struct {
 		Return []FdsetInfo `json:"return"`
@@ -273,6 +252,33 @@ func (m *Monitor) RemoveFDFromFDSet(name string) error {
 	return nil
 }
 
+// MigrateSetCapabilities sets the capabilities used during migration.
+func (m *Monitor) MigrateSetCapabilities(caps map[string]bool) error {
+	var args struct {
+		Capabilities []struct {
+			Capability string `json:"capability"`
+			State      bool   `json:"state"`
+		} `json:"capabilities"`
+	}
+
+	for capName, state := range caps {
+		args.Capabilities = append(args.Capabilities, struct {
+			Capability string `json:"capability"`
+			State      bool   `json:"state"`
+		}{
+			Capability: capName,
+			State:      state,
+		})
+	}
+
+	err := m.run("migrate-set-capabilities", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
 // Migrate starts a migration stream.
 func (m *Monitor) Migrate(uri string) error {
 	// Query the status.
@@ -282,10 +288,15 @@ func (m *Monitor) Migrate(uri string) error {
 		return err
 	}
 
+	return nil
+}
+
+// MigrateWait waits until migration job reaches the specified status.
+// Returns nil if the migraton job reaches the specified status or an error if the migration job is in the failed
+// status.
+func (m *Monitor) MigrateWait(state string) error {
 	// Wait until it completes or fails.
 	for {
-		time.Sleep(1 * time.Second)
-
 		// Prepare the response.
 		var resp struct {
 			Return struct {
@@ -299,19 +310,35 @@ func (m *Monitor) Migrate(uri string) error {
 		}
 
 		if resp.Return.Status == "failed" {
-			return fmt.Errorf("Migration call failed")
+			return fmt.Errorf("Migrate call failed")
 		}
 
-		if resp.Return.Status == "completed" {
-			break
+		if resp.Return.Status == state {
+			return nil
 		}
+
+		time.Sleep(1 * time.Second)
+	}
+}
+
+// MigrateContinue continues a migration stream.
+func (m *Monitor) MigrateContinue(fromState string) error {
+	var args struct {
+		State string `json:"state"`
+	}
+
+	args.State = fromState
+
+	err := m.run("migrate-continue", args, nil)
+	if err != nil {
+		return err
 	}
 
 	return nil
 }
 
 // MigrateIncoming starts the receiver of a migration stream.
-func (m *Monitor) MigrateIncoming(uri string) error {
+func (m *Monitor) MigrateIncoming(ctx context.Context, uri string) error {
 	// Query the status.
 	args := map[string]string{"uri": uri}
 	err := m.run("migrate-incoming", args, nil)
@@ -321,9 +348,7 @@ func (m *Monitor) MigrateIncoming(uri string) error {
 
 	// Wait until it completes or fails.
 	for {
-		time.Sleep(1 * time.Second)
-
-		// Preapre the response.
+		// Prepare the response.
 		var resp struct {
 			Return struct {
 				Status string `json:"status"`
@@ -336,15 +361,23 @@ func (m *Monitor) MigrateIncoming(uri string) error {
 		}
 
 		if resp.Return.Status == "failed" {
-			return fmt.Errorf("Migration call failed")
+			return fmt.Errorf("Migrate incoming call failed")
 		}
 
 		if resp.Return.Status == "completed" {
-			break
+			return nil
 		}
-	}
 
-	return nil
+		// Check context is cancelled last after checking job status.
+		// This way if the context is cancelled when the migration stream is ended this gives a chance to
+		// check for job success/failure before checking if the context has been cancelled.
+		err = ctx.Err()
+		if err != nil {
+			return err
+		}
+
+		time.Sleep(1 * time.Second)
+	}
 }
 
 // Powerdown tells the VM to gracefully shutdown.
@@ -437,6 +470,11 @@ func (m *Monitor) AddBlockDevice(blockDev map[string]any, device map[string]stri
 	revert := revert.New()
 	defer revert.Fail()
 
+	nodeName, ok := blockDev["node-name"].(string)
+	if !ok {
+		return fmt.Errorf("Device node name must be a string")
+	}
+
 	if blockDev != nil {
 		err := m.run("blockdev-add", blockDev, nil)
 		if err != nil {
@@ -444,11 +482,7 @@ func (m *Monitor) AddBlockDevice(blockDev map[string]any, device map[string]stri
 		}
 
 		revert.Add(func() {
-			blockDevDel := map[string]any{
-				"node-name": blockDev["devName"],
-			}
-
-			_ = m.run("blockdev-del", blockDevDel, nil)
+			_ = m.RemoveBlockDevice(nodeName)
 		})
 	}
 
@@ -468,7 +502,6 @@ func (m *Monitor) RemoveBlockDevice(blockDevName string) error {
 			"node-name": blockDevName,
 		}
 
-		// Retry a few times in case the blockdev is in use.
 		err := m.run("blockdev-del", blockDevName, nil)
 		if err != nil {
 			if strings.Contains(err.Error(), "is in use") {
@@ -486,13 +519,40 @@ func (m *Monitor) RemoveBlockDevice(blockDevName string) error {
 	return nil
 }
 
-// AddDevice adds a new device.
-func (m *Monitor) AddDevice(device map[string]string) error {
-	// Check if disconnected
-	if m.disconnected {
-		return ErrMonitorDisconnect
+// AddCharDevice adds a new character device.
+func (m *Monitor) AddCharDevice(device map[string]any) error {
+	if device != nil {
+		err := m.run("chardev-add", device, nil)
+		if err != nil {
+			return err
+		}
 	}
 
+	return nil
+}
+
+// RemoveCharDevice removes a character device.
+func (m *Monitor) RemoveCharDevice(deviceID string) error {
+	if deviceID != "" {
+		deviceID := map[string]string{
+			"id": deviceID,
+		}
+
+		err := m.run("chardev-remove", deviceID, nil)
+		if err != nil {
+			if strings.Contains(err.Error(), "not found") {
+				return nil
+			}
+
+			return err
+		}
+	}
+
+	return nil
+}
+
+// AddDevice adds a new device.
+func (m *Monitor) AddDevice(device map[string]string) error {
 	if device != nil {
 		err := m.run("device_add", device, nil)
 		if err != nil {
@@ -505,11 +565,6 @@ func (m *Monitor) AddDevice(device map[string]string) error {
 
 // RemoveDevice removes a device.
 func (m *Monitor) RemoveDevice(deviceID string) error {
-	// Check if disconnected
-	if m.disconnected {
-		return ErrMonitorDisconnect
-	}
-
 	if deviceID != "" {
 		deviceID := map[string]string{
 			"id": deviceID,
@@ -690,3 +745,299 @@ func (m *Monitor) AddSecret(id string, secret string) error {
 
 	return nil
 }
+
+// AMDSEVCapabilities represents the SEV capabilities of QEMU.
+type AMDSEVCapabilities struct {
+	PDH             string `json:"pdh"`               // Platform Diffie-Hellman key (base64-encoded)
+	CertChain       string `json:"cert-chain"`        // PDH certificate chain (base64-encoded)
+	CPU0Id          string `json:"cpu0-id"`           // Unique ID of CPU0 (base64-encoded)
+	CBitPos         int    `json:"cbitpos"`           // C-bit location in page table entry
+	ReducedPhysBits int    `json:"reduced-phys-bits"` // Number of physical address bit reduction when SEV is enabled
+}
+
+// SEVCapabilities is used to get the SEV capabilities, and is supported on AMD X86 platforms only.
+func (m *Monitor) SEVCapabilities() (AMDSEVCapabilities, error) {
+	// Prepare the response
+	var resp struct {
+		Return AMDSEVCapabilities `json:"return"`
+	}
+
+	err := m.run("query-sev-capabilities", nil, &resp)
+	if err != nil {
+		return AMDSEVCapabilities{}, fmt.Errorf("Failed querying SEV capability for QEMU: %w", err)
+	}
+
+	return resp.Return, nil
+}
+
+// NBDServerStart starts internal NBD server and returns a connection to it.
+func (m *Monitor) NBDServerStart() (net.Conn, error) {
+	var args struct {
+		Addr struct {
+			Data struct {
+				Path     string `json:"path"`
+				Abstract bool   `json:"abstract"`
+			} `json:"data"`
+			Type string `json:"type"`
+		} `json:"addr"`
+		MaxConnections int `json:"max-connections"`
+	}
+
+	// Create abstract unix listener.
+	listener, err := net.Listen("unix", "")
+	if err != nil {
+		return nil, fmt.Errorf("Failed creating unix listener: %w", err)
+	}
+
+	// Get the random address, and then close the listener, and pass the address for use with nbd-server-start.
+	listenAddress := listener.Addr().String()
+	_ = listener.Close()
+
+	args.Addr.Type = "unix"
+	args.Addr.Data.Path = strings.TrimPrefix(listenAddress, "@")
+	args.Addr.Data.Abstract = true
+	args.MaxConnections = 1
+
+	err = m.run("nbd-server-start", args, nil)
+	if err != nil {
+		return nil, err
+	}
+
+	// Connect to the NBD server and return the connection.
+	conn, err := net.Dial("unix", listenAddress)
+	if err != nil {
+		return nil, fmt.Errorf("Failed connecting to NBD server: %w", err)
+	}
+
+	return conn, nil
+}
+
+// NBDServerStop stops the internal NBD server.
+func (m *Monitor) NBDServerStop() error {
+	err := m.run("nbd-server-stop", nil, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// NBDBlockExportAdd exports a writable device via the NBD server.
+func (m *Monitor) NBDBlockExportAdd(deviceNodeName string) error {
+	var args struct {
+		ID       string `json:"id"`
+		Type     string `json:"type"`
+		NodeName string `json:"node-name"`
+		Writable bool   `json:"writable"`
+	}
+
+	args.ID = deviceNodeName
+	args.Type = "nbd"
+	args.NodeName = deviceNodeName
+	args.Writable = true
+
+	err := m.run("block-export-add", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// BlockDevSnapshot creates a snapshot of a device using the specified snapshot device.
+func (m *Monitor) BlockDevSnapshot(deviceNodeName string, snapshotNodeName string) error {
+	var args struct {
+		Node    string `json:"node"`
+		Overlay string `json:"overlay"`
+	}
+
+	args.Node = deviceNodeName
+	args.Overlay = snapshotNodeName
+
+	err := m.run("blockdev-snapshot", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// blockJobWaitReady waits until the specified jobID is ready, errored or missing.
+// Returns nil if the job is ready, otherwise an error.
+func (m *Monitor) blockJobWaitReady(jobID string) error {
+	for {
+		var resp struct {
+			Return []struct {
+				Device string `json:"device"`
+				Ready  bool   `json:"ready"`
+				Error  string `json:"error"`
+			} `json:"return"`
+		}
+
+		err := m.run("query-block-jobs", nil, &resp)
+		if err != nil {
+			return err
+		}
+
+		found := false
+		for _, job := range resp.Return {
+			if job.Device != jobID {
+				continue
+			}
+
+			if job.Error != "" {
+				return fmt.Errorf("Failed block job: %s", job.Error)
+			}
+
+			if job.Ready {
+				return nil
+			}
+
+			found = true
+		}
+
+		if !found {
+			return fmt.Errorf("Specified block job not found")
+		}
+
+		time.Sleep(1 * time.Second)
+	}
+}
+
+// BlockCommit merges a snapshot device back into its parent device.
+func (m *Monitor) BlockCommit(deviceNodeName string) error {
+	var args struct {
+		Device string `json:"device"`
+		JobID  string `json:"job-id"`
+	}
+
+	args.Device = deviceNodeName
+	args.JobID = args.Device
+
+	err := m.run("block-commit", args, nil)
+	if err != nil {
+		return err
+	}
+
+	err = m.blockJobWaitReady(args.JobID)
+	if err != nil {
+		return err
+	}
+
+	err = m.BlockJobComplete(args.JobID)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// BlockDevMirror mirrors the top device to the target device.
+func (m *Monitor) BlockDevMirror(deviceNodeName string, targetNodeName string) error {
+	var args struct {
+		Device   string `json:"device"`
+		Target   string `json:"target"`
+		Sync     string `json:"sync"`
+		JobID    string `json:"job-id"`
+		CopyMode string `json:"copy-mode"`
+	}
+
+	args.Device = deviceNodeName
+	args.Target = targetNodeName
+	args.JobID = deviceNodeName
+
+	// Only synchronise the top level device (usually a snapshot).
+	args.Sync = "top"
+
+	// When data is written to the source, write it (synchronously) to the target as well.
+	// In addition, data is copied in background just like in background mode.
+	// This ensures that the source and target converge at the cost of I/O performance during sync.
+	args.CopyMode = "write-blocking"
+
+	err := m.run("blockdev-mirror", args, nil)
+	if err != nil {
+		return err
+	}
+
+	err = m.blockJobWaitReady(args.JobID)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// BlockJobCancel cancels an ongoing block job.
+func (m *Monitor) BlockJobCancel(deviceNodeName string) error {
+	var args struct {
+		Device string `json:"device"`
+	}
+
+	args.Device = deviceNodeName
+
+	err := m.run("block-job-cancel", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// BlockJobComplete completes a block job that is in reader state.
+func (m *Monitor) BlockJobComplete(deviceNodeName string) error {
+	var args struct {
+		Device string `json:"device"`
+	}
+
+	args.Device = deviceNodeName
+
+	err := m.run("block-job-complete", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// Eject ejects a removable drive.
+func (m *Monitor) Eject(id string) error {
+	var args struct {
+		ID string `json:"id"`
+	}
+
+	args.ID = id
+
+	err := m.run("eject", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// SetBlockThrottle applies an I/O limit on a disk.
+func (m *Monitor) SetBlockThrottle(id string, bytesRead int, bytesWrite int, iopsRead int, iopsWrite int) error {
+	var args struct {
+		ID string `json:"id"`
+
+		Bytes      int `json:"bps"`
+		BytesRead  int `json:"bps_rd"`
+		BytesWrite int `json:"bps_wr"`
+		IOPs       int `json:"iops"`
+		IOPsRead   int `json:"iops_rd"`
+		IOPsWrite  int `json:"iops_wr"`
+	}
+
+	args.ID = id
+	args.BytesRead = bytesRead
+	args.BytesWrite = bytesWrite
+	args.IOPsRead = iopsRead
+	args.IOPsWrite = iopsWrite
+
+	err := m.run("block_set_io_throttle", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
