@@ -1,6 +1,7 @@
 package instance
 
 import (
+	"context"
 	"crypto/x509"
 	"io"
 	"net"
@@ -9,17 +10,18 @@ import (
 
 	liblxc "github.com/lxc/go-lxc"
 	"github.com/pkg/sftp"
+	"google.golang.org/protobuf/proto"
 
-	"github.com/lxc/lxd/lxd/backup"
-	"github.com/lxc/lxd/lxd/cgroup"
-	"github.com/lxc/lxd/lxd/db"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/instance/operationlock"
-	"github.com/lxc/lxd/lxd/metrics"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/idmap"
+	"github.com/canonical/lxd/lxd/backup"
+	"github.com/canonical/lxd/lxd/cgroup"
+	"github.com/canonical/lxd/lxd/db"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/instance/operationlock"
+	"github.com/canonical/lxd/lxd/metrics"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // HookStart hook used when instance has started.
@@ -49,6 +51,12 @@ const TemplateTriggerCopy TemplateTrigger = "copy"
 // TemplateTriggerRename for when an instance is renamed.
 const TemplateTriggerRename TemplateTrigger = "rename"
 
+// PowerStateRunning represents the power state stored when an instance is running.
+const PowerStateRunning = "RUNNING"
+
+// PowerStateStopped represents the power state stored when an instance is stopped.
+const PowerStateStopped = "STOPPED"
+
 // ConfigReader is used to read instance config.
 type ConfigReader interface {
 	Project() api.Project
@@ -70,9 +78,9 @@ type Instance interface {
 	Start(stateful bool) error
 	Stop(stateful bool) error
 	Restart(timeout time.Duration) error
+	Rebuild(img *api.Image, op *operations.Operation) error
 	Unfreeze() error
 	RegisterDevices()
-	SaveConfigFile() error
 
 	Info() Info
 	IsPrivileged() bool
@@ -135,6 +143,7 @@ type Instance interface {
 
 	// Paths.
 	Path() string
+	ExecOutputPath() string
 	RootfsPath() string
 	TemplatesPath() string
 	StatePath() string
@@ -148,7 +157,8 @@ type Instance interface {
 
 	// Migration.
 	CanMigrate() (bool, bool)
-	Migrate(args *CriuMigrationArgs) error
+	MigrateSend(args MigrateSendArgs) error
+	MigrateReceive(args MigrateReceiveArgs) error
 
 	// Progress reporting.
 	SetOperation(op *operations.Operation)
@@ -169,7 +179,7 @@ type Container interface {
 	ConsoleLog(opts liblxc.ConsoleLogOptions) (string, error)
 	InsertSeccompUnixDevice(prefix string, m deviceConfig.Device, pid int) error
 	DevptsFd() (*os.File, error)
-	IdmappedStorage(path string) idmap.IdmapStorageType
+	IdmappedStorage(path string, fstype string) idmap.IdmapStorageType
 }
 
 // VM interface is for VM specific functions.
@@ -177,6 +187,10 @@ type VM interface {
 	Instance
 
 	AgentCertificate() *x509.Certificate
+
+	// UEFI vars handling.
+	UEFIVars() (*api.InstanceUEFIVars, error)
+	UEFIVarsUpdate(newUEFIVarsSet api.InstanceUEFIVars) error
 }
 
 // CriuMigrationArgs arguments for CRIU migration.
@@ -189,6 +203,7 @@ type CriuMigrationArgs struct {
 	DumpDir      string
 	PreDumpDir   string
 	Features     liblxc.CriuFeatures
+	Op           *operationlock.InstanceOperation
 }
 
 // Info represents information about an instance driver.
@@ -197,5 +212,32 @@ type Info struct {
 	Version  string            // Version number of a loaded instance driver
 	Error    error             // Whether there is an operational impediment.
 	Type     instancetype.Type // Instance type that the driver provides support for.
-	Features []string          // List of supported features.
+	Features map[string]any    // Map of supported features.
+}
+
+// MigrateArgs represent arguments for instance migration send and receive.
+type MigrateArgs struct {
+	ControlSend           func(m proto.Message) error
+	ControlReceive        func(m proto.Message) error
+	StateConn             func(ctx context.Context) (io.ReadWriteCloser, error)
+	FilesystemConn        func(ctx context.Context) (io.ReadWriteCloser, error)
+	Snapshots             bool
+	Live                  bool
+	Disconnect            func()
+	ClusterMoveSourceName string // Will be empty if not a cluster move, othwise indicates the source instance.
+}
+
+// MigrateSendArgs represent arguments for instance migration send.
+type MigrateSendArgs struct {
+	MigrateArgs
+
+	AllowInconsistent bool
+}
+
+// MigrateReceiveArgs represent arguments for instance migration receive.
+type MigrateReceiveArgs struct {
+	MigrateArgs
+
+	InstanceOperation *operationlock.InstanceOperation
+	Refresh           bool
 }
