@@ -3,8 +3,10 @@ package main
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
+	"io/fs"
 	"net/http"
 	"net/url"
 	"os"
@@ -18,20 +20,21 @@ import (
 	"github.com/gorilla/websocket"
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/db/operationtype"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/cancel"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/netutils"
-	"github.com/lxc/lxd/shared/tcp"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/cluster"
+	"github.com/canonical/lxd/lxd/db/operationtype"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/cancel"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/tcp"
+	"github.com/canonical/lxd/shared/version"
+	"github.com/canonical/lxd/shared/ws"
 )
 
 const execWSControl = -1
@@ -51,11 +54,12 @@ type execWs struct {
 	s                     *state.State
 }
 
+// Metadata returns a map of metadata.
 func (s *execWs) Metadata() any {
 	fds := shared.Jmap{}
 	for fd, secret := range s.fds {
 		if fd == execWSControl {
-			fds["control"] = secret
+			fds[api.SecretNameControl] = secret
 		} else {
 			fds[strconv.Itoa(fd)] = secret
 		}
@@ -69,6 +73,7 @@ func (s *execWs) Metadata() any {
 	}
 }
 
+// Connect connects to the websocket.
 func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
@@ -77,13 +82,12 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 
 	for fd, fdSecret := range s.fds {
 		if secret == fdSecret {
-			conn, err := shared.WebsocketUpgrader.Upgrade(w, r, nil)
+			conn, err := ws.Upgrader.Upgrade(w, r, nil)
 			if err != nil {
 				return err
 			}
 
 			s.connsLock.Lock()
-			defer s.connsLock.Unlock()
 
 			val, found := s.conns[fd]
 			if found && val == nil {
@@ -94,7 +98,7 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 				if remoteTCP != nil {
 					err = tcp.SetTimeouts(remoteTCP, 0)
 					if err != nil {
-						logger.Error("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
+						logger.Warn("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
 					}
 				}
 
@@ -130,17 +134,19 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 					}
 
 					if c == nil {
+						s.connsLock.Unlock()
 						return nil // Not all required connections connected yet.
 					}
 				}
 
 				s.waitRequiredConnected.Cancel() // All required connections now connected.
+				s.connsLock.Unlock()
 				return nil
 			} else if !found {
 				return fmt.Errorf("Unknown websocket number")
-			} else {
-				return fmt.Errorf("Websocket number already connected")
 			}
+
+			return fmt.Errorf("Websocket number already connected")
 		}
 	}
 
@@ -149,6 +155,7 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 	return os.ErrPermission
 }
 
+// Do connects to the websocket and executes the operation.
 func (s *execWs) Do(op *operations.Operation) error {
 	// Once this function ends ensure that any connected websockets are closed.
 	defer func() {
@@ -166,7 +173,6 @@ func (s *execWs) Do(op *operations.Operation) error {
 	logger.Debug("Waiting for exec websockets to connect")
 	select {
 	case <-s.waitRequiredConnected.Done():
-		break
 	case <-time.After(time.Second * 5):
 		return fmt.Errorf("Timed out waiting for websockets to connect")
 	}
@@ -188,7 +194,11 @@ func (s *execWs) Do(op *operations.Operation) error {
 			var rootUID, rootGID int64
 			var devptsFd *os.File
 
-			c := s.instance.(instance.Container)
+			c, ok := s.instance.(instance.Container)
+			if !ok {
+				return fmt.Errorf("Invalid instance type")
+			}
+
 			idmapset, err := c.CurrentIdmap()
 			if err != nil {
 				return err
@@ -248,13 +258,13 @@ func (s *execWs) Do(op *operations.Operation) error {
 		stderr = ttys[execWSStderr]
 	}
 
-	waitAttachedChildIsDead := cancel.New(context.Background())
+	waitAttachedChildIsDead, markAttachedChildIsDead := context.WithCancel(context.Background())
 	var wgEOF sync.WaitGroup
 
 	// Define a function to clean up TTYs and sockets when done.
 	finisher := func(cmdResult int, cmdErr error) error {
-		// Close this before closing the control connection so control handler can detect command ending.
-		waitAttachedChildIsDead.Cancel()
+		// Cancel this before closing the control connection so control handler can detect command ending.
+		markAttachedChildIsDead()
 
 		for _, tty := range ttys {
 			_ = tty.Close()
@@ -293,7 +303,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 		return finisher(-1, err)
 	}
 
-	l := logger.AddContext(logger.Log, logger.Ctx{"project": s.instance.Project().Name, "instance": s.instance.Name(), "PID": cmd.PID(), "interactive": s.req.Interactive})
+	l := logger.AddContext(logger.Ctx{"project": s.instance.Project().Name, "instance": s.instance.Name(), "PID": cmd.PID(), "interactive": s.req.Interactive})
 	l.Debug("Instance process started")
 
 	var cmdKillOnce sync.Once
@@ -400,36 +410,39 @@ func (s *execWs) Do(op *operations.Operation) error {
 		go func() {
 			defer wgEOF.Done()
 
+			var readErr, writeErr error
 			l.Debug("Exec mirror websocket started", logger.Ctx{"number": 0})
-			defer l.Debug("Exec mirror websocket finished", logger.Ctx{"number": 0})
+			defer func() {
+				l.Debug("Exec mirror websocket finished", logger.Ctx{"number": 0, "readErr": readErr, "writeErr": writeErr})
+			}()
 
 			s.connsLock.Lock()
 			conn := s.conns[0]
 			s.connsLock.Unlock()
 
-			var readDone, writeDone chan bool
-
+			var readDone, writeDone chan error
 			if s.instance.Type() == instancetype.Container {
 				// For containers, we are running the command via the local LXD managed PTY and so
-				// need special signal handling provided by netutils.WebsocketExecMirror.
-				readDone, writeDone = netutils.WebsocketExecMirror(conn, ptys[0], ptys[0], waitAttachedChildIsDead.Done(), int(ptys[0].Fd()))
+				// need to use the same PTY handle for both read and write.
+				readDone, writeDone = ws.Mirror(conn, shared.NewExecWrapper(waitAttachedChildIsDead, ptys[0]))
 			} else {
-				// For VMs we are just relaying the websockets between client and lxd-agent, so no
-				// need for the special signal handling provided by netutils.WebsocketExecMirror.
-				readDone = shared.WebsocketSendStream(conn, ptys[execWSStdout], -1)
-				writeDone = shared.WebsocketRecvStream(ttys[execWSStdin], conn)
+				readDone = ws.MirrorRead(conn, ptys[execWSStdout])
+				writeDone = ws.MirrorWrite(conn, ttys[execWSStdin])
 			}
 
-			<-readDone
-			<-writeDone
+			readErr = <-readDone
+			writeErr = <-writeDone
 			_ = conn.Close()
 		}()
 	} else {
 		wgEOF.Add(len(ttys) - 1)
 		for i := 0; i < len(ttys); i++ {
 			go func(i int) {
+				var err error
 				l.Debug("Exec mirror websocket started", logger.Ctx{"number": i})
-				defer l.Debug("Exec mirror websocket finished", logger.Ctx{"number": i})
+				defer func() {
+					l.Debug("Exec mirror websocket finished", logger.Ctx{"number": i, "err": err})
+				}()
 
 				s.connsLock.Lock()
 				conn := s.conns[i]
@@ -459,10 +472,10 @@ func (s *execWs) Do(op *operations.Operation) error {
 				}
 
 				if i == execWSStdin {
-					<-shared.WebsocketRecvStream(ttys[i], conn)
+					err = <-ws.MirrorWrite(conn, ttys[i])
 					_ = ttys[i].Close()
 				} else {
-					<-shared.WebsocketSendStream(conn, ptys[i], -1)
+					err = <-ws.MirrorRead(conn, shared.NewExecWrapper(waitAttachedChildIsDead, ptys[i]))
 					_ = ptys[i].Close()
 					wgEOF.Done()
 				}
@@ -478,49 +491,51 @@ func (s *execWs) Do(op *operations.Operation) error {
 
 // swagger:operation POST /1.0/instances/{name}/exec instances instance_exec_post
 //
-// Run a command
+//	Run a command
 //
-// Executes a command inside an instance.
+//	Executes a command inside an instance.
 //
-// The returned operation metadata will contain either 2 or 4 websockets.
-// In non-interactive mode, you'll get one websocket for each of stdin, stdout and stderr.
-// In interactive mode, a single bi-directional websocket is used for stdin and stdout/stderr.
+//	The returned operation metadata will contain either 2 or 4 websockets.
+//	In non-interactive mode, you'll get one websocket for each of stdin, stdout and stderr.
+//	In interactive mode, a single bi-directional websocket is used for stdin and stdout/stderr.
 //
-// An additional "control" socket is always added on top which can be used for out of band communication with LXD.
-// This allows sending signals and window sizing information through.
+//	An additional "control" socket is always added on top which can be used for out of band communication with LXD.
+//	This allows sending signals and window sizing information through.
 //
-// ---
-// consumes:
-//   - application/json
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: project
-//     description: Project name
-//     type: string
-//     example: default
-//   - in: body
-//     name: exec
-//     description: Exec request
-//     schema:
-//       $ref: "#/definitions/InstanceExecPost"
-// responses:
-//   "202":
-//     $ref: "#/responses/Operation"
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	consumes:
+//	  - application/json
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	  - in: body
+//	    name: exec
+//	    description: Exec request
+//	    schema:
+//	      $ref: "#/definitions/InstanceExecPost"
+//	responses:
+//	  "202":
+//	    $ref: "#/responses/Operation"
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func instanceExecPost(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	instanceType, err := urlInstanceTypeDetect(r)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	projectName := projectParam(r)
+	projectName := request.ProjectParam(r)
 	name, err := url.PathUnescape(mux.Vars(r)["name"])
 	if err != nil {
 		return response.SmartError(err)
@@ -541,14 +556,23 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
+	// Constraint validations.
+	if post.RecordOutput && post.WaitForWS {
+		return response.BadRequest(fmt.Errorf("Cannot use %q in combination with %q", "record-output", "wait-for-websocket"))
+	}
+
+	if post.Interactive && post.RecordOutput {
+		return response.BadRequest(fmt.Errorf("Cannot use %q in combination with %q", "interactive", "record-output"))
+	}
+
 	// Forward the request if the container is remote.
-	client, err := cluster.ConnectIfInstanceIsRemote(d.db.Cluster, projectName, name, d.endpoints.NetworkCert(), d.serverCert(), r, instanceType)
+	client, err := cluster.ConnectIfInstanceIsRemote(s, projectName, name, r, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	if client != nil {
-		url := api.NewURL().Path("1.0", "instances", name, "exec").Project(projectName)
+		url := api.NewURL().Path(version.APIVersion, "instances", name, "exec").Project(projectName)
 		resp, _, err := client.RawQuery("POST", url.String(), post, "")
 		if err != nil {
 			return response.SmartError(err)
@@ -562,7 +586,7 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 		return operations.ForwardedOperationResponse(projectName, opAPI)
 	}
 
-	inst, err := instance.LoadByProjectAndName(d.State(), projectName, name)
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -662,14 +686,14 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 		ws.instance = inst
 		ws.req = post
 
-		resources := map[string][]string{}
-		resources["instances"] = []string{ws.instance.Name()}
+		resources := map[string][]api.URL{}
+		resources["instances"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", ws.instance.Name())}
 
 		if ws.instance.Type() == instancetype.Container {
 			resources["containers"] = resources["instances"]
 		}
 
-		op, err := operations.OperationCreate(d.State(), projectName, operations.OperationClassWebsocket, operationtype.CommandExec, resources, ws.Metadata(), ws.Do, nil, ws.Connect, r)
+		op, err := operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.CommandExec, resources, ws.Metadata(), ws.Do, nil, ws.Connect, r)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -684,15 +708,22 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 		var stdout, stderr *os.File
 
 		if post.RecordOutput {
+			// Ensure exec-output directory exists
+			execOutputDir := inst.ExecOutputPath()
+			err = os.Mkdir(execOutputDir, 0600)
+			if err != nil && !errors.Is(err, fs.ErrExist) {
+				return err
+			}
+
 			// Prepare stdout and stderr recording.
-			stdout, err = os.OpenFile(filepath.Join(inst.LogPath(), fmt.Sprintf("exec_%s.stdout", op.ID())), os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
+			stdout, err = os.OpenFile(filepath.Join(execOutputDir, fmt.Sprintf("exec_%s.stdout", op.ID())), os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
 			if err != nil {
 				return err
 			}
 
 			defer func() { _ = stdout.Close() }()
 
-			stderr, err = os.OpenFile(filepath.Join(inst.LogPath(), fmt.Sprintf("exec_%s.stderr", op.ID())), os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
+			stderr, err = os.OpenFile(filepath.Join(execOutputDir, fmt.Sprintf("exec_%s.stderr", op.ID())), os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
 			if err != nil {
 				return err
 			}
@@ -701,8 +732,8 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 
 			// Update metadata with the right URLs.
 			metadata["output"] = shared.Jmap{
-				"1": fmt.Sprintf("/%s/instances/%s/logs/%s", version.APIVersion, inst.Name(), filepath.Base(stdout.Name())),
-				"2": fmt.Sprintf("/%s/instances/%s/logs/%s", version.APIVersion, inst.Name(), filepath.Base(stderr.Name())),
+				"1": fmt.Sprintf("/%s/instances/%s/logs/exec-output/%s", version.APIVersion, inst.Name(), filepath.Base(stdout.Name())),
+				"2": fmt.Sprintf("/%s/instances/%s/logs/exec-output/%s", version.APIVersion, inst.Name(), filepath.Base(stderr.Name())),
 			}
 		}
 
@@ -712,7 +743,7 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 			return err
 		}
 
-		l := logger.AddContext(logger.Log, logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "PID": cmd.PID(), "recordOutput": post.RecordOutput})
+		l := logger.AddContext(logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "PID": cmd.PID(), "recordOutput": post.RecordOutput})
 		l.Debug("Instance process started")
 
 		exitStatus, cmdErr := cmd.Wait()
@@ -731,14 +762,14 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 		return nil
 	}
 
-	resources := map[string][]string{}
-	resources["instances"] = []string{name}
+	resources := map[string][]api.URL{}
+	resources["instances"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", name)}
 
 	if inst.Type() == instancetype.Container {
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(d.State(), projectName, operations.OperationClassTask, operationtype.CommandExec, resources, nil, run, nil, nil, r)
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.CommandExec, resources, nil, run, nil, nil, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
