@@ -15,17 +15,20 @@ import (
 	"github.com/gorilla/mux"
 	"github.com/pkg/sftp"
 
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/lifecycle"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/lifecycle"
+	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 func instanceFileHandler(d *Daemon, r *http.Request) response.Response {
-	projectName := projectParam(r)
+	s := d.State()
+
+	projectName := request.ProjectParam(r)
 	name, err := url.PathUnescape(mux.Vars(r)["name"])
 	if err != nil {
 		return response.SmartError(err)
@@ -41,7 +44,7 @@ func instanceFileHandler(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	resp, err := forwardedResponseIfInstanceIsRemote(d, r, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -51,7 +54,7 @@ func instanceFileHandler(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Load the instance.
-	inst, err := instance.LoadByProjectAndName(d.State(), projectName, name)
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -68,13 +71,13 @@ func instanceFileHandler(d *Daemon, r *http.Request) response.Response {
 
 	switch r.Method {
 	case "GET":
-		return instanceFileGet(d.State(), inst, path, r)
+		return instanceFileGet(s, inst, path, r)
 	case "HEAD":
-		return instanceFileHead(d.State(), inst, path, r)
+		return instanceFileHead(s, inst, path, r)
 	case "POST":
-		return instanceFilePost(d.State(), inst, path, r)
+		return instanceFilePost(s, inst, path, r)
 	case "DELETE":
-		return instanceFileDelete(d.State(), inst, path, r)
+		return instanceFileDelete(s, inst, path, r)
 	default:
 		return response.NotFound(fmt.Errorf("Method %q not found", r.Method))
 	}
@@ -82,72 +85,72 @@ func instanceFileHandler(d *Daemon, r *http.Request) response.Response {
 
 // swagger:operation GET /1.0/instances/{name}/files instances instance_files_get
 //
-// Get a file
+//	Get a file
 //
-// Gets the file content. If it's a directory, a json list of files will be returned instead.
+//	Gets the file content. If it's a directory, a json list of files will be returned instead.
 //
-// ---
-// produces:
-//   - application/json
-//   - application/octet-stream
-// parameters:
-//   - in: query
-//     name: path
-//     description: Path to the file
-//     type: string
-//     example: default
-//   - in: query
-//     name: project
-//     description: Project name
-//     type: string
-//     example: default
-// responses:
-//   "200":
-//      description: Raw file or directory listing
-//      headers:
-//        X-LXD-uid:
-//          description: File owner UID
-//          schema:
-//            type: integer
-//        X-LXD-gid:
-//          description: File owner GID
-//          schema:
-//            type: integer
-//        X-LXD-mode:
-//          description: Mode mask
-//          schema:
-//            type: integer
-//        X-LXD-modified:
-//          description: Last modified date
-//          schema:
-//            type: string
-//        X-LXD-type:
-//          description: Type of file (file, symlink or directory)
-//          schema:
-//            type: string
-//      content:
-//        application/octet-stream:
-//          schema:
-//            type: string
-//            example: some-text
-//        application/json:
-//          schema:
-//            type: array
-//            items:
-//              type: string
-//            example: |-
-//              [
-//                "/etc",
-//                "/home"
-//              ]
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "404":
-//     $ref: "#/responses/NotFound"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	  - application/octet-stream
+//	parameters:
+//	  - in: query
+//	    name: path
+//	    description: Path to the file
+//	    type: string
+//	    example: default
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	responses:
+//	  "200":
+//	     description: Raw file or directory listing
+//	     headers:
+//	       X-LXD-uid:
+//	         description: File owner UID
+//	         schema:
+//	           type: integer
+//	       X-LXD-gid:
+//	         description: File owner GID
+//	         schema:
+//	           type: integer
+//	       X-LXD-mode:
+//	         description: Mode mask
+//	         schema:
+//	           type: integer
+//	       X-LXD-modified:
+//	         description: Last modified date
+//	         schema:
+//	           type: string
+//	       X-LXD-type:
+//	         description: Type of file (file, symlink or directory)
+//	         schema:
+//	           type: string
+//	     content:
+//	       application/octet-stream:
+//	         schema:
+//	           type: string
+//	           example: some-text
+//	       application/json:
+//	         schema:
+//	           type: array
+//	           items:
+//	             type: string
+//	           example: |-
+//	             [
+//	               "/etc",
+//	               "/home"
+//	             ]
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "404":
+//	    $ref: "#/responses/NotFound"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func instanceFileGet(s *state.State, inst instance.Instance, path string, r *http.Request) response.Response {
 	revert := revert.New()
 	defer revert.Fail()
@@ -263,54 +266,54 @@ func instanceFileGet(s *state.State, inst instance.Instance, path string, r *htt
 
 // swagger:operation HEAD /1.0/instances/{name}/files instances instance_files_head
 //
-// Get metadata for a file
+//	Get metadata for a file
 //
-// Gets the file or directory metadata.
+//	Gets the file or directory metadata.
 //
-// ---
-// parameters:
-//   - in: query
-//     name: path
-//     description: Path to the file
-//     type: string
-//     example: default
-//   - in: query
-//     name: project
-//     description: Project name
-//     type: string
-//     example: default
-// responses:
-//   "200":
-//      description: Raw file or directory listing
-//      headers:
-//        X-LXD-uid:
-//          description: File owner UID
-//          schema:
-//            type: integer
-//        X-LXD-gid:
-//          description: File owner GID
-//          schema:
-//            type: integer
-//        X-LXD-mode:
-//          description: Mode mask
-//          schema:
-//            type: integer
-//        X-LXD-modified:
-//          description: Last modified date
-//          schema:
-//            type: string
-//        X-LXD-type:
-//          description: Type of file (file, symlink or directory)
-//          schema:
-//            type: string
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "404":
-//     $ref: "#/responses/NotFound"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	parameters:
+//	  - in: query
+//	    name: path
+//	    description: Path to the file
+//	    type: string
+//	    example: default
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	responses:
+//	  "200":
+//	     description: Raw file or directory listing
+//	     headers:
+//	       X-LXD-uid:
+//	         description: File owner UID
+//	         schema:
+//	           type: integer
+//	       X-LXD-gid:
+//	         description: File owner GID
+//	         schema:
+//	           type: integer
+//	       X-LXD-mode:
+//	         description: Mode mask
+//	         schema:
+//	           type: integer
+//	       X-LXD-modified:
+//	         description: Last modified date
+//	         schema:
+//	           type: string
+//	       X-LXD-type:
+//	         description: Type of file (file, symlink or directory)
+//	         schema:
+//	           type: string
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "404":
+//	    $ref: "#/responses/NotFound"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func instanceFileHead(s *state.State, inst instance.Instance, path string, r *http.Request) response.Response {
 	revert := revert.New()
 	defer revert.Fail()
@@ -362,70 +365,76 @@ func instanceFileHead(s *state.State, inst instance.Instance, path string, r *ht
 
 // swagger:operation POST /1.0/instances/{name}/files instances instance_files_post
 //
-// Create or replace a file
+//	Create or replace a file
 //
-// Creates a new file in the instance.
+//	Creates a new file in the instance.
 //
-// ---
-// consumes:
-//   - application/octet-stream
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: path
-//     description: Path to the file
-//     type: string
-//     example: default
-//   - in: query
-//     name: project
-//     description: Project name
-//     type: string
-//     example: default
-//   - in: body
-//     name: raw_file
-//     description: Raw file content
-//   - in: header
-//     name: X-LXD-uid
-//     description: File owner UID
-//     schema:
-//       type: integer
-//     example: 1000
-//   - in: header
-//     name: X-LXD-gid
-//     description: File owner GID
-//     schema:
-//       type: integer
-//     example: 1000
-//   - in: header
-//     name: X-LXD-mode
-//     description: File mode
-//     schema:
-//       type: integer
-//     example: 0644
-//   - in: header
-//     name: X-LXD-type
-//     description: Type of file (file, symlink or directory)
-//     schema:
-//       type: string
-//     example: file
-//   - in: header
-//     name: X-LXD-write
-//     description: Write mode (overwrite or append)
-//     schema:
-//       type: string
-//     example: overwrite
-// responses:
-//   "200":
-//     $ref: "#/responses/EmptySyncResponse"
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "404":
-//     $ref: "#/responses/NotFound"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	consumes:
+//	  - application/octet-stream
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: path
+//	    description: Path to the file
+//	    type: string
+//	    example: default
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	  - in: body
+//	    name: raw_file
+//	    description: Raw file content
+//	  - in: header
+//	    name: X-LXD-uid
+//	    description: Base 10 32-bit integer for the file owner UID
+//	    schema:
+//	      type: integer
+//	    example: 1000
+//	  - in: header
+//	    name: X-LXD-gid
+//	    description: Base 10 32-bit integer for the file owner GID
+//	    schema:
+//	      type: integer
+//	    example: 1000
+//	  - in: header
+//	    name: X-LXD-mode
+//	    description: Base 10 (no leading `0`) or base 8 (leading `0`) unix permissions bits (other bits are truncated)
+//	    schema:
+//	      type: integer
+//	    example: 0644
+//	  - in: header
+//	    name: X-LXD-modify-perm
+//	    description: Comma-separated list of permissions to set for pre-existing files (0 or more of `uid`, `gid`, `mode`)
+//	    schema:
+//	      type: integer
+//	    example: uid,gid,mode
+//	  - in: header
+//	    name: X-LXD-type
+//	    description: Type of file (file, symlink or directory)
+//	    schema:
+//	      type: string
+//	    example: file
+//	  - in: header
+//	    name: X-LXD-write
+//	    description: Write mode (overwrite or append)
+//	    schema:
+//	      type: string
+//	    example: overwrite
+//	responses:
+//	  "200":
+//	    $ref: "#/responses/EmptySyncResponse"
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "404":
+//	    $ref: "#/responses/NotFound"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func instanceFilePost(s *state.State, inst instance.Instance, path string, r *http.Request) response.Response {
 	// Get a SFTP client.
 	client, err := inst.FileSFTP()
@@ -436,20 +445,19 @@ func instanceFilePost(s *state.State, inst instance.Instance, path string, r *ht
 	defer func() { _ = client.Close() }()
 
 	// Extract file ownership and mode from headers
-	uid, gid, mode, type_, write := shared.ParseLXDFileHeaders(r.Header)
-
-	if !shared.StringInSlice(write, []string{"overwrite", "append"}) {
-		return response.BadRequest(fmt.Errorf("Bad file write mode: %s", write))
+	headers, err := shared.ParseLXDFileHeaders(r.Header)
+	if err != nil {
+		return response.BadRequest(err)
 	}
 
 	// Check if the file already exists.
 	_, err = client.Stat(path)
 	exists := err == nil
 
-	if type_ == "file" {
+	if headers.Type == "file" {
 		fileMode := os.O_RDWR
 
-		if write == "overwrite" {
+		if headers.Write == "overwrite" {
 			fileMode |= os.O_CREATE | os.O_TRUNC
 		}
 
@@ -473,18 +481,24 @@ func instanceFilePost(s *state.State, inst instance.Instance, path string, r *ht
 			return response.InternalError(err)
 		}
 
-		if !exists {
-			// Set file permissions.
-			if mode >= 0 {
-				err = file.Chmod(fs.FileMode(mode))
+		/* backwards-compat: the permissions headers did not modify permissions
+		 * for existing files before the `instances_files_modify_permissions`
+		 * api extension.
+		 */
+		if !exists || headers.ModeModifyExisting {
+			if headers.Mode >= 0 {
+				err = file.Chmod(fs.FileMode(headers.Mode))
 				if err != nil {
 					return response.SmartError(err)
 				}
 			}
+		}
 
-			// Set file ownership.
-			if uid >= 0 || gid >= 0 {
-				err = file.Chown(int(uid), int(gid))
+		// Set file ownership.
+		if !exists || headers.UIDModifyExisting || headers.GIDModifyExisting {
+			if headers.UID >= 0 || headers.GID >= 0 {
+				// -1 leaves the id unchanged
+				err = file.Chown(int(headers.UID), int(headers.GID))
 				if err != nil {
 					return response.SmartError(err)
 				}
@@ -493,7 +507,7 @@ func instanceFilePost(s *state.State, inst instance.Instance, path string, r *ht
 
 		s.Events.SendLifecycle(inst.Project().Name, lifecycle.InstanceFilePushed.Event(inst, logger.Ctx{"path": path}))
 		return response.EmptySyncResponse
-	} else if type_ == "symlink" {
+	} else if headers.Type == "symlink" {
 		// Figure out target.
 		target, err := io.ReadAll(r.Body)
 		if err != nil {
@@ -514,7 +528,7 @@ func instanceFilePost(s *state.State, inst instance.Instance, path string, r *ht
 
 		s.Events.SendLifecycle(inst.Project().Name, lifecycle.InstanceFilePushed.Event(inst, logger.Ctx{"path": path}))
 		return response.EmptySyncResponse
-	} else if type_ == "directory" {
+	} else if headers.Type == "directory" {
 		// Check if it already exists.
 		if exists {
 			return response.EmptySyncResponse
@@ -527,19 +541,19 @@ func instanceFilePost(s *state.State, inst instance.Instance, path string, r *ht
 		}
 
 		// Set file permissions.
-		if mode < 0 {
+		if headers.Mode < 0 {
 			// Default mode for directories (sftp doesn't know about umask).
-			mode = 0750
+			headers.Mode = 0750
 		}
 
-		err = client.Chmod(path, fs.FileMode(mode))
+		err = client.Chmod(path, fs.FileMode(headers.Mode))
 		if err != nil {
 			return response.SmartError(err)
 		}
 
 		// Set file ownership.
-		if uid >= 0 || gid >= 0 {
-			err = client.Chown(path, int(uid), int(gid))
+		if headers.UID >= 0 || headers.GID >= 0 {
+			err = client.Chown(path, int(headers.UID), int(headers.GID))
 			if err != nil {
 				return response.SmartError(err)
 			}
@@ -548,41 +562,41 @@ func instanceFilePost(s *state.State, inst instance.Instance, path string, r *ht
 		s.Events.SendLifecycle(inst.Project().Name, lifecycle.InstanceFilePushed.Event(inst, logger.Ctx{"path": path}))
 		return response.EmptySyncResponse
 	} else {
-		return response.BadRequest(fmt.Errorf("Bad file type: %s", type_))
+		return response.BadRequest(fmt.Errorf("Bad file type: %s", headers.Type))
 	}
 }
 
 // swagger:operation DELETE /1.0/instances/{name}/files instances instance_files_delete
 //
-// Delete a file
+//	Delete a file
 //
-// Removes the file.
+//	Removes the file.
 //
-// ---
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: path
-//     description: Path to the file
-//     type: string
-//     example: default
-//   - in: query
-//     name: project
-//     description: Project name
-//     type: string
-//     example: default
-// responses:
-//   "200":
-//     $ref: "#/responses/EmptySyncResponse"
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "404":
-//     $ref: "#/responses/NotFound"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: path
+//	    description: Path to the file
+//	    type: string
+//	    example: default
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	responses:
+//	  "200":
+//	    $ref: "#/responses/EmptySyncResponse"
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "404":
+//	    $ref: "#/responses/NotFound"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func instanceFileDelete(s *state.State, inst instance.Instance, path string, r *http.Request) response.Response {
 	// Get a SFTP client.
 	client, err := inst.FileSFTP()
