@@ -8,14 +8,17 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/lxd/auth"
+	"github.com/canonical/lxd/lxd/cluster"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/version"
 )
 
 func coalesceErrors(local bool, errors map[string]error) error {
@@ -41,43 +44,45 @@ func coalesceErrors(local bool, errors map[string]error) error {
 
 // swagger:operation PUT /1.0/instances instances instances_put
 //
-// Bulk instance state update
+//	Bulk instance state update
 //
-// Changes the running state of all instances.
+//	Changes the running state of all instances.
 //
-// ---
-// consumes:
-//   - application/json
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: project
-//     description: Project name
-//     type: string
-//     example: default
-//   - in: body
-//     name: state
-//     description: State
-//     required: false
-//     schema:
-//       $ref: "#/definitions/InstancesPut"
-// responses:
-//   "202":
-//     $ref: "#/responses/Operation"
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	consumes:
+//	  - application/json
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	  - in: body
+//	    name: state
+//	    description: State
+//	    required: false
+//	    schema:
+//	      $ref: "#/definitions/InstancesPut"
+//	responses:
+//	  "202":
+//	    $ref: "#/responses/Operation"
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func instancesPut(d *Daemon, r *http.Request) response.Response {
-	projectName := projectParam(r)
+	projectName := request.ProjectParam(r)
 
 	// Don't mess with instances while in setup mode.
 	<-d.waitReady.Done()
 
-	c, err := instance.LoadNodeAll(d.State(), instancetype.Any)
+	s := d.State()
+
+	c, err := instance.LoadNodeAll(s, instancetype.Any)
 	if err != nil {
 		return response.BadRequest(err)
 	}
@@ -90,7 +95,23 @@ func instancesPut(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	action := shared.InstanceAction(req.State.Action)
+	action := instancetype.InstanceAction(req.State.Action)
+
+	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), r, auth.EntitlementCanUpdateState, entity.TypeInstance)
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	for _, inst := range c {
+		if inst.Project().Name != projectName {
+			continue
+		}
+
+		// Check permission for all instances so that we apply the state change to all or none.
+		if !userHasPermission(entity.InstanceURL(inst.Project().Name, inst.Name())) {
+			return response.Forbidden(nil)
+		}
+	}
 
 	var names []string
 	var instances []instance.Instance
@@ -100,27 +121,27 @@ func instancesPut(d *Daemon, r *http.Request) response.Response {
 		}
 
 		switch action {
-		case shared.Freeze:
+		case instancetype.Freeze:
 			if !inst.IsRunning() {
 				continue
 			}
 
-		case shared.Restart:
+		case instancetype.Restart:
 			if !inst.IsRunning() {
 				continue
 			}
 
-		case shared.Start:
+		case instancetype.Start:
 			if inst.IsRunning() {
 				continue
 			}
 
-		case shared.Stop:
+		case instancetype.Stop:
 			if !inst.IsRunning() {
 				continue
 			}
 
-		case shared.Unfreeze:
+		case instancetype.Unfreeze:
 			if inst.IsRunning() {
 				continue
 			}
@@ -167,20 +188,14 @@ func instancesPut(d *Daemon, r *http.Request) response.Response {
 			return localAction(false)
 		}
 
-		// Check if clustered.
-		clustered, err := cluster.Enabled(d.db.Node)
-		if err != nil {
-			return err
-		}
-
 		// If not clustered, return the local data.
-		if !clustered {
+		if !s.ServerClustered {
 			return localAction(true)
 		}
 
 		// Get all members in cluster.
 		var members []db.NodeInfo
-		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			var err error
 
 			members, err = tx.GetNodes(ctx)
@@ -195,14 +210,14 @@ func instancesPut(d *Daemon, r *http.Request) response.Response {
 		}
 
 		// Get local cluster address.
-		localClusterAddress := d.State().LocalConfig.ClusterAddress()
+		localClusterAddress := s.LocalConfig.ClusterAddress()
 
 		// Record the results.
 		failures := map[string]error{}
 		failuresLock := sync.Mutex{}
 		wgAction := sync.WaitGroup{}
 
-		networkCert := d.endpoints.NetworkCert()
+		networkCert := s.Endpoints.NetworkCert()
 		for _, member := range members {
 			wgAction.Add(1)
 			go func(member db.NodeInfo) {
@@ -221,7 +236,7 @@ func instancesPut(d *Daemon, r *http.Request) response.Response {
 				}
 
 				// Connect to the remote server.
-				client, err := cluster.Connect(member.Address, networkCert, d.serverCert(), r, true)
+				client, err := cluster.Connect(member.Address, networkCert, s.ServerCert(), r, true)
 				if err != nil {
 					failuresLock.Lock()
 					failures[member.Name] = err
@@ -254,9 +269,12 @@ func instancesPut(d *Daemon, r *http.Request) response.Response {
 		return coalesceErrors(true, failures)
 	}
 
-	resources := map[string][]string{}
-	resources["instances"] = names
-	op, err := operations.OperationCreate(d.State(), projectName, operations.OperationClassTask, opType, resources, nil, do, nil, nil, r)
+	resources := map[string][]api.URL{}
+	for _, instName := range names {
+		resources["instances"] = append(resources["instances"], *api.NewURL().Path(version.APIVersion, "instances", instName))
+	}
+
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, opType, resources, nil, do, nil, nil, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
