@@ -6,15 +6,15 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db/operationtype"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/task"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db/operationtype"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/task"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // This task function expires logs when executed. It's started by the Daemon
@@ -27,17 +27,23 @@ func expireLogsTask(state *state.State) (task.Func, task.Schedule) {
 
 		op, err := operations.OperationCreate(state, "", operations.OperationClassTask, operationtype.LogsExpire, nil, nil, opRun, nil, nil, nil)
 		if err != nil {
-			logger.Error("Failed to start log expiry operation", logger.Ctx{"err": err})
+			logger.Error("Failed creating log files expiry operation", logger.Ctx{"err": err})
 			return
 		}
 
 		logger.Info("Expiring log files")
 		err = op.Start()
 		if err != nil {
-			logger.Error("Failed to expire logs", logger.Ctx{"err": err})
+			logger.Error("Failed starting log files expiry operation", logger.Ctx{"err": err})
+			return
+		}
+
+		err = op.Wait(ctx)
+		if err != nil {
+			logger.Error("Failed expiring log files", logger.Ctx{"err": err})
+			return
 		}
 
-		_, _ = op.Wait(ctx)
 		logger.Info("Done expiring log files")
 	}
 
@@ -106,7 +112,7 @@ func expireLogs(ctx context.Context, state *state.State) error {
 		}
 
 		// Check if the instance still exists.
-		if shared.StringInSlice(fi.Name(), names) {
+		if shared.ValueInSlice(fi.Name(), names) {
 			instDirEntries, err := os.ReadDir(shared.LogPath(fi.Name()))
 			if err != nil {
 				return err
