@@ -1,16 +1,5 @@
 package main
 
-import (
-	"os"
-	"runtime"
-
-	"golang.org/x/sys/unix"
-
-	_ "github.com/lxc/lxd/lxd/include" // Used by cgo
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
-)
-
 /*
 #include "config.h"
 
@@ -589,6 +578,11 @@ static bool kernel_supports_idmapped_mounts(void)
 */
 import "C"
 
+import (
+	_ "github.com/canonical/lxd/lxd/include" // Used by cgo
+	"github.com/canonical/lxd/shared/logger"
+)
+
 func canUseNetnsGetifaddrs() bool {
 	if !bool(C.is_empty_string(&C.errbuf[0])) {
 		logger.Debugf("%s", C.GoString(&C.errbuf[0]))
@@ -617,41 +611,6 @@ func canUsePidFds() bool {
 	return bool(C.pidfd_aware)
 }
 
-func canUseShiftfs() bool {
-	runtime.LockOSThread()
-	defer runtime.UnlockOSThread()
-
-	hostMntNs, err := os.Open("/proc/self/ns/mnt")
-	if err != nil {
-		logger.Debugf("%s - Failed to open host mount namespace", err)
-		return false
-	}
-
-	defer func() { _ = hostMntNs.Close() }()
-
-	err = unix.Unshare(unix.CLONE_NEWNS)
-	if err != nil {
-		logger.Debugf("%s - Failed to unshare mount namespace", err)
-		return false
-	}
-
-	defer func() {
-		err = unix.Setns(int(hostMntNs.Fd()), unix.CLONE_NEWNS)
-		if err != nil {
-			logger.Debugf("%s - Failed to reattach to host mount namespace", err)
-		}
-	}()
-
-	err = unix.Mount("/", "/", "", unix.MS_REC|unix.MS_PRIVATE, "")
-	if err != nil {
-		logger.Debugf("%s - Failed to turn \"/\" into private mount", err)
-		return false
-	}
-
-	err = unix.Mount(shared.VarPath(), shared.VarPath(), "shiftfs", 0, "mark")
-	return err == nil
-}
-
 // We're only using this during daemon startup to give an indication whether
 // the underlying kernel has the necessary infrastructure to support idmapped
 // mounts. This check does not give any indication whether the relevant
