@@ -14,15 +14,15 @@ import (
 	"golang.org/x/sys/unix"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/utils"
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/node"
-	"github.com/lxc/lxd/lxd/sys"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/termios"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/cluster"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/node"
+	"github.com/canonical/lxd/lxd/sys"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/termios"
 )
 
 type cmdCluster struct {
@@ -323,7 +323,7 @@ func (c *cmdClusterShow) Run(cmd *cobra.Command, args []string) error {
 	if len(config.Members) > 0 {
 		fmt.Printf(SegmentComment+"\n\n%s", segmentID, data)
 	} else {
-		fmt.Printf("%s", data)
+		fmt.Print(data)
 	}
 
 	return nil
@@ -363,7 +363,7 @@ func (c *cmdClusterListDatabase) Run(cmd *cobra.Command, args []string) error {
 		data[i] = []string{address}
 	}
 
-	_ = utils.RenderTable(utils.TableFormatTable, columns, data, nil)
+	_ = cli.RenderTable(cli.TableFormatTable, columns, data, nil)
 
 	return nil
 }
@@ -412,7 +412,7 @@ func (c *cmdClusterRecoverFromQuorumLoss) Run(cmd *cobra.Command, args []string)
 
 func (c *cmdClusterRecoverFromQuorumLoss) promptConfirmation() error {
 	reader := bufio.NewReader(os.Stdin)
-	fmt.Printf(`You should run this command only if you are *absolutely* certain that this is
+	fmt.Print(`You should run this command only if you are *absolutely* certain that this is
 the only database node left in your cluster AND that other database nodes will
 never come back (i.e. their LXD daemon won't ever be started again).
 
@@ -425,14 +425,14 @@ database, so you can possibly inspect it for further recovery.
 You'll be able to permanently delete from the database all information about
 former cluster members by running "lxc cluster remove <member-name> --force".
 
-See https://linuxcontainers.org/lxd/docs/master/clustering#recover-from-quorum-loss for more
+See https://documentation.ubuntu.com/lxd/en/latest/howto/cluster_recover/#recover-from-quorum-loss for more
 info.
 
 Do you want to proceed? (yes/no): `)
 	input, _ := reader.ReadString('\n')
 	input = strings.TrimSuffix(input, "\n")
 
-	if !shared.StringInSlice(strings.ToLower(input), []string{"yes"}) {
+	if !shared.ValueInSlice(strings.ToLower(input), []string{"yes"}) {
 		return fmt.Errorf("Recover operation aborted")
 	}
 
@@ -488,7 +488,7 @@ func (c *cmdClusterRemoveRaftNode) Run(cmd *cobra.Command, args []string) error
 
 func (c *cmdClusterRemoveRaftNode) promptConfirmation() error {
 	reader := bufio.NewReader(os.Stdin)
-	fmt.Printf(`You should run this command only if you ended up in an
+	fmt.Print(`You should run this command only if you ended up in an
 inconsistent state where a node has been uncleanly removed (i.e. it doesn't show
 up in "lxc cluster list" but it's still in the raft configuration).
 
@@ -496,7 +496,7 @@ Do you want to proceed? (yes/no): `)
 	input, _ := reader.ReadString('\n')
 	input = strings.TrimSuffix(input, "\n")
 
-	if !shared.StringInSlice(strings.ToLower(input), []string{"yes"}) {
+	if !shared.ValueInSlice(strings.ToLower(input), []string{"yes"}) {
 		return fmt.Errorf("Remove raft node operation aborted")
 	}
 
