@@ -8,7 +8,8 @@ import (
 	"github.com/spf13/cobra"
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/shared"
+	"github.com/canonical/lxd/lxd/linux"
+	"github.com/canonical/lxd/shared"
 )
 
 type cmdForkstart struct {
@@ -53,7 +54,7 @@ func (c *cmdForkstart) Run(cmd *cobra.Command, args []string) error {
 	lxcpath := args[1]
 	configPath := args[2]
 
-	err := shared.CloseRange(uint32(os.Stderr.Fd())+1, ^uint32(0), shared.CLOSE_RANGE_CLOEXEC)
+	err := linux.CloseRange(uint32(os.Stderr.Fd())+1, ^uint32(0), linux.CLOSE_RANGE_CLOEXEC)
 	if err != nil {
 		return fmt.Errorf("Aborting attach to prevent leaking file descriptors into container")
 	}
