@@ -13,18 +13,17 @@ import (
 	"golang.org/x/sys/unix"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/network"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/validate"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/cluster"
+	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/network"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/validate"
+	"github.com/canonical/lxd/shared/version"
 )
 
 func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.InstanceServer, server *api.Server) (*api.InitPreseed, error) {
@@ -77,7 +76,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 	}
 
 	// Print the YAML
-	preSeedPrint, err := cli.AskBool("Would you like a YAML \"lxd init\" preseed to be printed? (yes/no) [default=no]: ", "no")
+	preSeedPrint, err := c.global.asker.AskBool("Would you like a YAML \"lxd init\" preseed to be printed? (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return nil, err
 	}
@@ -107,7 +106,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 }
 
 func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, server *api.Server) error {
-	clustering, err := cli.AskBool("Would you like to use LXD clustering? (yes/no) [default=no]: ", "no")
+	clustering, err := c.global.asker.AskBool("Would you like to use LXD clustering? (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return err
 	}
@@ -117,7 +116,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 		config.Cluster.Enabled = true
 
 		askForServerName := func() error {
-			config.Cluster.ServerName, err = cli.AskString(fmt.Sprintf("What member name should be used to identify this server in the cluster? [default=%s]: ", c.defaultHostname()), c.defaultHostname(), nil)
+			config.Cluster.ServerName, err = c.global.asker.AskString(fmt.Sprintf("What member name should be used to identify this server in the cluster? [default=%s]: ", c.defaultHostname()), c.defaultHostname(), nil)
 			if err != nil {
 				return err
 			}
@@ -131,7 +130,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			address := util.CanonicalNetworkAddress(value, shared.HTTPSDefaultPort)
 
 			host, _, _ := net.SplitHostPort(address)
-			if shared.StringInSlice(host, []string{"", "[::]", "0.0.0.0"}) {
+			if shared.ValueInSlice(host, []string{"", "[::]", "0.0.0.0"}) {
 				return fmt.Errorf("Invalid IP address or DNS name")
 			}
 
@@ -151,7 +150,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			return nil
 		}
 
-		serverAddress, err := cli.AskString(fmt.Sprintf("What IP address or DNS name should be used to reach this server? [default=%s]: ", address), address, validateServerAddress)
+		serverAddress, err := c.global.asker.AskString(fmt.Sprintf("What IP address or DNS name should be used to reach this server? [default=%s]: ", address), address, validateServerAddress)
 		if err != nil {
 			return err
 		}
@@ -159,7 +158,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 		serverAddress = util.CanonicalNetworkAddress(serverAddress, shared.HTTPSDefaultPort)
 		config.Node.Config["core.https_address"] = serverAddress
 
-		clusterJoin, err := cli.AskBool("Are you joining an existing cluster? (yes/no) [default=no]: ", "no")
+		clusterJoin, err := c.global.asker.AskBool("Are you joining an existing cluster? (yes/no) [default=no]: ", "no")
 		if err != nil {
 			return err
 		}
@@ -186,9 +185,9 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			}
 
 			validInput := func(input string) error {
-				if shared.StringInSlice(strings.ToLower(input), []string{"yes", "y"}) {
+				if shared.ValueInSlice(strings.ToLower(input), []string{"yes", "y"}) {
 					return nil
-				} else if shared.StringInSlice(strings.ToLower(input), []string{"no", "n"}) {
+				} else if shared.ValueInSlice(strings.ToLower(input), []string{"no", "n"}) {
 					return nil
 				} else if validJoinToken(input) != nil {
 					return fmt.Errorf("Not yes/no, or invalid join token")
@@ -197,14 +196,14 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 				return nil
 			}
 
-			clusterJoinToken, err := cli.AskString("Do you have a join token? (yes/no/[token]) [default=no]: ", "no", validInput)
+			clusterJoinToken, err := c.global.asker.AskString("Do you have a join token? (yes/no/[token]) [default=no]: ", "no", validInput)
 			if err != nil {
 				return err
 			}
 
-			if !shared.StringInSlice(strings.ToLower(clusterJoinToken), []string{"no", "n"}) {
-				if shared.StringInSlice(strings.ToLower(clusterJoinToken), []string{"yes", "y"}) {
-					clusterJoinToken, err = cli.AskString("Please provide join token: ", "", validJoinToken)
+			if !shared.ValueInSlice(strings.ToLower(clusterJoinToken), []string{"no", "n"}) {
+				if shared.ValueInSlice(strings.ToLower(clusterJoinToken), []string{"yes", "y"}) {
+					clusterJoinToken, err = c.global.asker.AskString("Please provide join token: ", "", validJoinToken)
 					if err != nil {
 						return err
 					}
@@ -250,7 +249,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 
 				for {
 					// Cluster URL
-					clusterAddress, err := cli.AskString("IP address or FQDN of an existing cluster member (may include port): ", "", nil)
+					clusterAddress, err := c.global.asker.AskString("IP address or FQDN of an existing cluster member (may include port): ", "", nil)
 					if err != nil {
 						return err
 					}
@@ -265,27 +264,27 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 					}
 
 					certDigest := shared.CertFingerprint(cert)
-					fmt.Printf("Cluster fingerprint: %s\n", certDigest)
-					fmt.Printf("You can validate this fingerprint by running \"lxc info\" locally on an existing cluster member.\n")
+					fmt.Println("Cluster fingerprint:", certDigest)
+					fmt.Println("You can validate this fingerprint by running \"lxc info\" locally on an existing cluster member.")
 
 					validator := func(input string) error {
 						if input == certDigest {
 							return nil
-						} else if shared.StringInSlice(strings.ToLower(input), []string{"yes", "y"}) {
+						} else if shared.ValueInSlice(strings.ToLower(input), []string{"yes", "y"}) {
 							return nil
-						} else if shared.StringInSlice(strings.ToLower(input), []string{"no", "n"}) {
+						} else if shared.ValueInSlice(strings.ToLower(input), []string{"no", "n"}) {
 							return nil
 						}
 
 						return fmt.Errorf("Not yes/no or fingerprint")
 					}
 
-					fingerprintCorrect, err := cli.AskString("Is this the correct fingerprint? (yes/no/[fingerprint]) [default=no]: ", "no", validator)
+					fingerprintCorrect, err := c.global.asker.AskString("Is this the correct fingerprint? (yes/no/[fingerprint]) [default=no]: ", "no", validator)
 					if err != nil {
 						return err
 					}
 
-					if shared.StringInSlice(strings.ToLower(fingerprintCorrect), []string{"no", "n"}) {
+					if shared.ValueInSlice(strings.ToLower(fingerprintCorrect), []string{"no", "n"}) {
 						return fmt.Errorf("User aborted configuration")
 					}
 
@@ -298,7 +297,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			}
 
 			// Confirm wiping
-			clusterWipeMember, err := cli.AskBool("All existing data is lost when joining a cluster, continue? (yes/no) [default=no] ", "no")
+			clusterWipeMember, err := c.global.asker.AskBool("All existing data is lost when joining a cluster, continue? (yes/no) [default=no] ", "no")
 			if err != nil {
 				return err
 			}
@@ -345,7 +344,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 				question := fmt.Sprintf("Choose %s: ", config.Description)
 
 				// Allow for empty values.
-				configValue, err := cli.AskString(question, "", validate.Optional())
+				configValue, err := c.global.asker.AskString(question, "", validate.Optional())
 				if err != nil {
 					return err
 				}
@@ -360,16 +359,6 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 			if err != nil {
 				return err
 			}
-
-			// Password authentication
-			clusterUsePassword, err := cli.AskBool("Setup password authentication on the cluster? (yes/no) [default=no]: ", "no")
-			if err != nil {
-				return err
-			}
-
-			if clusterUsePassword {
-				config.Node.Config["core.trust_password"] = cli.AskPassword("Trust password for new clients: ")
-			}
 		}
 	}
 
@@ -377,7 +366,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, s
 }
 
 func (c *cmdInit) askMAAS(config *api.InitPreseed, d lxd.InstanceServer) error {
-	maas, err := cli.AskBool("Would you like to connect to a MAAS server? (yes/no) [default=no]: ", "no")
+	maas, err := c.global.asker.AskBool("Would you like to connect to a MAAS server? (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return err
 	}
@@ -386,7 +375,7 @@ func (c *cmdInit) askMAAS(config *api.InitPreseed, d lxd.InstanceServer) error {
 		return nil
 	}
 
-	maasHostname, err := cli.AskString(fmt.Sprintf("What's the name of this host in MAAS? [default=%s]: ", c.defaultHostname()), c.defaultHostname(), nil)
+	maasHostname, err := c.global.asker.AskString(fmt.Sprintf("What's the name of this host in MAAS? [default=%s]: ", c.defaultHostname()), c.defaultHostname(), nil)
 	if err != nil {
 		return err
 	}
@@ -395,12 +384,12 @@ func (c *cmdInit) askMAAS(config *api.InitPreseed, d lxd.InstanceServer) error {
 		config.Node.Config["maas.machine"] = maasHostname
 	}
 
-	config.Node.Config["maas.api.url"], err = cli.AskString("URL of your MAAS server (e.g. http://1.2.3.4:5240/MAAS): ", "", nil)
+	config.Node.Config["maas.api.url"], err = c.global.asker.AskString("URL of your MAAS server (e.g. http://1.2.3.4:5240/MAAS): ", "", nil)
 	if err != nil {
 		return err
 	}
 
-	config.Node.Config["maas.api.key"], err = cli.AskString("API key for your MAAS server: ", "", nil)
+	config.Node.Config["maas.api.key"], err = c.global.asker.AskString("API key for your MAAS server: ", "", nil)
 	if err != nil {
 		return err
 	}
@@ -413,7 +402,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 	localBridgeCreate := false
 
 	if config.Cluster == nil {
-		localBridgeCreate, err = cli.AskBool("Would you like to create a new local network bridge? (yes/no) [default=yes]: ", "yes")
+		localBridgeCreate, err = c.global.asker.AskBool("Would you like to create a new local network bridge? (yes/no) [default=yes]: ", "yes")
 		if err != nil {
 			return err
 		}
@@ -429,14 +418,14 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 			}
 		}
 
-		useExistingInterface, err := cli.AskBool("Would you like to configure LXD to use an existing bridge or host interface? (yes/no) [default=no]: ", "no")
+		useExistingInterface, err := c.global.asker.AskBool("Would you like to configure LXD to use an existing bridge or host interface? (yes/no) [default=no]: ", "no")
 		if err != nil {
 			return err
 		}
 
 		if useExistingInterface {
 			for {
-				interfaceName, err := cli.AskString("Name of the existing bridge or host interface: ", "", nil)
+				interfaceName, err := c.global.asker.AskString("Name of the existing bridge or host interface: ", "", nil)
 				if err != nil {
 					return err
 				}
@@ -459,13 +448,13 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 				}
 
 				if config.Node.Config["maas.api.url"] != nil {
-					maasConnect, err := cli.AskBool("Is this interface connected to your MAAS server? (yes/no) [default=yes]: ", "yes")
+					maasConnect, err := c.global.asker.AskBool("Is this interface connected to your MAAS server? (yes/no) [default=yes]: ", "yes")
 					if err != nil {
 						return err
 					}
 
 					if maasConnect {
-						maasSubnetV4, err := cli.AskString("MAAS IPv4 subnet name for this interface (empty for no subnet): ", "", validate.Optional())
+						maasSubnetV4, err := c.global.asker.AskString("MAAS IPv4 subnet name for this interface (empty for no subnet): ", "", validate.Optional())
 						if err != nil {
 							return err
 						}
@@ -474,7 +463,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 							config.Node.Profiles[0].Devices["eth0"]["maas.subnet.ipv4"] = maasSubnetV4
 						}
 
-						maasSubnetV6, err := cli.AskString("MAAS IPv6 subnet name for this interface (empty for no subnet): ", "", validate.Optional())
+						maasSubnetV6, err := c.global.asker.AskString("MAAS IPv6 subnet name for this interface (empty for no subnet): ", "", validate.Optional())
 						if err != nil {
 							return err
 						}
@@ -488,7 +477,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 				break
 			}
 		} else if config.Cluster != nil && fanKernel {
-			fan, err := cli.AskBool("Would you like to create a new Fan overlay network? (yes/no) [default=yes]: ", "yes")
+			fan, err := c.global.asker.AskBool("Would you like to create a new Fan overlay network? (yes/no) [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
@@ -497,13 +486,13 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 				// Define the network
 				networkPost := api.InitNetworksProjectPost{}
 				networkPost.Name = "lxdfan0"
-				networkPost.Project = project.Default
+				networkPost.Project = api.ProjectDefaultName
 				networkPost.Config = map[string]string{
 					"bridge.mode": "fan",
 				}
 
 				// Select the underlay
-				networkPost.Config["fan.underlay_subnet"], err = cli.AskString("What subnet should be used as the Fan underlay? [default=auto]: ", "auto", func(value string) error {
+				networkPost.Config["fan.underlay_subnet"], err = c.global.asker.AskString("What subnet should be used as the Fan underlay? [default=auto]: ", "auto", func(value string) error {
 					var err error
 					var subnet *net.IPNet
 
@@ -554,10 +543,10 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 		// Define the network
 		net := api.InitNetworksProjectPost{}
 		net.Config = map[string]string{}
-		net.Project = project.Default
+		net.Project = api.ProjectDefaultName
 
 		// Network name
-		net.Name, err = cli.AskString("What should the new bridge be called? [default=lxdbr0]: ", "lxdbr0", func(netName string) error {
+		net.Name, err = c.global.asker.AskString("What should the new bridge be called? [default=lxdbr0]: ", "lxdbr0", func(netName string) error {
 			netType, err := network.LoadByType("bridge")
 			if err != nil {
 				return err
@@ -583,8 +572,8 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 		}
 
 		// IPv4
-		net.Config["ipv4.address"], err = cli.AskString("What IPv4 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
-			if shared.StringInSlice(value, []string{"auto", "none"}) {
+		net.Config["ipv4.address"], err = c.global.asker.AskString("What IPv4 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
+			if shared.ValueInSlice(value, []string{"auto", "none"}) {
 				return nil
 			}
 
@@ -594,8 +583,8 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 			return err
 		}
 
-		if !shared.StringInSlice(net.Config["ipv4.address"], []string{"auto", "none"}) {
-			netIPv4UseNAT, err := cli.AskBool("Would you like LXD to NAT IPv4 traffic on your bridge? [default=yes]: ", "yes")
+		if !shared.ValueInSlice(net.Config["ipv4.address"], []string{"auto", "none"}) {
+			netIPv4UseNAT, err := c.global.asker.AskBool("Would you like LXD to NAT IPv4 traffic on your bridge? [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
@@ -604,8 +593,8 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 		}
 
 		// IPv6
-		net.Config["ipv6.address"], err = cli.AskString("What IPv6 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
-			if shared.StringInSlice(value, []string{"auto", "none"}) {
+		net.Config["ipv6.address"], err = c.global.asker.AskString("What IPv6 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
+			if shared.ValueInSlice(value, []string{"auto", "none"}) {
 				return nil
 			}
 
@@ -615,8 +604,8 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 			return err
 		}
 
-		if !shared.StringInSlice(net.Config["ipv6.address"], []string{"auto", "none"}) {
-			netIPv6UseNAT, err := cli.AskBool("Would you like LXD to NAT IPv6 traffic on your bridge? [default=yes]: ", "yes")
+		if !shared.ValueInSlice(net.Config["ipv6.address"], []string{"auto", "none"}) {
+			netIPv6UseNAT, err := c.global.asker.AskBool("Would you like LXD to NAT IPv6 traffic on your bridge? [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
@@ -634,7 +623,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 
 func (c *cmdInit) askStorage(config *api.InitPreseed, d lxd.InstanceServer, server *api.Server) error {
 	if config.Cluster != nil {
-		localStoragePool, err := cli.AskBool("Do you want to configure a new local storage pool? (yes/no) [default=yes]: ", "yes")
+		localStoragePool, err := c.global.asker.AskBool("Do you want to configure a new local storage pool? (yes/no) [default=yes]: ", "yes")
 		if err != nil {
 			return err
 		}
@@ -646,7 +635,7 @@ func (c *cmdInit) askStorage(config *api.InitPreseed, d lxd.InstanceServer, serv
 			}
 		}
 
-		remoteStoragePool, err := cli.AskBool("Do you want to configure a new remote storage pool? (yes/no) [default=no]: ", "no")
+		remoteStoragePool, err := c.global.asker.AskBool("Do you want to configure a new remote storage pool? (yes/no) [default=no]: ", "no")
 		if err != nil {
 			return err
 		}
@@ -661,7 +650,7 @@ func (c *cmdInit) askStorage(config *api.InitPreseed, d lxd.InstanceServer, serv
 		return nil
 	}
 
-	storagePool, err := cli.AskBool("Do you want to configure a new storage pool? (yes/no) [default=yes]: ", "yes")
+	storagePool, err := c.global.asker.AskBool("Do you want to configure a new storage pool? (yes/no) [default=yes]: ", "yes")
 	if err != nil {
 		return err
 	}
@@ -691,11 +680,11 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 	}
 
 	defaultStorage := "dir"
-	if backingFs == "btrfs" && shared.StringInSlice("btrfs", availableBackends) {
+	if backingFs == "btrfs" && shared.ValueInSlice("btrfs", availableBackends) {
 		defaultStorage = "btrfs"
-	} else if shared.StringInSlice("zfs", availableBackends) {
+	} else if shared.ValueInSlice("zfs", availableBackends) {
 		defaultStorage = "zfs"
-	} else if shared.StringInSlice("btrfs", availableBackends) {
+	} else if shared.ValueInSlice("btrfs", availableBackends) {
 		defaultStorage = "btrfs"
 	}
 
@@ -705,7 +694,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		pool.Config = map[string]string{}
 
 		if poolType == util.PoolTypeAny {
-			pool.Name, err = cli.AskString("Name of the new storage pool [default=default]: ", "default", nil)
+			pool.Name, err = c.global.asker.AskString("Name of the new storage pool [default=default]: ", "default", nil)
 			if err != nil {
 				return err
 			}
@@ -736,14 +725,14 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		if len(availableBackends) > 1 {
 			defaultBackend := defaultStorage
 			if poolType == util.PoolTypeRemote {
-				if shared.StringInSlice("ceph", availableBackends) {
+				if shared.ValueInSlice("ceph", availableBackends) {
 					defaultBackend = "ceph"
 				} else {
 					defaultBackend = availableBackends[0] // Default to first remote driver.
 				}
 			}
 
-			pool.Driver, err = cli.AskChoice(fmt.Sprintf("Name of the storage backend to use (%s) [default=%s]: ", strings.Join(availableBackends, ", "), defaultBackend), availableBackends, defaultBackend)
+			pool.Driver, err = c.global.asker.AskChoice(fmt.Sprintf("Name of the storage backend to use (%s) [default=%s]: ", strings.Join(availableBackends, ", "), defaultBackend), availableBackends, defaultBackend)
 			if err != nil {
 				return err
 			}
@@ -759,7 +748,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 
 		// Optimization for btrfs on btrfs
 		if pool.Driver == "btrfs" && backingFs == "btrfs" {
-			btrfsSubvolume, err := cli.AskBool(fmt.Sprintf("Would you like to create a new btrfs subvolume under %s? (yes/no) [default=yes]: ", shared.VarPath("")), "yes")
+			btrfsSubvolume, err := c.global.asker.AskBool(fmt.Sprintf("Would you like to create a new btrfs subvolume under %s? (yes/no) [default=yes]: ", shared.VarPath("")), "yes")
 			if err != nil {
 				return err
 			}
@@ -775,7 +764,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		if pool.Driver == "zfs" && backingFs == "zfs" {
 			poolName, _ := shared.RunCommand("zpool", "get", "-H", "-o", "value", "name", "rpool")
 			if strings.TrimSpace(poolName) == "rpool" {
-				zfsDataset, err := cli.AskBool("Would you like to create a new zfs dataset under rpool/lxd? (yes/no) [default=yes]: ", "yes")
+				zfsDataset, err := c.global.asker.AskBool("Would you like to create a new zfs dataset under rpool/lxd? (yes/no) [default=yes]: ", "yes")
 				if err != nil {
 					return err
 				}
@@ -788,7 +777,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 			}
 		}
 
-		poolCreate, err := cli.AskBool(fmt.Sprintf("Create a new %s pool? (yes/no) [default=yes]: ", strings.ToUpper(pool.Driver)), "yes")
+		poolCreate, err := c.global.asker.AskBool(fmt.Sprintf("Create a new %s pool? (yes/no) [default=yes]: ", strings.ToUpper(pool.Driver)), "yes")
 		if err != nil {
 			return err
 		}
@@ -796,42 +785,42 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		if poolCreate {
 			if pool.Driver == "ceph" {
 				// Ask for the name of the cluster
-				pool.Config["ceph.cluster_name"], err = cli.AskString("Name of the existing CEPH cluster [default=ceph]: ", "ceph", nil)
+				pool.Config["ceph.cluster_name"], err = c.global.asker.AskString("Name of the existing CEPH cluster [default=ceph]: ", "ceph", nil)
 				if err != nil {
 					return err
 				}
 
 				// Ask for the name of the osd pool
-				pool.Config["ceph.osd.pool_name"], err = cli.AskString("Name of the OSD storage pool [default=lxd]: ", "lxd", nil)
+				pool.Config["ceph.osd.pool_name"], err = c.global.asker.AskString("Name of the OSD storage pool [default=lxd]: ", "lxd", nil)
 				if err != nil {
 					return err
 				}
 
 				// Ask for the number of placement groups
-				pool.Config["ceph.osd.pg_num"], err = cli.AskString("Number of placement groups [default=32]: ", "32", nil)
+				pool.Config["ceph.osd.pg_num"], err = c.global.asker.AskString("Number of placement groups [default=32]: ", "32", nil)
 				if err != nil {
 					return err
 				}
 			} else if pool.Driver == "cephfs" {
 				// Ask for the name of the cluster
-				pool.Config["cephfs.cluster_name"], err = cli.AskString("Name of the existing CEPHfs cluster [default=ceph]: ", "ceph", nil)
+				pool.Config["cephfs.cluster_name"], err = c.global.asker.AskString("Name of the existing CEPHfs cluster [default=ceph]: ", "ceph", nil)
 				if err != nil {
 					return err
 				}
 
 				// Ask for the name of the cluster
-				pool.Config["source"], err = cli.AskString("Name of the CEPHfs volume: ", "", nil)
+				pool.Config["source"], err = c.global.asker.AskString("Name of the CEPHfs volume: ", "", nil)
 				if err != nil {
 					return err
 				}
 			} else {
-				useEmptyBlockDev, err := cli.AskBool("Would you like to use an existing empty block device (e.g. a disk or partition)? (yes/no) [default=no]: ", "no")
+				useEmptyBlockDev, err := c.global.asker.AskBool("Would you like to use an existing empty block device (e.g. a disk or partition)? (yes/no) [default=no]: ", "no")
 				if err != nil {
 					return err
 				}
 
 				if useEmptyBlockDev {
-					pool.Config["source"], err = cli.AskString("Path to the existing block device: ", "", func(path string) error {
+					pool.Config["source"], err = c.global.asker.AskString("Path to the existing block device: ", "", func(path string) error {
 						if !shared.IsBlockdevPath(path) {
 							return fmt.Errorf("%q is not a block device", path)
 						}
@@ -858,7 +847,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 						defaultSize = 5
 					}
 
-					pool.Config["size"], err = cli.AskString(
+					pool.Config["size"], err = c.global.asker.AskString(
 						fmt.Sprintf("Size in GiB of the new loop device (1GiB minimum) [default=%dGiB]: ", defaultSize),
 						fmt.Sprintf("%dGiB", defaultSize),
 						func(input string) error {
@@ -888,13 +877,13 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		} else {
 			if pool.Driver == "ceph" {
 				// ask for the name of the cluster
-				pool.Config["ceph.cluster_name"], err = cli.AskString("Name of the existing CEPH cluster [default=ceph]: ", "ceph", nil)
+				pool.Config["ceph.cluster_name"], err = c.global.asker.AskString("Name of the existing CEPH cluster [default=ceph]: ", "ceph", nil)
 				if err != nil {
 					return err
 				}
 
 				// ask for the name of the existing pool
-				pool.Config["source"], err = cli.AskString("Name of the existing OSD storage pool [default=lxd]: ", "lxd", nil)
+				pool.Config["source"], err = c.global.asker.AskString("Name of the existing OSD storage pool [default=lxd]: ", "lxd", nil)
 				if err != nil {
 					return err
 				}
@@ -902,7 +891,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 				pool.Config["ceph.osd.pool_name"] = pool.Config["source"]
 			} else {
 				question := fmt.Sprintf("Name of the existing %s pool or dataset: ", strings.ToUpper(pool.Driver))
-				pool.Config["source"], err = cli.AskString(question, "", nil)
+				pool.Config["source"], err = c.global.asker.AskString(question, "", nil)
 				if err != nil {
 					return err
 				}
@@ -912,17 +901,16 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		if pool.Driver == "lvm" {
 			_, err := exec.LookPath("thin_check")
 			if err != nil {
-				fmt.Printf(`
-The LVM thin provisioning tools couldn't be found. LVM can still be used
-without thin provisioning but this will disable over-provisioning,
-increase the space requirements and creation time of images, containers
-and snapshots.
+				fmt.Print(`
+The LVM thin provisioning tools couldn't be found.
+LVM can still be used without thin provisioning but this will disable over-provisioning,
+increase the space requirements and creation time of images, instances and snapshots.
 
-If you wish to use thin provisioning, abort now, install the tools from
-your Linux distribution and run "lxd init" again afterwards.
+If you wish to use thin provisioning, abort now, install the tools from your Linux distribution
+and make sure that your user can see and run the "thin_check" command before running "lxd init" again.
 
 `)
-				lvmContinueNoThin, err := cli.AskBool("Do you want to continue without thin provisioning? (yes/no) [default=yes]: ", "yes")
+				lvmContinueNoThin, err := c.global.asker.AskBool("Do you want to continue without thin provisioning? (yes/no) [default=yes]: ", "yes")
 				if err != nil {
 					return err
 				}
@@ -946,7 +934,7 @@ func (c *cmdInit) askDaemon(config *api.InitPreseed, d lxd.InstanceServer, serve
 	// Detect lack of uid/gid
 	idmapset, err := idmap.DefaultIdmapSet("", "")
 	if (err != nil || len(idmapset.Idmap) == 0 || idmapset.Usable() != nil) && shared.RunningInUserNS() {
-		fmt.Printf(`
+		fmt.Print(`
 We detected that you are running inside an unprivileged container.
 This means that unless you manually configured your host otherwise,
 you will not have enough uids and gids to allocate to your containers.
@@ -958,7 +946,7 @@ they otherwise would.
 
 `)
 
-		shareParentAllocation, err := cli.AskBool("Would you like to have your containers share their parent's allocation? (yes/no) [default=yes]: ", "yes")
+		shareParentAllocation, err := c.global.asker.AskBool("Would you like to have your containers share their parent's allocation? (yes/no) [default=yes]: ", "yes")
 		if err != nil {
 			return err
 		}
@@ -970,7 +958,7 @@ they otherwise would.
 
 	// Network listener
 	if config.Cluster == nil {
-		lxdOverNetwork, err := cli.AskBool("Would you like the LXD server to be available over the network? (yes/no) [default=no]: ", "no")
+		lxdOverNetwork, err := c.global.asker.AskBool("Would you like the LXD server to be available over the network? (yes/no) [default=no]: ", "no")
 		if err != nil {
 			return err
 		}
@@ -984,7 +972,7 @@ they otherwise would.
 				return nil
 			}
 
-			netAddr, err := cli.AskString("Address to bind LXD to (not including port) [default=all]: ", "all", isIPAddress)
+			netAddr, err := c.global.asker.AskString("Address to bind LXD to (not including port) [default=all]: ", "all", isIPAddress)
 			if err != nil {
 				return err
 			}
@@ -997,8 +985,8 @@ they otherwise would.
 				netAddr = fmt.Sprintf("[%s]", netAddr)
 			}
 
-			netPort, err := cli.AskInt(fmt.Sprintf("Port to bind LXD to [default=%d]: ", shared.HTTPSDefaultPort), 1, 65535, fmt.Sprintf("%d", shared.HTTPSDefaultPort), func(netPort int64) error {
-				address := util.CanonicalNetworkAddressFromAddressAndPort(netAddr, int(netPort), shared.HTTPSDefaultPort)
+			netPort, err := c.global.asker.AskInt(fmt.Sprintf("Port to bind LXD to [default=%d]: ", shared.HTTPSDefaultPort), 1, 65535, fmt.Sprintf("%d", shared.HTTPSDefaultPort), func(netPort int64) error {
+				address := util.CanonicalNetworkAddressFromAddressAndPort(netAddr, netPort, shared.HTTPSDefaultPort)
 
 				if err == nil {
 					if server.Config["cluster.https_address"] == address || server.Config["core.https_address"] == address {
@@ -1019,16 +1007,12 @@ they otherwise would.
 				return err
 			}
 
-			config.Node.Config["core.https_address"] = util.CanonicalNetworkAddressFromAddressAndPort(netAddr, int(netPort), shared.HTTPSDefaultPort)
-			config.Node.Config["core.trust_password"] = cli.AskPassword("Trust password for new clients: ")
-			if config.Node.Config["core.trust_password"] == "" {
-				fmt.Printf("No password set, client certificates will have to be manually trusted.")
-			}
+			config.Node.Config["core.https_address"] = util.CanonicalNetworkAddressFromAddressAndPort(netAddr, netPort, shared.HTTPSDefaultPort)
 		}
 	}
 
 	// Ask if the user wants images to be automatically refreshed
-	imageStaleRefresh, err := cli.AskBool("Would you like stale cached images to be updated automatically? (yes/no) [default=yes]: ", "yes")
+	imageStaleRefresh, err := c.global.asker.AskBool("Would you like stale cached images to be updated automatically? (yes/no) [default=yes]: ", "yes")
 	if err != nil {
 		return err
 	}
