@@ -4,19 +4,29 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // LoadByName loads and initialises a Network ACL from the database by project and name.
 func LoadByName(s *state.State, projectName string, name string) (NetworkACL, error) {
-	id, aclInfo, err := s.DB.Cluster.GetNetworkACL(projectName, name)
+	var id int64
+	var aclInfo *api.NetworkACL
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		id, aclInfo, err = tx.GetNetworkACL(ctx, projectName, name)
+
+		return err
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -42,8 +52,12 @@ func Create(s *state.State, projectName string, aclInfo *api.NetworkACLsPost) er
 		return err
 	}
 
-	// Insert DB record.
-	_, err = s.DB.Cluster.CreateNetworkACL(projectName, aclInfo)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Insert DB record.
+		_, err := tx.CreateNetworkACL(ctx, projectName, aclInfo)
+
+		return err
+	})
 	if err != nil {
 		return err
 	}
@@ -54,7 +68,15 @@ func Create(s *state.State, projectName string, aclInfo *api.NetworkACLsPost) er
 // Exists checks the ACL name(s) provided exists in the project.
 // If multiple names are provided, also checks that duplicate names aren't specified in the list.
 func Exists(s *state.State, projectName string, name ...string) error {
-	existingACLNames, err := s.DB.Cluster.GetNetworkACLs(projectName)
+	var existingACLNames []string
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		existingACLNames, err = tx.GetNetworkACLs(ctx, projectName)
+
+		return err
+	})
 	if err != nil {
 		return err
 	}
@@ -62,7 +84,7 @@ func Exists(s *state.State, projectName string, name ...string) error {
 	checkedACLNames := make(map[string]struct{}, len(name))
 
 	for _, aclName := range name {
-		if !shared.StringInSlice(aclName, existingACLNames) {
+		if !shared.ValueInSlice(aclName, existingACLNames) {
 			return fmt.Errorf("Network ACL %q does not exist", aclName)
 		}
 
@@ -79,44 +101,45 @@ func Exists(s *state.State, projectName string, name ...string) error {
 
 // UsedBy finds all networks, profiles and instance NICs that use any of the specified ACLs and executes usageFunc
 // once for each resource using one or more of the ACLs with info about the resource and matched ACLs being used.
-func UsedBy(s *state.State, aclProjectName string, usageFunc func(matchedACLNames []string, usageType any, nicName string, nicConfig map[string]string) error, matchACLNames ...string) error {
+func UsedBy(s *state.State, aclProjectName string, usageFunc func(ctx context.Context, tx *db.ClusterTx, matchedACLNames []string, usageType any, nicName string, nicConfig map[string]string) error, matchACLNames ...string) error {
 	if len(matchACLNames) <= 0 {
 		return nil
 	}
 
-	// Find networks using the ACLs. Cheapest to do.
-	networkNames, err := s.DB.Cluster.GetCreatedNetworks(aclProjectName)
-	if err != nil && !response.IsNotFoundError(err) {
-		return fmt.Errorf("Failed loading networks for project %q: %w", aclProjectName, err)
-	}
-
-	for _, networkName := range networkNames {
-		_, network, _, err := s.DB.Cluster.GetNetworkInAnyState(aclProjectName, networkName)
-		if err != nil {
-			return fmt.Errorf("Failed to get network config for %q: %w", networkName, err)
-		}
-
-		netACLNames := shared.SplitNTrimSpace(network.Config["security.acls"], ",", -1, true)
-		matchedACLNames := []string{}
-		for _, netACLName := range netACLNames {
-			if shared.StringInSlice(netACLName, matchACLNames) {
-				matchedACLNames = append(matchedACLNames, netACLName)
-			}
-		}
-
-		if len(matchedACLNames) > 0 {
-			// Call usageFunc with a list of matched ACLs and info about the network.
-			err := usageFunc(matchedACLNames, network, "", nil)
-			if err != nil {
-				return err
-			}
-		}
-	}
-
-	// Look for profiles. Next cheapest to do.
 	var profiles []cluster.Profile
 	profileDevices := map[string]map[string]cluster.Device{}
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Find networks using the ACLs. Cheapest to do.
+		networkNames, err := tx.GetCreatedNetworkNamesByProject(ctx, aclProjectName)
+		if err != nil && !response.IsNotFoundError(err) {
+			return fmt.Errorf("Failed loading networks for project %q: %w", aclProjectName, err)
+		}
+
+		for _, networkName := range networkNames {
+			_, network, _, err := tx.GetNetworkInAnyState(ctx, aclProjectName, networkName)
+			if err != nil {
+				return fmt.Errorf("Failed to get network config for %q: %w", networkName, err)
+			}
+
+			netACLNames := shared.SplitNTrimSpace(network.Config["security.acls"], ",", -1, true)
+			matchedACLNames := []string{}
+			for _, netACLName := range netACLNames {
+				if shared.ValueInSlice(netACLName, matchACLNames) {
+					matchedACLNames = append(matchedACLNames, netACLName)
+				}
+			}
+
+			if len(matchedACLNames) > 0 {
+				// Call usageFunc with a list of matched ACLs and info about the network.
+				err := usageFunc(ctx, tx, matchedACLNames, network, "", nil)
+				if err != nil {
+					return err
+				}
+			}
+		}
+
+		// Look for profiles. Next cheapest to do.
 		profiles, err = cluster.GetProfiles(ctx, tx.Tx())
 		if err != nil {
 			return err
@@ -152,7 +175,9 @@ func UsedBy(s *state.State, aclProjectName string, usageFunc func(matchedACLName
 			matchedACLNames := isInUseByDevice(devConfig, matchACLNames...)
 			if len(matchedACLNames) > 0 {
 				// Call usageFunc with a list of matched ACLs and info about the instance NIC.
-				err := usageFunc(matchedACLNames, profile, devName, devConfig)
+				err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+					return usageFunc(ctx, tx, matchedACLNames, profile, devName, devConfig)
+				})
 				if err != nil {
 					return err
 				}
@@ -160,77 +185,90 @@ func UsedBy(s *state.State, aclProjectName string, usageFunc func(matchedACLName
 		}
 	}
 
-	// Find ACLs that have rules that reference the ACLs.
-	aclNames, err := s.DB.Cluster.GetNetworkACLs(aclProjectName)
+	var aclNames []string
+
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Find ACLs that have rules that reference the ACLs.
+		aclNames, err = tx.GetNetworkACLs(ctx, aclProjectName)
+
+		return err
+	})
 	if err != nil {
 		return err
 	}
 
 	for _, aclName := range aclNames {
-		_, aclInfo, err := s.DB.Cluster.GetNetworkACL(aclProjectName, aclName)
-		if err != nil {
-			return err
-		}
-
-		matchedACLNames := []string{}
-
-		// Ingress rules can specify ACL names in their Source subjects.
-		for _, rule := range aclInfo.Ingress {
-			for _, subject := range shared.SplitNTrimSpace(rule.Source, ",", -1, true) {
-				// Look for new matching ACLs, but ignore our own ACL reference in our own rules.
-				if shared.StringInSlice(subject, matchACLNames) && !shared.StringInSlice(subject, matchedACLNames) && subject != aclInfo.Name {
-					matchedACLNames = append(matchedACLNames, subject)
-				}
-			}
-		}
-
-		// Egress rules can specify ACL names in their Destination subjects.
-		for _, rule := range aclInfo.Egress {
-			for _, subject := range shared.SplitNTrimSpace(rule.Destination, ",", -1, true) {
-				// Look for new matching ACLs, but ignore our own ACL reference in our own rules.
-				if shared.StringInSlice(subject, matchACLNames) && !shared.StringInSlice(subject, matchedACLNames) && subject != aclInfo.Name {
-					matchedACLNames = append(matchedACLNames, subject)
-				}
-			}
-		}
-
-		if len(matchedACLNames) > 0 {
-			// Call usageFunc with a list of matched ACLs and info about the ACL.
-			err := usageFunc(matchedACLNames, aclInfo, "", nil)
+		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			_, aclInfo, err := tx.GetNetworkACL(ctx, aclProjectName, aclName)
 			if err != nil {
 				return err
 			}
-		}
-	}
 
-	// Find instances using the ACLs. Most expensive to do.
-	err = s.DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
-		// Get the instance's effective network project name.
-		instNetworkProject := project.NetworkProjectFromRecord(&p)
+			matchedACLNames := []string{}
 
-		// Skip instances who's effective network project doesn't match this Network ACL's project.
-		if instNetworkProject != aclProjectName {
-			return nil
-		}
+			// Ingress rules can specify ACL names in their Source subjects.
+			for _, rule := range aclInfo.Ingress {
+				for _, subject := range shared.SplitNTrimSpace(rule.Source, ",", -1, true) {
+					// Look for new matching ACLs, but ignore our own ACL reference in our own rules.
+					if shared.ValueInSlice(subject, matchACLNames) && !shared.ValueInSlice(subject, matchedACLNames) && subject != aclInfo.Name {
+						matchedACLNames = append(matchedACLNames, subject)
+					}
+				}
+			}
 
-		devices := db.ExpandInstanceDevices(inst.Devices.Clone(), inst.Profiles)
+			// Egress rules can specify ACL names in their Destination subjects.
+			for _, rule := range aclInfo.Egress {
+				for _, subject := range shared.SplitNTrimSpace(rule.Destination, ",", -1, true) {
+					// Look for new matching ACLs, but ignore our own ACL reference in our own rules.
+					if shared.ValueInSlice(subject, matchACLNames) && !shared.ValueInSlice(subject, matchedACLNames) && subject != aclInfo.Name {
+						matchedACLNames = append(matchedACLNames, subject)
+					}
+				}
+			}
 
-		// Iterate through each of the instance's devices, looking for NICs that are using any of the ACLs.
-		for devName, devConfig := range devices {
-			matchedACLNames := isInUseByDevice(devConfig, matchACLNames...)
 			if len(matchedACLNames) > 0 {
-				// Call usageFunc with a list of matched ACLs and info about the instance NIC.
-				err := usageFunc(matchedACLNames, inst, devName, devConfig)
+				// Call usageFunc with a list of matched ACLs and info about the ACL.
+				err = usageFunc(ctx, tx, matchedACLNames, aclInfo, "", nil)
 				if err != nil {
 					return err
 				}
 			}
-		}
 
-		return nil
-	})
-	if err != nil {
-		return err
+			// Find instances using the ACLs. Most expensive to do.
+			err = tx.InstanceList(ctx, func(inst db.InstanceArgs, p api.Project) error {
+				// Get the instance's effective network project name.
+				instNetworkProject := project.NetworkProjectFromRecord(&p)
+
+				// Skip instances who's effective network project doesn't match this Network ACL's project.
+				if instNetworkProject != aclProjectName {
+					return nil
+				}
+
+				devices := instancetype.ExpandInstanceDevices(inst.Devices.Clone(), inst.Profiles)
+
+				// Iterate through each of the instance's devices, looking for NICs that are using any of the ACLs.
+				for devName, devConfig := range devices {
+					matchedACLNames := isInUseByDevice(devConfig, matchACLNames...)
+					if len(matchedACLNames) > 0 {
+						// Call usageFunc with a list of matched ACLs and info about the instance NIC.
+						err := usageFunc(ctx, tx, matchedACLNames, inst, devName, devConfig)
+						if err != nil {
+							return err
+						}
+					}
+				}
+
+				return nil
+			})
+			if err != nil {
+				return err
+			}
+
+			return nil
+		})
+		if err != nil {
+			return err
+		}
 	}
 
 	return nil
@@ -246,7 +284,7 @@ func isInUseByDevice(d deviceConfig.Device, matchACLNames ...string) []string {
 	}
 
 	for _, nicACLName := range shared.SplitNTrimSpace(d["security.acls"], ",", -1, true) {
-		if shared.StringInSlice(nicACLName, matchACLNames) {
+		if shared.ValueInSlice(nicACLName, matchACLNames) {
 			matchedACLNames = append(matchedACLNames, nicACLName)
 		}
 	}
@@ -267,15 +305,15 @@ func NetworkUsage(s *state.State, aclProjectName string, aclNames []string, aclN
 	supportedNetTypes := []string{"bridge", "ovn"}
 
 	// Find all networks and instance/profile NICs that use any of the specified Network ACLs.
-	err := UsedBy(s, aclProjectName, func(matchedACLNames []string, usageType any, _ string, nicConfig map[string]string) error {
+	err := UsedBy(s, aclProjectName, func(ctx context.Context, tx *db.ClusterTx, matchedACLNames []string, usageType any, _ string, nicConfig map[string]string) error {
 		switch u := usageType.(type) {
 		case db.InstanceArgs, cluster.Profile:
-			networkID, network, _, err := s.DB.Cluster.GetNetworkInAnyState(aclProjectName, nicConfig["network"])
+			networkID, network, _, err := tx.GetNetworkInAnyState(ctx, aclProjectName, nicConfig["network"])
 			if err != nil {
 				return fmt.Errorf("Failed to load network %q: %w", nicConfig["network"], err)
 			}
 
-			if shared.StringInSlice(network.Type, supportedNetTypes) {
+			if shared.ValueInSlice(network.Type, supportedNetTypes) {
 				_, found := aclNets[network.Name]
 				if !found {
 					aclNets[network.Name] = NetworkACLUsage{
@@ -288,10 +326,10 @@ func NetworkUsage(s *state.State, aclProjectName string, aclNames []string, aclN
 			}
 
 		case *api.Network:
-			if shared.StringInSlice(u.Type, supportedNetTypes) {
+			if shared.ValueInSlice(u.Type, supportedNetTypes) {
 				_, found := aclNets[u.Name]
 				if !found {
-					networkID, network, _, err := s.DB.Cluster.GetNetworkInAnyState(aclProjectName, u.Name)
+					networkID, network, _, err := tx.GetNetworkInAnyState(ctx, aclProjectName, u.Name)
 					if err != nil {
 						return fmt.Errorf("Failed to load network %q: %w", u.Name, err)
 					}
