@@ -8,16 +8,16 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/network/openvswitch"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/validate"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/network/openvswitch"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 // OVN ACL rule priorities.
@@ -133,8 +133,14 @@ func OVNEnsureACLs(s *state.State, l logger.Logger, client *openvswitch.OVN, acl
 		}
 
 		if portGroupUUID == "" {
-			// Load the config we'll need to create the port group with ACL rules.
-			_, aclInfo, err := s.DB.Cluster.GetNetworkACL(aclProjectName, aclName)
+			var aclInfo *api.NetworkACL
+
+			err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				// Load the config we'll need to create the port group with ACL rules.
+				_, aclInfo, err = tx.GetNetworkACL(ctx, aclProjectName, aclName)
+
+				return err
+			})
 			if err != nil {
 				return nil, fmt.Errorf("Failed loading Network ACL %q: %w", aclName, err)
 			}
@@ -164,7 +170,11 @@ func OVNEnsureACLs(s *state.State, l logger.Logger, client *openvswitch.OVN, acl
 			// the default rule we add. We also need to reapply the rules if we are adding any
 			// new per-ACL-per-network port groups.
 			if reapplyRules || !portGroupHasACLs || len(addACLNets) > 0 {
-				_, aclInfo, err = s.DB.Cluster.GetNetworkACL(aclProjectName, aclName)
+				err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+					_, aclInfo, err = tx.GetNetworkACL(ctx, aclProjectName, aclName)
+
+					return err
+				})
 				if err != nil {
 					return nil, fmt.Errorf("Failed loading Network ACL %q: %w", aclName, err)
 				}
@@ -299,7 +309,7 @@ func ovnAddReferencedACLs(info *api.NetworkACL, referencedACLNames map[string]st
 				continue // Skip subjects already seen.
 			}
 
-			if shared.StringInSlice(subject, append(ruleSubjectInternalAliases, ruleSubjectExternalAliases...)) {
+			if shared.ValueInSlice(subject, append(ruleSubjectInternalAliases, ruleSubjectExternalAliases...)) {
 				continue // Skip special reserved subjects that are not ACL names.
 			}
 
@@ -481,7 +491,7 @@ func ovnRuleCriteriaToOVNACLRule(direction string, rule *api.NetworkACLRule, por
 	}
 
 	// Add protocol filters.
-	if shared.StringInSlice(rule.Protocol, []string{"tcp", "udp"}) {
+	if shared.ValueInSlice(rule.Protocol, []string{"tcp", "udp"}) {
 		matchParts = append(matchParts, rule.Protocol)
 
 		if rule.SourcePort != "" {
@@ -491,7 +501,7 @@ func ovnRuleCriteriaToOVNACLRule(direction string, rule *api.NetworkACLRule, por
 		if rule.DestinationPort != "" {
 			matchParts = append(matchParts, ovnRulePortToOVNACLMatch(rule.Protocol, "dst", shared.SplitNTrimSpace(rule.DestinationPort, ",", -1, false)...))
 		}
-	} else if shared.StringInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
+	} else if shared.ValueInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
 		matchParts = append(matchParts, rule.Protocol)
 
 		if rule.ICMPType != "" {
@@ -537,19 +547,19 @@ func ovnRuleSubjectToOVNACLMatch(direction string, aclNameIDs map[string]int64,
 	for _, subjectCriterion := range subjectCriteria {
 		if validate.IsNetworkRange(subjectCriterion) == nil {
 			criterionParts := strings.SplitN(subjectCriterion, "-", 2)
-			if len(criterionParts) > 1 {
-				ip := net.ParseIP(criterionParts[0])
-				if ip != nil {
-					protocol := "ip4"
-					if ip.To4() == nil {
-						protocol = "ip6"
-					}
-
-					fieldParts = append(fieldParts, fmt.Sprintf("(%s.%s >= %s && %s.%s <= %s)", protocol, direction, criterionParts[0], protocol, direction, criterionParts[1]))
-				}
-			} else {
+			if len(criterionParts) <= 1 {
 				return "", false, nil, fmt.Errorf("Invalid IP range %q", subjectCriterion)
 			}
+
+			ip := net.ParseIP(criterionParts[0])
+			if ip != nil {
+				protocol := "ip4"
+				if ip.To4() == nil {
+					protocol = "ip6"
+				}
+
+				fieldParts = append(fieldParts, fmt.Sprintf("(%s.%s >= %s && %s.%s <= %s)", protocol, direction, criterionParts[0], protocol, direction, criterionParts[1]))
+			}
 		} else {
 			// Try parsing subject as single IP or CIDR.
 			ip := net.ParseIP(subjectCriterion)
@@ -567,13 +577,13 @@ func ovnRuleSubjectToOVNACLMatch(direction string, aclNameIDs map[string]int64,
 			} else {
 				// If not valid IP subnet, check if subject is ACL name or network peer name.
 				var subjectPortSelector openvswitch.OVNPortGroup
-				if shared.StringInSlice(subjectCriterion, ruleSubjectInternalAliases) {
+				if shared.ValueInSlice(subjectCriterion, ruleSubjectInternalAliases) {
 					// Use pseudo port group name for special reserved port selector types.
 					// These will be expanded later for each network specific rule.
 					// Convert deprecated #internal to non-deprecated @internal if needed.
 					subjectPortSelector = openvswitch.OVNPortGroup(ruleSubjectInternal)
 					networkSpecific = true
-				} else if shared.StringInSlice(subjectCriterion, ruleSubjectExternalAliases) {
+				} else if shared.ValueInSlice(subjectCriterion, ruleSubjectExternalAliases) {
 					// Use pseudo port group name for special reserved port selector types.
 					// These will be expanded later for each network specific rule.
 					// Convert deprecated #external to non-deprecated @external if needed.
@@ -747,26 +757,35 @@ func OVNApplyNetworkBaselineRules(client *openvswitch.OVN, switchName openvswitc
 // the desired ACLs are considered unused by the usage type even if the referring config has not yet been removed
 // from the database.
 func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvswitch.OVN, aclProjectName string, ignoreUsageType any, ignoreUsageNicName string, keepACLs ...string) error {
-	// Get map of ACL names to DB IDs (used for generating OVN port group names).
-	aclNameIDs, err := s.DB.Cluster.GetNetworkACLIDsByNames(aclProjectName)
-	if err != nil {
-		return fmt.Errorf("Failed getting network ACL IDs for security ACL port group removal: %w", err)
-	}
-
-	// Convert aclNameIDs to aclNames slice for use with UsedBy.
-	aclNames := make([]string, 0, len(aclNameIDs))
-	for aclName := range aclNameIDs {
-		aclNames = append(aclNames, aclName)
-	}
-
-	// Get project ID.
+	var aclNameIDs map[string]int64
+	var aclNames []string
 	var projectID int64
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Get map of ACL names to DB IDs (used for generating OVN port group names).
+		aclNameIDs, err = tx.GetNetworkACLIDsByNames(ctx, aclProjectName)
+		if err != nil {
+			return fmt.Errorf("Failed getting network ACL IDs for security ACL port group removal: %w", err)
+		}
+
+		// Convert aclNameIDs to aclNames slice for use with UsedBy.
+		aclNames = make([]string, 0, len(aclNameIDs))
+		for aclName := range aclNameIDs {
+			aclNames = append(aclNames, aclName)
+		}
+
+		// Get project ID.
 		projectID, err = cluster.GetProjectID(ctx, tx.Tx(), aclProjectName)
-		return err
+		if err != nil {
+			return fmt.Errorf("Failed getting project ID for project %q: %w", aclProjectName, err)
+		}
+
+		return nil
 	})
 	if err != nil {
-		return fmt.Errorf("Failed getting project ID for project %q: %w", aclProjectName, err)
+		return err
 	}
 
 	// Get list of OVN port groups associated to this project.
@@ -813,7 +832,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 	// Find alls ACLs that are either directly referred to by OVN entities (networks, instance/profile NICs)
 	// or indirectly by being referred to by a ruleset of another ACL that is itself in use by OVN entities.
 	// For the indirectly referred to ACLs, store a list of the ACLs that are referring to it.
-	err = UsedBy(s, aclProjectName, func(matchedACLNames []string, usageType any, nicName string, nicConfig map[string]string) error {
+	err = UsedBy(s, aclProjectName, func(ctx context.Context, tx *db.ClusterTx, matchedACLNames []string, usageType any, nicName string, nicConfig map[string]string) error {
 		switch u := usageType.(type) {
 		case db.InstanceArgs:
 			ignoreInst, isIgnoreInst := ignoreUsageType.(instance.Instance)
@@ -825,11 +844,11 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 			// If an ignore instance was provided, then skip the device that the ACLs were just removed
 			// from. In case DB record is not updated until the update process has completed otherwise
 			// we would still consider it using the ACL.
-			if isIgnoreInst && ignoreInst.Name() == u.Name && ignoreInst.Project() == u.Project && ignoreUsageNicName == nicName {
+			if isIgnoreInst && ignoreInst.Name() == u.Name && ignoreInst.Project().Name == u.Project && ignoreUsageNicName == nicName {
 				return nil
 			}
 
-			netID, network, _, err := s.DB.Cluster.GetNetworkInAnyState(aclProjectName, nicConfig["network"])
+			netID, network, _, err := tx.GetNetworkInAnyState(ctx, aclProjectName, nicConfig["network"])
 			if err != nil {
 				return fmt.Errorf("Failed to load network %q: %w", nicConfig["network"], err)
 			}
@@ -858,7 +877,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 			}
 
 			if u.Type == "ovn" {
-				netID, _, _, err := s.DB.Cluster.GetNetworkInAnyState(aclProjectName, u.Name)
+				netID, _, _, err := tx.GetNetworkInAnyState(ctx, aclProjectName, u.Name)
 				if err != nil {
 					return fmt.Errorf("Failed to load network %q: %w", nicConfig["network"], err)
 				}
@@ -885,7 +904,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 				return nil
 			}
 
-			netID, network, _, err := s.DB.Cluster.GetNetworkInAnyState(aclProjectName, nicConfig["network"])
+			netID, network, _, err := tx.GetNetworkInAnyState(ctx, aclProjectName, nicConfig["network"])
 			if err != nil {
 				return fmt.Errorf("Failed to load network %q: %w", nicConfig["network"], err)
 			}
@@ -906,7 +925,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 					aclUsedACLS[matchedACLName] = make([]string, 0, 1)
 				}
 
-				if !shared.StringInSlice(u.Name, aclUsedACLS[matchedACLName]) {
+				if !shared.ValueInSlice(u.Name, aclUsedACLS[matchedACLName]) {
 					// Record as in use by another ACL entity.
 					aclUsedACLS[matchedACLName] = append(aclUsedACLS[matchedACLName], u.Name)
 				}
@@ -974,11 +993,11 @@ func OVNPortGroupInstanceNICSchedule(portUUID openvswitch.OVNSwitchPortUUID, cha
 
 // OVNApplyInstanceNICDefaultRules applies instance NIC default rules to per-network port group.
 func OVNApplyInstanceNICDefaultRules(client *openvswitch.OVN, switchPortGroup openvswitch.OVNPortGroup, logPrefix string, nicPortName openvswitch.OVNSwitchPort, ingressAction string, ingressLogged bool, egressAction string, egressLogged bool) error {
-	if !shared.StringInSlice(ingressAction, ValidActions) {
+	if !shared.ValueInSlice(ingressAction, ValidActions) {
 		return fmt.Errorf("Invalid ingress action %q", ingressAction)
 	}
 
-	if !shared.StringInSlice(egressAction, ValidActions) {
+	if !shared.ValueInSlice(egressAction, ValidActions) {
 		return fmt.Errorf("Invalid egress action %q", egressAction)
 	}
 
@@ -1059,12 +1078,12 @@ func ovnParseLogEntry(input string, prefix string) string {
 	}
 
 	// Get the protocol.
-	severityFields := strings.Split(aclEntry["severity"], " ")
-	if len(severityFields) != 2 {
+	directionFields := strings.Split(aclEntry["direction"], " ")
+	if len(directionFields) != 2 {
 		return ""
 	}
 
-	protocol := severityFields[1]
+	protocol := directionFields[1]
 
 	// Get the source and destination addresses.
 	srcAddr, ok := aclEntry["nw_src"]
