@@ -2,6 +2,7 @@ package acl
 
 import (
 	"bufio"
+	"context"
 	"fmt"
 	"net"
 	"os"
@@ -9,21 +10,20 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/cluster/request"
-	"github.com/lxc/lxd/lxd/db"
-	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/network/openvswitch"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/validate"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/cluster"
+	"github.com/canonical/lxd/lxd/cluster/request"
+	"github.com/canonical/lxd/lxd/db"
+	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/network/openvswitch"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/validate"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // Define type for rule directions.
@@ -66,7 +66,7 @@ func (d *common) init(state *state.State, id int64, projectName string, info *ap
 		d.info = info
 	}
 
-	d.logger = logger.AddContext(logger.Log, logger.Ctx{"project": projectName, "networkACL": d.info.Name})
+	d.logger = logger.AddContext(logger.Ctx{"project": projectName, "networkACL": d.info.Name})
 	d.id = id
 	d.projectName = projectName
 	d.state = state
@@ -97,7 +97,7 @@ func (d *common) ID() int64 {
 	return d.id
 }
 
-// Name returns the project.
+// Project returns the project name.
 func (d *common) Project() string {
 	return d.projectName
 }
@@ -122,32 +122,32 @@ func (d *common) usedBy(firstOnly bool) ([]string, error) {
 	usedBy := []string{}
 
 	// Find all networks, profiles and instance NICs that use this Network ACL.
-	err := UsedBy(d.state, d.projectName, func(_ []string, usageType any, _ string, _ map[string]string) error {
+	err := UsedBy(d.state, d.projectName, func(ctx context.Context, tx *db.ClusterTx, _ []string, usageType any, _ string, _ map[string]string) error {
 		switch u := usageType.(type) {
 		case db.InstanceArgs:
 			uri := fmt.Sprintf("/%s/instances/%s", version.APIVersion, u.Name)
-			if u.Project != project.Default {
+			if u.Project != api.ProjectDefaultName {
 				uri += fmt.Sprintf("?project=%s", u.Project)
 			}
 
 			usedBy = append(usedBy, uri)
 		case *api.Network:
 			uri := fmt.Sprintf("/%s/networks/%s", version.APIVersion, u.Name)
-			if d.projectName != project.Default {
+			if d.projectName != api.ProjectDefaultName {
 				uri += fmt.Sprintf("?project=%s", d.projectName)
 			}
 
 			usedBy = append(usedBy, uri)
 		case dbCluster.Profile:
 			uri := fmt.Sprintf("/%s/profiles/%s", version.APIVersion, u.Name)
-			if u.Project != project.Default {
+			if u.Project != api.ProjectDefaultName {
 				uri += fmt.Sprintf("?project=%s", u.Project)
 			}
 
 			usedBy = append(usedBy, uri)
 		case *api.NetworkACL:
 			uri := fmt.Sprintf("/%s/network-acls/%s", version.APIVersion, u.Name)
-			if d.projectName != project.Default {
+			if d.projectName != api.ProjectDefaultName {
 				uri += fmt.Sprintf("?project=%s", d.projectName)
 			}
 
@@ -261,7 +261,7 @@ func (d *common) validateConfigMap(config map[string]string, rules map[string]fu
 
 	// Run the validator against each field.
 	for k, validator := range rules {
-		checkedFields[k] = struct{}{} //Mark field as checked.
+		checkedFields[k] = struct{}{} // Mark field as checked.
 		err := validator(config[k])
 		if err != nil {
 			return fmt.Errorf("Invalid value for config option %q: %w", k, err)
@@ -289,18 +289,26 @@ func (d *common) validateConfigMap(config map[string]string, rules map[string]fu
 // validateRule validates the rule supplied.
 func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule) error {
 	// Validate Action field (required).
-	if !shared.StringInSlice(rule.Action, ValidActions) {
+	if !shared.ValueInSlice(rule.Action, ValidActions) {
 		return fmt.Errorf("Action must be one of: %s", strings.Join(ValidActions, ", "))
 	}
 
 	// Validate State field (required).
 	validStates := []string{"enabled", "disabled", "logged"}
-	if !shared.StringInSlice(rule.State, validStates) {
+	if !shared.ValueInSlice(rule.State, validStates) {
 		return fmt.Errorf("State must be one of: %s", strings.Join(validStates, ", "))
 	}
 
-	// Get map of ACL names to DB IDs (used for generating OVN port group names).
-	acls, err := d.state.DB.Cluster.GetNetworkACLIDsByNames(d.Project())
+	var acls map[string]int64
+
+	err := d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Get map of ACL names to DB IDs (used for generating OVN port group names).
+		acls, err = tx.GetNetworkACLIDsByNames(ctx, d.Project())
+
+		return err
+	})
 	if err != nil {
 		return fmt.Errorf("Failed getting network ACLs for security ACL subject validation: %w", err)
 	}
@@ -345,13 +353,13 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 	// Validate Protocol field.
 	if rule.Protocol != "" {
 		validProtocols := []string{"icmp4", "icmp6", "tcp", "udp"}
-		if !shared.StringInSlice(rule.Protocol, validProtocols) {
+		if !shared.ValueInSlice(rule.Protocol, validProtocols) {
 			return fmt.Errorf("Protocol must be one of: %s", strings.Join(validProtocols, ", "))
 		}
 	}
 
 	// Validate protocol dependent fields.
-	if shared.StringInSlice(rule.Protocol, []string{"tcp", "udp"}) {
+	if shared.ValueInSlice(rule.Protocol, []string{"tcp", "udp"}) {
 		if rule.ICMPType != "" {
 			return fmt.Errorf("ICMP type cannot be used with non-ICMP protocol")
 		}
@@ -375,7 +383,7 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 				return fmt.Errorf("Invalid Destination port: %w", err)
 			}
 		}
-	} else if shared.StringInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
+	} else if shared.ValueInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
 		if rule.SourcePort != "" {
 			return fmt.Errorf("Source port cannot be used with %q protocol", rule.Protocol)
 		}
@@ -441,7 +449,7 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 // validateRuleSubjects checks that the source or destination subjects for a rule are valid.
 // Accepts a validSubjectNames list of valid ACL or special classifier names.
 // Returns whether the subjects include names, IPv4 and IPv6 addresses respectively.
-func (d *common) validateRuleSubjects(fieldName string, direction ruleDirection, subjects []string, validSubjectNames []string) (bool, bool, bool, error) {
+func (d *common) validateRuleSubjects(fieldName string, direction ruleDirection, subjects []string, validSubjectNames []string) (hasName bool, hasIPv4 bool, hasIPv6 bool, err error) {
 	// Check if named subjects are allowed in field/direction combination.
 	allowSubjectNames := false
 	if (fieldName == "Source" && direction == ruleDirectionIngress) || (fieldName == "Destination" && direction == ruleDirectionEgress) {
@@ -535,10 +543,6 @@ func (d *common) validateRuleSubjects(fieldName string, direction ruleDirection,
 		return 0, fmt.Errorf("Invalid subject %q", subject)
 	}
 
-	hasIPv4 := false
-	hasIPv6 := false
-	hasName := false
-
 	for _, s := range subjects {
 		ipVersion, err := validSubject(s)
 		if err != nil {
@@ -581,22 +585,27 @@ func (d *common) Update(config *api.NetworkACLPut, clientType request.ClientType
 	defer revert.Fail()
 
 	if clientType == request.ClientTypeNormal {
-		oldConfig := d.info.NetworkACLPut
+		oldConfig := d.info.Writable()
 
-		// Update database. Its important this occurs before we attempt to apply to networks using the ACL
-		// as usage functions will inspect the database.
-		err = d.state.DB.Cluster.UpdateNetworkACL(d.id, config)
+		err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Update database. Its important this occurs before we attempt to apply to networks using the ACL
+			// as usage functions will inspect the database.
+			return tx.UpdateNetworkACL(ctx, d.id, *config)
+		})
 		if err != nil {
 			return err
 		}
 
 		// Apply changes internally and reinitialise.
-		d.info.NetworkACLPut = *config
+		d.info.SetWritable(*config)
 		d.init(d.state, d.id, d.projectName, d.info)
 
 		revert.Add(func() {
-			_ = d.state.DB.Cluster.UpdateNetworkACL(d.id, &oldConfig)
-			d.info.NetworkACLPut = oldConfig
+			_ = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				return tx.UpdateNetworkACL(ctx, d.id, oldConfig)
+			})
+
+			d.info.SetWritable(oldConfig)
 			d.init(d.state, d.id, d.projectName, d.info)
 		})
 	}
@@ -636,8 +645,14 @@ func (d *common) Update(config *api.NetworkACLPut, clientType request.ClientType
 			return fmt.Errorf("Failed to get OVN client: %w", err)
 		}
 
-		// Get map of ACL names to DB IDs (used for generating OVN port group names).
-		aclNameIDs, err := d.state.DB.Cluster.GetNetworkACLIDsByNames(d.Project())
+		var aclNameIDs map[string]int64
+
+		err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Get map of ACL names to DB IDs (used for generating OVN port group names).
+			aclNameIDs, err = tx.GetNetworkACLIDsByNames(ctx, d.Project())
+
+			return err
+		})
 		if err != nil {
 			return fmt.Errorf("Failed getting network ACL IDs for security ACL update: %w", err)
 		}
@@ -672,7 +687,7 @@ func (d *common) Update(config *api.NetworkACLPut, clientType request.ClientType
 		}
 
 		err = notifier(func(client lxd.InstanceServer) error {
-			return client.UseProject(d.projectName).UpdateNetworkACL(d.info.Name, d.info.NetworkACLPut, "")
+			return client.UseProject(d.projectName).UpdateNetworkACL(d.info.Name, d.info.Writable(), "")
 		})
 		if err != nil {
 			return err
@@ -704,7 +719,9 @@ func (d *common) Rename(newName string) error {
 		return err
 	}
 
-	err = d.state.DB.Cluster.RenameNetworkACL(d.id, newName)
+	err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.RenameNetworkACL(ctx, d.id, newName)
+	})
 	if err != nil {
 		return err
 	}
@@ -726,7 +743,9 @@ func (d *common) Delete() error {
 		return fmt.Errorf("Cannot delete an ACL that is in use")
 	}
 
-	return d.state.DB.Cluster.DeleteNetworkACL(d.id)
+	return d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.DeleteNetworkACL(ctx, d.id)
+	})
 }
 
 // GetLog gets the ACL log.
