@@ -7,21 +7,22 @@ import (
 	"os"
 	"strconv"
 	"strings"
+	"unicode"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/cluster/request"
-	"github.com/lxc/lxd/lxd/db"
-	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/network/acl"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/resources"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/validate"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/cluster"
+	"github.com/canonical/lxd/lxd/cluster/request"
+	"github.com/canonical/lxd/lxd/db"
+	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/network/acl"
+	"github.com/canonical/lxd/lxd/resources"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/validate"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // Info represents information about a network driver.
@@ -29,6 +30,7 @@ type Info struct {
 	Projects           bool // Indicates if driver can be used in network enabled projects.
 	NodeSpecificConfig bool // Whether driver has cluster node specific config as a prerequisite for creation.
 	AddressForwards    bool // Indicates if driver supports address forwards.
+	LoadBalancers      bool // Indicates if driver supports load balancers.
 	Peering            bool // Indicates if the driver supports network peering.
 }
 
@@ -45,6 +47,12 @@ type forwardPortMap struct {
 	target      forwardTarget
 }
 
+type loadBalancerPortMap struct {
+	listenPorts []uint64
+	protocol    string
+	targets     []forwardTarget
+}
+
 // subnetUsageType indicates the type of use for a subnet.
 type subnetUsageType uint
 
@@ -52,7 +60,9 @@ const (
 	subnetUsageNetwork subnetUsageType = iota
 	subnetUsageNetworkSNAT
 	subnetUsageNetworkForward
+	subnetUsageNetworkLoadBalancer
 	subnetUsageInstance
+	subnetUsageProxy
 )
 
 // externalSubnetUsage represents usage of a subnet by a network or NIC.
@@ -83,7 +93,7 @@ type common struct {
 
 // init initialise internal variables.
 func (n *common) init(state *state.State, id int64, projectName string, netInfo *api.Network, netNodes map[int64]db.NetworkNode) {
-	n.logger = logger.AddContext(logger.Log, logger.Ctx{"project": projectName, "driver": netInfo.Type, "network": netInfo.Name})
+	n.logger = logger.AddContext(logger.Ctx{"project": projectName, "driver": netInfo.Type, "network": netInfo.Name})
 	n.id = id
 	n.project = projectName
 	n.name = netInfo.Name
@@ -120,7 +130,7 @@ func (n *common) validate(config map[string]string, driverRules map[string]func(
 
 	// Run the validator against each field.
 	for k, validator := range rules {
-		checkedFields[k] = struct{}{} //Mark field as checked.
+		checkedFields[k] = struct{}{} // Mark field as checked.
 		err := validator(config[k])
 		if err != nil {
 			return fmt.Errorf("Invalid value for network %q option %q: %w", n.name, k, err)
@@ -145,11 +155,51 @@ func (n *common) validate(config map[string]string, driverRules map[string]func(
 	return nil
 }
 
-// validateZoneName checks that a user provided zone name is valid.
-func (n *common) validateZoneName(name string) error {
-	_, _, _, err := n.state.DB.Cluster.GetNetworkZone(name)
+// validateZoneNames checks the DNS zone names are valid in config.
+func (n *common) validateZoneNames(config map[string]string) error {
+	// Check if DNS zones in use.
+	if config["dns.zone.forward"] == "" && config["dns.zone.reverse.ipv4"] == "" && config["dns.zone.reverse.ipv6"] == "" {
+		return nil
+	}
+
+	var err error
+	var zoneProjects map[string]string
+	err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		zoneProjects, err = tx.GetNetworkZones(ctx)
+		if err != nil {
+			return fmt.Errorf("Failed to load all network zones: %w", err)
+		}
+
+		return nil
+	})
 	if err != nil {
-		return fmt.Errorf("Network zone %q doesn't exist", name)
+		return err
+	}
+
+	for _, keyName := range []string{"dns.zone.forward", "dns.zone.reverse.ipv4", "dns.zone.reverse.ipv6"} {
+		keyZoneNames := shared.SplitNTrimSpace(config[keyName], ",", -1, true)
+		keyZoneNamesLen := len(keyZoneNames)
+		if keyZoneNamesLen < 1 {
+			continue
+		} else if keyZoneNamesLen > 1 && (keyName == "dns.zone.reverse.ipv4" || keyName == "dns.zone.reverse.ipv6") {
+			return fmt.Errorf("Invalid %q must contain only single DNS zone name", keyName)
+		}
+
+		zoneProjectsUsed := make(map[string]struct{}, 0)
+
+		for _, keyZoneName := range keyZoneNames {
+			zoneProjectName, found := zoneProjects[keyZoneName]
+			if !found {
+				return fmt.Errorf("Invalid %q, network zone %q not found", keyName, keyZoneName)
+			}
+
+			_, zoneProjectUsed := zoneProjectsUsed[zoneProjectName]
+			if zoneProjectUsed {
+				return fmt.Errorf("Invalid %q, contains multiple zones from the same project", keyName)
+			}
+
+			zoneProjectsUsed[zoneProjectName] = struct{}{}
+		}
 	}
 
 	return nil
@@ -219,16 +269,18 @@ func (n *common) Config() map[string]string {
 	return n.config
 }
 
+// IsManaged returns true if the network is managed by LXD and false otherwise.
 func (n *common) IsManaged() bool {
 	return n.managed
 }
 
-// Config returns the common network driver info.
+// Info returns the common network driver info.
 func (n *common) Info() Info {
 	return Info{
 		Projects:           false,
 		NodeSpecificConfig: true,
 		AddressForwards:    false,
+		LoadBalancers:      false,
 	}
 }
 
@@ -323,7 +375,7 @@ func (n *common) update(applyNetwork api.NetworkPut, targetNode string, clientTy
 			sendNetwork.Config = make(map[string]string)
 			for k, v := range applyNetwork.Config {
 				// Don't forward node specific keys (these will be merged in on recipient node).
-				if shared.StringInSlice(k, db.NodeSpecificNetworkConfig) {
+				if shared.ValueInSlice(k, db.NodeSpecificNetworkConfig) {
 					continue
 				}
 
@@ -338,8 +390,10 @@ func (n *common) update(applyNetwork api.NetworkPut, targetNode string, clientTy
 			}
 		}
 
-		// Update the database.
-		err := n.state.DB.Cluster.UpdateNetwork(n.project, n.name, applyNetwork.Description, applyNetwork.Config)
+		err := n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Update the database.
+			return tx.UpdateNetwork(ctx, n.project, n.name, applyNetwork.Description, applyNetwork.Config)
+		})
 		if err != nil {
 			return err
 		}
@@ -376,7 +430,7 @@ func (n *common) configChanged(newNetwork api.NetworkPut) (bool, []string, api.N
 			dbUpdateNeeded = true
 
 			// Add non-user changed key to list of changed keys.
-			if !strings.HasPrefix(k, "user.") && !shared.StringInSlice(k, changedKeys) {
+			if !strings.HasPrefix(k, "user.") && !shared.ValueInSlice(k, changedKeys) {
 				changedKeys = append(changedKeys, k)
 			}
 		}
@@ -387,7 +441,7 @@ func (n *common) configChanged(newNetwork api.NetworkPut) (bool, []string, api.N
 			dbUpdateNeeded = true
 
 			// Add non-user changed key to list of changed keys.
-			if !strings.HasPrefix(k, "user.") && !shared.StringInSlice(k, changedKeys) {
+			if !strings.HasPrefix(k, "user.") && !shared.ValueInSlice(k, changedKeys) {
 				changedKeys = append(changedKeys, k)
 			}
 		}
@@ -411,8 +465,10 @@ func (n *common) rename(newName string) error {
 		}
 	}
 
-	// Rename the database entry.
-	err := n.state.DB.Cluster.RenameNetwork(n.project, n.name, newName)
+	err := n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Rename the database entry.
+		return tx.RenameNetwork(ctx, n.project, n.name, newName)
+	})
 	if err != nil {
 		return err
 	}
@@ -426,7 +482,7 @@ func (n *common) rename(newName string) error {
 // warningsDelete deletes any persistent warnings for the network.
 func (n *common) warningsDelete() error {
 	err := n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		return dbCluster.DeleteWarnings(ctx, tx.Tx(), dbCluster.TypeNetwork, int(n.ID()))
+		return dbCluster.DeleteWarnings(ctx, tx.Tx(), dbCluster.EntityType(entity.TypeNetwork), int(n.ID()))
 	})
 	if err != nil {
 		return fmt.Errorf("Failed deleting persistent warnings: %w", err)
@@ -473,7 +529,7 @@ func (n *common) HandleHeartbeat(heartbeatData *cluster.APIHeartbeat) error {
 
 // notifyDependentNetworks allows any dependent networks to apply changes to themselves when this network changes.
 func (n *common) notifyDependentNetworks(changedKeys []string) {
-	if n.Project() != project.Default {
+	if n.Project() != api.ProjectDefaultName {
 		return // Only networks in the default project can be used as dependent networks.
 	}
 
@@ -491,8 +547,14 @@ func (n *common) notifyDependentNetworks(changedKeys []string) {
 	}
 
 	for _, projectName := range projectNames {
-		// Get a list of managed networks in project.
-		depNets, err := n.state.DB.Cluster.GetCreatedNetworks(projectName)
+		var depNets []string
+
+		err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Get a list of managed networks in project.
+			depNets, err = tx.GetCreatedNetworkNamesByProject(ctx, projectName)
+
+			return err
+		})
 		if err != nil {
 			n.logger.Error("Failed to load networks in project", logger.Ctx{"project": projectName, "err": err})
 			continue // Continue to next project.
@@ -528,14 +590,14 @@ func (n *common) bgpValidationRules(config map[string]string) (map[string]func(v
 	rules := map[string]func(value string) error{}
 	for k := range config {
 		// BGP keys have the peer name in their name, extract the suffix.
-		if !strings.HasPrefix(k, "bgp.") {
+		if !strings.HasPrefix(k, "bgp.peers.") {
 			continue
 		}
 
 		// Validate remote name in key.
 		fields := strings.Split(k, ".")
 		if len(fields) != 4 {
-			return nil, fmt.Errorf("Invalid network configuration key: %s", k)
+			return nil, fmt.Errorf("Invalid network configuration key: %q", k)
 		}
 
 		bgpKey := fields[3]
@@ -548,6 +610,8 @@ func (n *common) bgpValidationRules(config map[string]string) (map[string]func(v
 			rules[k] = validate.Optional(validate.IsInRange(1, 4294967294))
 		case "password":
 			rules[k] = validate.Optional(validate.IsAny)
+		case "holdtime":
+			rules[k] = validate.Optional(validate.IsInRange(9, 65535))
 		}
 	}
 
@@ -621,7 +685,7 @@ func (n *common) bgpSetupPeers(oldConfig map[string]string) error {
 
 	// Remove old peers.
 	for _, peer := range oldPeers {
-		if shared.StringInSlice(peer, newPeers) {
+		if shared.ValueInSlice(peer, newPeers) {
 			continue
 		}
 
@@ -635,7 +699,7 @@ func (n *common) bgpSetupPeers(oldConfig map[string]string) error {
 
 	// Add new peers.
 	for _, peer := range newPeers {
-		if shared.StringInSlice(peer, oldPeers) {
+		if shared.ValueInSlice(peer, oldPeers) {
 			continue
 		}
 
@@ -646,7 +710,15 @@ func (n *common) bgpSetupPeers(oldConfig map[string]string) error {
 			return err
 		}
 
-		err = n.state.BGP.AddPeer(net.ParseIP(fields[0]), uint32(asn), fields[2])
+		var holdTime uint64
+		if fields[3] != "" {
+			holdTime, err = strconv.ParseUint(fields[3], 10, 32)
+			if err != nil {
+				return err
+			}
+		}
+
+		err = n.state.BGP.AddPeer(net.ParseIP(fields[0]), uint32(asn), fields[2], holdTime)
 		if err != nil {
 			return err
 		}
@@ -707,7 +779,7 @@ func (n *common) bgpSetupPrefixes(oldConfig map[string]string) error {
 					return err
 				}
 			}
-		} else if !shared.StringInSlice(n.config[fmt.Sprintf("ipv%d.address", ipVersion)], []string{"", "none"}) {
+		} else if !shared.ValueInSlice(n.config[fmt.Sprintf("ipv%d.address", ipVersion)], []string{"", "none"}) {
 			// If network has NAT disabled, then export the network's subnet if specified.
 			netAddress := n.config[fmt.Sprintf("ipv%d.address", ipVersion)]
 			_, subnet, err := net.ParseCIDR(netAddress)
@@ -735,7 +807,7 @@ func (n *common) bgpGetPeers(config map[string]string) []string {
 		}
 
 		fields := strings.Split(k, ".")
-		if !shared.StringInSlice(fields[2], peerNames) {
+		if !shared.ValueInSlice(fields[2], peerNames) {
 			peerNames = append(peerNames, fields[2])
 		}
 	}
@@ -746,9 +818,10 @@ func (n *common) bgpGetPeers(config map[string]string) []string {
 		peerAddress := config[fmt.Sprintf("bgp.peers.%s.address", peerName)]
 		peerASN := config[fmt.Sprintf("bgp.peers.%s.asn", peerName)]
 		peerPassword := config[fmt.Sprintf("bgp.peers.%s.password", peerName)]
+		peerHoldTime := config[fmt.Sprintf("bgp.peers.%s.holdtime", peerName)]
 
 		if peerAddress != "" && peerASN != "" {
-			peers = append(peers, fmt.Sprintf("%s,%s,%s", peerAddress, peerASN, peerPassword))
+			peers = append(peers, fmt.Sprintf("%s,%s,%s,%s", peerAddress, peerASN, peerPassword, peerHoldTime))
 		}
 	}
 
@@ -756,11 +829,15 @@ func (n *common) bgpGetPeers(config map[string]string) []string {
 }
 
 // forwardValidate validates the forward request.
-func (n *common) forwardValidate(listenAddress net.IP, forward *api.NetworkForwardPut) ([]*forwardPortMap, error) {
+func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwardPut) ([]*forwardPortMap, error) {
 	if listenAddress == nil {
 		return nil, fmt.Errorf("Invalid listen address")
 	}
 
+	if listenAddress.IsUnspecified() {
+		return nil, fmt.Errorf("Cannot use unspecified address: %q", listenAddress.String())
+	}
+
 	listenIsIP4 := listenAddress.To4() != nil
 
 	// For checking target addresses are within network's subnet.
@@ -825,7 +902,7 @@ func (n *common) forwardValidate(listenAddress net.IP, forward *api.NetworkForwa
 	// Maps portSpecID to a portMap struct.
 	portMaps := make([]*forwardPortMap, 0, len(forward.Ports))
 	for portSpecID, portSpec := range forward.Ports {
-		if !shared.StringInSlice(portSpec.Protocol, validPortProcols) {
+		if !shared.ValueInSlice(portSpec.Protocol, validPortProcols) {
 			return nil, fmt.Errorf("Invalid port protocol in port specification %d, protocol must be one of: %s", portSpecID, strings.Join(validPortProcols, ", "))
 		}
 
@@ -930,8 +1007,16 @@ func (n *common) ForwardDelete(listenAddress string, clientType request.ClientTy
 
 // forwardBGPSetupPrefixes exports external forward addresses as prefixes.
 func (n *common) forwardBGPSetupPrefixes() error {
-	// Retrieve network forwards before clearing existing prefixes, and separate them by IP family.
-	fwdListenAddresses, err := n.state.DB.Cluster.GetNetworkForwardListenAddresses(n.ID(), true)
+	var fwdListenAddresses map[int64]string
+
+	err := n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Retrieve network forwards before clearing existing prefixes, and separate them by IP family.
+		fwdListenAddresses, err = tx.GetNetworkForwardListenAddresses(ctx, n.ID(), true)
+
+		return err
+	})
 	if err != nil {
 		return fmt.Errorf("Failed loading network forwards: %w", err)
 	}
@@ -994,12 +1079,325 @@ func (n *common) forwardBGPSetupPrefixes() error {
 	return nil
 }
 
+// getExternalSubnetInUse returns information about usage of external subnets by networks connected to, or used by,
+// the specified uplinkNetworkName.
+func (n *common) getExternalSubnetInUse(ctx context.Context, tx *db.ClusterTx, uplinkNetworkName string, memberSpecific bool) ([]externalSubnetUsage, error) {
+	var err error
+	var projectNetworksForwardsOnUplink, projectNetworksLoadBalancersOnUplink map[string]map[string][]string
+
+	// Get all network forward listen addresses for all networks (of any type) connected to our uplink.
+	projectNetworksForwardsOnUplink, err = tx.GetProjectNetworkForwardListenAddressesByUplink(ctx, uplinkNetworkName, memberSpecific)
+	if err != nil {
+		return nil, fmt.Errorf("Failed loading network forward listen addresses: %w", err)
+	}
+
+	// Get all network load balancer listen addresses for all networks (of any type) connected to our uplink.
+	projectNetworksLoadBalancersOnUplink, err = tx.GetProjectNetworkLoadBalancerListenAddressesByUplink(ctx, uplinkNetworkName, memberSpecific)
+	if err != nil {
+		return nil, fmt.Errorf("Failed loading network forward listen addresses: %w", err)
+	}
+
+	externalSubnets := make([]externalSubnetUsage, 0, len(projectNetworksForwardsOnUplink)+len(projectNetworksLoadBalancersOnUplink))
+
+	// Add forward listen addresses to this list.
+	for projectName, networks := range projectNetworksForwardsOnUplink {
+		for networkName, listenAddresses := range networks {
+			for _, listenAddress := range listenAddresses {
+				// Convert listen address to subnet.
+				listenAddressNet, err := ParseIPToNet(listenAddress)
+				if err != nil {
+					return nil, fmt.Errorf("Invalid existing forward listen address %q", listenAddress)
+				}
+
+				// Create an externalSubnetUsage for the listen address by using the network ID
+				// of the listen address to retrieve the already loaded network name from the
+				// projectNetworks map.
+				externalSubnets = append(externalSubnets, externalSubnetUsage{
+					subnet:         *listenAddressNet,
+					networkProject: projectName,
+					networkName:    networkName,
+					usageType:      subnetUsageNetworkForward,
+				})
+			}
+		}
+	}
+
+	// Add load balancer listen addresses to this list.
+	for projectName, networks := range projectNetworksLoadBalancersOnUplink {
+		for networkName, listenAddresses := range networks {
+			for _, listenAddress := range listenAddresses {
+				// Convert listen address to subnet.
+				listenAddressNet, err := ParseIPToNet(listenAddress)
+				if err != nil {
+					return nil, fmt.Errorf("Invalid existing load balancer listen address %q", listenAddress)
+				}
+
+				// Create an externalSubnetUsage for the listen address by using the network ID
+				// of the listen address to retrieve the already loaded network name from the
+				// projectNetworks map.
+				externalSubnets = append(externalSubnets, externalSubnetUsage{
+					subnet:         *listenAddressNet,
+					networkProject: projectName,
+					networkName:    networkName,
+					usageType:      subnetUsageNetworkLoadBalancer,
+				})
+			}
+		}
+	}
+
+	return externalSubnets, nil
+}
+
+// loadBalancerValidate validates the load balancer request.
+func (n *common) loadBalancerValidate(listenAddress net.IP, forward api.NetworkLoadBalancerPut) ([]*loadBalancerPortMap, error) {
+	if listenAddress == nil {
+		return nil, fmt.Errorf("Invalid listen address")
+	}
+
+	listenIsIP4 := listenAddress.To4() != nil
+
+	// For checking target addresses are within network's subnet.
+	netIPKey := "ipv4.address"
+	if !listenIsIP4 {
+		netIPKey = "ipv6.address"
+	}
+
+	netIPAddress := n.config[netIPKey]
+
+	var err error
+	var netSubnet *net.IPNet
+	if netIPAddress != "" {
+		_, netSubnet, err = net.ParseCIDR(n.config[netIPKey])
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	// Look for any unknown config fields.
+	for k := range forward.Config {
+		// User keys are not validated.
+		if shared.IsUserConfig(k) {
+			continue
+		}
+
+		return nil, fmt.Errorf("Invalid option %q", k)
+	}
+
+	// Validate port rules.
+	validPortProcols := []string{"tcp", "udp"}
+
+	// Used to ensure that each listen port is only used once.
+	listenPorts := map[string]map[int64]struct{}{
+		"tcp": make(map[int64]struct{}),
+		"udp": make(map[int64]struct{}),
+	}
+
+	// Check backends config and store the parsed target by backend name.
+	backendsByName := make(map[string]*forwardTarget, len(forward.Backends))
+	for backendSpecID, backendSpec := range forward.Backends {
+		for _, r := range backendSpec.Name {
+			if unicode.IsSpace(r) {
+				return nil, fmt.Errorf("Name cannot contain white space in backend specification %d", backendSpecID)
+			}
+		}
+
+		_, found := backendsByName[backendSpec.Name]
+		if found {
+			return nil, fmt.Errorf("Duplicate name %q in backend specification %d", backendSpec.Name, backendSpecID)
+		}
+
+		targetAddress := net.ParseIP(backendSpec.TargetAddress)
+		if targetAddress == nil {
+			return nil, fmt.Errorf("Invalid target address for backend %q", backendSpec.Name)
+		}
+
+		targetIsIP4 := targetAddress.To4() != nil
+		if listenIsIP4 != targetIsIP4 {
+			return nil, fmt.Errorf("Cannot mix IP versions in listen address and backend %q target address", backendSpec.Name)
+		}
+
+		// Check target address is within network's subnet.
+		if netSubnet != nil && !SubnetContainsIP(netSubnet, targetAddress) {
+			return nil, fmt.Errorf("Target address is not within the network subnet for backend %q", backendSpec.Name)
+		}
+
+		// Check valid target port(s) supplied.
+		target := forwardTarget{
+			address: targetAddress,
+		}
+
+		for portSpecID, portSpec := range shared.SplitNTrimSpace(backendSpec.TargetPort, ",", -1, true) {
+			portFirst, portRange, err := ParsePortRange(portSpec)
+			if err != nil {
+				return nil, fmt.Errorf("Invalid backend port specification %d in backend specification %d: %w", portSpecID, backendSpecID, err)
+			}
+
+			for i := int64(0); i < portRange; i++ {
+				port := portFirst + i
+				target.ports = append(target.ports, uint64(port))
+			}
+		}
+
+		backendsByName[backendSpec.Name] = &target
+	}
+
+	// Check ports config.
+	portMaps := make([]*loadBalancerPortMap, 0, len(forward.Ports))
+	for portSpecID, portSpec := range forward.Ports {
+		if !shared.ValueInSlice(portSpec.Protocol, validPortProcols) {
+			return nil, fmt.Errorf("Invalid port protocol in port specification %d, protocol must be one of: %s", portSpecID, strings.Join(validPortProcols, ", "))
+		}
+
+		// Check valid listen port(s) supplied.
+		listenPortRanges := shared.SplitNTrimSpace(portSpec.ListenPort, ",", -1, true)
+		if len(listenPortRanges) <= 0 {
+			return nil, fmt.Errorf("Missing listen port in port specification %d", portSpecID)
+		}
+
+		portMap := loadBalancerPortMap{
+			listenPorts: make([]uint64, 0),
+			protocol:    portSpec.Protocol,
+			targets:     make([]forwardTarget, 0, len(portSpec.TargetBackend)),
+		}
+
+		for _, pr := range listenPortRanges {
+			portFirst, portRange, err := ParsePortRange(pr)
+			if err != nil {
+				return nil, fmt.Errorf("Invalid listen port in port specification %d: %w", portSpecID, err)
+			}
+
+			for i := int64(0); i < portRange; i++ {
+				port := portFirst + i
+				_, found := listenPorts[portSpec.Protocol][port]
+				if found {
+					return nil, fmt.Errorf("Duplicate listen port %d for protocol %q in port specification %d", port, portSpec.Protocol, portSpecID)
+				}
+
+				listenPorts[portSpec.Protocol][port] = struct{}{}
+				portMap.listenPorts = append(portMap.listenPorts, uint64(port))
+			}
+		}
+
+		// Check each of the backends specified are compatible with the listen ports.
+		for _, backendName := range portSpec.TargetBackend {
+			// Check backend exists.
+			backend, found := backendsByName[backendName]
+			if !found {
+				return nil, fmt.Errorf("Invalid target backend name %q in port specification %d", backendName, portSpecID)
+			}
+
+			// Only check if the target port count matches the listen port count if the target ports
+			// are greater than 1, because we allow many-to-one type mapping and one-to-one mapping if
+			// no target ports specified.
+			portSpectTargetPortsLen := len(backend.ports)
+			if portSpectTargetPortsLen > 1 && len(portMap.listenPorts) != portSpectTargetPortsLen {
+				return nil, fmt.Errorf("Mismatch of listen port(s) and target port(s) count for backend %q in port specification %d", backendName, portSpecID)
+			}
+
+			portMap.targets = append(portMap.targets, *backend)
+		}
+
+		portMaps = append(portMaps, &portMap)
+	}
+
+	return portMaps, err
+}
+
+// LoadBalancerCreate returns ErrNotImplemented for drivers that do not support load balancers.
+func (n *common) LoadBalancerCreate(loadBalancer api.NetworkLoadBalancersPost, clientType request.ClientType) error {
+	return ErrNotImplemented
+}
+
+// LoadBalancerUpdate returns ErrNotImplemented for drivers that do not support load balancers..
+func (n *common) LoadBalancerUpdate(listenAddress string, newLoadBalancer api.NetworkLoadBalancerPut, clientType request.ClientType) error {
+	return ErrNotImplemented
+}
+
+// LoadBalancerDelete returns ErrNotImplemented for drivers that do not support load balancers..
+func (n *common) LoadBalancerDelete(listenAddress string, clientType request.ClientType) error {
+	return ErrNotImplemented
+}
+
+// loadBalancerBGPSetupPrefixes exports external load balancer addresses as prefixes.
+func (n *common) loadBalancerBGPSetupPrefixes() error {
+	var listenAddresses map[int64]string
+
+	err := n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Retrieve network forwards before clearing existing prefixes, and separate them by IP family.
+		listenAddresses, err = tx.GetNetworkLoadBalancerListenAddresses(ctx, n.ID(), true)
+
+		return err
+	})
+	if err != nil {
+		return fmt.Errorf("Failed loading network forwards: %w", err)
+	}
+
+	listenAddressesByFamily := map[uint][]string{
+		4: make([]string, 0),
+		6: make([]string, 0),
+	}
+
+	for _, listenAddress := range listenAddresses {
+		if strings.Contains(listenAddress, ":") {
+			listenAddressesByFamily[6] = append(listenAddressesByFamily[6], listenAddress)
+		} else {
+			listenAddressesByFamily[4] = append(listenAddressesByFamily[4], listenAddress)
+		}
+	}
+
+	// Use load balancer specific owner string (different from the network prefixes) so that these can be
+	// reapplied independently of the network's own prefixes.
+	bgpOwner := fmt.Sprintf("network_%d_load_balancer", n.id)
+
+	// Clear existing address load balancer prefixes for network.
+	err = n.state.BGP.RemovePrefixByOwner(bgpOwner)
+	if err != nil {
+		return err
+	}
+
+	// Add the new prefixes.
+	for _, ipVersion := range []uint{4, 6} {
+		nextHopAddr := n.bgpNextHopAddress(ipVersion)
+		natEnabled := shared.IsTrue(n.config[fmt.Sprintf("ipv%d.nat", ipVersion)])
+		_, netSubnet, _ := net.ParseCIDR(n.config[fmt.Sprintf("ipv%d.address", ipVersion)])
+
+		routeSubnetSize := 128
+		if ipVersion == 4 {
+			routeSubnetSize = 32
+		}
+
+		// Export external forward listen addresses.
+		for _, listenAddress := range listenAddressesByFamily[ipVersion] {
+			listenAddr := net.ParseIP(listenAddress)
+
+			// Don't export internal address forwards (those inside the NAT enabled network's subnet).
+			if natEnabled && netSubnet != nil && netSubnet.Contains(listenAddr) {
+				continue
+			}
+
+			_, ipRouteSubnet, err := net.ParseCIDR(fmt.Sprintf("%s/%d", listenAddr.String(), routeSubnetSize))
+			if err != nil {
+				return err
+			}
+
+			err = n.state.BGP.AddPrefix(*ipRouteSubnet, nextHopAddr, bgpOwner)
+			if err != nil {
+				return err
+			}
+		}
+	}
+
+	return nil
+}
+
 // Leases returns ErrNotImplemented for drivers that don't support address leases.
 func (n *common) Leases(projectName string, clientType request.ClientType) ([]api.NetworkLease, error) {
 	return nil, ErrNotImplemented
 }
 
-// PeerCrete returns ErrNotImplemented for drivers that do not support forwards.
+// PeerCreate returns ErrNotImplemented for drivers that do not support forwards.
 func (n *common) PeerCreate(forward api.NetworkPeersPost) error {
 	return ErrNotImplemented
 }
@@ -1021,7 +1419,7 @@ func (n *common) peerValidate(peerName string, peer *api.NetworkPeerPut) error {
 		return err
 	}
 
-	if shared.StringInSlice(peerName, acl.ReservedNetworkSubects) {
+	if shared.ValueInSlice(peerName, acl.ReservedNetworkSubects) {
 		return fmt.Errorf("Name cannot be one of the reserved network subjects: %v", acl.ReservedNetworkSubects)
 	}
 
@@ -1087,14 +1485,28 @@ func (n *common) peerUsedBy(peerName string, firstOnly bool) ([]string, error) {
 		return false
 	}
 
-	// Find ACLs that have rules that reference the peer connection.
-	aclNames, err := n.state.DB.Cluster.GetNetworkACLs(n.Project())
+	var aclNames []string
+
+	err := n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Find ACLs that have rules that reference the peer connection.
+		aclNames, err = tx.GetNetworkACLs(ctx, n.Project())
+
+		return err
+	})
 	if err != nil {
 		return nil, err
 	}
 
 	for _, aclName := range aclNames {
-		_, aclInfo, err := n.state.DB.Cluster.GetNetworkACL(n.Project(), aclName)
+		var aclInfo *api.NetworkACL
+
+		err := n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			_, aclInfo, err = tx.GetNetworkACL(ctx, n.Project(), aclName)
+
+			return err
+		})
 		if err != nil {
 			return nil, err
 		}
@@ -1116,6 +1528,7 @@ func (n *common) peerUsedBy(peerName string, firstOnly bool) ([]string, error) {
 	return usedBy, nil
 }
 
+// State returns the api.NetworkState for the network.
 func (n *common) State() (*api.NetworkState, error) {
 	return resources.GetNetworkState(n.name)
 }
