@@ -4,6 +4,7 @@ import (
 	"bufio"
 	"bytes"
 	"context"
+	cryptorand "crypto/rand"
 	"encoding/hex"
 	"fmt"
 	"math/big"
@@ -15,22 +16,23 @@ import (
 	"sync"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/device/nictype"
-	"github.com/lxc/lxd/lxd/dnsmasq"
-	"github.com/lxc/lxd/lxd/dnsmasq/dhcpalloc"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/ip"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/device/nictype"
+	"github.com/canonical/lxd/lxd/dnsmasq"
+	"github.com/canonical/lxd/lxd/dnsmasq/dhcpalloc"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/ip"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/validate"
+	"github.com/canonical/lxd/shared/version"
 )
 
 func networkValidPort(value string) error {
@@ -56,7 +58,11 @@ func networkValidPort(value string) error {
 func RandomDevName(prefix string) string {
 	// Return a new random veth device name.
 	randBytes := make([]byte, 4)
-	rand.Read(randBytes)
+	_, err := cryptorand.Read(randBytes)
+	if err != nil {
+		return ""
+	}
+
 	iface := prefix + hex.EncodeToString(randBytes)
 	if len(iface) > 13 {
 		return ""
@@ -65,10 +71,35 @@ func RandomDevName(prefix string) string {
 	return iface
 }
 
+// MACDevName returns interface name with prefix 'lxd' and MAC without leading 2 digits.
+func MACDevName(mac net.HardwareAddr) string {
+	devName := strings.Join(strings.Split(mac.String(), ":"), "")
+	return fmt.Sprintf("lxd%s", devName[2:])
+}
+
 // UsedByInstanceDevices looks for instance NIC devices using the network and runs the supplied usageFunc for each.
 // Accepts optional filter arguments to specify a subset of instances.
 func UsedByInstanceDevices(s *state.State, networkProjectName string, networkName string, networkType string, usageFunc func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error, filters ...cluster.InstanceFilter) error {
-	return s.DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
+	// Get the instances.
+	projects := map[string]api.Project{}
+	instances := []db.InstanceArgs{}
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.InstanceList(ctx, func(inst db.InstanceArgs, p api.Project) error {
+			projects[inst.Project] = p
+			instances = append(instances, inst)
+
+			return nil
+		}, filters...)
+	})
+	if err != nil {
+		return err
+	}
+
+	// Go through the instances and run usageFunc.
+	for _, inst := range instances {
+		p := projects[inst.Project]
+
 		// Get the instance's effective network project name.
 		instNetworkProject := project.NetworkProjectFromRecord(&p)
 
@@ -78,7 +109,7 @@ func UsedByInstanceDevices(s *state.State, networkProjectName string, networkNam
 		}
 
 		// Look for NIC devices using this network.
-		devices := db.ExpandInstanceDevices(inst.Devices.Clone(), inst.Profiles)
+		devices := instancetype.ExpandInstanceDevices(inst.Devices.Clone(), inst.Profiles)
 		for devName, devConfig := range devices {
 			if isInUseByDevice(networkName, networkType, devConfig) {
 				err := usageFunc(inst, devName, devConfig)
@@ -87,9 +118,9 @@ func UsedByInstanceDevices(s *state.State, networkProjectName string, networkNam
 				}
 			}
 		}
+	}
 
-		return nil
-	}, filters...)
+	return nil
 }
 
 // UsedBy returns list of API resources using network. Accepts firstOnly argument to indicate that only the first
@@ -100,7 +131,13 @@ func UsedBy(s *state.State, networkProjectName string, networkID int64, networkN
 
 	// If managed network being passed in, check if it has any peerings in a created state.
 	if networkID > 0 {
-		peers, err := s.DB.Cluster.GetNetworkPeers(networkID)
+		var peers map[int64]*api.NetworkPeer
+
+		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			peers, err = tx.GetNetworkPeers(ctx, networkID)
+
+			return err
+		})
 		if err != nil {
 			return nil, fmt.Errorf("Failed getting network peers: %w", err)
 		}
@@ -118,7 +155,7 @@ func UsedBy(s *state.State, networkProjectName string, networkID int64, networkN
 	}
 
 	// Only networks defined in the default project can be used by other networks. Cheapest to do.
-	if networkProjectName == project.Default {
+	if networkProjectName == api.ProjectDefaultName {
 		// Get all managed networks across all projects.
 		var projectNetworks map[string]map[int64]api.Network
 
@@ -366,8 +403,12 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 	if networkName == "" {
 		var err error
 
-		// Pass project.Default here, as currently dnsmasq (bridged) networks do not support projects.
-		networks, err = s.DB.Cluster.GetNetworks(project.Default)
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			// Pass api.ProjectDefaultName here, as currently dnsmasq (bridged) networks do not support projects.
+			networks, err = tx.GetNetworks(ctx, api.ProjectDefaultName)
+
+			return err
+		})
 		if err != nil {
 			return err
 		}
@@ -402,7 +443,7 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 			}
 
 			// Skip devices not connected to managed networks.
-			if !shared.StringInSlice(d["parent"], networks) {
+			if !shared.ValueInSlice(d["parent"], networks) {
 				continue
 			}
 
@@ -447,10 +488,10 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 			continue
 		}
 
-		// Pass project.Default here, as currently dnsmasq (bridged) networks do not support projects.
-		n, err := LoadByName(s, project.Default, network)
+		// Pass api.ProjectDefaultName here, as currently dnsmasq (bridged) networks do not support projects.
+		n, err := LoadByName(s, api.ProjectDefaultName, network)
 		if err != nil {
-			return fmt.Errorf("Failed to load network %q in project %q for dnsmasq update: %w", project.Default, network, err)
+			return fmt.Errorf("Failed to load network %q in project %q for dnsmasq update: %w", api.ProjectDefaultName, network, err)
 		}
 
 		config := n.Config()
@@ -639,7 +680,7 @@ func inRoutingTable(subnet *net.IPNet) bool {
 		// Get the mask
 		var mask net.IPMask
 		if filename == "ipv6_route" {
-			size, err := strconv.ParseInt(fmt.Sprintf("0x%s", fields[1]), 0, 64)
+			size, err := strconv.ParseInt(fields[1], 16, 0)
 			if err != nil {
 				continue
 			}
@@ -675,15 +716,23 @@ func inRoutingTable(subnet *net.IPNet) bool {
 	return false
 }
 
-// pingIP sends a single ping packet to the specified IP, returns true if responds, false if not.
-func pingIP(ip net.IP) bool {
+// pingIP sends a single ping packet to the specified IP, returns nil error if IP is reachable.
+// If ctx doesn't have a deadline then the default timeout used is 1s.
+func pingIP(ctx context.Context, ip net.IP) error {
 	cmd := "ping"
 	if ip.To4() == nil {
 		cmd = "ping6"
 	}
 
-	_, err := shared.RunCommand(cmd, "-n", "-q", ip.String(), "-c", "1", "-W", "1")
-	return err == nil
+	timeout := time.Second * 1
+	deadline, ok := ctx.Deadline()
+	if ok {
+		timeout = time.Until(deadline)
+	}
+
+	_, err := shared.RunCommandContext(ctx, cmd, "-n", "-q", ip.String(), "-c", "1", "-w", fmt.Sprintf("%d", int(timeout.Seconds())))
+
+	return err
 }
 
 func pingSubnet(subnet *net.IPNet) bool {
@@ -694,7 +743,7 @@ func pingSubnet(subnet *net.IPNet) bool {
 	ping := func(ip net.IP) {
 		defer wgChecks.Done()
 
-		if !pingIP(ip) {
+		if pingIP(context.TODO(), ip) != nil {
 			return
 		}
 
@@ -877,7 +926,7 @@ func usesIPv4Firewall(netConfig map[string]string) bool {
 		return false
 	}
 
-	if netConfig["ipv4.firewall"] == "" || shared.IsTrue(netConfig["ipv4.firewall"]) {
+	if shared.IsTrueOrEmpty(netConfig["ipv4.firewall"]) {
 		return true
 	}
 
@@ -894,7 +943,7 @@ func usesIPv6Firewall(netConfig map[string]string) bool {
 		return false
 	}
 
-	if netConfig["ipv6.firewall"] == "" || shared.IsTrue(netConfig["ipv6.firewall"]) {
+	if shared.IsTrueOrEmpty(netConfig["ipv6.firewall"]) {
 		return true
 	}
 
@@ -920,116 +969,6 @@ func randomHwaddr(r *rand.Rand) string {
 	return ret.String()
 }
 
-// parseIPRange parses an IP range in the format "start-end" and converts it to a shared.IPRange.
-// If allowedNets are supplied, then each IP in the range is checked that it belongs to at least one of them.
-// IPs in the range can be zero prefixed, e.g. "::1" or "0.0.0.1", however they should not overlap with any
-// supplied allowedNets prefixes. If they are within an allowed network, any zero prefixed addresses are
-// returned combined with the first allowed network they are within.
-// If no allowedNets supplied they are returned as-is.
-func parseIPRange(ipRange string, allowedNets ...*net.IPNet) (*shared.IPRange, error) {
-	inAllowedNet := func(ip net.IP, allowedNet *net.IPNet) net.IP {
-		if ip == nil {
-			return nil
-		}
-
-		ipv4 := ip.To4()
-
-		// Only match IPv6 addresses against IPv6 networks.
-		if ipv4 == nil && allowedNet.IP.To4() != nil {
-			return nil
-		}
-
-		// Combine IP with network prefix if IP starts with a zero.
-		// If IP is v4, then compare against 4-byte representation, otherwise use 16 byte representation.
-		if (ipv4 != nil && ipv4[0] == 0) || (ipv4 == nil && ip[0] == 0) {
-			allowedNet16 := allowedNet.IP.To16()
-			ipCombined := make(net.IP, net.IPv6len)
-			for i, b := range ip {
-				ipCombined[i] = allowedNet16[i] | b
-			}
-
-			ip = ipCombined
-		}
-
-		// Check start IP is within one of the allowed networks.
-		if !allowedNet.Contains(ip) {
-			return nil
-		}
-
-		return ip
-	}
-
-	rangeParts := strings.SplitN(ipRange, "-", 2)
-	if len(rangeParts) != 2 {
-		return nil, fmt.Errorf("IP range %q must contain start and end IP addresses", ipRange)
-	}
-
-	startIP := net.ParseIP(rangeParts[0])
-	endIP := net.ParseIP(rangeParts[1])
-
-	if startIP == nil {
-		return nil, fmt.Errorf("Start IP %q is invalid", rangeParts[0])
-	}
-
-	if endIP == nil {
-		return nil, fmt.Errorf("End IP %q is invalid", rangeParts[1])
-	}
-
-	if bytes.Compare(startIP, endIP) > 0 {
-		return nil, fmt.Errorf("Start IP %q must be less than End IP %q", startIP, endIP)
-	}
-
-	if len(allowedNets) > 0 {
-		matchFound := false
-		for _, allowedNet := range allowedNets {
-			if allowedNet == nil {
-				return nil, fmt.Errorf("Invalid allowed network")
-			}
-
-			combinedStartIP := inAllowedNet(startIP, allowedNet)
-			if combinedStartIP == nil {
-				continue
-			}
-
-			combinedEndIP := inAllowedNet(endIP, allowedNet)
-			if combinedEndIP == nil {
-				continue
-			}
-
-			// If both match then replace parsed IPs with combined IPs and stop searching.
-			matchFound = true
-			startIP = combinedStartIP
-			endIP = combinedEndIP
-			break
-		}
-
-		if !matchFound {
-			return nil, fmt.Errorf("IP range %q does not fall within any of the allowed networks %v", ipRange, allowedNets)
-		}
-	}
-
-	return &shared.IPRange{
-		Start: startIP,
-		End:   endIP,
-	}, nil
-}
-
-// parseIPRanges parses a comma separated list of IP ranges using parseIPRange.
-func parseIPRanges(ipRangesList string, allowedNets ...*net.IPNet) ([]*shared.IPRange, error) {
-	ipRanges := strings.Split(ipRangesList, ",")
-	netIPRanges := make([]*shared.IPRange, 0, len(ipRanges))
-	for _, ipRange := range ipRanges {
-		netIPRange, err := parseIPRange(strings.TrimSpace(ipRange), allowedNets...)
-		if err != nil {
-			return nil, err
-		}
-
-		netIPRanges = append(netIPRanges, netIPRange)
-	}
-
-	return netIPRanges, nil
-}
-
 // VLANInterfaceCreate creates a VLAN interface on parent interface (if needed).
 // Returns boolean indicating if VLAN interface was created.
 func VLANInterfaceCreate(parent string, vlanDevice string, vlanID string, gvrp bool) (bool, error) {
@@ -1090,6 +1029,16 @@ func InterfaceExists(nic string) bool {
 	return false
 }
 
+// IPInSlice returns true if slice has IP element.
+func IPInSlice(key net.IP, list []net.IP) bool {
+	for _, entry := range list {
+		if entry.Equal(key) {
+			return true
+		}
+	}
+	return false
+}
+
 // SubnetContains returns true if outerSubnet contains innerSubnet.
 func SubnetContains(outerSubnet *net.IPNet, innerSubnet *net.IPNet) bool {
 	if outerSubnet == nil || innerSubnet == nil {
@@ -1182,32 +1131,6 @@ func SubnetParseAppend(subnets []*net.IPNet, parseSubnet ...string) ([]*net.IPNe
 	return subnets, nil
 }
 
-// InterfaceBindWait waits for network interface to appear after being bound to a driver.
-func InterfaceBindWait(ifName string) error {
-	for i := 0; i < 10; i++ {
-		if InterfaceExists(ifName) {
-			return nil
-		}
-
-		time.Sleep(50 * time.Millisecond)
-	}
-
-	return fmt.Errorf("Bind of interface %q took too long", ifName)
-}
-
-// IPRangesOverlap checks whether two ip ranges have ip addresses in common.
-func IPRangesOverlap(r1, r2 *shared.IPRange) bool {
-	if r1.End == nil {
-		return r2.ContainsIP(r1.Start)
-	}
-
-	if r2.End == nil {
-		return r1.ContainsIP(r2.Start)
-	}
-
-	return r1.ContainsIP(r2.Start) || r1.ContainsIP(r2.End)
-}
-
 // InterfaceStatus returns the global unicast IP addresses configured on an interface and whether it is up or not.
 func InterfaceStatus(nicName string) ([]net.IP, bool, error) {
 	iface, err := net.InterfaceByName(nicName)
@@ -1238,18 +1161,18 @@ func InterfaceStatus(nicName string) ([]net.IP, bool, error) {
 }
 
 // ParsePortRange validates a port range in the form start-end.
-func ParsePortRange(r string) (int64, int64, error) {
+func ParsePortRange(r string) (base int64, size int64, err error) {
 	entries := strings.Split(r, "-")
 	if len(entries) > 2 {
 		return -1, -1, fmt.Errorf("Invalid port range %q", r)
 	}
 
-	base, err := strconv.ParseInt(entries[0], 10, 64)
+	base, err = strconv.ParseInt(entries[0], 10, 64)
 	if err != nil {
 		return -1, -1, err
 	}
 
-	size := int64(1)
+	size = int64(1)
 	if len(entries) > 1 {
 		size, err = strconv.ParseInt(entries[1], 10, 64)
 		if err != nil {
@@ -1297,6 +1220,19 @@ func ParseIPCIDRToNet(ipAddressCIDR string) (*net.IPNet, error) {
 	return listenAddressNet, err
 }
 
+// IPToNet converts an IP to a single host IPNet.
+func IPToNet(ip net.IP) net.IPNet {
+	bits := 32
+	if ip.To4() == nil {
+		bits = 128
+	}
+
+	return net.IPNet{
+		IP:   ip,
+		Mask: net.CIDRMask(bits, bits),
+	}
+}
+
 // NICUsesNetwork returns true if the nicDev's "network" or "parent" property matches one of the networks names.
 func NICUsesNetwork(nicDev map[string]string, networks ...*api.Network) bool {
 	for _, network := range networks {
@@ -1330,3 +1266,67 @@ func BridgeNetfilterEnabled(ipVersion uint) error {
 
 	return nil
 }
+
+// ProxyParseAddr validates a proxy address and parses it into its constituent parts.
+func ProxyParseAddr(data string) (*deviceConfig.ProxyAddress, error) {
+	// Split into <protocol> and <address>.
+	fields := strings.SplitN(data, ":", 2)
+
+	if !shared.ValueInSlice(fields[0], []string{"tcp", "udp", "unix"}) {
+		return nil, fmt.Errorf("Unknown protocol type %q", fields[0])
+	}
+
+	if len(fields) < 2 || fields[1] == "" {
+		return nil, fmt.Errorf("Missing address")
+	}
+
+	newProxyAddr := &deviceConfig.ProxyAddress{
+		ConnType: fields[0],
+		Abstract: strings.HasPrefix(fields[1], "@"),
+	}
+
+	// unix addresses cannot have ports.
+	if newProxyAddr.ConnType == "unix" {
+		newProxyAddr.Address = fields[1]
+
+		return newProxyAddr, nil
+	}
+
+	// Split <address> into <address> and <ports>.
+	address, port, err := net.SplitHostPort(fields[1])
+	if err != nil {
+		return nil, err
+	}
+
+	// Validate that it's a valid address.
+	if shared.ValueInSlice(newProxyAddr.ConnType, []string{"udp", "tcp"}) {
+		err := validate.Optional(validate.IsNetworkAddress)(address)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	newProxyAddr.Address = address
+
+	// Split <ports> into individual ports and port ranges.
+	ports := strings.SplitN(port, ",", -1)
+
+	newProxyAddr.Ports = make([]uint64, 0, len(ports))
+
+	for _, p := range ports {
+		portFirst, portRange, err := ParsePortRange(p)
+		if err != nil {
+			return nil, err
+		}
+
+		for i := int64(0); i < portRange; i++ {
+			newProxyAddr.Ports = append(newProxyAddr.Ports, uint64(portFirst+i))
+		}
+	}
+
+	if len(newProxyAddr.Ports) <= 0 {
+		return nil, fmt.Errorf("At least one port is required")
+	}
+
+	return newProxyAddr, nil
+}
