@@ -1,14 +1,17 @@
 package openvswitch
 
 import (
+	"context"
 	"fmt"
 	"net"
+	"os"
 	"strconv"
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
+	"github.com/canonical/lxd/lxd/linux"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
 )
 
 // OVNRouter OVN router name.
@@ -71,6 +74,7 @@ const ovnExtIDLXDSwitch = "lxd_switch"
 const ovnExtIDLXDSwitchPort = "lxd_switch_port"
 const ovnExtIDLXDProjectID = "lxd_project_id"
 const ovnExtIDLXDPortGroup = "lxd_port_group"
+const ovnExtIDLXDLocation = "lxd_location"
 
 // OVNIPv6RAOpts IPv6 router advertisements options that can be applied to a router.
 type OVNIPv6RAOpts struct {
@@ -98,6 +102,7 @@ type OVNDHCPv4Opts struct {
 	DomainName         string
 	LeaseTime          time.Duration
 	MTU                uint32
+	Netmask            string
 }
 
 // OVNDHCPv6Opts IPv6 DHCP option set that can be created (and then applied to a switch port by resulting ID).
@@ -113,6 +118,9 @@ type OVNSwitchPortOpts struct {
 	IPs          []net.IP           // Optional, if empty IPs will be set to dynamic.
 	DHCPv4OptsID OVNDHCPOptionsUUID // Optional, if empty, no DHCPv4 enabled on port.
 	DHCPv6OptsID OVNDHCPOptionsUUID // Optional, if empty, no DHCPv6 enabled on port.
+	Parent       OVNSwitchPort      // Optional, if set a nested port is created.
+	VLAN         uint16             // Optional, use with Parent to request a specific VLAN for nested port.
+	Location     string             // Optional, use to indicate the name of the LXD server this port is bound to.
 }
 
 // OVNACLRule represents an ACL rule that can be added to a logical switch or port group.
@@ -144,6 +152,7 @@ type OVNRouterRoute struct {
 	Prefix  net.IPNet
 	NextHop net.IP
 	Port    OVNRouterPort
+	Discard bool
 }
 
 // OVNRouterPolicy represents a router policy.
@@ -171,16 +180,66 @@ type OVNRouterPeering struct {
 
 // NewOVN initialises new OVN client wrapper with the connection set in network.ovn.northbound_connection config.
 func NewOVN(s *state.State) (*OVN, error) {
+	// Get database connection strings.
 	nbConnection := s.GlobalConfig.NetworkOVNNorthboundConnection()
-
 	sbConnection, err := NewOVS().OVNSouthboundDBRemoteAddress()
 	if err != nil {
 		return nil, fmt.Errorf("Failed to get OVN southbound connection string: %w", err)
 	}
 
-	client := &OVN{}
-	client.SetNorthboundDBAddress(nbConnection)
-	client.SetSouthboundDBAddress(sbConnection)
+	// Create the OVN struct.
+	client := &OVN{
+		nbDBAddr: nbConnection,
+		sbDBAddr: sbConnection,
+	}
+
+	// If using SSL, then get the CA and client key pair.
+	if strings.Contains(nbConnection, "ssl:") {
+		sslCACert, sslClientCert, sslClientKey := s.GlobalConfig.NetworkOVNSSL()
+
+		if sslCACert == "" {
+			content, err := os.ReadFile("/etc/ovn/ovn-central.crt")
+			if err != nil {
+				if os.IsNotExist(err) {
+					return nil, fmt.Errorf("OVN configured to use SSL but no SSL CA certificate defined")
+				}
+
+				return nil, err
+			}
+
+			sslCACert = string(content)
+		}
+
+		if sslClientCert == "" {
+			content, err := os.ReadFile("/etc/ovn/cert_host")
+			if err != nil {
+				if os.IsNotExist(err) {
+					return nil, fmt.Errorf("OVN configured to use SSL but no SSL client certificate defined")
+				}
+
+				return nil, err
+			}
+
+			sslClientCert = string(content)
+		}
+
+		if sslClientKey == "" {
+			content, err := os.ReadFile("/etc/ovn/key_host")
+			if err != nil {
+				if os.IsNotExist(err) {
+					return nil, fmt.Errorf("OVN configured to use SSL but no SSL client key defined")
+				}
+
+				return nil, err
+			}
+
+			sslClientKey = string(content)
+		}
+
+		client.sslCACert = sslCACert
+		client.sslClientCert = sslClientCert
+		client.sslClientKey = sslClientKey
+	}
 
 	return client, nil
 }
@@ -189,6 +248,10 @@ func NewOVN(s *state.State) (*OVN, error) {
 type OVN struct {
 	nbDBAddr string
 	sbDBAddr string
+
+	sslCACert     string
+	sslClientCert string
+	sslClientKey  string
 }
 
 // SetNorthboundDBAddress sets the address that runs the OVN northbound databases.
@@ -245,16 +308,45 @@ func (o *OVN) xbctl(southbound bool, extraArgs ...string) (string, error) {
 	// Figure out args.
 	args := []string{"--timeout=10", "--db", dbAddr}
 
+	// Handle SSL args.
+	files := []*os.File{}
 	if strings.Contains(dbAddr, "ssl:") {
+		// Handle client certificate.
+		clientCertFile, err := linux.CreateMemfd([]byte(o.sslClientCert))
+		if err != nil {
+			return "", err
+		}
+
+		defer clientCertFile.Close()
+		files = append(files, clientCertFile)
+
+		// Handle client key.
+		clientKeyFile, err := linux.CreateMemfd([]byte(o.sslClientKey))
+		if err != nil {
+			return "", err
+		}
+
+		defer clientKeyFile.Close()
+		files = append(files, clientKeyFile)
+
+		// Handle CA certificate.
+		caCertFile, err := linux.CreateMemfd([]byte(o.sslCACert))
+		if err != nil {
+			return "", err
+		}
+
+		defer caCertFile.Close()
+		files = append(files, caCertFile)
+
 		args = append(args,
-			"-c", "/etc/ovn/cert_host",
-			"-p", "/etc/ovn/key_host",
-			"-C", "/etc/ovn/ovn-central.crt",
+			"-c", "/proc/self/fd/3",
+			"-p", "/proc/self/fd/4",
+			"-C", "/proc/self/fd/5",
 		)
 	}
 
 	args = append(args, extraArgs...)
-	return shared.RunCommand(cmd, args...)
+	return shared.RunCommandInheritFds(context.Background(), files, cmd, args...)
 }
 
 // LogicalRouterAdd adds a named logical router.
@@ -380,7 +472,13 @@ func (o *OVN) LogicalRouterRouteAdd(routerName OVNRouter, mayExist bool, routes
 			args = append(args, "--may-exist")
 		}
 
-		args = append(args, "lr-route-add", string(routerName), route.Prefix.String(), route.NextHop.String())
+		args = append(args, "lr-route-add", string(routerName), route.Prefix.String())
+
+		if route.Discard {
+			args = append(args, "discard")
+		} else {
+			args = append(args, route.NextHop.String())
+		}
 
 		if route.Port != "" {
 			args = append(args, string(route.Port))
@@ -751,7 +849,7 @@ func (o *OVN) LogicalSwitchDHCPv4RevervationsSet(switchName OVNSwitch, reservedI
 
 // LogicalSwitchDHCPv4RevervationsGet gets the DHCPv4 IP reservations.
 func (o *OVN) LogicalSwitchDHCPv4RevervationsGet(switchName OVNSwitch) ([]shared.IPRange, error) {
-	excludeIPsRaw, err := o.nbctl("get", "logical_switch", string(switchName), "other_config:exclude_ips")
+	excludeIPsRaw, err := o.nbctl("--if-exists", "get", "logical_switch", string(switchName), "other_config:exclude_ips")
 	if err != nil {
 		return nil, err
 	}
@@ -759,7 +857,7 @@ func (o *OVN) LogicalSwitchDHCPv4RevervationsGet(switchName OVNSwitch) ([]shared
 	excludeIPsRaw = strings.TrimSpace(excludeIPsRaw)
 
 	// Check if no dynamic IPs set.
-	if excludeIPsRaw == "[]" {
+	if excludeIPsRaw == "" || excludeIPsRaw == "[]" {
 		return []shared.IPRange{}, nil
 	}
 
@@ -858,6 +956,10 @@ func (o *OVN) LogicalSwitchDHCPv4OptionsSet(switchName OVNSwitch, uuid OVNDHCPOp
 		args = append(args, fmt.Sprintf("mtu=%d", opts.MTU))
 	}
 
+	if opts.Netmask != "" {
+		args = append(args, fmt.Sprintf("netmask=%s", opts.Netmask))
+	}
+
 	_, err = o.nbctl(args...)
 	if err != nil {
 		return err
@@ -1069,6 +1171,39 @@ func (o *OVN) LogicalSwitchPorts(switchName OVNSwitch) (map[OVNSwitchPort]OVNSwi
 	return ports, nil
 }
 
+// LogicalSwitchIPs returns a list of IPs associated to each port connected to switch.
+func (o *OVN) LogicalSwitchIPs(switchName OVNSwitch) (map[OVNSwitchPort][]net.IP, error) {
+	output, err := o.nbctl("--format=csv", "--no-headings", "--data=bare", "--colum=name,addresses,dynamic_addresses", "find", "logical_switch_port",
+		fmt.Sprintf("external_ids:%s=%s", ovnExtIDLXDSwitch, switchName),
+	)
+	if err != nil {
+		return nil, err
+	}
+
+	lines := shared.SplitNTrimSpace(strings.TrimSpace(output), "\n", -1, true)
+	portIPs := make(map[OVNSwitchPort][]net.IP, len(lines))
+
+	for _, line := range lines {
+		fields := shared.SplitNTrimSpace(line, ",", -1, true)
+		portName := OVNSwitchPort(fields[0])
+		var ips []net.IP
+
+		// Parse all IPs mentioned in addresses and dynamic_addresses fields.
+		for i := 1; i < len(fields); i++ {
+			for _, address := range shared.SplitNTrimSpace(fields[i], " ", -1, true) {
+				ip := net.ParseIP(address)
+				if ip != nil {
+					ips = append(ips, ip)
+				}
+			}
+		}
+
+		portIPs[portName] = ips
+	}
+
+	return portIPs, nil
+}
+
 // LogicalSwitchPortUUID returns the logical switch port UUID or empty string if port doesn't exist.
 func (o *OVN) LogicalSwitchPortUUID(portName OVNSwitchPort) (OVNSwitchPortUUID, error) {
 	portInfo, err := o.nbctl("--format=csv", "--no-headings", "--data=bare", "--colum=_uuid,name", "find", "logical_switch_port",
@@ -1102,6 +1237,11 @@ func (o *OVN) LogicalSwitchPortAdd(switchName OVNSwitch, portName OVNSwitchPort,
 
 	// Set switch port options if supplied.
 	if opts != nil {
+		// Created nested VLAN port if requested.
+		if opts.Parent != "" {
+			args = append(args, string(opts.Parent), fmt.Sprintf("%d", opts.VLAN))
+		}
+
 		ipStr := make([]string, 0, len(opts.IPs))
 		for _, ip := range opts.IPs {
 			ipStr = append(ipStr, ip.String())
@@ -1125,8 +1265,14 @@ func (o *OVN) LogicalSwitchPortAdd(switchName OVNSwitch, portName OVNSwitchPort,
 		if opts.DHCPv6OptsID != "" {
 			args = append(args, "--", "lsp-set-dhcpv6-options", string(portName), string(opts.DHCPv6OptsID))
 		}
+
+		if opts.Location != "" {
+			args = append(args, "--", "set", "logical_switch_port", string(portName), fmt.Sprintf("external_ids:%s=%s", ovnExtIDLXDLocation, opts.Location))
+		}
 	}
 
+	args = append(args, "--", "set", "logical_switch_port", string(portName), fmt.Sprintf("external_ids:%s=%s", ovnExtIDLXDSwitch, switchName))
+
 	_, err := o.nbctl(args...)
 	if err != nil {
 		return err
@@ -1135,6 +1281,26 @@ func (o *OVN) LogicalSwitchPortAdd(switchName OVNSwitch, portName OVNSwitchPort,
 	return nil
 }
 
+// LogicalSwitchPortIPs returns a list of IPs for a switch port.
+func (o *OVN) LogicalSwitchPortIPs(portName OVNSwitchPort) ([]net.IP, error) {
+	addressesRaw, err := o.nbctl("--format=csv", "--no-headings", "--data=bare", "--column=addresses,dynamic_addresses", "find", "logical_switch_port", fmt.Sprintf("name=%s", string(portName)))
+	if err != nil {
+		return nil, err
+	}
+
+	addresses := strings.Split(strings.Replace(strings.TrimSpace(addressesRaw), ",", " ", 1), " ")
+	ips := make([]net.IP, 0)
+
+	for _, address := range addresses {
+		ip := net.ParseIP(address)
+		if ip != nil {
+			ips = append(ips, ip)
+		}
+	}
+
+	return ips, nil
+}
+
 // LogicalSwitchPortDynamicIPs returns a list of dynamc IPs for a switch port.
 func (o *OVN) LogicalSwitchPortDynamicIPs(portName OVNSwitchPort) ([]net.IP, error) {
 	dynamicAddressesRaw, err := o.nbctl("get", "logical_switch_port", string(portName), "dynamic_addresses")
@@ -1167,19 +1333,35 @@ func (o *OVN) LogicalSwitchPortDynamicIPs(portName OVNSwitchPort) ([]net.IP, err
 	return dynamicIPs, nil
 }
 
+// LogicalSwitchPortLocationGet returns the last set location of a logical switch port.
+func (o *OVN) LogicalSwitchPortLocationGet(portName OVNSwitchPort) (string, error) {
+	location, err := o.nbctl("--if-exists", "get", "logical_switch_port", string(portName), fmt.Sprintf("external-ids:%s", ovnExtIDLXDLocation))
+	if err != nil {
+		return "", err
+	}
+
+	return strings.TrimSpace(location), nil
+}
+
+// LogicalSwitchPortOptionsSet sets the options for a logical switch port.
+func (o *OVN) LogicalSwitchPortOptionsSet(portName OVNSwitchPort, options map[string]string) error {
+	args := []string{"lsp-set-options", string(portName)}
+
+	for key, value := range options {
+		args = append(args, fmt.Sprintf("%s=%s", key, value))
+	}
+
+	_, err := o.nbctl(args...)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
 // LogicalSwitchPortSetDNS sets up the switch port DNS records for the DNS name.
 // Returns the DNS record UUID, IPv4 and IPv6 addresses used for DNS records.
-func (o *OVN) LogicalSwitchPortSetDNS(switchName OVNSwitch, portName OVNSwitchPort, dnsName string, dnsIPv4 net.IP, dnsIPv6 net.IP) (OVNDNSUUID, error) {
-	// Create a list of IPs for the DNS record.
-	dnsIPs := make([]string, 0, 2)
-	if dnsIPv4 != nil {
-		dnsIPs = append(dnsIPs, dnsIPv4.String())
-	}
-
-	if dnsIPv6 != nil {
-		dnsIPs = append(dnsIPs, dnsIPv6.String())
-	}
-
+func (o *OVN) LogicalSwitchPortSetDNS(switchName OVNSwitch, portName OVNSwitchPort, dnsName string, dnsIPs []net.IP) (OVNDNSUUID, error) {
 	// Check if existing DNS record exists for switch port.
 	dnsUUID, err := o.nbctl("--format=csv", "--no-headings", "--data=bare", "--colum=_uuid", "find", "dns",
 		fmt.Sprintf("external_ids:%s=%s", ovnExtIDLXDSwitchPort, portName),
@@ -1195,7 +1377,16 @@ func (o *OVN) LogicalSwitchPortSetDNS(switchName OVNSwitch, portName OVNSwitchPo
 
 	// Only include DNS name record if IPs supplied.
 	if len(dnsIPs) > 0 {
-		cmdArgs = append(cmdArgs, fmt.Sprintf(`records={"%s"="%s"}`, strings.ToLower(dnsName), strings.Join(dnsIPs, " ")))
+		var dnsIPsStr strings.Builder
+		for i, dnsIP := range dnsIPs {
+			if i > 0 {
+				dnsIPsStr.WriteString(" ")
+			}
+
+			dnsIPsStr.WriteString(dnsIP.String())
+		}
+
+		cmdArgs = append(cmdArgs, fmt.Sprintf(`records={"%s"="%s"}`, strings.ToLower(dnsName), dnsIPsStr.String()))
 	}
 
 	dnsUUID = strings.TrimSpace(dnsUUID)
@@ -1434,7 +1625,7 @@ func (o *OVN) ChassisGroupChassisDelete(haChassisGroupName OVNChassisGroup, chas
 		members := shared.SplitNTrimSpace(lines[1], " ", -1, true)
 
 		// Remove chassis from group if exists.
-		if existingChassisGroup == string(haChassisGroupName) && shared.StringInSlice(chassisID, members) {
+		if existingChassisGroup == string(haChassisGroupName) && shared.ValueInSlice(chassisID, members) {
 			_, err := o.nbctl("ha-chassis-group-remove-chassis", string(haChassisGroupName), chassisID)
 			if err != nil {
 				return err
