@@ -4,11 +4,11 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/lxc/lxd/lxd/config"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/validate"
+	"github.com/canonical/lxd/lxd/config"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 // Config holds node-local configuration values for a certain LXD instance.
@@ -98,6 +98,16 @@ func (c *Config) MAASMachine() string {
 	return c.m.GetString("maas.machine")
 }
 
+// StorageBucketsAddress returns the address and port to setup the storage buckets listener on.
+func (c *Config) StorageBucketsAddress() string {
+	objectAddress := c.m.GetString("core.storage_buckets_address")
+	if objectAddress != "" {
+		return util.CanonicalNetworkAddress(objectAddress, shared.HTTPSStorageBucketsDefaultPort)
+	}
+
+	return objectAddress
+}
+
 // StorageBackupsVolume returns the name of the pool/volume to use for storing backup tarballs.
 func (c *Config) StorageBackupsVolume() string {
 	return c.m.GetString("storage.backups_volume")
@@ -108,6 +118,11 @@ func (c *Config) StorageImagesVolume() string {
 	return c.m.GetString("storage.images_volume")
 }
 
+// SyslogSocket returns true if the syslog socket is enabled, otherwise false.
+func (c *Config) SyslogSocket() bool {
+	return c.m.GetBool("core.syslog_socket")
+}
+
 // Dump current configuration keys and their values. Keys with values matching
 // their defaults are omitted.
 func (c *Config) Dump() map[string]any {
@@ -146,30 +161,121 @@ func (c *Config) update(values map[string]any) (map[string]string, error) {
 // ConfigSchema defines available server configuration keys.
 var ConfigSchema = config.Schema{
 	// Network address for this LXD server
+
+	// lxdmeta:generate(entities=server; group=core; key=core.https_address)
+	// See {ref}`server-expose`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Address to bind for the remote API (HTTPS)
 	"core.https_address": {Validator: validate.Optional(validate.IsListenAddress(true, true, false))},
 
 	// Network address for cluster communication
+
+	// lxdmeta:generate(entities=server; group=cluster; key=cluster.https_address)
+	// See {ref}`cluster-https-address`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Address to use for clustering traffic
 	"cluster.https_address": {Validator: validate.Optional(validate.IsListenAddress(true, false, false))},
 
 	// Network address for the BGP server
+
+	// lxdmeta:generate(entities=server; group=core; key=core.bgp_address)
+	// See {ref}`network-bgp`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Address to bind the BGP server to
 	"core.bgp_address": {Validator: validate.Optional(validate.IsListenAddress(true, true, false))},
 
 	// Unique router ID for the BGP server
+
+	// lxdmeta:generate(entities=server; group=core; key=core.bgp_routerid)
+	// The identifier must be formatted as an IPv4 address.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: A unique identifier for the BGP server
 	"core.bgp_routerid": {Validator: validate.Optional(validate.IsNetworkAddressV4)},
 
 	// Network address for the debug server
+
+	// lxdmeta:generate(entities=server; group=core; key=core.debug_address)
+	//
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Address to bind the `pprof` debug server to (HTTP)
 	"core.debug_address": {Validator: validate.Optional(validate.IsListenAddress(true, true, false))},
 
 	// Network address for the DNS server
+
+	// lxdmeta:generate(entities=server; group=core; key=core.dns_address)
+	// See {ref}`network-dns-server`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Address to bind the authoritative DNS server to
 	"core.dns_address": {Validator: validate.Optional(validate.IsListenAddress(true, true, false))},
 
 	// Network address for the metrics server
+
+	// lxdmeta:generate(entities=server; group=core; key=core.metrics_address)
+	// See {ref}`metrics`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Address to bind the metrics server to (HTTPS)
 	"core.metrics_address": {Validator: validate.Optional(validate.IsListenAddress(true, true, false))},
 
+	// Network address for the storage buckets server
+
+	// lxdmeta:generate(entities=server; group=core; key=core.storage_buckets_address)
+	// See {ref}`howto-storage-buckets`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Address to bind the storage object server to (HTTPS)
+	"core.storage_buckets_address": {Validator: validate.Optional(validate.IsListenAddress(true, true, false))},
+
+	// Syslog socket
+
+	// lxdmeta:generate(entities=server; group=core; key=core.syslog_socket)
+	// Set this option to `true` to enable the syslog unixgram socket to receive log messages from external processes.
+	// ---
+	//  type: bool
+	//  scope: local
+	//  defaultdesc: `false`
+	//  shortdesc: Whether to enable the syslog unixgram socket listener
+	"core.syslog_socket": {Validator: validate.Optional(validate.IsBool), Type: config.Bool},
+
 	// MAAS machine this LXD instance is associated with
+
+	// lxdmeta:generate(entities=server; group=miscellaneous; key=maas.machine)
+	//
+	// ---
+	//  type: string
+	//  scope: local
+	//  defaultdesc: host name
+	//  shortdesc: Name of this LXD host in MAAS
 	"maas.machine": {},
 
 	// Storage volumes to store backups/images on
+
+	// lxdmeta:generate(entities=server; group=miscellaneous; key=storage.backups_volume)
+	// Specify the volume using the syntax `POOL/VOLUME`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Volume to use to store backup tarballs
 	"storage.backups_volume": {},
-	"storage.images_volume":  {},
+	// lxdmeta:generate(entities=server; group=miscellaneous; key=storage.images_volume)
+	// Specify the volume using the syntax `POOL/VOLUME`.
+	// ---
+	//  type: string
+	//  scope: local
+	//  shortdesc: Volume to use to store the image tarballs
+	"storage.images_volume": {},
 }
