@@ -11,21 +11,22 @@ import (
 
 	"github.com/gorilla/mux"
 
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/db"
-	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/operationtype"
-	"github.com/lxc/lxd/lxd/lifecycle"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/rbac"
-	"github.com/lxc/lxd/lxd/request"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/task"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/auth"
+	"github.com/canonical/lxd/lxd/cluster"
+	"github.com/canonical/lxd/lxd/db"
+	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/operationtype"
+	"github.com/canonical/lxd/lxd/lifecycle"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/task"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 var operationCmd = APIEndpoint{
@@ -65,7 +66,7 @@ func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdown
 				logger.Error("Failed cleaning up operations")
 			}
 
-			return nil
+			return nil //nolint:revive // False positive: raises "return in a defer function has no effect".
 		})
 	}()
 
@@ -135,39 +136,41 @@ func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdown
 
 // swagger:operation GET /1.0/operations/{id} operations operation_get
 //
-// Get the operation state
+//	Get the operation state
 //
-// Gets the operation state.
+//	Gets the operation state.
 //
-// ---
-// produces:
-//   - application/json
-// responses:
-//   "200":
-//     description: Operation
-//     schema:
-//       type: object
-//       description: Sync response
-//       properties:
-//         type:
-//           type: string
-//           description: Response type
-//           example: sync
-//         status:
-//           type: string
-//           description: Status description
-//           example: Success
-//         status_code:
-//           type: integer
-//           description: Status code
-//           example: 200
-//         metadata:
-//           $ref: "#/definitions/Operation"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	responses:
+//	  "200":
+//	    description: Operation
+//	    schema:
+//	      type: object
+//	      description: Sync response
+//	      properties:
+//	        type:
+//	          type: string
+//	          description: Response type
+//	          example: sync
+//	        status:
+//	          type: string
+//	          description: Status description
+//	          example: Success
+//	        status_code:
+//	          type: integer
+//	          description: Status code
+//	          example: 200
+//	        metadata:
+//	          $ref: "#/definitions/Operation"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func operationGet(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	id, err := url.PathUnescape(mux.Vars(r)["id"])
 	if err != nil {
 		return response.SmartError(err)
@@ -188,7 +191,7 @@ func operationGet(d *Daemon, r *http.Request) response.Response {
 
 	// Then check if the query is from an operation on another node, and, if so, forward it
 	var address string
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		filter := dbCluster.OperationFilter{UUID: &id}
 		ops, err := dbCluster.GetOperations(ctx, tx.Tx(), filter)
 		if err != nil {
@@ -212,7 +215,7 @@ func operationGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, d.endpoints.NetworkCert(), d.serverCert(), r, false)
+	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -222,23 +225,25 @@ func operationGet(d *Daemon, r *http.Request) response.Response {
 
 // swagger:operation DELETE /1.0/operations/{id} operations operation_delete
 //
-// Cancel the operation
+//	Cancel the operation
 //
-// Cancels the operation if supported.
+//	Cancels the operation if supported.
 //
-// ---
-// produces:
-//   - application/json
-// responses:
-//   "200":
-//     $ref: "#/responses/EmptySyncResponse"
-//   "400":
-//     $ref: "#/responses/BadRequest"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	responses:
+//	  "200":
+//	    $ref: "#/responses/EmptySyncResponse"
+//	  "400":
+//	    $ref: "#/responses/BadRequest"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func operationDelete(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	id, err := url.PathUnescape(mux.Vars(r)["id"])
 	if err != nil {
 		return response.SmartError(err)
@@ -248,13 +253,49 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 	op, err := operations.OperationGetInternal(id)
 	if err == nil {
 		projectName := op.Project()
-		if op.Permission() != "" {
-			if projectName == "" {
-				projectName = project.Default
+		if projectName == "" {
+			projectName = api.ProjectDefaultName
+		}
+
+		// Separate resources by entity type. If there are multiple entries of a particular entity type we can reduce
+		// the number of calls to the authorizer.
+		objectType, entitlement := op.Permission()
+		urlsByEntityType := make(map[entity.Type][]api.URL)
+		if objectType != "" {
+			for _, v := range op.Resources() {
+				for _, u := range v {
+					entityType, _, _, _, err := entity.ParseURL(u.URL)
+					if err != nil {
+						return response.InternalError(fmt.Errorf("Failed to parse operation resource entity URL: %w", err))
+					}
+
+					urlsByEntityType[entityType] = append(urlsByEntityType[entityType], u)
+				}
+			}
+		}
+
+		for entityType, urls := range urlsByEntityType {
+			// If only one entry of this type, check directly.
+			if len(urls) == 1 {
+				err := s.Authorizer.CheckPermission(r.Context(), r, &urls[0], entitlement)
+				if err != nil {
+					return response.SmartError(err)
+				}
+
+				continue
 			}
 
-			if !rbac.UserHasPermission(r, projectName, op.Permission()) {
-				return response.Forbidden(nil)
+			// Otherwise get a permission checker for the entity type.
+			hasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), r, entitlement, entityType)
+			if err != nil {
+				return response.SmartError(err)
+			}
+
+			// Check each URL.
+			for _, u := range urls {
+				if !hasPermission(&u) {
+					return response.Forbidden(nil)
+				}
 			}
 		}
 
@@ -263,14 +304,14 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 			return response.BadRequest(err)
 		}
 
-		d.State().Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(op, request.CreateRequestor(r), nil))
+		s.Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(op, request.CreateRequestor(r), nil))
 
 		return response.EmptySyncResponse
 	}
 
 	// Then check if the query is from an operation on another node, and, if so, forward it
 	var address string
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		filter := dbCluster.OperationFilter{UUID: &id}
 		ops, err := dbCluster.GetOperations(ctx, tx.Tx(), filter)
 		if err != nil {
@@ -294,7 +335,7 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, d.endpoints.NetworkCert(), d.serverCert(), r, false)
+	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -303,7 +344,7 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 }
 
 // operationCancel cancels an operation that exists on any member.
-func operationCancel(d *Daemon, r *http.Request, projectName string, op *api.Operation) error {
+func operationCancel(s *state.State, r *http.Request, projectName string, op *api.Operation) error {
 	// Check if operation is local and if so, cancel it.
 	localOp, _ := operations.OperationGetInternal(op.ID)
 	if localOp != nil {
@@ -314,7 +355,7 @@ func operationCancel(d *Daemon, r *http.Request, projectName string, op *api.Ope
 			}
 		}
 
-		d.State().Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(localOp, request.CreateRequestor(r), nil))
+		s.Events.SendLifecycle(projectName, lifecycle.OperationCancelled.Event(localOp, request.CreateRequestor(r), nil))
 
 		return nil
 	}
@@ -322,7 +363,7 @@ func operationCancel(d *Daemon, r *http.Request, projectName string, op *api.Ope
 	// If not found locally, try connecting to remote member to delete it.
 	var memberAddress string
 	var err error
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		filter := dbCluster.OperationFilter{UUID: &op.ID}
 		ops, err := dbCluster.GetOperations(ctx, tx.Tx(), filter)
 		if err != nil {
@@ -346,7 +387,7 @@ func operationCancel(d *Daemon, r *http.Request, projectName string, op *api.Ope
 		return err
 	}
 
-	client, err := cluster.Connect(memberAddress, d.endpoints.NetworkCert(), d.serverCert(), r, true)
+	client, err := cluster.Connect(memberAddress, s.Endpoints.NetworkCert(), s.ServerCert(), r, true)
 	if err != nil {
 		return fmt.Errorf("Failed to connect to %q: %w", memberAddress, err)
 	}
@@ -361,91 +402,127 @@ func operationCancel(d *Daemon, r *http.Request, projectName string, op *api.Ope
 
 // swagger:operation GET /1.0/operations operations operations_get
 //
-// Get the operations
+//  Get the operations
 //
-// Returns a dict of operation type to operation list (URLs).
+//  Returns a JSON object of operation type to operation list (URLs).
 //
-// ---
-// produces:
-//   - application/json
-// responses:
-//   "200":
-//     description: API endpoints
-//     schema:
-//       type: object
-//       description: Sync response
-//       properties:
-//         type:
-//           type: string
-//           description: Response type
-//           example: sync
-//         status:
-//           type: string
-//           description: Status description
-//           example: Success
-//         status_code:
-//           type: integer
-//           description: Status code
-//           example: 200
-//         metadata:
-//           type: object
-//           additionalProperties:
-//             type: array
-//             items:
-//               type: string
-//           description: Dict of operation types to operation URLs
-//           example: |-
-//             {
-//               "running": [
-//                 "/1.0/operations/6916c8a6-9b7d-4abd-90b3-aedfec7ec7da"
-//               ]
-//             }
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//  ---
+//  produces:
+//    - application/json
+//  parameters:
+//    - in: query
+//      name: project
+//      description: Project name
+//      type: string
+//      example: default
+//    - in: query
+//      name: all-projects
+//      description: Retrieve operations from all projects
+//      type: boolean
+//  responses:
+//    "200":
+//      description: API endpoints
+//      schema:
+//        type: object
+//        description: Sync response
+//        properties:
+//          type:
+//            type: string
+//            description: Response type
+//            example: sync
+//          status:
+//            type: string
+//            description: Status description
+//            example: Success
+//          status_code:
+//            type: integer
+//            description: Status code
+//            example: 200
+//          metadata:
+//            type: object
+//            additionalProperties:
+//              type: array
+//              items:
+//                type: string
+//            description: JSON object of operation types to operation URLs
+//            example: |-
+//              {
+//                "running": [
+//                  "/1.0/operations/6916c8a6-9b7d-4abd-90b3-aedfec7ec7da"
+//                ]
+//              }
+//    "403":
+//      $ref: "#/responses/Forbidden"
+//    "500":
+//      $ref: "#/responses/InternalServerError"
 
 // swagger:operation GET /1.0/operations?recursion=1 operations operations_get_recursion1
 //
-// Get the operations
+//	Get the operations
 //
-// Returns a list of operations (structs).
+//	Returns a list of operations (structs).
 //
-// ---
-// produces:
-//   - application/json
-// responses:
-//   "200":
-//     description: API endpoints
-//     schema:
-//       type: object
-//       description: Sync response
-//       properties:
-//         type:
-//           type: string
-//           description: Response type
-//           example: sync
-//         status:
-//           type: string
-//           description: Status description
-//           example: Success
-//         status_code:
-//           type: integer
-//           description: Status code
-//           example: 200
-//         metadata:
-//           type: array
-//           description: List of operations
-//           items:
-//             $ref: "#/definitions/Operation"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: project
+//	    description: Project name
+//	    type: string
+//	    example: default
+//	  - in: query
+//	    name: all-projects
+//	    description: Retrieve operations from all projects
+//	    type: boolean
+//	responses:
+//	  "200":
+//	    description: API endpoints
+//	    schema:
+//	      type: object
+//	      description: Sync response
+//	      properties:
+//	        type:
+//	          type: string
+//	          description: Response type
+//	          example: sync
+//	        status:
+//	          type: string
+//	          description: Status description
+//	          example: Success
+//	        status_code:
+//	          type: integer
+//	          description: Status code
+//	          example: 200
+//	        metadata:
+//	          type: array
+//	          description: List of operations
+//	          items:
+//	            $ref: "#/definitions/Operation"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func operationsGet(d *Daemon, r *http.Request) response.Response {
-	projectName := projectParam(r)
+	s := d.State()
+
+	projectName := request.QueryParam(r, "project")
+	allProjects := shared.IsTrue(request.QueryParam(r, "all-projects"))
 	recursion := util.IsRecursionRequest(r)
 
+	if allProjects && projectName != "" {
+		return response.SmartError(
+			api.StatusErrorf(http.StatusBadRequest, "Cannot specify a project when requesting all projects"),
+		)
+	} else if !allProjects && projectName == "" {
+		projectName = api.ProjectDefaultName
+	}
+
+	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), r, auth.EntitlementCanViewOperations, entity.TypeProject)
+	if err != nil {
+		return response.InternalError(fmt.Errorf("Failed to get operation permission checker: %w", err))
+	}
+
 	localOperationURLs := func() (shared.Jmap, error) {
 		// Get all the operations.
 		localOps := operations.Clone()
@@ -454,7 +531,11 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		body := shared.Jmap{}
 
 		for _, v := range localOps {
-			if v.Project() != "" && v.Project() != projectName {
+			if !allProjects && v.Project() != "" && v.Project() != projectName {
+				continue
+			}
+
+			if !userHasPermission(entity.ProjectURL(v.Project())) {
 				continue
 			}
 
@@ -478,7 +559,11 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		body := shared.Jmap{}
 
 		for _, v := range localOps {
-			if v.Project() != "" && v.Project() != projectName {
+			if !allProjects && v.Project() != "" && v.Project() != projectName {
+				continue
+			}
+
+			if !userHasPermission(entity.ProjectURL(v.Project())) {
 				continue
 			}
 
@@ -523,7 +608,6 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 
 	// Start with local operations.
 	var md shared.Jmap
-	var err error
 
 	if recursion {
 		md, err = localOperations()
@@ -537,32 +621,25 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	// Check if clustered.
-	clustered, err := cluster.Enabled(d.db.Node)
-	if err != nil {
-		return response.InternalError(err)
-	}
-
 	// If not clustered, then just return local operations.
-	if !clustered {
+	if !s.ServerClustered {
 		return response.SyncResponse(true, md)
 	}
 
 	// Get all nodes with running operations in this project.
 	var membersWithOps []string
-	var offlineThreshold time.Duration
 	var members []db.NodeInfo
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
-		membersWithOps, err = tx.GetNodesWithOperations(ctx, projectName)
-		if err != nil {
-			return fmt.Errorf("Failed getting members with operations: %w", err)
+		if allProjects {
+			membersWithOps, err = tx.GetAllNodesWithOperations(ctx)
+		} else {
+			membersWithOps, err = tx.GetNodesWithOperations(ctx, projectName)
 		}
 
-		offlineThreshold, err = tx.GetNodeOfflineThreshold(ctx)
 		if err != nil {
-			return fmt.Errorf("Failed getting member offline threshold value: %w", err)
+			return fmt.Errorf("Failed getting members with operations: %w", err)
 		}
 
 		members, err = tx.GetNodes(ctx)
@@ -577,7 +654,8 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Get local address.
-	localClusterAddress := d.State().LocalConfig.ClusterAddress()
+	localClusterAddress := s.LocalConfig.ClusterAddress()
+	offlineThreshold := s.GlobalConfig.OfflineThreshold()
 
 	memberOnline := func(memberAddress string) bool {
 		for _, member := range members {
@@ -594,7 +672,7 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		return false
 	}
 
-	networkCert := d.endpoints.NetworkCert()
+	networkCert := s.Endpoints.NetworkCert()
 	for _, memberAddress := range membersWithOps {
 		if memberAddress == localClusterAddress {
 			continue
@@ -605,13 +683,19 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		}
 
 		// Connect to the remote server. Use notify=true to only get local operations on remote member.
-		client, err := cluster.Connect(memberAddress, networkCert, d.serverCert(), r, true)
+		client, err := cluster.Connect(memberAddress, networkCert, s.ServerCert(), r, true)
 		if err != nil {
 			return response.SmartError(fmt.Errorf("Failed connecting to member %q: %w", memberAddress, err))
 		}
 
 		// Get operation data.
-		ops, err := client.UseProject(projectName).GetOperations()
+		var ops []api.Operation
+		if allProjects {
+			ops, err = client.GetOperationsAllProjects()
+		} else {
+			ops, err = client.UseProject(projectName).GetOperations()
+		}
+
 		if err != nil {
 			logger.Warn("Failed getting operations from member", logger.Ctx{"address": memberAddress, "err": err})
 			continue
@@ -643,7 +727,7 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 }
 
 // operationsGetByType gets all operations for a project and type.
-func operationsGetByType(d *Daemon, r *http.Request, projectName string, opType operationtype.Type) ([]*api.Operation, error) {
+func operationsGetByType(s *state.State, r *http.Request, projectName string, opType operationtype.Type) ([]*api.Operation, error) {
 	ops := make([]*api.Operation, 0)
 
 	// Get local operations for project.
@@ -660,26 +744,16 @@ func operationsGetByType(d *Daemon, r *http.Request, projectName string, opType
 		ops = append(ops, apiOp)
 	}
 
-	// Check if clustered.
-	clustered, err := cluster.Enabled(d.db.Node)
-	if err != nil {
-		return nil, err
-	}
-
 	// Return just local operations if not clustered.
-	if !clustered {
+	if !s.ServerClustered {
 		return ops, nil
 	}
 
 	// Get all operations of the specified type in project.
-	var offlineThreshold time.Duration
 	var members []db.NodeInfo
 	memberOps := make(map[string]map[string]dbCluster.Operation)
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		offlineThreshold, err = tx.GetNodeOfflineThreshold(ctx)
-		if err != nil {
-			return fmt.Errorf("Failed getting member offline threshold value: %w", err)
-		}
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
 
 		members, err = tx.GetNodes(ctx)
 		if err != nil {
@@ -707,7 +781,8 @@ func operationsGetByType(d *Daemon, r *http.Request, projectName string, opType
 	}
 
 	// Get local address.
-	localClusterAddress := d.State().LocalConfig.ClusterAddress()
+	localClusterAddress := s.LocalConfig.ClusterAddress()
+	offlineThreshold := s.GlobalConfig.OfflineThreshold()
 
 	memberOnline := func(memberAddress string) bool {
 		for _, member := range members {
@@ -724,8 +799,8 @@ func operationsGetByType(d *Daemon, r *http.Request, projectName string, opType
 		return false
 	}
 
-	networkCert := d.endpoints.NetworkCert()
-	serverCert := d.serverCert()
+	networkCert := s.Endpoints.NetworkCert()
+	serverCert := s.ServerCert()
 	for memberAddress := range memberOps {
 		if memberAddress == localClusterAddress {
 			continue
@@ -765,93 +840,95 @@ func operationsGetByType(d *Daemon, r *http.Request, projectName string, opType
 
 // swagger:operation GET /1.0/operations/{id}/wait?public operations operation_wait_get_untrusted
 //
-// Wait for the operation
+//  Wait for the operation
 //
-// Waits for the operation to reach a final state (or timeout) and retrieve its final state.
+//  Waits for the operation to reach a final state (or timeout) and retrieve its final state.
 //
-// When accessed by an untrusted user, the secret token must be provided.
+//  When accessed by an untrusted user, the secret token must be provided.
 //
-// ---
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: secret
-//     description: Authentication token
-//     type: string
-//     example: random-string
-//   - in: query
-//     name: timeout
-//     description: Timeout in seconds (-1 means never)
-//     type: integer
-//     example: -1
-// responses:
-//   "200":
-//     description: Operation
-//     schema:
-//       type: object
-//       description: Sync response
-//       properties:
-//         type:
-//           type: string
-//           description: Response type
-//           example: sync
-//         status:
-//           type: string
-//           description: Status description
-//           example: Success
-//         status_code:
-//           type: integer
-//           description: Status code
-//           example: 200
-//         metadata:
-//           $ref: "#/definitions/Operation"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//  ---
+//  produces:
+//    - application/json
+//  parameters:
+//    - in: query
+//      name: secret
+//      description: Authentication token
+//      type: string
+//      example: random-string
+//    - in: query
+//      name: timeout
+//      description: Timeout in seconds (-1 means never)
+//      type: integer
+//      example: -1
+//  responses:
+//    "200":
+//      description: Operation
+//      schema:
+//        type: object
+//        description: Sync response
+//        properties:
+//          type:
+//            type: string
+//            description: Response type
+//            example: sync
+//          status:
+//            type: string
+//            description: Status description
+//            example: Success
+//          status_code:
+//            type: integer
+//            description: Status code
+//            example: 200
+//          metadata:
+//            $ref: "#/definitions/Operation"
+//    "403":
+//      $ref: "#/responses/Forbidden"
+//    "500":
+//      $ref: "#/responses/InternalServerError"
 
 // swagger:operation GET /1.0/operations/{id}/wait operations operation_wait_get
 //
-// Wait for the operation
+//	Wait for the operation
 //
-// Waits for the operation to reach a final state (or timeout) and retrieve its final state.
+//	Waits for the operation to reach a final state (or timeout) and retrieve its final state.
 //
-// ---
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: timeout
-//     description: Timeout in seconds (-1 means never)
-//     type: integer
-//     example: -1
-// responses:
-//   "200":
-//     description: Operation
-//     schema:
-//       type: object
-//       description: Sync response
-//       properties:
-//         type:
-//           type: string
-//           description: Response type
-//           example: sync
-//         status:
-//           type: string
-//           description: Status description
-//           example: Success
-//         status_code:
-//           type: integer
-//           description: Status code
-//           example: 200
-//         metadata:
-//           $ref: "#/definitions/Operation"
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: timeout
+//	    description: Timeout in seconds (-1 means never)
+//	    type: integer
+//	    example: -1
+//	responses:
+//	  "200":
+//	    description: Operation
+//	    schema:
+//	      type: object
+//	      description: Sync response
+//	      properties:
+//	        type:
+//	          type: string
+//	          description: Response type
+//	          example: sync
+//	        status:
+//	          type: string
+//	          description: Status description
+//	          example: Success
+//	        status_code:
+//	          type: integer
+//	          description: Status code
+//	          example: 200
+//	        metadata:
+//	          $ref: "#/definitions/Operation"
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func operationWaitGet(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	id, err := url.PathUnescape(mux.Vars(r)["id"])
 	if err != nil {
 		return response.SmartError(err)
@@ -859,7 +936,11 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 
 	secret := r.FormValue("secret")
 
-	trusted, _, _, _ := d.Authenticate(nil, r)
+	trusted, err := request.GetCtxValue[bool](r.Context(), request.CtxTrusted)
+	if err != nil {
+		return response.SmartError(fmt.Errorf("Failed to get authentication status: %w", err))
+	}
+
 	if !trusted && secret == "" {
 		return response.Forbidden(nil)
 	}
@@ -886,24 +967,42 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 			ctx, cancel = context.WithCancel(r.Context())
 		}
 
-		defer cancel()
+		waitResponse := func(w http.ResponseWriter) error {
+			defer cancel()
 
-		_, err = op.Wait(ctx)
-		if err != nil {
-			return response.InternalError(err)
+			// Write header to avoid client side timeouts.
+			w.Header().Set("Connection", "keep-alive")
+			w.Header().Set("Content-Type", "application/json")
+			w.Header().Set("X-Content-Type-Options", "nosniff")
+			w.WriteHeader(http.StatusOK)
+			f, ok := w.(http.Flusher)
+			if ok {
+				f.Flush()
+			}
+
+			// Wait for the operation.
+			err = op.Wait(ctx)
+			if err != nil {
+				_ = response.SmartError(err).Render(w)
+				return nil
+			}
+
+			_, body, err := op.Render()
+			if err != nil {
+				_ = response.SmartError(err).Render(w)
+				return nil
+			}
+
+			_ = response.SyncResponse(true, body).Render(w)
+			return nil
 		}
 
-		_, body, err := op.Render()
-		if err != nil {
-			return response.SmartError(err)
-		}
-
-		return response.SyncResponse(true, body)
+		return response.ManualResponse(waitResponse)
 	}
 
 	// Then check if the query is from an operation on another node, and, if so, forward it
 	var address string
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		filter := dbCluster.OperationFilter{UUID: &id}
 		ops, err := dbCluster.GetOperations(ctx, tx.Tx(), filter)
 		if err != nil {
@@ -927,7 +1026,7 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, d.endpoints.NetworkCert(), d.serverCert(), r, false)
+	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -940,6 +1039,7 @@ type operationWebSocket struct {
 	op  *operations.Operation
 }
 
+// Render implements response.Response for operationWebSocket.
 func (r *operationWebSocket) Render(w http.ResponseWriter) error {
 	chanErr, err := r.op.Connect(r.req, w)
 	if err != nil {
@@ -950,6 +1050,7 @@ func (r *operationWebSocket) Render(w http.ResponseWriter) error {
 	return err
 }
 
+// String implements fmt.Stringer for operationWebSocket.
 func (r *operationWebSocket) String() string {
 	_, md, err := r.op.Render()
 	if err != nil {
@@ -961,59 +1062,61 @@ func (r *operationWebSocket) String() string {
 
 // swagger:operation GET /1.0/operations/{id}/websocket?public operations operation_websocket_get_untrusted
 //
-// Get the websocket stream
+//  Get the websocket stream
 //
-// Connects to an associated websocket stream for the operation.
-// This should almost never be done directly by a client, instead it's
-// meant for LXD to LXD communication with the client only relaying the
-// connection information to the servers.
+//  Connects to an associated websocket stream for the operation.
+//  This should almost never be done directly by a client, instead it's
+//  meant for LXD to LXD communication with the client only relaying the
+//  connection information to the servers.
 //
-// The untrusted endpoint is used by the target server to connect to the source server.
-// Authentication is performed through the secret token.
+//  The untrusted endpoint is used by the target server to connect to the source server.
+//  Authentication is performed through the secret token.
 //
-// ---
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: secret
-//     description: Authentication token
-//     type: string
-//     example: random-string
-// responses:
-//   "200":
-//     description: Websocket operation messages (dependent on operation)
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//  ---
+//  produces:
+//    - application/json
+//  parameters:
+//    - in: query
+//      name: secret
+//      description: Authentication token
+//      type: string
+//      example: random-string
+//  responses:
+//    "200":
+//      description: Websocket operation messages (dependent on operation)
+//    "403":
+//      $ref: "#/responses/Forbidden"
+//    "500":
+//      $ref: "#/responses/InternalServerError"
 
 // swagger:operation GET /1.0/operations/{id}/websocket operations operation_websocket_get
 //
-// Get the websocket stream
+//	Get the websocket stream
 //
-// Connects to an associated websocket stream for the operation.
-// This should almost never be done directly by a client, instead it's
-// meant for LXD to LXD communication with the client only relaying the
-// connection information to the servers.
+//	Connects to an associated websocket stream for the operation.
+//	This should almost never be done directly by a client, instead it's
+//	meant for LXD to LXD communication with the client only relaying the
+//	connection information to the servers.
 //
-// ---
-// produces:
-//   - application/json
-// parameters:
-//   - in: query
-//     name: secret
-//     description: Authentication token
-//     type: string
-//     example: random-string
-// responses:
-//   "200":
-//     description: Websocket operation messages (dependent on operation)
-//   "403":
-//     $ref: "#/responses/Forbidden"
-//   "500":
-//     $ref: "#/responses/InternalServerError"
+//	---
+//	produces:
+//	  - application/json
+//	parameters:
+//	  - in: query
+//	    name: secret
+//	    description: Authentication token
+//	    type: string
+//	    example: random-string
+//	responses:
+//	  "200":
+//	    description: Websocket operation messages (dependent on operation)
+//	  "403":
+//	    $ref: "#/responses/Forbidden"
+//	  "500":
+//	    $ref: "#/responses/InternalServerError"
 func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
+	s := d.State()
+
 	id, err := url.PathUnescape(mux.Vars(r)["id"])
 	if err != nil {
 		return response.SmartError(err)
@@ -1028,11 +1131,11 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 	// Then check if the query is from an operation on another node, and, if so, forward it
 	secret := r.FormValue("secret")
 	if secret == "" {
-		return response.BadRequest(fmt.Errorf("missing secret"))
+		return response.BadRequest(fmt.Errorf("Missing websocket secret"))
 	}
 
 	var address string
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		filter := dbCluster.OperationFilter{UUID: &id}
 		ops, err := dbCluster.GetOperations(ctx, tx.Tx(), filter)
 		if err != nil {
@@ -1056,7 +1159,7 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	client, err := cluster.Connect(address, d.endpoints.NetworkCert(), d.serverCert(), r, false)
+	client, err := cluster.Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1071,7 +1174,9 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 
 func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
 	f := func(ctx context.Context) {
-		localClusterAddress := d.State().LocalConfig.ClusterAddress()
+		s := d.State()
+
+		localClusterAddress := s.LocalConfig.ClusterAddress()
 
 		leader, err := d.gateway.LeaderAddress()
 		if err != nil {
@@ -1089,22 +1194,26 @@ func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
 		}
 
 		opRun := func(op *operations.Operation) error {
-			return autoRemoveOrphanedOperations(ctx, d)
+			return autoRemoveOrphanedOperations(ctx, s)
 		}
 
-		op, err := operations.OperationCreate(d.State(), "", operations.OperationClassTask, operationtype.RemoveOrphanedOperations, nil, nil, opRun, nil, nil, nil)
+		op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.RemoveOrphanedOperations, nil, nil, opRun, nil, nil, nil)
 		if err != nil {
-			logger.Error("Failed to start remove orphaned operations operation", logger.Ctx{"err": err})
+			logger.Error("Failed creating remove orphaned operations operation", logger.Ctx{"err": err})
 			return
 		}
 
 		err = op.Start()
 		if err != nil {
-			logger.Error("Failed to remove orphaned operations", logger.Ctx{"err": err})
+			logger.Error("Failed starting remove orphaned operations operation", logger.Ctx{"err": err})
 			return
 		}
 
-		_, _ = op.Wait(ctx)
+		err = op.Wait(ctx)
+		if err != nil {
+			logger.Error("Failed removing orphaned operations", logger.Ctx{"err": err})
+			return
+		}
 	}
 
 	return f, task.Hourly()
@@ -1114,16 +1223,12 @@ func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
 // behind if a cluster member abruptly becomes unreachable. If the affected cluster members comes
 // back online, these operations won't be cleaned up. We therefore need to periodically clean up
 // such operations.
-func autoRemoveOrphanedOperations(ctx context.Context, d *Daemon) error {
+func autoRemoveOrphanedOperations(ctx context.Context, s *state.State) error {
 	logger.Debug("Removing orphaned operations across the cluster")
 
-	err := d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		// Get offline threshold.
-		offlineThreshold, err := tx.GetNodeOfflineThreshold(ctx)
-		if err != nil {
-			return fmt.Errorf("Load offline threshold config: %w", err)
-		}
+	offlineThreshold := s.GlobalConfig.OfflineThreshold()
 
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		members, err := tx.GetNodes(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed getting cluster members: %w", err)
