@@ -4,81 +4,88 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared/api"
 )
 
-func doProfileUpdate(d *Daemon, p api.Project, profileName string, id int64, profile *api.Profile, req api.ProfilePut) error {
+func doProfileUpdate(s *state.State, p api.Project, profileName string, id int64, profile *api.Profile, req api.ProfilePut) error {
 	// Check project limits.
-	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		return project.AllowProfileUpdate(tx, p.Name, profileName, req)
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return project.AllowProfileUpdate(s.GlobalConfig, tx, p.Name, profileName, req)
 	})
 	if err != nil {
 		return err
 	}
 
 	// Quick checks.
-	err = instance.ValidConfig(d.os, req.Config, false, instancetype.Any)
+	err = instance.ValidConfig(s.OS, req.Config, false, instancetype.Any)
 	if err != nil {
 		return err
 	}
 
 	// Profiles can be applied to any instance type, so just use instancetype.Any type for validation so that
 	// instance type specific validation checks are not performed.
-	err = instance.ValidDevices(d.State(), p, instancetype.Any, deviceConfig.NewDevices(req.Devices), nil)
+	err = instance.ValidDevices(s, p, instancetype.Any, deviceConfig.NewDevices(req.Devices), nil)
 	if err != nil {
 		return err
 	}
 
-	insts, projects, err := getProfileInstancesInfo(d.db.Cluster, p.Name, profileName)
+	insts, projects, err := getProfileInstancesInfo(s.DB.Cluster, p.Name, profileName)
 	if err != nil {
 		return fmt.Errorf("Failed to query instances associated with profile %q: %w", profileName, err)
 	}
 
 	// Check if the root disk device's pool would be changed or removed and prevent that if there are instances
 	// using that root disk device.
-	oldProfileRootDiskDeviceKey, oldProfileRootDiskDevice, _ := shared.GetRootDiskDevice(profile.Devices)
-	_, newProfileRootDiskDevice, _ := shared.GetRootDiskDevice(req.Devices)
+	oldProfileRootDiskDeviceKey, oldProfileRootDiskDevice, _ := instancetype.GetRootDiskDevice(profile.Devices)
+	_, newProfileRootDiskDevice, _ := instancetype.GetRootDiskDevice(req.Devices)
 	if len(insts) > 0 && oldProfileRootDiskDevice["pool"] != "" && newProfileRootDiskDevice["pool"] == "" || (oldProfileRootDiskDevice["pool"] != newProfileRootDiskDevice["pool"]) {
 		// Check for instances using the device.
 		for _, inst := range insts {
 			// Check if the device is locally overridden.
-			k, v, _ := shared.GetRootDiskDevice(inst.Devices.CloneNative())
+			k, v, _ := instancetype.GetRootDiskDevice(inst.Devices.CloneNative())
 			if k != "" && v["pool"] != "" {
 				continue
 			}
 
-			// Check what profile the device comes from by working backwards along the profiles list.
-			for i := len(inst.Profiles) - 1; i >= 0; i-- {
-				_, profile, err := d.db.Cluster.GetProfile(p.Name, inst.Profiles[i].Name)
-				if err != nil {
-					return err
-				}
-
-				// Check if we find a match for the device.
-				_, ok := profile.Devices[oldProfileRootDiskDeviceKey]
-				if ok {
-					// Found the profile.
-					if inst.Profiles[i].Name == profileName {
-						// If it's the current profile, then we can't modify that root device.
-						return fmt.Errorf("At least one instance relies on this profile's root disk device")
+			err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				// Check what profile the device comes from by working backwards along the profiles list.
+				for i := len(inst.Profiles) - 1; i >= 0; i-- {
+					_, profile, err := tx.GetProfile(ctx, p.Name, inst.Profiles[i].Name)
+					if err != nil {
+						return err
 					}
 
-					// If it's not, then move on to the next instance.
-					break
+					// Check if we find a match for the device.
+					_, ok := profile.Devices[oldProfileRootDiskDeviceKey]
+					if ok {
+						// Found the profile.
+						if inst.Profiles[i].Name == profileName {
+							// If it's the current profile, then we can't modify that root device.
+							return fmt.Errorf("At least one instance relies on this profile's root disk device")
+						}
+
+						// If it's not, then move on to the next instance.
+						break
+					}
 				}
+
+				return nil
+			})
+			if err != nil {
+				return err
 			}
 		}
 	}
 
 	// Update the database.
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		devices, err := cluster.APIToDevices(req.Devices)
 		if err != nil {
 			return err
@@ -124,17 +131,15 @@ func doProfileUpdate(d *Daemon, p api.Project, profileName string, id int64, pro
 	}
 
 	// Update all the instances on this node using the profile. Must be done after db.TxCommit due to DB lock.
-	serverName := d.State().ServerName
-
 	failures := map[*db.InstanceArgs]error{}
 	for _, it := range insts {
 		inst := it // Local var for instance pointer.
 
-		if inst.Node != "" && inst.Node != serverName {
+		if inst.Node != "" && inst.Node != s.ServerName {
 			continue // This instance does not belong to this member, skip.
 		}
 
-		err := doProfileUpdateInstance(d, inst, *projects[inst.Project])
+		err := doProfileUpdateInstance(s, inst, *projects[inst.Project])
 		if err != nil {
 			failures[&inst] = err
 		}
@@ -154,10 +159,8 @@ func doProfileUpdate(d *Daemon, p api.Project, profileName string, id int64, pro
 
 // Like doProfileUpdate but does not update the database, since it was already
 // updated by doProfileUpdate itself, called on the notifying node.
-func doProfileUpdateCluster(d *Daemon, projectName string, profileName string, old api.ProfilePut) error {
-	serverName := d.State().ServerName
-
-	insts, projects, err := getProfileInstancesInfo(d.db.Cluster, projectName, profileName)
+func doProfileUpdateCluster(s *state.State, projectName string, profileName string, old api.ProfilePut) error {
+	insts, projects, err := getProfileInstancesInfo(s.DB.Cluster, projectName, profileName)
 	if err != nil {
 		return fmt.Errorf("Failed to query instances associated with profile %q: %w", profileName, err)
 	}
@@ -166,7 +169,7 @@ func doProfileUpdateCluster(d *Daemon, projectName string, profileName string, o
 	for _, it := range insts {
 		inst := it // Local var for instance pointer.
 
-		if inst.Node != "" && inst.Node != serverName {
+		if inst.Node != "" && inst.Node != s.ServerName {
 			continue // This instance does not belong to this member, skip.
 		}
 
@@ -181,7 +184,7 @@ func doProfileUpdateCluster(d *Daemon, projectName string, profileName string, o
 			}
 		}
 
-		err := doProfileUpdateInstance(d, inst, *projects[inst.Project])
+		err := doProfileUpdateInstance(s, inst, *projects[inst.Project])
 		if err != nil {
 			failures[&inst] = err
 		}
@@ -200,19 +203,28 @@ func doProfileUpdateCluster(d *Daemon, projectName string, profileName string, o
 }
 
 // Profile update of a single instance.
-func doProfileUpdateInstance(d *Daemon, args db.InstanceArgs, p api.Project) error {
+func doProfileUpdateInstance(s *state.State, args db.InstanceArgs, p api.Project) error {
 	profileNames := make([]string, 0, len(args.Profiles))
+
 	for _, profile := range args.Profiles {
 		profileNames = append(profileNames, profile.Name)
 	}
 
-	profiles, err := d.db.Cluster.GetProfiles(args.Project, profileNames)
+	var profiles []api.Profile
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		profiles, err = tx.GetProfiles(ctx, args.Project, profileNames)
+
+		return err
+	})
 	if err != nil {
 		return err
 	}
 
 	// Load the instance using the old profile config.
-	inst, err := instance.Load(d.State(), args, p)
+	inst, err := instance.Load(s, args, p)
 	if err != nil {
 		return err
 	}
@@ -233,8 +245,16 @@ func doProfileUpdateInstance(d *Daemon, args db.InstanceArgs, p api.Project) err
 
 // Query the db for information about instances associated with the given profile.
 func getProfileInstancesInfo(dbCluster *db.Cluster, projectName string, profileName string) (map[int]db.InstanceArgs, map[string]*api.Project, error) {
+	var projectInstNames map[string][]string
+
 	// Query the db for information about instances associated with the given profile.
-	projectInstNames, err := dbCluster.GetInstancesWithProfile(projectName, profileName)
+	err := dbCluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		projectInstNames, err = tx.GetInstancesWithProfile(ctx, projectName, profileName)
+
+		return err
+	})
 	if err != nil {
 		return nil, nil, fmt.Errorf("Failed to query instances with profile %q: %w", profileName, err)
 	}
