@@ -9,22 +9,30 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	deviceconfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/rbac"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/validate"
+	"github.com/canonical/lxd/lxd/auth"
+	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	deviceconfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 // AllowInstanceCreation returns an error if any project-specific limit or
 // restriction is violated when creating a new instance.
-func AllowInstanceCreation(tx *db.ClusterTx, projectName string, req api.InstancesPost) error {
-	info, err := fetchProject(tx, projectName, true)
+func AllowInstanceCreation(globalConfig *clusterConfig.Config, tx *db.ClusterTx, projectName string, req api.InstancesPost) error {
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	info, err := fetchProject(globalConfigDump, tx, projectName, true)
 	if err != nil {
 		return err
 	}
@@ -58,16 +66,18 @@ func AllowInstanceCreation(tx *db.ClusterTx, projectName string, req api.Instanc
 	}
 
 	// Add the instance being created.
-	info.Instances = append(info.Instances, api.Instance{
-		Name:        req.Name,
-		Project:     projectName,
-		InstancePut: req.InstancePut,
-	})
+	instance := api.Instance{
+		Name:    req.Name,
+		Project: projectName,
+	}
+
+	instance.SetWritable(req.InstancePut)
+	info.Instances = append(info.Instances, instance)
 
 	// Special case restriction checks on volatile.* keys.
 	strip := false
 
-	if shared.StringInSlice(req.Source.Type, []string{"copy", "migration"}) {
+	if shared.ValueInSlice(req.Source.Type, []string{"copy", "migration"}) {
 		// Allow stripping volatile keys if dealing with a copy or migration.
 		strip = true
 	}
@@ -78,7 +88,7 @@ func AllowInstanceCreation(tx *db.ClusterTx, projectName string, req api.Instanc
 		return err
 	}
 
-	err = checkRestrictionsAndAggregateLimits(tx, info)
+	err = checkRestrictionsAndAggregateLimits(globalConfig, tx, info)
 	if err != nil {
 		return fmt.Errorf("Failed checking if instance creation allowed: %w", err)
 	}
@@ -100,7 +110,7 @@ func checkTotalInstanceCountLimit(info *projectInfo) error {
 	return nil
 }
 
-func getTotalInstanceCountLimit(info *projectInfo) (int, int, error) {
+func getTotalInstanceCountLimit(info *projectInfo) (instanceCount int, limit int, err error) {
 	overallValue, ok := info.Project.Config["limits.instances"]
 	if ok {
 		limit, err := strconv.Atoi(overallValue)
@@ -128,7 +138,7 @@ func checkInstanceCountLimit(info *projectInfo, instanceType instancetype.Type)
 	return nil
 }
 
-func getInstanceCountLimit(info *projectInfo, instanceType instancetype.Type) (int, int, error) {
+func getInstanceCountLimit(info *projectInfo, instanceType instancetype.Type) (instanceCount int, limit int, err error) {
 	var key string
 	switch instanceType {
 	case instancetype.Container:
@@ -139,7 +149,6 @@ func getInstanceCountLimit(info *projectInfo, instanceType instancetype.Type) (i
 		return -1, -1, fmt.Errorf("Unexpected instance type %q", instanceType)
 	}
 
-	instanceCount := 0
 	for _, inst := range info.Instances {
 		if inst.Type == instanceType.String() {
 			instanceCount++
@@ -179,11 +188,11 @@ func checkRestrictionsOnVolatileConfig(project api.Project, instanceType instanc
 
 	// Checker for safe volatile keys.
 	isSafeKey := func(key string) bool {
-		if shared.StringInSlice(key, []string{"volatile.apply_template", "volatile.base_image", "volatile.last_state.power"}) {
+		if shared.ValueInSlice(key, []string{"volatile.apply_template", "volatile.base_image", "volatile.last_state.power"}) {
 			return true
 		}
 
-		if strings.HasPrefix(key, shared.ConfigVolatilePrefix) {
+		if strings.HasPrefix(key, instancetype.ConfigVolatilePrefix) {
 			if strings.HasSuffix(key, ".apply_quota") {
 				return true
 			}
@@ -197,7 +206,7 @@ func checkRestrictionsOnVolatileConfig(project api.Project, instanceType instanc
 	}
 
 	for key, value := range config {
-		if !strings.HasPrefix(key, shared.ConfigVolatilePrefix) {
+		if !strings.HasPrefix(key, instancetype.ConfigVolatilePrefix) {
 			continue
 		}
 
@@ -226,8 +235,13 @@ func checkRestrictionsOnVolatileConfig(project api.Project, instanceType instanc
 
 // AllowVolumeCreation returns an error if any project-specific limit or
 // restriction is violated when creating a new custom volume in a project.
-func AllowVolumeCreation(tx *db.ClusterTx, projectName string, req api.StorageVolumesPost) error {
-	info, err := fetchProject(tx, projectName, true)
+func AllowVolumeCreation(globalConfig *clusterConfig.Config, tx *db.ClusterTx, projectName string, req api.StorageVolumesPost) error {
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	info, err := fetchProject(globalConfigDump, tx, projectName, true)
 	if err != nil {
 		return err
 	}
@@ -247,7 +261,7 @@ func AllowVolumeCreation(tx *db.ClusterTx, projectName string, req api.StorageVo
 		Config: req.Config,
 	})
 
-	err = checkRestrictionsAndAggregateLimits(tx, info)
+	err = checkRestrictionsAndAggregateLimits(globalConfig, tx, info)
 	if err != nil {
 		return fmt.Errorf("Failed checking if volume creation allowed: %w", err)
 	}
@@ -259,8 +273,13 @@ func AllowVolumeCreation(tx *db.ClusterTx, projectName string, req api.StorageVo
 // for writing images.
 //
 // If no limit is in place, return -1.
-func GetImageSpaceBudget(tx *db.ClusterTx, projectName string) (int64, error) {
-	info, err := fetchProject(tx, projectName, true)
+func GetImageSpaceBudget(globalConfig *clusterConfig.Config, tx *db.ClusterTx, projectName string) (int64, error) {
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	info, err := fetchProject(globalConfigDump, tx, projectName, true)
 	if err != nil {
 		return -1, err
 	}
@@ -285,7 +304,7 @@ func GetImageSpaceBudget(tx *db.ClusterTx, projectName string) (int64, error) {
 		return -1, err
 	}
 
-	instances, err := expandInstancesConfigAndDevices(info.Instances, info.Profiles)
+	instances, err := expandInstancesConfigAndDevices(globalConfigDump, info.Instances, info.Profiles)
 	if err != nil {
 		return -1, err
 	}
@@ -306,13 +325,13 @@ func GetImageSpaceBudget(tx *db.ClusterTx, projectName string) (int64, error) {
 
 // Check that we would not violate the project limits or restrictions if we
 // were to commit the given instances and profiles.
-func checkRestrictionsAndAggregateLimits(tx *db.ClusterTx, info *projectInfo) error {
+func checkRestrictionsAndAggregateLimits(globalConfig *clusterConfig.Config, tx *db.ClusterTx, info *projectInfo) error {
 	// List of config keys for which we need to check aggregate values
 	// across all project instances.
 	aggregateKeys := []string{}
 	isRestricted := false
 	for key, value := range info.Project.Config {
-		if shared.StringInSlice(key, allAggregateLimits) {
+		if shared.ValueInSlice(key, allAggregateLimits) {
 			aggregateKeys = append(aggregateKeys, key)
 			continue
 		}
@@ -327,7 +346,12 @@ func checkRestrictionsAndAggregateLimits(tx *db.ClusterTx, info *projectInfo) er
 		return nil
 	}
 
-	instances, err := expandInstancesConfigAndDevices(info.Instances, info.Profiles)
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	instances, err := expandInstancesConfigAndDevices(globalConfigDump, info.Instances, info.Profiles)
 	if err != nil {
 		return err
 	}
@@ -818,15 +842,16 @@ var allowableIntercept = []string{
 	"security.syscalls.intercept.mount",
 	"security.syscalls.intercept.mount.fuse",
 	"security.syscalls.intercept.setxattr",
+	"security.syscalls.intercept.sysinfo",
 }
 
 // Return true if a low-level container option is forbidden.
 func isContainerLowLevelOptionForbidden(key string) bool {
-	if strings.HasPrefix(key, "security.syscalls.intercept") && !shared.StringInSlice(key, allowableIntercept) {
+	if strings.HasPrefix(key, "security.syscalls.intercept") && !shared.ValueInSlice(key, allowableIntercept) {
 		return true
 	}
 
-	if shared.StringInSlice(key, []string{
+	if shared.ValueInSlice(key, []string{
 		"boot.host_shutdown_timeout",
 		"linux.kernel_modules",
 		"raw.apparmor",
@@ -845,7 +870,7 @@ func isContainerLowLevelOptionForbidden(key string) bool {
 
 // Return true if a low-level VM option is forbidden.
 func isVMLowLevelOptionForbidden(key string) bool {
-	return shared.StringInSlice(key, []string{
+	return shared.ValueInSlice(key, []string{
 		"boot.host_shutdown_timeout",
 		"limits.memory.hugepages",
 		"raw.idmap",
@@ -855,9 +880,15 @@ func isVMLowLevelOptionForbidden(key string) bool {
 
 // AllowInstanceUpdate returns an error if any project-specific limit or
 // restriction is violated when updating an existing instance.
-func AllowInstanceUpdate(tx *db.ClusterTx, projectName, instanceName string, req api.InstancePut, currentConfig map[string]string) error {
+func AllowInstanceUpdate(globalConfig *clusterConfig.Config, tx *db.ClusterTx, projectName, instanceName string, req api.InstancePut, currentConfig map[string]string) error {
 	var updatedInstance *api.Instance
-	info, err := fetchProject(tx, projectName, true)
+
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	info, err := fetchProject(globalConfigDump, tx, projectName, true)
 	if err != nil {
 		return err
 	}
@@ -891,7 +922,7 @@ func AllowInstanceUpdate(tx *db.ClusterTx, projectName, instanceName string, req
 		return err
 	}
 
-	err = checkRestrictionsAndAggregateLimits(tx, info)
+	err = checkRestrictionsAndAggregateLimits(globalConfig, tx, info)
 	if err != nil {
 		return fmt.Errorf("Failed checking if instance update allowed: %w", err)
 	}
@@ -901,8 +932,13 @@ func AllowInstanceUpdate(tx *db.ClusterTx, projectName, instanceName string, req
 
 // AllowVolumeUpdate returns an error if any project-specific limit or
 // restriction is violated when updating an existing custom volume.
-func AllowVolumeUpdate(tx *db.ClusterTx, projectName, volumeName string, req api.StorageVolumePut, currentConfig map[string]string) error {
-	info, err := fetchProject(tx, projectName, true)
+func AllowVolumeUpdate(globalConfig *clusterConfig.Config, tx *db.ClusterTx, projectName, volumeName string, req api.StorageVolumePut, currentConfig map[string]string) error {
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	info, err := fetchProject(globalConfigDump, tx, projectName, true)
 	if err != nil {
 		return err
 	}
@@ -925,7 +961,7 @@ func AllowVolumeUpdate(tx *db.ClusterTx, projectName, volumeName string, req api
 		info.Volumes[i].Config = req.Config
 	}
 
-	err = checkRestrictionsAndAggregateLimits(tx, info)
+	err = checkRestrictionsAndAggregateLimits(globalConfig, tx, info)
 	if err != nil {
 		return fmt.Errorf("Failed checking if volume update allowed: %w", err)
 	}
@@ -935,8 +971,13 @@ func AllowVolumeUpdate(tx *db.ClusterTx, projectName, volumeName string, req api
 
 // AllowProfileUpdate checks that project limits and restrictions are not
 // violated when changing a profile.
-func AllowProfileUpdate(tx *db.ClusterTx, projectName, profileName string, req api.ProfilePut) error {
-	info, err := fetchProject(tx, projectName, true)
+func AllowProfileUpdate(globalConfig *clusterConfig.Config, tx *db.ClusterTx, projectName, profileName string, req api.ProfilePut) error {
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	info, err := fetchProject(globalConfigDump, tx, projectName, true)
 	if err != nil {
 		return err
 	}
@@ -955,7 +996,7 @@ func AllowProfileUpdate(tx *db.ClusterTx, projectName, profileName string, req a
 		info.Profiles[i].Devices = req.Devices
 	}
 
-	err = checkRestrictionsAndAggregateLimits(tx, info)
+	err = checkRestrictionsAndAggregateLimits(globalConfig, tx, info)
 	if err != nil {
 		return fmt.Errorf("Failed checking if profile update allowed: %w", err)
 	}
@@ -964,13 +1005,18 @@ func AllowProfileUpdate(tx *db.ClusterTx, projectName, profileName string, req a
 }
 
 // AllowProjectUpdate checks the new config to be set on a project is valid.
-func AllowProjectUpdate(tx *db.ClusterTx, projectName string, config map[string]string, changed []string) error {
-	info, err := fetchProject(tx, projectName, false)
+func AllowProjectUpdate(globalConfig *clusterConfig.Config, tx *db.ClusterTx, projectName string, config map[string]string, changed []string) error {
+	var globalConfigDump map[string]any
+	if globalConfig != nil {
+		globalConfigDump = globalConfig.Dump()
+	}
+
+	info, err := fetchProject(globalConfigDump, tx, projectName, false)
 	if err != nil {
 		return err
 	}
 
-	info.Instances, err = expandInstancesConfigAndDevices(info.Instances, info.Profiles)
+	info.Instances, err = expandInstancesConfigAndDevices(globalConfigDump, info.Instances, info.Profiles)
 	if err != nil {
 		return err
 	}
@@ -982,10 +1028,8 @@ func AllowProjectUpdate(tx *db.ClusterTx, projectName string, config map[string]
 	for _, key := range changed {
 		if strings.HasPrefix(key, "restricted.") {
 			project := api.Project{
-				Name: projectName,
-				ProjectPut: api.ProjectPut{
-					Config: config,
-				},
+				Name:   projectName,
+				Config: config,
 			}
 
 			err := checkRestrictions(project, info.Instances, info.Profiles)
@@ -1149,7 +1193,7 @@ type projectInfo struct {
 // If the skipIfNoLimits flag is true, then profiles, instances and volumes
 // won't be loaded if the profile has no limits set on it, and nil will be
 // returned.
-func fetchProject(tx *db.ClusterTx, projectName string, skipIfNoLimits bool) (*projectInfo, error) {
+func fetchProject(globalConfig map[string]any, tx *db.ClusterTx, projectName string, skipIfNoLimits bool) (*projectInfo, error) {
 	ctx := context.Background()
 	dbProject, err := cluster.GetProject(ctx, tx.Tx(), projectName)
 	if err != nil {
@@ -1170,8 +1214,8 @@ func fetchProject(tx *db.ClusterTx, projectName string, skipIfNoLimits bool) (*p
 	// If the project has the profiles feature enabled, we use its own
 	// profiles to expand the instances configs, otherwise we use the
 	// profiles from the default project.
-	defaultProject := Default
-	if projectName == Default || shared.IsTrue(project.Config["features.profiles"]) {
+	defaultProject := api.ProjectDefaultName
+	if projectName == api.ProjectDefaultName || shared.IsTrue(project.Config["features.profiles"]) {
 		profilesFilter.Project = &projectName
 	} else {
 		profilesFilter.Project = &defaultProject
@@ -1199,7 +1243,7 @@ func fetchProject(tx *db.ClusterTx, projectName string, skipIfNoLimits bool) (*p
 
 	instances := make([]api.Instance, 0, len(dbInstances))
 	for _, instance := range dbInstances {
-		apiInstance, err := instance.ToAPI(ctx, tx.Tx())
+		apiInstance, err := instance.ToAPI(ctx, tx.Tx(), globalConfig)
 		if err != nil {
 			return nil, fmt.Errorf("Failed to get API data for instance %q in project %q: %w", instance.Name, instance.Project, err)
 		}
@@ -1224,7 +1268,7 @@ func fetchProject(tx *db.ClusterTx, projectName string, skipIfNoLimits bool) (*p
 
 // Expand the configuration and devices of the given instances, taking the give
 // project profiles into account.
-func expandInstancesConfigAndDevices(instances []api.Instance, profiles []api.Profile) ([]api.Instance, error) {
+func expandInstancesConfigAndDevices(globalConfig map[string]any, instances []api.Instance, profiles []api.Profile) ([]api.Instance, error) {
 	expandedInstances := make([]api.Instance, len(instances))
 
 	// Index of all profiles by name.
@@ -1242,15 +1286,15 @@ func expandInstancesConfigAndDevices(instances []api.Instance, profiles []api.Pr
 		}
 
 		expandedInstances[i] = instance
-		expandedInstances[i].Config = db.ExpandInstanceConfig(instance.Config, apiProfiles)
-		expandedInstances[i].Devices = db.ExpandInstanceDevices(deviceconfig.NewDevices(instance.Devices), apiProfiles).CloneNative()
+		expandedInstances[i].Config = instancetype.ExpandInstanceConfig(globalConfig, instance.Config, apiProfiles)
+		expandedInstances[i].Devices = instancetype.ExpandInstanceDevices(deviceconfig.NewDevices(instance.Devices), apiProfiles).CloneNative()
 	}
 
 	return expandedInstances, nil
 }
 
 // Sum of the effective values for the given limits across all project
-// enties (instances and custom volumes).
+// entities (instances and custom volumes).
 func getTotalsAcrossProjectEntities(info *projectInfo, keys []string, skipUnset bool) (map[string]int64, error) {
 	totals := map[string]int64{}
 
@@ -1301,7 +1345,7 @@ func getInstanceLimits(instance api.Instance, keys []string, skipUnset bool) (ma
 		parser := aggregateLimitConfigValueParsers[key]
 
 		if key == "limits.disk" {
-			_, device, err := shared.GetRootDiskDevice(instance.Devices)
+			_, device, err := instancetype.GetRootDiskDevice(instance.Devices)
 			if err != nil {
 				return nil, fmt.Errorf("Failed getting root disk device for instance %q in project %q: %w", instance.Name, instance.Project, err)
 			}
@@ -1328,6 +1372,11 @@ func getInstanceLimits(instance api.Instance, keys []string, skipUnset bool) (ma
 			if instance.Type == instancetype.VM.String() {
 				sizeStateValue, ok := device["size.state"]
 				if !ok {
+					// TODO: In case the VMs storage drivers config drive size isn't the default,
+					// the limits accounting will be incorrect.
+					// This applies for the PowerFlex storage driver whose config drive size
+					// is 8 GB as set in the DefaultVMPowerFlexBlockFilesystemSize variable.
+					// See https://github.com/canonical/lxd/issues/12567
 					sizeStateValue = deviceconfig.DefaultVMBlockFilesystemSize
 				}
 
@@ -1416,36 +1465,71 @@ var aggregateLimitConfigValuePrinters = map[string]func(int64) string{
 	},
 }
 
-// FilterUsedBy filters a UsedBy list based on project access.
-func FilterUsedBy(r *http.Request, entries []string) []string {
-	// Shortcut for admins and non-RBAC environments.
-	if rbac.UserIsAdmin(r) {
-		return entries
+// FilterUsedBy filters a UsedBy list based on the entities that the requestor is able to view.
+func FilterUsedBy(authorizer auth.Authorizer, r *http.Request, entries []string) []string {
+	// Get a map of URLs by entity type. If there are multiple entries of a particular entity type we can reduce the
+	// number of calls to the authorizer.
+	urlsByEntityType := make(map[entity.Type][]*api.URL)
+	for _, entry := range entries {
+		u, err := url.Parse(entry)
+		if err != nil {
+			logger.Warn("Failed to parse project used-by entity URL", logger.Ctx{"url": entry, "error": err})
+			continue
+		}
+
+		entityType, projectName, location, pathArguments, err := entity.ParseURL(*u)
+		if err != nil {
+			logger.Warn("Failed to parse project used-by entity URL", logger.Ctx{"url": entry, "error": err})
+			continue
+		}
+
+		entityURL, err := entityType.URL(projectName, location, pathArguments...)
+		if err != nil {
+			logger.Warn("Failed to create canonical entity URL for project used-by filtering", logger.Ctx{"url": entry, "error": err})
+			continue
+		}
+
+		urlsByEntityType[entityType] = append(urlsByEntityType[entityType], entityURL)
 	}
 
 	// Filter the entries.
-	usedBy := []string{}
-	for _, entry := range entries {
-		projectName := Default
+	usedBy := make([]string, 0, len(entries))
 
-		// Try to parse the query part of the URL.
-		u, err := url.Parse(entry)
+	// Used-by lists do not include the project query parameter if it is the default project.
+	appendUsedBy := func(u *api.URL) {
+		if u.Query().Get("project") == api.ProjectDefaultName {
+			q := u.Query()
+			q.Del("project")
+			u.RawQuery = q.Encode()
+		}
+
+		usedBy = append(usedBy, u.String())
+	}
+
+	for entityType, urls := range urlsByEntityType {
+		// If only one entry of this type, check directly.
+		if len(urls) == 1 {
+			err := authorizer.CheckPermission(r.Context(), r, urls[0], auth.EntitlementCanView)
+			if err != nil {
+				continue
+			}
+
+			appendUsedBy(urls[0])
+			continue
+		}
+
+		// Otherwise get a permission checker for the entity type.
+		canViewEntity, err := authorizer.GetPermissionChecker(r.Context(), r, auth.EntitlementCanView, entityType)
 		if err != nil {
-			// Skip URLs we can't parse.
-			continue
+			logger.Warn("Failed to get permission checker for project used-by filtering", logger.Ctx{"entity_type": entityType, "error": err})
 		}
 
-		// Check if project= is specified in the URL.
-		val := u.Query().Get("project")
-		if val != "" {
-			projectName = val
+		// Check each url and append.
+		for _, u := range urls {
+			if canViewEntity(u) {
+				appendUsedBy(u)
+			}
 		}
-
-		if !rbac.UserHasPermission(r, projectName, "view") {
-			continue
-		}
-
-		usedBy = append(usedBy, entry)
 	}
 
 	return usedBy
@@ -1470,14 +1554,15 @@ func projectHasRestriction(project *api.Project, restrictionKey string, blockVal
 }
 
 // CheckClusterTargetRestriction check if user is allowed to use cluster member targeting.
-func CheckClusterTargetRestriction(r *http.Request, project *api.Project, targetFlag string) error {
-	// Allow server administrators to move instances around even when restricted (node evacuation, ...)
-	if rbac.UserIsAdmin(r) {
-		return nil
-	}
-
+func CheckClusterTargetRestriction(authorizer auth.Authorizer, r *http.Request, project *api.Project, targetFlag string) error {
 	if projectHasRestriction(project, "restricted.cluster.target", "block") && targetFlag != "" {
-		return fmt.Errorf("This project doesn't allow cluster member targeting")
+		// Allow server administrators to move instances around even when restricted (node evacuation, ...)
+		err := authorizer.CheckPermission(r.Context(), r, entity.ServerURL(), auth.EntitlementCanOverrideClusterTargetRestriction)
+		if err != nil && auth.IsDeniedError(err) {
+			return api.StatusErrorf(http.StatusForbidden, "This project doesn't allow cluster member targeting")
+		} else if err != nil {
+			return err
+		}
 	}
 
 	return nil
@@ -1513,3 +1598,114 @@ func AllowSnapshotCreation(p *api.Project) error {
 
 	return nil
 }
+
+// GetRestrictedClusterGroups returns a slice of restricted cluster groups for the given project.
+func GetRestrictedClusterGroups(p *api.Project) []string {
+	return shared.SplitNTrimSpace(p.Config["restricted.cluster.groups"], ",", -1, true)
+}
+
+// AllowClusterMember returns nil if the given project is allowed to use the cluster member.
+func AllowClusterMember(p *api.Project, member *db.NodeInfo) error {
+	clusterGroupsAllowed := GetRestrictedClusterGroups(p)
+
+	if shared.IsTrue(p.Config["restricted"]) && len(clusterGroupsAllowed) > 0 {
+		for _, memberGroupName := range member.Groups {
+			if shared.ValueInSlice(memberGroupName, clusterGroupsAllowed) {
+				return nil
+			}
+		}
+
+		return fmt.Errorf("Project isn't allowed to use this cluster member: %q", member.Name)
+	}
+
+	return nil
+}
+
+// AllowClusterGroup returns nil if the given project is allowed to use the cluster groupName.
+func AllowClusterGroup(p *api.Project, groupName string) error {
+	clusterGroupsAllowed := GetRestrictedClusterGroups(p)
+
+	// Skip the check if the project is not restricted
+	if shared.IsFalseOrEmpty(p.Config["restricted"]) {
+		return nil
+	}
+
+	if len(clusterGroupsAllowed) > 0 && !shared.ValueInSlice(groupName, clusterGroupsAllowed) {
+		return fmt.Errorf("Project isn't allowed to use this cluster group: %q", groupName)
+	}
+
+	return nil
+}
+
+// CheckTargetMember checks if the given targetMemberName is present in allMembers
+// and is allowed for the project.
+// If the target member is allowed it returns the resolved node information.
+func CheckTargetMember(p *api.Project, targetMemberName string, allMembers []db.NodeInfo) (*db.NodeInfo, error) {
+	// Find target member.
+	for _, potentialMember := range allMembers {
+		if potentialMember.Name == targetMemberName {
+			// If restricted groups are specified then check member is in at least one of them.
+			err := AllowClusterMember(p, &potentialMember)
+			if err != nil {
+				return nil, api.StatusErrorf(http.StatusForbidden, err.Error())
+			}
+
+			return &potentialMember, nil
+		}
+	}
+
+	return nil, api.StatusErrorf(http.StatusNotFound, "Cluster member %q not found", targetMemberName)
+}
+
+// CheckTargetGroup checks if the given groupName is allowed for the project.
+func CheckTargetGroup(ctx context.Context, tx *db.ClusterTx, p *api.Project, groupName string) error {
+	// If restricted groups are specified then check the requested group is in the list.
+	err := AllowClusterGroup(p, groupName)
+	if err != nil {
+		return api.StatusErrorf(http.StatusForbidden, err.Error())
+	}
+
+	// Check if the target group exists.
+	targetGroupExists, err := cluster.ClusterGroupExists(ctx, tx.Tx(), groupName)
+	if err != nil {
+		return err
+	}
+
+	if !targetGroupExists {
+		return api.StatusErrorf(http.StatusBadRequest, "Cluster group %q doesn't exist", groupName)
+	}
+
+	return nil
+}
+
+// CheckTarget checks if the given cluster target (member or group) is allowed.
+// If target is a cluster member and is found in allMembers it returns the resolved node information object.
+// If target is a cluster group it returns the cluster group name.
+// In case of error, neither node information nor cluster group name gets returned.
+func CheckTarget(ctx context.Context, authorizer auth.Authorizer, r *http.Request, tx *db.ClusterTx, p *api.Project, target string, allMembers []db.NodeInfo) (*db.NodeInfo, string, error) {
+	targetMemberName, targetGroupName := shared.TargetDetect(target)
+
+	// Check manual cluster member targeting restrictions.
+	err := CheckClusterTargetRestriction(authorizer, r, p, target)
+	if err != nil {
+		return nil, "", err
+	}
+
+	if targetMemberName != "" {
+		member, err := CheckTargetMember(p, targetMemberName, allMembers)
+		if err != nil {
+			return nil, "", err
+		}
+
+		return member, "", nil
+	} else if targetGroupName != "" {
+		err := CheckTargetGroup(ctx, tx, p, targetGroupName)
+		if err != nil {
+			return nil, "", err
+		}
+
+		return nil, targetGroupName, nil
+	}
+
+	return nil, "", nil
+}
