@@ -5,21 +5,18 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 )
 
-// Default is the string used for a default project.
-const Default = "default"
-
 // separator is used to delimit the project name from the suffix.
 const separator = "_"
 
 // Instance adds the "<project>_" prefix to instance name when the given project name is not "default".
 func Instance(projectName string, instanceName string) string {
-	if projectName != Default {
+	if projectName != api.ProjectDefaultName {
 		return fmt.Sprintf("%s%s%s", projectName, separator, instanceName)
 	}
 
@@ -28,7 +25,7 @@ func Instance(projectName string, instanceName string) string {
 
 // DNS adds ".<project>" as a suffix to instance name when the given project name is not "default".
 func DNS(projectName string, instanceName string) string {
-	if projectName != Default {
+	if projectName != api.ProjectDefaultName {
 		return fmt.Sprintf("%s.%s", instanceName, projectName)
 	}
 
@@ -44,7 +41,7 @@ func InstanceParts(projectInstanceName string) (string, string) {
 	i := strings.LastIndex(projectInstanceName, separator)
 	if i < 0 {
 		// This string is not project prefixed or is part of default project.
-		return Default, projectInstanceName
+		return api.ProjectDefaultName, projectInstanceName
 	}
 
 	// As project names can container separator, we effectively split once from the right hand side as
@@ -61,6 +58,12 @@ func StorageVolume(projectName string, storageVolumeName string) string {
 // name as separate variables.
 func StorageVolumeParts(projectStorageVolumeName string) (string, string) {
 	parts := strings.SplitN(projectStorageVolumeName, "_", 2)
+
+	// If the given name doesn't contain any project, only return the volume name.
+	if len(parts) == 1 {
+		return "", projectStorageVolumeName
+	}
+
 	return parts[0], parts[1]
 }
 
@@ -72,13 +75,8 @@ func StorageVolumeParts(projectStorageVolumeName string) (string, string) {
 func StorageVolumeProject(c *db.Cluster, projectName string, volumeType int) (string, error) {
 	// Image volumes are effectively a cache and so are always linked to default project.
 	// Optimisation to avoid loading project record.
-	if volumeType == db.StoragePoolVolumeTypeImage {
-		return Default, nil
-	}
-
-	// Non-custom volumes always use the project specified. Optimisation to avoid loading project record.
-	if volumeType != db.StoragePoolVolumeTypeCustom {
-		return projectName, nil
+	if volumeType == cluster.StoragePoolVolumeTypeImage {
+		return api.ProjectDefaultName, nil
 	}
 
 	var project *api.Project
@@ -106,12 +104,12 @@ func StorageVolumeProject(c *db.Cluster, projectName string, volumeType int) (st
 // For all other volume types the supplied project's name is returned.
 func StorageVolumeProjectFromRecord(p *api.Project, volumeType int) string {
 	// Image volumes are effectively a cache and so are always linked to default project.
-	if volumeType == db.StoragePoolVolumeTypeImage {
-		return Default
+	if volumeType == cluster.StoragePoolVolumeTypeImage {
+		return api.ProjectDefaultName
 	}
 
 	// Non-custom volumes always use the project specified.
-	if volumeType != db.StoragePoolVolumeTypeCustom {
+	if volumeType != cluster.StoragePoolVolumeTypeCustom {
 		return p.Name
 	}
 
@@ -121,7 +119,42 @@ func StorageVolumeProjectFromRecord(p *api.Project, volumeType int) string {
 		return p.Name
 	}
 
-	return Default
+	return api.ProjectDefaultName
+}
+
+// StorageBucketProject returns the effective project name to use to for the bucket based on the requested project.
+// If the project specified has the "features.storage.buckets" flag enabled then the project name is returned,
+// otherwise the default project name is returned.
+func StorageBucketProject(ctx context.Context, c *db.Cluster, projectName string) (string, error) {
+	var p *api.Project
+	err := c.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		dbProject, err := cluster.GetProject(ctx, tx.Tx(), projectName)
+		if err != nil {
+			return err
+		}
+
+		p, err = dbProject.ToAPI(ctx, tx.Tx())
+
+		return err
+	})
+	if err != nil {
+		return "", fmt.Errorf("Failed to load project %q: %w", projectName, err)
+	}
+
+	return StorageBucketProjectFromRecord(p), nil
+}
+
+// StorageBucketProjectFromRecord returns the project name to use to for the bucket based on the supplied project.
+// If the project supplied has the "features.storage.buckets" flag enabled then the project name is returned,
+// otherwise the default project name is returned.
+func StorageBucketProjectFromRecord(p *api.Project) string {
+	// Buckets only use the project specified if the project has the features.storage.buckets feature
+	// enabled, otherwise the default project is used.
+	if shared.IsTrue(p.Config["features.storage.buckets"]) {
+		return p.Name
+	}
+
+	return api.ProjectDefaultName
 }
 
 // NetworkProject returns the effective project name to use for the network based on the requested project.
@@ -159,7 +192,7 @@ func NetworkProjectFromRecord(p *api.Project) string {
 		return p.Name
 	}
 
-	return Default
+	return api.ProjectDefaultName
 }
 
 // NetworkAllowed returns whether access is allowed to a particular network based on projectConfig.
@@ -175,7 +208,7 @@ func NetworkAllowed(reqProjectConfig map[string]string, networkName string, isMa
 	}
 
 	// Don't allow access to unmanaged networks if only managed network access is allowed.
-	if shared.StringInSlice(reqProjectConfig["restricted.devices.nic"], []string{"managed", ""}) && !isManaged {
+	if shared.ValueInSlice(reqProjectConfig["restricted.devices.nic"], []string{"managed", ""}) && !isManaged {
 		return false
 	}
 
@@ -186,7 +219,7 @@ func NetworkAllowed(reqProjectConfig map[string]string, networkName string, isMa
 
 	// Check if reqquested network is in list of allowed networks.
 	allowedRestrictedNetworks := shared.SplitNTrimSpace(reqProjectConfig["restricted.networks.access"], ",", -1, false)
-	return shared.StringInSlice(networkName, allowedRestrictedNetworks)
+	return shared.ValueInSlice(networkName, allowedRestrictedNetworks)
 }
 
 // ProfileProject returns the effective project to use for the profile based on the requested project.
@@ -207,7 +240,7 @@ func ProfileProject(c *db.Cluster, projectName string) (*api.Project, error) {
 
 		effectiveProjectName := ProfileProjectFromRecord(p)
 
-		if effectiveProjectName == Default {
+		if effectiveProjectName == api.ProjectDefaultName {
 			dbProject, err = cluster.GetProject(ctx, tx.Tx(), effectiveProjectName)
 			if err != nil {
 				return fmt.Errorf("Failed loading project %q: %w", effectiveProjectName, err)
@@ -238,5 +271,43 @@ func ProfileProjectFromRecord(p *api.Project) string {
 		return p.Name
 	}
 
-	return Default
+	return api.ProjectDefaultName
+}
+
+// NetworkZoneProject returns the effective project name to use for network zone based on the requested project.
+// If the requested project has the "features.networks.zones" flag enabled then the requested project's name is
+// returned, otherwise the default project name is returned.
+// The second return value is always the requested project's info.
+func NetworkZoneProject(c *db.Cluster, projectName string) (string, *api.Project, error) {
+	var p *api.Project
+	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbProject, err := cluster.GetProject(ctx, tx.Tx(), projectName)
+		if err != nil {
+			return err
+		}
+
+		p, err = dbProject.ToAPI(ctx, tx.Tx())
+
+		return err
+	})
+	if err != nil {
+		return "", nil, fmt.Errorf("Failed to load project %q: %w", projectName, err)
+	}
+
+	effectiveProjectName := NetworkZoneProjectFromRecord(p)
+
+	return effectiveProjectName, p, nil
+}
+
+// NetworkZoneProjectFromRecord returns the project name to use for the network zone based on the supplied project.
+// If the project supplied has the "features.networks.zones" flag enabled then the project name is returned,
+// otherwise the default project name is returned.
+func NetworkZoneProjectFromRecord(p *api.Project) string {
+	// Network zones only use the project specified if the project has the features.networks.zones feature
+	// enabled, otherwise the legacy behaviour of using the default project for network zones is used.
+	if shared.IsTrue(p.Config["features.networks.zones"]) {
+		return p.Name
+	}
+
+	return api.ProjectDefaultName
 }
