@@ -12,7 +12,8 @@ import (
 	"github.com/jaypipes/pcidb"
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 )
 
 var sysClassNet = "/sys/class/net"
@@ -24,19 +25,51 @@ var netProtocols = map[uint64]string{
 }
 
 func networkAddDeviceInfo(devicePath string, pciDB *pcidb.PCIDB, uname unix.Utsname, card *api.ResourcesNetworkCard) error {
+	deviceDeviceDir, err := getDeviceDir(devicePath)
+	if err != nil {
+		return fmt.Errorf("Failed to read %q: %w", devicePath, err)
+	}
+
+	// VDPA
+	vDPAMatches, err := filepath.Glob(filepath.Join(deviceDeviceDir, "vdpa*"))
+	if err != nil {
+		return fmt.Errorf("Malformed VDPA device name search pattern: %w", err)
+	}
+
+	if len(vDPAMatches) > 0 {
+		vdpa := api.ResourcesNetworkCardVDPA{}
+
+		splittedPath := strings.Split(vDPAMatches[0], "/")
+		vdpa.Name = splittedPath[len(splittedPath)-1]
+		vDPADevMatches, err := filepath.Glob(filepath.Join(vDPAMatches[0], "vhost-vdpa-*"))
+		if err != nil {
+			return fmt.Errorf("Malformed VDPA device name search pattern: %w", err)
+		}
+
+		if len(vDPADevMatches) > 0 {
+			splittedPath = strings.Split(vDPADevMatches[0], "/")
+			vdpa.Device = splittedPath[len(splittedPath)-1]
+		} else {
+			return fmt.Errorf("Failed to find VDPA device at device path %q", vDPAMatches[0])
+		}
+
+		// Add the VDPA data to the card
+		card.VDPA = &vdpa
+	}
+
 	// SRIOV
-	if sysfsExists(filepath.Join(devicePath, "sriov_numvfs")) {
+	if sysfsExists(filepath.Join(deviceDeviceDir, "sriov_numvfs")) {
 		sriov := api.ResourcesNetworkCardSRIOV{}
 
 		// Get maximum and current VF count
-		vfMaximum, err := readUint(filepath.Join(devicePath, "sriov_totalvfs"))
+		vfMaximum, err := readUint(filepath.Join(deviceDeviceDir, "sriov_totalvfs"))
 		if err != nil {
-			return fmt.Errorf("Failed to read %q: %w", filepath.Join(devicePath, "sriov_totalvfs"), err)
+			return fmt.Errorf("Failed to read %q: %w", filepath.Join(deviceDeviceDir, "sriov_totalvfs"), err)
 		}
 
-		vfCurrent, err := readUint(filepath.Join(devicePath, "sriov_numvfs"))
+		vfCurrent, err := readUint(filepath.Join(deviceDeviceDir, "sriov_numvfs"))
 		if err != nil {
-			return fmt.Errorf("Failed to read %q: %w", filepath.Join(devicePath, "sriov_numvfs"), err)
+			return fmt.Errorf("Failed to read %q: %w", filepath.Join(deviceDeviceDir, "sriov_numvfs"), err)
 		}
 
 		sriov.MaximumVFs = vfMaximum
@@ -47,10 +80,10 @@ func networkAddDeviceInfo(devicePath string, pciDB *pcidb.PCIDB, uname unix.Utsn
 	}
 
 	// NUMA node
-	if sysfsExists(filepath.Join(devicePath, "numa_node")) {
-		numaNode, err := readInt(filepath.Join(devicePath, "numa_node"))
+	if sysfsExists(filepath.Join(deviceDeviceDir, "numa_node")) {
+		numaNode, err := readInt(filepath.Join(deviceDeviceDir, "numa_node"))
 		if err != nil {
-			return fmt.Errorf("Failed to read %q: %w", filepath.Join(devicePath, "numa_node"), err)
+			return fmt.Errorf("Failed to read %q: %w", filepath.Join(deviceDeviceDir, "numa_node"), err)
 		}
 
 		if numaNode > 0 {
@@ -59,9 +92,9 @@ func networkAddDeviceInfo(devicePath string, pciDB *pcidb.PCIDB, uname unix.Utsn
 	}
 
 	// USB address
-	usbAddr, err := usbAddress(devicePath)
+	usbAddr, err := usbAddress(deviceDeviceDir)
 	if err != nil {
-		return fmt.Errorf("Failed to find USB address for %q: %w", devicePath, err)
+		return fmt.Errorf("Failed to find USB address for %q: %w", deviceDeviceDir, err)
 	}
 
 	if usbAddr != "" {
@@ -69,7 +102,7 @@ func networkAddDeviceInfo(devicePath string, pciDB *pcidb.PCIDB, uname unix.Utsn
 	}
 
 	// Vendor and product
-	deviceVendorPath := filepath.Join(devicePath, "vendor")
+	deviceVendorPath := filepath.Join(deviceDeviceDir, "vendor")
 	if sysfsExists(deviceVendorPath) {
 		id, err := os.ReadFile(deviceVendorPath)
 		if err != nil {
@@ -79,7 +112,7 @@ func networkAddDeviceInfo(devicePath string, pciDB *pcidb.PCIDB, uname unix.Utsn
 		card.VendorID = strings.TrimPrefix(strings.TrimSpace(string(id)), "0x")
 	}
 
-	deviceDevicePath := filepath.Join(devicePath, "device")
+	deviceDevicePath := filepath.Join(deviceDeviceDir, "device")
 	if sysfsExists(deviceDevicePath) {
 		id, err := os.ReadFile(deviceDevicePath)
 		if err != nil {
@@ -105,11 +138,11 @@ func networkAddDeviceInfo(devicePath string, pciDB *pcidb.PCIDB, uname unix.Utsn
 	}
 
 	// Driver information
-	driverPath := filepath.Join(devicePath, "driver")
+	driverPath := filepath.Join(deviceDeviceDir, "driver")
 	if sysfsExists(driverPath) {
 		linkTarget, err := filepath.EvalSymlinks(driverPath)
 		if err != nil {
-			return fmt.Errorf("Failed to find %q: %w", driverPath, err)
+			return fmt.Errorf("Failed to find device directory %q: %w", driverPath, err)
 		}
 
 		// Set the driver name
@@ -151,9 +184,9 @@ func networkAddDeviceInfo(devicePath string, pciDB *pcidb.PCIDB, uname unix.Utsn
 				protocol, ok := netProtocols[devType]
 				if !ok {
 					info.Protocol = "unknown"
+				} else {
+					info.Protocol = protocol
 				}
-
-				info.Protocol = protocol
 			}
 
 			// Add MAC address
@@ -327,7 +360,7 @@ func GetNetwork() (*api.ResourcesNetwork, error) {
 				card.PCIAddress = pciAddr
 
 				// Skip devices we already know about
-				if stringInSlice(card.PCIAddress, pciKnown) {
+				if shared.ValueInSlice(card.PCIAddress, pciKnown) {
 					continue
 				}
 
@@ -375,7 +408,7 @@ func GetNetwork() (*api.ResourcesNetwork, error) {
 			devicePath := filepath.Join(sysBusPci, entryName)
 
 			// Skip devices we already know about
-			if stringInSlice(entryName, pciKnown) {
+			if shared.ValueInSlice(entryName, pciKnown) {
 				continue
 			}
 
