@@ -2,36 +2,6 @@
 
 package seccomp
 
-import (
-	"context"
-	"fmt"
-	"io"
-	"net"
-	"os"
-	"path"
-	"regexp"
-	"runtime"
-	"strconv"
-	"strings"
-	"unsafe"
-
-	liblxc "github.com/lxc/go-lxc"
-	"golang.org/x/sys/unix"
-
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	_ "github.com/lxc/lxd/lxd/include" // Used by cgo
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/ucred"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/netutils"
-	"github.com/lxc/lxd/shared/osarch"
-)
-
 /*
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -128,6 +98,7 @@ struct lxd_seccomp_data_arch {
 	int nr_mount;
 	int nr_bpf;
 	int nr_sched_setscheduler;
+	int nr_sysinfo;
 };
 
 #define LXD_SECCOMP_NOTIFY_MKNOD    0
@@ -136,66 +107,67 @@ struct lxd_seccomp_data_arch {
 #define LXD_SECCOMP_NOTIFY_MOUNT 3
 #define LXD_SECCOMP_NOTIFY_BPF 4
 #define LXD_SECCOMP_NOTIFY_SCHED_SETSCHEDULER 5
+#define LXD_SECCOMP_NOTIFY_SYSINFO 6
 
 // ordered by likelihood of usage...
 static const struct lxd_seccomp_data_arch seccomp_notify_syscall_table[] = {
-	{ -1, LXD_SECCOMP_NOTIFY_MKNOD, LXD_SECCOMP_NOTIFY_MKNODAT, LXD_SECCOMP_NOTIFY_SETXATTR, LXD_SECCOMP_NOTIFY_MOUNT, LXD_SECCOMP_NOTIFY_BPF, LXD_SECCOMP_NOTIFY_SCHED_SETSCHEDULER },
+	{ -1, LXD_SECCOMP_NOTIFY_MKNOD, LXD_SECCOMP_NOTIFY_MKNODAT, LXD_SECCOMP_NOTIFY_SETXATTR, LXD_SECCOMP_NOTIFY_MOUNT, LXD_SECCOMP_NOTIFY_BPF, LXD_SECCOMP_NOTIFY_SCHED_SETSCHEDULER, LXD_SECCOMP_NOTIFY_SYSINFO},
 #ifdef AUDIT_ARCH_X86_64
-	{ AUDIT_ARCH_X86_64,      133, 259, 188, 165, 321, 144 },
+	{ AUDIT_ARCH_X86_64,      133, 259, 188, 165, 321, 144, 99 },
 #endif
 #ifdef AUDIT_ARCH_I386
-	{ AUDIT_ARCH_I386,         14, 297, 226,  21, 357, 156 },
+	{ AUDIT_ARCH_I386,         14, 297, 226,  21, 357, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_AARCH64
-	{ AUDIT_ARCH_AARCH64,      -1,  33,   5,  21, 386, 156 },
+	{ AUDIT_ARCH_AARCH64,      -1,  33,   5,  21, 386, 156, 179 },
 #endif
 #ifdef AUDIT_ARCH_ARM
-	{ AUDIT_ARCH_ARM,          14, 324, 226,  21, 386, 156 },
+	{ AUDIT_ARCH_ARM,          14, 324, 226,  21, 386, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_ARMEB
-	{ AUDIT_ARCH_ARMEB,        14, 324, 226,  21, 386, 156 },
+	{ AUDIT_ARCH_ARMEB,        14, 324, 226,  21, 386, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_S390
-	{ AUDIT_ARCH_S390,         14, 290, 224,  21, 386, 156 },
+	{ AUDIT_ARCH_S390,         14, 290, 224,  21, 386, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_S390X
-	{ AUDIT_ARCH_S390X,        14, 290, 224,  21, 351, 156 },
+	{ AUDIT_ARCH_S390X,        14, 290, 224,  21, 351, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_PPC
-	{ AUDIT_ARCH_PPC,          14, 288, 209,  21, 361, 156 },
+	{ AUDIT_ARCH_PPC,          14, 288, 209,  21, 361, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_PPC64
-	{ AUDIT_ARCH_PPC64,        14, 288, 209,  21, 361, 156 },
+	{ AUDIT_ARCH_PPC64,        14, 288, 209,  21, 361, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_PPC64LE
-	{ AUDIT_ARCH_PPC64LE,      14, 288, 209,  21, 361, 156 },
+	{ AUDIT_ARCH_PPC64LE,      14, 288, 209,  21, 361, 156, 116 },
 #endif
 #ifdef AUDIT_ARCH_RISCV64
-	{ AUDIT_ARCH_RISCV64,      -1,  33,   5,  40, 280, -1 },
+	{ AUDIT_ARCH_RISCV64,      -1,  33,   5,  40, 280, -1, 179 },
 #endif
 #ifdef AUDIT_ARCH_SPARC
-	{ AUDIT_ARCH_SPARC,        14, 286, 169, 167, 349, 243 },
+	{ AUDIT_ARCH_SPARC,        14, 286, 169, 167, 349, 243, 214 },
 #endif
 #ifdef AUDIT_ARCH_SPARC64
-	{ AUDIT_ARCH_SPARC64,      14, 286, 169, 167, 349, 243 },
+	{ AUDIT_ARCH_SPARC64,      14, 286, 169, 167, 349, 243, 214 },
 #endif
 #ifdef AUDIT_ARCH_MIPS
-	{ AUDIT_ARCH_MIPS,         14, 290, 224,  21,  -1, 141 },
+	{ AUDIT_ARCH_MIPS,         14, 290, 224,  21,  -1, 141, 4116 },
 #endif
 #ifdef AUDIT_ARCH_MIPSEL
-	{ AUDIT_ARCH_MIPSEL,       14, 290, 224,  21,  -1, 141 },
+	{ AUDIT_ARCH_MIPSEL,       14, 290, 224,  21,  -1, 141, 4116 },
 #endif
 #ifdef AUDIT_ARCH_MIPS64
-	{ AUDIT_ARCH_MIPS64,      131, 249, 180, 160,  -1, 141 },
+	{ AUDIT_ARCH_MIPS64,      131, 249, 180, 160,  -1, 141, 5097 },
 #endif
 #ifdef AUDIT_ARCH_MIPS64N32
-	{ AUDIT_ARCH_MIPS64N32,   131, 253, 180, 160,  -1, 141 },
+	{ AUDIT_ARCH_MIPS64N32,   131, 253, 180, 160,  -1, 141, 4116 },
 #endif
 #ifdef AUDIT_ARCH_MIPSEL64
-	{ AUDIT_ARCH_MIPSEL64,    131, 249, 180, 160,  -1, 141 },
+	{ AUDIT_ARCH_MIPSEL64,    131, 249, 180, 160,  -1, 141, 5097 },
 #endif
 #ifdef AUDIT_ARCH_MIPSEL64N32
-	{ AUDIT_ARCH_MIPSEL64N32, 131, 253, 180, 160,  -1, 141 },
+	{ AUDIT_ARCH_MIPSEL64N32, 131, 253, 180, 160,  -1, 141, 4116 },
 #endif
 };
 
@@ -233,6 +205,9 @@ static int seccomp_notify_get_syscall(struct seccomp_notif *req,
 		if (entry->nr_sched_setscheduler == req->data.nr)
 			return LXD_SECCOMP_NOTIFY_SCHED_SETSCHEDULER;
 
+		if (entry->nr_sysinfo == req->data.nr)
+			return LXD_SECCOMP_NOTIFY_SYSINFO;
+
 		break;
 	}
 
@@ -276,7 +251,7 @@ static inline int pidfd_getfd(int pidfd, int fd, int flags)
 	return syscall(__NR_pidfd_getfd, pidfd, fd, flags);
 }
 
-#define ptr_to_u64(p) ((__aligned_u64)((uintptr_t)(p)))
+#define ptr_to_u64(p) ((__u64)((uintptr_t)(p)))
 
 static inline int bpf(int cmd, union bpf_attr *attr, size_t size)
 {
@@ -468,12 +443,46 @@ static int handle_bpf_syscall(pid_t pid_target, int notify_fd, int mem_fd,
 */
 import "C"
 
+import (
+	"context"
+	"fmt"
+	"io"
+	"net"
+	"os"
+	"path/filepath"
+	"regexp"
+	"runtime"
+	"strconv"
+	"strings"
+	"time"
+	"unsafe"
+
+	liblxc "github.com/lxc/go-lxc"
+	"golang.org/x/sys/unix"
+
+	"github.com/canonical/lxd/lxd/cgroup"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/idmap"
+	_ "github.com/canonical/lxd/lxd/include" // Used by cgo
+	"github.com/canonical/lxd/lxd/linux"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/ucred"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/netutils"
+	"github.com/canonical/lxd/shared/osarch"
+)
+
 const lxdSeccompNotifyMknod = C.LXD_SECCOMP_NOTIFY_MKNOD
 const lxdSeccompNotifyMknodat = C.LXD_SECCOMP_NOTIFY_MKNODAT
 const lxdSeccompNotifySetxattr = C.LXD_SECCOMP_NOTIFY_SETXATTR
 const lxdSeccompNotifyMount = C.LXD_SECCOMP_NOTIFY_MOUNT
 const lxdSeccompNotifyBpf = C.LXD_SECCOMP_NOTIFY_BPF
 const lxdSeccompNotifySchedSetscheduler = C.LXD_SECCOMP_NOTIFY_SCHED_SETSCHEDULER
+const lxdSeccompNotifySysinfo = C.LXD_SECCOMP_NOTIFY_SYSINFO
 
 const seccompHeader = `2
 `
@@ -487,7 +496,8 @@ finit_module errno 38
 delete_module errno 38
 `
 
-//          8 == SECCOMP_FILTER_FLAG_NEW_LISTENER
+//	8 == SECCOMP_FILTER_FLAG_NEW_LISTENER
+//
 // 2146435072 == SECCOMP_RET_TRACE
 const seccompNotifyDisallow = `seccomp errno 22 [1,2146435072,SCMP_CMP_MASKED_EQ,2146435072]
 seccomp errno 22 [1,8,SCMP_CMP_MASKED_EQ,8]
@@ -504,6 +514,9 @@ const seccompNotifySetxattr = `setxattr notify [3,1,SCMP_CMP_EQ]
 const seccompNotifySchedSetscheduler = `sched_setscheduler notify
 `
 
+const seccompNotifySysinfo = `sysinfo notify
+`
+
 const seccompBlockNewMountAPI = `fsopen errno 38
 fsconfig errno 38
 fsinfo errno 38
@@ -600,9 +613,10 @@ type Instance interface {
 	IsPrivileged() bool
 	Architecture() int
 	RootfsPath() string
+	CGroup() (*cgroup.CGroup, error)
 	CurrentIdmap() (*idmap.IdmapSet, error)
 	DiskIdmap() (*idmap.IdmapSet, error)
-	IdmappedStorage(path string) idmap.IdmapStorageType
+	IdmappedStorage(path string, fstype string) idmap.IdmapStorageType
 	InsertSeccompUnixDevice(prefix string, m deviceConfig.Device, pid int) error
 }
 
@@ -610,7 +624,7 @@ var seccompPath = shared.VarPath("security", "seccomp")
 
 // ProfilePath returns the seccomp path for the instance.
 func ProfilePath(c Instance) string {
-	return path.Join(seccompPath, project.Instance(c.Project().Name, c.Name()))
+	return filepath.Join(seccompPath, project.Instance(c.Project().Name, c.Name()))
 }
 
 // InstanceNeedsPolicy returns whether the instance needs a policy or not.
@@ -622,8 +636,6 @@ func InstanceNeedsPolicy(c Instance) bool {
 		"raw.seccomp",
 		"security.syscalls.allow",
 		"security.syscalls.deny",
-		"security.syscalls.whitelist",
-		"security.syscalls.blacklist",
 	}
 
 	for _, k := range keys {
@@ -636,10 +648,10 @@ func InstanceNeedsPolicy(c Instance) bool {
 	// Check for boolean keys that default to false
 	keys = []string{
 		"security.syscalls.deny_compat",
-		"security.syscalls.blacklist_compat",
 		"security.syscalls.intercept.mknod",
 		"security.syscalls.intercept.sched_setscheduler",
 		"security.syscalls.intercept.setxattr",
+		"security.syscalls.intercept.sysinfo",
 		"security.syscalls.intercept.mount",
 		"security.syscalls.intercept.bpf",
 	}
@@ -652,10 +664,6 @@ func InstanceNeedsPolicy(c Instance) bool {
 
 	// Check for boolean keys that default to true
 	value, ok := config["security.syscalls.deny_default"]
-	if !ok {
-		value, ok = config["security.syscalls.blacklist_default"]
-	}
-
 	if !ok || shared.IsTrue(value) {
 		return true
 	}
@@ -681,6 +689,7 @@ func InstanceNeedsIntercept(s *state.State, c Instance) (bool, error) {
 		"security.syscalls.intercept.mknod":              lxcSupportSeccompNotify,
 		"security.syscalls.intercept.sched_setscheduler": lxcSupportSeccompNotify,
 		"security.syscalls.intercept.setxattr":           lxcSupportSeccompNotify,
+		"security.syscalls.intercept.sysinfo":            lxcSupportSeccompNotify,
 		"security.syscalls.intercept.mount":              lxcSupportSeccompNotifyContinue,
 		"security.syscalls.intercept.bpf":                lxcSupportSeccompNotifyAddfd,
 	}
@@ -705,7 +714,7 @@ func InstanceNeedsIntercept(s *state.State, c Instance) (bool, error) {
 // MakePidFd prepares a pidfd to inherit for the init process of the container.
 func MakePidFd(pid int, s *state.State) (int, *os.File) {
 	if s.OS.PidFds {
-		pidFdFile, err := shared.PidFdOpen(pid, 0)
+		pidFdFile, err := linux.PidFdOpen(pid, 0)
 		if err != nil {
 			return -1, nil
 		}
@@ -728,30 +737,22 @@ func seccompGetPolicyContent(s *state.State, c Instance) (string, error) {
 	// Policy header
 	policy := seccompHeader
 	allowlist := config["security.syscalls.allow"]
-	if allowlist == "" {
-		allowlist = config["security.syscalls.whitelist"]
-	}
 
 	if allowlist != "" {
-		if s.OS.LXCFeatures["seccomp_allow_deny_syntax"] {
-			policy += "allowlist\n[all]\n"
-		} else {
-			policy += "whitelist\n[all]\n"
+		if !s.OS.LXCFeatures["seccomp_allow_deny_syntax"] {
+			return "", fmt.Errorf("Unable to configure allowlist, liblxc is does not support: %q", "seccomp_allow_deny_syntax")
 		}
 
+		policy += "allowlist\n[all]\n"
 		policy += allowlist
 	} else {
-		if s.OS.LXCFeatures["seccomp_allow_deny_syntax"] {
-			policy += "denylist\n[all]\n"
-		} else {
-			policy += "blacklist\n[all]\n"
+		if !s.OS.LXCFeatures["seccomp_allow_deny_syntax"] {
+			return "", fmt.Errorf("Unable to configure denylist, liblxc is does not support: %q", "seccomp_allow_deny_syntax")
 		}
 
+		policy += "denylist\n[all]\n"
+
 		defaultFlag, ok := config["security.syscalls.deny_default"]
-		if !ok {
-			defaultFlag, ok = config["security.syscalls.blacklist_default"]
-		}
-
 		if !ok || shared.IsTrue(defaultFlag) {
 			policy += defaultSeccompPolicy
 		}
@@ -780,6 +781,10 @@ func seccompGetPolicyContent(s *state.State, c Instance) (string, error) {
 			policy += seccompNotifySetxattr
 		}
 
+		if shared.IsTrue(config["security.syscalls.intercept.sysinfo"]) {
+			policy += seccompNotifySysinfo
+		}
+
 		if shared.IsTrue(config["security.syscalls.intercept.mount"]) {
 			policy += seccompNotifyMount
 			// We block the new mount api for now to simplify mount
@@ -799,11 +804,7 @@ func seccompGetPolicyContent(s *state.State, c Instance) (string, error) {
 	}
 
 	// Additional deny entries
-	compat, ok := config["security.syscalls.deny_compat"]
-	if !ok {
-		compat = config["security.syscalls.blacklist_compat"]
-	}
-
+	compat := config["security.syscalls.deny_compat"]
 	if shared.IsTrue(compat) {
 		arch, err := osarch.ArchitectureName(c.Architecture())
 		if err != nil {
@@ -813,11 +814,7 @@ func seccompGetPolicyContent(s *state.State, c Instance) (string, error) {
 		policy += fmt.Sprintf(compatBlockingPolicy, arch)
 	}
 
-	denylist, ok := config["security.syscalls.deny"]
-	if !ok {
-		denylist = config["security.syscalls.blacklist"]
-	}
-
+	denylist := config["security.syscalls.deny"]
 	if denylist != "" {
 		policy += denylist
 	}
@@ -1098,8 +1095,16 @@ func TaskIDs(pid int) (int64, int64, int64, int64, error) {
 		return -1, -1, -1, -1, err
 	}
 
-	reUID := regexp.MustCompile(`^Uid:\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)`)
-	reGID := regexp.MustCompile(`^Gid:\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)`)
+	reUID, err := regexp.Compile(`^Uid:\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)`)
+	if err != nil {
+		return -1, -1, -1, -1, err
+	}
+
+	reGID, err := regexp.Compile(`^Gid:\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+([0-9]+)`)
+	if err != nil {
+		return -1, -1, -1, -1, err
+	}
+
 	var UID int64 = -1
 	var GID int64 = -1
 	var fsUID int64 = -1
@@ -1168,34 +1173,38 @@ func TaskIDs(pid int) (int64, int64, int64, int64, error) {
 }
 
 // FindTGID returns the task group leader ID from /proc/<pid> fd
-func FindTGID(procFd int) (int, error) {
+func FindTGID(procFd int) (uint32, error) {
 	var statusFile *os.File
 	fd, err := unix.Openat(procFd, "status", unix.O_RDONLY|unix.O_CLOEXEC, 0)
 	if err != nil {
-		return -1, err
+		return 0, err
 	}
 
 	statusFile = os.NewFile(uintptr(fd), "/proc/<pid>/status")
 	status, err := io.ReadAll(statusFile)
 	_ = statusFile.Close()
 	if err != nil {
-		return -1, err
+		return 0, err
+	}
+
+	reTGID, err := regexp.Compile(`^Tgid:\s+([0-9]+)`)
+	if err != nil {
+		return 0, err
 	}
 
-	reTGID := regexp.MustCompile(`^Tgid:\s+([0-9]+)`)
 	for _, line := range strings.Split(string(status), "\n") {
 		m := reTGID.FindStringSubmatch(line)
 		if len(m) > 1 {
 			result, err := strconv.ParseUint(m[1], 10, 32)
 			if err != nil {
-				return -1, err
+				return 0, err
 			}
 
-			return int(result), nil
+			return uint32(result), nil
 		}
 	}
 
-	return -1, nil
+	return 0, fmt.Errorf("Task group leader ID not found")
 }
 
 // CallForkmknod executes fork mknod.
@@ -1697,6 +1706,156 @@ func (s *Server) HandleSchedSetschedulerSyscall(c Instance, siov *Iovec) int {
 	return 0
 }
 
+// HandleSysinfoSyscall handles sysinfo syscalls.
+func (s *Server) HandleSysinfoSyscall(c Instance, siov *Iovec) int {
+	l := logger.AddContext(logger.Ctx{
+		"container":             c.Name(),
+		"project":               c.Project().Name,
+		"syscall_number":        siov.req.data.nr,
+		"audit_architecture":    siov.req.data.arch,
+		"seccomp_notify_id":     siov.req.id,
+		"seccomp_notify_flags":  siov.req.flags,
+		"seccomp_notify_pid":    siov.req.pid,
+		"seccomp_notify_fd":     siov.notifyFd,
+		"seccomp_notify_mem_fd": siov.memFd,
+	})
+
+	defer l.Debug("Handling sysinfo syscall")
+
+	// Pre-fill sysinfo struct with metrics from host system.
+	info := unix.Sysinfo_t{}
+	err := unix.Sysinfo(&info)
+	if err != nil {
+		l.Warn("Failed getting sysinfo", logger.Ctx{"err": err})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	instMetrics := Sysinfo{} // Architecture independent place to hold instance metrics.
+
+	cg, err := cgroup.NewFileReadWriter(int(siov.msg.init_pid), liblxc.HasAPIExtension("cgroup2"))
+	if err != nil {
+		l.Warn("Failed loading cgroup", logger.Ctx{"err": err, "pid": siov.msg.init_pid})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	// Get instance uptime.
+	pidStat, err := os.ReadFile(fmt.Sprintf("/proc/%d/stat", siov.msg.init_pid))
+	if err != nil {
+		l.Warn("Failed getting init process info", logger.Ctx{"err": err, "pid": siov.msg.init_pid})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	fields := strings.Fields(string(pidStat))
+	tickValue, err := strconv.ParseInt(fields[21], 10, 64)
+	if err != nil {
+		l.Warn("Failed parsing init process info", logger.Ctx{"err": err, "pid": siov.msg.init_pid})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	age := float64(tickValue / 100)
+	if age > 0 {
+		instMetrics.Uptime = int64(time.Since(s.s.OS.BootTime).Seconds() - age)
+	}
+
+	// Get instance process count.
+	pids, err := cg.GetProcessesUsage()
+	if err != nil {
+		l.Warn("Failed getting process count", logger.Ctx{"err": err})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	instMetrics.Procs = uint16(pids)
+
+	// Get instance memory stats.
+	memStats, err := cg.GetMemoryStats()
+	if err != nil {
+		l.Warn("Failed getting memory stats", logger.Ctx{"err": err})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	for k, v := range memStats {
+		switch k {
+		case "shmem":
+			instMetrics.Sharedram = v
+		case "cache":
+			instMetrics.Bufferram = v
+		}
+	}
+
+	// Get instance memory limit.
+	memoryLimit, err := cg.GetEffectiveMemoryLimit()
+	if err != nil {
+		l.Warn("Failed getting effective memory limit", logger.Ctx{"err": err})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	// Get instance memory usage.
+	memoryUsage, err := cg.GetMemoryUsage()
+	if err != nil {
+		l.Warn("Failed to get memory usage", logger.Ctx{"err": err})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	instMetrics.Totalram = uint64(memoryLimit)
+	instMetrics.Freeram = instMetrics.Totalram - uint64(memoryUsage) - instMetrics.Bufferram
+
+	// Get instance swap info.
+	if s.s.OS.CGInfo.Supports(cgroup.MemorySwapUsage, cg) {
+		swapLimit, err := cg.GetMemorySwapLimit()
+		if err != nil {
+			l.Warn("Failed getting swap limit", logger.Ctx{"err": err})
+			C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+			return 0
+		}
+
+		swapUsage, err := cg.GetMemorySwapUsage()
+		if err != nil {
+			l.Warn("Failed getting swap usage", logger.Ctx{"err": err})
+			C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+			return 0
+		}
+
+		instMetrics.Totalswap = uint64(swapLimit)
+		instMetrics.Freeswap = instMetrics.Totalswap - uint64(swapUsage)
+	}
+
+	// Get writable pointer to buffer of sysinfo syscall result.
+	const sz = int(unsafe.Sizeof(info))
+	var b []byte = (*(*[sz]byte)(unsafe.Pointer(&info)))[:]
+
+	// Write instance metrics to native sysinfo struct.
+	instMetrics.ToNative(&info)
+
+	// Write sysinfo response into buffer.
+	_, err = unix.Pwrite(siov.memFd, b, int64(siov.req.data.args[0]))
+	if err != nil {
+		l.Warn("Failed writing sysinfo", logger.Ctx{"err": err})
+		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
+
+		return 0
+	}
+
+	return 0
+}
+
 // MountArgs arguments for mount.
 type MountArgs struct {
 	source    string
@@ -1888,7 +2047,6 @@ func (s *Server) HandleMountSyscall(c Instance, siov *Iovec) int {
 	}
 	args.source = C.GoString(&mntSource[0])
 	ctx["source"] = args.source
-	args.idmapType = s.MountSyscallShift(c, args.source)
 
 	// const char *target
 	if siov.req.data.args[1] != 0 {
@@ -1916,6 +2074,14 @@ func (s *Server) HandleMountSyscall(c Instance, siov *Iovec) int {
 	args.fstype = C.GoString(&mntFs[0])
 	ctx["fstype"] = args.fstype
 
+	// idmap shift
+	fullSrcPath := filepath.Join(fmt.Sprintf("/proc/%d/root/", args.pid), args.source)
+	if shared.PathExists(fullSrcPath) {
+		args.idmapType = s.MountSyscallShift(c, fullSrcPath)
+	} else {
+		args.idmapType = s.MountSyscallShift(c, args.source)
+	}
+
 	// unsigned long mountflags
 	args.flags = int(siov.req.data.args[3])
 
@@ -1932,7 +2098,7 @@ func (s *Server) HandleMountSyscall(c Instance, siov *Iovec) int {
 	args.data = C.GoString(&mntData[0])
 	ctx["data"] = args.data
 
-	err := shared.PidfdSendSignal(int(pidFd.Fd()), 0, 0)
+	err := linux.PidfdSendSignal(int(pidFd.Fd()), 0, 0)
 	if err != nil {
 		ctx["err"] = fmt.Sprintf("Failed to send signal to target process for of mount syscall: %s", err)
 		ctx["syscall_continue"] = "true"
@@ -2077,7 +2243,7 @@ func (s *Server) HandleBpfSyscall(c Instance, siov *Iovec) int {
 	}
 
 	tgid, err := FindTGID(siov.procFd)
-	if err != nil || tgid == -1 {
+	if err != nil {
 		ctx["syscall_continue"] = "true"
 		ctx["syscall_handler_reason"] = "Could not find thread group leader ID"
 		C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
@@ -2126,6 +2292,8 @@ func (s *Server) handleSyscall(c Instance, siov *Iovec) int {
 		return s.HandleBpfSyscall(c, siov)
 	case lxdSeccompNotifySchedSetscheduler:
 		return s.HandleSchedSetschedulerSyscall(c, siov)
+	case lxdSeccompNotifySysinfo:
+		return s.HandleSysinfoSyscall(c, siov)
 	}
 
 	return int(-C.EINVAL)
@@ -2284,7 +2452,7 @@ func (s *Server) MountSyscallValid(c Instance, args *MountArgs) (bool, string) {
 	return false, ""
 }
 
-// MountSyscallShift checks whether this mount syscall needs shiftfs.
+// MountSyscallShift checks whether this mount syscall needs shifting.
 func (s *Server) MountSyscallShift(c Instance, path string) idmap.IdmapStorageType {
 	if shared.IsTrue(c.ExpandedConfig()["security.syscalls.intercept.mount.shift"]) {
 		diskIdmap, err := c.DiskIdmap()
@@ -2293,7 +2461,7 @@ func (s *Server) MountSyscallShift(c Instance, path string) idmap.IdmapStorageTy
 		}
 
 		if diskIdmap == nil {
-			return c.IdmappedStorage(path)
+			return c.IdmappedStorage(path, "none")
 		}
 	}
 
