@@ -6,10 +6,10 @@ import (
 	"context"
 	"testing"
 
-	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/firewall"
-	"github.com/lxc/lxd/lxd/sys"
+	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/firewall"
+	"github.com/canonical/lxd/lxd/sys"
 )
 
 // NewTestState returns a State object initialized with testable instances of
@@ -29,12 +29,12 @@ func NewTestState(t *testing.T) (*State, func()) {
 	}
 
 	state := &State{
-		ShutdownCtx:            context.TODO(),
-		DB:                     &db.DB{Node: node, Cluster: cluster},
-		OS:                     os,
-		Firewall:               firewall.New(),
-		UpdateCertificateCache: func() {},
-		GlobalConfig:           &clusterConfig.Config{},
+		ShutdownCtx:         context.TODO(),
+		DB:                  &db.DB{Node: node, Cluster: cluster},
+		OS:                  os,
+		Firewall:            firewall.New(),
+		UpdateIdentityCache: func() {},
+		GlobalConfig:        &clusterConfig.Config{},
 	}
 
 	return state, cleanup
