@@ -9,14 +9,16 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/lxd/migration"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/validate"
-	"github.com/lxc/lxd/shared/version"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/migration"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/validate"
+	"github.com/canonical/lxd/shared/version"
 )
 
 var btrfsVersion string
@@ -31,8 +33,11 @@ type btrfs struct {
 func (d *btrfs) load() error {
 	// Register the patches.
 	d.patches = map[string]func() error{
-		"storage_lvm_skipactivation":       nil,
-		"storage_missing_snapshot_records": nil,
+		"storage_lvm_skipactivation":                         nil,
+		"storage_missing_snapshot_records":                   nil,
+		"storage_delete_old_snapshot_records":                nil,
+		"storage_zfs_drop_block_volume_filesystem_extension": nil,
+		"storage_prefix_bucket_names_with_project":           nil,
 	}
 
 	// Done if previously loaded.
@@ -84,34 +89,28 @@ func (d *btrfs) load() error {
 // Info returns info about the driver and its environment.
 func (d *btrfs) Info() Info {
 	return Info{
-		Name:                  "btrfs",
-		Version:               btrfsVersion,
-		OptimizedImages:       true,
-		OptimizedBackups:      true,
-		OptimizedBackupHeader: true,
-		PreservesInodes:       !d.state.OS.RunningInUserNS,
-		Remote:                d.isRemote(),
-		VolumeTypes:           []VolumeType{VolumeTypeBucket, VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
-		BlockBacking:          false,
-		RunningCopyFreeze:     false,
-		DirectIO:              true,
-		IOUring:               true,
-		MountedRoot:           true,
-		Buckets:               true,
+		Name:                         "btrfs",
+		Version:                      btrfsVersion,
+		DefaultVMBlockFilesystemSize: deviceConfig.DefaultVMBlockFilesystemSize,
+		OptimizedImages:              true,
+		OptimizedBackups:             true,
+		OptimizedBackupHeader:        true,
+		PreservesInodes:              !d.state.OS.RunningInUserNS,
+		Remote:                       d.isRemote(),
+		VolumeTypes:                  []VolumeType{VolumeTypeBucket, VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
+		BlockBacking:                 false,
+		RunningCopyFreeze:            false,
+		DirectIO:                     true,
+		IOUring:                      true,
+		MountedRoot:                  true,
+		Buckets:                      true,
 	}
 }
 
-// Create is called during pool creation and is effectively using an empty driver struct.
-// WARNING: The Create() function cannot rely on any of the struct attributes being set.
-func (d *btrfs) Create() error {
-	// Store the provided source as we are likely to be mangling it.
-	d.config["volatile.initial_source"] = d.config["source"]
-
+// FillConfig populates the storage pool's configuration file with the default values.
+func (d *btrfs) FillConfig() error {
 	loopPath := loopFilePath(d.name)
 	if d.config["source"] == "" || d.config["source"] == loopPath {
-		// Create a loop based pool.
-		d.config["source"] = loopPath
-
 		// Pick a default size of the loop file if not specified.
 		if d.config["size"] == "" {
 			defaultSize, err := loopFileSizeDefault()
@@ -121,6 +120,44 @@ func (d *btrfs) Create() error {
 
 			d.config["size"] = fmt.Sprintf("%dGiB", defaultSize)
 		}
+	} else {
+		// Unset size property since it's irrelevant.
+		d.config["size"] = ""
+	}
+
+	// Store the provided source as we are likely to be mangling it.
+	d.config["volatile.initial_source"] = d.config["source"]
+
+	// Set the block device's UUID in case it already has one.
+	// This allows to recover the pools configuration without actually
+	// creating the storage pool.
+	// Downstream functions should use `volatile.initial_source` to ensure
+	// they are using the path instead of the volume's UUID.
+	if shared.IsBlockdevPath(d.config["source"]) {
+		devUUID, err := fsUUID(d.config["source"])
+		if err == nil {
+			d.config["source"] = devUUID
+		}
+	}
+
+	return nil
+}
+
+// Create is called during pool creation and is effectively using an empty driver struct.
+// WARNING: The Create() function cannot rely on any of the struct attributes being set.
+func (d *btrfs) Create() error {
+	revert := revert.New()
+	defer revert.Fail()
+
+	err := d.FillConfig()
+	if err != nil {
+		return err
+	}
+
+	loopPath := loopFilePath(d.name)
+	if d.config["source"] == "" || d.config["source"] == loopPath {
+		// Create a loop based pool.
+		d.config["source"] = loopPath
 
 		// Create the loop file itself.
 		size, err := units.ParseByteSizeString(d.config["size"])
@@ -133,46 +170,58 @@ func (d *btrfs) Create() error {
 			return fmt.Errorf("Failed to create the sparse file: %w", err)
 		}
 
+		revert.Add(func() { _ = os.Remove(d.config["source"]) })
+
 		// Format the file.
 		_, err = makeFSType(d.config["source"], "btrfs", &mkfsOptions{Label: d.name})
 		if err != nil {
 			return fmt.Errorf("Failed to format sparse file: %w", err)
 		}
-	} else if shared.IsBlockdevPath(d.config["source"]) {
-		// Unset size property since it's irrelevant.
-		d.config["size"] = ""
+	} else if shared.IsBlockdevPath(d.config["volatile.initial_source"]) {
+		// Make sure to use the block volumes `volatile.initial_source` here
+		// as an earlier call to the drivers FillConfig() might have set
+		// the `source` property to the block volumes UUID in case it's not empty.
+
+		// Wipe if requested.
+		if shared.IsTrue(d.config["source.wipe"]) {
+			err := wipeBlockHeaders(d.config["volatile.initial_source"])
+			if err != nil {
+				return fmt.Errorf("Failed to wipe headers from disk %q: %w", d.config["volatile.initial_source"], err)
+			}
+
+			d.config["source.wipe"] = ""
+		}
 
 		// Format the block device.
-		_, err := makeFSType(d.config["source"], "btrfs", &mkfsOptions{Label: d.name})
+		_, err := makeFSType(d.config["volatile.initial_source"], "btrfs", &mkfsOptions{Label: d.name})
 		if err != nil {
 			return fmt.Errorf("Failed to format block device: %w", err)
 		}
 
 		// Record the UUID as the source.
-		devUUID, err := fsUUID(d.config["source"])
+		devUUID, err := fsUUID(d.config["volatile.initial_source"])
 		if err != nil {
 			return err
 		}
 
 		// Confirm that the symlink is appearing (give it 10s).
+		// In case of timeout it falls back to using the volume's path
+		// instead of its UUID.
 		if tryExists(fmt.Sprintf("/dev/disk/by-uuid/%s", devUUID)) {
 			// Override the config to use the UUID.
 			d.config["source"] = devUUID
 		}
 	} else if d.config["source"] != "" {
-		// Unset size property since it's irrelevant.
-		d.config["size"] = ""
-
 		hostPath := shared.HostPath(d.config["source"])
 		if d.isSubvolume(hostPath) {
 			// Existing btrfs subvolume.
-			subvols, err := d.getSubvolumes(hostPath)
+			hasSubvolumes, err := d.hasSubvolumes(hostPath)
 			if err != nil {
 				return fmt.Errorf("Could not determine if existing btrfs subvolume is empty: %w", err)
 			}
 
 			// Check that the provided subvolume is empty.
-			if len(subvols) > 0 {
+			if hasSubvolumes {
 				return fmt.Errorf("Requested btrfs subvolume exists but is not empty")
 			}
 		} else {
@@ -183,7 +232,7 @@ func (d *btrfs) Create() error {
 			if shared.PathExists(hostPath) {
 				hostPathFS, _ := filesystem.Detect(hostPath)
 				if hostPathFS != "btrfs" {
-					return fmt.Errorf("Provided path does not reside on a btrfs filesystem")
+					return fmt.Errorf("Provided path does not reside on a btrfs filesystem (detected %s)", hostPathFS)
 				}
 			}
 
@@ -194,7 +243,7 @@ func (d *btrfs) Create() error {
 
 				storagePoolDirFS, _ := filesystem.Detect(shared.VarPath("storage-pools"))
 				if storagePoolDirFS != "btrfs" {
-					return fmt.Errorf("Provided path does not reside on a btrfs filesystem")
+					return fmt.Errorf("Provided path does not reside on a btrfs filesystem (detected %s)", storagePoolDirFS)
 				}
 
 				// Delete the current directory to replace by subvolume.
@@ -214,6 +263,7 @@ func (d *btrfs) Create() error {
 		return fmt.Errorf(`Invalid "source" property`)
 	}
 
+	revert.Success()
 	return nil
 }
 
@@ -283,7 +333,13 @@ func (d *btrfs) Delete(op *operations.Operation) error {
 // Validate checks that all provide keys are supported and that no conflicting or missing configuration is present.
 func (d *btrfs) Validate(config map[string]string) error {
 	rules := map[string]func(value string) error{
-		"size":                validate.Optional(validate.IsSize),
+		"size": validate.Optional(validate.IsSize),
+		// lxdmeta:generate(entities=storage-btrfs; group=pool-conf; key=btrfs.mount_options)
+		//
+		// ---
+		//  type: string
+		//  defaultdesc: `user_subvol_rm_allowed`
+		//  shortdesc: Mount options for block devices
 		"btrfs.mount_options": validate.IsAny,
 	}
 
@@ -294,23 +350,63 @@ func (d *btrfs) Validate(config map[string]string) error {
 func (d *btrfs) Update(changedConfig map[string]string) error {
 	// We only care about btrfs.mount_options.
 	val, ok := changedConfig["btrfs.mount_options"]
-	if !ok {
-		return nil
+	if ok {
+		// Custom mount options don't work inside containers
+		if d.state.OS.RunningInUserNS {
+			return nil
+		}
+
+		// Trigger a re-mount.
+		d.config["btrfs.mount_options"] = val
+		mntFlags, mntOptions := filesystem.ResolveMountOptions(strings.Split(d.getMountOptions(), ","))
+		mntFlags |= unix.MS_REMOUNT
+
+		err := TryMount("", GetPoolMountPath(d.name), "none", mntFlags, mntOptions)
+		if err != nil {
+			return err
+		}
 	}
 
-	// Custom mount options don't work inside containers
-	if d.state.OS.RunningInUserNS {
-		return nil
-	}
+	size, ok := changedConfig["size"]
+	if ok {
+		// Figure out loop path
+		loopPath := loopFilePath(d.name)
 
-	// Trigger a re-mount.
-	d.config["btrfs.mount_options"] = val
-	mntFlags, mntOptions := resolveMountOptions(d.getMountOptions())
-	mntFlags |= unix.MS_REMOUNT
+		if d.config["source"] != loopPath {
+			return fmt.Errorf("Cannot resize non-loopback pools")
+		}
 
-	err := TryMount("", GetPoolMountPath(d.name), "none", mntFlags, mntOptions)
-	if err != nil {
-		return err
+		// Resize loop file
+		f, err := os.OpenFile(loopPath, os.O_RDWR, 0600)
+		if err != nil {
+			return err
+		}
+
+		defer func() { _ = f.Close() }()
+
+		sizeBytes, _ := units.ParseByteSizeString(size)
+
+		err = f.Truncate(sizeBytes)
+		if err != nil {
+			return err
+		}
+
+		loopDevPath, err := loopDeviceSetup(loopPath)
+		if err != nil {
+			return err
+		}
+
+		defer func() { _ = loopDeviceAutoDetach(loopDevPath) }()
+
+		err = loopDeviceSetCapacity(loopDevPath)
+		if err != nil {
+			return err
+		}
+
+		_, err = shared.RunCommand("btrfs", "filesystem", "resize", "max", GetPoolMountPath(d.name))
+		if err != nil {
+			return err
+		}
 	}
 
 	return nil
@@ -346,7 +442,7 @@ func (d *btrfs) Mount() (bool, error) {
 
 			mntSrcFS, _ := filesystem.Detect(mntSrc)
 			if mntSrcFS != "btrfs" {
-				return false, fmt.Errorf("Source path %q isn't btrfs", mntSrc)
+				return false, fmt.Errorf("Source path %q isn't btrfs (detected %s)", mntSrc, mntSrcFS)
 			}
 		}
 	} else {
@@ -355,7 +451,7 @@ func (d *btrfs) Mount() (bool, error) {
 	}
 
 	// Get the custom mount flags/options.
-	mntFlags, mntOptions := resolveMountOptions(d.getMountOptions())
+	mntFlags, mntOptions := filesystem.ResolveMountOptions(strings.Split(d.getMountOptions(), ","))
 
 	// Handle bind-mounts first.
 	if mntFilesystem == "none" {
@@ -406,7 +502,7 @@ func (d *btrfs) GetResources() (*api.ResourcesStoragePool, error) {
 }
 
 // MigrationType returns the type of transfer methods to be used when doing migrations between pools in preference order.
-func (d *btrfs) MigrationTypes(contentType ContentType, refresh bool) []migration.Type {
+func (d *btrfs) MigrationTypes(contentType ContentType, refresh bool, copySnapshots bool) []migration.Type {
 	var rsyncFeatures []string
 	btrfsFeatures := []string{migration.BTRFSFeatureMigrationHeader, migration.BTRFSFeatureSubvolumes, migration.BTRFSFeatureSubvolumeUUIDs}
 
@@ -422,7 +518,7 @@ func (d *btrfs) MigrationTypes(contentType ContentType, refresh bool) []migratio
 	if d.state.OS.RunningInUserNS {
 		var transportType migration.MigrationFSType
 
-		if contentType == ContentTypeBlock {
+		if IsContentBlock(contentType) {
 			transportType = migration.MigrationFSType_BLOCK_AND_RSYNC
 		} else {
 			transportType = migration.MigrationFSType_RSYNC
@@ -436,7 +532,7 @@ func (d *btrfs) MigrationTypes(contentType ContentType, refresh bool) []migratio
 		}
 	}
 
-	if contentType == ContentTypeBlock {
+	if IsContentBlock(contentType) {
 		return []migration.Type{
 			{
 				FSType:   migration.MigrationFSType_BTRFS,
@@ -449,6 +545,15 @@ func (d *btrfs) MigrationTypes(contentType ContentType, refresh bool) []migratio
 		}
 	}
 
+	if refresh && !copySnapshots {
+		return []migration.Type{
+			{
+				FSType:   migration.MigrationFSType_RSYNC,
+				Features: rsyncFeatures,
+			},
+		}
+	}
+
 	return []migration.Type{
 		{
 			FSType:   migration.MigrationFSType_BTRFS,
