@@ -12,20 +12,23 @@ import (
 	"strings"
 	"time"
 
-	"github.com/pborman/uuid"
+	"github.com/google/uuid"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/units"
 )
 
 // cephBlockVolSuffix suffix used for block content type volumes.
 const cephBlockVolSuffix = ".block"
 
+// cephISOVolSuffix suffix used for iso content type volumes.
+const cephISOVolSuffix = ".iso"
+
 const cephVolumeTypeZombieImage = VolumeType("zombie_image")
 
 // CephDefaultCluster represents the default ceph cluster name.
@@ -36,14 +39,14 @@ const CephDefaultUser = "admin"
 
 // cephVolTypePrefixes maps volume type to storage volume name prefix.
 var cephVolTypePrefixes = map[VolumeType]string{
-	VolumeTypeContainer: db.StoragePoolVolumeTypeNameContainer,
-	VolumeTypeVM:        db.StoragePoolVolumeTypeNameVM,
-	VolumeTypeImage:     db.StoragePoolVolumeTypeNameImage,
-	VolumeTypeCustom:    db.StoragePoolVolumeTypeNameCustom,
+	VolumeTypeContainer: cluster.StoragePoolVolumeTypeNameContainer,
+	VolumeTypeVM:        cluster.StoragePoolVolumeTypeNameVM,
+	VolumeTypeImage:     cluster.StoragePoolVolumeTypeNameImage,
+	VolumeTypeCustom:    cluster.StoragePoolVolumeTypeNameCustom,
 }
 
 // osdPoolExists checks whether a given OSD pool exists.
-func (d *ceph) osdPoolExists() bool {
+func (d *ceph) osdPoolExists() (bool, error) {
 	_, err := shared.RunCommand(
 		"ceph",
 		"--name", fmt.Sprintf("client.%s", d.config["ceph.user.name"]),
@@ -54,16 +57,29 @@ func (d *ceph) osdPoolExists() bool {
 		d.config["ceph.osd.pool_name"],
 		"size")
 
-	return err == nil
+	if err != nil {
+		status, _ := shared.ExitStatus(err)
+		// If the error status code is 2, the pool definitely doesn't exist.
+		if status == 2 {
+			return false, nil
+		}
+
+		// Else, the error status is not 0 or 2,
+		// so we can't be sure if the pool exists or not
+		// as it might be a network issue, an internal ceph issue, etc.
+		return false, err
+	}
+
+	return true, nil
 }
 
 // osdDeletePool destroys an OSD pool.
-// - A call to osdDeletePool will destroy a pool including any storage
-//   volumes that still exist in the pool.
-// - In case the OSD pool that is supposed to be deleted does not exist this
-//   command will still exit 0. This means that if the caller wants to be sure
-//   that this call actually deleted an OSD pool it needs to check for the
-//   existence of the pool first.
+//   - A call to osdDeletePool will destroy a pool including any storage
+//     volumes that still exist in the pool.
+//   - In case the OSD pool that is supposed to be deleted does not exist this
+//     command will still exit 0. This means that if the caller wants to be sure
+//     that this call actually deleted an OSD pool it needs to check for the
+//     existence of the pool first.
 func (d *ceph) osdDeletePool() error {
 	_, err := shared.RunCommand(
 		"ceph",
@@ -122,10 +138,10 @@ func (d *ceph) rbdCreateVolume(vol Volume, size string) error {
 }
 
 // rbdDeleteVolume deletes an RBD storage volume.
-// - In case the RBD storage volume that is supposed to be deleted does not
-//   exist this command will still exit 0. This means that if the caller wants
-//   to be sure that this call actually deleted an RBD storage volume it needs
-//   to check for the existence of the pool first.
+//   - In case the RBD storage volume that is supposed to be deleted does not
+//     exist this command will still exit 0. This means that if the caller wants
+//     to be sure that this call actually deleted an RBD storage volume it needs
+//     to check for the existence of the pool first.
 func (d *ceph) rbdDeleteVolume(vol Volume) error {
 	_, err := shared.RunCommand(
 		"rbd",
@@ -468,12 +484,12 @@ func (d *ceph) rbdRenameVolumeSnapshot(vol Volume, oldSnapshotName string, newSn
 }
 
 // rbdGetVolumeParent will return the snapshot the RBD clone was created from:
-// - If the RBD storage volume is not a clone then this function will return
-//   db.NoSuchObjectError.
-// - The snapshot will be returned as
-//   <osd-pool-name>/<rbd-volume-name>@<rbd-snapshot-name>
-//   The caller will usually want to parse this according to its needs. This
-//   helper library provides two small functions to do this but see below.
+//   - If the RBD storage volume is not a clone then this function will return
+//     db.NoSuchObjectError.
+//   - The snapshot will be returned as
+//     <osd-pool-name>/<rbd-volume-name>@<rbd-snapshot-name>
+//     The caller will usually want to parse this according to its needs. This
+//     helper library provides two small functions to do this but see below.
 func (d *ceph) rbdGetVolumeParent(vol Volume) (string, error) {
 	msg, err := shared.RunCommand(
 		"rbd",
@@ -573,10 +589,12 @@ func (d *ceph) rbdListVolumeSnapshots(vol Volume) ([]string, error) {
 	return snapshots, nil
 }
 
-// copyWithSnapshots creates a non-sparse copy of a container including its snapshots.
+// copyVolumeDiff creates a sparse copy of a volume by exporting and importing the diff
+// between `sourceVolumeName` and its optional `sourceParentSnapshot` onto `targetVolumeName`.
 // This does not introduce a dependency relation between the source RBD storage
 // volume and the target RBD storage volume.
-func (d *ceph) copyWithSnapshots(sourceVolumeName string, targetVolumeName string, sourceParentSnapshot string) error {
+// Unlike the classic RBD export only the modified sectors on the RBD storage volume get copied.
+func (d *ceph) copyVolumeDiff(sourceVolumeName string, targetVolumeName string, sourceParentSnapshot string) error {
 	args := []string{
 		"export-diff",
 		"--id", d.config["ceph.user.name"],
@@ -604,6 +622,8 @@ func (d *ceph) copyWithSnapshots(sourceVolumeName string, targetVolumeName strin
 	rbdRecvCmd.Stdout = os.Stdout
 	rbdRecvCmd.Stderr = os.Stderr
 
+	d.logger.Debug("Copying RBD volume", logger.Ctx{"srcVolName": sourceVolumeName, "volName": targetVolumeName, "srcParentSnap": sourceParentSnapshot})
+
 	err := rbdRecvCmd.Start()
 	if err != nil {
 		return err
@@ -623,22 +643,22 @@ func (d *ceph) copyWithSnapshots(sourceVolumeName string, targetVolumeName strin
 }
 
 // deleteVolume deletes the RBD storage volume of a container including any dependencies.
-// - This function takes care to delete any RBD storage entities that are marked
-//   as zombie and whose existence is solely dependent on the RBD storage volume
-//   for the container to be deleted.
-// - This function will mark any storage entities of the container to be deleted
-//   as zombies in case any RBD storage entities in the storage pool have a
-//   dependency relation with it.
-// - This function uses a C-style convention to return error or success simply
-//   because it is more elegant and simple than the go way.
-//   The function will return
-//   -1 on error
-//    0 if the RBD storage volume has been deleted
-//    1 if the RBD storage volume has been marked as a zombie
-// - deleteVolume in conjunction with deleteVolumeSnapshot
-//   recurses through an OSD storage pool to find and delete any storage
-//   entities that were kept around because of dependency relations but are not
-//   deletable.
+//   - This function takes care to delete any RBD storage entities that are marked
+//     as zombie and whose existence is solely dependent on the RBD storage volume
+//     for the container to be deleted.
+//   - This function will mark any storage entities of the container to be deleted
+//     as zombies in case any RBD storage entities in the storage pool have a
+//     dependency relation with it.
+//   - This function uses a C-style convention to return error or success simply
+//     because it is more elegant and simple than the go way.
+//     The function will return
+//     -1 on error
+//     0 if the RBD storage volume has been deleted
+//     1 if the RBD storage volume has been marked as a zombie
+//   - deleteVolume in conjunction with deleteVolumeSnapshot
+//     recurses through an OSD storage pool to find and delete any storage
+//     entities that were kept around because of dependency relations but are not
+//     deletable.
 func (d *ceph) deleteVolume(vol Volume) (int, error) {
 	snaps, err := d.rbdListVolumeSnapshots(vol)
 	if err == nil {
@@ -663,7 +683,7 @@ func (d *ceph) deleteVolume(vol Volume) (int, error) {
 				return 1, nil
 			}
 
-			newVolumeName := fmt.Sprintf("%s_%s", vol.name, uuid.New())
+			newVolumeName := fmt.Sprintf("%s_%s", vol.name, uuid.New().String())
 			err := d.rbdMarkVolumeDeleted(vol, newVolumeName)
 			if err != nil {
 				return -1, err
@@ -733,22 +753,22 @@ func (d *ceph) deleteVolume(vol Volume) (int, error) {
 }
 
 // deleteVolumeSnapshot deletes an RBD snapshot of a container including any dependencies.
-// - This function takes care to delete any RBD storage entities that are marked
-//   as zombie and whose existence is solely dependent on the RBD snapshot for
-//   the container to be deleted.
-// - This function will mark any storage entities of the container to be deleted
-//   as zombies in case any RBD storage entities in the storage pool have a
-//   dependency relation with it.
-// - This function uses a C-style convention to return error or success simply
-//   because it is more elegant and simple than the go way.
-//   The function will return
-//   -1 on error
-//    0 if the RBD storage volume has been deleted
-//    1 if the RBD storage volume has been marked as a zombie
-// - deleteVolumeSnapshot in conjunction with deleteVolume
-//   recurses through an OSD storage pool to find and delete any storage
-//   entities that were kept around because of dependency relations but are not
-//   deletable.
+//   - This function takes care to delete any RBD storage entities that are marked
+//     as zombie and whose existence is solely dependent on the RBD snapshot for
+//     the container to be deleted.
+//   - This function will mark any storage entities of the container to be deleted
+//     as zombies in case any RBD storage entities in the storage pool have a
+//     dependency relation with it.
+//   - This function uses a C-style convention to return error or success simply
+//     because it is more elegant and simple than the go way.
+//     The function will return
+//     -1 on error
+//     0 if the RBD storage volume has been deleted
+//     1 if the RBD storage volume has been marked as a zombie
+//   - deleteVolumeSnapshot in conjunction with deleteVolume
+//     recurses through an OSD storage pool to find and delete any storage
+//     entities that were kept around because of dependency relations but are not
+//     deletable.
 func (d *ceph) deleteVolumeSnapshot(vol Volume, snapshotName string) (int, error) {
 	clones, err := d.rbdListSnapshotClones(vol, snapshotName)
 	if err != nil {
@@ -845,7 +865,7 @@ func (d *ceph) deleteVolumeSnapshot(vol Volume, snapshotName string) (int, error
 			return -1, err
 		}
 
-		newSnapshotName := fmt.Sprintf("zombie_snapshot_%s", uuid.New())
+		newSnapshotName := fmt.Sprintf("zombie_snapshot_%s", uuid.New().String())
 		err = d.rbdRenameVolumeSnapshot(vol, snapshotName, newSnapshotName)
 		if err != nil {
 			return -1, err
@@ -873,7 +893,11 @@ func (d *ceph) parseParent(parent string) (Volume, string, error) {
 	// Looks for volumes like:
 	// pool/zombie_image_9e90b7b9ccdd7a671a987fadcf07ab92363be57e7f056d18d42af452cdaf95bb_ext4.block@readonly
 	// pool/image_9e90b7b9ccdd7a671a987fadcf07ab92363be57e7f056d18d42af452cdaf95bb_xfs
-	reImage := regexp.MustCompile(`^((?:zombie_)?image)_([A-Za-z0-9]+)_([A-Za-z0-9]+)\.?(block)?@?([-\w]+)?$`)
+	reImage, err := regexp.Compile(`^((?:zombie_)?image)_([A-Za-z0-9]+)_([A-Za-z0-9]+)\.?(block)?@?([-\w]+)?$`)
+	if err != nil {
+		return vol, "", err
+	}
+
 	imageRes := reImage.FindStringSubmatch(slider)
 	if imageRes != nil {
 		vol.volType = VolumeType(imageRes[1])
@@ -895,16 +919,23 @@ func (d *ceph) parseParent(parent string) (Volume, string, error) {
 	// Match normal instance volumes.
 	// Looks for volumes like:
 	// pool/container_bar@zombie_snapshot_ce77e971-6c1b-45c0-b193-dba9ec5e7d82
-	reInst := regexp.MustCompile(`^((?:zombie_)?[a-z-]+)_([\w-]+)\.?(block)?@?([-\w]+)?$`)
+	reInst, err := regexp.Compile(`^((?:zombie_)?[a-z-]+)_([\w-]+)\.?(block|iso)?@?([-\w]+)?$`)
+	if err != nil {
+		return vol, "", err
+	}
+
 	instRes := reInst.FindStringSubmatch(slider)
 	if instRes != nil {
 		vol.volType = VolumeType(instRes[1])
 		vol.pool = poolName
 		vol.name = instRes[2]
 
-		if instRes[3] == "block" {
+		switch instRes[3] {
+		case "block":
 			vol.contentType = ContentTypeBlock
-		} else {
+		case "iso":
+			vol.contentType = ContentTypeISO
+		default:
 			vol.contentType = ContentTypeFS
 		}
 
@@ -1037,7 +1068,7 @@ func (d *ceph) getRBDMappedDevPath(vol Volume, mapIfMissing bool) (bool, string,
 			if len(rbdNameParts) == 2 && rbdNameParts[1] == devSnapName {
 				return false, fmt.Sprintf("/dev/rbd%d", idx), nil // We found a match.
 			}
-		} else if shared.StringInSlice(devSnapName, []string{"-", ""}) {
+		} else if shared.ValueInSlice(devSnapName, []string{"-", ""}) {
 			// Volume is not a snapshot and neither is this device.
 			return false, fmt.Sprintf("/dev/rbd%d", idx), nil // We found a match.
 		}
@@ -1106,6 +1137,8 @@ func (d *ceph) getRBDVolumeName(vol Volume, snapName string, zombie bool, withPo
 //	rbd export-diff pool1/container_a@snapshot_snap1 --from-snap snapshot_snap0 - | rbd import-diff - pool2/container_a
 //	rbd export-diff pool1/container_a --from-snap snapshot_snap1 - | rbd import-diff - pool2/container_a
 func (d *ceph) sendVolume(conn io.ReadWriteCloser, volumeName string, volumeParentName string, tracker *ioprogress.ProgressTracker) error {
+	defer func() { _ = conn.Close() }()
+
 	args := []string{
 		"export-diff",
 		"--id", d.config["ceph.user.name"],
@@ -1122,32 +1155,23 @@ func (d *ceph) sendVolume(conn io.ReadWriteCloser, volumeName string, volumePare
 
 	cmd := exec.Command("rbd", args...)
 
-	stdout, err := cmd.StdoutPipe()
-	if err != nil {
-		return err
-	}
-
 	stderr, err := cmd.StderrPipe()
 	if err != nil {
 		return err
 	}
 
 	// Setup progress tracker.
-	stdoutPipe := stdout
+	var stdout io.WriteCloser = conn
 	if tracker != nil {
-		stdoutPipe = &ioprogress.ProgressReader{
-			ReadCloser: stdout,
-			Tracker:    tracker,
+		stdout = &ioprogress.ProgressWriter{
+			WriteCloser: conn,
+			Tracker:     tracker,
 		}
 	}
 
-	// Forward any output on stdout.
-	chStdoutPipe := make(chan error, 1)
-	go func() {
-		_, err := io.Copy(conn, stdoutPipe)
-		chStdoutPipe <- err
-		_ = conn.Close()
-	}()
+	cmd.Stdout = stdout
+
+	d.logger.Debug("Sending RBD volume", logger.Ctx{"volName": volumeName, "volParentName": volumeParentName})
 
 	err = cmd.Start()
 	if err != nil {
@@ -1157,20 +1181,9 @@ func (d *ceph) sendVolume(conn io.ReadWriteCloser, volumeName string, volumePare
 	output, _ := io.ReadAll(stderr)
 
 	// Handle errors.
-	errs := []error{}
-	chStdoutPipeErr := <-chStdoutPipe
-
 	err = cmd.Wait()
 	if err != nil {
-		errs = append(errs, err)
-
-		if chStdoutPipeErr != nil {
-			errs = append(errs, chStdoutPipeErr)
-		}
-	}
-
-	if len(errs) > 0 {
-		return fmt.Errorf("ceph export-diff failed: %v (%s)", errs, string(output))
+		return fmt.Errorf("ceph export-diff failed: %w (%s)", err, string(output))
 	}
 
 	return nil
@@ -1205,6 +1218,8 @@ func (d *ceph) receiveVolume(volumeName string, conn io.ReadWriteCloser, writeWr
 		chCopyConn <- err
 	}()
 
+	d.logger.Debug("Receiving RBD volume", logger.Ctx{"volName": volumeName})
+
 	// Run the command.
 	err = cmd.Start()
 	if err != nil {
@@ -1257,3 +1272,74 @@ func (d *ceph) resizeVolume(vol Volume, sizeBytes int64, allowShrink bool) error
 
 	return err
 }
+
+// findLastCommonSnapshotIndex finds the last common snapshot from the list of targetSnapshots based on its name.
+// The list of targetSnapshots represents the wanted list of snapshots on the target volume.
+// This list is identical to the target volume's snapshots in the database.
+// In refreshSnapshots provide a list of snapshot names that require refresh on the target side.
+// The returned number is the index of the last common snapshot in the list of targetSnapshots.
+//
+// The function identifies the last common snapshot based on the following criteria:
+//  1. (Ideal case)
+//     There aren't any snapshots marked for refresh.
+//     Return the index for the last snapshot as this is the last common one.
+//  2. If the first target snapshot matches the first one that requires refresh,
+//     there isn't any common snapshot. Return -1.
+//  3. If the last target snapshot is not the last one that requires refresh,
+//     the snapshots are out of sync.
+//     The last common snapshot is the predecessor of the first one that requires refresh.
+//     Return the index of the last common snapshot.
+//  4. The target is missing the last x snapshots from the source.
+//     The last common snapshot is the predecessor of the first one that requires refresh.
+//     Return the index of the last common snapshot.
+//  5. If there isn't a single target snapshot return -1.
+func (d *ceph) findLastCommonSnapshotIndex(targetSnapshots []Volume, refreshSnapshots []string) int {
+	if len(targetSnapshots) > 0 {
+		// Case 1:
+		// The volume snapshots on the source and target might look like this:
+		// sourceVol   ->   targetVol
+		// \_ snap0         \_ snap0
+		// \_ snap1         \_ snap1
+		// \_ snap2         \_ snap2
+		if len(refreshSnapshots) == 0 {
+			return len(targetSnapshots) - 1
+		}
+
+		// Case 2:
+		// The volume snapshots on the source and target might look like this:
+		// sourceVol   ->   targetVol
+		// \_ snap0
+		// \_ snap1
+		// \_ snap2
+		_, firstTargetSnapshotName, _ := api.GetParentAndSnapshotName(targetSnapshots[0].name)
+		if firstTargetSnapshotName == refreshSnapshots[0] {
+			return -1
+		}
+
+		// Case 3:
+		// The volume snapshots on the source and target might look like this:
+		// sourceVol   ->   targetVol
+		// \_ snap0         \_ snap0
+		// \_ snap1         \_ snap2
+		// \_ snap2
+		// Case 4:
+		// The volume snapshots on the source and target might look like this:
+		// sourceVol   ->   targetVol
+		// \_ snap0         \_ snap0
+		// \_ snap1         \_ snap1
+		// \_ snap2
+		for i, targetSnapshot := range targetSnapshots {
+			// Find the last common snapshot between the source and target.
+			// Start by looking up the position of the first snapshot that requires a refresh.
+			_, targetSnapshotName, _ := api.GetParentAndSnapshotName(targetSnapshot.name)
+			if targetSnapshotName == refreshSnapshots[0] {
+				return i - 1
+			}
+		}
+	}
+
+	// Case 5.
+	// The volume snapshots on the source and target might look like this:
+	// sourceVol   ->   targetVol
+	return -1
+}
