@@ -10,13 +10,14 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/validate"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 const lvmVgPoolMarker = "lxd_pool" // Indicator tag used to mark volume groups as in use by LXD.
@@ -24,8 +25,6 @@ const lvmVgPoolMarker = "lxd_pool" // Indicator tag used to mark volume groups a
 var lvmLoaded bool
 var lvmVersion string
 
-var lvmAllowedFilesystems = []string{"btrfs", "ext4", "xfs"}
-
 type lvm struct {
 	common
 }
@@ -33,8 +32,11 @@ type lvm struct {
 func (d *lvm) load() error {
 	// Register the patches.
 	d.patches = map[string]func() error{
-		"storage_lvm_skipactivation":       d.patchStorageSkipActivation,
-		"storage_missing_snapshot_records": nil,
+		"storage_lvm_skipactivation":                         d.patchStorageSkipActivation,
+		"storage_missing_snapshot_records":                   nil,
+		"storage_delete_old_snapshot_records":                nil,
+		"storage_zfs_drop_block_volume_filesystem_extension": nil,
+		"storage_prefix_bucket_names_with_project":           nil,
 	}
 
 	// Done if previously loaded.
@@ -83,21 +85,32 @@ func (d *lvm) load() error {
 // Info returns info about the driver and its environment.
 func (d *lvm) Info() Info {
 	return Info{
-		Name:              "lvm",
-		Version:           lvmVersion,
-		OptimizedImages:   d.usesThinpool(), // Only thinpool pools support optimized images.
-		PreservesInodes:   false,
-		Remote:            d.isRemote(),
-		VolumeTypes:       []VolumeType{VolumeTypeBucket, VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
-		BlockBacking:      true,
-		RunningCopyFreeze: true,
-		DirectIO:          true,
-		IOUring:           true,
-		MountedRoot:       false,
-		Buckets:           true,
+		Name:                         "lvm",
+		Version:                      lvmVersion,
+		DefaultVMBlockFilesystemSize: deviceConfig.DefaultVMBlockFilesystemSize,
+		OptimizedImages:              d.usesThinpool(), // Only thinpool pools support optimized images.
+		PreservesInodes:              false,
+		Remote:                       d.isRemote(),
+		VolumeTypes:                  []VolumeType{VolumeTypeBucket, VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
+		BlockBacking:                 true,
+		RunningCopyFreeze:            true,
+		DirectIO:                     true,
+		IOUring:                      true,
+		MountedRoot:                  false,
+		Buckets:                      true,
 	}
 }
 
+// FillConfig populates the storage pool's configuration file with the default values.
+func (d *lvm) FillConfig() error {
+	// Set default thin pool name if not specified.
+	if d.usesThinpool() && d.config["lvm.thinpool_name"] == "" {
+		d.config["lvm.thinpool_name"] = lvmThinpoolDefaultName
+	}
+
+	return nil
+}
+
 // Create creates the storage pool on the storage device.
 func (d *lvm) Create() error {
 	d.config["volatile.initial_source"] = d.config["source"]
@@ -111,12 +124,16 @@ func (d *lvm) Create() error {
 	revert := revert.New()
 	defer revert.Fail()
 
-	// Set default thin pool name if not specified.
-	if d.usesThinpool() && d.config["lvm.thinpool_name"] == "" {
-		d.config["lvm.thinpool_name"] = lvmThinpoolDefaultName
+	err = d.FillConfig()
+	if err != nil {
+		return err
 	}
 
+	var usingLoopFile bool
+
 	if d.config["source"] == "" || d.config["source"] == defaultSource {
+		usingLoopFile = true
+
 		// We are using a LXD internal loopback file.
 		d.config["source"] = defaultSource
 		if d.config["lvm.vg_name"] == "" {
@@ -139,7 +156,7 @@ func (d *lvm) Create() error {
 		}
 
 		if shared.PathExists(d.config["source"]) {
-			return fmt.Errorf("Source file location already exists")
+			return fmt.Errorf("Source file location %q already exists", d.config["source"])
 		}
 
 		err = ensureSparseFile(d.config["source"], size)
@@ -164,17 +181,27 @@ func (d *lvm) Create() error {
 			return err
 		}
 
+		if pvExists {
+			return fmt.Errorf("A physical volume already exists for %q", pvName)
+		}
+
 		// Check if the volume group already exists.
 		vgExists, vgTags, err = d.volumeGroupExists(d.config["lvm.vg_name"])
 		if err != nil {
 			return err
 		}
+
+		if vgExists {
+			return fmt.Errorf("A volume group already exists called %q", d.config["lvm.vg_name"])
+		}
 	} else if filepath.IsAbs(d.config["source"]) {
 		// We are using an existing physical device.
 		srcPath := shared.HostPath(d.config["source"])
 
-		// Size is ignored as the physical device is a fixed size.
-		d.config["size"] = ""
+		// Size is invalid as the physical device is already sized.
+		if d.config["size"] != "" && !d.usesThinpool() {
+			return fmt.Errorf("Cannot specify size when using an existing physical device for non-thin pool")
+		}
 
 		if d.config["lvm.vg_name"] == "" {
 			d.config["lvm.vg_name"] = d.name
@@ -186,6 +213,16 @@ func (d *lvm) Create() error {
 			return fmt.Errorf("Custom loop file locations are not supported")
 		}
 
+		// Wipe if requested.
+		if shared.IsTrue(d.config["source.wipe"]) {
+			err := wipeBlockHeaders(d.config["source"])
+			if err != nil {
+				return fmt.Errorf("Failed to wipe headers from disk %q: %w", d.config["source"], err)
+			}
+
+			d.config["source.wipe"] = ""
+		}
+
 		// Check if the volume group already exists.
 		vgExists, vgTags, err = d.volumeGroupExists(d.config["lvm.vg_name"])
 		if err != nil {
@@ -206,8 +243,10 @@ func (d *lvm) Create() error {
 		// We are using an existing volume group, so physical must exist already.
 		pvExists = true
 
-		// Size is ignored as the existing device is a fixed size.
-		d.config["size"] = ""
+		// Size is invalid as the volume group is already sized.
+		if d.config["size"] != "" && !d.usesThinpool() {
+			return fmt.Errorf("Cannot specify size when using an existing volume group for non-thin pool")
+		}
 
 		if d.config["lvm.vg_name"] != "" && d.config["lvm.vg_name"] != d.config["source"] {
 			return fmt.Errorf("Invalid combination of source and lvm.vg_name properties")
@@ -275,7 +314,7 @@ func (d *lvm) Create() error {
 			}
 
 			// Check the tags on the volume group to check it is not already being used by LXD.
-			if shared.StringInSlice(lvmVgPoolMarker, vgTags) {
+			if shared.ValueInSlice(lvmVgPoolMarker, vgTags) {
 				return fmt.Errorf("Volume group %q is already used by LXD", d.config["lvm.vg_name"])
 			}
 		}
@@ -306,17 +345,31 @@ func (d *lvm) Create() error {
 	}
 
 	// Create thin pool if needed.
-	if d.usesThinpool() && !thinPoolExists {
-		err = d.createDefaultThinPool(d.Info().Version, d.config["lvm.vg_name"], d.thinpoolName(), d.config["lvm.thinpool_metadata_size"])
-		if err != nil {
-			return err
+	if d.usesThinpool() {
+		if !thinPoolExists {
+			var thinpoolSizeBytes int64
+
+			// If not using loop file then the size setting controls the size of the thinpool volume.
+			if !usingLoopFile {
+				thinpoolSizeBytes, err = d.roundedSizeBytesString(d.config["size"])
+				if err != nil {
+					return fmt.Errorf("Invalid size: %w", err)
+				}
+			}
+
+			err = d.createDefaultThinPool(d.Info().Version, d.thinpoolName(), thinpoolSizeBytes)
+			if err != nil {
+				return err
+			}
+
+			d.logger.Debug("Thin pool created", logger.Ctx{"vg_name": d.config["lvm.vg_name"], "thinpool_name": d.thinpoolName()})
+
+			revert.Add(func() {
+				_ = d.removeLogicalVolume(d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName()))
+			})
+		} else if d.config["size"] != "" {
+			return fmt.Errorf("Cannot specify size when using an existing thin pool")
 		}
-
-		d.logger.Debug("Thin pool created", logger.Ctx{"vg_name": d.config["lvm.vg_name"], "thinpool_name": d.thinpoolName()})
-
-		revert.Add(func() {
-			_ = d.removeLogicalVolume(d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName()))
-		})
 	}
 
 	// Mark the volume group with the lvmVgPoolMarker tag to indicate it is now in use by LXD.
@@ -407,7 +460,7 @@ func (d *lvm) Delete(op *operations.Operation) error {
 			d.logger.Debug("Volume group removed", logger.Ctx{"vg_name": d.config["lvm.vg_name"]})
 		} else {
 			// Otherwise just remove the lvmVgPoolMarker tag to indicate LXD no longer uses this VG.
-			if shared.StringInSlice(lvmVgPoolMarker, vgTags) {
+			if shared.ValueInSlice(lvmVgPoolMarker, vgTags) {
 				_, err = shared.TryRunCommand("vgchange", "--deltag", lvmVgPoolMarker, d.config["lvm.vg_name"])
 				if err != nil {
 					return fmt.Errorf("Failed to remove marker tag on volume group for the lvm storage pool: %w", err)
@@ -452,12 +505,42 @@ func (d *lvm) Delete(op *operations.Operation) error {
 
 func (d *lvm) Validate(config map[string]string) error {
 	rules := map[string]func(value string) error{
-		"size":                       validate.Optional(validate.IsSize),
-		"lvm.vg_name":                validate.IsAny,
-		"lvm.thinpool_name":          validate.IsAny,
+		"size": validate.Optional(validate.IsSize),
+		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.vg_name)
+		//
+		// ---
+		//  type: string
+		//  defaultdesc: name of the pool
+		//  shortdesc: Name of the volume group to create
+		"lvm.vg_name": validate.IsAny,
+		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.thinpool_name)
+		//
+		// ---
+		//  type: string
+		//  defaultdesc: `LXDThinPool`
+		//  shortdesc: Thin pool where volumes are created
+		"lvm.thinpool_name": validate.IsAny,
+		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.thinpool_metadata_size)
+		// By default, LVM calculates an appropriate size.
+		// ---
+		//  type: string
+		//  defaultdesc: `0` (auto)
+		//  shortdesc: The size of the thin pool metadata volume
 		"lvm.thinpool_metadata_size": validate.Optional(validate.IsSize),
-		"lvm.use_thinpool":           validate.Optional(validate.IsBool),
-		"lvm.vg.force_reuse":         validate.Optional(validate.IsBool),
+		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.use_thinpool)
+		//
+		// ---
+		//  type: bool
+		//  defaultdesc: `true`
+		//  shortdesc: Whether the storage pool uses a thin pool for logical volumes
+		"lvm.use_thinpool": validate.Optional(validate.IsBool),
+		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.vg.force_reuse)
+		//
+		// ---
+		//  type: bool
+		//  defaultdesc: `false`
+		//  shortdesc: Force using an existing non-empty volume group
+		"lvm.vg.force_reuse": validate.Optional(validate.IsBool),
 	}
 
 	err := d.validatePool(config, rules, d.commonVolumeRules())
@@ -518,6 +601,57 @@ func (d *lvm) Update(changedConfig map[string]string) error {
 		d.logger.Debug("Thin pool volume renamed", logger.Ctx{"vg_name": d.config["lvm.vg_name"], "thinpool": d.thinpoolName(), "new_thinpool": changedConfig["lvm.thinpool_name"]})
 	}
 
+	size, ok := changedConfig["size"]
+	if ok {
+		// Figure out loop path
+		loopPath := loopFilePath(d.name)
+
+		if d.config["source"] != loopPath {
+			return fmt.Errorf("Cannot resize non-loopback pools")
+		}
+
+		// Resize loop file
+		f, err := os.OpenFile(loopPath, os.O_RDWR, 0600)
+		if err != nil {
+			return err
+		}
+
+		defer func() { _ = f.Close() }()
+
+		sizeBytes, _ := units.ParseByteSizeString(size)
+
+		err = f.Truncate(sizeBytes)
+		if err != nil {
+			return err
+		}
+
+		loopDevPath, err := loopDeviceSetup(loopPath)
+		if err != nil {
+			return err
+		}
+
+		err = loopDeviceSetCapacity(loopDevPath)
+		if err != nil {
+			return err
+		}
+
+		// Resize physical volume so that lvresize is able to resize as well.
+		_, err = shared.RunCommand("pvresize", "-y", loopDevPath)
+		if err != nil {
+			return err
+		}
+
+		if d.usesThinpool() {
+			lvPath := d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName())
+
+			// Use the remaining space in the volume group.
+			_, err = shared.RunCommand("lvresize", "-f", "-l", "+100%FREE", lvPath)
+			if err != nil {
+				return err
+			}
+		}
+	}
+
 	return nil
 }
 
@@ -589,7 +723,7 @@ func (d *lvm) Mount() (bool, error) {
 }
 
 // Unmount unmounts the storage pool (this does nothing).
-// LVM doesn't currently support unmounting, please see https://github.com/lxc/lxd/issues/9278
+// LVM doesn't currently support unmounting, please see https://github.com/canonical/lxd/issues/9278
 func (d *lvm) Unmount() (bool, error) {
 	return false, nil
 }
