@@ -8,16 +8,17 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/lxc/lxd/lxd/migration"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/validate"
-	"github.com/lxc/lxd/shared/version"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/migration"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/validate"
+	"github.com/canonical/lxd/shared/version"
 )
 
 var zfsVersion string
@@ -25,9 +26,9 @@ var zfsLoaded bool
 var zfsDirectIO bool
 var zfsTrim bool
 var zfsRaw bool
+var zfsDelegate bool
 
 var zfsDefaultSettings = map[string]string{
-	"atime":      "off",
 	"relatime":   "on",
 	"mountpoint": "legacy",
 	"setuid":     "on",
@@ -45,8 +46,11 @@ type zfs struct {
 func (d *zfs) load() error {
 	// Register the patches.
 	d.patches = map[string]func() error{
-		"storage_lvm_skipactivation":       nil,
-		"storage_missing_snapshot_records": nil,
+		"storage_lvm_skipactivation":                         nil,
+		"storage_missing_snapshot_records":                   nil,
+		"storage_delete_old_snapshot_records":                nil,
+		"storage_zfs_drop_block_volume_filesystem_extension": d.patchDropBlockVolumeFilesystemExtension,
+		"storage_prefix_bucket_names_with_project":           nil,
 	}
 
 	// Done if previously loaded.
@@ -78,8 +82,8 @@ func (d *zfs) load() error {
 		zfsVersion = version
 	}
 
-	// Decide whether we can use direct I/O with datasets (which was added in v0.8).
-	ver80, err := version.Parse("0.8.0")
+	// Decide whether we can use features added by 0.8.0.
+	ver080, err := version.Parse("0.8.0")
 	if err != nil {
 		return err
 	}
@@ -89,20 +93,21 @@ func (d *zfs) load() error {
 		return err
 	}
 
-	// If running 0.8 or older, we can use direct I/O and trim.
-	if ourVer.Compare(ver80) >= 0 {
+	// If running 0.8.0 or newer, we can use direct I/O, trim and raw.
+	if ourVer.Compare(ver080) >= 0 {
 		zfsDirectIO = true
 		zfsTrim = true
+		zfsRaw = true
 	}
 
-	ver080, err := version.Parse("0.8.0")
+	// Detect support for ZFS delegation.
+	ver220, err := version.Parse("2.2.0")
 	if err != nil {
 		return err
 	}
 
-	// If running 0.8.0 or newer, we can use the raw flag.
-	if ourVer.Compare(ver080) >= 0 {
-		zfsRaw = true
+	if ourVer.Compare(ver220) >= 0 {
+		zfsDelegate = true
 	}
 
 	zfsLoaded = true
@@ -112,18 +117,19 @@ func (d *zfs) load() error {
 // Info returns info about the driver and its environment.
 func (d *zfs) Info() Info {
 	info := Info{
-		Name:              "zfs",
-		Version:           zfsVersion,
-		OptimizedImages:   true,
-		OptimizedBackups:  true,
-		PreservesInodes:   true,
-		Remote:            d.isRemote(),
-		VolumeTypes:       []VolumeType{VolumeTypeBucket, VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
-		BlockBacking:      false,
-		RunningCopyFreeze: false,
-		DirectIO:          zfsDirectIO,
-		MountedRoot:       false,
-		Buckets:           true,
+		Name:                         "zfs",
+		Version:                      zfsVersion,
+		DefaultVMBlockFilesystemSize: deviceConfig.DefaultVMBlockFilesystemSize,
+		OptimizedImages:              true,
+		OptimizedBackups:             true,
+		PreservesInodes:              true,
+		Remote:                       d.isRemote(),
+		VolumeTypes:                  []VolumeType{VolumeTypeBucket, VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
+		BlockBacking:                 shared.IsTrue(d.config["volume.zfs.block_mode"]),
+		RunningCopyFreeze:            false,
+		DirectIO:                     zfsDirectIO,
+		MountedRoot:                  false,
+		Buckets:                      true,
 	}
 
 	return info
@@ -149,12 +155,8 @@ func (d zfs) ensureInitialDatasets(warnOnExistingPolicyApplyError bool) error {
 
 	for _, dataset := range d.initialDatasets() {
 		properties := []string{"mountpoint=legacy"}
-		if shared.StringInSlice(dataset, []string{"virtual-machines", "deleted/virtual-machines"}) {
-			if len(zfsVersion) >= 3 && zfsVersion[0:3] == "0.6" {
-				d.logger.Warn("Unable to set volmode on parent virtual-machines datasets due to ZFS being too old")
-			} else {
-				properties = append(properties, "volmode=none")
-			}
+		if shared.ValueInSlice(dataset, []string{"virtual-machines", "deleted/virtual-machines"}) {
+			properties = append(properties, "volmode=none")
 		}
 
 		datasetPath := filepath.Join(d.config["zfs.pool_name"], dataset)
@@ -183,12 +185,8 @@ func (d zfs) ensureInitialDatasets(warnOnExistingPolicyApplyError bool) error {
 	return nil
 }
 
-// Create is called during pool creation and is effectively using an empty driver struct.
-// WARNING: The Create() function cannot rely on any of the struct attributes being set.
-func (d *zfs) Create() error {
-	// Store the provided source as we are likely to be mangling it.
-	d.config["volatile.initial_source"] = d.config["source"]
-
+// FillConfig populates the storage pool's configuration file with the default values.
+func (d *zfs) FillConfig() error {
 	loopPath := loopFilePath(d.name)
 	if d.config["source"] == "" || d.config["source"] == loopPath {
 		// Create a loop based pool.
@@ -199,11 +197,6 @@ func (d *zfs) Create() error {
 			d.config["zfs.pool_name"] = d.name
 		}
 
-		// Validate pool_name.
-		if strings.Contains(d.config["zfs.pool_name"], "/") {
-			return fmt.Errorf("zfs.pool_name can't point to a dataset when source isn't set")
-		}
-
 		// Pick a default size of the loop file if not specified.
 		if d.config["size"] == "" {
 			defaultSize, err := loopFileSizeDefault()
@@ -213,6 +206,44 @@ func (d *zfs) Create() error {
 
 			d.config["size"] = fmt.Sprintf("%dGiB", defaultSize)
 		}
+	} else if filepath.IsAbs(d.config["source"]) {
+		// Set default pool_name.
+		if d.config["zfs.pool_name"] == "" {
+			d.config["zfs.pool_name"] = d.name
+		}
+
+		// Unset size property since it's irrelevant.
+		d.config["size"] = ""
+	} else {
+		// Handle an existing zpool.
+		if d.config["zfs.pool_name"] == "" {
+			d.config["zfs.pool_name"] = d.config["source"]
+		}
+
+		// Unset size property since it's irrelevant.
+		d.config["size"] = ""
+	}
+
+	return nil
+}
+
+// Create is called during pool creation and is effectively using an empty driver struct.
+// WARNING: The Create() function cannot rely on any of the struct attributes being set.
+func (d *zfs) Create() error {
+	// Store the provided source as we are likely to be mangling it.
+	d.config["volatile.initial_source"] = d.config["source"]
+
+	err := d.FillConfig()
+	if err != nil {
+		return err
+	}
+
+	loopPath := loopFilePath(d.name)
+	if d.config["source"] == "" || d.config["source"] == loopPath {
+		// Validate pool_name.
+		if strings.Contains(d.config["zfs.pool_name"], "/") {
+			return fmt.Errorf("zfs.pool_name can't point to a dataset when source isn't set")
+		}
 
 		// Create the loop file itself.
 		size, err := units.ParseByteSizeString(d.config["size"])
@@ -226,7 +257,7 @@ func (d *zfs) Create() error {
 		}
 
 		// Create the zpool.
-		_, err = shared.RunCommand("zpool", "create", "-f", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], loopPath)
+		_, err = shared.RunCommand("zpool", "create", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], loopPath)
 		if err != nil {
 			return err
 		}
@@ -244,23 +275,31 @@ func (d *zfs) Create() error {
 			return fmt.Errorf("Custom loop file locations are not supported")
 		}
 
-		// Unset size property since it's irrelevant.
-		d.config["size"] = ""
-
-		// Set default pool_name.
-		if d.config["zfs.pool_name"] == "" {
-			d.config["zfs.pool_name"] = d.name
-		}
-
 		// Validate pool_name.
 		if strings.Contains(d.config["zfs.pool_name"], "/") {
 			return fmt.Errorf("zfs.pool_name can't point to a dataset when source isn't set")
 		}
 
-		// Create the zpool.
-		_, err := shared.RunCommand("zpool", "create", "-f", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], d.config["source"])
-		if err != nil {
-			return err
+		// Wipe if requested.
+		if shared.IsTrue(d.config["source.wipe"]) {
+			err := wipeBlockHeaders(d.config["source"])
+			if err != nil {
+				return fmt.Errorf("Failed to wipe headers from disk %q: %w", d.config["source"], err)
+			}
+
+			d.config["source.wipe"] = ""
+
+			// Create the zpool.
+			_, err = shared.RunCommand("zpool", "create", "-f", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], d.config["source"])
+			if err != nil {
+				return err
+			}
+		} else {
+			// Create the zpool.
+			_, err := shared.RunCommand("zpool", "create", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], d.config["source"])
+			if err != nil {
+				return err
+			}
 		}
 
 		// Apply auto-trim if supported.
@@ -274,14 +313,6 @@ func (d *zfs) Create() error {
 		// We don't need to keep the original source path around for import.
 		d.config["source"] = d.config["zfs.pool_name"]
 	} else {
-		// Handle an existing zpool.
-		if d.config["zfs.pool_name"] == "" {
-			d.config["zfs.pool_name"] = d.config["source"]
-		}
-
-		// Unset size property since it's irrelevant.
-		d.config["size"] = ""
-
 		// Validate pool_name.
 		if d.config["zfs.pool_name"] != d.config["source"] {
 			return fmt.Errorf("The source must match zfs.pool_name if specified")
@@ -309,13 +340,13 @@ func (d *zfs) Create() error {
 		}
 
 		// Confirm that the existing pool/dataset is all empty.
-		datasets, err := d.getDatasets(d.config["zfs.pool_name"])
+		datasets, err := d.getDatasets(d.config["zfs.pool_name"], "all")
 		if err != nil {
 			return err
 		}
 
 		if len(datasets) > 0 {
-			return fmt.Errorf(`Provided ZFS pool (or dataset) isn't empty, run "sudo zfs list %s" to see existing entries`, d.config["zfs.pool_name"])
+			return fmt.Errorf(`Provided ZFS pool (or dataset) isn't empty, run "sudo zfs list -r %s" to see existing entries`, d.config["zfs.pool_name"])
 		}
 	}
 
@@ -326,7 +357,7 @@ func (d *zfs) Create() error {
 	revert.Add(func() { _ = d.Delete(nil) })
 
 	// Apply our default configuration.
-	err := d.ensureInitialDatasets(false)
+	err = d.ensureInitialDatasets(false)
 	if err != nil {
 		return err
 	}
@@ -348,14 +379,16 @@ func (d *zfs) Delete(op *operations.Operation) error {
 	}
 
 	// Confirm that nothing's been left behind
-	datasets, err := d.getDatasets(d.config["zfs.pool_name"])
+	datasets, err := d.getDatasets(d.config["zfs.pool_name"], "all")
 	if err != nil {
 		return err
 	}
 
 	initialDatasets := d.initialDatasets()
 	for _, dataset := range datasets {
-		if shared.StringInSlice(dataset, initialDatasets) {
+		dataset = strings.TrimPrefix(dataset, "/")
+
+		if shared.ValueInSlice(dataset, initialDatasets) {
 			continue
 		}
 
@@ -398,8 +431,22 @@ func (d *zfs) Delete(op *operations.Operation) error {
 // Validate checks that all provide keys are supported and that no conflicting or missing configuration is present.
 func (d *zfs) Validate(config map[string]string) error {
 	rules := map[string]func(value string) error{
-		"size":          validate.Optional(validate.IsSize),
+		"size": validate.Optional(validate.IsSize),
+		// lxdmeta:generate(entities=storage-zfs; group=pool-conf; key=zfs.pool_name)
+		//
+		// ---
+		//  type: string
+		//  defaultdesc: name of the pool
+		//  shortdesc: Name of the zpool
 		"zfs.pool_name": validate.IsAny,
+		// lxdmeta:generate(entities=storage-zfs; group=pool-conf; key=zfs.clone_copy)
+		// Set this option to `true` or `false` to enable or disable using ZFS lightweight clones rather
+		// than full dataset copies.
+		// Set the option to `rebase` to copy based on the initial image.
+		// ---
+		//  type: string
+		//  defaultdesc: `true`
+		//  shortdesc: Whether to use ZFS lightweight clones
 		"zfs.clone_copy": validate.Optional(func(value string) error {
 			if value == "rebase" {
 				return nil
@@ -407,6 +454,12 @@ func (d *zfs) Validate(config map[string]string) error {
 
 			return validate.IsBool(value)
 		}),
+		// lxdmeta:generate(entities=storage-zfs; group=pool-conf; key=zfs.export)
+		//
+		// ---
+		//  type: bool
+		//  defaultdesc: `true`
+		//  shortdesc: Disable zpool export while an unmount is being performed
 		"zfs.export": validate.Optional(validate.IsBool),
 	}
 
@@ -420,6 +473,36 @@ func (d *zfs) Update(changedConfig map[string]string) error {
 		return fmt.Errorf("zfs.pool_name cannot be modified")
 	}
 
+	size, ok := changedConfig["size"]
+	if ok {
+		// Figure out loop path
+		loopPath := loopFilePath(d.name)
+
+		if d.config["source"] != loopPath {
+			return fmt.Errorf("Cannot resize non-loopback pools")
+		}
+
+		// Resize loop file
+		f, err := os.OpenFile(loopPath, os.O_RDWR, 0600)
+		if err != nil {
+			return err
+		}
+
+		defer func() { _ = f.Close() }()
+
+		sizeBytes, _ := units.ParseByteSizeString(size)
+
+		err = f.Truncate(sizeBytes)
+		if err != nil {
+			return err
+		}
+
+		_, err = shared.RunCommand("zpool", "online", "-e", d.config["zfs.pool_name"], loopPath)
+		if err != nil {
+			return err
+		}
+	}
+
 	return nil
 }
 
@@ -559,7 +642,7 @@ func (d *zfs) GetResources() (*api.ResourcesStoragePool, error) {
 }
 
 // MigrationType returns the type of transfer methods to be used when doing migrations between pools in preference order.
-func (d *zfs) MigrationTypes(contentType ContentType, refresh bool) []migration.Type {
+func (d *zfs) MigrationTypes(contentType ContentType, refresh bool, copySnapshots bool) []migration.Type {
 	var rsyncFeatures []string
 
 	// Do not pass compression argument to rsync if the associated
@@ -571,13 +654,13 @@ func (d *zfs) MigrationTypes(contentType ContentType, refresh bool) []migration.
 	}
 
 	// Detect ZFS features.
-	features := []string{migration.ZFSFeatureMigrationHeader}
+	features := []string{migration.ZFSFeatureMigrationHeader, "compress"}
 
-	if len(zfsVersion) >= 3 && zfsVersion[0:3] != "0.6" {
-		features = append(features, "compress")
+	if contentType == ContentTypeFS {
+		features = append(features, migration.ZFSFeatureZvolFilesystems)
 	}
 
-	if contentType == ContentTypeBlock {
+	if IsContentBlock(contentType) {
 		return []migration.Type{
 			{
 				FSType:   migration.MigrationFSType_ZFS,
@@ -590,6 +673,15 @@ func (d *zfs) MigrationTypes(contentType ContentType, refresh bool) []migration.
 		}
 	}
 
+	if refresh && !copySnapshots {
+		return []migration.Type{
+			{
+				FSType:   migration.MigrationFSType_RSYNC,
+				Features: rsyncFeatures,
+			},
+		}
+	}
+
 	return []migration.Type{
 		{
 			FSType:   migration.MigrationFSType_ZFS,
@@ -601,3 +693,39 @@ func (d *zfs) MigrationTypes(contentType ContentType, refresh bool) []migration.
 		},
 	}
 }
+
+// patchDropBlockVolumeFilesystemExtension removes the filesystem extension (e.g _ext4) from VM image block volumes.
+func (d *zfs) patchDropBlockVolumeFilesystemExtension() error {
+	poolName, ok := d.config["zfs.pool_name"]
+	if !ok {
+		poolName = d.name
+	}
+
+	out, err := shared.RunCommand("zfs", "list", "-H", "-r", "-o", "name", "-t", "volume", fmt.Sprintf("%s/images", poolName))
+	if err != nil {
+		return fmt.Errorf("Failed listing images: %w", err)
+	}
+
+	for _, volume := range strings.Split(out, "\n") {
+		fields := strings.SplitN(volume, fmt.Sprintf("%s/images/", poolName), 2)
+
+		if len(fields) != 2 || fields[1] == "" {
+			continue
+		}
+
+		// Ignore non-block images, and images without filesystem extension
+		if !strings.HasSuffix(fields[1], ".block") || !strings.Contains(fields[1], "_") {
+			continue
+		}
+
+		// Rename zfs dataset. Snapshots will automatically be renamed.
+		newName := fmt.Sprintf("%s/images/%s.block", poolName, strings.Split(fields[1], "_")[0])
+
+		_, err = shared.RunCommand("zfs", "rename", volume, newName)
+		if err != nil {
+			return fmt.Errorf("Failed renaming zfs dataset: %w", err)
+		}
+	}
+
+	return nil
+}
