@@ -9,19 +9,19 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/archive"
-	"github.com/lxc/lxd/lxd/migration"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/rsync"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/lxd/sys"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/instancewriter"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/archive"
+	"github.com/canonical/lxd/lxd/instancewriter"
+	"github.com/canonical/lxd/lxd/migration"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/rsync"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/lxd/sys"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // genericVolumeBlockExtension extension used for generic block volume disk files.
@@ -30,6 +30,9 @@ const genericVolumeBlockExtension = "img"
 // genericVolumeDiskFile used to indicate the file name used for block volume disk files.
 const genericVolumeDiskFile = "root.img"
 
+// genericISOVolumeSuffix suffix used for generic iso content type volumes.
+const genericISOVolumeSuffix = ".iso"
+
 // genericVFSGetResources is a generic GetResources implementation for VFS-only drivers.
 func genericVFSGetResources(d Driver) (*api.ResourcesStoragePool, error) {
 	// Get the VFS information
@@ -144,7 +147,7 @@ func genericVFSRenameVolumeSnapshot(d Driver, snapVol Volume, newSnapshotName st
 }
 
 // genericVFSMigrateVolume is a generic MigrateVolume implementation for VFS-only drivers.
-func genericVFSMigrateVolume(d Driver, s *state.State, vol Volume, conn io.ReadWriteCloser, volSrcArgs *migration.VolumeSourceArgs, op *operations.Operation) error {
+func genericVFSMigrateVolume(d Driver, s *state.State, vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs *migration.VolumeSourceArgs, op *operations.Operation) error {
 	bwlimit := d.Config()["rsync.bwlimit"]
 	var rsyncArgs []string
 
@@ -156,8 +159,14 @@ func genericVFSMigrateVolume(d Driver, s *state.State, vol Volume, conn io.ReadW
 		}
 
 		rsyncArgs = []string{"--exclude", genericVolumeDiskFile}
-	} else if vol.contentType == ContentTypeBlock && volSrcArgs.MigrationType.FSType != migration.MigrationFSType_BLOCK_AND_RSYNC || vol.contentType == ContentTypeFS && volSrcArgs.MigrationType.FSType != migration.MigrationFSType_RSYNC {
-		return ErrNotSupported
+	} else if vol.contentType == ContentTypeBlock {
+		if volSrcArgs.MigrationType.FSType != migration.MigrationFSType_BLOCK_AND_RSYNC {
+			return ErrNotSupported
+		}
+	} else if vol.contentType == ContentTypeFS {
+		if !shared.ValueInSlice(volSrcArgs.MigrationType.FSType, []migration.MigrationFSType{migration.MigrationFSType_RSYNC, migration.MigrationFSType_RBD_AND_RSYNC}) {
+			return ErrNotSupported
+		}
 	}
 
 	// Define function to send a filesystem volume.
@@ -227,22 +236,32 @@ func genericVFSMigrateVolume(d Driver, s *state.State, vol Volume, conn io.ReadW
 
 	// Send all snapshots to target.
 	for _, snapName := range volSrcArgs.Snapshots {
-		snapshot, err := vol.NewSnapshot(snapName)
-		if err != nil {
-			return err
+		found := false
+		var snapVol Volume
+		for _, snapshot := range vol.Snapshots {
+			_, snapshotName, _ := api.GetParentAndSnapshotName(snapshot.name)
+			if snapshotName == snapName {
+				snapVol = snapshot
+				found = true
+				break
+			}
+		}
+
+		if !found {
+			return fmt.Errorf("Snapshot %q missing in volume's list", snapName)
 		}
 
 		// Send snapshot to target (ensure local snapshot volume is mounted if needed).
-		err = snapshot.MountTask(func(mountPath string, op *operations.Operation) error {
+		err := snapVol.MountTask(func(mountPath string, op *operations.Operation) error {
 			if vol.contentType != ContentTypeBlock || vol.volType != VolumeTypeCustom {
-				err := sendFSVol(snapshot, conn, mountPath)
+				err := sendFSVol(snapVol, conn, mountPath)
 				if err != nil {
 					return err
 				}
 			}
 
 			if vol.IsVMBlock() || (vol.contentType == ContentTypeBlock && vol.volType == VolumeTypeCustom) {
-				err = sendBlockVol(snapshot, conn)
+				err := sendBlockVol(snapVol, conn)
 				if err != nil {
 					return err
 				}
@@ -257,15 +276,15 @@ func genericVFSMigrateVolume(d Driver, s *state.State, vol Volume, conn io.ReadW
 
 	// Send volume to target (ensure local volume is mounted if needed).
 	return vol.MountTask(func(mountPath string, op *operations.Operation) error {
-		if vol.contentType != ContentTypeBlock || vol.volType != VolumeTypeCustom {
-			err := sendFSVol(vol, conn, mountPath)
+		if !IsContentBlock(vol.contentType) || vol.volType != VolumeTypeCustom {
+			err := sendFSVol(vol.Volume, conn, mountPath)
 			if err != nil {
 				return err
 			}
 		}
 
-		if vol.IsVMBlock() || (vol.contentType == ContentTypeBlock && vol.volType == VolumeTypeCustom) {
-			err := sendBlockVol(vol, conn)
+		if vol.IsVMBlock() || (IsContentBlock(vol.contentType) && vol.volType == VolumeTypeCustom) {
+			err := sendBlockVol(vol.Volume, conn)
 			if err != nil {
 				return err
 			}
@@ -277,14 +296,14 @@ func genericVFSMigrateVolume(d Driver, s *state.State, vol Volume, conn io.ReadW
 
 // genericVFSCreateVolumeFromMigration receives a volume and its snapshots over a non-optimized method.
 // initVolume is run against the main volume (not the snapshots) and is often used for quota initialization.
-func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (revert.Hook, error), vol Volume, conn io.ReadWriteCloser, volTargetArgs migration.VolumeTargetArgs, preFiller *VolumeFiller, op *operations.Operation) error {
+func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (revert.Hook, error), vol VolumeCopy, conn io.ReadWriteCloser, volTargetArgs migration.VolumeTargetArgs, preFiller *VolumeFiller, op *operations.Operation) (revert.Hook, error) {
 	// Check migration transport type matches volume type.
-	if vol.contentType == ContentTypeBlock {
+	if IsContentBlock(vol.contentType) {
 		if volTargetArgs.MigrationType.FSType != migration.MigrationFSType_BLOCK_AND_RSYNC {
-			return ErrNotSupported
+			return nil, ErrNotSupported
 		}
-	} else if volTargetArgs.MigrationType.FSType != migration.MigrationFSType_RSYNC {
-		return ErrNotSupported
+	} else if !shared.ValueInSlice(volTargetArgs.MigrationType.FSType, []migration.MigrationFSType{migration.MigrationFSType_RSYNC, migration.MigrationFSType_RBD_AND_RSYNC}) {
+		return nil, ErrNotSupported
 	}
 
 	revert := revert.New()
@@ -292,12 +311,12 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 
 	// Create the main volume if not refreshing.
 	if !volTargetArgs.Refresh {
-		err := d.CreateVolume(vol, preFiller, op)
+		err := d.CreateVolume(vol.Volume, preFiller, op)
 		if err != nil {
-			return err
+			return nil, err
 		}
 
-		revert.Add(func() { _ = d.DeleteVolume(vol, op) })
+		revert.Add(func() { _ = d.DeleteVolume(vol.Volume, op) })
 	}
 
 	recvFSVol := func(volName string, conn io.ReadWriteCloser, path string) error {
@@ -306,7 +325,9 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 			wrapper = migration.ProgressTracker(op, "fs_progress", volName)
 		}
 
-		d.Logger().Debug("Receiving filesystem volume", logger.Ctx{"volName": volName, "path": path})
+		d.Logger().Debug("Receiving filesystem volume started", logger.Ctx{"volName": volName, "path": path, "features": volTargetArgs.MigrationType.Features})
+		defer d.Logger().Debug("Receiving filesystem volume stopped", logger.Ctx{"volName": volName, "path": path})
+
 		return rsync.Recv(path, conn, wrapper, volTargetArgs.MigrationType.Features)
 	}
 
@@ -332,7 +353,9 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 			}
 		}
 
-		d.Logger().Debug("Receiving block volume", logger.Ctx{"volName": volName, "path": path})
+		d.Logger().Debug("Receiving block volume started", logger.Ctx{"volName": volName, "path": path})
+		defer d.Logger().Debug("Receiving block volume stopped", logger.Ctx{"volName": volName, "path": path})
+
 		_, err = io.Copy(to, fromPipe)
 		if err != nil {
 			return fmt.Errorf("Error copying from migration connection to %q: %w", path, err)
@@ -350,8 +373,8 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 		path := shared.AddSlash(mountPath)
 		pathBlock := ""
 
-		if vol.IsVMBlock() || (vol.contentType == ContentTypeBlock && vol.volType == VolumeTypeCustom) {
-			pathBlock, err = d.GetVolumeDiskPath(vol)
+		if vol.IsVMBlock() || (IsContentBlock(vol.contentType) && vol.volType == VolumeTypeCustom) {
+			pathBlock, err = d.GetVolumeDiskPath(vol.Volume)
 			if err != nil {
 				return fmt.Errorf("Error getting VM block volume disk path: %w", err)
 			}
@@ -359,8 +382,20 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 
 		// Snapshots are sent first by the sender, so create these first.
 		for _, snapName := range volTargetArgs.Snapshots {
-			fullSnapshotName := GetSnapshotVolumeName(vol.name, snapName)
-			snapVol := NewVolume(d, d.Name(), vol.volType, vol.contentType, fullSnapshotName, vol.config, vol.poolConfig)
+			found := false
+			var snapVol Volume
+			for _, snapshot := range vol.Snapshots {
+				_, snapshotName, _ := api.GetParentAndSnapshotName(snapshot.name)
+				if snapshotName == snapName {
+					snapVol = snapshot
+					found = true
+					break
+				}
+			}
+
+			if !found {
+				return fmt.Errorf("Snapshot %q missing in volume's list", snapName)
+			}
 
 			if snapVol.contentType != ContentTypeBlock || snapVol.volType != VolumeTypeCustom { // Receive the filesystem snapshot first (as it is sent first).
 				err = recvFSVol(snapVol.name, conn, path)
@@ -392,13 +427,13 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 
 		// Run volume-specific init logic.
 		if initVolume != nil {
-			_, err := initVolume(vol)
+			_, err := initVolume(vol.Volume)
 			if err != nil {
 				return err
 			}
 		}
 
-		if vol.contentType != ContentTypeBlock || vol.volType != VolumeTypeCustom {
+		if !IsContentBlock(vol.contentType) || vol.volType != VolumeTypeCustom {
 			// Receive main volume.
 			err = recvFSVol(vol.name, conn, path)
 			if err != nil {
@@ -407,7 +442,7 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 		}
 
 		// Receive the final main volume sync if needed.
-		if volTargetArgs.Live && (vol.contentType != ContentTypeBlock || vol.volType != VolumeTypeCustom) {
+		if volTargetArgs.Live && (!IsContentBlock(vol.contentType) || vol.volType != VolumeTypeCustom) {
 			d.Logger().Debug("Starting main volume final sync", logger.Ctx{"volName": vol.name, "path": path})
 			err = recvFSVol(vol.name, conn, path)
 			if err != nil {
@@ -423,7 +458,7 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 		}
 
 		// Receive the block volume next (if needed).
-		if vol.IsVMBlock() || (vol.contentType == ContentTypeBlock && vol.volType == VolumeTypeCustom) {
+		if vol.IsVMBlock() || (IsContentBlock(vol.contentType) && vol.volType == VolumeTypeCustom) {
 			err = recvBlockVol(vol.name, conn, pathBlock)
 			if err != nil {
 				return err
@@ -433,11 +468,12 @@ func genericVFSCreateVolumeFromMigration(d Driver, initVolume func(vol Volume) (
 		return nil
 	}, op)
 	if err != nil {
-		return err
+		return nil, err
 	}
 
+	cleanup := revert.Clone().Fail
 	revert.Success()
-	return nil
+	return cleanup, nil
 }
 
 // genericVFSHasVolume is a generic HasVolume implementation for VFS-only drivers.
@@ -456,7 +492,7 @@ func genericVFSHasVolume(vol Volume) (bool, error) {
 
 // genericVFSGetVolumeDiskPath is a generic GetVolumeDiskPath implementation for VFS-only drivers.
 func genericVFSGetVolumeDiskPath(vol Volume) (string, error) {
-	if vol.contentType != ContentTypeBlock {
+	if !IsContentBlock(vol.contentType) {
 		return "", ErrNotSupported
 	}
 
@@ -464,10 +500,10 @@ func genericVFSGetVolumeDiskPath(vol Volume) (string, error) {
 }
 
 // genericVFSBackupVolume is a generic BackupVolume implementation for VFS-only drivers.
-func genericVFSBackupVolume(d Driver, vol Volume, tarWriter *instancewriter.InstanceTarWriter, snapshots []string, op *operations.Operation) error {
+func genericVFSBackupVolume(d Driver, vol VolumeCopy, tarWriter *instancewriter.InstanceTarWriter, snapshots []string, op *operations.Operation) error {
 	if len(snapshots) > 0 {
 		// Check requested snapshot match those in storage.
-		err := vol.SnapshotsMatch(snapshots, op)
+		err := d.CheckVolumeSnapshots(vol.Volume, vol.Snapshots, op)
 		if err != nil {
 			return err
 		}
@@ -479,89 +515,7 @@ func genericVFSBackupVolume(d Driver, vol Volume, tarWriter *instancewriter.Inst
 			// Reset hard link cache as we are copying a new volume (instance or snapshot).
 			tarWriter.ResetHardLinkMap()
 
-			if v.contentType == ContentTypeBlock {
-				blockPath, err := d.GetVolumeDiskPath(v)
-				if err != nil {
-					errMsg := "Error getting VM block volume disk path"
-					if vol.volType == VolumeTypeCustom {
-						errMsg = "Error getting custom block volume disk path"
-					}
-
-					return fmt.Errorf(errMsg+": %w", err)
-				}
-
-				// Get size of disk block device for tarball header.
-				blockDiskSize, err := BlockDiskSizeBytes(blockPath)
-				if err != nil {
-					return fmt.Errorf("Error getting block device size %q: %w", blockPath, err)
-				}
-
-				var exclude []string // Files to exclude from filesystem volume backup.
-				if !shared.IsBlockdevPath(blockPath) {
-					// Exclude the volume root disk file from the filesystem volume backup.
-					// We will read it as a block device later instead.
-					exclude = append(exclude, blockPath)
-				}
-
-				if v.IsVMBlock() {
-					logMsg := "Copying virtual machine config volume"
-
-					d.Logger().Debug(logMsg, logger.Ctx{"sourcePath": mountPath, "prefix": prefix})
-					err = filepath.Walk(mountPath, func(srcPath string, fi os.FileInfo, err error) error {
-						if err != nil {
-							return err
-						}
-
-						// Skip any exluded files.
-						if shared.StringHasPrefix(srcPath, exclude...) {
-							return nil
-						}
-
-						name := filepath.Join(prefix, strings.TrimPrefix(srcPath, mountPath))
-						err = tarWriter.WriteFile(name, srcPath, fi, false)
-						if err != nil {
-							return fmt.Errorf("Error adding %q as %q to tarball: %w", srcPath, name, err)
-						}
-
-						return nil
-					})
-					if err != nil {
-						return err
-					}
-				}
-
-				name := fmt.Sprintf("%s.%s", prefix, genericVolumeBlockExtension)
-
-				logMsg := "Copying virtual machine block volume"
-				if vol.volType == VolumeTypeCustom {
-					logMsg = "Copying custom block volume"
-				}
-
-				d.Logger().Debug(logMsg, logger.Ctx{"sourcePath": blockPath, "file": name, "size": blockDiskSize})
-				from, err := os.Open(blockPath)
-				if err != nil {
-					return fmt.Errorf("Error opening file for reading %q: %w", blockPath, err)
-				}
-
-				defer func() { _ = from.Close() }()
-
-				fi := instancewriter.FileInfo{
-					FileName:    name,
-					FileSize:    blockDiskSize,
-					FileMode:    0600,
-					FileModTime: time.Now(),
-				}
-
-				err = tarWriter.WriteFileFromReader(from, &fi)
-				if err != nil {
-					return fmt.Errorf("Error copying %q as %q to tarball: %w", blockPath, name, err)
-				}
-
-				err = from.Close()
-				if err != nil {
-					return fmt.Errorf("Failed to close file %q: %w", blockPath, err)
-				}
-			} else {
+			if v.contentType != ContentTypeBlock {
 				logMsg := "Copying container filesystem volume"
 				if vol.volType == VolumeTypeCustom {
 					logMsg = "Copying custom filesystem volume"
@@ -604,6 +558,88 @@ func genericVFSBackupVolume(d Driver, vol Volume, tarWriter *instancewriter.Inst
 				})
 			}
 
+			blockPath, err := d.GetVolumeDiskPath(v)
+			if err != nil {
+				errMsg := "Error getting VM block volume disk path"
+				if vol.volType == VolumeTypeCustom {
+					errMsg = "Error getting custom block volume disk path"
+				}
+
+				return fmt.Errorf(errMsg+": %w", err)
+			}
+
+			// Get size of disk block device for tarball header.
+			blockDiskSize, err := BlockDiskSizeBytes(blockPath)
+			if err != nil {
+				return fmt.Errorf("Error getting block device size %q: %w", blockPath, err)
+			}
+
+			var exclude []string // Files to exclude from filesystem volume backup.
+			if !shared.IsBlockdevPath(blockPath) {
+				// Exclude the volume root disk file from the filesystem volume backup.
+				// We will read it as a block device later instead.
+				exclude = append(exclude, blockPath)
+			}
+
+			if v.IsVMBlock() {
+				logMsg := "Copying virtual machine config volume"
+
+				d.Logger().Debug(logMsg, logger.Ctx{"sourcePath": mountPath, "prefix": prefix})
+				err = filepath.Walk(mountPath, func(srcPath string, fi os.FileInfo, err error) error {
+					if err != nil {
+						return err
+					}
+
+					// Skip any exluded files.
+					if shared.StringHasPrefix(srcPath, exclude...) {
+						return nil
+					}
+
+					name := filepath.Join(prefix, strings.TrimPrefix(srcPath, mountPath))
+					err = tarWriter.WriteFile(name, srcPath, fi, false)
+					if err != nil {
+						return fmt.Errorf("Error adding %q as %q to tarball: %w", srcPath, name, err)
+					}
+
+					return nil
+				})
+				if err != nil {
+					return err
+				}
+			}
+
+			name := fmt.Sprintf("%s.%s", prefix, genericVolumeBlockExtension)
+
+			logMsg := "Copying virtual machine block volume"
+			if vol.volType == VolumeTypeCustom {
+				logMsg = "Copying custom block volume"
+			}
+
+			d.Logger().Debug(logMsg, logger.Ctx{"sourcePath": blockPath, "file": name, "size": blockDiskSize})
+			from, err := os.Open(blockPath)
+			if err != nil {
+				return fmt.Errorf("Error opening file for reading %q: %w", blockPath, err)
+			}
+
+			defer func() { _ = from.Close() }()
+
+			fi := instancewriter.FileInfo{
+				FileName:    name,
+				FileSize:    blockDiskSize,
+				FileMode:    0600,
+				FileModTime: time.Now(),
+			}
+
+			err = tarWriter.WriteFileFromReader(from, &fi)
+			if err != nil {
+				return fmt.Errorf("Error copying %q as %q to tarball: %w", blockPath, name, err)
+			}
+
+			err = from.Close()
+			if err != nil {
+				return fmt.Errorf("Failed to close file %q: %w", blockPath, err)
+			}
+
 			return nil
 		}, op)
 	}
@@ -618,13 +654,23 @@ func genericVFSBackupVolume(d Driver, vol Volume, tarWriter *instancewriter.Inst
 		}
 
 		for _, snapName := range snapshots {
-			prefix := filepath.Join(snapshotsPrefix, snapName)
-			snapVol, err := vol.NewSnapshot(snapName)
-			if err != nil {
-				return err
+			found := false
+			var snapVol Volume
+			for _, snapshot := range vol.Snapshots {
+				_, snapshotName, _ := api.GetParentAndSnapshotName(snapshot.name)
+				if snapshotName == snapName {
+					snapVol = snapshot
+					found = true
+					break
+				}
 			}
 
-			err = backupVolume(snapVol, prefix)
+			if !found {
+				return fmt.Errorf("Snapshot %q missing in volume's list", snapName)
+			}
+
+			prefix := filepath.Join(snapshotsPrefix, snapName)
+			err := backupVolume(snapVol, prefix)
 			if err != nil {
 				return err
 			}
@@ -639,7 +685,7 @@ func genericVFSBackupVolume(d Driver, vol Volume, tarWriter *instancewriter.Inst
 		prefix = "backup/volume"
 	}
 
-	err := backupVolume(vol, prefix)
+	err := backupVolume(vol.Volume, prefix)
 	if err != nil {
 		return err
 	}
@@ -652,7 +698,7 @@ func genericVFSBackupVolume(d Driver, vol Volume, tarWriter *instancewriter.Inst
 // created and a revert function that can be used to undo the actions this function performs should something
 // subsequently fail. For VolumeTypeCustom volumes, a nil post hook is returned as it is expected that the DB
 // record be created before the volume is unpacked due to differences in the archive format that allows this.
-func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []string, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error) {
+func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol VolumeCopy, snapshots []string, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error) {
 	// Define function to unpack a volume from a backup tarball file.
 	unpackVolume := func(r io.ReadSeeker, tarArgs []string, unpacker []string, srcPrefix string, mountPath string) error {
 		volTypeName := "container"
@@ -700,7 +746,7 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 
 			// Extract filesystem volume.
 			d.Logger().Debug(fmt.Sprintf("Unpacking %s filesystem volume", volTypeName), logger.Ctx{"source": srcPrefix, "target": mountPath, "args": fmt.Sprintf("%+v", args)})
-			_, err := srcData.Seek(0, 0)
+			_, err := srcData.Seek(0, io.SeekStart)
 			if err != nil {
 				return err
 			}
@@ -725,7 +771,7 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 
 		// Extract block file to block volume.
 		if vol.contentType == ContentTypeBlock {
-			targetPath, err := d.GetVolumeDiskPath(vol)
+			targetPath, err := d.GetVolumeDiskPath(vol.Volume)
 			if err != nil {
 				return err
 			}
@@ -739,6 +785,43 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 
 			defer cancelFunc()
 
+			unpack := func(size int64) error {
+				var allowUnsafeResize bool
+
+				// Open block file (use O_CREATE to support drivers that use image files).
+				to, err := os.OpenFile(targetPath, os.O_WRONLY|os.O_TRUNC|os.O_CREATE, 0644)
+				if err != nil {
+					return fmt.Errorf("Error opening file for writing %q: %w", targetPath, err)
+				}
+
+				defer to.Close()
+
+				// Restore original size of volume from raw block backup file size.
+				d.Logger().Debug("Setting volume size from source", logger.Ctx{"source": srcFile, "target": targetPath, "size": size})
+
+				// Allow potentially destructive resize of volume as we are going to be
+				// overwriting it entirely anyway. This allows shrinking of block volumes.
+				allowUnsafeResize = true
+				err = d.SetVolumeQuota(vol.Volume, fmt.Sprintf("%d", size), allowUnsafeResize, op)
+				if err != nil {
+					return err
+				}
+
+				logMsg := "Unpacking virtual machine block volume"
+				if vol.volType == VolumeTypeCustom {
+					logMsg = "Unpacking custom block volume"
+				}
+
+				d.Logger().Debug(logMsg, logger.Ctx{"source": srcFile, "target": targetPath})
+				_, err = io.Copy(to, tr)
+				if err != nil {
+					return err
+				}
+
+				cancelFunc()
+				return nil
+			}
+
 			for {
 				hdr, err := tr.Next()
 				if err == io.EOF {
@@ -750,40 +833,7 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 				}
 
 				if hdr.Name == srcFile {
-					var allowUnsafeResize bool
-
-					// Open block file (use O_CREATE to support drivers that use image files).
-					to, err := os.OpenFile(targetPath, os.O_WRONLY|os.O_TRUNC|os.O_CREATE, 0644)
-					if err != nil {
-						return fmt.Errorf("Error opening file for writing %q: %w", targetPath, err)
-					}
-
-					defer func() { _ = to.Close() }()
-
-					// Restore original size of volume from raw block backup file size.
-					d.Logger().Debug("Setting volume size from source", logger.Ctx{"source": srcFile, "target": targetPath, "size": hdr.Size})
-
-					// Allow potentially destructive resize of volume as we are going to be
-					// overwriting it entirely anyway. This allows shrinking of block volumes.
-					allowUnsafeResize = true
-					err = d.SetVolumeQuota(vol, fmt.Sprintf("%d", hdr.Size), allowUnsafeResize, op)
-					if err != nil {
-						return err
-					}
-
-					logMsg := "Unpacking virtual machine block volume"
-					if vol.volType == VolumeTypeCustom {
-						logMsg = "Unpacking custom block volume"
-					}
-
-					d.Logger().Debug(logMsg, logger.Ctx{"source": srcFile, "target": targetPath})
-					_, err = io.Copy(to, tr)
-					if err != nil {
-						return err
-					}
-
-					cancelFunc()
-					return to.Close()
+					return unpack(hdr.Size)
 				}
 			}
 
@@ -797,7 +847,7 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 	defer revert.Fail()
 
 	// Find the compression algorithm used for backup source data.
-	_, err := srcData.Seek(0, 0)
+	_, err := srcData.Seek(0, io.SeekStart)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -807,7 +857,7 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 		return nil, nil, err
 	}
 
-	volExists, err := d.HasVolume(vol)
+	volExists, err := d.HasVolume(vol.Volume)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -817,12 +867,12 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 	}
 
 	// Create new empty volume.
-	err = d.CreateVolume(vol, nil, nil)
+	err = d.CreateVolume(vol.Volume, nil, nil)
 	if err != nil {
 		return nil, nil, err
 	}
 
-	revert.Add(func() { _ = d.DeleteVolume(vol, op) })
+	revert.Add(func() { _ = d.DeleteVolume(vol.Volume, op) })
 
 	if len(snapshots) > 0 {
 		// Create new snapshots directory.
@@ -840,6 +890,21 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 	}
 
 	for _, snapName := range snapshots {
+		found := false
+		var snapVol Volume
+		for _, snapshot := range vol.Snapshots {
+			_, snapshotName, _ := api.GetParentAndSnapshotName(snapshot.name)
+			if snapshotName == snapName {
+				snapVol = snapshot
+				found = true
+				break
+			}
+		}
+
+		if !found {
+			return nil, nil, fmt.Errorf("Snapshot %q missing in volume's list", snapName)
+		}
+
 		err = vol.MountTask(func(mountPath string, op *operations.Operation) error {
 			backupSnapshotPrefix := fmt.Sprintf("%s/%s", backupSnapshotsPrefix, snapName)
 			return unpackVolume(srcData, tarArgs, unpacker, backupSnapshotPrefix, mountPath)
@@ -848,11 +913,6 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 			return nil, nil, err
 		}
 
-		snapVol, err := vol.NewSnapshot(snapName)
-		if err != nil {
-			return nil, nil, err
-		}
-
 		d.Logger().Debug("Creating volume snapshot", logger.Ctx{"snapshotName": snapVol.Name()})
 		err = d.CreateVolumeSnapshot(snapVol, op)
 		if err != nil {
@@ -862,12 +922,12 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 		revert.Add(func() { _ = d.DeleteVolumeSnapshot(snapVol, op) })
 	}
 
-	err = d.MountVolume(vol, op)
+	err = d.MountVolume(vol.Volume, op)
 	if err != nil {
 		return nil, nil, err
 	}
 
-	revert.Add(func() { _, _ = d.UnmountVolume(vol, false, op) })
+	revert.Add(func() { _, _ = d.UnmountVolume(vol.Volume, false, op) })
 
 	backupPrefix := "backup/container"
 	if vol.IsVMBlock() {
@@ -907,7 +967,7 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 		}
 	} else {
 		// For custom volumes unmount now, there is no post hook as there is no backup.yaml to generate.
-		_, err = d.UnmountVolume(vol, false, op)
+		_, err = d.UnmountVolume(vol.Volume, false, op)
 		if err != nil {
 			return nil, nil, err
 		}
@@ -918,9 +978,9 @@ func genericVFSBackupUnpack(d Driver, sysOS *sys.OS, vol Volume, snapshots []str
 
 // genericVFSCopyVolume copies a volume and its snapshots using a non-optimized method.
 // initVolume is run against the main volume (not the snapshots) and is often used for quota initialization.
-func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, error), vol Volume, srcVol Volume, srcSnapshots []Volume, refresh bool, allowInconsistent bool, op *operations.Operation) error {
+func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, error), vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots []string, refresh bool, allowInconsistent bool, op *operations.Operation) (revert.Hook, error) {
 	if vol.contentType != srcVol.contentType {
-		return fmt.Errorf("Content type of source and target must be the same")
+		return nil, fmt.Errorf("Content type of source and target must be the same")
 	}
 
 	bwlimit := d.Config()["rsync.bwlimit"]
@@ -936,12 +996,12 @@ func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, er
 
 	// Create the main volume if not refreshing.
 	if !refresh {
-		err := d.CreateVolume(vol, nil, op)
+		err := d.CreateVolume(vol.Volume, nil, op)
 		if err != nil {
-			return err
+			return nil, err
 		}
 
-		revert.Add(func() { _ = d.DeleteVolume(vol, op) })
+		revert.Add(func() { _ = d.DeleteVolume(vol.Volume, op) })
 	}
 
 	// Define function to send a filesystem volume.
@@ -981,10 +1041,8 @@ func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, er
 	// Ensure the volume is mounted.
 	err := vol.MountTask(func(targetMountPath string, op *operations.Operation) error {
 		// If copying snapshots is indicated, check the source isn't itself a snapshot.
-		if len(srcSnapshots) > 0 && !srcVol.IsSnapshot() {
-			for _, srcVol := range srcSnapshots {
-				_, snapName, _ := api.GetParentAndSnapshotName(srcVol.name)
-
+		if len(refreshSnapshots) > 0 && !srcVol.IsSnapshot() {
+			for _, refreshSnapshot := range refreshSnapshots {
 				// Mount the source snapshot and copy it to the target main volume.
 				// A snapshot will then be taken next so it is stored in the correct volume and
 				// subsequent filesystem rsync transfers benefit from only transferring the files
@@ -998,7 +1056,7 @@ func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, er
 					}
 
 					if srcVol.IsVMBlock() || srcVol.contentType == ContentTypeBlock && srcVol.volType == VolumeTypeCustom {
-						err := sendBlockVol(srcVol, vol)
+						err := sendBlockVol(srcVol.Volume, vol.Volume)
 						if err != nil {
 							return err
 						}
@@ -1010,8 +1068,20 @@ func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, er
 					return err
 				}
 
-				fullSnapName := GetSnapshotVolumeName(vol.name, snapName)
-				snapVol := NewVolume(d, d.Name(), vol.volType, vol.contentType, fullSnapName, vol.config, vol.poolConfig)
+				found := false
+				var snapVol Volume
+				for _, snapshot := range vol.Snapshots {
+					_, snapshotName, _ := api.GetParentAndSnapshotName(snapshot.name)
+					if snapshotName == refreshSnapshot {
+						snapVol = snapshot
+						found = true
+						break
+					}
+				}
+
+				if !found {
+					return fmt.Errorf("Snapshot %q missing in volume's list", refreshSnapshot)
+				}
 
 				// Create the snapshot itself.
 				d.Logger().Debug("Creating snapshot", logger.Ctx{"volName": snapVol.Name()})
@@ -1029,7 +1099,7 @@ func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, er
 
 		// Run volume-specific init logic.
 		if initVolume != nil {
-			_, err := initVolume(vol)
+			_, err := initVolume(vol.Volume)
 			if err != nil {
 				return err
 			}
@@ -1045,7 +1115,7 @@ func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, er
 			}
 
 			if srcVol.IsVMBlock() || srcVol.contentType == ContentTypeBlock && srcVol.volType == VolumeTypeCustom {
-				err := sendBlockVol(srcVol, vol)
+				err := sendBlockVol(srcVol.Volume, vol.Volume)
 				if err != nil {
 					return err
 				}
@@ -1067,11 +1137,12 @@ func genericVFSCopyVolume(d Driver, initVolume func(vol Volume) (revert.Hook, er
 		return nil
 	}, op)
 	if err != nil {
-		return err
+		return nil, err
 	}
 
+	cleanup := revert.Clone().Fail
 	revert.Success()
-	return nil
+	return cleanup, nil
 }
 
 // genericVFSListVolumes returns a list of LXD volumes in storage pool.
@@ -1093,14 +1164,21 @@ func genericVFSListVolumes(d Driver) ([]Volume, error) {
 		}
 
 		for _, ent := range ents {
+			volName := ent.Name()
+
 			contentType := ContentTypeFS
 			if volType == VolumeTypeVM {
 				contentType = ContentTypeBlock
-			} else if volType == VolumeTypeCustom && shared.PathExists(filepath.Join(volTypePath, ent.Name(), genericVolumeDiskFile)) {
-				contentType = ContentTypeBlock
+			} else if volType == VolumeTypeCustom && shared.PathExists(filepath.Join(volTypePath, volName, genericVolumeDiskFile)) {
+				if strings.HasSuffix(ent.Name(), genericISOVolumeSuffix) {
+					contentType = ContentTypeISO
+					volName = strings.TrimSuffix(volName, genericISOVolumeSuffix)
+				} else {
+					contentType = ContentTypeBlock
+				}
 			}
 
-			vols = append(vols, NewVolume(d, poolName, volType, contentType, ent.Name(), make(map[string]string), poolConfig))
+			vols = append(vols, NewVolume(d, poolName, volType, contentType, volName, make(map[string]string), poolConfig))
 		}
 	}
 
