@@ -4,14 +4,14 @@ import (
 	"io"
 	"net/url"
 
-	"github.com/lxc/lxd/lxd/backup"
-	"github.com/lxc/lxd/lxd/migration"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/instancewriter"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/backup"
+	"github.com/canonical/lxd/lxd/instancewriter"
+	"github.com/canonical/lxd/lxd/migration"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 // driver is the extended internal interface.
@@ -29,6 +29,7 @@ type Driver interface {
 	Info() Info
 	HasVolume(vol Volume) (bool, error)
 	roundVolumeBlockSizeBytes(sizeBytes int64) int64
+	isBlockBacked(vol Volume) bool
 
 	// Export struct details.
 	Name() string
@@ -36,6 +37,7 @@ type Driver interface {
 	Logger() logger.Logger
 
 	// Pool.
+	FillConfig() error
 	Create() error
 	Delete(op *operations.Operation) error
 	// Mount mounts a storage pool if needed, returns true if we caused a new mount, false if already mounted.
@@ -63,8 +65,8 @@ type Driver interface {
 	FillVolumeConfig(vol Volume) error
 	ValidateVolume(vol Volume, removeUnknownKeys bool) error
 	CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Operation) error
-	CreateVolumeFromCopy(vol Volume, srcVol Volume, copySnapshots bool, allowInconsistent bool, op *operations.Operation) error
-	RefreshVolume(vol Volume, srcVol Volume, srcSnapshots []Volume, allowInconsistent bool, op *operations.Operation) error
+	CreateVolumeFromCopy(vol VolumeCopy, srcVol VolumeCopy, allowInconsistent bool, op *operations.Operation) error
+	RefreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots []string, allowInconsistent bool, op *operations.Operation) error
 	DeleteVolume(vol Volume, op *operations.Operation) error
 	RenameVolume(vol Volume, newName string, op *operations.Operation) error
 	UpdateVolume(vol Volume, changedConfig map[string]string) error
@@ -79,6 +81,12 @@ type Driver interface {
 	// MountVolumeSnapshot mounts a storage volume snapshot as readonly.
 	MountVolumeSnapshot(snapVol Volume, op *operations.Operation) error
 
+	// CanDelegateVolume checks whether the volume can be delegated.
+	CanDelegateVolume(vol Volume) bool
+
+	// DelegateVolume allows for the volume to be managed by the instance.
+	DelegateVolume(vol Volume, pid int) error
+
 	// UnmountVolume unmounts a storage volume, returns true if unmounted, false if was not
 	// mounted.
 	UnmountVolume(vol Volume, keepBlockDev bool, op *operations.Operation) (bool, error)
@@ -91,14 +99,15 @@ type Driver interface {
 	DeleteVolumeSnapshot(snapVol Volume, op *operations.Operation) error
 	RenameVolumeSnapshot(snapVol Volume, newSnapshotName string, op *operations.Operation) error
 	VolumeSnapshots(vol Volume, op *operations.Operation) ([]string, error)
-	RestoreVolume(vol Volume, snapshotName string, op *operations.Operation) error
+	CheckVolumeSnapshots(vol Volume, snapVols []Volume, op *operations.Operation) error
+	RestoreVolume(vol Volume, snapVol Volume, op *operations.Operation) error
 
 	// Migration.
-	MigrationTypes(contentType ContentType, refresh bool) []migration.Type
-	MigrateVolume(vol Volume, conn io.ReadWriteCloser, volSrcArgs *migration.VolumeSourceArgs, op *operations.Operation) error
-	CreateVolumeFromMigration(vol Volume, conn io.ReadWriteCloser, volTargetArgs migration.VolumeTargetArgs, preFiller *VolumeFiller, op *operations.Operation) error
+	MigrationTypes(contentType ContentType, refresh bool, copySnapshots bool) []migration.Type
+	MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs *migration.VolumeSourceArgs, op *operations.Operation) error
+	CreateVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteCloser, volTargetArgs migration.VolumeTargetArgs, preFiller *VolumeFiller, op *operations.Operation) error
 
 	// Backup.
-	BackupVolume(vol Volume, tarWriter *instancewriter.InstanceTarWriter, optimized bool, snapshots []string, op *operations.Operation) error
-	CreateVolumeFromBackup(vol Volume, srcBackup backup.Info, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error)
+	BackupVolume(vol VolumeCopy, tarWriter *instancewriter.InstanceTarWriter, optimized bool, snapshots []string, op *operations.Operation) error
+	CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error)
 }
