@@ -1,8 +1,8 @@
 package drivers
 
 import (
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 var drivers = map[string]func() driver{
@@ -12,6 +12,7 @@ var drivers = map[string]func() driver{
 	"cephobject": func() driver { return &cephobject{} },
 	"dir":        func() driver { return &dir{} },
 	"lvm":        func() driver { return &lvm{} },
+	"powerflex":  func() driver { return &powerflex{} },
 	"zfs":        func() driver { return &zfs{} },
 }
 
