@@ -2,6 +2,8 @@ package drivers
 
 import (
 	"fmt"
+	"io"
+	"io/fs"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -11,17 +13,20 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // MinBlockBoundary minimum block boundary size to use.
 const MinBlockBoundary = 8192
 
+// blockBackedAllowedFilesystems allowed filesystems for block volumes.
+var blockBackedAllowedFilesystems = []string{"btrfs", "ext4", "xfs"}
+
 // wipeDirectory empties the contents of a directory, but leaves it in place.
 func wipeDirectory(path string) error {
 	// List all entries.
@@ -409,75 +414,13 @@ func makeFSType(path string, fsType string, options *mkfsOptions) (string, error
 	return "", nil
 }
 
-// mountOption represents an individual mount option.
-type mountOption struct {
-	capture bool
-	flag    uintptr
-}
-
-// mountOptions represents a list of possible mount options.
-var mountOptions = map[string]mountOption{
-	"async":         {false, unix.MS_SYNCHRONOUS},
-	"atime":         {false, unix.MS_NOATIME},
-	"bind":          {true, unix.MS_BIND},
-	"defaults":      {true, 0},
-	"dev":           {false, unix.MS_NODEV},
-	"diratime":      {false, unix.MS_NODIRATIME},
-	"dirsync":       {true, unix.MS_DIRSYNC},
-	"exec":          {false, unix.MS_NOEXEC},
-	"lazytime":      {true, unix.MS_LAZYTIME},
-	"mand":          {true, unix.MS_MANDLOCK},
-	"noatime":       {true, unix.MS_NOATIME},
-	"nodev":         {true, unix.MS_NODEV},
-	"nodiratime":    {true, unix.MS_NODIRATIME},
-	"noexec":        {true, unix.MS_NOEXEC},
-	"nomand":        {false, unix.MS_MANDLOCK},
-	"norelatime":    {false, unix.MS_RELATIME},
-	"nostrictatime": {false, unix.MS_STRICTATIME},
-	"nosuid":        {true, unix.MS_NOSUID},
-	"rbind":         {true, unix.MS_BIND | unix.MS_REC},
-	"relatime":      {true, unix.MS_RELATIME},
-	"remount":       {true, unix.MS_REMOUNT},
-	"ro":            {true, unix.MS_RDONLY},
-	"rw":            {false, unix.MS_RDONLY},
-	"strictatime":   {true, unix.MS_STRICTATIME},
-	"suid":          {false, unix.MS_NOSUID},
-	"sync":          {true, unix.MS_SYNCHRONOUS},
-}
-
-// resolveMountOptions resolves the provided mount options.
-func resolveMountOptions(options string) (uintptr, string) {
-	mountFlags := uintptr(0)
-	tmp := strings.SplitN(options, ",", -1)
-	for i := 0; i < len(tmp); i++ {
-		opt := tmp[i]
-		do, ok := mountOptions[opt]
-		if !ok {
-			continue
-		}
-
-		if do.capture {
-			mountFlags |= do.flag
-		} else {
-			mountFlags &= ^do.flag
-		}
-
-		copy(tmp[i:], tmp[i+1:])
-		tmp[len(tmp)-1] = ""
-		tmp = tmp[:len(tmp)-1]
-		i--
-	}
-
-	return mountFlags, strings.Join(tmp, ",")
-}
-
 // filesystemTypeCanBeShrunk indicates if filesystems of fsType can be shrunk.
 func filesystemTypeCanBeShrunk(fsType string) bool {
 	if fsType == "" {
 		fsType = DefaultFilesystem
 	}
 
-	if shared.StringInSlice(fsType, []string{"ext4", "btrfs"}) {
+	if shared.ValueInSlice(fsType, []string{"ext4", "btrfs"}) {
 		return true
 	}
 
@@ -697,7 +640,7 @@ func UnshiftBtrfsRootfs(path string, diskIdmap *idmap.IdmapSet) error {
 	return shiftBtrfsRootfs(path, diskIdmap, false)
 }
 
-// shiftBtrfsRootfs shiftfs a filesystem that main include read-only subvolumes.
+// shiftBtrfsRootfs shifts a filesystem that main include read-only subvolumes.
 func shiftBtrfsRootfs(path string, diskIdmap *idmap.IdmapSet, shift bool) error {
 	var err error
 	roSubvols := []string{}
@@ -735,24 +678,24 @@ func BTRFSSubVolumesGet(path string) ([]string, error) {
 		path = path + "/"
 	}
 
-	// Unprivileged users can't get to fs internals
-	_ = filepath.Walk(path, func(fpath string, fi os.FileInfo, err error) error {
+	// Unprivileged users can't get to fs internals.
+	_ = filepath.WalkDir(path, func(fpath string, entry fs.DirEntry, err error) error {
 		// Skip walk errors
 		if err != nil {
 			return nil
 		}
 
-		// Ignore the base path
+		// Ignore the base path.
 		if strings.TrimRight(fpath, "/") == strings.TrimRight(path, "/") {
 			return nil
 		}
 
-		// Subvolumes can only be directories
-		if !fi.IsDir() {
+		// Subvolumes can only be directories.
+		if !entry.IsDir() {
 			return nil
 		}
 
-		// Check if a btrfs subvolume
+		// Check if a btrfs subvolume.
 		if btrfsIsSubVolume(fpath) {
 			result = append(result, strings.TrimPrefix(fpath, path))
 		}
@@ -763,6 +706,7 @@ func BTRFSSubVolumesGet(path string) ([]string, error) {
 	return result, nil
 }
 
+// Deprecated: Use IsSubvolume from the Btrfs driver instead.
 // btrfsIsSubvolume checks if a given path is a subvolume.
 func btrfsIsSubVolume(subvolPath string) bool {
 	fs := unix.Stat_t{}
@@ -850,26 +794,25 @@ func OperationLockName(operationName string, poolName string, volType VolumeType
 	return fmt.Sprintf("%s/%s/%s/%s/%s", operationName, poolName, volType, contentType, volName)
 }
 
-// loopFileSizeDefault returns the size in Gigabytes to use as the default size for a pool loop file.
-// This is based on the free space available in LXD's VarPath().
+// loopFileSizeDefault returns the size in GiB to use as the default size for a pool loop file.
+// This is based on the size of the filesystem of LXD's VarPath().
 func loopFileSizeDefault() (uint64, error) {
 	st := unix.Statfs_t{}
 	err := unix.Statfs(shared.VarPath(), &st)
 	if err != nil {
-		return 0, fmt.Errorf("Couldn't statfs %q: %w", shared.VarPath(), err)
+		return 0, fmt.Errorf("Failed getting free space of %q: %w", shared.VarPath(), err)
 	}
 
-	/* choose 5 GiB < x < 30GiB, where x is 20% of the disk size */
-	defaultSize := uint64(st.Frsize) * st.Blocks / (1024 * 1024 * 1024) / 5
-	if defaultSize > 30 {
-		defaultSize = 30
+	gibAvailable := uint64(st.Frsize) * st.Bavail / (1024 * 1024 * 1024)
+	if gibAvailable > 30 {
+		return 30, nil // Default to no more than 30GiB.
+	} else if gibAvailable > 5 {
+		return gibAvailable / 5, nil // Use 20% of free space otherwise.
+	} else if gibAvailable == 5 {
+		return gibAvailable, nil // Need at least 5GiB free.
 	}
 
-	if defaultSize < 5 {
-		defaultSize = 5
-	}
-
-	return defaultSize, nil
+	return 0, fmt.Errorf("Insufficient free space to create default sized 5GiB pool")
 }
 
 // loopFileSetup sets up a loop device for the provided sourcePath.
@@ -877,7 +820,7 @@ func loopFileSizeDefault() (uint64, error) {
 func loopDeviceSetup(sourcePath string) (string, error) {
 	out, err := shared.RunCommand("losetup", "--find", "--nooverlap", "--direct-io=on", "--show", sourcePath)
 	if err != nil {
-		if strings.Contains(err.Error(), "direct io") {
+		if strings.Contains(err.Error(), "direct io") || strings.Contains(err.Error(), "Invalid argument") {
 			out, err = shared.RunCommand("losetup", "--find", "--nooverlap", "--show", sourcePath)
 			if err != nil {
 				return "", err
@@ -895,3 +838,41 @@ func loopDeviceAutoDetach(loopDevPath string) error {
 	_, err := shared.RunCommand("losetup", "--detach", loopDevPath)
 	return err
 }
+
+// loopDeviceSetCapacity forces the loop driver to reread the size of the file associated with the specified loop device.
+func loopDeviceSetCapacity(loopDevPath string) error {
+	_, err := shared.RunCommand("losetup", "--set-capacity", loopDevPath)
+	return err
+}
+
+// wipeBlockHeaders will wipe the first 4MB of a block device.
+func wipeBlockHeaders(path string) error {
+	// Open /dev/zero.
+	fdZero, err := os.Open("/dev/zero")
+	if err != nil {
+		return err
+	}
+
+	defer fdZero.Close()
+
+	// Open the target disk.
+	fdDisk, err := os.OpenFile(path, os.O_RDWR, 0600)
+	if err != nil {
+		return err
+	}
+
+	defer fdDisk.Close()
+
+	// Wipe the 4MiB header.
+	_, err = io.CopyN(fdDisk, fdZero, 1024*1024*4)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// IsContentBlock returns true if the content type is either block or iso.
+func IsContentBlock(contentType ContentType) bool {
+	return contentType == ContentTypeBlock || contentType == ContentTypeISO
+}
