@@ -6,14 +6,14 @@ import (
 	"os"
 	"sort"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // InstancePath returns the directory of an instance or snapshot.
@@ -143,12 +143,12 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 		for _, vol := range volumes {
 			var u *api.URL
 
-			if shared.StringInSlice(vol.Type, ignoreVolumeType) {
+			if shared.ValueInSlice(vol.Type, ignoreVolumeType) {
 				continue
 			}
 
 			// Generate URL for volume based on types that map to other entities.
-			if vol.Type == db.StoragePoolVolumeTypeNameContainer || vol.Type == db.StoragePoolVolumeTypeNameVM {
+			if vol.Type == cluster.StoragePoolVolumeTypeNameContainer || vol.Type == cluster.StoragePoolVolumeTypeNameVM {
 				volName, snapName, isSnap := api.GetParentAndSnapshotName(vol.Name)
 				if isSnap {
 					u = api.NewURL().Path(version.APIVersion, "instances", volName, "snapshots", snapName).Project(vol.Project)
@@ -157,7 +157,7 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 				}
 
 				usedBy = append(usedBy, u.String())
-			} else if vol.Type == db.StoragePoolVolumeTypeNameImage {
+			} else if vol.Type == cluster.StoragePoolVolumeTypeNameImage {
 				imgProjectNames, err := tx.GetProjectsUsingImage(ctx, vol.Name)
 				if err != nil {
 					return fmt.Errorf("Failed loading projects using image %q: %w", vol.Name, err)
@@ -170,11 +170,11 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 					}
 				} else {
 					// Handle orphaned image volumes that are not associated to an image.
-					u = vol.URL(version.APIVersion, pool.Name())
+					u = vol.URL(version.APIVersion)
 					usedBy = append(usedBy, u.String())
 				}
 			} else {
-				u = vol.URL(version.APIVersion, pool.Name())
+				u = vol.URL(version.APIVersion)
 				usedBy = append(usedBy, u.String())
 			}
 
@@ -183,6 +183,26 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 			}
 		}
 
+		// Get all buckets using the storage pool.
+		poolID := pool.ID()
+		filters := []db.StorageBucketFilter{{
+			PoolID: &poolID,
+		}}
+
+		buckets, err := tx.GetStoragePoolBuckets(ctx, memberSpecific, filters...)
+		if err != nil {
+			return fmt.Errorf("Failed loading storage buckets: %w", err)
+		}
+
+		for _, bucket := range buckets {
+			u := bucket.URL(version.APIVersion, pool.Name(), bucket.Project)
+			usedBy = append(usedBy, u.String())
+
+			if firstOnly {
+				return nil
+			}
+		}
+
 		// Get all the profiles using the storage pool.
 		profiles, err := cluster.GetProfiles(ctx, tx.Tx())
 		if err != nil {
@@ -210,6 +230,8 @@ func UsedBy(ctx context.Context, s *state.State, pool Pool, firstOnly bool, memb
 				if firstOnly {
 					return nil
 				}
+
+				break
 			}
 		}
 
