@@ -11,26 +11,26 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/lxd/apparmor"
-	"github.com/lxc/lxd/lxd/archive"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/migration"
-	"github.com/lxc/lxd/lxd/node"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/rsync"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/storage/drivers"
-	"github.com/lxc/lxd/lxd/sys"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/validate"
+	"github.com/canonical/lxd/lxd/apparmor"
+	"github.com/canonical/lxd/lxd/archive"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/migration"
+	"github.com/canonical/lxd/lxd/node"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/rsync"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/storage/drivers"
+	"github.com/canonical/lxd/lxd/sys"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/validate"
 )
 
 // ConfigDiff returns a diff of the provided configs. Additionally, it returns whether or not
@@ -44,7 +44,7 @@ func ConfigDiff(oldConfig map[string]string, newConfig map[string]string) ([]str
 				userOnly = false
 			}
 
-			if !shared.StringInSlice(key, changedConfig) {
+			if !shared.ValueInSlice(key, changedConfig) {
 				changedConfig = append(changedConfig, key)
 			}
 		}
@@ -56,7 +56,7 @@ func ConfigDiff(oldConfig map[string]string, newConfig map[string]string) ([]str
 				userOnly = false
 			}
 
-			if !shared.StringInSlice(key, changedConfig) {
+			if !shared.ValueInSlice(key, changedConfig) {
 				changedConfig = append(changedConfig, key)
 			}
 		}
@@ -73,14 +73,14 @@ func ConfigDiff(oldConfig map[string]string, newConfig map[string]string) ([]str
 // VolumeTypeNameToDBType converts a volume type string to internal volume type DB code.
 func VolumeTypeNameToDBType(volumeTypeName string) (int, error) {
 	switch volumeTypeName {
-	case db.StoragePoolVolumeTypeNameContainer:
-		return db.StoragePoolVolumeTypeContainer, nil
-	case db.StoragePoolVolumeTypeNameVM:
-		return db.StoragePoolVolumeTypeVM, nil
-	case db.StoragePoolVolumeTypeNameImage:
-		return db.StoragePoolVolumeTypeImage, nil
-	case db.StoragePoolVolumeTypeNameCustom:
-		return db.StoragePoolVolumeTypeCustom, nil
+	case cluster.StoragePoolVolumeTypeNameContainer:
+		return cluster.StoragePoolVolumeTypeContainer, nil
+	case cluster.StoragePoolVolumeTypeNameVM:
+		return cluster.StoragePoolVolumeTypeVM, nil
+	case cluster.StoragePoolVolumeTypeNameImage:
+		return cluster.StoragePoolVolumeTypeImage, nil
+	case cluster.StoragePoolVolumeTypeNameCustom:
+		return cluster.StoragePoolVolumeTypeCustom, nil
 	}
 
 	return -1, fmt.Errorf("Invalid storage volume type name")
@@ -90,13 +90,13 @@ func VolumeTypeNameToDBType(volumeTypeName string) (int, error) {
 func VolumeTypeToDBType(volType drivers.VolumeType) (int, error) {
 	switch volType {
 	case drivers.VolumeTypeContainer:
-		return db.StoragePoolVolumeTypeContainer, nil
+		return cluster.StoragePoolVolumeTypeContainer, nil
 	case drivers.VolumeTypeVM:
-		return db.StoragePoolVolumeTypeVM, nil
+		return cluster.StoragePoolVolumeTypeVM, nil
 	case drivers.VolumeTypeImage:
-		return db.StoragePoolVolumeTypeImage, nil
+		return cluster.StoragePoolVolumeTypeImage, nil
 	case drivers.VolumeTypeCustom:
-		return db.StoragePoolVolumeTypeCustom, nil
+		return cluster.StoragePoolVolumeTypeCustom, nil
 	}
 
 	return -1, fmt.Errorf("Invalid storage volume type: %q", volType)
@@ -105,13 +105,13 @@ func VolumeTypeToDBType(volType drivers.VolumeType) (int, error) {
 // VolumeDBTypeToType converts internal volume type DB code to storage driver volume type.
 func VolumeDBTypeToType(volDBType int) (drivers.VolumeType, error) {
 	switch volDBType {
-	case db.StoragePoolVolumeTypeContainer:
+	case cluster.StoragePoolVolumeTypeContainer:
 		return drivers.VolumeTypeContainer, nil
-	case db.StoragePoolVolumeTypeVM:
+	case cluster.StoragePoolVolumeTypeVM:
 		return drivers.VolumeTypeVM, nil
-	case db.StoragePoolVolumeTypeImage:
+	case cluster.StoragePoolVolumeTypeImage:
 		return drivers.VolumeTypeImage, nil
-	case db.StoragePoolVolumeTypeCustom:
+	case cluster.StoragePoolVolumeTypeCustom:
 		return drivers.VolumeTypeCustom, nil
 	}
 
@@ -146,9 +146,11 @@ func VolumeTypeToAPIInstanceType(volType drivers.VolumeType) (api.InstanceType,
 func VolumeContentTypeToDBContentType(contentType drivers.ContentType) (int, error) {
 	switch contentType {
 	case drivers.ContentTypeBlock:
-		return db.StoragePoolVolumeContentTypeBlock, nil
+		return cluster.StoragePoolVolumeContentTypeBlock, nil
 	case drivers.ContentTypeFS:
-		return db.StoragePoolVolumeContentTypeFS, nil
+		return cluster.StoragePoolVolumeContentTypeFS, nil
+	case drivers.ContentTypeISO:
+		return cluster.StoragePoolVolumeContentTypeISO, nil
 	}
 
 	return -1, fmt.Errorf("Invalid volume content type")
@@ -157,10 +159,12 @@ func VolumeContentTypeToDBContentType(contentType drivers.ContentType) (int, err
 // VolumeDBContentTypeToContentType converts internal content type DB code to driver representation.
 func VolumeDBContentTypeToContentType(volDBType int) (drivers.ContentType, error) {
 	switch volDBType {
-	case db.StoragePoolVolumeContentTypeBlock:
+	case cluster.StoragePoolVolumeContentTypeBlock:
 		return drivers.ContentTypeBlock, nil
-	case db.StoragePoolVolumeContentTypeFS:
+	case cluster.StoragePoolVolumeContentTypeFS:
 		return drivers.ContentTypeFS, nil
+	case cluster.StoragePoolVolumeContentTypeISO:
+		return drivers.ContentTypeISO, nil
 	}
 
 	return "", fmt.Errorf("Invalid volume content type")
@@ -169,10 +173,12 @@ func VolumeDBContentTypeToContentType(volDBType int) (drivers.ContentType, error
 // VolumeContentTypeNameToContentType converts volume content type string internal code.
 func VolumeContentTypeNameToContentType(contentTypeName string) (int, error) {
 	switch contentTypeName {
-	case db.StoragePoolVolumeContentTypeNameFS:
-		return db.StoragePoolVolumeContentTypeFS, nil
-	case db.StoragePoolVolumeContentTypeNameBlock:
-		return db.StoragePoolVolumeContentTypeBlock, nil
+	case cluster.StoragePoolVolumeContentTypeNameFS:
+		return cluster.StoragePoolVolumeContentTypeFS, nil
+	case cluster.StoragePoolVolumeContentTypeNameBlock:
+		return cluster.StoragePoolVolumeContentTypeBlock, nil
+	case cluster.StoragePoolVolumeContentTypeNameISO:
+		return cluster.StoragePoolVolumeContentTypeISO, nil
 	}
 
 	return -1, fmt.Errorf("Invalid volume content type name")
@@ -214,7 +220,7 @@ func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType dr
 // VolumeDBCreate creates a volume in the database.
 // If volumeConfig is supplied, it is modified with any driver level default config options (if not set).
 // If removeUnknownKeys is true, any unknown config keys are removed from volumeConfig rather than failing.
-func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDescription string, volumeType drivers.VolumeType, snapshot bool, volumeConfig map[string]string, expiryDate time.Time, contentType drivers.ContentType, removeUnknownKeys bool) error {
+func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDescription string, volumeType drivers.VolumeType, snapshot bool, volumeConfig map[string]string, creationDate time.Time, expiryDate time.Time, contentType drivers.ContentType, removeUnknownKeys bool, hasSource bool) error {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
 		return fmt.Errorf("Pool is not a lxdBackend")
@@ -259,6 +265,9 @@ func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDesc
 
 	vol := drivers.NewVolume(pool.Driver(), pool.Name(), volType, contentType, volumeName, volumeConfig, pool.Driver().Config())
 
+	// Set source indicator.
+	vol.SetHasSource(hasSource)
+
 	// Fill default config.
 	err = pool.Driver().FillVolumeConfig(vol)
 	if err != nil {
@@ -271,13 +280,16 @@ func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDesc
 		return err
 	}
 
-	// Create the database entry for the storage volume.
-	if snapshot {
-		_, err = p.state.DB.Cluster.CreateStorageVolumeSnapshot(projectName, volumeName, volumeDescription, volDBType, pool.ID(), vol.Config(), expiryDate)
-	} else {
-		_, err = p.state.DB.Cluster.CreateStoragePoolVolume(projectName, volumeName, volumeDescription, volDBType, pool.ID(), vol.Config(), volDBContentType)
-	}
+	err = p.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Create the database entry for the storage volume.
+		if snapshot {
+			_, err = tx.CreateStorageVolumeSnapshot(ctx, projectName, volumeName, volumeDescription, volDBType, pool.ID(), vol.Config(), creationDate, expiryDate)
+		} else {
+			_, err = tx.CreateStoragePoolVolume(ctx, projectName, volumeName, volumeDescription, volDBType, pool.ID(), vol.Config(), volDBContentType, creationDate)
+		}
 
+		return err
+	})
 	if err != nil {
 		return fmt.Errorf("Error inserting volume %q for project %q in pool %q of type %q into database %q", volumeName, projectName, pool.Name(), volumeType, err)
 	}
@@ -298,7 +310,9 @@ func VolumeDBDelete(pool Pool, projectName string, volumeName string, volumeType
 		return err
 	}
 
-	err = p.state.DB.Cluster.RemoveStoragePoolVolume(projectName, volumeName, volDBType, pool.ID())
+	err = p.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.RemoveStoragePoolVolume(ctx, projectName, volumeName, volDBType, pool.ID())
+	})
 	if err != nil && !response.IsNotFoundError(err) {
 		return fmt.Errorf("Error deleting storage volume from database: %w", err)
 	}
@@ -318,7 +332,13 @@ func VolumeDBSnapshotsGet(pool Pool, projectName string, volume string, volumeTy
 		return nil, err
 	}
 
-	snapshots, err := p.state.DB.Cluster.GetLocalStoragePoolVolumeSnapshotsWithType(projectName, volume, volDBType, pool.ID())
+	var snapshots []db.StorageVolumeArgs
+
+	err = p.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		snapshots, err = tx.GetLocalStoragePoolVolumeSnapshotsWithType(ctx, projectName, volume, volDBType, pool.ID())
+
+		return err
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -326,38 +346,273 @@ func VolumeDBSnapshotsGet(pool Pool, projectName string, volume string, volumeTy
 	return snapshots, nil
 }
 
+// BucketDBGet loads a bucket from the database.
+func BucketDBGet(pool Pool, projectName string, bucketName string, memberSpecific bool) (*db.StorageBucket, error) {
+	p, ok := pool.(*lxdBackend)
+	if !ok {
+		return nil, fmt.Errorf("Pool is not a lxdBackend")
+	}
+
+	var err error
+	var bucket *db.StorageBucket
+
+	// Get the storage bucket.
+	err = p.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		bucket, err = tx.GetStoragePoolBucket(ctx, pool.ID(), projectName, memberSpecific, bucketName)
+		if err != nil {
+			if response.IsNotFoundError(err) {
+				return fmt.Errorf("Storage bucket %q in project %q does not exist on pool %q: %w", bucketName, projectName, pool.Name(), err)
+			}
+
+			return err
+		}
+
+		return nil
+	})
+	if err != nil {
+		return nil, err
+	}
+
+	return bucket, nil
+}
+
+// BucketDBCreate creates a bucket in the database.
+// The supplied bucket's config may be modified with defaults for the storage pool being used.
+// Returns bucket DB record ID.
+func BucketDBCreate(ctx context.Context, pool Pool, projectName string, memberSpecific bool, bucket *api.StorageBucketsPost) (int64, error) {
+	p, ok := pool.(*lxdBackend)
+	if !ok {
+		return -1, fmt.Errorf("Pool is not a lxdBackend")
+	}
+
+	// Make sure that we don't pass a nil to the next function.
+	if bucket.Config == nil {
+		bucket.Config = map[string]string{}
+	}
+
+	bucketVolName := project.StorageVolume(projectName, bucket.Name)
+	bucketVol := drivers.NewVolume(pool.Driver(), pool.Name(), drivers.VolumeTypeBucket, drivers.ContentTypeFS, bucketVolName, bucket.Config, pool.Driver().Config())
+
+	// Fill default config.
+	err := pool.Driver().FillVolumeConfig(bucketVol)
+	if err != nil {
+		return -1, err
+	}
+
+	// Validate bucket name.
+	err = pool.Driver().ValidateBucket(bucketVol)
+	if err != nil {
+		return -1, err
+	}
+
+	// Validate bucket volume config.
+	err = pool.Driver().ValidateVolume(bucketVol, false)
+	if err != nil {
+		return -1, err
+	}
+
+	var bucketID int64
+
+	err = p.state.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		// Create the database entry for the storage bucket.
+		bucketID, err = tx.CreateStoragePoolBucket(ctx, p.ID(), projectName, memberSpecific, *bucket)
+
+		return err
+	})
+	if err != nil {
+		return -1, fmt.Errorf("Failed inserting storage bucket %q for project %q in pool %q into database: %w", bucket.Name, projectName, pool.Name(), err)
+	}
+
+	return bucketID, nil
+}
+
+// BucketDBDelete deletes a bucket from the database.
+func BucketDBDelete(ctx context.Context, pool Pool, bucketID int64) error {
+	p, ok := pool.(*lxdBackend)
+	if !ok {
+		return fmt.Errorf("Pool is not a lxdBackend")
+	}
+
+	err := p.state.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.DeleteStoragePoolBucket(ctx, p.ID(), bucketID)
+	})
+	if err != nil && !response.IsNotFoundError(err) {
+		return fmt.Errorf("Failed deleting storage bucket from database: %w", err)
+	}
+
+	return nil
+}
+
 // poolAndVolumeCommonRules returns a map of pool and volume config common rules common to all drivers.
 // When vol argument is nil function returns pool specific rules.
 func poolAndVolumeCommonRules(vol *drivers.Volume) map[string]func(string) error {
 	rules := map[string]func(string) error{
 		// Note: size should not be modifiable for non-custom volumes and should be checked
 		// in the relevant volume update functions.
+
+		// lxdmeta:generate(entities=storage-btrfs,storage-lvm,storage-zfs; group=pool-conf; key=size)
+		//
+		// When creating loop-based pools, specify the size in bytes ({ref}`suffixes <instances-limit-units>` are supported).
+		// You can increase the size to grow the storage pool.
+		//
+		// The default (`auto`) creates a storage pool that uses 20% of the free disk space,
+		// with a minimum of 5 GiB and a maximum of 30 GiB.
+		// ---
+		//  type: string
+		//  defaultdesc: auto (20% of free disk space, >= 5 GiB and <= 30 GiB)
+		//  shortdesc: Size of the storage pool (for loop-based pools)
+
+		// lxdmeta:generate(entities=storage-btrfs,storage-cephfs,storage-ceph,storage-dir,storage-lvm,storage-zfs; group=volume-conf; key=size)
+		//
+		// ---
+		//  type: string
+		//  condition: appropriate driver
+		//  defaultdesc: same as `volume.size`
+		//  shortdesc: Size/quota of the storage volume
+
+		// lxdmeta:generate(entities=storage-cephobject; group=bucket-conf; key=size)
+		//
+		// ---
+		//  type: string
+		//  shortdesc: Quota of the storage bucket
+
+		// lxdmeta:generate(entities=storage-btrfs,storage-lvm,storage-zfs; group=bucket-conf; key=size)
+		//
+		// ---
+		//  type: string
+		//  condition: appropriate driver
+		//  defaultdesc: same as `volume.size`
+		//  shortdesc: Size/quota of the storage bucket
 		"size": validate.Optional(validate.IsSize),
+		// lxdmeta:generate(entities=storage-btrfs,storage-cephfs,storage-ceph,storage-dir,storage-lvm,storage-zfs,storage-powerflex; group=volume-conf; key=snapshots.expiry)
+		// Specify an expression like `1M 2H 3d 4w 5m 6y`.
+		// ---
+		//  type: string
+		//  condition: custom volume
+		//  defaultdesc: same as `volume.snapshots.expiry`
+		//  shortdesc: When snapshots are to be deleted
 		"snapshots.expiry": func(value string) error {
 			// Validate expression
 			_, err := shared.GetExpiry(time.Time{}, value)
 			return err
 		},
+		// lxdmeta:generate(entities=storage-btrfs,storage-cephfs,storage-ceph,storage-dir,storage-lvm,storage-zfs,storage-powerflex; group=volume-conf; key=snapshots.schedule)
+		// Specify either a cron expression (`<minute> <hour> <dom> <month> <dow>`), a comma-separated list of schedule aliases (`@hourly`, `@daily`, `@midnight`, `@weekly`, `@monthly`, `@annually`, `@yearly`), or leave empty to disable automatic snapshots (the default).
+		// ---
+		//  type: string
+		//  condition: custom volume
+		//  defaultdesc: same as `snapshots.schedule`
+		//  shortdesc: Schedule for automatic volume snapshots
 		"snapshots.schedule": validate.Optional(validate.IsCron([]string{"@hourly", "@daily", "@midnight", "@weekly", "@monthly", "@annually", "@yearly"})),
-		"snapshots.pattern":  validate.IsAny,
+		// lxdmeta:generate(entities=storage-btrfs,storage-cephfs,storage-ceph,storage-dir,storage-lvm,storage-zfs,storage-powerflex; group=volume-conf; key=snapshots.pattern)
+		// You can specify a naming template that is used for scheduled snapshots and unnamed snapshots.
+		//
+		// {{snapshot_pattern_detail}}
+		// ---
+		//  type: string
+		//  condition: custom volume
+		//  defaultdesc: same as `volume.snapshots.pattern` or `snap%d`
+		//  shortdesc: Template for the snapshot name
+		"snapshots.pattern": validate.IsAny,
 	}
 
 	// security.shifted and security.unmapped are only relevant for custom filesystem volumes.
 	if (vol == nil) || (vol != nil && vol.Type() == drivers.VolumeTypeCustom && vol.ContentType() == drivers.ContentTypeFS) {
+		// lxdmeta:generate(entities=storage-btrfs,storage-cephfs,storage-ceph,storage-dir,storage-lvm,storage-zfs,storage-powerflex; group=volume-conf; key=security.shifted)
+		// Enabling this option allows attaching the volume to multiple isolated instances.
+		// ---
+		//  type: bool
+		//  condition: custom volume
+		//  defaultdesc: same as `volume.security.shifted` or `false`
+		//  shortdesc: Enable ID shifting overlay
 		rules["security.shifted"] = validate.Optional(validate.IsBool)
+		// lxdmeta:generate(entities=storage-btrfs,storage-cephfs,storage-ceph,storage-dir,storage-lvm,storage-zfs,storage-powerflex; group=volume-conf; key=security.unmapped)
+		//
+		// ---
+		//  type: bool
+		//  condition: custom volume
+		//  defaultdesc: same as `volume.security.unmappped` or `false`
+		//  shortdesc: Disable ID mapping for the volume
 		rules["security.unmapped"] = validate.Optional(validate.IsBool)
 	}
 
+	// Those keys are only valid for volumes.
+	if vol != nil {
+		// lxdmeta:generate(entities=storage-btrfs,storage-cephfs,storage-ceph,storage-dir,storage-lvm,storage-zfs,storage-powerflex; group=volume-conf; key=volatile.uuid)
+		//
+		// ---
+		//  type: string
+		//  defaultdesc: random UUID
+		//  shortdesc: The volume's UUID
+		rules["volatile.uuid"] = validate.Optional(validate.IsUUID)
+	}
+
 	return rules
 }
 
 // validatePoolCommonRules returns a map of pool config rules common to all drivers.
 func validatePoolCommonRules() map[string]func(string) error {
 	rules := map[string]func(string) error{
-		"source":                  validate.IsAny,
+		// lxdmeta:generate(entities=storage-btrfs; group=pool-conf; key=source)
+		//
+		// ---
+		//  type: string
+		//  shortdesc: Path to an existing block device, loop file, or Btrfs subvolume
+
+		// lxdmeta:generate(entities=storage-cephfs; group=pool-conf; key=source)
+		//
+		// ---
+		//  type: string
+		//  shortdesc: Existing CephFS file system or file system path to use
+
+		// lxdmeta:generate(entities=storage-ceph; group=pool-conf; key=source)
+		//
+		// ---
+		//  type: string
+		//  shortdesc: Existing OSD storage pool to use
+
+		// lxdmeta:generate(entities=storage-dir; group=pool-conf; key=source)
+		//
+		// ---
+		//  type: string
+		//  shortdesc: Path to an existing directory
+
+		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=source)
+		//
+		// ---
+		//  type: string
+		//  shortdesc: Path to an existing block device, loop file, or LVM volume group
+
+		// lxdmeta:generate(entities=storage-zfs; group=pool-conf; key=source)
+		//
+		// ---
+		//  type: string
+		//  shortdesc: Path to an existing block device, loop file, or ZFS dataset/pool
+		"source": validate.IsAny,
+		// lxdmeta:generate(entities=storage-btrfs,storage-lvm,storage-zfs; group=pool-conf; key=source.wipe)
+		// Set this option to `true` to wipe the block device specified in `source`
+		// prior to creating the storage pool.
+		// ---
+		//  type: bool
+		//  defaultdesc: `false`
+		//  shortdesc: Whether to wipe the block device before creating the pool
+		"source.wipe":             validate.Optional(validate.IsBool),
 		"volatile.initial_source": validate.IsAny,
-		"rsync.bwlimit":           validate.Optional(validate.IsSize),
-		"rsync.compression":       validate.Optional(validate.IsBool),
+		// lxdmeta:generate(entities=storage-dir,storage-lvm,storage-powerflex; group=pool-conf; key=rsync.bwlimit)
+		// When `rsync` must be used to transfer storage entities, this option specifies the upper limit
+		// to be placed on the socket I/O.
+		// ---
+		//  type: string
+		//  defaultdesc: `0` (no limit)
+		//  shortdesc: Upper limit on the socket I/O for `rsync`
+		"rsync.bwlimit": validate.Optional(validate.IsSize),
+		// lxdmeta:generate(entities=storage-dir,storage-lvm,storage-powerflex; group=pool-conf; key=rsync.compression)
+		//
+		// ---
+		//  type: bool
+		//  defaultdesc: `true`
+		//  shortdesc: Whether to use compression while migrating storage pools
+		"rsync.compression": validate.Optional(validate.IsBool),
 	}
 
 	// Add to pool config rules (prefixed with volume.*) which are common for pool and volume.
@@ -400,15 +655,17 @@ func validateVolumeCommonRules(vol drivers.Volume) map[string]func(string) error
 // ImageUnpack unpacks a filesystem image into the destination path.
 // There are several formats that images can come in:
 // Container Format A: Separate metadata tarball and root squashfs file.
-// 	- Unpack metadata tarball into mountPath.
-//	- Unpack root squashfs file into mountPath/rootfs.
+//   - Unpack metadata tarball into mountPath.
+//   - Unpack root squashfs file into mountPath/rootfs.
+//
 // Container Format B: Combined tarball containing metadata files and root squashfs.
-//	- Unpack combined tarball into mountPath.
+//   - Unpack combined tarball into mountPath.
+//
 // VM Format A: Separate metadata tarball and root qcow2 file.
-// 	- Unpack metadata tarball into mountPath.
-//	- Check rootBlockPath is a file and convert qcow2 file into raw format in rootBlockPath.
-func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blockBackend bool, sysOS *sys.OS, allowUnsafeResize bool, tracker *ioprogress.ProgressTracker) (int64, error) {
-	l := logger.AddContext(logger.Log, logger.Ctx{"imageFile": imageFile, "volName": vol.Name()})
+//   - Unpack metadata tarball into mountPath.
+//   - Check rootBlockPath is a file and convert qcow2 file into raw format in rootBlockPath.
+func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sysOS *sys.OS, allowUnsafeResize bool, tracker *ioprogress.ProgressTracker) (int64, error) {
+	l := logger.Log.AddContext(logger.Ctx{"imageFile": imageFile, "volName": vol.Name()})
 	l.Info("Image unpack started")
 	defer l.Info("Image unpack stopped")
 
@@ -421,7 +678,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 		rootfsPath := filepath.Join(destPath, "rootfs")
 
 		// Unpack the main image file.
-		err := archive.Unpack(imageFile, destPath, blockBackend, sysOS, tracker)
+		err := archive.Unpack(imageFile, destPath, vol.IsBlockBacked(), sysOS, tracker)
 		if err != nil {
 			return -1, err
 		}
@@ -433,7 +690,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 				return -1, fmt.Errorf("Error creating rootfs directory")
 			}
 
-			err = archive.Unpack(imageRootfsFile, rootfsPath, blockBackend, sysOS, tracker)
+			err = archive.Unpack(imageRootfsFile, rootfsPath, vol.IsBlockBacked(), sysOS, tracker)
 			if err != nil {
 				return -1, err
 			}
@@ -467,10 +724,10 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 		// Get info about qcow2 file. Force input format to qcow2 so we don't rely on qemu-img's detection
 		// logic as that has been known to have vulnerabilities and we only support qcow2 images anyway.
 		// Use prlimit because qemu-img can consume considerable RAM & CPU time if fed a maliciously
-		// crafted disk image. Since cloud tenants are not to be trusted, ensure QEMU is limits to 1 GB
+		// crafted disk image. Since cloud tenants are not to be trusted, ensure QEMU is limits to 1 GiB
 		// address space and 2 seconds CPU time, which ought to be more than enough for real world images.
-		cmd := []string{"prlimit", "--cpu=2", "--as=1000000000", "qemu-img", "info", "-f", "qcow2", "--output=json", imgPath}
-		imgJSON, err := apparmor.QemuImg(sysOS, cmd, imgPath, "")
+		cmd := []string{"prlimit", "--cpu=2", "--as=1073741824", "qemu-img", "info", "-f", "qcow2", "--output=json", imgPath}
+		imgJSON, err := apparmor.QemuImg(sysOS, cmd, imgPath, dstPath)
 		if err != nil {
 			return -1, fmt.Errorf("Failed reading image info %q: %w", imgPath, err)
 		}
@@ -482,7 +739,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 
 		err = json.Unmarshal([]byte(imgJSON), &imgInfo)
 		if err != nil {
-			return -1, err
+			return -1, fmt.Errorf("Failed unmarshalling image info %q: %w (%q)", imgPath, err, imgJSON)
 		}
 
 		// Belt and braces qcow2 check.
@@ -562,7 +819,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 
 	if shared.PathExists(imageRootfsFile) {
 		// Unpack the main image file.
-		err := archive.Unpack(imageFile, destPath, blockBackend, sysOS, tracker)
+		err := archive.Unpack(imageFile, destPath, vol.IsBlockBacked(), sysOS, tracker)
 		if err != nil {
 			return -1, err
 		}
@@ -582,7 +839,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 		defer func() { _ = os.RemoveAll(tempDir) }()
 
 		// Unpack the whole image.
-		err = archive.Unpack(imageFile, tempDir, blockBackend, sysOS, tracker)
+		err = archive.Unpack(imageFile, tempDir, vol.IsBlockBacked(), sysOS, tracker)
 		if err != nil {
 			return -1, err
 		}
@@ -733,59 +990,61 @@ func VolumeUsedByInstanceDevices(s *state.State, poolName string, projectName st
 		return err
 	}
 
-	return s.DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
-		// If the volume has a specific cluster member which is different than the instance then skip as
-		// instance cannot be using this volume.
-		if vol.Location != "" && inst.Node != vol.Location {
-			return nil
-		}
-
-		instStorageProject := project.StorageVolumeProjectFromRecord(&p, volumeType)
-		if err != nil {
-			return err
-		}
-
-		// Check instance's storage project is the same as the volume's project.
-		// If not then the volume names mentioned in the instance's config cannot be referring to volumes
-		// in the volume's project we are trying to match, and this instance cannot possibly be using it.
-		if projectName != instStorageProject {
-			return nil
-		}
-
-		// Use local devices for usage check by if expandDevices is false (but don't modify instance).
-		devices := inst.Devices
-
-		// Expand devices for usage check if expandDevices is true.
-		if expandDevices {
-			devices = db.ExpandInstanceDevices(devices.Clone(), inst.Profiles)
-		}
-
-		var usedByDevices []string
-
-		// Iterate through each of the instance's devices, looking for disks in the same pool as volume.
-		// Then try and match the volume name against the instance device's "source" property.
-		for devName, dev := range devices {
-			if dev["type"] != "disk" {
-				continue
+	return s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.InstanceList(ctx, func(inst db.InstanceArgs, p api.Project) error {
+			// If the volume has a specific cluster member which is different than the instance then skip as
+			// instance cannot be using this volume.
+			if vol.Location != "" && inst.Node != vol.Location {
+				return nil
 			}
 
-			if dev["pool"] != poolName {
-				continue
-			}
-
-			if dev["source"] == vol.Name {
-				usedByDevices = append(usedByDevices, devName)
-			}
-		}
-
-		if len(usedByDevices) > 0 {
-			err = instanceFunc(inst, p, usedByDevices)
+			instStorageProject := project.StorageVolumeProjectFromRecord(&p, volumeType)
 			if err != nil {
 				return err
 			}
-		}
 
-		return nil
+			// Check instance's storage project is the same as the volume's project.
+			// If not then the volume names mentioned in the instance's config cannot be referring to volumes
+			// in the volume's project we are trying to match, and this instance cannot possibly be using it.
+			if projectName != instStorageProject {
+				return nil
+			}
+
+			// Use local devices for usage check by if expandDevices is false (but don't modify instance).
+			devices := inst.Devices
+
+			// Expand devices for usage check if expandDevices is true.
+			if expandDevices {
+				devices = instancetype.ExpandInstanceDevices(devices.Clone(), inst.Profiles)
+			}
+
+			var usedByDevices []string
+
+			// Iterate through each of the instance's devices, looking for disks in the same pool as volume.
+			// Then try and match the volume name against the instance device's "source" property.
+			for devName, dev := range devices {
+				if dev["type"] != "disk" {
+					continue
+				}
+
+				if dev["pool"] != poolName {
+					continue
+				}
+
+				if dev["source"] == vol.Name {
+					usedByDevices = append(usedByDevices, devName)
+				}
+			}
+
+			if len(usedByDevices) > 0 {
+				err = instanceFunc(inst, p, usedByDevices)
+				if err != nil {
+					return err
+				}
+			}
+
+			return nil
+		})
 	})
 }
 
@@ -850,7 +1109,7 @@ func VolumeUsedByDaemon(s *state.State, poolName string, volumeName string) (boo
 
 // FallbackMigrationType returns the fallback migration transport to use based on volume content type.
 func FallbackMigrationType(contentType drivers.ContentType) migration.MigrationFSType {
-	if contentType == drivers.ContentTypeBlock {
+	if drivers.IsContentBlock(contentType) {
 		return migration.MigrationFSType_BLOCK_AND_RSYNC
 	}
 
@@ -871,7 +1130,10 @@ func RenderSnapshotUsage(s *state.State, snapInst instance.Instance) func(respon
 			// It is important that the snapshot not be mounted here as mounting a snapshot can trigger a very
 			// expensive filesystem UUID regeneration, so we rely on the driver implementation to get the info
 			// we are requesting as cheaply as possible.
-			apiRes.Size, _ = pool.GetInstanceUsage(snapInst)
+			volumeState, err := pool.GetInstanceUsage(snapInst)
+			if err == nil {
+				apiRes.Size = volumeState.Used
+			}
 		}
 
 		return nil
