@@ -10,11 +10,11 @@ import (
 
 	"github.com/syndtr/gocapability/capability"
 
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/warningtype"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/warningtype"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // Initialize AppArmor-specific attributes.
@@ -23,7 +23,7 @@ func (s *OS) initAppArmor() []cluster.Warning {
 
 	/* Detect AppArmor availability */
 	_, err := exec.LookPath("apparmor_parser")
-	if os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
+	if shared.IsFalse(os.Getenv("LXD_SECURITY_APPARMOR")) {
 		logger.Warnf("AppArmor support has been manually disabled")
 		dbWarnings = append(dbWarnings, cluster.Warning{
 			TypeCode:    warningtype.AppArmorNotAvailable,
@@ -71,9 +71,16 @@ func (s *OS) initAppArmor() []cluster.Warning {
 
 	/* Detect AppArmor confinment */
 	profile := util.AppArmorProfile()
-	if profile != "unconfined" && profile != "" {
+	/* if AppArmor is enabled on the system but there is no profile then
+	   the application has the profile name "unconfined", or if AppArmor
+	   is not supported then the label will be empty and finally newer
+	   AppArmor releases support a profile mode "unconfined" where an
+	   application can have a profile defined for it which effectively
+	   is equivalent to the traditional "unconfined" label - in this case
+	   the label will have the suffix " (unconfined)" */
+	if profile != "unconfined" && profile != "" && !strings.HasSuffix(profile, "(unconfined)") {
 		if s.AppArmorAvailable {
-			logger.Warnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor")
+			logger.Warnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor via profile %q", profile)
 		}
 
 		s.AppArmorConfined = true
