@@ -11,17 +11,15 @@ import (
 	"sync"
 	"time"
 
-	"github.com/mdlayher/vsock"
-
-	"github.com/lxc/lxd/lxd/cgroup"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/cgroup"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/idmap"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // InotifyTargetInfo records the inotify information associated with a given
@@ -83,7 +81,6 @@ type OS struct {
 	PidFdSetns              bool
 	SeccompListener         bool
 	SeccompListenerContinue bool
-	Shiftfs                 bool
 	UeventInjection         bool
 	VFS3Fscaps              bool
 
@@ -95,9 +92,6 @@ type OS struct {
 	// LXC features
 	LXCFeatures map[string]bool
 
-	// VM features
-	VsockID uint32
-
 	// OS info
 	ReleaseInfo   map[string]string
 	KernelVersion version.DottedVersion
@@ -181,18 +175,6 @@ func (s *OS) Init() ([]cluster.Warning, error) {
 	cgroup.Init()
 	s.CGInfo = cgroup.GetInfo()
 
-	// Fill in the VsockID.
-	_ = util.LoadModule("vhost_vsock")
-
-	vsockID, err := vsock.ContextID()
-	if err != nil || vsockID > 2147483647 {
-		// Fallback to the default ID for a host system if we're getting
-		// an error or are getting a clearly invalid value.
-		vsockID = 2
-	}
-
-	s.VsockID = vsockID
-
 	// Fill in the OS release info.
 	osInfo, err := osarch.GetLSBRelease()
 	if err != nil {
@@ -208,7 +190,7 @@ func (s *OS) Init() ([]cluster.Warning, error) {
 
 	s.Uname = uname
 
-	kernelVersion, err := version.Parse(strings.Split(uname.Release, "-")[0])
+	kernelVersion, err := version.Parse(uname.Release)
 	if err == nil {
 		s.KernelVersion = *kernelVersion
 	}
