@@ -1,54 +1,75 @@
 package task
 
 import (
+	"context"
 	"fmt"
 	"strconv"
-	"strings"
 	"sync"
 	"time"
-
-	"golang.org/x/net/context"
 )
 
 // Group of tasks sharing the same lifecycle.
 //
 // All tasks in a group will be started and stopped at the same time.
 type Group struct {
-	cancel  func()
+	cancel  context.CancelFunc
 	wg      sync.WaitGroup
 	tasks   []Task
 	running map[int]bool
 	mu      sync.Mutex
 }
 
+// NewGroup returns new initialised Group.
+func NewGroup() *Group {
+	return &Group{
+		running: make(map[int]bool),
+	}
+}
+
 // Add a new task to the group, returning its index.
 func (g *Group) Add(f Func, schedule Schedule) *Task {
-	i := len(g.tasks)
+	g.mu.Lock()
 	g.tasks = append(g.tasks, Task{
 		f:        f,
 		schedule: schedule,
 		reset:    make(chan struct{}, 16), // Buffered to not block senders
 	})
-	return &g.tasks[i]
+	t := &g.tasks[len(g.tasks)-1] // Get the task we added to g.tasks.
+	g.mu.Unlock()
+
+	return t
 }
 
 // Start all the tasks in the group.
-func (g *Group) Start() {
-	ctx := context.Background()
+func (g *Group) Start(ctx context.Context) {
+	// Lock access to the g.running and g.tasks map for the entirety of this function so that
+	// concurrent calls to Start() or Add(0) don't race. This ensures all tasks in this group
+	// are started based on a consistent snapshot of g.running and g.tasks.
+	g.mu.Lock()
+
 	ctx, g.cancel = context.WithCancel(ctx)
 	g.wg.Add(len(g.tasks))
-	g.running = make(map[int]bool)
+
 	for i := range g.tasks {
-		task := g.tasks[i] // Local variable for the closure below.
+		if g.running[i] {
+			continue
+		}
+
 		g.running[i] = true
+		task := g.tasks[i] // Local variable for the closure below.
+
 		go func(i int) {
 			task.loop(ctx)
-			g.wg.Done()
+
+			// Ensure running map is updated before wait group Done() is called.
 			g.mu.Lock()
-			defer g.mu.Unlock()
 			g.running[i] = false
+			g.mu.Unlock()
+			g.wg.Done()
 		}(i)
 	}
+
+	g.mu.Unlock()
 }
 
 // Stop all tasks in the group.
@@ -70,6 +91,7 @@ func (g *Group) Stop(timeout time.Duration) error {
 		// We were not even started
 		return nil
 	}
+
 	g.cancel()
 
 	graceful := make(chan struct{}, 1)
@@ -83,15 +105,17 @@ func (g *Group) Stop(timeout time.Duration) error {
 	defer cancel()
 	select {
 	case <-ctx.Done():
+		g.mu.Lock()
 		running := []string{}
 		for i, value := range g.running {
 			if value {
 				running = append(running, strconv.Itoa(i))
 			}
 		}
-		return fmt.Errorf("tasks %s are still running", strings.Join(running, ", "))
+		g.mu.Unlock()
+
+		return fmt.Errorf("Task(s) still running: IDs %v", running)
 	case <-graceful:
 		return nil
-
 	}
 }
