@@ -1,9 +1,9 @@
 package util
 
 import (
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // PoolType represents a type of storage pool (local, remote or any).
@@ -37,7 +37,7 @@ func AvailableStorageDrivers(supportedDrivers []api.ServerStorageDriverInfo, poo
 			continue
 		}
 
-		if poolType == PoolTypeAny && driver.Name == "cephfs" {
+		if poolType == PoolTypeAny && (driver.Name == "cephfs" || driver.Name == "cephobject") {
 			continue
 		}
 
