@@ -12,9 +12,9 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/osarch"
 )
 
 var urlDefaultOS = map[string]string{
@@ -138,6 +138,10 @@ func (s *SimpleStreams) cachedDownload(path string) ([]byte, error) {
 		return nil, err
 	}
 
+	if len(body) == 0 {
+		return nil, fmt.Errorf("No content in download from %q", uri)
+	}
+
 	// Attempt to store in cache
 	if s.cachePath != "" {
 		cacheName := filepath.Join(s.cachePath, fileName)
@@ -159,11 +163,13 @@ func (s *SimpleStreams) parseStream() (*Stream, error) {
 		return nil, err
 	}
 
+	pathURL, _ := shared.JoinUrls(s.url, path)
+
 	// Parse the idnex
 	stream := Stream{}
 	err = json.Unmarshal(body, &stream)
 	if err != nil {
-		return nil, fmt.Errorf("Failed decoding stream JSON from %q: %w", path, err)
+		return nil, fmt.Errorf("Failed decoding stream JSON from %q: %w (%q)", pathURL, err, string(body))
 	}
 
 	s.cachedStream = &stream
@@ -351,6 +357,10 @@ func (s *SimpleStreams) GetFiles(fingerprint string) (map[string]DownloadableFil
 				files := map[string]DownloadableFile{}
 
 				for _, path := range downloads[image.Fingerprint] {
+					if len(path) < 4 {
+						return nil, fmt.Errorf("Invalid path content: %q", path)
+					}
+
 					size, err := strconv.ParseInt(path[3], 10, 64)
 					if err != nil {
 						return nil, err
@@ -367,7 +377,7 @@ func (s *SimpleStreams) GetFiles(fingerprint string) (map[string]DownloadableFil
 		}
 	}
 
-	return nil, fmt.Errorf("Couldn't find the requested image")
+	return nil, fmt.Errorf("Couldn't find the requested image for fingerprint %q", fingerprint)
 }
 
 // ListAliases returns a list of image aliases for the provided image fingerprint.
@@ -491,9 +501,9 @@ func (s *SimpleStreams) GetImage(fingerprint string) (*api.Image, error) {
 	}
 
 	if len(matches) == 0 {
-		return nil, fmt.Errorf("The requested image couldn't be found")
+		return nil, fmt.Errorf("The requested image couldn't be found for fingerprint %q", fingerprint)
 	} else if len(matches) > 1 {
-		return nil, fmt.Errorf("More than one match for the provided partial fingerprint")
+		return nil, fmt.Errorf("More than one match for the provided partial fingerprint %q", fingerprint)
 	}
 
 	return &matches[0], nil
