@@ -27,11 +27,11 @@ import (
 
 	"github.com/flosch/pongo2"
 
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/cancel"
-	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/cancel"
+	"github.com/canonical/lxd/shared/ioprogress"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
 )
 
 const SnapshotDelimiter = "/"
@@ -39,6 +39,9 @@ const HTTPSDefaultPort = 8443
 const HTTPDefaultPort = 8080
 const HTTPSMetricsDefaultPort = 9100
 
+// HTTPSStorageBucketsDefaultPort the default port for the storage buckets listener.
+const HTTPSStorageBucketsDefaultPort = 9000
+
 // URLEncode encodes a path and query parameters to a URL.
 func URLEncode(path string, query map[string]string) (string, error) {
 	u, err := url.Parse(path)
@@ -85,7 +88,7 @@ func PathIsEmpty(path string) (bool, error) {
 	defer func() { _ = f.Close() }()
 
 	// read in ONLY one file
-	_, err = f.Readdir(1)
+	_, err = f.ReadDir(1)
 
 	// and if the file is EOF... well, the dir is empty.
 	if err == io.EOF {
@@ -254,36 +257,78 @@ func LogPath(path ...string) string {
 	return filepath.Join(items...)
 }
 
-func ParseLXDFileHeaders(headers http.Header) (uid int64, gid int64, mode int, type_ string, write string) {
-	uid, err := strconv.ParseInt(headers.Get("X-LXD-uid"), 10, 64)
-	if err != nil {
-		uid = -1
-	}
+// LXDFileHeaders is extracted from the `X-LXD-*` family of file permissions
+// headers.
+type LXDFileHeaders struct {
+	UID  int64
+	GID  int64
+	Mode int
 
-	gid, err = strconv.ParseInt(headers.Get("X-LXD-gid"), 10, 64)
-	if err != nil {
-		gid = -1
-	}
+	GIDModifyExisting  bool
+	UIDModifyExisting  bool
+	ModeModifyExisting bool
 
-	mode, err = strconv.Atoi(headers.Get("X-LXD-mode"))
-	if err != nil {
-		mode = -1
-	} else {
-		rawMode, err := strconv.ParseInt(headers.Get("X-LXD-mode"), 0, 0)
-		if err == nil {
-			mode = int(os.FileMode(rawMode) & os.ModePerm)
+	Type  string
+	Write string
+}
+
+// ParseLXDFileHeaders parses and validates the `X-LXD-*` family of file
+// permissions headers.
+//   - `X-LXD-uid`, `X-LXD-gid`
+//     Base 10 integer
+//   - `X-LXD-mode`
+//     Base 10 integer (no leading `0`) or base 8 integer (leading `0`) for the
+//     unix permissions bits
+//   - `X-LXD-type`
+//     One of `file`, `symlink`, `directory`
+//   - `X-LXD-write`
+//     One of `overwrite`, `append`
+//   - `X-LXD-modify-perm`
+//     Comma separated list; 0 or more of `mode`, `uid`, `gid`
+func ParseLXDFileHeaders(headers http.Header) (*LXDFileHeaders, error) {
+	var uid, gid int64 = -1, -1
+	var mode = -1
+	var err error
+
+	rawUID := headers.Get("X-LXD-uid")
+	if rawUID != "" {
+		uid, err = strconv.ParseInt(rawUID, 10, 32)
+		if err != nil {
+			return nil, fmt.Errorf("Invalid UID: %w", err)
 		}
 	}
 
-	type_ = headers.Get("X-LXD-type")
+	rawGID := headers.Get("X-LXD-gid")
+	if rawGID != "" {
+		gid, err = strconv.ParseInt(rawGID, 10, 32)
+		if err != nil {
+			return nil, fmt.Errorf("Invalid GID: %w", err)
+		}
+	}
+
+	rawMode := headers.Get("X-LXD-mode")
+	if rawMode != "" {
+		mode64, err := strconv.ParseInt(rawMode, 0, 0)
+		if err != nil || mode64 < 0 {
+			return nil, fmt.Errorf("Invalid Mode: %w", err)
+		}
+
+		mode = int(mode64 & int64(os.ModePerm))
+	}
+
+	filetype := headers.Get("X-LXD-type")
 	/* backwards compat: before "type" was introduced, we could only
 	 * manipulate files
 	 */
-	if type_ == "" {
-		type_ = "file"
+	if filetype == "" {
+		filetype = "file"
 	}
 
-	write = headers.Get("X-LXD-write")
+	if !ValueInSlice(filetype, []string{"file", "symlink", "directory"}) {
+		return nil, fmt.Errorf("Invalid file type: %q", filetype)
+	}
+
+	write := headers.Get("X-LXD-write")
 	/* backwards compat: before "write" was introduced, we could only
 	 * overwrite files
 	 */
@@ -291,7 +336,40 @@ func ParseLXDFileHeaders(headers http.Header) (uid int64, gid int64, mode int, t
 		write = "overwrite"
 	}
 
-	return uid, gid, mode, type_, write
+	if !ValueInSlice(write, []string{"overwrite", "append"}) {
+		return nil, fmt.Errorf("Invalid file write mode: %q", write)
+	}
+
+	UIDModifyExisting := false
+	GIDModifyExisting := false
+	modeModifyExisting := false
+
+	modifyPermHeader := headers.Get("X-LXD-modify-perm")
+
+	if modifyPermHeader != "" {
+		for _, perm := range strings.Split(modifyPermHeader, ",") {
+			UIDModifyExisting = UIDModifyExisting || perm == "uid"
+			GIDModifyExisting = GIDModifyExisting || perm == "gid"
+			modeModifyExisting = modeModifyExisting || perm == "mode"
+
+			if !ValueInSlice(perm, []string{"uid", "gid", "mode"}) {
+				return nil, fmt.Errorf("Invalid modify-perm field: %q", perm)
+			}
+		}
+	}
+
+	return &LXDFileHeaders{
+		UID:  uid,
+		GID:  gid,
+		Mode: mode,
+
+		UIDModifyExisting:  UIDModifyExisting,
+		GIDModifyExisting:  GIDModifyExisting,
+		ModeModifyExisting: modeModifyExisting,
+
+		Type:  filetype,
+		Write: write,
+	}, nil
 }
 
 func ReaderToChannel(r io.Reader, bufferSize int) <-chan []byte {
@@ -326,7 +404,7 @@ func ReaderToChannel(r io.Reader, bufferSize int) <-chan []byte {
 	return ch
 }
 
-// Returns a random base64 encoded string from crypto/rand.
+// Returns a random hex encoded string from crypto/rand.
 func RandomCryptoString() (string, error) {
 	buf := make([]byte, 32)
 	n, err := rand.Read(buf)
@@ -622,18 +700,31 @@ func HasKey[K comparable, V any](key K, m map[K]V) bool {
 	return found
 }
 
-func StringInSlice(key string, list []string) bool {
+// ValueInSlice returns true if key is in list.
+func ValueInSlice[T comparable](key T, list []T) bool {
 	for _, entry := range list {
 		if entry == key {
 			return true
 		}
 	}
+
 	return false
 }
 
-// RemoveElementsFromStringSlice returns a slice equivalent to removing the given elements from the given list.
+// StringPrefixInSlice returns true if any element in the list has the given prefix.
+func StringPrefixInSlice(key string, list []string) bool {
+	for _, entry := range list {
+		if strings.HasPrefix(entry, key) {
+			return true
+		}
+	}
+
+	return false
+}
+
+// RemoveElementsFromSlice returns a slice equivalent to removing the given elements from the given list.
 // Elements not present in the list are ignored.
-func RemoveElementsFromStringSlice(list []string, elements ...string) []string {
+func RemoveElementsFromSlice[T comparable](list []T, elements ...T) []T {
 	for i := len(elements) - 1; i >= 0; i-- {
 		element := elements[i]
 		match := false
@@ -663,36 +754,9 @@ func StringHasPrefix(value string, prefixes ...string) bool {
 	return false
 }
 
-func IntInSlice(key int, list []int) bool {
-	for _, entry := range list {
-		if entry == key {
-			return true
-		}
-	}
-	return false
-}
-
-func Int64InSlice(key int64, list []int64) bool {
-	for _, entry := range list {
-		if entry == key {
-			return true
-		}
-	}
-	return false
-}
-
-func Uint64InSlice(key uint64, list []uint64) bool {
-	for _, entry := range list {
-		if entry == key {
-			return true
-		}
-	}
-	return false
-}
-
 // IsTrue returns true if value is "true", "1", "yes" or "on" (case insensitive).
 func IsTrue(value string) bool {
-	return StringInSlice(strings.ToLower(value), []string{"true", "1", "yes", "on"})
+	return ValueInSlice(strings.ToLower(value), []string{"true", "1", "yes", "on"})
 }
 
 // IsTrueOrEmpty returns true if value is empty or if IsTrue() returns true.
@@ -702,7 +766,7 @@ func IsTrueOrEmpty(value string) bool {
 
 // IsFalse returns true if value is "false", "0", "no" or "off" (case insensitive).
 func IsFalse(value string) bool {
-	return StringInSlice(strings.ToLower(value), []string{"false", "0", "no", "off"})
+	return ValueInSlice(strings.ToLower(value), []string{"false", "0", "no", "off"})
 }
 
 // IsFalseOrEmpty returns true if value is empty or if IsFalse() returns true.
@@ -996,11 +1060,11 @@ func RunCommandInheritFds(ctx context.Context, filesInherit []*os.File, name str
 	return stdout, err
 }
 
-// RunCommandCLocale runs a command with a LANG=C.UTF-8 environment set with optional arguments and
+// RunCommandCLocale runs a command with a LC_ALL=C.UTF-8 and LANGUAGE=en environment set with optional arguments and
 // returns stdout. If the command fails to start or returns a non-zero exit code then an error is
 // returned containing the output of stderr.
 func RunCommandCLocale(name string, arg ...string) (string, error) {
-	stdout, _, err := RunCommandSplit(context.TODO(), append(os.Environ(), "LANG=C.UTF-8"), nil, name, arg...)
+	stdout, _, err := RunCommandSplit(context.TODO(), append(os.Environ(), "LC_ALL=C.UTF-8", "LANGUAGE=en"), nil, name, arg...)
 	return stdout, err
 }
 
@@ -1096,7 +1160,7 @@ func SetProgressMetadata(metadata map[string]any, stage, displayPrefix string, p
 
 func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.HTTPRequestCanceller, filename string, url string, hash string, hashFunc hash.Hash, target io.WriteSeeker) (int64, error) {
 	// Always seek to the beginning
-	_, _ = target.Seek(0, 0)
+	_, _ = target.Seek(0, io.SeekStart)
 
 	var req *http.Request
 	var err error
@@ -1117,7 +1181,7 @@ func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent st
 	}
 
 	// Perform the request
-	r, doneCh, err := cancel.CancelableDownload(canceler, httpClient, req)
+	r, doneCh, err := cancel.CancelableDownload(canceler, httpClient.Do, req)
 	if err != nil {
 		return -1, err
 	}
@@ -1241,7 +1305,11 @@ func GetExpiry(refDate time.Time, s string) (time.Time, error) {
 		return time.Time{}, nil
 	}
 
-	re := regexp.MustCompile(`^(\d+)(S|M|H|d|w|m|y)$`)
+	re, err := regexp.Compile(`^(\d+)(S|M|H|d|w|m|y)$`)
+	if err != nil {
+		return time.Time{}, err
+	}
+
 	expiry := map[string]int{
 		"S": 0,
 		"M": 0,
@@ -1322,7 +1390,7 @@ func SplitNTrimSpace(s string, sep string, n int, nilIfEmpty bool) []string {
 	return parts
 }
 
-// JoinTokenDecode decodes a base64 and JSON encode join token.
+// JoinTokenDecode decodes a base64 and JSON encoded join token.
 func JoinTokenDecode(input string) (*api.ClusterMemberJoinToken, error) {
 	joinTokenJSON, err := base64.StdEncoding.DecodeString(input)
 	if err != nil {
@@ -1353,3 +1421,45 @@ func JoinTokenDecode(input string) (*api.ClusterMemberJoinToken, error) {
 
 	return &j, nil
 }
+
+// TargetDetect returns either target node or group based on the provided prefix:
+// An invocation with `target=h1` returns "h1", "" and `target=@g1` returns "", "g1".
+func TargetDetect(target string) (targetNode string, targetGroup string) {
+	if strings.HasPrefix(target, "@") {
+		targetGroup = strings.TrimPrefix(target, "@")
+	} else {
+		targetNode = target
+	}
+
+	return
+}
+
+// ApplyDeviceOverrides handles the logic for applying device overrides.
+// Receives the profile and local devices and the device overrides.
+// Returns the resulting list of devices.
+func ApplyDeviceOverrides(localDevices map[string]map[string]string, profileDevices map[string]map[string]string, deviceOverrides map[string]map[string]string) (map[string]map[string]string, error) {
+	// Allow setting device overrides.
+	for deviceName := range deviceOverrides {
+		_, isLocalDevice := localDevices[deviceName]
+		if isLocalDevice {
+			// Apply overrides to local device.
+			for k, v := range deviceOverrides[deviceName] {
+				localDevices[deviceName][k] = v
+			}
+		} else {
+			// Check device exists in expanded profile devices.
+			profileDeviceConfig, found := profileDevices[deviceName]
+			if !found {
+				return nil, fmt.Errorf("Cannot override config for device %q: Device not found in profile devices", deviceName)
+			}
+
+			for k, v := range deviceOverrides[deviceName] {
+				profileDeviceConfig[k] = v
+			}
+
+			localDevices[deviceName] = profileDeviceConfig
+		}
+	}
+
+	return localDevices, nil
+}
