@@ -13,13 +13,13 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/google/uuid"
 	"github.com/kballard/go-shellquote"
-	"github.com/pborman/uuid"
 	"github.com/robfig/cron/v3"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/shared/osarch"
-	"github.com/lxc/lxd/shared/units"
+	"github.com/canonical/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/units"
 )
 
 // stringInSlice checks whether the supplied string is present in the supplied slice.
@@ -217,12 +217,8 @@ func IsSize(value string) error {
 
 // IsDeviceID validates string is four lowercase hex characters suitable as Vendor or Device ID.
 func IsDeviceID(value string) error {
-	regexHexLc, err := regexp.Compile("^[0-9a-f]+$")
-	if err != nil {
-		return err
-	}
-
-	if len(value) != 4 || !regexHexLc.MatchString(value) {
+	match, _ := regexp.MatchString(`^[0-9a-f]{4}$`, value)
+	if !match {
 		return fmt.Errorf("Invalid value, must be four lower case hex characters")
 	}
 
@@ -241,7 +237,7 @@ func IsInterfaceName(value string) error {
 	}
 
 	// Validate the character set.
-	match, _ := regexp.MatchString("^[-_a-zA-Z0-9.]+$", value)
+	match, _ := regexp.MatchString(`^[-_a-zA-Z0-9.]+$`, value)
 	if !match {
 		return fmt.Errorf("Network interface contains invalid characters")
 	}
@@ -285,7 +281,7 @@ func IsNetwork(value string) error {
 	return nil
 }
 
-// IsNetworkAddressCIDR validates an IP addresss string in CIDR format.
+// IsNetworkAddressCIDR validates an IP address string in CIDR format.
 func IsNetworkAddressCIDR(value string) error {
 	_, _, err := net.ParseCIDR(value)
 	if err != nil {
@@ -341,7 +337,7 @@ func IsNetworkV4(value string) error {
 	return nil
 }
 
-// IsNetworkAddressV4 validates an IPv4 addresss string.
+// IsNetworkAddressV4 validates an IPv4 address string.
 func IsNetworkAddressV4(value string) error {
 	ip := net.ParseIP(value)
 	if ip == nil || ip.To4() == nil {
@@ -351,7 +347,7 @@ func IsNetworkAddressV4(value string) error {
 	return nil
 }
 
-// IsNetworkAddressCIDRV4 validates an IPv4 addresss string in CIDR format.
+// IsNetworkAddressCIDRV4 validates an IPv4 address string in CIDR format.
 func IsNetworkAddressCIDRV4(value string) error {
 	ip, subnet, err := net.ParseCIDR(value)
 	if err != nil {
@@ -404,7 +400,7 @@ func IsNetworkV6(value string) error {
 	return nil
 }
 
-// IsNetworkAddressV6 validates an IPv6 addresss string.
+// IsNetworkAddressV6 validates an IPv6 address string.
 func IsNetworkAddressV6(value string) error {
 	ip := net.ParseIP(value)
 	if ip == nil || ip.To4() != nil {
@@ -414,7 +410,7 @@ func IsNetworkAddressV6(value string) error {
 	return nil
 }
 
-// IsNetworkAddressCIDRV6 validates an IPv6 addresss string in CIDR format.
+// IsNetworkAddressCIDRV6 validates an IPv6 address string in CIDR format.
 func IsNetworkAddressCIDRV6(value string) error {
 	ip, subnet, err := net.ParseCIDR(value)
 	if err != nil {
@@ -533,7 +529,8 @@ func IsURLSegmentSafe(value string) error {
 
 // IsUUID validates whether a value is a UUID.
 func IsUUID(value string) error {
-	if uuid.Parse(value) == nil {
+	_, err := uuid.Parse(value)
+	if err != nil {
 		return fmt.Errorf("Invalid UUID")
 	}
 
@@ -542,12 +539,8 @@ func IsUUID(value string) error {
 
 // IsPCIAddress validates whether a value is a PCI address.
 func IsPCIAddress(value string) error {
-	regexHex, err := regexp.Compile(`^([0-9a-fA-F]{4}?:)?[0-9a-fA-F]{2}:[0-9a-fA-F]{2}\.[0-9a-fA-F]$`)
-	if err != nil {
-		return err
-	}
-
-	if !regexHex.MatchString(value) {
+	match, _ := regexp.MatchString(`^(?:[0-9a-fA-F]{4}:)?[0-9a-fA-F]{2}:[0-9a-fA-F]{2}\.[0-9a-fA-F]$`, value)
+	if !match {
 		return fmt.Errorf("Invalid PCI address")
 	}
 
@@ -735,9 +728,9 @@ func IsHostname(name string) error {
 		return fmt.Errorf(`Name must not end with "-" character`)
 	}
 
-	_, err := strconv.Atoi(string(name[0]))
+	_, err := strconv.ParseUint(name, 10, 64)
 	if err == nil {
-		return fmt.Errorf("Name must not start with a number")
+		return fmt.Errorf("Name cannot be a number")
 	}
 
 	match, err := regexp.MatchString(`^[\-a-zA-Z0-9]+$`, name)
@@ -821,7 +814,7 @@ func IsYAML(value string) error {
 // IsValidCPUSet checks value is a valid CPU set.
 func IsValidCPUSet(value string) error {
 	// Validate the CPU set syntax.
-	match, _ := regexp.MatchString("^([0-9]+([,-][0-9]+)?)(,[0-9]+([,-][0-9]+)*)?$", value)
+	match, _ := regexp.MatchString(`^(?:[0-9]+(?:[,-][0-9]+)?)(?:,[0-9]+(?:[,-][0-9]+)*)?$`, value)
 	if !match {
 		return fmt.Errorf("Invalid CPU limit syntax")
 	}
@@ -870,3 +863,24 @@ func IsValidCPUSet(value string) error {
 
 	return nil
 }
+
+// IsMultipleOfUnit checks if value is in multiples of unit.
+func IsMultipleOfUnit(unit string) func(value string) error {
+	return func(value string) error {
+		bytes, err := units.ParseByteSizeString(value)
+		if err != nil {
+			return fmt.Errorf("Invalid value: %s", value)
+		}
+
+		unitBytes, err := units.ParseByteSizeString(unit)
+		if err != nil {
+			return fmt.Errorf("Invalid unit value: %s", unit)
+		}
+
+		if bytes%unitBytes != 0 {
+			return fmt.Errorf("Value not in multiples of %s: %s", unit, value)
+		}
+
+		return nil
+	}
+}
