@@ -8,7 +8,7 @@ import (
 	"golang.org/x/text/cases"
 	"golang.org/x/text/language"
 
-	"github.com/lxc/lxd/shared/osarch"
+	"github.com/canonical/lxd/shared/osarch"
 )
 
 // UserAgent contains a string suitable as a user-agent.
@@ -32,6 +32,9 @@ func getUserAgent() string {
 
 	// Initial version string
 	agent := fmt.Sprintf("LXD %s", Version)
+	if IsLTSVersion {
+		agent = fmt.Sprintf("%s LTS", agent)
+	}
 
 	// OS information
 	agent = fmt.Sprintf("%s (%s)", agent, strings.Join(osTokens, "; "))
