@@ -8,7 +8,6 @@ setup_clustering_bridge() {
   ip link add "${name}" up type bridge
   ip addr add 10.1.1.1/16 dev "${name}"
 
-  # shellcheck disable=SC2039
   iptables -w -t nat -A POSTROUTING -s 10.1.0.0/16 -d 0.0.0.0/0 -j MASQUERADE
   echo 1 > /proc/sys/net/ipv4/ip_forward
 }
@@ -110,7 +109,7 @@ teardown_clustering_netns() {
 }
 
 spawn_lxd_and_bootstrap_cluster() {
-  # shellcheck disable=2039,3043
+  # shellcheck disable=SC2039,SC3043
   local LXD_NETNS
 
   set -e
@@ -151,27 +150,29 @@ EOF
     if [ "${driver}" = "btrfs" ]; then
       cat >> "${LXD_DIR}/preseed.yaml" <<EOF
   config:
-    size: 100GB
+    size: 1GiB
 EOF
     fi
     if [ "${driver}" = "zfs" ]; then
       cat >> "${LXD_DIR}/preseed.yaml" <<EOF
   config:
-    size: 100GB
+    size: 1GiB
     zfs.pool_name: lxdtest-$(basename "${TEST_DIR}")-${ns}
 EOF
     fi
     if [ "${driver}" = "lvm" ]; then
       cat >> "${LXD_DIR}/preseed.yaml" <<EOF
   config:
-    volume.size: 25MB
+    volume.size: 25MiB
+    size: 1GiB
+    lvm.vg_name: lxdtest-$(basename "${TEST_DIR}")-${ns}
 EOF
     fi
     if [ "${driver}" = "ceph" ]; then
       cat >> "${LXD_DIR}/preseed.yaml" <<EOF
   config:
     source: lxdtest-$(basename "${TEST_DIR}")
-    volume.size: 25GB
+    volume.size: 25MiB
     ceph.osd.pg_num: 16
 EOF
     fi
@@ -198,7 +199,7 @@ EOF
 }
 
 spawn_lxd_and_join_cluster() {
-  # shellcheck disable=2039,2034,3043
+  # shellcheck disable=SC2039,SC3043
   local LXD_NETNS
 
   set -e
@@ -256,6 +257,10 @@ EOF
     name: data
     key: zfs.pool_name
     value: lxdtest-$(basename "${TEST_DIR}")-${ns}
+  - entity: storage-pool
+    name: data
+    key: size
+    value: 1GiB
 EOF
       fi
       if [ "${driver}" = "lvm" ]; then
@@ -264,6 +269,18 @@ EOF
     name: data
     key: lvm.vg_name
     value: lxdtest-$(basename "${TEST_DIR}")-${ns}
+  - entity: storage-pool
+    name: data
+    key: size
+    value: 1GiB
+EOF
+      fi
+      if [ "${driver}" = "btrfs" ]; then
+        cat >> "${LXD_DIR}/preseed.yaml" <<EOF
+  - entity: storage-pool
+    name: data
+    key: size
+    value: 1GiB
 EOF
       fi
     fi
@@ -273,7 +290,7 @@ EOF
 }
 
 respawn_lxd_cluster_member() {
-  # shellcheck disable=2039,2034,3043
+  # shellcheck disable=SC2039,SC2034,SC3043
   local LXD_NETNS
 
   set -e
