@@ -83,11 +83,25 @@ cleanup() {
     read -r _
   fi
 
-  echo "==> Cleaning up"
+  echo ""
+  echo "df -h output:"
+  df -h
 
-  umount -l "${TEST_DIR}/dev"
-  kill_external_auth_daemon "$TEST_DIR"
-  cleanup_lxds "$TEST_DIR"
+  if [ "${TEST_RESULT}" != "success" ]; then
+    # dmesg may contain oops, IO errors, crashes, etc
+    echo "::group::dmesg logs"
+    journalctl --quiet --no-hostname --no-pager --boot=0 --lines=100 --dmesg
+    echo "::endgroup::"
+  fi
+
+  if [ -n "${GITHUB_ACTIONS:-}" ]; then
+    echo "==> Skipping cleanup (GitHub Action runner detected)"
+  else
+    echo "==> Cleaning up"
+
+    umount -l "${TEST_DIR}/dev"
+    cleanup_lxds "$TEST_DIR"
+  fi
 
   echo ""
   echo ""
@@ -113,7 +127,7 @@ TEST_DIR=$(mktemp -d -p "$(pwd)" tmp.XXX)
 chmod +x "${TEST_DIR}"
 
 if [ -n "${LXD_TMPFS:-}" ]; then
-  mount -t tmpfs tmpfs "${TEST_DIR}" -o mode=0751
+  mount -t tmpfs tmpfs "${TEST_DIR}" -o mode=0751 -o size=6G
 fi
 
 mkdir -p "${TEST_DIR}/dev"
@@ -130,8 +144,6 @@ spawn_lxd "${LXD_DIR}" true
 LXD_ADDR=$(cat "${LXD_DIR}/lxd.addr")
 export LXD_ADDR
 
-start_external_auth_daemon "${LXD_DIR}"
-
 run_test() {
   TEST_CURRENT=${1}
   TEST_CURRENT_DESCRIPTION=${2:-${1}}
@@ -194,8 +206,11 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_database_no_disk_space "database out of disk space"
     run_test test_sql "lxd sql"
     run_test test_tls_restrictions "TLS restrictions"
+    run_test test_oidc "OpenID Connect"
+    run_test test_authorization "Authorization"
     run_test test_certificate_edit "Certificate edit"
     run_test test_basic_usage "basic usage"
+    run_test test_server_info "server info"
     run_test test_remote_url "remote url handling"
     run_test test_remote_admin "remote administration"
     run_test test_remote_usage "remote usage"
@@ -214,6 +229,7 @@ if [ "${1:-"all"}" != "standalone" ]; then
     run_test test_clustering_shutdown_nodes "clustering shutdown"
     run_test test_clustering_projects "clustering projects"
     run_test test_clustering_update_cert "clustering update cert"
+    run_test test_clustering_update_cert_reversion "clustering update cert reversion"
     run_test test_clustering_address "clustering address"
     run_test test_clustering_image_replication "clustering image replication"
     run_test test_clustering_dns "clustering DNS"
@@ -224,12 +240,15 @@ if [ "${1:-"all"}" != "standalone" ]; then
     run_test test_clustering_failure_domains "clustering failure domains"
     run_test test_clustering_image_refresh "clustering image refresh"
     run_test test_clustering_evacuation "clustering evacuation"
+    run_test test_clustering_instance_placement_scriptlet "clustering instance placement scriptlet"
+    run_test test_clustering_move "clustering move"
     run_test test_clustering_edit_configuration "clustering config edit"
     run_test test_clustering_remove_members "clustering config remove members"
     run_test test_clustering_autotarget "clustering autotarget member"
     # run_test test_clustering_upgrade "clustering upgrade"
     run_test test_clustering_groups "clustering groups"
     run_test test_clustering_events "clustering events"
+    run_test test_clustering_uuid "clustering uuid"
 fi
 
 if [ "${1:-"all"}" != "cluster" ]; then
@@ -266,6 +285,8 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_container_devices_unix_char "container devices - unix-char"
     run_test test_container_devices_unix_block "container devices - unix-block"
     run_test test_container_devices_tpm "container devices - tpm"
+    run_test test_container_move "container server-side move"
+    run_test test_container_syscall_interception "container syscall interception"
     run_test test_security "security features"
     run_test test_security_protection "container protection"
     run_test test_image_expiry "image expiry"
@@ -274,8 +295,10 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_image_prefer_cached "image prefer cached"
     run_test test_image_import_dir "import image from directory"
     run_test test_image_refresh "image refresh"
+    run_test test_image_acl "image acl"
     run_test test_cloud_init "cloud-init"
     run_test test_exec "exec"
+    run_test test_exec_exit_code "exec exit code"
     run_test test_concurrent_exec "concurrent exec"
     run_test test_concurrent "concurrent startup"
     run_test test_snapshots "container snapshots"
@@ -285,6 +308,7 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_snap_volume_db_recovery "snapshot volume database record recovery"
     run_test test_config_profiles "profiles and configuration"
     run_test test_config_edit "container configuration edit"
+    run_test test_property "container property"
     run_test test_config_edit_container_snapshot_pool_config "container and snapshot volume configuration edit"
     run_test test_container_metadata "manage container metadata and templates"
     run_test test_container_snapshot_config "container snapshot configuration"
@@ -300,6 +324,7 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_devlxd "/dev/lxd"
     run_test test_fuidshift "fuidshift"
     run_test test_migration "migration"
+    run_test test_lxc_to_lxd "LXC to LXD"
     run_test test_fdleak "fd leak"
     run_test test_storage "storage"
     run_test test_storage_volume_snapshots "storage volume snapshots"
@@ -308,13 +333,18 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_init_preseed "lxd init preseed"
     run_test test_storage_profiles "storage profiles"
     run_test test_container_recover "container recover"
+    run_test test_bucket_recover "bucket recover"
+    run_test test_get_operations "test_get_operations"
     run_test test_storage_volume_attach "attaching storage volumes"
     run_test test_storage_driver_btrfs "btrfs storage driver"
     run_test test_storage_driver_ceph "ceph storage driver"
     run_test test_storage_driver_cephfs "cephfs storage driver"
+    run_test test_storage_driver_zfs "zfs storage driver"
+    run_test test_storage_buckets "storage buckets"
+    run_test test_storage_volume_import "storage volume import"
+    run_test test_storage_volume_initial_config "storage volume initial configuration"
     run_test test_resources "resources"
     run_test test_kernel_limits "kernel limits"
-    run_test test_macaroon_auth "macaroon authentication"
     run_test test_console "console"
     run_test test_query "query"
     run_test test_storage_local_volume_handling "storage local volume handling"
@@ -322,7 +352,11 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_backup_export "backup export"
     run_test test_backup_rename "backup rename"
     run_test test_backup_volume_export "backup volume export"
+    run_test test_backup_export_import_instance_only "backup export and import instance only"
     run_test test_backup_volume_rename_delete "backup volume rename and delete"
+    run_test test_backup_different_instance_uuid "backup instance and check instance UUIDs"
+    run_test test_backup_volume_expiry "backup volume expiry"
+    run_test test_backup_export_import_recover "backup export, import, and recovery"
     run_test test_container_local_cross_pool_handling "container local cross pool handling"
     run_test test_incremental_copy "incremental container copy"
     run_test test_profiles_project_default "profiles in default project"
@@ -332,6 +366,9 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_filtering "API filtering"
     run_test test_warnings "Warnings"
     run_test test_metrics "Metrics"
+    run_test test_storage_volume_recover "Recover storage volumes"
+    run_test test_syslog_socket "Syslog socket"
+    run_test test_lxd_user "lxd user"
 fi
 
 # shellcheck disable=SC2034
