@@ -1,3 +1,70 @@
+test_storage_volume_recover() {
+  LXD_IMPORT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
+  chmod +x "${LXD_IMPORT_DIR}"
+  spawn_lxd "${LXD_IMPORT_DIR}" true
+
+  poolName=$(lxc profile device get default root pool)
+  poolDriver=$(lxc storage show "${poolName}" | awk '/^driver:/ {print $2}')
+
+  # Create custom block volume.
+  lxc storage volume create "${poolName}" vol1 --type=block
+
+  # Import ISO.
+  truncate -s 8MiB foo.iso
+  lxc storage volume import "${poolName}" ./foo.iso vol2 --type=iso
+
+  # Delete database entry of the created custom block volume.
+  lxd sql global "PRAGMA foreign_keys=ON; DELETE FROM storage_volumes WHERE name='vol1'"
+  lxd sql global "PRAGMA foreign_keys=ON; DELETE FROM storage_volumes WHERE name='vol2'"
+
+  # Ensure the custom block volume is no longer listed.
+  ! lxc storage volume show "${poolName}" vol1 || false
+  ! lxc storage volume show "${poolName}" vol2 || false
+
+  if [ "$poolDriver" = "zfs" ]; then
+    # Create filesystem volume.
+    lxc storage volume create "${poolName}" vol3
+
+    # Create block_mode enabled volume.
+    lxc storage volume create "${poolName}" vol4 zfs.block_mode=true size=200MiB
+
+    # Delete database entries of the created custom volumes.
+    lxd sql global "PRAGMA foreign_keys=ON; DELETE FROM storage_volumes WHERE name='vol3'"
+    lxd sql global "PRAGMA foreign_keys=ON; DELETE FROM storage_volumes WHERE name='vol4'"
+
+    # Ensure the custom volumes are no longer listed.
+    ! lxc storage volume show "${poolName}" vol3 || false
+    ! lxc storage volume show "${poolName}" vol4 || false
+  fi
+
+  # Recover custom block volume.
+  cat <<EOF | lxd recover
+no
+yes
+yes
+EOF
+
+  # Ensure custom storage volume has been recovered.
+  lxc storage volume show "${poolName}" vol1 | grep -q 'content_type: block'
+  lxc storage volume show "${poolName}" vol2 | grep -q 'content_type: iso'
+
+  if [ "$poolDriver" = "zfs" ]; then
+    # Ensure custom storage volumes have been recovered.
+    lxc storage volume show "${poolName}" vol3 | grep -q 'content_type: filesystem'
+    lxc storage volume show "${poolName}" vol4 | grep -q 'content_type: filesystem'
+
+    # Cleanup
+    lxc storage volume delete "${poolName}" vol3
+    lxc storage volume delete "${poolName}" vol4
+  fi
+
+  # Cleanup
+  rm -f foo.iso
+  lxc storage volume delete "${poolName}" vol1
+  lxc storage volume delete "${poolName}" vol2
+  shutdown_lxd "${LXD_IMPORT_DIR}"
+}
+
 test_container_recover() {
   LXD_IMPORT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
   chmod +x "${LXD_IMPORT_DIR}"
@@ -22,7 +89,7 @@ test_container_recover() {
     lxc project switch test
 
     # Basic no-op check.
-    cat <<EOF | lxd recover | grep "No unknown volumes found. Nothing to do."
+    cat <<EOF | lxd recover | grep "No unknown storage pools or volumes found. Nothing to do."
 no
 yes
 EOF
@@ -212,6 +279,62 @@ EOF
   kill_lxd "${LXD_IMPORT_DIR}"
 }
 
+test_bucket_recover() {
+  if ! command -v "minio" >/dev/null 2>&1; then
+    echo "==> SKIP: Skip bucket recovery test due to missing minio"
+    return
+  fi
+
+  (
+    set -e
+
+    poolName=$(lxc profile device get default root pool)
+    poolDriver=$(lxc storage show "${poolName}" | awk '/^driver:/ {print $2}')
+    bucketName="bucket123"
+
+    # Skip ceph driver - ceph does not support storage buckets
+    if [ "${poolDriver}" = "ceph" ]; then
+      return 0
+    fi
+
+    # Create storage bucket
+    lxc storage bucket create "${poolName}" "${bucketName}"
+
+    # Create storage bucket keys
+    key1=$(lxc storage bucket key create "${poolName}" "${bucketName}" key1 --role admin)
+    key2=$(lxc storage bucket key create "${poolName}" "${bucketName}" key2 --role read-only)
+    key1_accessKey=$(echo "$key1" | awk '/^Access key/ { print $3 }')
+    key1_secretKey=$(echo "$key1" | awk '/^Secret key/ { print $3 }')
+    key2_accessKey=$(echo "$key2" | awk '/^Access key/ { print $3 }')
+    key2_secretKey=$(echo "$key2" | awk '/^Secret key/ { print $3 }')
+
+    # Remove bucket from global DB
+    lxd sql global "delete from storage_buckets where name = '${bucketName}'"
+
+    # Recover bucket
+    cat <<EOF | lxd recover
+no
+yes
+yes
+EOF
+
+    # Verify bucket is recovered
+    lxc storage bucket ls "${poolName}" --format compact | grep "${bucketName}"
+
+    # Verify bucket key with role admin is recovered
+    recoveredKey1=$(lxc storage bucket key show "${poolName}" "${bucketName}" "${key1_accessKey}")
+    echo "${recoveredKey1}" | grep "role: admin"
+    echo "${recoveredKey1}" | grep "access-key: ${key1_accessKey}"
+    echo "${recoveredKey1}" | grep "secret-key: ${key1_secretKey}"
+
+    # Verify bucket key with role read-only is recovered
+    recoveredKey2=$(lxc storage bucket key show "${poolName}" "${bucketName}" "${key2_accessKey}")
+    echo "${recoveredKey2}" | grep "role: read-only"
+    echo "${recoveredKey2}" | grep "access-key: ${key2_accessKey}"
+    echo "${recoveredKey2}" | grep "secret-key: ${key2_secretKey}"
+  )
+}
+
 test_backup_import() {
   test_backup_import_with_project
   test_backup_import_with_project fooproject
@@ -219,6 +342,7 @@ test_backup_import() {
 
 test_backup_import_with_project() {
   project="default"
+  pool="lxdtest-$(basename "${LXD_DIR}")"
 
   if [ "$#" -ne 0 ]; then
     # Create a projects
@@ -231,7 +355,6 @@ test_backup_import_with_project() {
     deps/import-busybox --project "$project-b" --alias testimage
 
     # Add a root device to the default profile of the project
-    pool="lxdtest-$(basename "${LXD_DIR}")"
     lxc profile device add default root disk path="/" pool="${pool}"
     lxc profile device add default root disk path="/" pool="${pool}" --project "$project-b"
   fi
@@ -274,6 +397,8 @@ test_backup_import_with_project() {
     lxc export c2 "${LXD_DIR}/c2-optimized.tar.gz" --optimized-storage
   fi
 
+  old_uuid="$(lxc storage volume get "${pool}" container/c2 volatile.uuid)"
+  old_snap0_uuid="$(lxc storage volume get "${pool}" container/c2/snap0 volatile.uuid)"
   lxc export c2 "${LXD_DIR}/c2.tar.gz"
   lxc delete --force c2
 
@@ -281,6 +406,13 @@ test_backup_import_with_project() {
   lxc import "${LXD_DIR}/c2.tar.gz" c3
   lxc info c2 | grep snap0
   lxc info c3 | grep snap0
+
+  # Check if the imported instance and its snapshot have a new UUID.
+  [ -n "$(lxc storage volume get "${pool}" container/c2 volatile.uuid)" ]
+  [ -n "$(lxc storage volume get "${pool}" container/c2/snap0 volatile.uuid)" ]
+  [ "$(lxc storage volume get "${pool}" container/c2 volatile.uuid)" != "${old_uuid}" ]
+  [ "$(lxc storage volume get "${pool}" container/c2/snap0 volatile.uuid)" != "${old_snap0_uuid}" ]
+
   lxc start c2
   lxc start c3
   lxc stop c2 --force
@@ -485,14 +617,14 @@ test_backup_rename() {
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
-  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Instance not found" ; then
+  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Instance backup not found" ; then
     echo "invalid rename response for missing container"
     false
   fi
 
   lxc init testimage c1
 
-  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Load backup from database: Instance backup not found" ; then
+  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Instance backup not found" ; then
     echo "invalid rename response for missing backup"
     false
   fi
@@ -650,6 +782,10 @@ test_backup_volume_export_with_project() {
 
   rm -rf "${LXD_DIR}/non-optimized/"*
 
+  old_uuid="$(lxc storage volume get "${custom_vol_pool}" testvol volatile.uuid)"
+  old_snap0_uuid="$(lxc storage volume get "${custom_vol_pool}" testvol/test-snap0 volatile.uuid)"
+  old_snap1_uuid="$(lxc storage volume get "${custom_vol_pool}" testvol/test-snap1 volatile.uuid)"
+
   # Test non-optimized import.
   lxc stop -f c1
   lxc storage volume detach "${custom_vol_pool}" testvol c1
@@ -661,6 +797,14 @@ test_backup_volume_export_with_project() {
   lxc storage volume get "${custom_vol_pool}" testvol/test-snap0 user.foo | grep -Fx "test-snap0"
   lxc storage volume get "${custom_vol_pool}" testvol/test-snap1 user.foo | grep -Fx "test-snap1"
 
+  # Check if the imported volume and its snapshots have a new UUID.
+  [ -n "$(lxc storage volume get "${custom_vol_pool}" testvol volatile.uuid)" ]
+  [ -n "$(lxc storage volume get "${custom_vol_pool}" testvol/test-snap0 volatile.uuid)" ]
+  [ -n "$(lxc storage volume get "${custom_vol_pool}" testvol/test-snap1 volatile.uuid)" ]
+  [ "$(lxc storage volume get "${custom_vol_pool}" testvol volatile.uuid)" != "${old_uuid}" ]
+  [ "$(lxc storage volume get "${custom_vol_pool}" testvol/test-snap0 volatile.uuid)" != "${old_snap0_uuid}" ]
+  [ "$(lxc storage volume get "${custom_vol_pool}" testvol/test-snap1 volatile.uuid)" != "${old_snap1_uuid}" ]
+
   lxc storage volume import "${custom_vol_pool}" "${LXD_DIR}/testvol.tar.gz" testvol2
   lxc storage volume attach "${custom_vol_pool}" testvol c1 /mnt
   lxc storage volume attach "${custom_vol_pool}" testvol2 c1 /mnt2
@@ -783,3 +927,111 @@ test_backup_volume_rename_delete() {
   lxc storage volume rm "${pool}" vol2
   ! stat "${LXD_DIR}"/backups/custom/"${pool}"/default_vol2 || false
 }
+
+test_backup_different_instance_uuid() {
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
+  echo "==> Checking instances UUID during backup operation"
+  lxc launch testimage c1
+  initialUUID=$(lxc config get c1 volatile.uuid)
+  initialGenerationID=$(lxc config get c1 volatile.uuid.generation)
+
+  # export and import to trigger new UUID generation
+  lxc export c1 "${LXD_DIR}/c1.tar.gz"
+  lxc delete -f c1
+  lxc import "${LXD_DIR}/c1.tar.gz"
+
+  newUUID=$(lxc config get c1 volatile.uuid)
+  newGenerationID=$(lxc config get c1 volatile.uuid.generation)
+
+  if [ "${initialGenerationID}" != "${newGenerationID}" ] || [ "${initialUUID}" != "${newUUID}" ]; then
+    echo "==> UUID of the instance should remain the same after importing the backup file"
+    false
+  fi
+
+  lxc delete -f c1
+}
+
+test_backup_volume_expiry() {
+  poolName=$(lxc profile device get default root pool)
+
+  # Create custom volume.
+  lxc storage volume create "${poolName}" vol1
+
+  # Create storage volume backups using the API directly.
+  # The first one is created with an expiry date, the second one never expires.
+  lxc query -X POST -d '{\"expires_at\":\"2023-07-17T00:00:00Z\"}' /1.0/storage-pools/"${poolName}"/volumes/custom/vol1/backups
+  lxc query -X POST -d '{}' /1.0/storage-pools/"${poolName}"/volumes/custom/vol1/backups
+
+  # Check that both backups are listed.
+  [ "$(lxc query /1.0/storage-pools/"${poolName}"/volumes/custom/vol1/backups | jq '.[]' | wc -l)" -eq 2 ]
+
+  # Restart LXD which will trigger the task which removes expired volume backups.
+  shutdown_lxd "${LXD_DIR}"
+  respawn_lxd "${LXD_DIR}" true
+
+  # Check that there's only one backup remaining.
+  [ "$(lxc query /1.0/storage-pools/"${poolName}"/volumes/custom/vol1/backups | jq '.[]' | wc -l)" -eq 1 ]
+
+  # Cleanup.
+  lxc storage volume delete "${poolName}" vol1
+}
+
+test_backup_export_import_recover() {
+  (
+    set -e
+
+    poolName=$(lxc profile device get default root pool)
+
+    ensure_import_testimage
+    ensure_has_localhost_remote "${LXD_ADDR}"
+
+    # Create and export an instance.
+    lxc launch testimage c1
+    lxc export c1 "${LXD_DIR}/c1.tar.gz"
+    lxc delete -f c1
+
+    # Import instance and remove no longer required tarball.
+    lxc import "${LXD_DIR}/c1.tar.gz" c2
+    rm "${LXD_DIR}/c1.tar.gz"
+
+    # Remove imported instance enteries from database.
+    lxd sql global "delete from instances where name = 'c2'"
+    lxd sql global "delete from storage_volumes where name = 'c2'"
+
+    # Recover removed instance.
+    cat <<EOF | lxd recover
+no
+yes
+yes
+EOF
+
+    # Remove recovered instance.
+    lxc rm -f c2
+  )
+}
+
+test_backup_export_import_instance_only() {
+  poolName=$(lxc profile device get default root pool)
+
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
+  # Create an instance with snapshot.
+  lxc init testimage c1
+  lxc snapshot c1
+
+  # Export the instance and remove it.
+  lxc export c1 "${LXD_DIR}/c1.tar.gz" --instance-only
+  lxc delete -f c1
+
+  # Import the instance from tarball.
+  lxc import "${LXD_DIR}/c1.tar.gz"
+
+  # Verify imported instance has no snapshots.
+  [ "$(lxc query "/1.0/storage-pools/${poolName}/volumes/container/c1/snapshots" | jq "length == 0")" = "true" ]
+
+  rm "${LXD_DIR}/c1.tar.gz"
+  lxc delete -f c1
+}
