@@ -149,13 +149,44 @@ test_basic_usage() {
   lxc publish bar --alias=foo-image --alias=foo-image2
   lxc launch testimage baz
   # change the container filesystem so the resulting image is different
-  lxc exec baz touch /somefile
+  lxc exec baz -- touch /somefile
   lxc stop baz --force
-  # publishing another image with same alias doesn't fail
-  lxc publish baz --alias=foo-image
+  # publishing another image with same alias should fail
+  ! lxc publish baz --alias=foo-image || false
+  # publishing another image with same alias and '--reuse' flag should success
+  lxc publish baz --alias=foo-image --reuse
+  fooImage=$(lxc image list -cF -fcsv foo-image)
+  fooImage2=$(lxc image list -cF -fcsv foo-image2)
   lxc delete baz
   lxc image delete foo-image foo-image2
 
+  # the first image should have foo-image2 alias and the second imgae foo-image alias
+  if [ "$fooImage" = "$fooImage2" ]; then
+    echo "foo-image and foo-image2 aliases should be assigned to two different images"
+    false
+  fi
+
+
+  # Test container publish with existing alias
+  lxc publish bar --alias=foo-image --alias=foo-image2
+  lxc launch testimage baz
+  # change the container filesystem so the resulting image is different
+  lxc exec baz -- touch /somefile
+  lxc stop baz --force
+  # publishing another image with same aliases
+  lxc publish baz --alias=foo-image --alias=foo-image2 --reuse
+  fooImage=$(lxc image list -cF -fcsv foo-image)
+  fooImage2=$(lxc image list -cF -fcsv foo-image2)
+  lxc delete baz
+  lxc image delete foo-image
+
+  # the second image should have foo-image and foo-image2 aliases and the first one should be removed
+  if [ "$fooImage" != "$fooImage2" ]; then
+    echo "foo-image and foo-image2 aliases should be assigned to the same image"
+    false
+  fi
+
+
   # Test image compression on publish
   lxc publish bar --alias=foo-image-compressed --compression=bzip2 prop=val1
   lxc image show foo-image-compressed | grep val1
@@ -195,11 +226,11 @@ test_basic_usage() {
   curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/"
   lxc image delete foo-image2
 
-  # Test invalid container names
+  # Test invalid instance names
   ! lxc init testimage -abc || false
   ! lxc init testimage abc- || false
   ! lxc init testimage 1234 || false
-  ! lxc init testimage 12test || false
+  ! lxc init testimage foo.bar || false
   ! lxc init testimage a_b_c || false
   ! lxc init testimage aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa || false
 
@@ -346,10 +377,10 @@ test_basic_usage() {
 
   # cycle it a few times
   lxc start foo
-  mac1=$(lxc exec foo cat /sys/class/net/eth0/address)
+  mac1=$(lxc exec foo -- cat /sys/class/net/eth0/address)
   lxc stop foo --force
   lxc start foo
-  mac2=$(lxc exec foo cat /sys/class/net/eth0/address)
+  mac2=$(lxc exec foo -- cat /sys/class/net/eth0/address)
 
   if [ -n "${mac1}" ] && [ -n "${mac2}" ] && [ "${mac1}" != "${mac2}" ]; then
     echo "==> MAC addresses didn't match across restarts (${mac1} vs ${mac2})"
@@ -360,7 +391,7 @@ test_basic_usage() {
   lxc launch testimage test-limits -t c0.5-m0.2
   [ "$(lxc config get test-limits limits.cpu)" = "1" ]
   [ "$(lxc config get test-limits limits.cpu.allowance)" = "50%" ]
-  [ "$(lxc config get test-limits limits.memory)" = "204MB" ]
+  [ "$(lxc config get test-limits limits.memory)" = "204MiB" ]
   lxc delete -f test-limits
 
   # Test last_used_at field is working properly
@@ -381,9 +412,9 @@ test_basic_usage() {
   [ "$(lxc exec foo --user 1234 --group 5678 --cwd /blah -- pwd)" = "/blah" ] || false
 
   # check that we can set the environment
-  lxc exec foo pwd | grep /root
-  lxc exec --env BEST_BAND=meshuggah foo env | grep meshuggah
-  lxc exec foo ip link show | grep eth0
+  lxc exec foo -- pwd | grep /root
+  lxc exec --env BEST_BAND=meshuggah foo -- env | grep meshuggah
+  lxc exec foo -- ip link show | grep eth0
 
   # check that we can get the return code for a non- wait-for-websocket exec
   op=$(my_curl -X POST "https://${LXD_ADDR}/1.0/containers/foo/exec" -d '{"command": ["echo", "test"], "environment": {}, "wait-for-websocket": false, "interactive": false}' | jq -r .operation)
@@ -393,11 +424,11 @@ test_basic_usage() {
   echo abc > "${LXD_DIR}/in"
 
   lxc file push "${LXD_DIR}/in" foo/root/
-  lxc exec foo /bin/cat /root/in | grep abc
+  lxc exec foo -- /bin/cat /root/in | grep -xF abc
   lxc exec foo -- /bin/rm -f root/in
 
   lxc file push "${LXD_DIR}/in" foo/root/in1
-  lxc exec foo /bin/cat /root/in1 | grep abc
+  lxc exec foo -- /bin/cat /root/in1 | grep -xF abc
   lxc exec foo -- /bin/rm -f root/in1
 
   # test lxc file edit doesn't change target file's owner and permissions
@@ -411,10 +442,14 @@ test_basic_usage() {
   # make sure stdin is chowned to our container root uid (Issue #590)
   [ -t 0 ] && [ -t 1 ] && lxc exec foo -- chown 1000:1000 /proc/self/fd/0
 
-  echo foo | lxc exec foo tee /tmp/foo
+  echo foo | lxc exec foo -- tee /tmp/foo
+
+  # test exec with/without "--" separator
+  lxc exec foo -- true
+  lxc exec foo true
 
   # Detect regressions/hangs in exec
-  sum=$(ps aux | tee "${LXD_DIR}/out" | lxc exec foo md5sum | cut -d' ' -f1)
+  sum=$(ps aux | tee "${LXD_DIR}/out" | lxc exec foo -- md5sum | cut -d' ' -f1)
   [ "${sum}" = "$(md5sum "${LXD_DIR}/out" | cut -d' ' -f1)" ]
   rm "${LXD_DIR}/out"
 
@@ -459,15 +494,19 @@ test_basic_usage() {
     echo "==> SKIP: apparmor tests (missing kernel support)"
   fi
 
-  lxc launch testimage lxd-seccomp-test
-  init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
-  [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "2" ]
-  lxc stop --force lxd-seccomp-test
-  lxc config set lxd-seccomp-test security.syscalls.deny_default false
-  lxc start lxd-seccomp-test
-  init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
-  [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "0" ]
-  lxc delete --force lxd-seccomp-test
+  if [ "$(awk '/^Seccomp:/ {print $2}' "/proc/self/status")" -eq "0" ]; then
+    lxc launch testimage lxd-seccomp-test
+    init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
+    [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "2" ]
+    lxc stop --force lxd-seccomp-test
+    lxc config set lxd-seccomp-test security.syscalls.deny_default false
+    lxc start lxd-seccomp-test
+    init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
+    [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "0" ]
+    lxc delete --force lxd-seccomp-test
+  else
+    echo "==> SKIP: seccomp tests (seccomp filtering is externally enabled)"
+  fi
 
   # make sure that privileged containers are not world-readable
   lxc profile create unconfined
@@ -531,7 +570,7 @@ test_basic_usage() {
         REBOOTED="true"
         break
       else
-        lxc exec foo reboot || true  # Signal to running old init processs to reboot if not rebooted yet.
+        lxc exec foo -- reboot || true  # Signal to running old init process to reboot if not rebooted yet.
       fi
     fi
 
@@ -583,4 +622,62 @@ test_basic_usage() {
   lxc storage volume delete bla vol1
   lxc storage volume delete bla vol2
   lxc storage delete bla
+
+  # Test rebuilding an instance with its original image.
+  lxc init testimage c1
+  lxc start c1
+  lxc exec c1 -- touch /data.txt
+  lxc stop c1
+  lxc rebuild testimage c1
+  lxc start c1
+  ! lxc exec c1 -- stat /data.txt || false
+  lxc delete c1 -f
+
+  # Test a forced rebuild
+  lxc launch testimage c1
+  ! lxc rebuild testimage c1 || false
+  lxc rebuild testimage c1 --force
+  lxc delete c1 -f
+
+  # Test rebuilding an instance with a new image.
+  lxc init c1 --empty
+  lxc remote add l1 "${LXD_ADDR}" --accept-certificate --password foo
+  lxc rebuild l1:testimage c1
+  lxc start c1
+  lxc delete c1 -f
+  lxc remote remove l1
+
+  # Test rebuilding an instance with an empty file system.
+  lxc init testimage c1
+  lxc rebuild c1 --empty
+  ! lxc config show c1 | grep -q 'image.*' || false
+  lxc delete c1 -f
+
+  # Test assigning an empty profile (with no root disk device) to an instance.
+  lxc init testimage c1
+  lxc profile create foo
+  ! lxc profile assign c1 foo || false
+  lxc profile delete foo
+  lxc delete -f c1
+
+  # Multiple ephemeral instances delete
+  lxc launch testimage c1
+  lxc launch testimage c2
+  lxc launch testimage c3
+
+  lxc delete -f c1 c2 c3
+  remaining_instances="$(lxc list --format csv)"
+  [ -z "${remaining_instances}" ]
+}
+
+test_server_info() {
+  # Ensure server always reports support for containers.
+  lxc query /1.0 | jq -e '.environment.instance_types | contains(["container"])'
+
+  # Ensure the version number has the format (X.Y.Z for LTSes and X.Y otherwise)
+  if lxc query /1.0 | jq -e '.environment.server_lts == true'; then
+    lxc query /1.0 | jq -re '.environment.server_version' | grep -E '[0-9]+\.[0-9]+\.[0-9]+'
+  else
+    lxc query /1.0 | jq -re '.environment.server_version' | grep -xE '[0-9]+\.[0-9]+'
+  fi
 }
