@@ -2,6 +2,13 @@ test_container_devices_nic_bridged() {
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
+  firewallDriver=$(lxc info | awk -F ":" '/firewall:/{gsub(/ /, "", $0); print $2}')
+
+  if [ "$firewallDriver" != "xtables" ] && [ "$firewallDriver" != "nftables" ]; then
+    echo "Unrecognised firewall driver: ${firewallDriver}"
+    false
+  fi
+
   vethHostName="veth$$"
   ctName="nt$$"
   ctMAC="0a:92:a7:0d:b7:d9"
@@ -36,8 +43,10 @@ test_container_devices_nic_bridged() {
   lxc profile device set "${ctName}" eth0 ipv6.routes "2001:db8::1${ipRand}/128"
   lxc profile device set "${ctName}" eth0 limits.ingress 1Mbit
   lxc profile device set "${ctName}" eth0 limits.egress 2Mbit
+  lxc profile device set "${ctName}" eth0 limits.priority 5
   lxc profile device set "${ctName}" eth0 host_name "${vethHostName}"
   lxc profile device set "${ctName}" eth0 mtu "1400"
+  lxc profile device set "${ctName}" eth0 queue.tx.length "1200"
   lxc profile device set "${ctName}" eth0 hwaddr "${ctMAC}"
 
   lxc init testimage "${ctName}" -p "${ctName}"
@@ -82,18 +91,39 @@ test_container_devices_nic_bridged() {
     false
   fi
 
+  # Check that limits.priority was correctly configured in the firewall.
+  if [ "$firewallDriver" = "xtables" ]; then
+    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
+    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0005"
+  else
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:5"
+  fi
+
   # Check profile custom MTU is applied in container on boot.
   if ! lxc exec "${ctName}" -- grep "1400" /sys/class/net/eth0/mtu ; then
     echo "container veth mtu invalid"
     false
   fi
 
-  # Check profile custom MTU is applied on host side of veth.
-  if ! grep "1400" /sys/class/net/"${vethHostName}"/mtu ; then
+  # Check profile custom MTU doesn't affect the host.
+  if ! grep "1500" /sys/class/net/"${vethHostName}"/mtu ; then
     echo "host veth mtu invalid"
     false
   fi
 
+  # Check profile custom txqueuelen is applied in container on boot.
+  if ! lxc exec "${ctName}" -- grep "1200" /sys/class/net/eth0/tx_queue_len ; then
+    echo "container veth txqueuelen invalid"
+    false
+  fi
+
+  # Check profile custom txqueuelen is applied on host side of veth.
+  if ! grep "1200" /sys/class/net/"${vethHostName}"/tx_queue_len ; then
+    echo "host veth txqueuelen invalid"
+    false
+  fi
+
   # Check profile custom MAC is applied in container on boot.
   if ! lxc exec "${ctName}" -- grep -Fix "${ctMAC}" /sys/class/net/eth0/address ; then
     echo "mac invalid"
@@ -117,6 +147,7 @@ test_container_devices_nic_bridged() {
     ipv6.routes="2001:db8::2${ipRand}/128" \
     limits.ingress=3Mbit \
     limits.egress=4Mbit \
+    limits.priority=6 \
     host_name="${vethHostName}" \
     hwaddr="${ctMAC}" \
     mtu=1401
@@ -151,14 +182,23 @@ test_container_devices_nic_bridged() {
     false
   fi
 
+  # Check that limits.priority was correctly configured in the firewall.
+  if [ "$firewallDriver" = "xtables" ]; then
+    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
+    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0006"
+  else
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:6"
+  fi
+
   # Check custom MTU is applied on hot-plug.
   if ! lxc exec "${ctName}" -- grep "1401" /sys/class/net/eth0/mtu ; then
     echo "container veth mtu invalid"
     false
   fi
 
-  # Check custom MTU is applied host-side on hot-plug.
-  if !  grep "1401" /sys/class/net/"${vethHostName}"/mtu ; then
+  # Check custom MTU doesn't affect the host.
+  if ! grep "1500" /sys/class/net/"${vethHostName}"/mtu ; then
     echo "host veth mtu invalid"
     false
   fi
@@ -202,14 +242,23 @@ test_container_devices_nic_bridged() {
     false
   fi
 
+  # Check that limits.priority was correctly configured in the firewall.
+  if [ "$firewallDriver" = "xtables" ]; then
+    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
+    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0005"
+  else
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:5"
+  fi
+
   # Check profile custom MTU is applied on hot-removal.
   if ! lxc exec "${ctName}" -- grep "1400" /sys/class/net/eth0/mtu ; then
     echo "container veth mtu invalid"
     false
   fi
 
-  # Checl profile custom MTU is applied host-side on hot-removal.
-  if ! grep "1400" /sys/class/net/"${vethHostName}"/mtu ; then
+  # Check custom MTU doesn't affect the host.
+  if ! grep "1500" /sys/class/net/"${vethHostName}"/mtu ; then
     echo "host veth mtu invalid"
     false
   fi
@@ -251,6 +300,7 @@ test_container_devices_nic_bridged() {
   lxc config device set "${ctName}" eth0 ipv6.routes "2001:db8::2${ipRand}/128"
   lxc config device set "${ctName}" eth0 limits.ingress 3Mbit
   lxc config device set "${ctName}" eth0 limits.egress 4Mbit
+  lxc config device set "${ctName}" eth0 limits.priority 6
   lxc config device set "${ctName}" eth0 mtu 1402
   lxc config device set "${ctName}" eth0 hwaddr "${ctMAC}"
 
@@ -284,6 +334,15 @@ test_container_devices_nic_bridged() {
     false
   fi
 
+  # Check that limits.priority was correctly configured in the firewall.
+  if [ "$firewallDriver" = "xtables" ]; then
+    iptables -t mangle -S | grep -c "${ctName} (${vethHostName}) netprio" | grep 1
+    iptables -t mangle -S | grep "${ctName} (${vethHostName}) netprio" | grep "0000:0006"
+  else
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep -c "meta priority set" | grep 1
+    nft -nn list chain netdev lxd "egress.netprio.${ctName}.${vethHostName}" | grep "meta priority set 0:6"
+  fi
+
   # Check custom MTU is applied update.
   if ! lxc exec "${ctName}" -- grep "1402" /sys/class/net/eth0/mtu ; then
     echo "mtu invalid"
@@ -547,6 +606,25 @@ test_container_devices_nic_bridged() {
     echo "bridge command doesn't support port isolation, skipping port isolation checks"
   fi
 
+  # Test interface naming scheme.
+  lxc init testimage test-naming
+  lxc start test-naming
+  lxc query "/1.0/instances/test-naming/state" | jq -r .network.eth0.host_name | grep ^veth
+  lxc stop -f test-naming
+
+  lxc config set instances.nic.host_name random
+  lxc start test-naming
+  lxc query "/1.0/instances/test-naming/state" | jq -r .network.eth0.host_name | grep ^veth
+  lxc stop -f test-naming
+
+  lxc config set instances.nic.host_name mac
+  lxc start test-naming
+  lxc query "/1.0/instances/test-naming/state" | jq -r .network.eth0.host_name | grep ^lxd
+  lxc stop -f test-naming
+
+  lxc config unset instances.nic.host_name
+  lxc delete -f test-naming
+
   # Test new container with conflicting addresses can be created as a copy.
   lxc config device set "${ctName}" eth0 \
     ipv4.address=192.0.2.232 \
@@ -555,7 +633,7 @@ test_container_devices_nic_bridged() {
   lxc copy "${ctName}" foo # Gets new MAC address but IPs still conflict.
   ! stat "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/foo.eth0" || false
   lxc snapshot foo
-  lxc export foo foo.tar.gz
+  lxc export foo foo.tar.gz # Export in pristine state for import tests below.
   ! lxc start foo || false
   lxc config device set foo eth0 \
     ipv4.address=192.0.2.233 \
@@ -604,9 +682,35 @@ test_container_devices_nic_bridged() {
 
   # Check MAC conflict detection:
   ! lxc config device set "${ctName}" eth0 hwaddr="0a:92:a7:0d:b7:c9" || false
+  lxc delete -f foo
+
+  # Test container can be imported with device override to fix conflict.
+  lxc import foo.tar.gz --device eth0,ipv4.address=192.0.2.233 --device eth0,ipv6.address=2001:db8::3
+  lxc config device get foo eth0 ipv4.address | grep -Fx '192.0.2.233'
+  lxc config device get foo eth0 ipv6.address | grep -Fx '2001:db8::3'
+  lxc start foo
+
+  # Test container can be copied with device override to fix conflict.
+  lxc copy foo foo-copy --device eth0,ipv4.address=192.0.2.234 --device eth0,ipv6.address=2001:db8::4 --device eth0,host_name=veth0 --device eth0,ipv4.routes=192.0.2.20/32 --device eth0,ipv6.routes=2001:db8::20/128
+  lxc config device get foo-copy eth0 ipv4.address | grep -Fx '192.0.2.234'
+  lxc config device get foo-copy eth0 ipv6.address | grep -Fx '2001:db8::4'
+  lxc config device get foo-copy eth0 ipv4.routes | grep -Fx '192.0.2.20/32'
+  lxc config device get foo-copy eth0 ipv6.routes | grep -Fx '2001:db8::20/128'
+  lxc start foo-copy
+  lxc delete -f foo-copy
+
+  # Test snapshot can be copied with device override to fix conflict.
+  lxc snapshot foo tester
+  lxc copy foo/tester snap-copy --device eth0,ipv4.address=192.0.2.235 --device eth0,ipv6.address=2001:db8::5 --device eth0,host_name=veth1 --device eth0,ipv4.routes=192.0.2.21/32 --device eth0,ipv6.routes=2001:db8::21/128
+  lxc config device get snap-copy eth0 ipv4.address | grep -Fx '192.0.2.235'
+  lxc config device get snap-copy eth0 ipv6.address | grep -Fx '2001:db8::5'
+  lxc config device get snap-copy eth0 ipv4.routes | grep -Fx '192.0.2.21/32'
+  lxc config device get snap-copy eth0 ipv6.routes | grep -Fx '2001:db8::21/128'
+  lxc start snap-copy
+  lxc delete -f snap-copy
+  lxc delete -f foo
 
   # Test container with conflicting addresses rebuilds DHCP lease if original conflicting instance is removed.
-  lxc delete -f foo
   ! stat "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/foo.eth0" || false
   lxc import foo.tar.gz
   rm foo.tar.gz
@@ -616,6 +720,18 @@ test_container_devices_nic_bridged() {
   grep -F "192.0.2.232" "${LXD_DIR}/networks/${brName}/dnsmasq.hosts/foo.eth0"
   lxc delete -f foo
 
+  # Test container without extra network configuration can be restored from backup.
+  lxc init testimage foo -p "${ctName}"
+  lxc export foo foo.tar.gz
+  lxc import foo.tar.gz foo2
+  rm foo.tar.gz
+  lxc profile assign foo2 "${ctName}"
+
+  # Test container start will fail due to volatile MAC conflict.
+  lxc config get foo volatile.eth0.hwaddr | grep -Fx "$(lxc config get foo2 volatile.eth0.hwaddr)"
+  ! lxc start foo2 || false
+  lxc delete -f foo foo2
+
   # Check we haven't left any NICS lying around.
   endNicCount=$(find /sys/class/net | wc -l)
   if [ "$startNicCount" != "$endNicCount" ]; then
