@@ -31,10 +31,10 @@ test_migration() {
     # shellcheck disable=2153
     storage_pool1="lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-migration"
     storage_pool2="lxdtest-$(basename "${LXD2_DIR}")-non-thinpool-lvm-migration"
-    lxc_remote storage create l1:"$storage_pool1" lvm lvm.use_thinpool=false volume.size=25MB
+    lxc_remote storage create l1:"$storage_pool1" lvm lvm.use_thinpool=false size=1GiB volume.size=25MiB
     lxc_remote profile device set l1:default root pool "$storage_pool1"
 
-    lxc_remote storage create l2:"$storage_pool2" lvm lvm.use_thinpool=false volume.size=25MB
+    lxc_remote storage create l2:"$storage_pool2" lvm lvm.use_thinpool=false size=1GiB volume.size=25MiB
     lxc_remote profile device set l2:default root pool "$storage_pool2"
 
     migration "$LXD2_DIR"
@@ -46,6 +46,35 @@ test_migration() {
     lxc_remote storage delete l2:"$storage_pool2"
   fi
 
+  if [ "${LXD_BACKEND}" = "zfs" ]; then
+    # Test that block mode zfs backends work fine with migration.
+    for fs in "ext4" "btrfs" "xfs"; do
+      if ! command -v "mkfs.${fs}" >/dev/null 2>&1; then
+        echo "==> SKIP: Skipping block mode test on ${fs} due to missing tools."
+        continue
+      fi
+
+      # shellcheck disable=2039,3043
+      local storage_pool1 storage_pool2
+      # shellcheck disable=2153
+      storage_pool1="lxdtest-$(basename "${LXD_DIR}")-block-mode"
+      storage_pool2="lxdtest-$(basename "${LXD2_DIR}")-block-mode"
+      lxc_remote storage create l1:"$storage_pool1" zfs size=1GiB volume.zfs.block_mode=true volume.block.filesystem="${fs}"
+      lxc_remote profile device set l1:default root pool "$storage_pool1"
+
+      lxc_remote storage create l2:"$storage_pool2" zfs size=1GiB volume.zfs.block_mode=true volume.block.filesystem="${fs}"
+      lxc_remote profile device set l2:default root pool "$storage_pool2"
+
+      migration "$LXD2_DIR"
+
+      lxc_remote profile device set l1:default root pool "lxdtest-$(basename "${LXD_DIR}")"
+      lxc_remote profile device set l2:default root pool "lxdtest-$(basename "${LXD2_DIR}")"
+
+      lxc_remote storage delete l1:"$storage_pool1"
+      lxc_remote storage delete l2:"$storage_pool2"
+    done
+  fi
+
   lxc_remote remote remove l1
   lxc_remote remote remove l2
   kill_lxd "$LXD2_DIR"
@@ -155,6 +184,10 @@ migration() {
   echo "after" | lxc file push - cccp/blah
   lxc storage volume set "${pool}" container/cccp user.foo=postsnap1
 
+  # Check storage volume creation times are set.
+  lxc query /1.0/storage-pools/"${pool}"/volumes/container/cccp | jq .created_at | grep -Fv '0001-01-01T00:00:00Z'
+  lxc query /1.0/storage-pools/"${pool}"/volumes/container/cccp/snapshots/snap0 | jq .created_at | grep -Fv '0001-01-01T00:00:00Z'
+
   # Local container only copy.
   lxc copy cccp udssr --instance-only
   [ "$(lxc info udssr | grep -c snap)" -eq 0 ]
@@ -259,9 +292,49 @@ migration() {
   lxc_remote copy l1:c1 l2:c2 --refresh
   lxc_remote start l2:c2
   lxc_remote file pull l2:c2/root/testfile1 .
+  [ "$(cat testfile1)" = "test" ]
   rm testfile1
   lxc_remote stop -f l2:c2
 
+  # Change the files modification time by adding one nanosecond.
+  # Perform the change on the test runner since the busybox instances `touch` doesn't support setting nanoseconds.
+  lxc_remote start l1:c1
+  c1_pid="$(lxc_remote query l1:/1.0/instances/c1?recursion=1 | jq -r .state.pid)"
+  mtime_old="$(stat -c %y "/proc/${c1_pid}/root/root/testfile1")"
+  mtime_old_ns="$(date -d "$mtime_old" +%N | sed 's/^0*//')"
+
+  # Ensure the final nanoseconds are padded with zeros to create a valid format.
+  mtime_new_ns="$(printf "%09d\n" "$((mtime_old_ns+1))")"
+  mtime_new="$(date -d "$mtime_old" "+%Y-%m-%d %H:%M:%S.${mtime_new_ns} %z")"
+  lxc_remote stop -f l1:c1
+
+  # Before setting the new mtime create a local copy too.
+  lxc_remote copy l1:c1 l1:c2
+
+  # Change the modification time.
+  lxc_remote start l1:c1
+  c1_pid="$(lxc_remote query l1:/1.0/instances/c1?recursion=1 | jq -r .state.pid)"
+  touch -m -d "$mtime_new" "/proc/${c1_pid}/root/root/testfile1"
+  lxc_remote stop -f l1:c1
+
+  # Starting from rsync 3.1.3 it should discover the change of +1 nanosecond.
+  # Check if the file got refreshed to a different remote.
+  lxc_remote copy l1:c1 l2:c2 --refresh
+  lxc_remote start l1:c1 l2:c2
+  c1_pid="$(lxc_remote query l1:/1.0/instances/c1?recursion=1 | jq -r .state.pid)"
+  c2_pid="$(lxc_remote query l2:/1.0/instances/c2?recursion=1 | jq -r .state.pid)"
+  [ "$(stat "/proc/${c1_pid}/root/root/testfile1" -c %y)" = "$(stat "/proc/${c2_pid}/root/root/testfile1" -c %y)" ]
+  lxc_remote stop -f l1:c1 l2:c2
+
+  # Check if the file got refreshed locally.
+  lxc_remote copy l1:c1 l1:c2 --refresh
+  lxc_remote start l1:c1 l1:c2
+  c1_pid="$(lxc_remote query l1:/1.0/instances/c1?recursion=1 | jq -r .state.pid)"
+  c2_pid="$(lxc_remote query l1:/1.0/instances/c2?recursion=1 | jq -r .state.pid)"
+  [ "$(stat "/proc/${c1_pid}/root/root/testfile1" -c %y)" = "$(stat "/proc/${c2_pid}/root/root/testfile1" -c %y)" ]
+  lxc_remote rm -f l1:c2
+  lxc_remote stop -f l1:c1
+
   # This will create snapshot c1/snap0 with test device and expiry date.
   lxc_remote config device add l1:c1 testsnapdev none
   lxc_remote config set l1:c1 snapshots.expiry '1d'
@@ -311,6 +384,12 @@ migration() {
   lxc_remote storage volume get l2:"$remote_pool2" vol2 user.foo | grep -Fx "postsnap1vol1"
   lxc_remote storage volume get l2:"$remote_pool2" vol2/snap0 user.foo | grep -Fx "snap0vol1"
   lxc_remote storage volume get l2:"$remote_pool2" vol2/snap1 user.foo | grep -Fx "snap1vol1"
+
+  # check copied volume and snapshots have different UUIDs
+  [ "$(lxc_remote storage volume get l1:"$remote_pool1" vol1 volatile.uuid)" != "$(lxc_remote storage volume get l2:"$remote_pool2" vol2 volatile.uuid)" ]
+  [ "$(lxc_remote storage volume get l1:"$remote_pool1" vol1/snap0 volatile.uuid)" != "$(lxc_remote storage volume get l2:"$remote_pool2" vol2/snap0 volatile.uuid)" ]
+  [ "$(lxc_remote storage volume get l1:"$remote_pool1" vol1/snap1 volatile.uuid)" != "$(lxc_remote storage volume get l2:"$remote_pool2" vol2/snap1 volatile.uuid)" ]
+
   lxc_remote storage volume delete l2:"$remote_pool2" vol2
 
   # check moving volume and snapshots.
@@ -351,20 +430,28 @@ migration() {
   lxc_remote storage volume set l1:"$remote_pool1" vol3 user.foo=postsnap1vol3
 
   # check snapshot volumes and snapshots are refreshed
-  # FIXME: Due to a known issue (https://github.com/lxc/lxd/issues/10436) we are currently only diffing the
-  # snapshots by name, so infact existing snapshots of the same name won't be overwritten even if their config or
-  # contents is different.
   lxc_remote storage volume copy l1:"$remote_pool1/vol3" l2:"$remote_pool2/vol2" --refresh
   lxc_remote storage volume ls l2:"$remote_pool2"
   lxc_remote storage volume delete l1:"$remote_pool1" vol3
-  lxc_remote storage volume get l2:"$remote_pool2" vol2 user.foo | grep -Fx "postsnap1vol1"   # Should be postsnap1vol3
-  lxc_remote storage volume get l2:"$remote_pool2" vol2/snap0 user.foo | grep -Fx "snap0vol1" # Should be snap0vol3
-  lxc_remote storage volume get l2:"$remote_pool2" vol2/snap1 user.foo | grep -Fx "snap1vol1" # Should be snap1vol3
+  lxc_remote storage volume get l2:"$remote_pool2" vol2 user.foo | grep -Fx "postsnap1vol1" # FIXME Should be postsnap1vol3
+  lxc_remote storage volume get l2:"$remote_pool2" vol2/snap0 user.foo | grep -Fx "snap0vol3"
+  lxc_remote storage volume get l2:"$remote_pool2" vol2/snap1 user.foo | grep -Fx "snap1vol3"
   lxc_remote storage volume get l2:"$remote_pool2" vol2/snap2 user.foo | grep -Fx "snap2vol3"
   ! lxc_remote storage volume show l2:"$remote_pool2" vol2/snapremove || false
-
   lxc_remote storage volume delete l2:"$remote_pool2" vol2
 
+  # check that a refresh doesn't change the volume's and snapshot's UUID.
+  lxc_remote storage volume create l1:"$remote_pool1" vol1
+  lxc_remote storage volume snapshot l1:"$remote_pool1" vol1
+  lxc_remote storage volume copy l1:"$remote_pool1"/vol1 l2:"$remote_pool2"/vol2
+  old_uuid="$(lxc storage volume get l2:"$remote_pool2" vol2 volatile.uuid)"
+  old_snap0_uuid="$(lxc storage volume get l2:"$remote_pool2" vol2/snap0 volatile.uuid)"
+  lxc_remote storage volume copy l1:"$remote_pool1/vol1" l2:"$remote_pool2/vol2" --refresh
+  [ "$(lxc storage volume get l2:"$remote_pool2" vol2 volatile.uuid)" = "${old_uuid}" ]
+  [ "$(lxc storage volume get l2:"$remote_pool2" vol2/snap0 volatile.uuid)" = "${old_snap0_uuid}" ]
+  lxc_remote storage volume delete l2:"$remote_pool2" vol2
+  lxc_remote storage volume delete l1:"$remote_pool1" vol1
+
   # remote storage volume migration in "push" mode
   lxc_remote storage volume create l1:"$remote_pool1" vol1
   lxc_remote storage volume create l1:"$remote_pool1" vol2
@@ -407,6 +494,7 @@ migration() {
   lxc_remote storage volume copy l1:"$remote_pool1"/foo l2:"$remote_pool2"/bar
   lxc_remote storage volume delete l1:"$remote_pool1" foo
   lxc_remote storage volume delete l2:"$remote_pool2" bar
+  lxc_remote storage unset l1:"$remote_pool1" rsync.compression
 
   # Test some migration between projects
   lxc_remote project create l1:proj -c features.images=false -c features.profiles=false
@@ -435,11 +523,21 @@ migration() {
   lxc_remote copy l2:c1 l1:
   lxc_remote start l1:c1
   lxc_remote delete l1:c1 -f
-  lxc_remote delete l2:c1
+  lxc_remote delete l2:c1 -f
 
   lxc_remote project switch l1:default
   lxc_remote project delete l1:proj
 
+  # Check snapshot creation dates after migration.
+  lxc_remote init testimage l1:c1
+  lxc_remote snapshot l1:c1
+  ! lxc_remote storage volume show "l1:${remote_pool1}" container/c1 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc_remote storage volume show "l1:${remote_pool1}" container/c1/snap0 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  lxc_remote copy l1:c1 l2:c1
+  ! lxc_remote storage volume show "l2:${remote_pool2}" container/c1 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  [ "$(lxc_remote storage volume show "l1:${remote_pool1}" container/c1/snap0 | awk /created_at:/)" = "$(lxc_remote storage volume show "l2:${remote_pool2}" container/c1/snap0 | awk /created_at:/)" ]
+  lxc_remote delete l1:c1 -f
+  lxc_remote delete l2:c1 -f
 
   # Check migration with invalid snapshot config (disks attached with missing source pool and source path).
   lxc_remote init testimage l1:c1
@@ -534,13 +632,27 @@ migration() {
   lxc_remote rm -f l1:c1
   lxc_remote rm -f l2:c1
 
+  # migrate ISO custom volumes
+  truncate -s 8MiB foo.iso
+  lxc storage volume import l1:"${pool}" ./foo.iso iso1
+  lxc storage volume copy l1:"${pool}"/iso1 l2:"${remote_pool}"/iso1
+
+  lxc storage volume show l2:"${remote_pool}" iso1 | grep -q 'content_type: iso'
+  lxc storage volume move l1:"${pool}"/iso1 l2:"${remote_pool}"/iso2
+  lxc storage volume show l2:"${remote_pool}" iso2 | grep -q 'content_type: iso'
+  ! lxc storage volume show l1:"${pool}" iso1 || false
+
+  lxc storage volume delete l2:"${remote_pool}" iso1
+  lxc storage volume delete l2:"${remote_pool}" iso2
+  rm -f foo.iso
+
   if ! command -v criu >/dev/null 2>&1; then
     echo "==> SKIP: live migration with CRIU (missing binary)"
     return
   fi
 
   echo "==> CRIU: starting testing live-migration"
-  lxc_remote launch testimage l1:migratee
+  lxc_remote launch testimage l1:migratee -c raw.lxc=lxc.console.path=none
 
   # Wait for the container to be done booting
   sleep 1
@@ -550,10 +662,18 @@ migration() {
   lxc_remote start l1:migratee
 
   # Test stateful snapshots
+  # There is apparently a bug in CRIU that prevents checkpointing an instance that has been started from a
+  # checkpoint. So stop instance first before taking stateful snapshot.
+  lxc_remote stop -f l1:migratee
+  lxc_remote start l1:migratee
   lxc_remote snapshot --stateful l1:migratee
   lxc_remote restore l1:migratee snap0
 
   # Test live migration of container
+  # There is apparently a bug in CRIU that prevents checkpointing an instance that has been started from a
+  # checkpoint. So stop instance first before taking stateful snapshot.
+  lxc_remote stop -f l1:migratee
+  lxc_remote start l1:migratee
   lxc_remote move l1:migratee l2:migratee
 
   # Test copy of stateful snapshot
