@@ -1,16 +1,18 @@
 test_snapshots() {
-  snapshots
+  snapshots "lxdtest-$(basename "${LXD_DIR}")"
 
   if [ "$(storage_backend "$LXD_DIR")" = "lvm" ]; then
-    # Test that non-thinpool lvm backends work fine with snaphots.
-    lxc storage create "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snapshots" lvm lvm.use_thinpool=false volume.size=25MB
-    lxc profile device set default root pool "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snapshots"
+    pool="lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snapshots"
 
-    snapshots
+    # Test that non-thinpool lvm backends work fine with snaphots.
+    lxc storage create "${pool}" lvm lvm.use_thinpool=false volume.size=25MiB
+    lxc profile device set default root pool "${pool}"
+
+    snapshots "${pool}"
 
     lxc profile device set default root pool "lxdtest-$(basename "${LXD_DIR}")"
 
-    lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snapshots"
+    lxc storage delete "${pool}"
   fi
 }
 
@@ -18,6 +20,7 @@ snapshots() {
   # shellcheck disable=2039,3043
   local lxd_backend
   lxd_backend=$(storage_backend "$LXD_DIR")
+  pool="$1"
 
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
@@ -30,6 +33,15 @@ snapshots() {
     [ -d "${LXD_DIR}/snapshots/foo/snap0" ]
   fi
 
+  # Check if the snapshot has an UUID
+  [ -n "$(lxc storage volume get "${pool}" container/foo/snap0 volatile.uuid)" ]
+
+  # Check if the snapshot's UUID is different from the parent volume
+  [ "$(lxc storage volume get "${pool}" container/foo/snap0 volatile.uuid)" != "$(lxc storage volume get "${pool}" container/foo volatile.uuid)" ]
+
+  # Check if the snapshot's UUID can be modified
+  ! lxc storage volume set "${pool}" container/foo/snap0 volatile.uuid "2d94c537-5eff-4751-95b1-6a1b7d11f849" || false
+
   lxc snapshot foo
   # FIXME: make this backend agnostic
   if [ "$lxd_backend" = "dir" ]; then
@@ -54,6 +66,13 @@ snapshots() {
     [ ! -d "${LXD_DIR}/snapshots/foo/snap0" ]
   fi
 
+  # test deleting multiple snapshots
+  lxc snapshot foo snap2
+  lxc snapshot foo snap3
+  lxc delete foo/snap2 foo/snap3
+  ! lxc info foo | grep -q snap2 || false
+  ! lxc info foo | grep -q snap3 || false
+
   # no CLI for this, so we use the API directly (rename a snapshot)
   wait_for "${LXD_ADDR}" my_curl -X POST "https://${LXD_ADDR}/1.0/containers/foo/snapshots/tester" -d "{\"name\":\"tester2\"}"
   # FIXME: make this backend agnostic
@@ -89,18 +108,20 @@ snapshots() {
 }
 
 test_snap_restore() {
-  snap_restore
+  snap_restore "lxdtest-$(basename "${LXD_DIR}")"
 
   if [ "$(storage_backend "$LXD_DIR")" = "lvm" ]; then
-    # Test that non-thinpool lvm backends work fine with snaphots.
-    lxc storage create "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snap-restore" lvm lvm.use_thinpool=false volume.size=25MB
-    lxc profile device set default root pool "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snap-restore"
+    pool="lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snap-restore"
 
-    snap_restore
+    # Test that non-thinpool lvm backends work fine with snaphots.
+    lxc storage create "${pool}" lvm lvm.use_thinpool=false volume.size=25MiB
+    lxc profile device set default root pool "${pool}"
+
+    snap_restore "${pool}"
 
     lxc profile device set default root pool "lxdtest-$(basename "${LXD_DIR}")"
 
-    lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-lvm-snap-restore"
+    lxc storage delete "${pool}"
   fi
 }
 
@@ -108,6 +129,7 @@ snap_restore() {
   # shellcheck disable=2039,3043
   local lxd_backend
   lxd_backend=$(storage_backend "$LXD_DIR")
+  pool="$1"
 
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
@@ -151,6 +173,9 @@ snap_restore() {
   lxc exec bar -- rm /root/statelink
   lxc exec bar -- ln -s file_only_in_snap1 /root/statelink
   lxc exec bar -- mkdir /root/dir_only_in_snap1
+  initialUUID=$(lxc config get bar volatile.uuid)
+  initialGenerationID=$(lxc config get bar volatile.uuid.generation)
+  initialVolumeUUID=$(lxc storage volume get "${pool}" container/bar volatile.uuid)
   lxc stop bar --force
   lxc storage volume set "${pool}" container/bar user.foo=snap1
 
@@ -200,6 +225,64 @@ snap_restore() {
   # test restore using full snapshot name
   restore_and_compare_fs snap1
 
+  # Check that instances UUID remain the same before and after snapshoting
+  newUUID=$(lxc config get bar volatile.uuid)
+  if [ "${initialUUID}" != "${newUUID}" ]; then
+    echo "==> UUID of the instance should remain the same after restoring its snapshot"
+    false
+  fi
+
+  # Check that the generation UUID from before changes compared to the one after snapshoting
+  newGenerationID=$(lxc config get bar volatile.uuid.generation)
+  if [ "${initialGenerationID}" = "${newGenerationID}" ]; then
+    echo "==> Generation UUID of the instance should change after restoring its snapshot"
+    false
+  fi
+
+  # Check if the volumes's UUID is the same as the original volume
+  [ "$(lxc storage volume get "${pool}" container/bar volatile.uuid)" = "${initialVolumeUUID}" ]
+
+  # Check that instances UUIS remain the same before and after snapshoting  (stateful mode)
+  if ! command -v criu >/dev/null 2>&1; then
+    echo "==> SKIP: stateful snapshotting with CRIU (missing binary)"
+  else
+    initialUUID=$(lxc config get bar volatile.uuid)
+    initialGenerationID=$(lxc config get bar volatile.uuid.generation)
+    lxc start bar
+    lxc snapshot bar snap2 --stateful
+    restore_and_compare_fs snap2
+
+    newUUID=$(lxc config get bar volatile.uuid)
+    if [ "${initialUUID}" != "${newUUID}" ]; then
+      echo "==> UUID of the instance should remain the same after restoring its stateful snapshot"
+      false
+    fi
+
+    newGenerationID=$(lxc config get bar volatile.uuid.generation)
+    if [ "${initialGenerationID}" = "${newGenerationID}" ]; then
+      echo "==> Generation UUID of the instance should change after restoring its stateful snapshot"
+      false
+    fi
+
+    lxc stop bar --force
+  fi
+
+  # Check that instances have two different UUID after a snapshot copy
+  lxc launch testimage bar2
+  initialUUID=$(lxc config get bar2 volatile.uuid)
+  initialGenerationID=$(lxc config get bar2 volatile.uuid.generation)
+  lxc copy bar2 bar3
+  newUUID=$(lxc config get bar3 volatile.uuid)
+  newGenerationID=$(lxc config get bar3 volatile.uuid.generation)
+
+  if [ "${initialGenerationID}" = "${newGenerationID}" ] || [ "${initialUUID}" = "${newUUID}" ]; then
+    echo "==> UUIDs of the instance should be different after copying snapshot into instance"
+    false
+  fi
+
+  lxc delete --force bar2
+  lxc delete --force bar3
+
   # Check config value in snapshot has been restored
   cpus=$(lxc config get bar limits.cpu)
   if [ "${cpus}" != "1" ]; then
@@ -234,6 +317,16 @@ snap_restore() {
   lxc snapshot a-b c-d
   lxc restore a-b c-d
   lxc delete -f a-b
+
+  # Check snapshot creation dates.
+  lxc init testimage c1
+  lxc snapshot c1
+  ! lxc storage volume show "${pool}" container/c1 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${pool}" container/c1/snap0 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  lxc copy c1 c2
+  ! lxc storage volume show "${pool}" container/c2 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  [ "$(lxc storage volume show "${pool}" container/c1/snap0 | awk /created_at:/)" = "$(lxc storage volume show "${pool}" container/c2/snap0 | awk /created_at:/)" ]
+  lxc delete -f c1 c2
 }
 
 restore_and_compare_fs() {
