@@ -31,6 +31,12 @@ test_storage() {
   lxc storage volume set "$storage_pool" "$storage_volume" user.abc def
   [ "$(lxc storage volume get "$storage_pool" "$storage_volume" user.abc)" = "def" ]
 
+  # Check if storage volume has an UUID.
+  [ -n "$(lxc storage volume get "$storage_pool" "$storage_volume" volatile.uuid)" ]
+
+  # Check if the volume's UUID can be modified
+  ! lxc storage volume set "$storage_pool" "$storage_volume" volatile.uuid "2d94c537-5eff-4751-95b1-6a1b7d11f849" || false
+
   lxc storage volume delete "$storage_pool" "$storage_volume"
 
   # Test copying pool volume.* key to the volume with prefix stripped at volume creation time
@@ -47,7 +53,7 @@ test_storage() {
       local btrfs_storage_pool btrfs_storage_volume
       btrfs_storage_pool="lxdtest-$(basename "${LXD_DIR}")-pool-btrfs"
       btrfs_storage_volume="${storage_pool}-vol"
-      lxc storage create "$btrfs_storage_pool" "$lxd_backend" volume.block.filesystem=btrfs volume.size=200MB
+      lxc storage create "$btrfs_storage_pool" "$lxd_backend" volume.block.filesystem=btrfs volume.size=200MiB
       lxc storage volume create "$btrfs_storage_pool" "$btrfs_storage_volume"
       lxc storage volume show "$btrfs_storage_pool" "$btrfs_storage_volume"
       lxc storage volume set "$btrfs_storage_pool" "$btrfs_storage_volume" size 256MiB
@@ -66,8 +72,25 @@ test_storage() {
       fi
       lxc delete --force uuid1
       lxc delete --force uuid2
-      lxc image delete testimage
 
+      # Test UUID re-generation in case of restore.
+      lxc init testimage uuid1 -s "${POOL}"
+      lxc snapshot uuid1
+      lxc start uuid1
+      if [ "$lxd_backend" = "lvm" ]; then
+        uuid="$(blkid -s UUID -o value -p /dev/"${POOL}"/containers_uuid1)"
+      elif [ "$lxd_backend" = "ceph" ]; then
+        uuid="$(blkid -s UUID -o value -p /dev/rbd/"${POOL}"/container_uuid1)"
+      fi
+      lxc restore uuid1 snap0
+      if [ "$lxd_backend" = "lvm" ]; then
+        [ "$(blkid -s UUID -o value -p /dev/"${POOL}"/containers_uuid1)" != "$uuid" ]
+      elif [ "$lxd_backend" = "ceph" ]; then
+        [ "$(blkid -s UUID -o value -p /dev/rbd/"${POOL}"/container_uuid1)" != "$uuid" ]
+      fi
+      lxc delete --force uuid1
+
+      lxc image delete testimage
       lxc storage delete "$btrfs_storage_pool"
   fi
   ensure_import_testimage
@@ -108,8 +131,6 @@ test_storage() {
       ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-zfs-pool-config" zfs lvm.thinpool_name=bla || false
       ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-zfs-pool-config" zfs lvm.use_thinpool=false || false
       ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-zfs-pool-config" zfs lvm.vg_name=bla || false
-      ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-zfs-pool-config" zfs volume.block.filesystem=ext4 || false
-      ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-zfs-pool-config" zfs volume.block.mount_options=discard || false
 
       # Test that all valid zfs storage pool configuration keys can be set.
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-zfs-pool-config" zfs volume.zfs.remove_snapshots=true
@@ -154,8 +175,8 @@ test_storage() {
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-btrfs-pool-config" btrfs rsync.bwlimit=1024
       lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-valid-btrfs-pool-config"
 
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-btrfs-pool-config" btrfs btrfs.mount_options="rw,strictatime,nospace_cache,user_subvol_rm_allowed"
-      lxc storage set "lxdtest-$(basename "${LXD_DIR}")-valid-btrfs-pool-config" btrfs.mount_options "rw,relatime,space_cache,user_subvol_rm_allowed"
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-btrfs-pool-config" btrfs btrfs.mount_options="rw,strictatime,user_subvol_rm_allowed"
+      lxc storage set "lxdtest-$(basename "${LXD_DIR}")-valid-btrfs-pool-config" btrfs.mount_options "rw,relatime,user_subvol_rm_allowed"
       lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-valid-btrfs-pool-config"
     fi
 
@@ -174,7 +195,7 @@ test_storage() {
     ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir lvm.thinpool_name=bla || false
     ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir lvm.use_thinpool=false || false
     ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir lvm.vg_name=bla || false
-    ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir size=10GB || false
+    ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir size=1GiB || false
     ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir volume.block.filesystem=ext4 || false
     ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir volume.block.mount_options=discard || false
     ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-dir-pool-config" dir volume.zfs.remove_snapshots=true || false
@@ -189,13 +210,13 @@ test_storage() {
       # Create lvm pool.
       configure_loop_device loop_file_3 loop_device_3
       # shellcheck disable=SC2154
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool6" lvm source="${loop_device_3}" volume.size=25MB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool6" lvm source="${loop_device_3}" volume.size=25MiB
 
       configure_loop_device loop_file_5 loop_device_5
       # shellcheck disable=SC2154
       # Should fail if vg does not exist, since we have no way of knowing where
       # to create the vg without a block device path set.
-      ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool10" lvm source=test_vg_1 volume.size=25MB || false
+      ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool10" lvm source=test_vg_1 volume.size=25MiB || false
       # shellcheck disable=SC2154
       deconfigure_loop_device "${loop_file_5}" "${loop_device_5}"
 
@@ -204,23 +225,23 @@ test_storage() {
       pvcreate "${loop_device_6}"
       vgcreate "lxdtest-$(basename "${LXD_DIR}")-pool11-test_vg_2" "${loop_device_6}"
       # Reuse existing volume group "test_vg_2" on existing physical volume.
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool11" lvm source="lxdtest-$(basename "${LXD_DIR}")-pool11-test_vg_2" volume.size=25MB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool11" lvm source="lxdtest-$(basename "${LXD_DIR}")-pool11-test_vg_2" volume.size=25MiB
 
       configure_loop_device loop_file_7 loop_device_7
       # shellcheck disable=SC2154
       pvcreate "${loop_device_7}"
       vgcreate "lxdtest-$(basename "${LXD_DIR}")-pool12-test_vg_3" "${loop_device_7}"
       # Reuse existing volume group "test_vg_3" on existing physical volume.
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool12" lvm source="lxdtest-$(basename "${LXD_DIR}")-pool12-test_vg_3" volume.size=25MB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool12" lvm source="lxdtest-$(basename "${LXD_DIR}")-pool12-test_vg_3" volume.size=25MiB
 
       configure_loop_device loop_file_8 loop_device_8
       # shellcheck disable=SC2154
       # Create new volume group "test_vg_4".
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool13" lvm source="${loop_device_8}" lvm.vg_name="lxdtest-$(basename "${LXD_DIR}")-pool13-test_vg_4" volume.size=25MB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool13" lvm source="${loop_device_8}" lvm.vg_name="lxdtest-$(basename "${LXD_DIR}")-pool13-test_vg_4" volume.size=25MiB
 
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool14" lvm volume.size=25MB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool14" lvm volume.size=25MiB
 
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-pool15" lvm lvm.use_thinpool=false volume.size=25MB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-pool15" lvm lvm.use_thinpool=false volume.size=25MiB
 
       # Test that no invalid lvm storage pool configuration keys can be set.
       ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-lvm-pool-config" lvm volume.zfs.remove_snapshots=true || false
@@ -232,10 +253,10 @@ test_storage() {
       # Test that all valid lvm storage pool configuration keys can be set.
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool16" lvm lvm.thinpool_name="lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config"
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool17" lvm lvm.vg_name="lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config"
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool18" lvm size=10GB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool18" lvm size=1GiB
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool19" lvm volume.block.filesystem=ext4
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool20" lvm volume.block.mount_options=discard
-      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool21" lvm volume.size=2GB
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool21" lvm volume.size=25MiB
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool22" lvm lvm.use_thinpool=true
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool23" lvm lvm.use_thinpool=true lvm.thinpool_name="lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config"
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-valid-lvm-pool-config-pool24" lvm rsync.bwlimit=1024
@@ -388,10 +409,10 @@ test_storage() {
       lxc launch testimage c12pool6 -s "lxdtest-$(basename "${LXD_DIR}")-pool6"
       lxc list -c b c12pool6 | grep "lxdtest-$(basename "${LXD_DIR}")-pool6"
       # grow lv
-      lxc config device set c12pool6 root size 30MB
+      lxc config device set c12pool6 root size 30MiB
       lxc restart c12pool6 --force
       # shrink lv
-      lxc config device set c12pool6 root size 25MB
+      lxc config device set c12pool6 root size 25MiB
       lxc restart c12pool6 --force
 
       lxc init testimage c10pool11 -s "lxdtest-$(basename "${LXD_DIR}")-pool11"
@@ -428,7 +449,7 @@ test_storage() {
       lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool6" volume.block.filesystem xfs
       lxc init testimage c1pool6 -s "lxdtest-$(basename "${LXD_DIR}")-pool6"
       lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool6" volume.block.filesystem btrfs
-      lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool6" volume.size 120MB
+      lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool6" volume.size 120MiB
       lxc init testimage c2pool6 -s "lxdtest-$(basename "${LXD_DIR}")-pool6"
 
       lxc storage volume create "lxdtest-$(basename "${LXD_DIR}")-pool6" c10pool6
@@ -761,31 +782,20 @@ test_storage() {
     fi
   )
 
-  # Test applying quota (expected size ranges are in KB and have an allowable range to account for allocation variations).
-  QUOTA1="10GB"
-  rootMinKB1="9456000"
-  rootMaxKB1="9999999"
+  # Test applying quota (expected size ranges are in KiB and have an allowable range to account for allocation variations).
+  QUOTA1="20MiB"
+  rootMinKiB1="13800"
+  rootMaxKiB1="23000"
 
-  QUOTA2="11GB"
-  rootMinKB2="10402000"
-  rootMaxKB2="10744999"
-
-  if [ "$lxd_backend" = "lvm" ]; then
-    QUOTA1="20MB"
-    rootMinKB1="13800"
-    rootMaxKB1="20000"
-
-    # Increase quota enough to require a new 4MB LVM extent.
-    QUOTA2="25MB"
-    rootMinKB2="18900"
-    rootMaxKB2="23000"
-  fi
+  QUOTA2="25MiB"
+  rootMinKiB2="18900"
+  rootMaxKiB2="28000"
 
   if [ "$lxd_backend" != "dir" ]; then
     lxc launch testimage quota1
-    rootOrigSizeKB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
-    rootOrigMinSizeKB=$((rootOrigSizeKB-2000))
-    rootOrigMaxSizeKB=$((rootOrigSizeKB+2000))
+    rootOrigSizeKiB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
+    rootOrigMinSizeKiB=$((rootOrigSizeKiB-2000))
+    rootOrigMaxSizeKiB=$((rootOrigSizeKiB+2000))
 
     lxc profile device set default root size "${QUOTA1}"
     lxc stop -f quota1
@@ -793,8 +803,8 @@ test_storage() {
 
     # BTRFS quota isn't accessible with the df tool.
     if [ "$lxd_backend" != "btrfs" ]; then
-    rootSizeKB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
-      if [ "$rootSizeKB" -gt "$rootMaxKB1" ] || [ "$rootSizeKB" -lt "$rootMinKB1" ] ; then
+    rootSizeKiB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
+      if [ "$rootSizeKiB" -gt "$rootMaxKiB1" ] || [ "$rootSizeKiB" -lt "$rootMinKiB1" ] ; then
         echo "root size not within quota range"
         false
       fi
@@ -815,8 +825,8 @@ test_storage() {
     lxc stop -f quota2
     lxc start quota2
     if [ "$lxd_backend" != "btrfs" ]; then
-      rootSizeKB=$(lxc exec quota2 -- df -P / | tail -n1 | awk '{print $2}')
-      if [ "$rootSizeKB" -gt "$rootMaxKB2" ] || [ "$rootSizeKB" -lt "$rootMinKB2" ] ; then
+      rootSizeKiB=$(lxc exec quota2 -- df -P / | tail -n1 | awk '{print $2}')
+      if [ "$rootSizeKiB" -gt "$rootMaxKiB2" ] || [ "$rootSizeKiB" -lt "$rootMinKiB2" ] ; then
         echo "root size not within quota range"
         false
       fi
@@ -829,8 +839,8 @@ test_storage() {
 
     # Only ZFS supports hot quota changes (LVM requires a reboot).
     if [ "$lxd_backend" = "zfs" ]; then
-      rootSizeKB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
-      if [ "$rootSizeKB" -gt "$rootOrigMaxSizeKB" ] || [ "$rootSizeKB" -lt "$rootOrigMinSizeKB" ] ; then
+      rootSizeKiB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
+      if [ "$rootSizeKiB" -gt "$rootOrigMaxSizeKiB" ] || [ "$rootSizeKiB" -lt "$rootOrigMinSizeKiB" ] ; then
         echo "original root size not restored"
         false
       fi
@@ -839,8 +849,8 @@ test_storage() {
     lxc stop -f quota1
     lxc start quota1
     if [ "$lxd_backend" = "zfs" ]; then
-      rootSizeKB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
-      if [ "$rootSizeKB" -gt "$rootOrigMaxSizeKB" ] || [ "$rootSizeKB" -lt "$rootOrigMinSizeKB" ] ; then
+      rootSizeKiB=$(lxc exec quota1 -- df -P / | tail -n1 | awk '{print $2}')
+      if [ "$rootSizeKiB" -gt "$rootOrigMaxSizeKiB" ] || [ "$rootSizeKiB" -lt "$rootOrigMinSizeKiB" ] ; then
         echo "original root size not restored"
         false
       fi
@@ -901,6 +911,66 @@ test_storage() {
   lxc storage delete "${storage_pool}"
   lxc image show testimage
 
+  # Test storage pool resize
+  if [ "${lxd_backend}" = "btrfs" ] || [ "${lxd_backend}" = "lvm" ] || [ "${lxd_backend}" = "zfs" ]; then
+    # shellcheck disable=SC1009
+    pool_name="lxdtest-$(basename "${LXD_DIR}")-pool1"
+
+    lxc storage create "${pool_name}" "${lxd_backend}" size=1GiB
+
+    lxc launch testimage c1 -s "${pool_name}"
+
+    expected_size=1073741824
+    # +/- 5% of the expected size
+    expected_size_min=1020054732
+    expected_size_max=1127428916
+
+    # Check pool file size
+    [ "$(stat --format="%s" "${LXD_DIR}/disks/${pool_name}.img")" = "${expected_size}" ]
+
+    if [ "${lxd_backend}" = "btrfs" ]; then
+      actual_size="$(btrfs filesystem show --raw "${LXD_DIR}/disks/${pool_name}.img" | awk '/devid/{print $4}')"
+    elif [ "${lxd_backend}" = "lvm" ]; then
+      actual_size="$(lvs --noheadings --nosuffix --units b --options='lv_size' "lxdtest-$(basename "${LXD_DIR}")/LXDThinPool")"
+    elif [ "${lxd_backend}" = "zfs" ]; then
+      actual_size="$(zpool list -Hp "${pool_name}" | awk '{print $2}')"
+    fi
+
+    # Check that pool size is within the expected range
+    [ "${actual_size}" -ge "${expected_size_min}" ] || [ "${actual_size}" -le "${expected_size_max}" ]
+
+    # Grow pool
+    lxc storage set "${pool_name}" size=2GiB
+
+    expected_size=2147483648
+    # +/- 5% of the expected size
+    expected_size_min=2040109465
+    expected_size_max=2254857831
+
+    # Check pool file size
+    [ "$(stat --format="%s" "${LXD_DIR}/disks/${pool_name}.img")" = "${expected_size}" ]
+
+    if [ "${lxd_backend}" = "btrfs" ]; then
+      actual_size="$(btrfs filesystem show --raw "${LXD_DIR}/disks/${pool_name}.img" | awk '/devid/{print $4}')"
+    elif [ "${lxd_backend}" = "lvm" ]; then
+      actual_size="$(lvs --noheadings --nosuffix --units b --options='lv_size' "lxdtest-$(basename "${LXD_DIR}")/LXDThinPool")"
+    elif [ "${lxd_backend}" = "zfs" ]; then
+      actual_size="$(zpool list -Hp "${pool_name}" | awk '{print $2}')"
+    fi
+
+    # Check that pool size is within the expected range
+    [ "${actual_size}" -ge "${expected_size_min}" ] || [ "${actual_size}" -le "${expected_size_max}" ]
+
+    # Shrinking the pool should fail
+    ! lxc storage set "${pool_name}" size=1GiB || false
+
+    # Ensure the pool is still usable after resizing by launching an instance
+    lxc launch testimage c2 -s "${pool_name}"
+
+    lxc rm -f c1 c2
+    lxc storage rm "${pool_name}"
+  fi
+
   # shellcheck disable=SC2031,2269
   LXD_DIR="${LXD_DIR}"
   kill_lxd "${LXD_STORAGE_DIR}"
